# -*- coding: utf-8 -*-

from at.orm.decorators import DBSwitch
from at.utiles.conversiones import respuesta
from at.cpbot.funciones import armar_arbol
from at.cpbot.bdcpbot import CpBotBd

from sqlalchemy import and_, or_


@DBSwitch('cpbot')
class Arboles:

    def __init__(self, datos):
        '''
        El init toma como parametro los datos de la request.
        '''
        self.datos = datos
        self.cpbotbd = CpBotBd()

    def __registro_a_dict(self, registro):
        """
        Funcion privada para formatear un registro de SQLAlchemy a diccionario.
        """
        dic = registro.__dict__
        try:
            dic.pop('_sa_instance_state')
        except:
            pass
        return dic

    def __obtener_id_hijos(self, id_nodo, lista_nodos=None, session=None):
        """Funcion privada que se encarga de obtener un nodo y todos sus hijos"""

        if session is None:
            session = self.cpbot.binded_session()

        if lista_nodos is None:
            lista_nodos = list()

        nodos = self.cpbot.tabla('nodos')

        q_obj = session.query(nodos).filter(nodos.padre == id_nodo)

        registros = q_obj.all()

        if registros is None:
            session.close()
            return lista_nodos

        else:

            for registro in registros:
                if registro.id_nodo not in lista_nodos:
                    lista_nodos.append(registro.id_nodo)
                self.__obtener_id_hijos(str(registro.id_nodo), lista_nodos=lista_nodos, session=session)
        session.close()
        return lista_nodos

    def editar_modo(self):
        """Edita un modo en la base de datos, y modifica el campo id_modo de los nodos que correspondan"""

        session = self.cpbot.binded_session()

        modos = self.cpbot.tabla('modos')

        nodos = self.cpbot.tabla('nodos')

        q_obj_modo = session.query(modos).filter(modos.id_modo == self.datos['id_modo'])

        q_obj_nodos = session.query(nodos).filter(nodos.id_modo == self.datos['id_modo'])

        if q_obj_modo.count() == 0:
            session.close()

            return respuesta(estado=False,
                             rta="No se encontro el modo %s para editarlo" % self.datos['id_modo'])

        dic_update = dict(id_modo=self.datos['nuevo_nombre'])

        rta_update_modo = q_obj_modo.update(dic_update)  # Actualizamos el modo con su nuevo nombre

        if rta_update_modo == 0:
            session.close()

            return respuesta(estado=False,
                             rta="No se pudo actualizar el modo %s" % self.datos['id_modo'])

        if q_obj_nodos.count() > 0:  # Si se encontraron nodos para el modo, actualizamos
            rta_update_nodos = q_obj_nodos.update(dic_update)

            if rta_update_nodos == 0:
                session.close()

                return respuesta(estado=False,
                                 rta="No se pudieron actualizar los nodos del modo %s" % self.datos['id_modo'])

        session.commit()

        session.close()

        nodos = self.obtener_nodos()

        return nodos

    def borrar_modo(self):
        """Borra un modo y todos sus nodos correspondientes"""

        session = self.cpbot.binded_session()

        modos = self.cpbot.tabla('modos')

        nodos = self.cpbot.tabla('nodos')

        q_obj_modo = session.query(modos).filter(modos.id_modo == self.datos['id_modo'])

        q_obj_nodos = session.query(nodos).filter(nodos.id_modo == self.datos['id_modo'])

        if q_obj_modo.count() == 0:
            session.close()

            return respuesta(estado=False,
                             rta="No se encontro el modo %s en la base de datos" % self.datos['id_modo'])

        # Borramos el modo de la base de datos
        q_obj_modo.delete(synchronize_session=False)

        # Borramos los nodos que pertenecen al modo borrado
        q_obj_nodos.delete(synchronize_session=False)

        session.commit()

        session.close()

        modos = self.obtener_modos()

        return modos

    def agregar_modo(self):
        """
        Se encarga de agregar un modo a la base de datos, con un nodo predeterminado como principal.
        """

        dic = dict(id_modo=self.datos['id_modo'],
                   habilitado=1,
                   volver_modo_anterior=0)

        return self.cpbot.insertar('modos', 'id_modo', **dic)


    def obtener_modos(self):
        """
        Se encarga de obtener todos los modos existentes y disponibles en la base de datos.
        """

        session = self.cpbot.binded_session()

        modos = self.cpbot.tabla('modos')

        q_obj = session.query(modos).filter(modos.habilitado == 1)

        registros = q_obj.all()

        if registros is False:  # Si no se encontraron modos habilitados
            session.close()
            return respuesta(estado=False,
                             rta="No se encontraron modos habilitados")

        lista = list()  # Inicializamos la lista a devolver

        for registro in registros:
            dic = self.__registro_a_dict(registro)

            lista.append(dic)  # Agregamos a la lista el nombre del modo
        session.close()
        return respuesta(estado=True,
                         rta="Se obtuvieron los modos habilitados",
                         dato=lista)

    def obtener_nodos(self):
        """
        Obtiene los nodos de un modo dado.
        """

        session = self.cpbot.binded_session()

        nodos = self.cpbot.tabla('nodos')

        funciones = self.cpbot.tabla('funciones')

        print(self.datos['id_modo'],type(self.datos['id_modo']))
        # La query es para obtener todos los nodos sin padre del modo dado
        q_obj = session.query(nodos).filter(and_(nodos.id_modo == self.datos['id_modo'],
                                                 nodos.padre == None))

        if q_obj.count() == 0:
            session.close()
            return respuesta(estado=True,
                             rta="El modo %s no tiene nodos" % self.datos['id_modo'],
                             dato=[])

        registros = q_obj.all()

        ramas = list()

        for registro in registros:

            rama = list()

            reg_dic = self.__registro_a_dict(registro)

            q_obj_func = session.query(funciones).filter(funciones.id_funcion == reg_dic['id_funcion'])

            if q_obj_func.count() == 0:
                session.close()
                return respuesta(estado=False,
                                 rta="Error al intentar buscar la funcion {}".format(reg_dic['id_funcion']))

            funcion = q_obj_func.first()

            nombre_funcion = self.__registro_a_dict(funcion)['nombre']

            reg_dic['funcion'] = nombre_funcion

            rama.append(reg_dic)

            id_hijos = self.__obtener_id_hijos(str(registro.id_nodo))

            for id_hijo in id_hijos:

                q_obj = session.query(nodos).filter(nodos.id_nodo == id_hijo)

                hijo = q_obj.first()

                if hijo is None:
                    session.close()
                    return respuesta(estado=False,
                                     rta="No se encontro el nodo hijo %s del nodo %s" % (id_hijo, registro.id_nodo))

                hijo = self.__registro_a_dict(hijo)

                q_obj_func = session.query(funciones).filter(funciones.id_funcion == hijo['id_funcion'])

                if q_obj_func.count() == 0:
                    session.close()
                    return respuesta(estado=False,
                                     rta="Error al intentar buscar la funcion {}".format(hijo['id_funcion']))

                funcion = q_obj_func.first()

                nombre_funcion = self.__registro_a_dict(funcion)['nombre']

                hijo['funcion'] = nombre_funcion

                rama.append(hijo)

            ramas.append(rama)
        session.close()
        return respuesta(estado=True,
                         rta="Se obtuvieron los nodos del modo %s" % self.datos['id_modo'],
                         dato=ramas)

    def agregar_nodo(self):
        """
        Agrega un nodo a un arbol dado.
        """

        session = self.cpbot.binded_session()

        funciones = self.cpbot.tabla('funciones')

        q_obj = session.query(funciones).filter(funciones.nombre == self.datos['funcion'])

        registro = q_obj.first()

        session.close()

        if registro is False:
            return respuesta(estado=False,
                             rta="No se encontro la funcion %s" % self.datos['funcion'])

        id_funcion = registro.id_funcion

        dic = dict(id_funcion=id_funcion,
                   id_modo=self.datos['id_modo'],
                   padre=self.datos['padre'],
                   principal=self.datos['principal'],
                   permitido=str(self.datos['permitido']),
                   orden=self.datos['orden'])

        rta = self.cpbot.insertar('nodos', 'id_nodo', **dic)

        if rta['Estado'] is False:
            return rta

        nodos = self.obtener_nodos()

        return nodos

    def borrar_nodo(self):
        """
        Borra un nodo del arbol dado
        """

        session = self.cpbot.binded_session()

        nodos = self.cpbot.tabla('nodos')

        # Creamos una lista con la id de los nodos
        hijos = self.__obtener_id_hijos(str(self.datos['id_nodo']))

        # Agregamos el id nodo que queremos borrar, para borrar todo de una
        hijos.append(self.datos['id_nodo'])

        for hijo in hijos:

            q_obj = session.query(nodos).filter(nodos.id_nodo == hijo)

            if q_obj.count() > 0:
                q_obj.delete(synchronize_session=False)
                # return respuesta(estado=False,
                #                  rta="No se encontro el nodo %s para el modo %s" % (
                #                      self.datos['id_nodo'], self.datos['id_modo']))
            else:
                rta = "No se encontro el nodo %s para el modo %s" \
                      % (self.datos['id_nodo'], self.datos['id_modo'])
                print(rta)

        session.commit()

        session.close()

        # Volvemos a setear el id submodo como el "principal"
        self.datos['id_submodo'] = "principal"

        rta_nodos = self.obtener_nodos()

        return respuesta(estado=True,
                         rta="Se borro el nodo correctamente",
                         dato=rta_nodos['Dato'])

    def editar_nodo(self):
        """
        Edita un nodo en con los datos dados.
        """

        session = self.cpbot.binded_session()

        nodos = self.cpbot.tabla('nodos')

        funciones = self.cpbot.tabla('funciones')

        func_reg = session.query(funciones).filter(funciones.nombre == self.datos['funcion']).first()

        if func_reg is None:
            session.close()

            return respuesta(estado=False,
                             rta="No se encontro la funcion %s" % self.datos['funcion'])

        funcion = self.__registro_a_dict(func_reg)

        q_obj = session.query(nodos).filter(nodos.id_nodo == self.datos['id_nodo'])

        if q_obj.count() <= 0:
            session.close()

            return respuesta(estado=False,
                             rta="No se encontro el nodo %s" % self.datos['id_nodo'])

        dic_update = dict(id_funcion=funcion['id_funcion'],
                          padre=self.datos['padre'],
                          permitido=str(self.datos['permitido']),
                          orden=self.datos['orden'])

        rta_update = q_obj.update(dic_update)

        if rta_update == 0:
            session.close()

            return respuesta(estado=False,
                             rta="No se pudo actualizar el nodo")

        session.commit()

        session.close()

        # Volvemos a setear el id submodo como el "principal"
        self.datos['id_submodo'] = "principal"

        rta_nodos = self.obtener_nodos()

        return respuesta(estado=True,
                         rta="Se edito el nodo correctamente",
                         dato=rta_nodos['Dato'])

    def cortar_rama(self):
        """Se encarga de cortar una rama del submodo principal, para deshabilitarla por completo del mismo"""

        session = self.cpbot.binded_session()

        nodos = self.cpbot.tabla('nodos')

        # Buscamos el nodo principal de la rama cortada
        q_obj = session.query(nodos).filter(nodos.id_nodo == self.datos['id_nodo'])

        if q_obj.count() == 0:
            session.close()

            return respuesta(estado=False,
                             rta="No se encontro el nodo %s para cortarlo" % self.datos['id_nodo'])

        q_obj.update(dict(padre=None))

        session.commit()

        session.close()

        rta_nodos = self.obtener_nodos()

        return respuesta(estado=True,
                         rta="Se corto la rama correctamente",
                         dato=rta_nodos['Dato'])

    def pegar_rama(self):
        """Se encarga de volver a habilitar una rama en el arbol principal de un modo"""

        session = self.cpbot.binded_session()

        nodos = self.cpbot.tabla('nodos')

        q_obj = session.query(nodos).get(self.datos['id_nodo'])

        print(self.datos['id_nodo_padre'],'tipo:',type(self.datos['id_nodo_padre']))
        q_obj.padre = self.datos['id_nodo_padre']

        print(self.datos)
        session.commit()

        session.close()

        return respuesta(estado=True,
                         rta="Se volvio a pegar el submodo a la rama principal")
