# -*- coding: utf-8 -*-

import json
from at.utiles.conversiones import respuesta
from at.utiles.http import enviar_por_http
from at.cpbot.bdcpbot import CpBotBd
import pprint

cpbotbd = CpBotBd()  # Instancio la clase de la base de datos de CpBot


def armar_url(ip, ruta):
    return 'http://' + ip + ruta


def enviar_atmsj(mensaje, tel_origen, tel_destino, ip_atmsj, id_mensaje_web=None):
    '''
    Esta funcion se encarga de enviar un mensaje a atmsj.

    :param mensaje: Un mensaje en unicode.
    :return: Respuesta
    '''
    if mensaje is None:
        return (False, u"El mensaje está vacio")

    url = armar_url(ip_atmsj, '/enviar_msj')

    params = {}
    params['msj'] = mensaje
    params['tel_origen'] = tel_origen
    params['tel_destino'] = tel_destino
    params['enviado_por'] = ''
    params['id_mensaje_web'] = id_mensaje_web
    params['id_producto'] = 4
    params['usuario_responsable'] = "Automatico"

    metodo = 'POST'
    # Envía el nuevo mensaje por HTTP
    rta = enviar_por_http(url, params, metodo, tipo_params='json')

    # Si no se pudo enviar el mensaje
    if rta['Estado'] is False:
        # Retorna el error
        return rta

    # Obtiene la respuesta recibida en formato JSON
    try:
        rta = json.loads(rta['Respuesta']['content'].decode('utf-8'))
    except:
        return (False, u"Hubo un error al decodificar la respuesta")

    # Retorna la respuesta recibida como un diccionario
    return rta


def anunciar_ip(mi_ip, ip_atmsj):
    if mi_ip is None:
        return (False, u"No existe la ip donde se levanta el bot")

    url = armar_url(ip_atmsj, '/anunciar_ip')

    params = {}
    params['ip_bot'] = mi_ip

    metodo = 'POST'
    # Envía el nuevo mensaje por HTTP
    rta = enviar_por_http(url, params, metodo, tipo_params='json')

    # Si no se pudo enviar el mensaje
    if rta['Estado'] is False:
        # Retorna el error
        return rta

    # Obtiene la respuesta recibida en formato JSON
    try:
        rta = json.loads(rta['Respuesta']['content'].decode('utf-8'))
    except:
        return (False, u"Hubo un error al decodificar la respuesta")

    # Retorna la respuesta recibida como un diccionario
    return rta


def finalizar_charla(ip_atmsj, telefono):
    if ip_atmsj is None:
        return (False, u"No existe la ip de atmsj")
    if telefono is None:
        return (False, u"No hay telefono para finalizar")

    url = armar_url(ip_atmsj, '/finalizar_charla')

    params = {}
    params['telefono'] = telefono

    metodo = 'POST'
    # Envía el nuevo mensaje por HTTP
    rta = enviar_por_http(url, params, metodo, tipo_params='json')
    print("finalizar charla: ", rta)
    # Si no se pudo enviar el mensaje
    if rta['Estado'] is False:
        # Retorna el error
        return rta

    # Obtiene la respuesta recibida en formato JSON
    try:
        rta = json.loads(rta['Respuesta']['content'].decode('utf-8'))
    except:
        return (False, u"Hubo un error al decodificar la respuesta")

    # Retorna la respuesta recibida como un diccionario
    return rta


def cambiar_automatico(telefono, automatico, ip_atmsj):
    print("va a cambiar a auto")
    '''
    Esta funcion se encarga de enviar un mensaje a atmsj indicando que debe cambiar el automatico en una charla.

    :param mensaje: Un mensaje en unicode.
    :return: Respuesta
    '''
    if telefono is None:
        return (False, u"El telefono está vacio")

    if automatico is None:
        return (False, u"El modo a modificar en automatico está vacio")

    url = armar_url(ip_atmsj, '/cambiar_automatico')

    params = {}
    params['telefono'] = telefono
    params['automatico'] = automatico

    metodo = 'POST'
    # Envía el nuevo mensaje por HTTP
    rta = enviar_por_http(url, params, metodo, tipo_params='json')

    # Si no se pudo enviar el mensaje
    if rta['Estado'] is False:
        # Retorna el error
        return rta

    # Obtiene la respuesta recibida en formato JSON
    try:
        rta = json.loads(rta['Respuesta']['content'].decode('utf-8'))
    except:
        return (False, u"Hubo un error al decodificar la respuesta")

    # Retorna la respuesta recibida como un diccionario
    return rta