# -*- coding: utf-8 -*-
import datetime
import pprint
from at.orm.decorators import DBSwitch
from at.utiles.conversiones import respuesta
from sqlalchemy.sql.expression import and_
from decimal import Decimal
from sqlalchemy import or_


@DBSwitch('cpbot')
class CpBotBd:

    def __init__(self):
        pass

    def __obtener_id_hijos(self, id_nodo, lista_nodos=None, session=None):
        """Funcion privada que se encarga de obtener un nodo y todos sus hijos"""

        if session is None:
            session = self.cpbot.binded_session()

        if lista_nodos is None:
            lista_nodos = list()

        nodos = self.cpbot.tabla('nodos')

        q_obj = session.query(nodos).filter(nodos.padre == id_nodo)

        registros = q_obj.all()

        if registros is None:
            session.close()
            return lista_nodos

        else:

            for registro in registros:
                if registro.id_nodo not in lista_nodos:
                    lista_nodos.append(registro.id_nodo)
                self.__obtener_id_hijos(str(registro.id_nodo), lista_nodos=lista_nodos, session=session)
        session.close()
        return lista_nodos

    def grabar_charla(self, charla):
        '''
            Esta función se encarga de grabar un registro
            en la tabla "charla".

            Parámetros:
                charla

            Funciones:
                at.orm -> abm -> ABM -> insertar

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
                    ['Dato'] (int)
        '''

        # Agrego un nuevo registro en la tabla "mensajes"
        rta = self.cpbot.insertar('charla', 'id_charla', **charla)

        "no pongo commit y close porque el insertar ya lo tiene"

        # Si no se pudo insertar el registro
        if rta['Estado'] is False:
            # Retorno el error
            return rta

        # Si se pudo insertar el registro retorno True y un mensaje
        return respuesta(True, rta['Respuesta'], rta['ValorCampo'])

    def grabar_logs(self, logs, datos_ejecucion, id_charla):
        '''
            Esta función se encarga de grabar los logs en la tabla "logs"

            Parámetros:
                logs

            Funciones:
                at.utiles -> conversiones -> respuesta
                at.utiles -> servidores -> DateTimeHelper -> get_now
                at.orm -> abm -> ABM -> insertar

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
                    ['Dato'] (int)
        '''

        # para cada uno de los logs los va insertando
        for log in logs:
            # Creo el diccionario que insertare en el campo datos
            log['datos']['datos_ejecucion'] = datos_ejecucion  # Inserto los datos de la ejecucion del nodo
            dic = {}
            dic['id_charla'] = id_charla
            dic['mensaje_enviado'] = log['texto']
            dic['id_nodo'] = log['id_nodo']
            dic['datos'] = unicode(log['datos'])
            dic['espera'] = log['espera']
            dic['estado'] = 'activo'
            dic['fecha_hora'] = datetime.datetime.now()
            dic['opcion'] = log['opcion']
            dic['fecha_hora_respuesta'] = None
            dic['seleccionado'] = 0

            rta = self.cpbot.insertar('logs', 'id_log', **dic)

            # Si no se pudo insertar el registro
            if rta['Estado'] is False:
                # Retorno el error
                return rta

        # Si se pudieron insertar los logs
        return respuesta(True, u"Se insertaron correctamente los logs", logs)

    def obtener_ips(self, telefono_cliente):
        '''
        Funcion que sirve para obtener las ips de un cliente mediante su telefono, *TABLA PARAMETROS*

        :param telefono_cliente: El telefono del cliente
        :return: (dict) Un diccionario con los datos de la tabla parametros pertenecientes a un cliente
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'parametros'
        parametros = self.cpbot.tabla('parametros')

        # Obtengo los datos que me interesan
        q_obj = session.query(
            parametros.ip_callprocessor,
            parametros.ip_atmsj
        )

        # Filtro por el telefono dado
        q_obj = q_obj.filter(telefono_cliente == parametros.telefono)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesion
        session.close()

        # Si no pude obtener el registro
        if registro is None:
            # Retorno una respuesta con False
            return respuesta(False, u'No se pudieron obtener los datos de la tabla parametros')

        # Guardo los datos obtenidos
        datos = {}
        datos['ip_callprocessor'] = registro.ip_callprocessor
        datos['ip_atmsj'] = registro.ip_atmsj

        # Retorno True y los datos obtenidos
        return respuesta(True, u'Se obtuvieron los datos de la tabla parametros', dato=datos)

    def obtener_tag(self, id_tag):
        '''
        obtiene el tag de la tabla tags

        :param id_tag: el id_tag a buscar
        :return: dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
                    ['Dato'] (int)
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'parametros'
        tag = self.cpbot.tabla('tags')

        # Obtengo los datos que me interesan
        q_obj = session.query(tag)

        # Filtro por el telefono dado
        q_obj = q_obj.filter_by(id_tag=id_tag)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesion
        session.close()

        # Si no pude obtener el registro
        if registro is None:
            # Retorno una respuesta con False
            return respuesta(False, u'No se pudo obtener el tag')

        # Guardo los datos obtenidos
        datos = registro.__dict__

        # Retorno True y los datos obtenidos
        return respuesta(True, u'Se obtuvieron los datos de la tabla tags', dato=datos)

    def obtener_datos_cliente(self, id_empresa):
        print("id_empresa:",id_empresa)
        '''
        Funcion que sirve para obtener la ip de un cliente segun su id_empresa, *TABLA PARAMETROS*

        :param telefono_cliente: El telefono del cliente
        :return: (dict) Un diccionario con los datos de la tabla parametros pertenecientes a un cliente
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'parametros'
        parametros = self.cpbot.tabla('parametros')

        # Obtengo los datos que me interesan
        q_obj = session.query(
            parametros
        )

        # Filtro por el telefono dado
        q_obj = q_obj.filter(id_empresa == parametros.id_empresa)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesion
        session.close()

        # Si no pude obtener el registro
        if registro is None:
            # Retorno una respuesta con False
            return respuesta(False, u'No se pudieron obtener la ip de atmsj de la tabla parametros')

        # Guardo los datos obtenidos
        datos = registro.__dict__

        # Retorno True y los datos obtenidos
        return respuesta(True, u'Se obtuvo la ip de atmsj de la tabla parametros', dato=datos)

    def obtener_parametros(self, telefono_cliente):
        '''
        Funcion que sirve para obtener los parametros de un cliente mediante su telefono, *TABLA PARAMETROS*

        :param telefono_cliente: El telefono del cliente
        :return: (dict) Un diccionario con los datos de la tabla parametros pertenecientes a un cliente
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'parametros'
        parametros = self.cpbot.tabla('parametros')

        # Obtengo los datos que me interesan
        q_obj = session.query(
            parametros.telefono,
            parametros.id_empresa,
            parametros.automatico,
            parametros.id_modo,
            parametros.limite_intentos,
            parametros.horarios,
            parametros.ip_atmsj,
            parametros.ip_callprocessor,
            parametros.id_ciudad,
            parametros.ip_atapis,
            parametros.coordenadas,
            parametros.ignorar_errores,
            parametros.formato_direcciones,
            parametros.ciudad,
            parametros.provincia,
            parametros.pais
        )

        # Filtro por el telefono dado
        q_obj = q_obj.filter(telefono_cliente == parametros.telefono)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesion
        session.close()

        # Si no pude obtener el registro
        if registro is None:
            # Retorno una respuesta con False
            return respuesta(False, u'No se pudieron obtener los datos de la tabla parametros')

        # Guardo los datos obtenidos
        datos = {}
        datos['telefono'] = str(registro.telefono)
        datos['id_empresa'] = str(registro.id_empresa)
        datos['automatico'] = str(registro.automatico)
        datos['id_modo'] = str(registro.id_modo)
        datos['horarios'] = str(registro.horarios)
        datos['ip_atmsj'] = str(registro.ip_atmsj)
        datos['ip_callprocessor'] = str(registro.ip_callprocessor)
        datos['limite_intentos'] = registro.limite_intentos
        datos['id_ciudad'] = registro.id_ciudad
        datos['ip_atapis'] = registro.ip_atapis
        datos['coordenadas'] = registro.coordenadas
        datos['ignorar_errores'] = registro.ignorar_errores
        datos['formato_direcciones'] = registro.formato_direcciones
        datos['ciudad'] = registro.ciudad
        datos['provincia'] = registro.provincia
        datos['pais'] = registro.pais

        # Retorno True y los datos obtenidos
        return respuesta(True, u'Se obtuvieron los datos de la tabla parametros', dato=datos)

    def obtener_modos_habilitados(self):
        '''
        Esta funcion se encarga de obtener todos los modos que esten habilidatos

        :return: (list) Una lista con los modos(int) habilitados y su funcion de inicio.
                Ejemplo: [(1, 22),(2, 2),(3, 22)] --> (ID_MODO, ID_FUNCION)

        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'modos'
        modos = self.cpbot.tabla('modos')

        # Obtengo los datos que me interesan
        q_obj = session.query(
            modos.id_modo,
            modos.habilitado,
            modos.volver_modo_anterior
        )

        # Filtro por los modos habilitados
        q_obj = q_obj.filter(bool(modos.habilitado))

        # Obtengo todos los modos habilitados
        registros = q_obj.all()

        # Cierro la sesion con la base de datos
        session.close()

        # Si no pude obtener registros
        if len(registros) == 0:
            # Devuelvo la respuesta con False
            return respuesta(False, u'No se pudieron obtener los modos habilitados')

        # Inicio la lista que voy a devolver
        lista = []

        # Si pude obtenerlos, itero a traves de ellos
        for registro in registros:
            # Creo el tuple con el id_modo
            tupla = (registro.id_modo)

            # Añado el tuple a la lista
            lista.append(tupla)

        # Devuelvo una respuesta True y la lista
        return respuesta(True, u'Se obtuvieron los modos habilitados con exito', dato=lista)

    def cambiar_estado_charla(self, id_charla, estado):
        """

        Esta funcion cambia el estado de una charla

        :param id_charla: la charla a modificar

        :return:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)

        """

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'tabla'
        charla = self.cpbot.tabla('charla')

        # obtiene el estado de la charla donde coincide el id_charla
        q_obj = session.query(charla.estado).filter(charla.id_charla == id_charla)

        # Modifica el estado
        dic = {'estado': estado}

        # Actualiza los datos
        rta_update = q_obj.update(dic)

        # Si no se actualizó ningún registro
        if rta_update == 0:
            # Cierra la sesión con la base de datos
            session.close()
            # Retorna False y un mensaje
            return respuesta(
                False,
                (u'No existe la charla %s para modificar')
                % id_charla
            )

        # Si se actualizaron registros
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Retorno True y un mensaje
        return respuesta(
            True,
            (u'Se modificó el estado de la charla %s a %s correctamente')
            % (id_charla, estado)
        )

    def actualizar_log_seleccionado(self, id_log):
        """

        Funcion que setea un log como seleccionado

        :param id_log: id del log a modificar

        :return:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)

        """

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'tabla'
        log = self.cpbot.tabla('logs')

        # obtiene el estado de la charla donde coincide el id_charla
        q_obj = session.query(log).filter(log.id_log == id_log)

        # Modifica el estado
        dic = {'seleccionado': '1'}

        # Actualiza los datos
        rta_update = q_obj.update(dic)

        # Si no se actualizó ningún registro
        if rta_update == 0:
            # Cierra la sesión con la base de datos
            session.close()
            # Retorna False y un mensaje
            return respuesta(
                False,
                (u'No existe el log %s para modificar')
                % id_log
            )

        # Si se actualizaron registros
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Retorno True y un mensaje
        return respuesta(
            True,
            (u'Se modificó el log %s como seleccionado')
            % (id_log)
        )

    def cambiar_automatico_charla(self, id_charla, automatico):
        """

        Esta funcion cambia si una charla es en automatico o en manual

        :param id_charla: la charla a modificar

        :return:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)

        """

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'tabla'
        charla = self.cpbot.tabla('charla')

        # obtiene el estado de la charla donde coincide el id_charla
        q_obj = session.query(charla.estado).filter(charla.id_charla == id_charla)

        # Modifica el estado
        dic = {'automatico': automatico}

        # Actualiza los datos
        rta_update = q_obj.update(dic)

        # Si no se actualizó ningún registro
        if rta_update == 0:
            # Cierra la sesión con la base de datos
            session.close()
            # Retorna False y un mensaje
            return respuesta(
                False,
                (u'No existe la charla %s para modificar')
                % id_charla
            )

        # Si se actualizaron registros
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Retorno True y un mensaje
        return respuesta(
            True,
            (u'Se modificó la charla %s a automatico en el valor de: %s')
            % (id_charla, automatico)
        )

    def modificar_usuario_responsable(self, id_charla, usuario_responsable):
        """

        Esta funcion cambia el usuario responsable de una charla

        :param id_charla: la charla a modificar
                usuario_responsable:el nuevo usuario responsable

        :return:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)

        """

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'tabla'
        charla = self.cpbot.tabla('charla')

        # obtiene el estado de la charla donde coincide el id_charla
        q_obj = session.query(charla.estado).filter(charla.id_charla == id_charla)

        # Modifica el estado
        dic = {'usuario_responsable': usuario_responsable}

        # Actualiza los datos
        rta_update = q_obj.update(dic)

        # Si no se actualizó ningún registro
        if rta_update == 0:
            # Cierra la sesión con la base de datos
            session.close()
            # Retorna False y un mensaje
            return respuesta(
                False,
                (u'No existe la charla %s para modificar')
                % id_charla
            )

        # Si se actualizaron registros
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Retorno True y un mensaje
        return respuesta(
            True,
            (u'Se modificó el usuario responsable de la charla %s a %s correctamente')
            % (id_charla, usuario_responsable)
        )

    def obtener_nodos(self, id_modo):
        '''
        Esta funcion se encarga de obtener todos los nodos, identificando su inicial, mediante la id_modo

        :param id_modo: La id del modo
        :return: (dict) Un diccionario con los nodos del arbol
        '''

        # Conecto con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'nodos'
        nodos = self.cpbot.tabla('nodos')

        # Obtengo los datos que me interesan
        q_obj = session.query(
            nodos.id_funcion,
            nodos.padre,
            nodos.id_nodo,
            nodos.permitido,
            nodos.orden
        )

        # Filtro para obtener el nodo principal de la rama activa
        nodo_principal = q_obj.filter(and_(nodos.id_modo == id_modo,
                                           nodos.principal == 1,
                                           or_(nodos.padre == None,
                                               nodos.padre == "")))

        # Obtengo los otros nodos
        nodo_principal = nodo_principal.first()

        # Si no obtuvimos el resto de nodos
        if nodo_principal is None:
            # Devuelvo la respuesta con False
            session.close()
            return respuesta(False, u'No se pudieron obtener los nodos del id_modo: %s' % id_modo)

        nodos_principal = self.__obtener_id_hijos(str(nodo_principal.id_nodo))

        nodos_principal.append(nodo_principal.id_nodo)

        # Inicializo el diccionario que voy a devolver
        dic = {}

        # Itero a traves de los nodos no iniciales
        for nodo in nodos_principal:

            q_obj_nodo = session.query(nodos).filter(nodos.id_nodo == nodo)

            if q_obj_nodo.count() == 0:
                session.close()
                return respuesta(estado=False,
                                 rta="Error al obtener el nodo %s de la rama principal" % nodo)

            nodo = q_obj_nodo.first()

            # Guardo los datos de la funcion
            funcion = self.obtener_funcion(nodo.id_funcion)['Dato']

            permitido = None

            if nodo.permitido is not None and nodo.permitido != u'':
                # Intentamos convertir los modos permitidos en el nodo, a un diccionario en caso de no ser None
                permitido = eval(nodo.permitido)

            try:
                padre = int(nodo.padre)
            except:
                padre = None

            # Inicializo un diccionario con la id del nodo como key
            dic[nodo.id_nodo] = {
                'id_nodo': nodo.id_nodo,
                'padre': padre,  # No tiene padre, ya que es la raiz del arbol
                'funcion': funcion['nombre'],
                'menu': funcion['titulo'],
                'permitidos': permitido,  # Hacemos un eval para convertirlo en diccionario
                'id_funcion': nodo.id_funcion,
                'orden': nodo.orden
            }
        # print("NOdOS: ", dic)
        session.close()
        # Devuelvo la respuesta con True y el diccionario de nodos como dato
        return respuesta(True, u'Se obtuvieron los nodos del modo correctamente', dato=dic)

    def obtener_nodo(self, id_nodo):

        session = self.cpbot.binded_session()
        try:
            nodos = self.cpbot.tabla('nodos')

            q_obj = session.query(nodos).filter(nodos.id_nodo == id_nodo).first()

            rta = respuesta(True, u'Se obtuvieron el filtro del nodo', dato=q_obj)
        except:
            rta = respuesta(False, u'No se pudo obtener el filtro del nodo: %s' % id_nodo)
        finally:
            session.close()
        return rta

    def obtener_funcion(self, id_funcion):
        '''
        Esta funcion se encarga de obtener el nombre de una funcion mediante su id

        :param id_funcion: La id de una funcion
        :return: (dict) Un diccionario con la informacion de la funcion, como el nombre o el menu
        '''

        # Conecto con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'funciones'
        funciones = self.cpbot.tabla('funciones')

        # Obtengo los datos que necesito
        q_obj = session.query(
            funciones.nombre,
            funciones.titulo
        )

        # Filtro por el id_funcion
        q_obj = q_obj.filter(funciones.id_funcion == id_funcion)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesión con la base de datos
        session.close()

        # Si no pude obtener el registro
        if registro is None:
            # Devuelvo la respuesta con False
            return respuesta(False, u'No se pudo obtener el nombre de la funcion con id: %s' % id_funcion)

        # Inicializo un diccionario
        dic = {}
        dic['nombre'] = registro.nombre  # Guardo el nombre de la funcion
        dic['titulo'] = registro.titulo  # Guardo el titulo de la funcion

        # Devuelvo una respuesta con True y el diccionario como dato
        return respuesta(True, u'Se obtuvieron los datos de la funcion con exito', dato=dic)

    def obtener_posicion_en_arbol(self, id_charla):
        '''
        Esta funcion se encarga de obtener en la base de datos el ultimo nodo en el que usuario estuvo posicionado

        :param id_charla: La id de una charla en curso.
        :return: (respuesta) Una respuesta con estado y un diccionario como dato en caso de ser True
        '''

        # Conecto con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla logs
        logs = self.cpbot.tabla('logs')

        # Busco los datos que necesito
        q_obj = session.query(
            logs.id_nodo
        )

        # Filtro por la id_charla
        q_obj = q_obj.filter(logs.id_charla == id_charla)

        # Ordeno por el ultimo id_log
        q_obj = q_obj.order_by(logs.id_log.desc())

        # Obtengo el primer elemento del order_by
        registro = q_obj.first()

        # Cierro la sesion con la base de datos
        session.close()

        # Si no pude obtener el elemento
        if registro is None:
            # Devuelvo la respuesta con False
            return respuesta(False, u'No se pudo obtener el ultimo nodo - id_charla: %s' % id_charla)

        # Inicializo un diccionario que devolvere al finalizar la funcion
        dic = {}
        dic['id_nodo'] = registro.id_nodo

        # Devuelvo la respuesta con True y el diccionario como dato
        return respuesta(True, u'Se obtuvo el ultimo nodo con exito', dato=dic)

    def obtener_titulo_funcion(self, id_funcion):
        '''
        Esta funcion se encarga de buscar el titulo (del menu) mediante la id_funcion

        :param id_funcion: (int) La id de una funcion
        :return: Un diccionario con Estado, Respuesta y Dato (de ser posible)
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'funciones'
        funciones = self.cpbot.tabla('funciones')

        # Obtengo los datos que necesito
        q_obj = session.query(
            funciones.titulo
        )

        # Filtro por el id_funcion
        q_obj = q_obj.filter(id_funcion == funciones.id_funcion)

        # Obtengo el primer registro
        registro = q_obj.first()

        # Cierro la sesion con la base de datos
        session.close()

        # Si no pude obtener el registro
        if registro is None:
            # Devuelvo una respuesta con estado False
            return respuesta(False, u'No se pudo obtener el titulo de la funcion: %s' % id_funcion)

        # Si pude obtener el registro, lo devuelvo junto con la respuesta en True
        return {'Estado': True, 'Respuesta': u'Se obtuvo con exito el titulo de la funcion', 'Dato': registro.titulo}

    def obtener_nodo_inicial(self, id_modo):
        '''
        Esta funcion se encarga de obtener el nodo inicial del arbol dada la id_modo

        :param id_modo: El id modo que utiliza un cliente
        :return: (respuesta) Una respuesta con Estado, String y Dato (de ser posible)
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Obtengo la tabla 'nodos'
        nodos = self.cpbot.tabla('nodos')

        # Obtengo el dato que necesito
        q_obj = session.query(
            nodos.id_nodo
        )

        # busca en nodos donde coincida el id modo y donde o bien no tenga padre o sea principal (creo que solo tendria
        # que validar que sea el principal, total hay un principal por modo)
        q_obj = q_obj.filter(and_(id_modo == nodos.id_modo,
                                  or_(nodos.padre == None, nodos.padre == ''),
                                  nodos.principal == 1))

        # Guardo el primer registro
        registro = q_obj.first()

        # Cierro la sesion con la base de dato
        session.close()

        if registro is None:  # Si no pude encontrar el registro
            # Devuelvo la respuesta con estado False y un mensaje
            return respuesta(False, u'No se encontro el nodo inicial del id_modo: %s' % id_modo)

        # Si pude obtener el registro lo devuelvo junto con la respuesta
        return respuesta(True, u'Se obtuvo el nodo inicial con exito', dato=registro.id_nodo)

    def obtener_texto_tag(self, id_tag):
        '''
        Esta funcion se encarga de obtener la informacion de un tag mediante su id_tag

        :param id_tag:La id del tag (puede ser un nombre)
        :return: (registro)
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'tags'
        tags = self.cpbot.tabla('tags')

        # Obtengo los datos que necesito
        q_obj = session.query(
            tags.texto
        )

        # Filtro por el id_tag
        q_obj = q_obj.filter(tags.id_tag == id_tag)

        # Guardo el primer registro
        registro = q_obj.first()

        # Cierro la sesion con la base de datos
        session.close()

        # Si no pude obtener ningun registro
        if registro is None:
            return respuesta(False,
                             u'No se pudo obtener el texto para el id_tag: %s' % id_tag)  # Devuelvo la respuesta con False

        # Devuelvo la respuesta junto con el texto como dato
        return respuesta(True, u'Se obtuvo el texto del id_tag con exito', dato=registro.texto)

    def verificar_seleccion_usuario(self, seleccion_usuario, id_charla):
        '''
        Esta funcion se encarga de verificar a que nodo debe dirigirse el usuario basado en su seleccion

        :param seleccion_usuario: Un mensaje que envio el usuario
        :param id_charla: La id_charla
        :return: (dict) Una respuesta con Estado y un Dato de ser posible
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'logs'
        logs = self.cpbot.tabla('logs')

        # Obtengo todos los campos de la tabla
        q_obj = session.query(
            logs
        )

        # Filtro mediante la id_charla y la seleccion del usuario
        q_obj = q_obj.filter(id_charla == logs.id_charla, logs.estado == 'activo', seleccion_usuario == logs.opcion)

        # Obtengo el primer registro
        registro = q_obj.first()
        session.close()
        # Si no pude obtener el registro
        if registro is None:
            # Devuelvo una respuesta con False
            return respuesta(False, u'No se pudo obtener el log para la seleccion: %s' % seleccion_usuario)

        # Si pude obtener el registro devuelvo la respuesta con el registro como dato
        return respuesta(True, u'Se obtuvo el log con exito', dato=registro)

    def incrementar_intentos_fallidos(self, id_charla):
        '''
        Esta funcion se encarga de incrementar en 1 los intentos fallidos de una charla

        :param id_charla: El id de una charla dada
        :return: (dict) Una respuesta con Estado y Dato de ser posible
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'charla'
        charla = self.cpbot.tabla('charla')

        # Obtengo los intentos fallidos al momento de llamar la funcion
        q_obj = session.query(
            charla.intentos_fallidos
        )

        # Filtro por el id_charla
        q_obj = q_obj.filter(charla.id_charla == id_charla)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Si no pude obtener los intentos
        if registro is None:
            session.close()
            # Devuelvo la respuesta con False y un mensaje
            return respuesta(False, u'No se pudieron obtener los intentos para la charla: %s' % id_charla)

        # Si pude obtener los intentos le sumo 1
        intentos_actualizado = registro.intentos_fallidos + 1

        # Creo el diccionario con el dato a actualizar
        dic = {
            'intentos_fallidos': intentos_actualizado
        }

        # Actualizo el parametro en la tabla
        q_obj.update(dic, synchronize_session=False)

        # Commiteo la sesion
        session.commit()

        # Cierro la sesion
        session.close()

        # Devuelvo la respuesta con True y los intentos fallidos como dato
        return respuesta(True, u'Se aumento la cantidad de intentos correctamente', dato=intentos_actualizado)

    def actualizar_posicion_usuario(self, id_charla, id_nodo):
        '''
        Esta funcion se encarga de actualizar el ultimo nodo en la tabla 'charla'

        :param id_nodo: La id del nodo al que se movio el usuario
        :param id_charla: La id de una charla
        :return: (dict) Una respuesta con Estado y Dato de ser necesario
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'charla'
        charla = self.cpbot.tabla('charla')

        # Obtengo solamente el campo id_ultimo_nodo
        q_obj = session.query(
            charla.id_ultimo_nodo
        )

        # Filtro por el id de la charla
        q_obj = q_obj.filter(charla.id_charla == id_charla)

        # Obtengo el primer registro para verificar que existe la charla
        registro = q_obj.first()

        # Si no pude obtener el registro
        if registro is None:
            session.close()
            # Devuelvo una respuesta con False
            return respuesta(False, u'Fallo al actualizar el ultimo nodo de la charla: %s' % id_charla)

        # Creo el diccionario con el dato a actualizar
        dic = {
            'id_ultimo_nodo': id_nodo
        }

        # Actualizo el campo en la tabla
        q_obj.update(dic, synchronize_session=False)

        # Commiteo la session
        session.commit()

        # Cierro la session con la base de datos
        session.close()

        # Devuelvo la respuesta con True, un string y el nodo actualizado como dato
        return respuesta(True, u'Se actualizo el ultimo nodo con exito', dato=id_nodo)

    def actualizar_estado_logs(self, id_charla):
        '''
        Esta funcion se encarga de actualizar el estado de todos los logs de una charla.

        :param id_charla: La id de una charla
        :return: (dict) Una respuesta con Estado, respuesta y un Dato de ser posible.
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'logs'
        logs = self.cpbot.tabla('logs')

        # Obtengo el dato a actualizar de la tabla
        q_obj = session.query(
            logs.estado
        )

        # Filtro por el id_charla
        q_obj = q_obj.filter(id_charla == logs.id_charla, logs.estado == 'activo')

        # Obtengo todos los registros
        registros = q_obj.all()

        # Si no pude obtener los registros
        if len(registros) == 0:
            # Cierro la sesion con la base de datos
            session.close()

            # Devuelvo la respuesta con False y un mensaje
            return respuesta(False, u'No se encontraron logs para la id_charla: %s' % id_charla)

        # Creo el diccionario con el dato a actualizar
        dic = {
            'estado': 'inactivo'
        }

        # Actualizo el campo en la tabla logs
        q_obj.update(dic, synchronize_session=False)

        # Commiteo la sesion
        session.commit()

        # Cierro la sesion con la base de datos
        session.close()

        # Devuelvo una respuesta con True y un mensaje
        return respuesta(True, u'Se actualizaron los logs de la charla: %s' % id_charla)

    def actualizar_datos_servicio(self, id_charla, datos_servicio):
        '''
        Esta funcion se encarga de actualizar los datos del servicio en la tabla 'charla'

        :param id_charla (int):
        :param datos (dict): Informacion relevante sobre un servicio
        :return: (dict) Una respuesta con Estado y de ser posible un dato.
        '''
        print(datos_servicio)

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'charla'
        charla = self.cpbot.tabla('charla')

        # Obtengo el campo datos_servicio
        q_obj = session.query(
            charla.datos_servicio
        )

        # Filtro por el id_charla
        q_obj = q_obj.filter(charla.id_charla == id_charla)

        # Obtengo el registro
        registro = q_obj.first()

        # Si no pude obtener el registro
        if registro is None:
            # Cierro la sesion con la base de datos
            session.close()

            # Devuelvo la respuesta con False y un mensaje
            return respuesta(False, u'No se pudo obtener los datos_servicio de la charla: %s' % id_charla)

        # Creo el diccionario con el campo a actualizar
        dic = {
            'datos_servicio': datos_servicio
        }

        # Actualizo el campo en la tabla 'charla'
        q_obj.update(dic, synchronize_session=False)

        # Commiteo la sesion
        session.commit()

        # Cierro la sesion con la base de datos
        session.close()

        # Devuelvo una respuesta con True y un mensaje
        return respuesta(True, u'Se actualizaron los datos_servicio de la charla: %s' % id_charla)

    def obtener_logs_coincidentes(self, tipo_seleccion, id_charla):
        '''
        Esta funcion se encarga de obtener todas las opciones posibles basandonos en el tipo de mensaje que se esperaba

        Ejemplo: si un usuario envia un texto, y nosotros disponemos de tres diferentes nodos que aceptan este tipo de
                 mensajes, los logs con todos sus datos de cada uno de estos nodos sera devuelto en esta funcion.

        :param tipo_seleccion (str): Un tipo de opcion correspondiente a la tabla logs
        :param id_charla (int): El id de la charla
        :return: (dict) Una respuesta con Estado, un mensaje y de ser posible un Dato
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'logs'
        logs = self.cpbot.tabla('logs')

        # Hago la query a todos los campos de la tabla logs
        q_obj = session.query(
            logs
        )

        # Filtro por el tipo_seleccion, si el log esta activo y la id_charla coincide
        q_obj = q_obj.filter(logs.estado == "activo", logs.id_charla == id_charla)

        # Obtengo todos los que cumplen el criterio
        registros = q_obj.all()
        # print("resigtros",registros)
        # Cierro la sesion con la base de datos
        session.close()

        if len(registros) == 0:  # Si no encontre ningun registro que cumpla el criterio
            return respuesta(False, u'No se encontraron logs con el tipo_seleccion: %s' % tipo_seleccion)

        lista = []  # Inicializo la lista que devolvere con la respuesta

        for registro in registros:  # Itero a traves de los registros

            dic = registro.__dict__  # Convierto el registro en diccionario

            # Verificamos que el log contenga el tipo de mensaje que ingreso el usuario
            if tipo_seleccion.replace('<', '').replace('>', '') not in dic['opcion']:
                continue

            lista.append(dic)  # Agrego el diccionario a la lista

        if not lista:  # Si no encontre ningun registro que cumpla el criterio
            return respuesta(False, u'No se encontraron logs con el tipo_seleccion.lista: %s' % tipo_seleccion)

        return respuesta(True, u'Se obtuvieron los logs con el tipo_seleccion', dato=lista)

    def obtener_log_estado(self, id_charla):
        '''
        Esta funcion se encarga de obtener todos los campos de un log que coincida con el id de una charla dado.

        :param id_charla (int): El id de una charla
        :return: (dict) Una respuesta con Estado, un mensaje y de ser posible un dato
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla logs
        logs = self.cpbot.tabla('logs')

        # Hago la query
        q_obj = session.query(
            logs
        )

        # Filtro por el id de la charla y que su estado sea "estado"
        q_obj = q_obj.filter(id_charla == logs.id_charla, logs.estado == "activo", logs.espera == "<estado>")

        # Intento guarda el primer registro
        registro = q_obj.first()
        session.close()
        # Si no pude encontrar ningun registro
        if registro is None:
            # Devuelvo la respuesta con False y un mensaje
            return respuesta(False, u'No se encontro el log con estado_servicio de la charla: %s' % id_charla)

        # Devuelvo una respuesta con True y el diccionario como dato
        return respuesta(True, u'Se encontro el log de estado_servicio para la charla', dato=registro)

    def obtener_predeterminado_funcion(self, id_funcion):
        '''
        Esta funcion se encarga de obtener el titulo predeterminado de una funcion dada

        :param id_funcion (int): El id de una funcion
        :return: (dict) Una respuesta con Estado, un mensaje y de ser posible un dato.
        '''

        # Abro la sesion con la BD
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'funciones'
        funciones = self.cpbot.tabla('funciones')

        # Hago la query
        q_obj = session.query(
            funciones.predeterminado
        )

        # Filtro por el id_funcion
        q_obj = q_obj.filter(funciones.id_funcion == id_funcion)

        # Intento obtener el primer registro
        registro = q_obj.first()

        session.close()

        # Si no pudimos obtener ningun registro
        if registro is None:
            return respuesta(False, u'No se encontro la funcion con id: %s' % id_funcion)

        # Devuelvo una respuesta con True y el texto predeterminado como dato
        return respuesta(True, u'Se encontro el titulo predeterminado con exito', dato=registro.predeterminado)

    def cambiar_modo_charla(self, id_charla, id_modo):
        '''
        Esta funcion se encarga de cambiar el modo de una charla.

        :param id_charla (int): El id de una charla
        :param id_modo (str): El id de un modo
        :return: (dict) Una respuesta con Estado, un mensaje y de ser necesario un dato.
        '''

        # Abro la sesion
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'charla'
        charla = self.cpbot.tabla('charla')

        # Obtengo solamente el campo id_ultimo_nodo
        q_obj = session.query(
            charla.id_modo
        )

        # Filtro por el id de la charla
        q_obj = q_obj.filter(charla.id_charla == id_charla)

        # Obtengo el primer registro para verificar que existe la charla
        registro = q_obj.first()

        # Si no pude obtener el registro
        if registro is None:
            session.close()
            # Devuelvo una respuesta con False
            return respuesta(False, u'Fallo al actualizar el ultimo nodo de la charla: %s' % id_charla)

        # Creo el diccionario con el dato a actualizar
        dic = {
            'id_modo': id_modo
        }

        # Actualizo el campo en la tabla
        q_obj.update(dic, synchronize_session=False)

        # Commiteo la session
        session.commit()

        # Cierro la session con la base de datos
        session.close()

        # Devuelvo la respuesta con True, un string y el nodo actualizado como dato
        return respuesta(True, u'Se actualizo el ultimo nodo con exito', dato=id_modo)

    def finalizar_charlas_viejas(self, hora_limite):

        # Abro la sesion con la BD
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'funciones'
        charla = self.cpbot.tabla('charla')

        # Hago la query
        q_obj = session.query(
            charla.estado,
            charla.telefono_origen
        )

        q_obj = q_obj.filter(charla.estado == 'activa', charla.fecha_hora_ultimo_mensaje < hora_limite)

        registros = q_obj.all()

        # Si no pude obtener registros
        if len(registros) == 0:
            # Cierro la sesion con la base de datos
            session.close()

            # Devuelvo una respuesta con False
            return respuesta(False, u'No hay charlas para finalizar')

        # Creo el diccionario con el dato a actualizar
        dic = {
            'estado': 'finalizada'
        }

        # Actualizo el campo en la tabla
        q_obj.update(dic, synchronize_session=False)

        # Commiteo la session
        session.commit()

        # Cierro la session con la base de datos
        session.close()

        lista = []
        for registro in registros:
            lista.append(registro.telefono_origen)

        # Devuelvo la respuesta con True, un string y el nodo actualizado como dato
        return respuesta(True, u'Se actualizaron las charlas', dato=lista)

    def actualizar_fecha_charla(self, id_charla):

        # Abro la sesion con la BD
        session = self.cpbot.binded_session()

        # Conecto con la tabla 'funciones'
        charla = self.cpbot.tabla('charla')

        # Hago la query
        q_obj = session.query(
            charla.fecha_hora_ultimo_mensaje
        )

        q_obj = q_obj.filter(charla.id_charla == id_charla)

        registro = q_obj.first()

        # Si no pude obtener registros
        if len(registro) == 0:
            # Cierro la sesion con la base de datos
            session.close()

            # Devuelvo una respuesta con False
            return respuesta(False, u'No hay charla para cambiar la fecha y hora')

        # Creo el diccionario con el dato a actualizar
        dic = {
            'fecha_hora_ultimo_mensaje': datetime.datetime.now()
        }

        # Actualizo el campo en la tabla
        q_obj.update(dic, synchronize_session=False)

        # Commiteo la session
        session.commit()

        # Cierro la session con la base de datos
        session.close()

        # Devuelvo la respuesta con True, un string y el nodo actualizado como dato
        return respuesta(True, u'Se actualizaron las charlas')

    def obtener_finalizar_en_arribo(self, id_empresa):
        '''
        Esta funcion se encarga de obtener el campo "finalizar_en_arribo" de la tabla "parametros"

        :param id_empresa: (str) La id de la empresa
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla "parametros"
        parametros = self.cpbot.tabla("parametros")

        # Obtengo el dato que necesito
        q_obj = session.query(
            parametros.finalizar_en_arribo
        )

        # Filtro por el id empresa
        q_obj = q_obj.filter(parametros.id_empresa == id_empresa)

        # Obtengo el primer elemento
        registro = q_obj.first()

        session.close()

        # Si no pude obtener el registro
        if registro is None:
            return respuesta(False, u'No se encontro la empresa: %s' % id_empresa)

        # Devolvemos el elemento junto con True
        return respuesta(True, u'Se obtuvo correctamente el campo finalizar_en_arribo',
                         dato=registro.finalizar_en_arribo)

    def verificar_modo_puede_volver(self, id_modo):
        '''
        Esta funcion se encarga de devolver el booleano si el id_modo ingresado debe volver al modo del que fue llamado.
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla modos
        modos = self.cpbot.tabla('modos')

        # Hago la query
        q_obj = session.query(
            modos.volver_modo_anterior
        )

        # Filtro por el id modo
        q_obj = q_obj.filter(modos.id_modo == id_modo)

        # Intentamos obtener el registro
        registro = q_obj.first()

        session.close()

        # Si no pudimos obtener el registro
        if registro is None:
            return respuesta(False, u'No se encontro el modo %s' % id_modo)

        return respuesta(True, u'Se encontro el booleano del modo con exito', dato=registro.volver_modo_anterior)


def testeamebb():
    cpbot = CpBotBd()
    dicky = cpbot.obtener_nodos("1")
    pprint.pprint(dicky)
