# -*- coding: utf-8 -*-
import re
import unidecode
import datetime
import pprint
import requests
from at.cpbot.funcionescpbot import *
from at.cpbot.bdcpbot import CpBotBd
import at.cpbot.atapisadapter as atapisadapter
from at.cpbot.cpadapter import CpBotAdapter, CallePostgres
import at.cpbot.funcionestags as funcionestags
from at.cpbot.modospermitidos import *
from at.utiles.conversiones import respuesta, decode_string, unicode_to_str, str_to_json
import at.cpbot.tagutiles as tagutiles
from at.map.calle import CallePostgres as Calles
import at.utiles.direcciones as Direccion

cpbotbd = CpBotBd()  # Instanciacion de la clase que interactua con la BD 'cpbot'
cpadapter = CpBotAdapter()  # Instanciacion de la clase que interactura con la BD 'cp2000'
postgresadapter = CallePostgres()  # Instanciacion de la clase que interactua con la BD 'gis2'
callespostgres = Calles()

'''
Este archivo guarda todas las funcionalidades que se pueden agregar a un arbol, definidas como clases.
De esta manera permitimos que los menus dinamicos sean generados en base a los nodos hijos del nodo que contiene cada clase/funcion
SOLO DEVUELVE LOS DINAMICOS DE LOS NODOS. LAS EJECUCIONES ESTAN EN FUNCIONESCPBOT
'''


class Funcion(object):

    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):

        self.titulo = titulo
        self.parametros = parametros
        self.funcion = funcion
        self.id_funcion = id_funcion
        self.id_nodo = id_nodo
        self.orden = orden

    # todo rehacer
    def ejecutar_funcion(self):
        '''
        Esta funcion se encargara de ejecutar la funcion del nodo y de ofrecer un menu dinamico basado en los hijos.

        :return: (respuesta) Una respuesta con estado y de ser posible un dato(dict).
        '''

        funcion = str(self.funcion)
        parametros = str(self.parametros)
        funcion_completa = funcion + "(" + parametros + ")"

        # Ejecuto la funcion asociada al nodo
        return eval(funcion_completa)

    def obtener_id_tags(self, titulo):
        regex = '(?<=\<)(.*?)(?=\>)'
        salida = re.findall(regex, titulo)
        return salida

    def obtener_tag(self, tag):
        '''
        Esta funcion se encarga de obtener el texto de un tag buscando en la base de datos

        :param tag (str): El id_tag
        :return: El texto referente al id_tag
        '''

        rta = cpbotbd.obtener_tag(tag)

        if rta['Estado'] is False:
            return

        return rta['Dato']

    def reemplazar_en_titulo(self, id_tag, texto_tag, titulo_original):
        '''
        Esta funcion se encarga de reemplazar todos los tags dentro del titulo por los textos ya obtenidos.

        :param id_tag (str): Un texto unico para identificar al tag.
        :param texto_tag (str): El texto generado dinamicamente o modificable de un tag.
        :param titulo_original (str): El titulo original que cambiaremos.
        :return: (str) El titulo modificado
        '''

        # Guardamos el regex que utilizaremos para identificar los tags que se ven como: <ID_TAG>
        regex = "<" + id_tag + ">"

        try:  # Iniciamos un try para levantar excepcion en caso de fallar

            # Reemplazamos en el titulo
            nuevo_titulo = titulo_original.replace(regex, texto_tag)

            if nuevo_titulo == titulo_original:  # Si el titulo original es igual al titulo que deberia estar cambiado
                raise Exception(u'ERROR EN TAG %s' % id_tag)  # Levanto una excepcion

            # Devuelvo el titulo modificado
            return nuevo_titulo

        except Exception as error:
            raise Exception(u'No se pudo reemplazar el tag', error)

    def obtener_titulo(self):
        '''
        Esta funcion se encarga de obtener el titulo del menu basado en la funcion en la que estamos

        :return: El titulo proveniente de la base de datos
        '''

        try:  # Iniciamos un try por si fallamos devolver el titulo predeterminado

            # Utilizando la clase de CpBotBd obtengo el titulo y guardo la respuesta
            rta = cpbotbd.obtener_titulo_funcion(self.id_funcion)
            # aca devuelve el titulo de la tabla funciones por ejemplo "seleccionar_direccion"

            # Si la respuesta es False
            if rta['Estado'] is False:
                # Devuelvo la respuesta
                return rta

            titulo_original = rta['Dato']  # Guardo el titulo original en una variable

            id_tags = self.obtener_id_tags(rta['Dato'])  # Lista de tags para buscar

            if len(id_tags) == 0:  # Si el titulo no tiene tags
                return titulo_original  # Devuelvo el titulo original

            tags = [self.obtener_tag(tag) for tag in id_tags]  # Guardo los textos de los tags

            if len(tags) == 0:  # Si no hay tags
                raise Exception  # Levanta una excepcion para devolver el titulo predeterminado

            dic = {}  # Creo un diccionario donde ire guardando el id_tag y su texto

            # Itero a traves de los tags obtenidos
            for tag in tags:

                if tag['tipo'] == "sistema":  # Si es un tag del sistema

                    dic[tag['id_tag']] = tag['texto']  # Guardo el texto del tag en un diccionario

                if tag['tipo'] == "funcional":  # Si es un tag funcional

                    rta_eval = eval("funcionestags." + tag['texto'] + "(self.parametros)")

                    if rta_eval['Estado'] is False:  # Si no pude ejecutar la funcion
                        raise Exception(u'ERROR EN TAG:' % tag)  # Levanto una excepcion

                    texto_generado = rta_eval['Dato']  # Guardo el texto generado por la funcion

                    dic[tag['id_tag']] = texto_generado  # Guardo en el diccionario junto con el id_tag

                if tag['tipo'] == "usuario":  # Si es un tag creado por el usuario

                    dic[tag['id_tag']] = tag['texto']  # Guardo el texto del tag en un diccionario

            # Itero a traves del diccionario
            for key, value in dic.items():
                # Utilizamos la funcion para modificar los tags en el titulo
                titulo_original = self.reemplazar_en_titulo(key, value, titulo_original)

            # Devolvemos el titulo modificado
            return titulo_original

        except Exception as error:
            # print("ERROR EN ARMAR TITULO: ", error)
            # Buscar titulo predeterminado
            rta = cpbotbd.obtener_predeterminado_funcion(self.id_funcion)

            # Si no pude obtener el titulo predeterminado
            if rta['Estado'] is False:
                return rta  # Devuelvo la respuesta

            # Devolver titulo predeterminado
            return rta['Dato']


class Saludar(Funcion):

    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Saludar, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        '''
        Esta funcion se encarga de obtener el menu dinamico de una funcion

        :return: (list) Un menu dinamico basado en la funcion ejecutada
        '''

        return respuesta(False, u'Esta funcion no tiene menu dinamico', dato=None)


class Buscar_por_google(Funcion):
    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Buscar_por_google, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        '''
        Esta funcion se encarga de obtener el menu dinamico de una funcion

        :return: (list) Un menu dinamico basado en la funcion ejecutada
        '''

        # Intento de obtener el nombre de la ciudad
        rta_obtener_ciudad = postgresadapter.obtener_nombre_ciudad(self.parametros['id_ciudad'])

        if rta_obtener_ciudad['Estado'] is False:  # Si fallo al obtener el nombre de la ciudad
            return rta_obtener_ciudad

        # print("PARAMETROS: ", self.parametros)

        ciudad_cliente = rta_obtener_ciudad['Dato']  # Guardo el nombre de la ciudad

        lat_lng = self.parametros['coordenadas'].split(',')  # Guardo las coordenadas del cliente

        localidad = {'lat': lat_lng[0], 'lng': lat_lng[1]}

        parametros = {}
        # Guardo cada parámetro
        parametros['id_api'] = 1  # 1 es googlemaps
        parametros['id_funcion'] = 5  # id_funcion 5 = "obtener_direcciones_posibles"
        parametros['id_producto'] = 5  # cpbot
        parametros['id_ciudad'] = int(self.parametros['id_ciudad'])
        parametros['id_cliente'] = self.parametros['id_empresa']
        parametros['msj'] = self.parametros['mensaje']  # aca tiene que estar la direccion
        parametros['localidad'] = localidad

        ip_atapis = self.parametros['ip_atapis']

        rta_obtener_direcciones_posibles = atapisadapter.obtener_direcciones_posibles(ip_atapis, parametros)
        # print("OBTENER DIRECCIOENS RTA: ", rta_obtener_direcciones_posibles)
        if rta_obtener_direcciones_posibles['Estado'] is False:

            #  Si la respuesta de google nos devolvio cero resultados
            if 'Dato' in rta_obtener_direcciones_posibles and rta_obtener_direcciones_posibles['Dato'] == {}:

                # Intentamos obtener el tag de direccion_no_encontrada
                rta_obtener_tag = cpbotbd.obtener_texto_tag('direccion_no_encontrada')

                if rta_obtener_tag['Estado'] is False:  # Si no pudimos obtener el tag
                    # Levanto la excepcion
                    raise Exception("Falla al obtener el tag de direccion_no_encontrada", rta_obtener_tag)

                # Guardo en un diccionario los datos relevantes de la direcciones
                dic = {
                    'id_nodo': self.id_nodo,  # La id del nodo
                    'texto': rta_obtener_tag['Dato'],
                    'opcion': "<mapa>",  # El numero de opcion
                    'espera': "<ubicacion>",
                    'datos': {'direccion': None, 'mensaje': self.parametros['mensaje']},
                    'orden': self.orden
                }

                return respuesta(True, u'Se obtuvo el dinamico con exito (sin direcciones)', dato=[dic])

            # Devuelvo la respuesta
            raise Exception(u'Falla al obtener direcciones posibles:', rta_obtener_direcciones_posibles['Respuesta'])

        direcciones = rta_obtener_direcciones_posibles['Dato']  # Guardo las direcciones obtenidas

        lista = []  # Inicializo la lista a devolver

        contador = 1  # Inicializo el contador para el dinamico

        for direccion, coordenadas in direcciones.items():  # Itero a traves de las direcciones obtenidas

            direccion_separada = direccion.split(', ')  # Separamos la direccion y la localidad

            try:  # Inicio un try para continuar el loop en caso de fallar
                # Si la ciudad de la direccion vino con el codigo postal
                if len(direccion_separada[1]) <= 3:
                    direccion_separada[1] = ciudad_cliente  # Reemplazamos el codigo postal por la localidad
            except:
                continue

            #  Limpiamos la direccion de cualquier tipo de acento o letras raras
            direccion_final = ', '.join([unidecode.unidecode(palabra) for palabra in direccion_separada])

            # print("DIRECCION FINAL: ", direccion_final)

            # Guardo en un diccionario los datos relevantes de la direcciones
            dic = {
                'id_nodo': self.id_nodo,  # La id del nodo
                'texto': direccion_final,
                'opcion': "<numero>",  # El numero de opcion
                'espera': contador,
                'datos': {'direccion': direccion, 'latitud': coordenadas[0], 'longitud': coordenadas[1]},
                'orden': self.orden
            }

            lista.append(dic)

            contador += 1

        return respuesta(True, u'Se encontraron direcciones exitosamente', dato=lista)


class Obtener_direcciones(Funcion):
    """
    el menu dinamico buscara las direcciones frecuentes
    """

    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Obtener_direcciones, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        # todo obtener la cantidad de direcciones por parametro
        return self._obtener_direcciones_frecuentes_cp2000(cantidad_direcciones=3)

    def _obtener_direcciones_frecuentes_cp2000(self, cantidad_direcciones):
        """
        devuelve direcciones frecuentes de un cliente segun el telefono
        """
        # Intento obtener las direcciones mas utilizadas de un numero de telefono
        rta = cpadapter.obtener_direcciones(self.parametros['charla']['telefono_origen'], cantidad_direcciones)
        # Si fallo al obtener las direcciones
        if rta['Estado'] is False:
            return rta

        direcciones = rta['Dato']
        lista = self.formatear_direcciones(direcciones)
        return respuesta(True, u'Se obtuvieron las direcciones con exito', dato=lista)

    def formatear_direcciones(self, direcciones):
        lista = []  # Inicializo una lista donde guardare las direcciones
        # Itero a traves de las direcciones que es un json
        for numero_opcion, value in direcciones.items():
            direccion = unidecode.unidecode(decode_string(value.Direccion))
            # Guardo en un diccionario los datos relevantes de la direcciones

            # si hay altura la agrega a la direccion
            if value.LetraAltura:
                direccion = direccion + ' ' + value.LetraAltura
            # si hay depto la agrega a la direccion
            if value.Depto:
                direccion = direccion + ' ' + value.Depto

            dic = {
                'id_nodo': self.id_nodo,  # La id del nodo
                'texto': direccion,  # La direccion
                'opcion': "<numero>",  # El numero de opcion
                'espera': numero_opcion,
                'datos': {'direccion': direccion,
                          'latitud': value.Latitud,
                          'longitud': value.Longitud},
                'orden': self.orden
            }

            lista.append(dic)  # Añado a la lista el diccionario creado
        return lista


class Cargar_servicio(Funcion):
    """

    esta clase no esta en uso!!!!
    esta clase no esta en uso!!!!
    esta clase no esta en uso!!!!

    """

    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Cargar_servicio, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        print("\nentra en dinamico de cargar servicio")
        pprint.pprint(self.parametros)

        cliente = self.parametros
        # todo no entiendo que es esto
        if 'id_empresa' not in self.parametros:  # Si no encontramos id_empresa en parametros (porque hizo salto de nodo)
            cliente = self.parametros['cliente']  # Seteamos la key cliente como variable

        # esto pasa de string a json o algo asi
        datos_log = self.parametros['datos_armar_dinamico']['Dato'][0]

        direccion = datos_log['direccion']

        nombre = "(desconocido)"

        # le dice al callProcessor que cargue el servicio
        rta = cpadapter.cargar_servicio(
            empresa=cliente['id_empresa'],  # id_empresa
            hash='',
            telefono=self.parametros['charla']['telefono_origen'],  # Telefono del usuario
            nombre=nombre,  # Como nombre utilizamos el telefono del usuario
            direccion=direccion,  # La direccion donde se pidio el movil
            telefonista="whatsapp",  # Ponemos como telefonista el CpBot
            latitud=float(datos_log['latitud']) * 3600,
            longitud=float(datos_log['longitud']) * 3600,
            host="http://" + cliente['ip_callprocessor'],  # Donde se encuentra callprocessor
            filtros=obtener_filtros_palabras(self.parametros['charla']['filtros_palabras'])

        )

        if rta['Estado'] is False:  # Si la respuesta es False
            return rta  # Devuelvo la respuesta

        # agrega la direccion a lo que va a retornar
        rta['Dato'][
            'direccion'] = direccion  # añade la direccion en el dato de la respuesta, xq el callprocesor no la devuelve

        return respuesta(False, '', '')


class Estado_servicio(Funcion):

    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Estado_servicio, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        '''
        Esta funcion se encarga de obtener el menu dinamico de una funcion

        :return: (string) Un menu dinamico basado en la funcion ejecutada
        '''
        '''
        Esta funcion se encarga de obtener el menu dinamico de una funcion

        :return: (list) Un menu dinamico basado en la funcion ejecutada
        '''

        rta = cpbotbd.obtener_texto_tag(u'estado_servicio')  # Obtengo el texto para ofrecer el estado servicio

        # Si la respuesta es False
        if rta['Estado'] is False:
            return rta  # Devuelvo la respuesta

        # Intento formatear el texto
        rta_formatear_texto = tagutiles.formatear_texto(rta['Dato'], self.parametros)

        if rta_formatear_texto['Estado'] is False:  # Si no puedo formatear el texto
            return rta_formatear_texto

        texto = rta_formatear_texto['Dato']  # Guardo el texto a ofrecer

        lista = []  # Inicializo la lista que devolvere al final

        # Creo el diccionario con las opciones 'Confirmar'
        estado = {
            'id_nodo': self.id_nodo,  # La id del nodo
            'texto': texto,  # No tiene texto ya que queda mejor en el titulo
            'opcion': '<texto>',  # El usuario debe ingresar un texto escrito (confirmar)
            'espera': "<estado>",  # Que debe enviar el usuario
            'datos': self.parametros['charla']['datos_servicio'],  # Conservo los datos del servicio
            'orden': self.orden
        }

        lista.append(estado)  # Agrego el diccionario cancelar a la lista

        # Devuelvo una respuesta con True y la lista como dato
        return respuesta(True, u'Se obtuvo el dinamico de Estado_servicio', dato=lista)


class Finalizar_charla(Funcion):

    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Finalizar_charla, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        '''
        Esta funcion se encarga de obtener el menu dinamico de una funcion

        :return: (string) Un menu dinamico basado en la funcion ejecutada
        '''

        return respuesta(False, u'Esta funcion no tiene menu dinamico', dato=None)


class Sugerir(Funcion):
    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Sugerir, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        print("\nobtener dinimico de sugerir:")
        # pprint.pprint(self.parametros)

        lista = []  # Inicializo una lista donde guardare las direcciones

        direcciones = self.filtrar_direcciones()

        for numero_opcion, value in enumerate(direcciones):
            # print("numero opcion:", numero_opcion)
            # print("value:", value)
            # Guardo en un diccionario los datos relevantes de la direcciones
            dic = {
                'id_nodo': self.id_nodo,  # id del nodo
                'texto': value['direccion'] + ', ' + value['ciudad'],  # La direccion
                'opcion': "<numero>",  # El numero de opcion
                'espera': numero_opcion + 1,
                'datos': {'direccion': value["direccion"],
                          'latitud': value["latitud"],
                          'ciudad': value["ciudad"],
                          'longitud': value["longitud"]},
                'orden': self.orden
            }

            lista.append(dic)  # Añado a la lista el diccionario creado

        # # añade la opcion para cuando no quiere ninguna de las anteriores
        # ojo que debe haber un hijo de tipo mapa si activo nuevamente esta
        # opcion_otra = {
        #     'id_nodo': self.id_nodo,  # id del nodo
        #     'texto': 'Ninguna de las anteriores',  # La direccion
        #     'opcion': "<numero>",  # El numero de opcion
        #     'espera': len(lista) + 1,
        #     'datos': {'direccion': self.parametros['direccion'],
        #               'latitud': '',
        #               'ciudad': '',
        #               'longitud': ''},
        #     'orden': self.orden
        # }
        # lista.append(opcion_otra)  # Añado a la lista el diccionario creado

        return respuesta(True, u'Se obtuvieron las direcciones con exito', dato=lista)

    def filtrar_direcciones(self):
        direcciones = None
        try:
            direcciones = self.obtener_direcciones_encontradas()

            # direcciones = self.filtrar_por_coincidencia(direcciones)
            # direcciones = self.filtrar_duplicadas(direcciones)
            direcciones = self.filtrar_por_cantidad(direcciones, 4)
            # direcciones = self.filtrar_por_ciudad(direcciones)
            # direcciones = self.filtrar_por_cercania(direcciones)
        except:
            pass

        return direcciones

    def obtener_direcciones_encontradas(self):
        return self.parametros['datos_armar_dinamico']['Dato']

    def filtrar_por_cantidad(self, direcciones, cantidad_maxima):
        # print("direcciones")
        # print(direcciones)

        return direcciones[:cantidad_maxima]  # solo deja cierta cantidad porque sino va a ser muy largo el mensaje


class Buscar_direcciones(Funcion):
    """
    es el que con las apis arcgis o photon
    """

    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Buscar_direcciones, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        rta = cpbotbd.obtener_texto_tag(u'buscar_direcciones')

        # Si la respuesta es False
        if rta['Estado'] is False:
            return rta  # Devuelvo la respuesta

        # Creo un diccionario con los datos necesarios
        dic = {
            'id_nodo': self.id_nodo,
            'texto': rta['Dato'],
            'opcion': "<direccion>",
            'espera': "<direccion>",
            'datos': {},
            'orden': self.orden
        }

        # Devuelvo el diccionario dentro de una lista junto con la respuesta
        return respuesta(True, u'Se obtuvo el dinamico de buscar direcciones', dato=[dic])


class Cancelar(Funcion):
    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Cancelar, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        '''
        Esta funcion se encarga de obtener el menu dinamico de una funcion

        :return: (dict) Un menu dinamico basado en la funcion ejecutada
        '''
        return respuesta(False, u'Esta funcion no tiene menu dinamico', dato=None)


class Ingresar_ubicacion(Funcion):
    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Ingresar_ubicacion, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        '''
        Esta funcion se encarga de obtener el menu dinamico de una funcion

        :return: (string) Un menu dinamico basado en la funcion ejecutada
        '''

        print("dinamico de ingresar ubicacion:")

        pprint.pprint(self.parametros)
        rta = cpbotbd.obtener_texto_tag(u'ingresar_ubicacion')

        # Si la respuesta es False
        if rta['Estado'] is False:
            return rta  # Devuelvo la respuesta

        # todo negrada para que pueda guardar la direccion en el log y usarla despues en caso de necesitar
        # cargar el servicio
        try:
            direccion = self.parametros['datos_armar_dinamico']['Direccion']
            datos = {'direccion': direccion}
        except:
            datos = {}

        # Creo un diccionario con los datos necesarios
        dic = {
            'id_nodo': self.id_nodo,
            'texto': rta['Dato'],
            'opcion': "<mapa>",
            'espera': "<ubicacion>",
            'datos': datos,
            'orden': self.orden
        }

        # Devuelvo el diccionario dentro de una lista junto con la respuesta
        return respuesta(True, u'Se obtuvo el dinamico de Ingresar_ubicacion', dato=[dic])


class Pedir_direccion(Funcion):
    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Pedir_direccion, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        '''
        Esta funcion se encarga de obtener el menu dinamico de una funcion

        :return: (string) Un menu dinamico basado en la funcion ejecutada
        '''

        rta = cpbotbd.obtener_texto_tag(u'pedir_direccion')

        # Si la respuesta es False
        if rta['Estado'] is False:
            return rta  # Devuelvo la respuesta

        # Creo un diccionario con los datos necesarios
        dic = {
            'id_nodo': self.id_nodo,
            'texto': rta['Dato'],
            'opcion': "<direccion>",
            'espera': "<direccion>",  # Esperamos que ingrese una direccion para la ubicacion, pero no validamos
            'datos': {},
            'orden': self.orden
        }

        # Devuelvo el diccionario dentro de una lista junto con la respuesta
        return respuesta(True, u'Se obtuvo el dinamico de Ingresar_ubicacion', dato=[dic])


class Pasar_a_manual(Funcion):
    def __init__(self, titulo, parametros, funcion, id_funcion, id_nodo, orden):
        super(Pasar_a_manual, self).__init__(titulo, parametros, funcion, id_funcion, id_nodo, orden)

    def obtener_dinamico(self):
        '''
        Esta funcion se encarga de obtener el menu dinamico de una funcion

        :return: (dict) Un menu dinamico basado en la funcion ejecutada
        '''
        return respuesta(False, u'Esta funcion no tiene menu dinamico', dato=None)
