# -*- coding: utf-8 -*-
import datetime
import json
import re
import sys, os
import pprint
from at.utiles.http import enviar_por_http
from at.cpbot.bdcpbot import CpBotBd
from treelib import Node, Tree
from at.utiles.conversiones import respuesta, unicode_to_str
from at.cpbot.nodoat import NodoAt
from at.utiles.validaciones_generales import ValidacionesGenerales
import at.utiles.validar as validar
from decimal import Decimal
import at.cpbot.atmsjadapter as atmsjadapter
import at.utiles.direcciones as Direccion
from at.cpbot.sqla_repository_filtros_palabras import FiltrosRepository
from at.cpbot.domain.filtro import *

CP_BOT_BD = CpBotBd()  # Instancia responsable de hablar con la BD de CpBot
filtros_repository = FiltrosRepository()
validaciones = ValidacionesGenerales()  # Instancia responsable de validar diversas cosas


def finalizar_charlas_viejas(ipatmsj):
    '''
    Esta funcion se encarga de finalizar las charlas que tienen mas de una hora de antiguedad.
    Se ejecuta al iniciar el CpBot y tambien cada 10 minutos.

    :return: (dict) Una respuesta con estado y un mensaje
    '''

    hora_actual = datetime.datetime.now()  # Obtengo la hora actual
    hora_limite = hora_actual - datetime.timedelta(minutes=30)  # Obtengo la hora actual - una hora

    # Intentamos finalizar las charlas en el CpBot
    rta = CP_BOT_BD.finalizar_charlas_viejas(hora_limite)

    # Si no se pudo enviar el mensaje
    if rta['Estado'] is False:
        # Retorna el error
        return rta

    telefonos = rta['Dato']  # Guardamos lo telefonos de las charlas finalizadas
    print("IP ATMSJ:", ipatmsj)
    print("TELEFONOS RE HEAVY: ", telefonos)
    errores = 0  # Inicializo un contador de errores
    telefonos_fallaron = []  # Inicializo una lista de los telefonos que fallaron

    for telefono in telefonos:  # Por cada telefono
        print("TELEFONO: ", telefono)
        rta_finalizar_atmsj = atmsjadapter.finalizar_charla(ipatmsj, telefono)  # Finalizamos la charla en atmsj
        print("RTA FINALIZAR ATMSJ: ", rta_finalizar_atmsj)
        if rta_finalizar_atmsj['Estado'] is False:  # Si no pude finalizar la charla en atmsj
            errores += 1  # Aumento el contador de errores
            telefonos_fallaron.append(telefono)  # Agrego el telefono a la lista de fallas

    print("ERRORES: ", errores)
    print("TELEFONOS: ", telefonos_fallaron)
    mensaje = u'Se finalizaron las charlas con {} errores en los telefonos: {}'.format(errores, telefonos_fallaron)

    # Devolvemos la respuesta con True y un mensaje
    return respuesta(True, mensaje)


def obtener_clase_msj(clase, mensaje):
    if clase == "mapa":
        return '<mapa>', None

    if clase == 'image':
        return '<image>', None

    if clase == 'audio':
        return '<audio>', None

    if clase == 'txt':
        if mensaje.isdigit():
            return "<numero>", None

        # todo: a Direccion.lista la tendria que buscar obtener direccion misma. siguiendo patron experto
        lista_palabras_corte_inicial = Direccion.lista
        direccion = Direccion.obtener_direccion(lista_palabras_corte_inicial, mensaje)
        if direccion < 0:
            return "<texto>", None
        return '<direccion>', direccion
    else:
        return None, None


def armar_arbol(dic_nodos):
    '''
    Esta funcion se encarga de armar un arbol.

    :param dic_nodos (dict): Un diccionario que obtenemos con los nodos y datos utiles sobre ellos
    :return: (Nodo) Un arbol con diferentes funciones utiles.
    '''

    added = set()
    tree = Tree()

    while dic_nodos:  # Itero hasta que el diccionario de nodos este vacio
        # print("AGREGADOS: ", added)
        for key, value in dic_nodos.items():  # Convierto el diccionario en una lista de tuplas y itero a traves de ella

            # Si el padre del nodo se encuentra en la lista de nodos agregados
            if value['padre'] in added:
                # Agrego el nodo al arbol, con el padre que encontre en la lista como parametros
                tree.create_node(value['funcion'], key, parent=value['padre'], data=NodoAt(
                    padre=value['padre'],
                    id_nodo=value['id_nodo'],
                    menu=value['menu'],
                    permitidos=value['permitidos'],
                    funcion=value['funcion'],
                    id_funcion=value['id_funcion'],
                    orden=value['orden']

                ))
                added.add(key)  # Agrego el nodo a la lista de agregados

                # todo no me gusta esto, elimina sobre el iterante
                dic_nodos.pop(key)  # Elimino la entrada del nodo en el diccionario de nodos

            # Si no tiene padre, es porque es la raiz del arbol
            # es la misma que la otra pero no agrega el parent
            elif value['padre'] is None or value['padre'] == '':
                # Agrego el nodo al arbol, sin padre
                tree.create_node(value['funcion'], key, data=NodoAt(
                    padre=value['padre'],
                    id_nodo=value['id_nodo'],
                    menu=value['menu'],
                    permitidos=value['permitidos'],
                    funcion=value['funcion'],
                    id_funcion=value['id_funcion'],
                    orden=value['orden']
                ))
                added.add(key)  # Agrego el nodo a la lista de agregados
                dic_nodos.pop(key)  # Elimino la entrada del nodo en el diccionario

    nodos = tree.all_nodes()  # una vez que termina obtengo todos los nodos

    # Itero a traves de los nodos del arbol
    for nodo in nodos:
        hijos = tree.children(nodo.identifier)  # Guardo los hijos del nodo
        nodo.data.hijos = hijos  # Asigno los hijos del nodo a la clase Funcionalidad dentro del Nodo
        padre = tree.parent(nodo.identifier)  # Guardo el padre del nodo
        nodo.data.padre = padre  # Asigno el padre del nodo a la clase Funcionalidad dentro del Nodo
    print(tree)
    return tree


def posicion_en_arbol(arbol, id_nodo):
    '''
    Esta funcion se encarga de obtener la posicion de un usuario y devolver un arbol con esa posicion como raiz.

    :param id_nodo: La id del nodo en la que el usuario se encuentra
    :param arbol: El arbol correspondiente a un modo.
    :return: (Tree) Un objeto Arbol con el nodo del usuario como raiz
    '''

    try:  # Inicio un try, para en caso de fallar devolver una respuesta con False
        arbol[id_nodo].data.usuario = True  # Asigno la variable que define si el usuario esta parado en el nodo dado
        return arbol.subtree(id_nodo)  # Devuelvo un sub-arbol con el nodo dado como raiz
    except:
        return respuesta(False, u'No se pudo posicionar al usuario en el arbol.')


def armar_url(ip, ruta):
    return 'http://' + ip + ruta


def enviar_atmsj(mensaje, tel_origen, tel_destino, ip_atmsj, id_mensaje_web=None):
    '''
    Esta funcion se encarga de enviar un mensaje a atmsj.

    :param mensaje: Un mensaje en unicode.
    :return: Respuesta
    '''
    if mensaje is None:
        return (False, u"El mensaje está vacio")

    print("ipatmsj:",ip_atmsj)
    url = armar_url(ip_atmsj, '/enviar_msj')
    print("url:",url)
    params = {}
    params['msj'] = mensaje
    params['tel_origen'] = tel_origen
    params['tel_destino'] = tel_destino
    params['enviado_por'] = 'Automatico'
    params['id_mensaje_web'] = id_mensaje_web
    params['id_producto'] = 4
    params['usuario_responsable'] = "Automatico"

    metodo = 'POST'
    # Envía el nuevo mensaje por HTTP
    rta = enviar_por_http(url, params, metodo, tipo_params='json')

    # Si no se pudo enviar el mensaje
    if rta['Estado'] is False:
        # Retorna el error
        return rta

    # Obtiene la respuesta recibida en formato JSON
    try:
        rta = json.loads(rta['Respuesta']['content'].decode('utf-8'))
    except:
        return respuesta(False, u"Hubo un error al decodificar la respuesta")

    # Retorna la respuesta recibida como un diccionario
    return rta


def cambiar_automatico(telefono, automatico, ip_atmsj):
    '''
    Esta funcion se encarga de enviar un mensaje a atmsj indicando que debe cambiar el automatico en una charla.

    :param mensaje: Un mensaje en unicode.
    :return: Respuesta
    '''
    if telefono is None:
        return (False, u"El telefono está vacio")

    if automatico is None:
        return (False, u"El modo a modificar en automatico está vacio")

    url = armar_url(ip_atmsj, '/cambiar_automatico')

    params = {}
    params['telefono'] = telefono
    params['automatico'] = automatico

    metodo = 'POST'
    # Envía el nuevo mensaje por HTTP
    rta = enviar_por_http(url, params, metodo, tipo_params='json')

    # Si no se pudo enviar el mensaje
    if rta['Estado'] is False:
        # Retorna el error
        return rta

    # Obtiene la respuesta recibida en formato JSON
    try:
        rta = json.loads(rta['Respuesta']['content'].decode('utf-8'))
    except:
        return (False, u"Hubo un error al decodificar la respuesta")

    # Retorna la respuesta recibida como un diccionario
    return rta


def validar_mensaje(logs, datos_usuario, clase_mensaje_recibido):
    """
    :param logs (list): Una list de logs disponibles para comparar
    :param datos_usuario (dict): lo que mando el usuario, clase,msj,telefonos,etc
    :param clase_mensaje_recibido(string): clase de lo que envió efectivamente el usuario. ejemplo <texto> , esta basada en la
                                  llave clase de datos_usuario

    :return: (dict) Una respuesta con Estado, un mensaje y de ser posible un dato.
    """

    mensaje = datos_usuario['msj']  # Guardo el mensaje que envio el usuario

    # Verificamos si el tipo de mensaje ingresado es un numero
    if clase_mensaje_recibido == "<numero>":
        print("\nentro un numero:%s\n" % mensaje)

        mensaje = int(mensaje)
        # Guardamos todos los log que coincidan con el mensaje que ingreso el usuario
        log_coincidente = None
        # import web_pdb
        # web_pdb.set_trace()

        # print("logs donde compara")
        # pprint.pprint(logs)

        for log in logs:
            # print(log['espera'], type(log['espera']))
            #
            es_unicode = isinstance(log['espera'], unicode)
            if es_unicode:
                convertido = unicode_to_str(log['espera'])
                # print("convertido", convertido, type(convertido))

                if mensaje == int(convertido):  # si es un numero y coincide con el mensaje
                    # print("cocoi")
                    log_coincidente = log
                    break

            else:  # si es str o int
                es_numero = isinstance(log['espera'], int)
                if not es_numero:
                    continue

                print("valor espera:", log['espera'], type(log['espera']), 'mensaje:', mensaje, (type(mensaje)))
                if mensaje == log['espera']:  # si es un numero y coincide con el mensaje
                    log_coincidente = log
                    break

        pprint.pprint(log_coincidente)
        if not log_coincidente:  # Si el numero que ingreso el usuario no coincide con ninguno que le otorgamos
            # print('no coinicide sale')
            return respuesta(False, u'El numero ingresado no coincide')  # Devolvemos la respuesta con estado False

        # Creamos un diccionario con dos entradas
        dic = {'log': log_coincidente, 'opcion_permitida': 'modo' in eval(log_coincidente['datos'])}
        # pprint.pprint("si coincide")
        # pprint.pprint(dic)
        # Devolvemos una respuesta con True y el log que coincide como dato
        return respuesta(True, u'El numero ingresado coincide', dato=dic)

    # Verificamos si el tipo de mensaje ingresado es una ubicacion
    if clase_mensaje_recibido == "<mapa>":
        print("entro un mapa\n")
        opciones_ubicacion = [log for log in logs if log['espera'] == '<ubicacion>']

        if len(opciones_ubicacion) == 0:  # Si no esta disponible la opcion de mandar ubicacion
            return respuesta(False, u'No se encontraron logs que acepten ubicacion')

        # Creamos un diccionario con dos entradas
        dic = {'log': opciones_ubicacion[0], 'opcion_permitida': 'modo' in eval(opciones_ubicacion[0]['datos'])}

        return respuesta(True, u'Existe la opcion ubicacion', dato=dic)

    # Verificamos si el tipo de mensaje ingresado es un texto
    if clase_mensaje_recibido == "<texto>":
        print("entro un texto\n")
        # Verificamos si el mensaje es una confirmacion
        rta_conf_cancelar = verificar_modo_alternativo(mensaje)

        # Si fallamos al verificar la confirmacion
        if rta_conf_cancelar['Estado'] is False:
            return rta_conf_cancelar  # Devuelvo la respuesta

        # evalua si se usa algun modo permitido
        es_confirmacion = rta_conf_cancelar['Dato']['confirmacion']  # Guardo el booleano para saber si es confirmacion
        es_cancelar = rta_conf_cancelar['Dato']['cancelar']  # Guardo el booleano para saber si es cancelar
        es_pasar_a_manual = rta_conf_cancelar['Dato']['pasar_a_manual']  # Guardo para saber si es P.A.M
        es_estado_servicio = rta_conf_cancelar['Dato']['estado_servicio']  # Guardo para saber si es estado_servicio

        if es_estado_servicio:
            print("OPCION ESTADO SERVICIO")

            # Guardo el log que esperaba esa respuesta
            log = [log for log in logs if log['espera'] == "<estado_servicio>"]

            if len(log) == 0:  # Si no encontramos el log
                return respuesta(False, u'Estado_servicio no es una opcion valida')  # Devuelvo la respuesta

            # Creamos un diccionario con dos entradas
            dic = {'log': log[0], 'opcion_permitida': 'modo' in eval(log[0]['datos'])}

            # Devuelvo la respuesta con True y el log como dato
            return respuesta(True, u'Mensaje de estado_servicio valido', dato=dic)

        if es_pasar_a_manual:  # Si es pasar a manual
            print("OPCION PASAR A MANUAL")
            # Guardo el log que esperaba esa respuesta
            log = [log for log in logs if log['espera'] == "<pasar_manual_opcion>"]

            if len(log) == 0:  # Si no encontramos el log
                return respuesta(False, u'P.A.M no es una opcion valida')  # Devuelvo la respuesta

            # Creamos un diccionario con dos entradas
            dic = {'log': log[0], 'opcion_permitida': 'modo' in eval(log[0]['datos'])}

            # Devuelvo la respuesta con True y el log como dato
            return respuesta(True, u'Mensaje de P.A.M valido', dato=dic)

        if es_confirmacion:  # Si es confirmacion
            print("OPCION CONFIRMAR")
            # Guardo el log que esperaba esa respuesta
            log = [log for log in logs if log['espera'] == "<confirmar>"]

            if len(log) == 0:  # Si no existia esa opcion como valida
                return respuesta(False, u'Confirmar no es una opcion valida')  # Devuelvo la respuesta

            # Creamos un diccionario con dos entradas
            dic = {'log': log[0], 'opcion_permitida': 'modo' in eval(log[0]['datos'])}

            # Devuelvo la respuesta con True y el log como dato
            return respuesta(True, u'Mensaje de confirmacion valido', dato=dic)

        if es_cancelar:  # Si es cancelar
            print("OPCION CANCELAR")
            # Guardo el log que esperaba esa respuesta
            log = [log for log in logs if log['espera'] == "<cancelar>"]

            if len(log) == 0:  # Si no existia esa opcion como valido
                return respuesta(False, u'Cancelar no es una opcion valida')  # Devuelvo la respuesta

            # Creamos un diccionario con dos entradas
            dic = {'log': log[0], 'opcion_permitida': 'modo' in eval(log[0]['datos'])}

            # Devuelvo la respuesta con True y el log como dato
            return respuesta(True, u'Mensaje de cancelar valido', dato=dic)

    # Si el usuario ingreso una direccion y es una opcion valida
    if clase_mensaje_recibido == "<direccion>":
        # import web_pdb
        # web_pdb.set_trace()
        try:
            log = [log for log in logs if log['espera'] == "<direccion>"][0]

            # Creamos un diccionario con dos entradas
            dic = {'log': log, 'opcion_permitida': 'modo' in eval(log['datos'])}

            return respuesta(True, u'La opcion coincide con una direccion esperada esperada', dato=dic)
        except:
            respuesta(False, u'No se encontro una opcion valida')
    # TODO: Agregar mas funcionalidad para tipo <encuesta>, <ayuda>, etc.
    return respuesta(False, u'No se encontro una opcion valida')


def verificar_modo_alternativo(mensaje):
    '''
    Esta funcion se encarga de verificar si un mensaje coincide con el tag de confirmacion o cancelar del cliente

    :param mensaje (str): Un mensaje recibido del usuario
    :return: (dict) Un diccionario con dos entradas, "confirmacion":(bool) y "cancelar":(bool).
    '''

    regex = "[^a-zA-Z0-9 \n\.]"  # Este regex busca todos los caracteres que no son letras o numeros

    dic = {}  # Inicializo el diccionario a devolver

    # Obtengo el tag de confirmacion del cliente
    rta_obtener_conf = CP_BOT_BD.obtener_texto_tag("confirmar")

    if rta_obtener_conf['Estado'] is False:  # Si no pude obtener el tag
        return rta_obtener_conf  # Devuelvo la respuesta

    confirmacion = re.sub(regex, '', rta_obtener_conf['Dato'])  # Guardo el tag de confirmacion

    # Obtengo el tag de cancelar del cliente
    rta_obtener_cancelar = CP_BOT_BD.obtener_texto_tag("opcion_cancelar")

    if rta_obtener_cancelar['Estado'] is False:  # Si no pude obtener el tag
        return rta_obtener_cancelar  # Devuelvo la respuesta

    cancelar = re.sub(regex, '', rta_obtener_cancelar['Dato'])  # Guardo el tag de cancelar

    rta_obtener_pam = CP_BOT_BD.obtener_texto_tag("manual_seleccion")  # Intentamos obtener el texto para P.A.M

    if rta_obtener_pam['Estado'] is False:  # Si no pudimos obtener el tag
        return rta_obtener_pam  # Devolvemos la respuesta

    pam = re.sub(regex, '', rta_obtener_pam['Dato'])  # Guardamos el texto de P.A.M (pasar a manual)

    rta_obtener_estado_servicio = CP_BOT_BD.obtener_texto_tag('servicio')  # Buscamos el tag de servicio

    if rta_obtener_estado_servicio['Estado'] is False:  # Si no pude obtener el tag
        return rta_obtener_estado_servicio  # Devuelvo la respuesta

    estado = re.sub(regex, '', rta_obtener_estado_servicio['Dato'])

    if mensaje.lower() == confirmacion.lower() or mensaje.lower() == "confirmar":  # Si el mensaje coincide con el tag de confirmacion

        # Guardo en el diccionario que es confirmacion y no cancelar
        dic['confirmacion'] = True
        dic['cancelar'] = False
        dic['pasar_a_manual'] = False
        dic['estado_servicio'] = False

    elif mensaje.lower() == estado.lower():  # Si el mensaje coincide con el pasar a estado servicio

        # Guardo en el diccionario que es estado
        dic['confirmacion'] = False
        dic['cancelar'] = False
        dic['pasar_a_manual'] = False
        dic['estado_servicio'] = True

    elif mensaje.lower() == pam.lower():  # Si el mensaje coincide con el pasar a manual

        # Guardo el diccionario que es pasar a manual
        dic['confirmacion'] = False
        dic['cancelar'] = False
        dic['pasar_a_manual'] = True
        dic['estado_servicio'] = False

    elif mensaje.lower() == cancelar.lower() or mensaje.lower() == "cancelar":  # Si el mensaje coincide con el tag de cancelar

        # Guardo en el diccionario que es cancelar y no confirmacion
        dic['confirmacion'] = False
        dic['cancelar'] = True
        dic['pasar_a_manual'] = False
        dic['estado_servicio'] = False

    else:  # Si no es ni confirmacion ni cancelar

        # Seteo ambos valores del diccionario como False
        dic['confirmacion'] = False
        dic['cancelar'] = False
        dic['pasar_a_manual'] = False
        dic['estado_servicio'] = False

    # Devuelvo la respuesta junto con el diccionario
    return respuesta(True, u'Se verifico el mensaje para confirmacion/cancelar', dato=dic)


def ping(ip_atapis):
    url = "http://" + ip_atapis + "/ping"

    params = {"producto": "CpBot"}

    metodo = "POST"

    enviar_por_http(url, params, metodo, tipo_params="json")

    # Envía el nuevo mensaje por HTTP
    rta = enviar_por_http(url, params, metodo, tipo_params='json')

    # Si no se pudo enviar el mensaje
    if rta['Estado'] is False:
        # Retorna el error
        return rta

    # Obtiene la respuesta recibida en formato JSON
    try:
        rta = json.loads(rta['Respuesta']['content'].decode('utf-8'))
    except:
        return respuesta(False, u"Hubo un error al decodificar la respuesta")

    # Retorna la respuesta recibida como un diccionario
    return rta


def puede_ser_numero(string):
    try:
        int(string)
        return True
    except:
        return False


def actualizar_filtros_palabras():
    rta = filtros_repository.listar()
    setear_filtros(rta.get('Dato')) #si viene algo lo setea. sino setea none
    return rta
