# -*- coding: utf-8 -*-

import geocoder
import json
import requests
from at.cpbot.bdcpbot import CpBotBd
from at.cpbot.cpadapter import CpBotAdapter
from at.utiles.conversiones import respuesta, str_to_json, unicode_to_str, string_to_list
from at.utiles.direcciones import obtener_direccion
from at.utiles.http import enviar_por_http
from decimal import Decimal
import at.cpbot.atmsjadapter as atmsjadapter
import at.cpbot.atapisadapter as atapisadapter
from at.cpbot.tagutiles import formatear_texto
from at.cpbot.utilesfunciones import *
from at.map.calle import CallePostgres
from at.utiles.direcciones import address_to_lat_lon
from at.cpbot.domain.parametros_globales import ParametrosGlobales

cpbotbd = CpBotBd()
cpadapter = CpBotAdapter()
callepostgres = CallePostgres()
parametros_globales = ParametrosGlobales()

'''
En este modulo se encuentran las funciones a ejecutar. no se porque no estan en cada funcion
Las mismas no conocen de los nodos y los arboles, simplemente hablan
con otros productos o ejecutan acciones en este mismo.

'''


def _request_a_arcgis(busqueda):
    token_arcgis = parametros_globales.obtener_token_arcgis()
    region = parametros_globales.obtener_region_busqueda()
    # TOKEN_380 = 'AAPKb79912263cbe40268e48110b6c621839KUuk1YsNU9manGNokDgF16kKs6B5qw3jM-X1zElvMcN-WJzVjFsLd3jZ30ZVsFzw' #ip@airtrack
    return address_to_lat_lon(busqueda,
                              maxRows=5,
                              x_max=region['x_max'],
                              x_min=region['x_min'],
                              y_max=region['y_max'],
                              y_min=region['y_min'],
                              TOKEN=token_arcgis)
    # ejemplo
    # return address_to_lat_lon(busqueda,
    #                           maxRows=5,
    #                           x_max=-60.614,
    #                           x_min=-60.781,
    #                           y_max=-32.851,
    #                           y_min=-33.017,
    #                           TOKEN=token_arcgis)


def buscar_direcciones(parametros):
    # import web_pdb
    # web_pdb.set_trace()

    direccion = parametros['direccion']
    busqueda = {
        'direccion': direccion,
        'ciudad': parametros['ciudad'],
        'provincia': parametros['provincia'],
        'pais': parametros['pais']
    }

    # todo mejorar
    # pide a arcgis
    rta_photon = _request_a_arcgis(busqueda)

    # si no trae coincidencias pide a photon
    # if rta_photon['Estado'] is False: rta_photon = _request_a_photon(busqueda)

    encontro = rta_photon['Estado']
    if not encontro:

        # rta = cpbotbd.obtener_texto_tag(u'buscar_direcciones')
        #
        # # Creo un diccionario con los datos necesarios
        # dic = {
        #     'id_nodo': parametros['log']['id_nodo'],
        #     'texto': rta['Dato'],
        #     'opcion': "<texto>",
        #     'espera': "<direccion>",  # Esperamos que ingrese una direccion para la ubicacion, pero no validamos
        #     'datos': {},
        #     'orden': 0
        #
        # }

        return {
            'Estado': True,
            'Respuesta': u'No se encontro la direccion: %s' % direccion,
            'Dato': direccion,
            'Direccion': direccion,
            'Valores_resultantes': 'cero'
        }

    else:
        # si encontro algo
        direcciones_encontradas = []

        # revisa las encontradas
        for index, encontrada in enumerate(rta_photon['Dato']):
            direcciones_encontradas.append(encontrada)

        cantidad_elementos = len(direcciones_encontradas)

        if cantidad_elementos > 1:
            return {
                'Estado': True,
                'Respuesta': u'Direcciones encontradas',
                'Dato': direcciones_encontradas,
                'Direccion': direccion,
                'Valores_resultantes': 'varias'
            }
        if cantidad_elementos == 1:
            return {
                'Estado': True,
                'Respuesta': u'Direccion encontrada',
                'Dato': direcciones_encontradas,
                'Direccion': direccion,
                'Valores_resultantes': 'una'
            }
        if cantidad_elementos == 0:
            return {
                'Estado': True,
                'Respuesta': u'No se encontro la direccion: %s' % direccion,
                'Dato': direccion,
                'Direccion': direccion,
                'Valores_resultantes': 'cero'
            }


def sugerir(parametros):
    cliente = parametros
    # todo no entiendo que es esto
    if 'id_empresa' not in parametros:  # Si no encontramos id_empresa en parametros (porque hizo salto de nodo)
        cliente = parametros['cliente']  # Seteamos la key cliente como variable

    # esto pasa de string a json o algo asi
    datos_log = eval(parametros['log']['datos'])

    direccion = datos_log['direccion']

    nombre = _obtener_nombre(parametros['charla']['telefono_origen'])

    debe_cargar_servicio = datos_log['latitud']
    if debe_cargar_servicio:

        # le dice al callProcessor que cargue el servicio
        filtros = obtener_filtros_palabras(parametros['charla'].get('filtros_palabras'))
        rta = cpadapter.cargar_servicio(
            empresa=cliente['id_empresa'],  # id_empresa
            hash='',
            telefono=parametros['charla']['telefono_origen'],  # Telefono del usuario
            nombre=nombre,  # Como nombre utilizamos el telefono del usuario
            direccion=direccion,  # La direccion donde se pidio el movil
            telefonista="whatsapp",  # Ponemos como telefonista el CpBot
            latitud=float(datos_log['latitud']) * 3600,
            longitud=float(datos_log['longitud']) * 3600,
            host="http://" + cliente['ip_callprocessor'],  # Donde se encuentra callprocessor
            filtros=filtros
        )

        if rta['Estado'] is False:  # Si la respuesta es False
            return rta  # Devuelvo la respuesta

        # añade la direccion en el dato de la respuesta, xq el callprocesor no la devuelve
        rta['Dato']['direccion'] = direccion
        rta['Valores_resultantes'] = 'exito'  # devuelve algo porque sino me traeria el nodo de mapa
        rta['Titulo'] = _obtener_texto_servicio_cargado(codigo_viaje=rta['Dato'].get('codigo_viaje')).get('Dato')


    else:
        rta = {'Estado': True,
               'Dato': direccion,
               'Respuesta': u"no lo cargo al servicio porque eligio una opcion que no tiene lat y lon",
               'Valores_resultantes': 'cero'}

    return rta


def obtener_filtros_palabras(palabras):
    try:
        filtros_palabras = string_to_list(palabras, ',')
        filtros_palabras = lista_string_a_lista_int(filtros_palabras)
        return filtros_palabras
    except:
        return None


def lista_string_a_lista_int(lista):
    return list(map(int, lista))


def pedir_direccion(parametros):
    """
       carga un servicio
       """
    cliente = parametros
    # todo no entiendo que es esto
    if 'id_empresa' not in parametros:  # Si no encontramos id_empresa en parametros (porque hizo salto de nodo)
        cliente = parametros['cliente']  # Seteamos la key cliente como variable

    # esto pasa de string a json o algo asi
    datos_log = eval(parametros['log']['datos'])
    latitud = float(datos_log['datos_ejecucion']['latitud']) * 3600
    longitud = float(datos_log['datos_ejecucion']['longitud']) * 3600

    nombre = _obtener_nombre(parametros['charla']['telefono_origen'])

    direccion = parametros['mensaje']  # Obtenemos la direccion a cargar el servicio

    # le dice al callProcessor que cargue el servicio
    rta = cpadapter.cargar_servicio(
        empresa=cliente['id_empresa'],  # id_empresa
        hash='',
        telefono=parametros['charla']['telefono_origen'],  # Telefono del usuario
        nombre=nombre,  # Como nombre utilizamos el telefono del usuario
        direccion=direccion,  # La direccion donde se pidio el movil
        telefonista="whatsapp",  # Ponemos como telefonista el CpBot
        latitud=latitud,  # La latitud de la ubicacion
        longitud=longitud,  # La longitud de la ubicacion
        host="http://" + cliente['ip_callprocessor'],  # Donde se encuentra callprocessor
        filtros=obtener_filtros_palabras(parametros['charla']['filtros_palabras'])

    )

    if rta['Estado'] is False:  # Si la respuesta es False

        return rta  # Devuelvo la respuesta

    # agrega la direccion a lo que va a retornar
    rta['Dato'][
        'direccion'] = direccion  # añade la direccion en el dato de la respuesta, xq el callprocesor no la devuelve
    rta['Titulo'] = _obtener_texto_servicio_cargado(codigo_viaje=rta['Dato'].get('codigo_viaje')).get('Dato')

    return rta


def buscar_por_google(parametros):
    '''
    Esta funcion se encarga de cargar un servicio mediante una request al CallProcessorWeb

    :param parametros (dict): Un diccionario con datos utiles para la ejecucion de funciones
    :return: (dict)
    '''

    datos_log = eval(parametros['log']['datos'])

    # Verificamos si es un numero de telefono el nombre agendado para evitar cargarlo con el mismo.
    nombre = _obtener_nombre(parametros['charla']['telefono_origen'])

    # Si la direccion que ingreso el usuario no era valida vamos a utilizar la ubicacion que ingreso
    if datos_log['direccion'] is None:

        latlong = parametros['mensaje'].split(',')  # Obtenemos una lista con la latitud y longitud, separando en la ','

        latitud = latlong[0]  # Guardamos la latitud

        longitud = latlong[1]  # Guardamos la longitud

        # Utilizo la funcion del adaptador de CallProcessor para cargar el servicio
        rta = cpadapter.cargar_servicio(
            empresa=parametros['id_empresa'],  # id_empresa
            hash='',
            telefono=parametros['charla']['telefono_origen'],  # Telefono del usuario
            nombre=nombre,  # Como nombre utilizamos el telefono del usuario
            direccion=datos_log['mensaje'],  # La direccion donde se pidio el movil
            telefonista="whatsapp",  # Ponemos como telefonista el CpBot
            latitud=float(latitud) * 3600,  # La latitud de la ubicacion
            longitud=float(longitud) * 3600,  # La longitud de la ubicacion
            host="http://" + parametros['ip_callprocessor'],  # Donde se encuentra callprocessor
            filtros=obtener_filtros_palabras(parametros['charla']['filtros_palabras'])

        )

        if rta['Estado'] is False:  # Si la respuesta es False
            return rta  # Devuelvo la respuesta

        rta['Dato']['direccion'] = datos_log['mensaje']  # Guardo la direccion en el dato de la respuesta
        rta['Titulo'] = _obtener_texto_servicio_cargado(codigo_viaje=rta['Dato'].get('codigo_viaje')).get('Dato')

        return rta  # Al finalizar devuelvo la respuesta de la carga del servicio

    direccion = datos_log['direccion']
    latitud = datos_log['latitud']
    longitud = datos_log['longitud']

    # Utilizo la funcion del adaptador de CallProcessor para cargar el servicio
    rta = cpadapter.cargar_servicio(
        empresa=parametros['id_empresa'],  # id_empresa
        hash='',
        telefono=parametros['charla']['telefono_origen'],  # Telefono del usuario
        nombre=nombre,  # Como nombre utilizamos el telefono del usuario
        direccion=direccion,  # La direccion donde se pidio el movil
        telefonista="whatsapp",  # Ponemos como telefonista el CpBot
        latitud=float(latitud) * 3600,  # La latitud de la ubicacion
        longitud=float(longitud) * 3600,  # La longitud de la ubicacion
        host="http://" + parametros['ip_callprocessor'],  # Donde se encuentra callprocessor
        filtros=obtener_filtros_palabras(parametros['charla']['filtros_palabras'])

    )

    if rta['Estado'] is False:  # Si la respuesta es False
        return rta  # Devuelvo la respuesta

    rta['Dato']['direccion'] = direccion  # Guardo en el dato de la respuesta la direccion
    rta['Titulo'] = _obtener_texto_servicio_cargado(codigo_viaje=rta['Dato'].get('codigo_viaje')).get('Dato')

    return rta  # Al finalizar devuelvo la respuesta de la carga del servicio


#
# def cargar_servicio(parametros):
#     '''
#     Esta funcion se encarga de cargar un servicio mediante una request al CallProcessorWeb
#         por el momento sin uso, pero estaria bueno que este como nodo
#
#     :param parametros (dict): Un diccionario con datos utiles para la ejecucion de funciones
#     :return: (dict)
#     '''
#
#     nombre = _obtener_nombre(parametros['charla']['telefono_origen'])
#
#     # Utilizo la funcion del adaptador de CallProcessor para cargar el servicio
#     rta = cpadapter.cargar_servicio(
#         empresa=parametros['id_empresa'],  # id_empresa
#         hash='',
#         telefono=parametros['charla']['telefono_origen'],  # Telefono del usuario
#         nombre=nombre,  # Como nombre utilizamos el telefono del usuario
#         direccion=parametros['charla']['datos_servicio']['direccion'],  # La direccion donde se pidio el movil
#         telefonista="whatsapp",  # Ponemos como telefonista el CpBot
#         latitud=float(parametros['charla']['datos_servicio']['latitud']),  # La latitud de la ubicacion
#         longitud=float(parametros['charla']['datos_servicio']['longitud']),  # La longitud de la ubicacion
#         host="http://" + parametros['ip_callprocessor']  # Donde se encuentra callprocessor
#     )
#
#     if rta['Estado'] is False:  # Si la respuesta es False
#         return rta  # Devuelvo la respuesta
#
#     # Guardo la direccion en el dato de la respuesta
#     rta['Dato']['direccion'] = parametros['charla']['datos_servicio']['direccion']
#
#     return rta  # Al finalizar devuelvo la respuesta de la carga del servicio


def finalizar_charla(parametros):
    return respuesta(True, u'CAMBIAR ESTA RESPUESTA', dato='')


def ingresar_ubicacion(parametros):
    """
        # este es el del ingreso del mapa
        entra: mensaje
        sale: dict con latitud y longitud
    """

    try:
        direccion = eval(parametros['log']['datos'])['direccion']
    except:
        direccion = None

    # si hay direccion, ya teniendo latitud y longitud carga el servicio
    if direccion:

        cliente = parametros
        # todo no entiendo que es esto
        if 'id_empresa' not in parametros:  # Si no encontramos id_empresa en parametros (porque hizo salto de nodo)
            cliente = parametros['cliente']  # Seteamos la key cliente como variable

        latlong = parametros['mensaje'].split(',')  # Obtenemos una lista con la latitud y longitud, separando en la ','
        latitud = float(latlong[0]) * 3600  # es la latitud del mapa recibido. NO DE SUGERIDAS
        longitud = float(latlong[1]) * 3600
        nombre = _obtener_nombre(parametros['charla']['telefono_origen'])

        # todo. negrada, aca tiene que venir una direccion y nada mas
        if isinstance(direccion, list):
            direccion = direccion[0]['direccion']

        # le dice al callProcessor que cargue el servicio
        rta = cpadapter.cargar_servicio(
            empresa=cliente['id_empresa'],  # id_empresa
            hash='',
            telefono=parametros['charla']['telefono_origen'],  # Telefono del usuario
            nombre=nombre,  # Como nombre utilizamos el telefono del usuario
            direccion=direccion,  # La direccion donde se pidio el movil
            telefonista="whatsapp",  # Ponemos como telefonista el CpBot
            latitud=latitud,  # La latitud de la ubicacion
            longitud=longitud,  # La longitud de la ubicacion
            host="http://" + cliente['ip_callprocessor'],  # Donde se encuentra callprocessor
            filtros=obtener_filtros_palabras(parametros['charla']['filtros_palabras'])

        )

        if rta['Estado'] is False:
            return rta  # Devuelvo la respuesta

        # agrega la direccion a lo que va a retornar
        rta['Dato'][
            'direccion'] = direccion  # añade la direccion en el dato de la respuesta, xq el callprocesor no la devuelve
        rta['Titulo'] = _obtener_texto_servicio_cargado(codigo_viaje=rta['Dato'].get('codigo_viaje')).get('Dato')

        return rta

    else:
        latlong = parametros['mensaje'].split(',')  # Obtenemos una lista con la latitud y longitud, separando en la ','

        dic = {'latitud': latlong[0], 'longitud': latlong[1]}

        return respuesta(True, u'CAMBIAR ESTA RESPUESTA', dato=dic)


def obtener_direcciones(parametros):
    """
    carga un servicio.
    # todo mejora: abstraer la carga y el armado de datos
    """

    cliente = parametros
    # todo no entiendo que es esto
    if 'id_empresa' not in parametros:  # Si no encontramos id_empresa en parametros (porque hizo salto de nodo)
        cliente = parametros['cliente']  # Seteamos la key cliente como variable

    # esto pasa de string a json o algo asi
    datos_log = eval(parametros['log']['datos'])

    latitud = float(datos_log['latitud'])
    longitud = float(datos_log['longitud'])

    nombre = _obtener_nombre(parametros['charla']['telefono_origen'])

    direccion = parametros['log']['mensaje_enviado']  # Obtenemos la direccion a cargar el servicio

    # le dice al callProcessor que cargue el servicio
    rta = cpadapter.cargar_servicio(
        empresa=cliente['id_empresa'],  # id_empresa
        hash='',
        telefono=parametros['charla']['telefono_origen'],  # Telefono del usuario
        nombre=nombre,  # Como nombre utilizamos el telefono del usuario
        direccion=direccion,  # La direccion donde se pidio el movil
        telefonista="whatsapp",  # Ponemos como telefonista el CpBot
        latitud=latitud,  # La latitud de la ubicacion
        longitud=longitud,  # La longitud de la ubicacion
        host="http://" + cliente['ip_callprocessor'],  # Donde se encuentra callprocessor
        filtros=obtener_filtros_palabras(parametros['charla']['filtros_palabras'])

    )

    if rta['Estado'] is False:  # Si la respuesta es False
        rta['Filtro'] = 'error'
        return rta  # Devuelvo la respuesta

    # agrega la direccion a lo que va a retornar
    rta['Dato']['direccion'] = direccion
    # añade la direccion en el dato de la respuesta, xq el callprocesor no la devuelve
    rta['Filtro'] = 'si'
    rta['Titulo'] = _obtener_texto_servicio_cargado(codigo_viaje=rta['Dato'].get('codigo_viaje')).get('Dato')
    print("titulo y todo:", rta)

    return rta


def _obtener_nombre(telefono):
    # Verificamos si es un numero de telefono el nombre agendado para evitar cargarlo con el mismo.
    # ve si tiene formato telefonico: 12798343 o de contacto: ivan

    # si falla devuelve el telefono si es un nombre. sino desconocido
    # si no encuentra nada devuelve el telefono si es un nombre. sino desconocido
    # si encuentra algo devuelve el nombre. sino desconocido
    try:
        respuesta = cpadapter.obtener_datos_cliente(telefono)
    except:
        return telefono if verificar_si_es_telefono(telefono) else None
    if respuesta['Estado']:
        telefono_de_bd = respuesta.get('Dato').get('Nombre')
        if telefono_de_bd:
            return telefono_de_bd if verificar_si_es_telefono(telefono_de_bd) else None
    # si estado es false o si el telefono esta vacio
    return telefono if verificar_si_es_telefono(telefono) else None


def _request_a_photon(direccion):
    """Funcion privada que se encarga de hacer la request a photon"""

    url_photon = "https://photon.komoot.io/api/?q={},{},{},{}".format(direccion['direccion'],
                                                                      direccion['ciudad'],
                                                                      direccion['provincia'],
                                                                      direccion['pais'])

    req = requests.get(url_photon)

    if req.status_code != 200:  # Si se pudo hacer la request
        return respuesta(False, u'No se pudo hacer la request a photon')

    dic = str_to_json(req.__dict__['_content'])['features']  # Convertimos la respuesta en un diccionario

    if not dic: return respuesta(False, u'No se encontraron direcciones en photon para la direccion %s' % direccion)

    formateadas = []
    for direccion in dic:
        try:
            dire = direccion['properties']['street']
            altura = direccion['properties']['housenumber']
            ciudad = direccion['properties']['city']
            direccion_completa = dire + ' ' + altura

            dic = {'latitud': direccion['geometry']['coordinates'][1],
                   'longitud': direccion['geometry']['coordinates'][0],
                   'ciudad': unicode_to_str(ciudad),
                   'direccion': unicode_to_str(direccion_completa)}

            formateadas.append(dic)
        except Exception as error:

            continue

    return respuesta(estado=True,
                     rta="Se obtuvieron las direcciones de photon",
                     dato=formateadas)


def saludar(parametros):
    '''
    :param parametros (dict): Un diccionario con datos utiles para la ejecucion de funciones
    :return: (dict)
    '''
    return respuesta(True, u'Funcion saludar() no tiene ejecucion', dato='')


def _obtener_texto_servicio_cargado(codigo_viaje):
    """
    busca el texto de servicio cargado, si falla setea un mensaje predeterminado

    nota:NO BUSCA EN LA BD EL TAG CODIGO_VIAJE, XQ ERA UN LIO. SOLO MANTUVE LA IDEA DE <codigo_viaje> para evitar errores
    al editar la bd

    ejemplo
    entrada codigo_viaje=44
    obtener_texto_tag=Su pedido fue tomado correctamente y en breve llegara su movil Su codigo de viaje es *_!codigo_viaje!_* . Muchas gracias por su preferencia
    salida: Su pedido fue tomado correctamente y en breve llegara su movil Su codigo de viaje es *_44_* . Muchas gracias por su preferencia
    """

    rta = cpbotbd.obtener_texto_tag('servicio_cargado')  # Obtenemos el texto del tag de tarde
    if rta['Estado'] is False:  # Si no pudimos obtener el texto del tag, seteo un mensaje predeterminado
        rta[
            'Dato'] = 'Su pedido fue tomado correctamente y en breve llegara su movil. Su codigo de viaje es *_<codigo_viaje>_* . Muchas gracias por su preferencia..'

    # busca estos caracteres
    regex = "<codigo_viaje>"  # crucial que coincida con el valor puesto en donde se invoque en la bd
    # ejemplo si en la bd esta "su servicio es !codigo_viaje! aca debe figurar igual"
    try:
        # si lo encuentra lo reemplaza
        nuevo_titulo = rta['Dato'].replace(regex, codigo_viaje)
        rta['Dato'] = nuevo_titulo
    except Exception as error:
        print("falla en reemplazar codigo viaje:", error)
    return respuesta(True, None, dato=rta['Dato'])
