# -*- coding: utf-8 -*-
from funciones import *
from clasesfunciones import *
from at.utiles.conversiones import respuesta, unicode_to_str
import at.cpbot.tagutiles as tagutiles
import datetime


def esta_elemento_array_en_otro_array(array1, array2):
    """
    devuelve los elementos que estan en ambas listas

    array1=[1,2]
    array2=[1,5]
    salida=[1]
    """
    return [i for i in array1 if i in array2]


class NodoAt(object):

    def __init__(self, padre, funcion, menu, permitidos, id_nodo, id_funcion, orden, parametros=None):
        self.padre = padre  # el id_nodo del padre
        self.funcion = funcion  # String que servira para instanciar la clase de dicha funcion
        self.menu = menu  # El titulo del menu (no dinamico)
        self.permitidos = permitidos  # Modos no implicitos en el arbol que tienen permitidos activarse
        self.id_nodo = id_nodo  # La id del nodo
        self.id_funcion = id_funcion  # La id de la funcion
        self.hijos = []  # Una lista que contiene los hijos del nodo (clase Node)
        self.parametros = parametros  # Una lista de parametros necesarios para la funcion, puede no tener.
        self.orden = orden

    def ejecutar_nodo(self):
        '''
        Esta funcion se encarga de ejecutar la funcion del nodo y de evaluar sus hijos para formar un menu.

        :return: (respuesta) Una respuesta con estado y de ser posible un diccionario
        '''

        # Utilizando una funcion instancio la clase correspondiente a la funcion
        clase = self.crear_clase_funcion()

        # Ejecuto la funcion y guardo la respuesta en una variable
        return clase.ejecutar_funcion()

    def visualizar(self):
        '''
        Esta funcion se encarga de hacer print a las funcionalidades del nodo, para debuggear mas facil

        :return: (str) Un string representativo de la instancia.
        '''

        print("-> PADRE: {} \n" \
              " -> FUNCION: {} \n" \
              "-> MENU: {} \n" \
              "-> PERMITIDOS: {} \n" \
              "-> ID_NODO:  {} \n" \
              "-> ID_FUNCION: {} \n" \
              "-> HIJOS: {} \n".format(self.padre, self.funcion, self.menu, self.permitidos,
                                       self.id_nodo, self.id_funcion, self.hijos))

    def crear_clase_funcion(self):

        titulo = "titulo=" + "'" + str(self.menu) + "'" + ","
        parametros = "parametros=" + str(self.parametros) + ","
        funcion = "funcion=" + "'" + str(self.funcion) + "'" + ","
        id_funcion = "id_funcion=" + str(self.id_funcion) + ","
        id_nodo = "id_nodo=" + str(self.id_nodo) + ","
        orden = "orden=" + str(self.orden)

        return eval(
            str(self.funcion).capitalize() + "(" + titulo + parametros + funcion + id_funcion + id_nodo + orden + ")"
        )

    def obtener_dinamicos(self, datos_ejecucion=None, filtros=None):
        '''
        Esta funcion se encarga de obtener el menu dinamico de los hijos del nodo.

        :return: (list) Una lista con los menus dinamicos generados por los hijos.
        '''

        # paso como parametro los datos de ejecucion para que solo traiga los que cumplen con la condicion en caso de que la posean
        # ej: valores_resultantes:una
        # solo trae los nodos que tengan una como filtro una o que no tengan filtro
        # todo. ojo que hace un pass, pero dps no hace nada
        if self.hijos is None:  # Verifico si el nodo es hoja (no tiene hijos)
            pass

        lista = []

        # print("datos ejecucion:")
        # pprint.pprint(datos_ejecucion)
        # print("filtros")
        # pprint.pprint(filtros)

        valores_resultantes = None
        if datos_ejecucion:
            valores_resultantes = datos_ejecucion.get('Valores_resultantes')

        # ejemplo
        # valores_resultantes='una'
        # filtros= [{'filtro': [u'cero'], 'id_nodo': 12}, {'filtro': [u'una'], 'id_nodo': 13},{'filtro': None, 'id_nodo': 4}]

        hijos_filtrados = self.filtrar_hijos(filtros, valores_resultantes)

        print("hijos que quedan:")
        pprint.pprint(hijos_filtrados)

        # obtiene el dinamico de cada hijo
        for hijo in hijos_filtrados:  # Itero a traves de los hijos

            # pprint.pprint(hijo.data.id_nodo)
            # si no hay filtro trae todos hijos
            # si el filtro coincide trae hijos
            # si el filtro no coincide continua

            clase = hijo.data.crear_clase_funcion()  # Instancio la clase del hijo

            clase.parametros = self.parametros  # Le paso los parametros al hijo porque a lo mejor lo usa para algo
            clase.parametros['datos_armar_dinamico'] = datos_ejecucion

            # todo esto me hace ruido
            rta = clase.obtener_dinamico()  # Intento obtener el menu dinamico del hijo

            if rta['Estado'] is False:  # Si la respuesta es False
                continue

            lista.append(rta['Dato'])  # Si pude obtener el menu dinamico, lo guardo en la lista

        # todo esto no entiendo porque
        lista = [opcion for sublista in lista for opcion in sublista]  # Elimino las sublistas

        hay_permitidos = self.permitidos is not None
        if hay_permitidos:  # Si tenemos modos permitidos establecidos en el nodo

            for key, value in self.permitidos.items():  # Iteramos a traves de los permitidos, permitidos es un json

                # Obtiene el id_nodo del primer nodo del modo que le pasa como parametro,
                # que es el nombre por ejemplo Cancelar
                # todo cambiar a que devuelva el nodo entero y depsues aca extraer el idnodo
                primer_nodo = cpbotbd.obtener_nodo_inicial(key)

                if primer_nodo['Estado'] is False:  # Si no pude obtener el primer nodo
                    continue  # Continuo con el loop

                # si puede obtener el primer nodo
                primer_nodo_id = primer_nodo['Dato']  # Guardo el id del primer nodo porque la lista de permitidos
                # solo tendria el nodo principal nada mas

                dic = {  # Creamos un diccionario que insertaremos en la lista
                    'orden': 99,  # Tiene un orden de 99, ya que debe ser los ultimos elementos a mostrar
                    'id_nodo': primer_nodo_id,  # Seteo el nombre de la key como el nodo al que se debe ir
                    # Si queremos que sea visible obtenemos el texto del tag
                    'texto': tagutiles.formatear_texto(cpbotbd.obtener_texto_tag(key)['Dato'], self.parametros)[
                        'Dato'] if value else '',
                    'datos': {'modo': key},  # Seteamos el nombre de la funcion como dato
                    'opcion': '<texto>',  # Seteamos la opcion texto, pareque que no puede venir otra cosa nunca
                    'espera': "<" + key + ">"  # Seteamos lo que espera,
                }

                lista.append(dic)  # Agregamos el diccionario a la lista

        return respuesta(True, u'Se obtuvieron los menus dinamicos de los hijos', dato=lista)

    def filtrar_hijos(self, filtros, valores_resultantes):
        hijos = self.hijos
        print("valores:", valores_resultantes)
        print("filtros:", filtros)
        if valores_resultantes and filtros:  # si hay resultantes y filtros
            for indice, objeto in enumerate(filtros):

                # si el filtro es none lo deja
                if objeto['filtro'] == None:
                    print("deja porque es none")
                    continue

                valor = esta_elemento_array_en_otro_array(objeto['filtro'], valores_resultantes)
                if valor: continue  # si el valor esta en la lista lo deja

                # el valor no esta en la lista, debe quitarlo
                for i, h in enumerate(self.hijos):
                    print("id:", h.data.id_nodo, " tipo:", type(h.data.id_nodo))
                    if h.data.id_nodo != objeto['id_nodo']:
                        print("no esta en vr, pero no es el elemento de los hijos.indice", i)
                        continue
                    print("va a sacar al indice :", i)
                    hijos.pop(i)
        else:
            pass
        return hijos

    def obtener_menu(self, datos_ejecucion=None, filtros=None):
        '''
        Esta funcion se encarga de obtener el menu completo del nodo.

        :return: (str) El menu para otorgar al usuario
        '''

        # import web_pdb
        # web_pdb.set_trace()
        clase = self.crear_clase_funcion()  # Instancio la clase de la funcion

        titulo = self.__obtener_titulo(clase, datos_ejecucion)  # Obtengo el titulo de la clase

        rta_dinamicos = self.obtener_dinamicos(
            datos_ejecucion, filtros)  # Obtengo los menus dinamicos (provenientes de los hijos). no importa
        # lo que tengan, despues se filtrara

        # [{'datos': {'latitud': -32.94726500408163, 'ciudad': 'Rosario', 'longitud': -60.640346042857146,'direccion': 'Rioja 1234'}}]
        # Si no pude obtener los dinamicos
        if rta_dinamicos['Estado'] is False:
            return rta_dinamicos  # Devuelvo la respuesta

        dinamicos = rta_dinamicos['Dato']

        # Si pude obtener todo correctamente, guardo el titulo y el dinamico en un diccionario
        dic = {
            'titulo': titulo,
            'dinamico': dinamicos
        }

        # Devuelvo la respuesta con estado True y el diccionario como dato
        return respuesta(True, u'Se armo el menu con exito', dato=dic)

    def __obtener_titulo(self, clase, datos_ejecucion):
        titulo=None
        try:
            titulo = datos_ejecucion.get('Titulo')
            if not titulo:
                titulo = clase.obtener_titulo()  # Obtengo el titulo de la clase
        except:
            titulo = clase.obtener_titulo()  # Obtengo el titulo de la clase
        finally:
            return titulo

    def armar_menu(self, titulo, dinamicos):
        '''
        Esta funcion se encarga de formar un string a partir del titulo(str) y el dinamico(list)

        :param titulo: Un string obtenido de la base de datos, es el titulo del menu
        :param dinamicos: Una lista de diccionarios con diferentes datos de cada opcion a ofrecer en un menu.
        :return: (str) Un string con formato de menu listo para enviar al usuario
        '''

        if titulo is None:  # Si el nodo no tiene titulo
            titulo = ''  # Lo cambio por un string vacio

        try:  # Inicio un try para devolver una respuseta en caso de fallar

            # Obtenemos el titulo utilizado en las opciones numericas
            titulo_opciones_numericas = self.obtener_titulo_opciones_numericas()

            separador = self.obtener_separador()  # Obtenemos el separador de opciones

            marcador_opcion = self.obtener_marcador_opcion()  # Obtenemos el marcador de opciones
            print(marcador_opcion)
            marcador_opcion_inicial = self.obtener_marcador_opcion_inicial()  # Obtenemos el marcador de opciones
            print(marcador_opcion_inicial)
            # input()
            menu = titulo + "\n"  # Iniciamos el menu con el titulo y un salto de linea

            # Guardo el numero de opcion y el texto en una lista para las opciones numericas
            primeros = [{'orden': dinamico['orden'], 'texto': dinamico['texto'], 'espera': dinamico['espera']} for
                        dinamico
                        in dinamicos if self.es_numero(dinamico['espera'])]

            # Guardo el orden y texto en una lista para las opciones no numericas
            segundos = [{'orden': dinamico['orden'], 'texto': dinamico['texto']} for dinamico in dinamicos if
                        self.es_numero(dinamico['espera']) is False]

            if primeros != []:  # Si tengo opciones numericas

                menu += separador + "\n"  # Agregamos un separador

                menu += titulo_opciones_numericas + "\n\n"

                for dinamico in sorted(primeros):  # Itero a traves de las opciones numericas

                    # Agrego cada opcion al menu, junto a su numero de opcion y un salto de linea
                    # menu += '( *' + str(dinamico['espera']) + '* ) ' + dinamico['texto'] + "\n\n"
                    menu += marcador_opcion_inicial + str(dinamico['espera']) + marcador_opcion + dinamico['texto'] + "\n\n"

                menu += separador  # Agregamos un separador al final de las opciones numericas

            if segundos != []:  # Si tengo opciones no numericas

                if primeros == []:  # Si no teniamos opciones numericas

                    menu += separador + "\n"  # Agregamos el separador con un salto de linea

                for dinamico in sorted(segundos):  # Itero a traves de las opciones no numericas

                    if dinamico['texto'] == '':  # Si la opcion no tiene un texto
                        continue  # Sigo con el loop

                    menu += dinamico['texto'] + "\n"  # Agrego al menu el texto junto a un salto de linea

                    menu += separador + "\n"  # Agregamos un separador

            return respuesta(True, u'Se armo el menu con exito', dato=menu)  # Devuelvo el menu junto con la respuesta

        except Exception as error:  # Si falla en algun lugar
            # Devuelvo la respuesta con False
            return respuesta(False, u'Error al intentar obtener el menu: %s' % error)

    def obtener_id_nodo_hijos(self):
        '''
        Esta funcion se encarga de devolver el id_nodo de todos los hijos

        :return: (list) Una lista con todos los id_nodo(int) de los hijos
        '''

        # Verificamos si es un nodo que no tiene hijos
        if self.hijos == []:
            return []  # Devolvemos una lista vacia

        lista = []  # Inicializo la lista que voy a devolver

        for hijo in self.hijos:  # Iteramos a traves de los hijos

            id_hijo = hijo.data.id_nodo  # Guardo el id_nodo del hijo en una variable

            lista.append(id_hijo)  # Agrego a la lista el id_nodo del hijo

        return lista  # Devuelvo la lista

    def es_numero(self, numero):
        '''
        Una funcion de utilidad para verificar si un string esta representando un int.

        :param numero (str): Un string de un numero
        :return: (bool) True si es un numero
        '''

        try:  # Iniciamos un try para devolver False en caso de fallar
            int(numero)  # Intentamos convertir el string en un int()
            return True  # Si llego hasta aca es porque se pudo convertir en int(), devolvemos True

        except:  # En caso de fallar
            return False  # Devuelvo False

    def obtener_separador(self):
        '''
        Esta funcion se encarga de obtener el separador que utilizaremos en el menu.
        '''

        rta_obtener_tag = cpbotbd.obtener_texto_tag('separador')  # Buscamos el separador que seteo el usuario

        separador = None  # Inicializamos el separador para luego reemplazarlo

        if rta_obtener_tag['Estado'] is False:  # Si fallamos al obtener el separador
            return ''  # Devolvemos un string vacio (no se usara separador)

        separador = rta_obtener_tag['Dato']  # Guardamos el separador que esta en la BD

        if len(separador) > 10:  # Si el separador es mas largo que 10 digitos
            separador = separador[:11]  # Cortamos el separador en el decimo caracter

        return separador * 2  # Devolvemos el separador

    def obtener_marcador_opcion(self):
        '''
        Esta funcion se encarga de obtener el marcador de las opciones que se ofrecen (por ej: 1- OPCION)
        '''

        rta_obtener_tag = cpbotbd.obtener_texto_tag('marcador_opcion')  # Buscamos el marcador que seteo el usuario

        marcador_opcion = None  # Inicializamos el marcador para luego reemplazarlo

        if rta_obtener_tag['Estado'] is False:  # Si fallamos al obtener el marcador
            marcador_opcion = ''  # Seteamos sin marcador

        else:
            marcador_opcion = rta_obtener_tag['Dato']  # Guardamos el marcador que esta en la BD

        # if len(marcador_opcion) > 1:  # Si el marcador es mas largo que 1 digitos
        #     marcador_opcion = marcador_opcion[:1]  # Cortamos el separador en el primer caracter

        return marcador_opcion  # Devolvemos el marcador
    def obtener_marcador_opcion_inicial(self):
        '''
        Esta funcion se encarga de obtener el marcador de las opciones que se ofrecen (por ej: 1- OPCION)
        '''

        rta_obtener_tag = cpbotbd.obtener_texto_tag('marcador_opcion_inicial')  # Buscamos el marcador que seteo el usuario

        marcador_opcion = None  # Inicializamos el marcador para luego reemplazarlo

        if rta_obtener_tag['Estado'] is False:  # Si fallamos al obtener el marcador
            marcador_opcion = ''  # Seteamos sin marcador

        else:
            marcador_opcion = rta_obtener_tag['Dato']  # Guardamos el marcador que esta en la BD

        # if len(marcador_opcion) > 1:  # Si el marcador es mas largo que 1 digitos
        #     marcador_opcion = marcador_opcion[:1]  # Cortamos el separador en el primer caracter

        return marcador_opcion  # Devolvemos el marcador

    def obtener_titulo_opciones_numericas(self):
        '''
        Esta funcion se encarga de obtener el subtitulo para las opciones numericas.
        '''

        rta = cpbotbd.obtener_texto_tag("titulo_numericas")  # Obtenemos el tag de las opciones numericas

        if rta['Estado'] is False:  # Si fallamos al obtener el tag de la base de datos
            return ''  # Devolvemos un string vacio (sin subtitulo)

        try:
            return unicode_to_str(rta['Dato'])  # Devolvemos el subtitulo
        except:
            return ''
