# -*- coding: utf-8 -*-
import datetime
import pprint
from sqlalchemy.sql.expression import and_
from decimal import Decimal
from sqlalchemy import or_
from at.orm.decorators import DBSwitch
from at.utiles.conversiones import *


@DBSwitch('cpbot')
class CharlasRepository:

    def __init__(self):
        pass

    def actualizar_filtros(self, id_charla, filtros):
        print(id_charla)
        print(filtros)
        """
        actualiza los filtros en una charla
        """

        session = self.cpbot.binded_session()
        try:
            tabla = self.cpbot.tabla('charla')

            # Filtro por el id_charla
            charla = session.query(tabla).get(id_charla)

            if charla: #si encontro la charla

                if charla.filtros_palabras:  # si habia ids

                    palabras = charla.filtros_palabras + ',' + filtros
                    array_palabras = string_to_list(palabras, ',')
                    array_palabras_limpio = remover_duplicados_lista(array_palabras)
                    string_limpio = lista_to_string(array_palabras_limpio, ',')
                else:  # si no habia los agrega asi como viene
                    string_limpio = filtros

                charla.filtros_palabras = string_limpio

                session.commit()
                rta = respuesta(True, u"ok")
            else:
                rta = respuesta(False, u"No hay charla para actualizar")

        except Exception as error:
            rta = respuesta(False, u"No se pudo actualizar los filtros de la charla %s" % id_charla, error)
            print(rta)
        finally:
            session.close()

        return rta

    def obtener_charla(self, telefono_usuario):
        '''
        Esta funcion se encarga de obtener la id de la charla mediante el telefono del usuario

        :param telefono_usuario: El telefono de un usuario que nos envio un mensaje
        :return: (respuesta) Un diccionario con Estado, un String y de ser posible un dato
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Conecto con la tabla charla
        charla = self.cpbot.tabla('charla')

        # Obtengo los datos que necesito
        q_obj = session.query(
            charla
        )

        # Filtro por el telefono del usuario
        q_obj = q_obj.filter(charla.telefono_origen == telefono_usuario,
                             charla.automatico == 1,
                             charla.estado != 'finalizada')

        # Si el registro esta vacio
        if q_obj is None:
            return respuesta(False, u'error al obtener charla con el telefono:%s' % telefono_usuario)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesion con la base de datos
        session.close()

        # Si el registro esta vacio
        if registro is None:
            return respuesta(False, u'No se encontro una charla activa con ese telefono')

        dic = {  # Inicializo el diccionario que voy a devolver
            'id_charla': registro.id_charla,
            'automatico': registro.automatico,
            'telefono_origen': registro.telefono_origen,
            'estado': registro.estado,
            'datos_servicio': eval(registro.datos_servicio),
            'telefono_destino': registro.telefono_destino,
            'usuario_responsable': registro.usuario_responsable,
            'intentos_fallidos': registro.intentos_fallidos,
            'id_modo': registro.id_modo,
            'id_ultimo_nodo': registro.id_ultimo_nodo,
            'filtros_palabras': registro.filtros_palabras
        }

        # Devuelvo la respuesta con el diccionario como dato
        return respuesta(True, u'Se obtuvo la id_charla con exito', dato=dic)
