# -*- coding: utf-8 -*-

from at.orm.decorators import DBSwitch
from at.utiles.conversiones import respuesta
from sqlalchemy.sql.expression import and_
from sqlalchemy import or_
from at.utiles.conversiones import sqla_to_dict
from at.utiles.singleton_decorador import singleton


# @singleton
@DBSwitch('cp')
class FiltrosRepository:

    def __init__(self):
        pass


    def listar(self):
        """
        lista todos los filtros_palabras
        """
        session = self.cp.binded_session()
        try:
            tabla = self.cp.tabla('filtros_palabras')

            # obtiene todos los filtros
            filtros = session.query(tabla).all()

            if filtros:  # si hay filtros
                # print("si hay filtrso")
                filtros = sqla_to_dict(filtros)  # lo pasa a dict
                rta = respuesta(True, u"ok", filtros)
            else:  # si no hay filtros
                rta = respuesta(False, u"No hay charla para actualizar")

        except Exception as error:
            rta = respuesta(False, u"No se pudo actualizar los filtros de la charla", error)
        finally:
            session.close()

        return rta

    def listar_filtros_call(self):
        # Abrimos la sesiòn
        session = self.cp.binded_session()

        # Obtenemos la tabla 'filtros_call'
        filtrosCall = self.cp.tabla('filtros')

        # Hacemos la query
        q_filtros = session.query(filtrosCall.IdFiltro, filtrosCall.Nombre)  # solo los campos IdFiltro y Nombre ???

        # Obtenemos todos los registros
        registros = q_filtros.all()

        session.close()

        # Si no existen registros en la tabla, retornamos False y el mensaje correspondiente
        if len(registros) == 0:
            return respuesta(False, u'No se encontraron filtros call')

        lista = []

        for registro in registros:
            ob = {'IdFiltro': registro[0],
                  'Nombre': registro[1]}

            # Insertamos en la lista el registro actual
            lista.append(ob)

        # Devolvemos una respuesta con True, un mensaje y el diccionario como dato.
        return respuesta(True, u'Se obtuvieron los filtros call con exito', dato=lista)

    def obtener(self, filtros_call_id):

        if not filtros_call_id:
            return respuesta(False, u'no hay id')

        # Abrimos la sesiòn
        session = self.cp.binded_session()

        # Obtenemos la tabla 'filtros_palabra'
        filtrosPalabra = self.cp.tabla('filtros_palabras')

        # Hacemos la query filtrando por el campo 'id_filtros_call' (clave forànea)
        q_filtros = session.query(filtrosPalabra).filter(filtrosPalabra.IdFiltro == filtros_call_id)

        # Obtenemos todos los registros
        registros = q_filtros.all()

        session.close()

        # Si no existen registros en la tabla, retornamos False y el mensaje correspondiente
        if len(registros) == 0:
            return respuesta(False, u'No se encontraron filtros call')

        lista = []

        # Iteramos los registros
        for registro in registros:
            # Convertimos el registro en diccionario
            reg_dict = self.__registro_a_diccionario(registro)

            # Insertamos en la lista el registro actual
            lista.append(reg_dict)

        # Devolvemos una respuesta con True, un mensaje y el diccionario como dato.
        return respuesta(True, u'Se obtuvieron los filtros call con exito', dato=lista)

    def agregar(self, palabra, id_filtro_call):

        dic = {
            'palabra': palabra,
            'IdFiltro': id_filtro_call
        }

        # Insertamos el registro en la BBDD
        rta = self.cp.insertar('filtros_palabras', 'id', **dic)

        # Si fallamos al insertar en la BBDD
        if rta['Estado'] is False:
            return respuesta(False, u'No se puedo insertar el filtro %s, posiblemente esté duplicada' % palabra)

        # Si se insertó correctamente retornamos la respuesta con True y un mensaje
        return respuesta(True, u'Se inserto correctamente el filtro %s' % palabra, rta['ValorCampo'])

    def actualizar(self, id_filtro_palabra, palabra):

        if not id_filtro_palabra:
            return respuesta(False, u'No existe la clave')

        # Abrimos la sesión
        session = self.cp.binded_session()
        try:
            # Conectamos con la tabla filtros_palabras
            filtros = self.cp.tabla('filtros_palabras')

            q_obj = session.query(filtros).get(id_filtro_palabra)
            q_obj.palabra = palabra
            session.commit()
            rta = respuesta(True, u'Se actualizo con exito la palabra %s' % palabra)

        except AttributeError as error:
            rta = respuesta(False, u'No existe el atributo, quizas olvido enviar la clave', error)

        except Exception as error:
            rta = respuesta(False, u'No se pudo actualizar el registro. Falla general', error)

        finally:
            session.close()

        return rta

    def borrar(self, id_filtro_palabra):

        # Abro una nueva sesión
        session = self.cp.binded_session()

        # Obtengo la tabla filtros_palabra
        filtros = self.cp.tabla('filtros_palabras')

        # Hacemos la query
        q_obj = session.query(filtros).filter(filtros.id == id_filtro_palabra)

        # si no se encontró el filtro
        if q_obj.count() == 0:
            session.close()

            return respuesta(False, u'No se encontro una palabra con id %s' % id_filtro_palabra)

        # Si existe dicho registro, lo borramos
        q_obj.delete()

        # Guardamos los cambios en la BBDD
        session.commit()

        # Cerramos sesión
        session.close()

        return respuesta(True, u'la palabra ha sido borrada con exito')

    def __registro_a_diccionario(self, registro):
        '''
        Esta funcion se encarga de transformar un objeto de SQLAlchemy a un diccionario.
        '''

        dic = registro.__dict__  # Transformamos el registro en diccionario

        dic.pop('_sa_instance_state')  # Borramos la entrada que contiene la instanciacion de la clase de SQLAlchemy

        return dic  # Devolvemos el diccionario
