# -*- coding: utf-8 -*-

import pprint
import re
from at.utiles.conversiones import respuesta
from at.cpbot.bdcpbot import CpBotBd
import at.cpbot.funcionestags


CP_BOT_BD = CpBotBd()


def formatear_texto(texto, parametros):
    '''
    Esta funcion se encarga de formatear un texto tag reemplazando los elementos que se encuentran entre "<>"

    :param texto_tag (str): El texto utilizado para un mensaje
    :return:(dict) Un respuesta con Estado, un mensaje y de ser posible un dato.
    '''

    try:
        regex = '(?<=\<)(.*?)(?=\>)'  # Este regex toma los elementos encerrados entre medio de < >, ejemplo: <empresa>
        tags = re.findall(regex, texto)  # Intentamos buscar todos los tags dentro del texto.

        if len(tags) == 0:  # Si no encontramos ningun tag
            # Devolvemos una respuesta con True y el texto como dato
            return respuesta(True, u'El texto no tiene ningun tag', dato=texto)

        # Obtenemos los datos correspondientes a cada tag y los guardamos en una lista
        tags_datos = [obtener_tag(tag) for tag in tags]

        dic = {}  # Creo un diccionario donde ire guardando el id_tag y su texto

        # Itero a traves de los tags obtenidos
        for tag in tags_datos:

            if tag['tipo'] == "sistema":  # Si es un tag del sistema

                dic[tag['id_tag']] = tag['texto']  # Guardo el texto del tag en un diccionario

            if tag['tipo'] == "funcional":  # Si es un tag funcional

                funcion = getattr(at.cpbot.funcionestags, tag['texto'])
                print("FUNCIONNNN: ", funcion)
                rta_eval = funcion(parametros)

                if rta_eval['Estado'] is False:  # Si no pude ejecutar la funcion
                    raise Exception(u'ERROR EN TAG:' % tag)  # Levanto una excepcion

                texto_generado = rta_eval['Dato']  # Guardo el texto generado por la funcion

                dic[tag['id_tag']] = texto_generado  # Guardo en el diccionario junto con el id_tag

            if tag['tipo'] == "usuario":  # Si es un tag creado por el usuario

                dic[tag['id_tag']] = tag['texto']  # Guardo el texto del tag en un diccionario

        texto_nuevo = texto  # Inicializo la variable del texto que devolvere

        # Itero a traves del diccionario
        for key, value in dic.items():

            if texto_nuevo == texto:  # Si todavia no reemplazamos el primer tag

                # Utilizamos la funcion para modificar los tags en el titulo
                texto_nuevo = reemplazar_en_texto(key, value, texto)

            else:  # Si ya reemplazamos algun tag

                texto_nuevo = reemplazar_en_texto(key,value,texto_nuevo)



        # Devolvemos el titulo modificado
        return respuesta(True, u'Se formateo el texto con exito', dato=texto_nuevo)

    except Exception as error:
        print("ERROR EN TAGUTILES: ", error)
        return respuesta(False, u'No se pudo formatear el texto: %s' % error)


def reemplazar_en_texto(id_tag, texto_tag, titulo_original):
    '''
    Esta funcion se encarga de reemplazar todos los tags dentro del titulo por los textos ya obtenidos.

    :param id_tag (str): Un texto unico para identificar al tag.
    :param texto_tag (str): El texto generado dinamicamente o modificable de un tag.
    :param titulo_original (str): El titulo original que cambiaremos.
    :return: (str) El titulo modificado
    '''

    # Guardamos el regex que utilizaremos para identificar los tags que se ven como: <ID_TAG>
    regex = "<" + id_tag + ">"

    try:  # Iniciamos un try para levantar excepcion en caso de fallar

        # Reemplazamos en el titulo
        nuevo_titulo = titulo_original.replace(regex, texto_tag)

        if nuevo_titulo == titulo_original:  # Si el titulo original es igual al titulo que deberia estar cambiado
            raise Exception(u'ERROR EN TAG %s' % id_tag)  # Levanto una excepcion

        # Devuelvo el titulo modificado
        return nuevo_titulo

    except Exception as error:
        print("FALLA EN TAGUTILES")
        raise Exception(u'No se pudo reemplazar el tag', error)


def obtener_tag(tag):
    '''
    Esta funcion se encarga de obtener el texto de un tag buscando en la base de datos

    :param tag (str): El id_tag
    :return: El texto referente al id_tag
    '''

    rta = CP_BOT_BD.obtener_tag(tag)

    if rta['Estado'] is False:
        return

    return rta['Dato']