# -*- coding: utf-8 -*-
import datetime
import unittest

from at.cpbot.sqla_repository_charla import CharlasRepository
from at.cpbot.sqla_repository_filtros_palabras import FiltrosRepository
from at.orm.abm import ABM
from srvweb.cpbot.controlador_bot import buscar_filtros_en_mensaje
from at.cpbot.domain.filtro import *
from srvweb.cpbot.controlador_filtros import ControladorFiltros


class TestControladorFiltros(unittest.TestCase):
    def setUp(self):
        self.controlador_filtros = ControladorFiltros()

    def test_abm_filtros(self):

        datos = {'metodo': 'agregar', 'palabra': 'chlho', 'id_filtro_call': 87}
        rta = self.controlador_filtros.filtros(datos)
        assert rta['Estado'] is True

        datos = {'metodo': 'actualizar', 'palabra': 'lactu', 'id_filtro_palabra': rta['Dato']}
        rta2 = self.controlador_filtros.filtros(datos)
        assert rta2['Estado'] is True

        datos = {'metodo': 'borrar', 'id_filtro_palabra': rta['Dato']}
        self.controlador_filtros.filtros(datos)
        assert rta['Estado'] is True

        # self.controlador_filtros.borrar(datos)
        # self.controlador_filtros.obtener


class TestFiltros(unittest.TestCase):

    def setUp(self):
        self.filtros_repository = FiltrosRepository()
        self.charlas_repository = CharlasRepository()

        self.cp = {
            'user': 'root',
            'passwd': 'changeme',
            'dbname': 'cp2000',
            'name': 'cp',
            'host': '192.168.0.51',
            'engine': 'mysql',
            'port': '3306',
        }

        self.filtros_repository.cp = ABM(self.cp)

        self.cpbot = {
            'user': 'postgres',
            'passwd': 'TrackAir',
            'dbname': 'cpbot',
            'name': 'cpbot',
            'host': '172.17.0.2',
            'engine': 'postgres',
            'port': '5433',
        }

        self.charlas_repository.cpbot = ABM(self.cpbot)

    def tearDown(self):
        setear_filtros(None)

    def limpiar_tabla_charla(self):
        self.charlas_repository.cpbot.borrar('charla')

    def insertar_registro(self):
        charla = {'telefono_origen': 'Ivancito',
                  'automatico': 1,
                  'estado': 'activa',
                  'datos_servicio': "{}",
                  'fecha_hora_ultimo_mensaje': datetime.datetime.now(),
                  'telefono_destino': '4131313',
                  'id_ultimo_nodo': 1,
                  'usuario_responsable': 'Automatico',
                  'intentos_fallidos': 0,
                  'filtros_palabras': None,
                  'id_modo': 1}

        self.charlas_repository.cpbot.insertar('charla', 'id_charla', **charla)

    def test_charlas_insertar(self):
        self.limpiar_tabla_charla()

        self.seteo()  # lista las palabras y las guarda en la variable filtros
        rta = buscar_filtros_en_mensaje(u"perro débito y perra")

        filtros_encontrados = rta['Dato']
        print(filtros_encontrados)
        charla = {'telefono_origen': 'Ivancito',
                  'automatico': 1,
                  'estado': 'activa',
                  'datos_servicio': "{}",
                  'fecha_hora_ultimo_mensaje': datetime.datetime.now(),
                  'telefono_destino': '4131313',
                  'id_ultimo_nodo': 1,
                  'usuario_responsable': 'Automatico',
                  'intentos_fallidos': 0,
                  'filtros_palabras': filtros_encontrados,
                  'id_modo': 1}

        self.charlas_repository.cpbot.insertar('charla', 'id_charla', **charla)
        # falta obtener el filtro y ver que sea el que inserte
        # falta obtener el filtro y ver que sea el que inserte
        rta = self.charlas_repository.obtener_charla('Ivancito')
        assert rta['Estado'] is True
        assert rta['Dato']['filtros_palabras'] == '91,87'  # porque tarjeta es 91 y perro 87

    def test_charlas_sin_coincidencia(self):
        self.limpiar_tabla_charla()

        self.seteo()  # lista las palabras y las guarda en la variable filtros
        rta = buscar_filtros_en_mensaje(u"hola voy a corrientes 123")

        filtros_encontrados = rta.get('Dato')
        charla = {'telefono_origen': 'Ivancito',
                  'automatico': 1,
                  'estado': 'activa',
                  'datos_servicio': "{}",
                  'fecha_hora_ultimo_mensaje': datetime.datetime.now(),
                  'telefono_destino': '4131313',
                  'id_ultimo_nodo': 1,
                  'usuario_responsable': 'Automatico',
                  'intentos_fallidos': 0,
                  'filtros_palabras': filtros_encontrados,
                  'id_modo': 1}

        self.charlas_repository.cpbot.insertar('charla', 'id_charla', **charla)

        # falta obtener el filtro y ver que sea el que inserte
        rta = self.charlas_repository.obtener_charla('Ivancito')
        assert rta['Estado'] is True
        assert rta['Dato']['filtros_palabras'] == None

    def test_charlas_actualizar(self):
        telefono = 'Ivancito'
        self.limpiar_tabla_charla()

        self.seteo()  # lista las palabras y las guarda en la variable filtros
        rta = buscar_filtros_en_mensaje(u"hola voy a rioja 123 con cta cte")
        filtros_encontrados = rta.get('Dato')  # encuentre lo que encuentre lo guarda

        charla = {'telefono_origen': telefono,
                  'automatico': 1,
                  'estado': 'activa',
                  'datos_servicio': "{}",
                  'fecha_hora_ultimo_mensaje': datetime.datetime.now(),
                  'telefono_destino': '4131313',
                  'id_ultimo_nodo': 1,
                  'usuario_responsable': 'Automatico',
                  'intentos_fallidos': 0,
                  'filtros_palabras': filtros_encontrados,
                  'id_modo': 1}

        self.charlas_repository.cpbot.insertar('charla', 'id_charla', **charla)  # crea la charla con el id del perro

        # falta obtener el filtro y ver que sea el que inserte
        rta = self.charlas_repository.obtener_charla(telefono)
        assert rta['Estado'] is True
        assert rta['Dato']['filtros_palabras'] == '1'

    def test_charlas_actualizar3(self):
        telefono = 'Ivancito'
        self.limpiar_tabla_charla()

        self.seteo()  # lista las palabras y las guarda en la variable filtros
        rta = buscar_filtros_en_mensaje(u"hola voy a rioja 123 con perro")
        filtros_encontrados = rta.get('Dato')  # encuentre lo que encuentre lo guarda

        charla = {'telefono_origen': telefono,
                  'automatico': 1,
                  'estado': 'activa',
                  'datos_servicio': "{}",
                  'fecha_hora_ultimo_mensaje': datetime.datetime.now(),
                  'telefono_destino': '4131313',
                  'id_ultimo_nodo': 1,
                  'usuario_responsable': 'Automatico',
                  'intentos_fallidos': 0,
                  'filtros_palabras': filtros_encontrados,
                  'id_modo': 1}

        self.charlas_repository.cpbot.insertar('charla', 'id_charla', **charla)  # crea la charla con el id del perro

        # obtener el id charla
        rta = self.charlas_repository.obtener_charla(telefono)
        assert rta['Estado'] is True
        id_charla_insertada = rta['Dato']['id_charla']

        rta = buscar_filtros_en_mensaje(u"me olvide que era con tarjeta y perra")  # llega un nuevo mensaje
        filtros_encontrados = rta.get('Dato')  # agarra el/los id

        # trata de acualizar los filtros
        self.charlas_repository.actualizar_filtros(id_charla_insertada, filtros_encontrados)
        # debe haber 2 ids ahora. el del perro y el de tarjeta. porque el de la perra se repite

        # falta obtener el filtro y ver que sea el que inserte
        rta = self.charlas_repository.obtener_charla(telefono)
        assert rta['Estado'] is True
        assert rta['Dato']['filtros_palabras'] == '91,87'  # porque tarjeta es 91 y perro 87

    def test_charlas_actualizar_none(self):
        telefono = 'Ivancito'
        self.limpiar_tabla_charla()

        self.seteo()  # lista las palabras y las guarda en la variable filtros
        rta = buscar_filtros_en_mensaje(u"hola voy a rioja 123")
        filtros_encontrados = rta.get('Dato')  # encuentre lo que encuentre lo guarda

        charla = {'telefono_origen': telefono,
                  'automatico': 1,
                  'estado': 'activa',
                  'datos_servicio': "{}",
                  'fecha_hora_ultimo_mensaje': datetime.datetime.now(),
                  'telefono_destino': '4131313',
                  'id_ultimo_nodo': 1,
                  'usuario_responsable': 'Automatico',
                  'intentos_fallidos': 0,
                  'filtros_palabras': filtros_encontrados,
                  'id_modo': 1}

        self.charlas_repository.cpbot.insertar('charla', 'id_charla', **charla)  # crea la charla con el id del perro

        # obtener el id charla
        rta = self.charlas_repository.obtener_charla(telefono)
        assert rta['Estado'] is True
        id_charla_insertada = rta['Dato']['id_charla']

        rta = buscar_filtros_en_mensaje(u"me olvide que era con tarjeta y perra")  # llega un nuevo mensaje
        filtros_encontrados = rta.get('Dato')  # agarra el/los id

        # trata de acualizar los filtros
        self.charlas_repository.actualizar_filtros(id_charla_insertada, filtros_encontrados)
        # debe haber 2 ids ahora. el del perro y el de tarjeta. porque el de la perra se repite

        # falta obtener el filtro y ver que sea el que inserte
        rta = self.charlas_repository.obtener_charla(telefono)
        assert rta['Estado'] is True
        assert rta['Dato']['filtros_palabras'] == '91,87'  # porque tarjeta es 91 y perro 87

    def test_charlas_actualizar_none_none(self):
        telefono = 'Ivancito'
        self.limpiar_tabla_charla()

        self.seteo()  # lista las palabras y las guarda en la variable filtros
        rta = buscar_filtros_en_mensaje(u"hola voy a rioja 123")
        filtros_encontrados = rta.get('Dato')  # encuentre lo que encuentre lo guarda

        charla = {'telefono_origen': telefono,
                  'automatico': 1,
                  'estado': 'activa',
                  'datos_servicio': "{}",
                  'fecha_hora_ultimo_mensaje': datetime.datetime.now(),
                  'telefono_destino': '4131313',
                  'id_ultimo_nodo': 1,
                  'usuario_responsable': 'Automatico',
                  'intentos_fallidos': 0,
                  'filtros_palabras': filtros_encontrados,
                  'id_modo': 1}

        self.charlas_repository.cpbot.insertar('charla', 'id_charla', **charla)

        # obtener el id charla
        rta = self.charlas_repository.obtener_charla(telefono)
        assert rta['Estado'] is True
        id_charla_insertada = rta['Dato']['id_charla']

        rta = buscar_filtros_en_mensaje(u"me olvide que era rapido")  # llega un nuevo mensaje
        filtros_encontrados = rta.get('Dato')  # agarra el/los id

        # trata de acualizar los filtros
        self.charlas_repository.actualizar_filtros(id_charla_insertada, filtros_encontrados)

        # falta obtener el filtro y ver que sea el que inserte
        rta = self.charlas_repository.obtener_charla(telefono)
        assert rta['Estado'] is True
        assert rta['Dato']['filtros_palabras'] == None

    def test_charlas_actualizar_perro_none(self):
        telefono = 'Ivancito'
        self.limpiar_tabla_charla()

        self.seteo()  # lista las palabras y las guarda en la variable filtros
        rta = buscar_filtros_en_mensaje(u"hola voy a rioja 123 con perro")
        filtros_encontrados = rta.get('Dato')  # encuentre lo que encuentre lo guarda

        charla = {'telefono_origen': telefono,
                  'automatico': 1,
                  'estado': 'activa',
                  'datos_servicio': "{}",
                  'fecha_hora_ultimo_mensaje': datetime.datetime.now(),
                  'telefono_destino': '4131313',
                  'id_ultimo_nodo': 1,
                  'usuario_responsable': 'Automatico',
                  'intentos_fallidos': 0,
                  'filtros_palabras': filtros_encontrados,
                  'id_modo': 1}

        self.charlas_repository.cpbot.insertar('charla', 'id_charla', **charla)  # crea la charla con el id del perro

        # obtener el id charla
        rta = self.charlas_repository.obtener_charla(telefono)
        assert rta['Estado'] is True
        id_charla_insertada = rta['Dato']['id_charla']

        rta = buscar_filtros_en_mensaje(u"me olvide que era rapido")  # llega un nuevo mensaje
        filtros_encontrados = rta.get('Dato')  # agarra el/los id

        # trata de acualizar los filtros
        self.charlas_repository.actualizar_filtros(id_charla_insertada, filtros_encontrados)
        # debe haber 2 ids ahora. el del perro y el de tarjeta. porque el de la perra se repite

        # falta obtener el filtro y ver que sea el que inserte
        rta = self.charlas_repository.obtener_charla(telefono)
        assert rta['Estado'] is True
        assert rta['Dato']['filtros_palabras'] == '87'  # porque tarjeta es 91 y perro 87

    def test_listar(self):
        rta = self.filtros_repository.listar()
        assert rta['Estado'] is True
        assert rta['Respuesta'] == u'ok'

    def seteo(self):
        rta = self.filtros_repository.listar()
        setear_filtros(rta['Dato'])

    def test_buscar_filtro_en_mensaje01(self):
        rta = buscar_filtros_en_mensaje(None)
        assert rta['Estado'] is False
        assert rta['Respuesta'] == u'no hay mensaje'

    def test_buscar_filtro_en_mensaje02(self):
        self.seteo()
        rta = buscar_filtros_en_mensaje("hola voy a rioja 123 llevo perro")
        assert rta['Estado'] is True
        assert rta['Respuesta'] == u'ok'

    def test_buscar_filtro_en_mensaje03(self):
        self.seteo()
        rta = buscar_filtros_en_mensaje("hola voy a rioja 123 llevo perro")
        assert rta['Estado'] is True
        assert rta['Respuesta'] == u'ok'

    def test_buscar_filtro_en_mensaje04(self):
        self.seteo()
        rta = buscar_filtros_en_mensaje("hola voy a rioja 123")
        assert rta['Estado'] is False
        assert rta['Respuesta'] == u'no hubo coincidencias'

    def test_buscar_filtro_en_mensaje05(self):
        self.seteo()
        rta = buscar_filtros_en_mensaje("hola voy hacia rioja 123 con perro y con tarjeta")
        assert rta['Estado'] is True
        assert rta['Respuesta'] == u'ok'
        assert rta['Dato'] == '91,87'

    def test_buscar_filtro_en_mensaje_varias_coincidencias(self):
        self.seteo()
        rta = buscar_filtros_en_mensaje("hola voy a rioja 123 con perro y con tarjeta y perra")
        assert rta['Estado'] is True
        assert rta['Respuesta'] == u'ok'
        assert rta['Dato'] == '91,87'

    def test_buscar_filtro_en_mensaje06(self):
        rta = buscar_filtros_en_mensaje("hola voy a rioja 123 con perro y con tarjeta y perra")
        assert rta['Estado'] is False
        assert rta['Respuesta'] == u'no hay filtros palabras guardados'


#


# ejecutar este comando en la terminal para correr el test
# /home/matias/workspace/buildoutweb.cpbot/bin/mi_python /home/matias/workspace/buildoutweb.cpbot/src/at.cpbot/at/cpbot/tests/test_filtros.py
if __name__ == '__main__':
    unittest.main()
