# -*- coding: utf-8 -*-

import unittest

from at.cpbot.tagsweb import Tags
from at.orm.abm import ABM




class TestTags(unittest.TestCase):

    def setUp(self):
        self.tags = Tags()
        self.datos_cpbot_test = {
            'user': 'postgres',
            'passwd': 'TrackAir',
            'dbname': 'cpbot_test',
            'name': 'cpbot_test',
            'host': '192.168.0.68',
            'engine': 'postgres',
            'port': '5432',
        }

        # abm devuelve instancia de bd creada con esos parametros
        self.tags.cpbot = ABM(self.datos_cpbot_test)
        self.tags.cpbot.borrar('tags')

        self.crear_tag('id1', 'loco', 'desc loco', 'usuario')






    def tearDown(self):
        self.tags.cpbot.borrar('tags')






    def crear_tag(self, id_tag, texto, tag, tipo):
        dic = {}
        dic['id_tag'] = id_tag
        dic['texto'] = texto
        dic['tag'] = tag
        dic['tipo'] = tipo
        self.tags.datos = dic

        rta = self.tags.agregar()

        # if rta['Estado'] is False:
        #     raise Exception("Fallo al crear tag")

        return rta






    def test_grabar(self):

        rta = self.tags.obtener()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == (u'Se obtuvieron los tags con exito'))
        self.assertTrue(
            rta['Dato'] == ([{u'id_tag': u'id1', u'texto': u'loco', u'tipo': u'usuario', u'tag': u'desc loco'}]))






    def test_grabar_el_mismo(self):

        rta = self.crear_tag('id1', 'loco', 'desc loco', 'usuario')

        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == (u'El tag id1 ya existe y no pudo ser agregado')
        )






    def test_obtener_no_hay_registros(self):
        self.tags.cpbot.borrar('tags')
        rta = self.tags.obtener()

        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == (u'No se encontraron tags')
        )






    # comento porque no borra los tags luego
    # def test_grabar_con_unicode(self):
    #     self.tags.datos = self.crear_tag('ióÑ', 'loco', 'desc loco', 'usuario')
    #
    #     rta = self.tags.agregar()
    #     self.assertTrue(rta['Estado'] is True)
    #     self.assertTrue(
    #         rta['Respuesta'] == (u'Se inserto con exito el tag ióÑ')
    #     )
    #
    #     rta = self.tags.obtener()
    #
    #     self.assertTrue(rta['Estado'] is True)
    #     self.assertTrue(rta['Respuesta'] == (u'Se obtuvieron los tags con exito'))
    #     self.assertTrue(
    #         rta['Dato'] == ([{u'id_tag': u'ióÑ', u'texto': u'loco', u'tipo': u'usuario', u'tag': u'desc loco'}]))






    def test_agregar_no_id_tag(self):
        self.crear_tag(None, 'loco', 'desc loco', 'usuario')

        rta = self.tags.agregar()
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == (u'Algun campo era nulo y no se pudo insertar el tag None')
        )






    def test_actualizar_ok(self):

        self.tags.datos = {'id_tag_viejo': 'id1', 'id_tag': 'id2', 'tag': 'desc loco2', 'texto': 'loco2',
                           'tipo': 'usuario'}
        rta = self.tags.actualizar()
        self.assertTrue(rta['Estado'] is True)

        self.assertTrue(
            rta['Respuesta'] == (u'Se modifico el tag %s con exito' % self.tags.datos['id_tag'])
        )

        rta = self.tags.obtener()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == (u'Se obtuvieron los tags con exito'))
        self.assertTrue(
            rta['Dato'] == ([{u'id_tag': u'id2', u'texto': u'loco2', u'tipo': u'usuario', u'tag': u'desc loco2'}]))






    def test_actualizar_sin_id_tag_viejo(self):

        self.tags.datos = {'id_tag': 'id2', 'tipo': 'usuario'}
        rta = self.tags.actualizar()
        self.assertTrue(rta['Estado'] is False)

        self.assertTrue(
            rta['Respuesta'] == (u'Falta la key id_tag_viejo en el diccionario')
        )






    def test_actualizar_sin_id_tag(self):

        self.tags.datos = {'id_tag_viejo': 'id2', 'tipo': 'usuario'}
        rta = self.tags.actualizar()
        self.assertTrue(rta['Estado'] is False)

        self.assertTrue(
            rta['Respuesta'] == (u'Falta la key id_tag en el diccionario')
        )






    def test_actualizar_sin_tipo(self):

        self.tags.datos = {'id_tag_viejo': 'id1', 'id_tag': 'id2'}
        rta = self.tags.actualizar()
        self.assertTrue(rta['Estado'] is False)

        self.assertTrue(
            rta['Respuesta'] == (u'Falta la key tipo en el diccionario')
        )






    def test_actualizar_id_inexistente(self):

        self.tags.datos = {'id_tag_viejo': 'xxx', 'id_tag': 'id2', 'tipo': 'usuario'}
        rta = self.tags.actualizar()
        self.assertTrue(rta['Estado'] is False)

        self.assertTrue(
            rta['Respuesta'] == (u'No se encontro un tag con la id: %s' % self.tags.datos['id_tag'])
        )

        rta = self.tags.obtener()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == (u'Se obtuvieron los tags con exito'))
        self.assertTrue(
            rta['Dato'] == ([{u'id_tag': u'id1', u'texto': u'loco', u'tipo': u'usuario', u'tag': u'desc loco'}]))






    def test_actualizar_sin_registros(self):
        self.tags.datos = {'id_tag_viejo': 'xxx', 'id_tag': 'id2', 'tipo': 'usuario'}
        rta = self.tags.actualizar()
        self.assertTrue(rta['Estado'] is False)

        self.assertTrue(
            rta['Respuesta'] == (u'No se encontro un tag con la id: %s' % self.tags.datos['id_tag'])
        )






    def test_actualizar_tipo_funcional(self):

        self.tags.datos = {'id_tag_viejo': 'id1', 'id_tag': 'id2', 'tipo': 'funcional'}
        rta = self.tags.actualizar()
        self.assertTrue(rta['Estado'] is False)

        print(rta)
        self.assertTrue(
            rta['Respuesta'] == (u'Los tags funcionales no pueden ser modificados')
        )






    def test_borrar(self):

        self.tags.datos = {'id_tag': 'id1', 'tipo': 'usuario'}
        rta = self.tags.borrar()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == (u'Se borro el tag %s con exito' % self.tags.datos['id_tag'])
        )

        rta = self.tags.obtener()
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == (u'No se encontraron tags')
        )






    def test_borrar_id_inexistente(self):

        self.tags.datos = {'id_tag': 'id', 'tipo': 'usuario'}
        rta = self.tags.borrar()
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == (u'No se encontro el tag: %s' % self.tags.datos['id_tag'])
        )






    def test_borrar_tipo_no_usuario(self):

        self.tags.datos = {'id_tag': 'id1', 'tipo': 'sistema'}
        rta = self.tags.borrar()
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == (u'Solo los tags del usuario pueden ser borrados')
        )





def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestTags))
    return suite
