# -*- coding: utf-8 -*-

import re

from at.cpbot.cpadapter import CpBotAdapter
from at.cpbot.bdcpbot import CpBotBd
from at.cpbot.atmsjadapter import finalizar_charla
from at.utiles.conversiones import respuesta
import at.cpbot.atmsjadapter as atmsj
from at.cpbot.tagutiles import formatear_texto
from at.cpbot.atmsjadapter import enviar_atmsj

cpadapter = CpBotAdapter()
cpbotbd = CpBotBd()

'''
En este archivo se encuentran funciones pequeñas que son utilizadas en modospermitidos.py y funcionescpbot.py
'''


def validar_direccion(direccion):
    """Se encarga de validar que una direccion sea valida"""
    try:
        int(direccion)  # Si quedo solo el numero luego de la busqueda
        return False
    except:
        return True


def existe_un_numero(s):
    try:
        return any(i.isdigit() for i in s)
    except:
        return False


def obtener_estado_del_servicio(id_servicio):
    '''
    Esta funcion se encarga de obtener el estado del servicio haciendo una query a la tabla 'actual' de la bd cp2000
    '''

    # Intento obtener el estado del servicio
    rta_obtener_estado_servicio = cpadapter.obtener_estado_servicio(id_servicio)

    if rta_obtener_estado_servicio['Estado'] is False:  # Si no pude obtener el estado del servicio
        # Levanto una excepcion
        raise Exception("Error al obtener el estado servicio", rta_obtener_estado_servicio['Respuesta'])

    return rta_obtener_estado_servicio['Dato']  # Devolvemos el estado del servicio


def obtener_texto_a_enviar(parametros, id_tag):
    '''
    Esta funcion se encarga de obtener el texto que se debe enviar de la tabla 'tags', luego lo formatea y lo devuelve.
    '''

    rta_texto_tag = cpbotbd.obtener_texto_tag(id_tag)  # Intento obtener el texto del tag cuando no esta despachado

    if rta_texto_tag['Estado'] is False:  # Si no pude obtener el texto del tag
        raise Exception("Error al obtener texto para enviar", rta_texto_tag['Respuesta'])

    texto_tag = rta_texto_tag['Dato']

    # Intentamos formatear el texto del mensaje
    rta_formatear_texto = formatear_texto(texto_tag, parametros)

    if rta_formatear_texto['Estado'] is False:  # Si no pudimos foramtear el texto
        raise Exception("Error al obtener texto para enviar", rta_formatear_texto['Respuesta'])

    return rta_formatear_texto['Dato']


def enviar_estado_servicio(parametros, texto):
    '''
    Esta funcion se encarga de enviar un mensaje con el estado del servicio hacia atmsj.
    '''

    # Envio el mensaje a traves de atmsj
    rta_enviar_atmsj = enviar_atmsj(mensaje=texto,
                                    tel_origen=parametros['charla']['telefono_destino'],
                                    tel_destino=parametros['charla']['telefono_origen'],
                                    ip_atmsj=parametros['ip_atmsj'])

    # Si no pude enviar el mensaje por atmsj
    if rta_enviar_atmsj['Estado'] is False:
        raise Exception(u'Error al enviar el mensaje a atmsj', rta_enviar_atmsj['Respuesta'])


def verificar_si_es_telefono(nombre):
    '''
    Funcion que se encarga de verificar si un telefono es un numero de telefono o un contacto del tipo ivan,
    asi evitamos cargar el servicio
    exponiendo el telefono real del usuario.

    :param nombre: (str) El nombre o telefono de un usuario
    :return: (bool)
    '''

    if not nombre: return False

    regex = "[a-zA-Z]"  # El regex que utilizaremos solamente busca letras encontradas en la codificacion ASCII

    resultados = re.findall(regex, nombre)  # Verificamos si el nombre contiene letras

    # Convertimos la lista obtenida a un booleano, si devuelve una lista vacia, sera False, sino True.
    es_nombre = bool(resultados)

    return es_nombre
