# -*- coding: utf-8 *-*

import os
import sys
import argparse
import cjson

from at.map import _
from multiprocessing import Process
from multiprocessing import Manager
from time import sleep
from sqlalchemy.sql.expression import func

from at.map.tile_generator import TileGenerator
from at.map.mapas import Mapas
from at.map.calle import CallePostgres
from at.map import RUTA_LOGS
from at.map.utiles.funciones_geo import FuncionesGeo
from at.utiles.archivos import crear_archivo_zip
from at.utiles.hilos import ejecutar_hilo
from at.utiles.procesos import ejecutar_multiples_procesos
from at.utiles.errores import obtener_traceback
from at.utiles.conversiones import respuesta

import socket
# Creamos el objeto de mapas
mapas = Mapas()
# Nuestro generador de tiles
tg = TileGenerator()

# Instancio FuncionesGeo
f_geo = FuncionesGeo()

# Instancio FuncionesGeo
calle_postgres = CallePostgres()

# Iniciamos el log
import at.map.log
at.map.log.LOG_PATH = RUTA_LOGS
at.map.log.start('%s.debug' % RUTA_LOGS)

from twisted.python import log


def _generate(tiles_en_proceso, x, y, z, basico, sub_directorio):
    try:
        x_tmp, y_tmp = mapas.obtener_tile_padre_xy(x, y)
        tile = str(x_tmp) + '-' + str(y_tmp) + '-' + str(z)
        tg.generate(x, y, z, basico, sub_directorio)
        if len(tiles_en_proceso) != 0:
            tiles_en_proceso.remove(tile)
    except Exception, e:
        if type(e) != socket.error:
            pass

def generate_area():
    parser = argparse.ArgumentParser(description='AirTrack')
    parser.add_argument(
        '-p1', action="store", nargs=2, dest="punto_inferior_izquierdo",
        required=True
    )
    parser.add_argument(
        '-p2', action="store", nargs=2, dest="punto_superior_derecho",
        required=True
    )
    parser.add_argument(
        '-z1', action="store", dest="z1", required=False
    )
    parser.add_argument(
        '-z2', action="store", dest="z2", required=False
    )
    parser.add_argument(
        '-f', action="store_const", dest="forzar", const="True", required=False
    )

    parser.add_argument(
        '-b', action="store_const", dest="basico",
        const="False", required=False
    )

    args = []
    args.extend(sys.argv)
    args.pop(0)
    args = parser.parse_args(args)

    # Obtenemos el z desde
    z1 = args.z1
    if z1 is None:
        z1 = 0
    try:
        z1 = int(z1)
    except:
        z1 = 0

    # Obtenemos el z hasta
    z2 = args.z2
    if z2 is None:
        z2 = 17
    try:
        z2 = int(z2)
    except:
        z2 = 17

    # Obtenemos el punto de inicio
    if len(args.punto_inferior_izquierdo) != 2:
        print 'Los puntos deben ingresarse: -p1 Lat Lon -p2 Lat Lon'
        sys.exit(-1)
    try:
        latitud0 = float(args.punto_inferior_izquierdo[0].strip())
        longitud0 = float(args.punto_inferior_izquierdo[1].strip())
    except:
        print 'Los puntos deben ingresarse: -p1 Lat Lon -p2 Lat Lon'
        sys.exit(-1)

    # Obtenemos el punto de fin
    if len(args.punto_superior_derecho) != 2:
        print 'Los puntos deben ingresarse: -p1 Lat Lon -p2 Lat Lon'
        sys.exit(-1)
    try:
        latitud1 = float(args.punto_superior_derecho[0].strip())
        longitud1 = float(args.punto_superior_derecho[1].strip())
    except:
        print 'Los puntos deben ingresarse: -p1 Lat Lon -p2 Lat Lon'
        sys.exit(-1)

    forzar = False
    if args.forzar:
        forzar = True

    basico = False
    if args.basico:
        basico = True

    # Creamos el area
    crear_area(
        latitud0, longitud0, latitud1, longitud1, z1, z2, forzar, basico
    )


def generate_map_zip():
    parser = argparse.ArgumentParser(description='AirTrack')
    parser.add_argument(
        '-z1', action="store", dest="z1", required=True
    )
    parser.add_argument(
        '-z2', action="store", dest="z2", required=True
    )

    args = []
    args.extend(sys.argv)
    args.pop(0)
    args = parser.parse_args(args)

    try:
        # Obtenemos el z desde
        z1 = int(args.z1)

        # Obtenemos el z hasta
        z2 = int(args.z2)
    except:
        print 'Las capas se deben ingresar: -z1 Z1 -z2 Z2'
        sys.exit(-1)

    # Creamos el mapa zip
    crear_mapas_zip(z1, z2)


def crear_area(latitud0, longitud0, latitud1, longitud1, z1, z2,
               forzar=False, basico=False, sub_directorio=False,
               multiprocess=True, func_rta=None):
    # Se le debe pasar un punto izquirdo inferior y un punto derecho superior
    # z1 y z2 es desde que capa hasta que capa si quiere crear
    # Si forzar es True crea las imágenes que no existen

    # Armo la respuesta.
    rta = {
        'sub_directorio': sub_directorio,
        'z1': z1,
        'z2': z2
    }

    # # SI viene con subdirectorio, le paso los zoom por defecto.
    # if sub_directorio is not False:
    #     z1 = 10
    #     z2 = 17

    if multiprocess is True:
        tiles_en_proceso = Manager().list()
    else:
        tiles_en_proceso = []

    funciones = []

    while z1 <= z2:
        # log.msg('Recorremos la capa %s' % z1, system='area')
        # Calculamos los x e y dinamicamente
        x0_tmp = int((longitud0 + 180) * (2 ** (z1 + 1)) / 360)
        y0_tmp = int((latitud0 + 90) * (2 ** z1) / 180)
        x1_tmp = int((longitud1 + 180) * (2 ** (z1 + 1)) / 360)
        y1_tmp = int((latitud1 + 90) * (2 ** z1) / 180)

        # Recorremos la extension desde el x0,y0 a x1, y1
        for i in range(x0_tmp, x1_tmp + 1):

            for j in range(y0_tmp, y1_tmp + 1):

                if multiprocess is False:
                    _generate(
                        tiles_en_proceso, i, j, z1, basico, sub_directorio
                    )
                else:
                    # # Si no existe vamos a ver si podemos crearlo. En caso
                    # # de que se este forzando se va a crear si o si
                    # if not mapas.existe_imagen_tile(i, j, z1, basico) or forzar:
                    #     x, y = mapas.obtener_tile_padre_xy(i, j)
                    #     tile = str(x) + '-' + str(y) + '-' + str(z1)
                    #     # Vemos si esta o no en proceso
                    #     if not tile in tiles_en_proceso:
                    #         for k in range(100):
                    #             # Si hay menos de 4 tiles en proceso, seguimos
                    #             if len(tiles_en_proceso) < 6:
                    #                 # Agregamos el tile en proceso
                    #                 tiles_en_proceso.append(tile)
                    #
                    #                 p = Process(
                    #                     target=_generate,
                    #                     args=(
                    #                         tiles_en_proceso,
                    #                         i,
                    #                         j,
                    #                         z1,
                    #                         basico,
                    #                         sub_directorio
                    #                     )
                    #                 )
                    #                 p.start()
                    #                 # Salimos del for
                    #                 break
                    #                 # Sino, nos dormimo
                    #             else:
                    #                 sleep(.1)
                    # NUEVA FORMA DE EJECUTAR EN MULTIPROCESOS
                    # Voy guardando todas las funciones que quiero ejecutar
                    funciones.append(
                        [
                            _generate,
                            (tiles_en_proceso, i, j, z1, basico, sub_directorio)
                        ]
                    )

        # Aumentamos el z index
        z1 += 1

    if len(funciones) > 0:
        daemon = False
        max_procesos = 4
        ejecutar_multiples_procesos(
            funciones, daemon, func_rta, rta, max_procesos
        )

    if sub_directorio is not False:
        return rta


def crear_mapas_zip(z1=12, z2=16):
    '''
        Esta función se encarga de crear un archivo ZIP que va a contener la
        carpeta /var/lib/at/mapas desde un Z mínimo a un Z máximo.
        Todas las imágenes tiene que tener la extensión .map en vez de .jpeg

        Funciones:
            crear_archivo_zip

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    ruta_archivo = '/var/lib/at/mapas/'

    ruta_destino = '/var/lib/at/mapas/comprimidos'
    if not os.path.isdir(ruta_destino):
        os.mkdir(ruta_destino)

    carpetas_excluidas = []
    for i in range(20):
        if i < z1 or i > z2:
            carpetas_excluidas.append('z' + str(i))
            carpetas_excluidas.append(str(i))

    archivos_excluidos = []

    nombre_zip = 'mapas_z' + str(z1) + '_z' + str(z2) + '.zip'

    carpeta_raiz = 'mapas'

    nueva_extension = '.map'
    rta = crear_archivo_zip(ruta_archivo, ruta_destino, nombre_zip,
                            carpeta_raiz, carpetas_excluidas,
                            archivos_excluidos, nueva_extension)

    return rta


def generar_area_linea(puntos, sub_directorio=False, z1=10, z2=17):

    # Verifico que sea un parámetro válido.
    if puntos is None:
        return respuesta(
            False, _(u'No se recibieron puntos.')
        )

    # Verfico que tenga los 2 puntos.
    if len(puntos) < 2:
        return respuesta(
            False, _(u'No se recibieron puntos.')
        )

    # Trato de obtener los extremos de la geom.
    rta_extremos = f_geo.obtener_extremos_linea(puntos)

    # Verifico el resultado.
    if rta_extremos['Estado'] is not True:
        return rta_extremos

    # Obtengo los extremos.
    extremos = rta_extremos['Respuesta']

    # Creo la imagen.
    return crear_area(
        extremos['inferior_izquierdo'][0],
        extremos['inferior_izquierdo'][1],
        extremos['superior_derecho'][0],
        extremos['superior_derecho'][1],
        z1,
        z2,
        forzar=True,
        sub_directorio=sub_directorio,
        multiprocess=False
    )


def generar_area_version(operaciones, sub_directorio=False):

    # Recorro las operaciones de la versión.
    for operacion in operaciones:

        # if type(operacion) isistance
        if isinstance(operacion, dict) is False:
            operacion = operacion.__dict__

        # Obtengo el id_accion
        id_accion = operacion['id_accion']

        # Si se debe generar el objeto nuevo de un segmento.
        if id_accion == 101 or id_accion == 102 or id_accion == 111 \
                or id_accion == 112:
            # Decodifico el objeto_nuevo.
            objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

            # Obtengo los puntos.
            puntos = objeto_nuevo['puntos']

            # Trato de generar las imagenes.
            generar_area_linea(puntos, sub_directorio=sub_directorio)

        # Si se debe generar el objeto original de un segmento.
        if id_accion == 102 or id_accion == 103 or id_accion == 112 \
                or id_accion == 113:
            # Decodifico el objeto_nuevo.
            objeto_original = cjson.decode(operacion['objeto_original'])

            # Obtengo los puntos.
            puntos = objeto_original['puntos']

            # Trato de generar las imagenes.
            generar_area_linea(puntos, sub_directorio=sub_directorio)

        # Si se eliminó una calle.
        if id_accion == 203 or id_accion == 213:

            # Decodifico el objeto_nuevo.
            objeto_original = cjson.decode(operacion['objeto_original'])

            # Obtengo los segmentos.
            segmentos = objeto_original['segmentos']

            # Recorro los segmentos.
            for segmento in segmentos:
                # Obtengo los puntos.
                puntos = segmento['Puntos']

                # Trato de generar las imagenes.
                generar_area_linea(puntos, sub_directorio=sub_directorio)

        # Si se modificó una calle.
        if id_accion == 202 or id_accion == 212:

            # Decodifico el objeto_nuevo.
            objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

            # Obtengo el id_calle_nombre.
            id_calle_nombre = objeto_nuevo['id']

            # obtener_calle
            rta_obtener_calle = calle_postgres.obtener_calle(
                id_calle_nombre, obtener_segmentos=True
            )

            # Valido.
            if rta_obtener_calle['Estado'] is True:

                # Obtengo los segmentos.
                segmentos = rta_obtener_calle['Respuesta']['Segmentos']

                # Recorro los segmentos.
                for segmento in segmentos:
                    # Obtengo los puntos.
                    puntos = segmento['Puntos']

                    # Trato de generar las imagenes.
                    generar_area_linea(
                        puntos, sub_directorio=sub_directorio
                    )

    # Cuando termino retorno el subdirectorio para sincronizar las imagenes.
    return sub_directorio
