# -*- coding: utf-8 -*-

from at.map import _
from at.map.utiles.validaciones_especificas import ValidacionesEspecificas

from at.orm.decorators import DBSwitch


@DBSwitch('gis2')
class PaisPostgres(ValidacionesEspecificas):
    '''
       Esta clase contiene todos lo métodos relacionados con los paises
       trabajando con Postgres y postgis.
    '''

    def obtener_pais(self, id_pais):
        '''
            Dado el ID de una ciudad obtengo su nombre.

            Args:
                id_pais (int)

            Returns:
                pais (str)

            Raises:
                'No se ha encontrado ningun pais con el ID ingresado'

            Functions:
                validar_id_map(id_pais)
                binded_session()
                tabla('ciudades')
        '''

        rta = {'Estado': False}
        try:
            self.validar_id_mapa(id_pais)
        except:
            rta['Respuesta'] = _(u'El id del país no es válido')
            return rta

        # Abrimos la session y buscamos los paises que coincidan
        session = self.gis2.binded_session()
        tabla_paises = self.gis2.db['paises']

        try:
            q_obj = session.query(tabla_paises)
            q_obj = q_obj.filter(tabla_paises.id == id_pais)
            pais = q_obj.first()
        except:
            # Cerramos la session
            session.close()
            rta['Respuesta'] = _(u'Falló al momento de obtener el país.')
            return rta

        # Cerramos la session
        session.close()

        # SI el pais esta vacio, salimos con error
        if pais is None:
            rta['Respuesta'] = _(u'''No se ha encontrado ningun pais con
 el id: %s.''' % id_pais)
            return rta

        # Salimos correctamente
        rta['Estado'] = True
        rta['Respuesta'] = pais.nombre.encode("latin1")
        return rta

    def listar_paises(self):
        '''
            Esta funcion devuelve un listado con todas los paises disponibles
            en el sistema. Estos datos corresponden al id y al nombre de la
            misma.

            Returns:
                ciudades (list [  list [(int),(str)] ])
        '''

        # Declaramos funcion
        rta = {'Estado': False}
        # Abrimos la session y buscamos los nombres
        session = self.gis2.binded_session()
        tabla_paises = self.gis2.db['paises']

        try:
            q_obj = session.query(
                tabla_paises.id, tabla_paises.nombre
            ).order_by(tabla_paises.nombre)
            paises_tmp = q_obj.all()
        except:
            session.close()
            rta['Respuesta'] = _(u'Falló al momento de listar los paises.')
            return rta

        # Cerramos la sesion
        session.close()

        # Generamos la lista de elementos que devolveremos
        paises = []
        for pais in paises_tmp:
            p_tmp = []
            p_tmp.append(int(pais.id))
            p_tmp.append(pais.nombre.encode("latin1"))

            paises.append(p_tmp)

        # Armamos la respuesta
        rta['Estado'] = True
        rta['Respuesta'] = paises

        # Devolvemos
        return rta

    def obtener_pais_de_la_ciudad(self, id_ciudad):
        '''
            Esta función se encarga de obtener el país de una ciudad dada.

            Argumentos:
                id_ciudad (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (int)
        '''

        rta = {}

        session = self.gis2.binded_session()

        ciudades = self.gis2.tabla('ciudades')
        provincias = self.gis2.tabla('provincias')
        paises = self.gis2.tabla('paises')

        q_obj = session.query(
            ciudades.id.label('id_ciudad'),
            provincias.id.label('id_provincia'),
            paises.id.label('id_pais'),
            paises.nombre.label('nombre_pais'),
            paises.capital.label('capital_pais'),
            paises.geom.label('geom_pais'),
            paises.zona_horaria.label('zona_horaria_pais'),
        )

        q_obj = q_obj.join((
            provincias,
            ciudades.id_provincia == provincias.id
        ))

        q_obj = q_obj.join((
            paises,
            provincias.id_pais == paises.id
        ))

        q_obj = q_obj.filter(ciudades.id == id_ciudad)

        registro = q_obj.first()

        session.close()

        if registro is None:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo obtener el país de la ciudad dada')
            return rta

        rta['Estado'] = True
        datos = {}
        datos['id_pais'] = registro.id_pais
        datos['nombre_pais'] = registro.nombre_pais
        datos['capital_pais'] = registro.capital_pais
        datos['geom_pais'] = registro.geom_pais
        datos['zona_horaria_pais'] = registro.zona_horaria_pais
        rta['Respuesta'] = datos
        return rta
