# -*- coding: iso-8859-1 -*-

from at.orm.decorators import DBSwitch

from at.utiles.validar import numero_entero_positivo
from at.utiles.validar import string
from at.utiles.conversiones import respuesta
from at.map.utiles.validaciones_especificas import ValidacionesEspecificas
from at.map.pais import PaisPostgres
from at.map import _


@DBSwitch('gis2')
class ProvinciaPostgres(ValidacionesEspecificas):
    '''
       Esta clase contiene todos lo mtodos relacionados con los provincias
       trabajando con Postgres y postgis.
    '''

    def __init__(self):
        self.pais_postgres = PaisPostgres()

    def obtener_provincia(self, id_provincia):
        '''
            Dado el ID de una ciudad obtengo su nombre.

            Argumentos:
                id_provincia (int)

            Retorna:
                rta['Estado']: (bool)
                rta['Respuesta'] : provincia (str)  error (str)

            Funciones:
                validar_id_map(id_provincia)
                binded_session()
                tabla('ciudades')
        '''

        # Definimos la respuesta
        rta = {'Estado': False}

        # Validamos
        try:
            self.validar_id_mapa(id_provincia)
        except:
            rta['Respuesta'] = _(u'El id de la provincia no es vlido.')
            return rta

        # Abrimos la session y obtenemos la provincia
        session = self.gis2.binded_session()
        tabla_provincias = self.gis2.db['provincias']

        try:
            q_obj = session.query(tabla_provincias)
            q_obj = q_obj.filter(tabla_provincias.id == id_provincia)
            provincia = q_obj.first()
        except:
            session.close()
            rta['Respuesta'] = _(u'Fall al momento de obtener la provincia.')
            return rta

        # Cerramos la session
        session.close()

        # Si la provincia esta vacia, salimos
        if provincia is None:
            rta['Respuesta'] = _('''No se ha encontrado ningun provincia
 con el id: %s.''' % str(id_provincia))
            return rta

        # Acomodo los datos.
        datos = {
            'nombre': provincia.nombre.encode('latin1'),
            'id_pais': provincia.id_pais
        }

        # Salimos bien entonces
        rta['Estado'] = True
        rta['Respuesta'] = provincia.nombre.encode('latin1')
        rta['Dato'] = datos
        return rta

    def listar_provincias(self, id_pais):
        '''
            Esta funcion devuelve un listado con todas los provincias
            disponibles en el sistema que pertenezcan al pais ingresado.
            Estos datos corresponden al id y al nombre de la misma.

            Argumentos:
                id_pais (int)
            Retorna:
                rta['Estado'] = (bool)
                rta['Repsuesta'] = provincias (list [  list [(int),(str)] ])
                     error(str)
        '''

        # Definimos la respuesta
        rta = {'Estado': False}

        # Abrimos session
        session = self.gis2.binded_session()
        tabla_provincias = self.gis2.db['provincias']

        try:
            q_obj = session.query(tabla_provincias).order_by(
                tabla_provincias.nombre
            )
            q_obj = q_obj.filter(tabla_provincias.id_pais == id_pais)
            provincias_tmp = q_obj.all()
        except:
            session.close()
            rta['Respuesta'] = _(u'Fall al momento de obtener las provincias.')
            return rta

        # Cerramos la sesion
        session.close()

        # Generamos la lista de elementos que devolveremos
        provincias = []
        for provincia in provincias_tmp:
            p_tmp = []
            p_tmp.append(int(provincia.id))
            p_tmp.append(provincia.nombre.encode('latin1'))
            provincias.append(p_tmp)

        # Salimos bien entonces
        rta['Estado'] = True
        rta['Respuesta'] = provincias
        return rta

    def __agregar(self, datos):

        # Agrego la provincia.
        agregar_provincia = self.gis2.insertar(
            'provincias', 'id', **datos
        )

        # Retorno la respuesta.
        if agregar_provincia['Estado'] is not True:
            print agregar_provincia
            return respuesta(False, _(u'No se pudo agregar la provincia.'))

        # Obtengo el id.
        id = agregar_provincia['ValorCampo']

        # Guardo el id.
        datos['id'] = id

        return respuesta(
            True, _(u'Se agreg correctamente la provincia.'), datos
        )

    def grabar(self, id_pais, nombre, id_provincia=None):

        '''
        Esta funcin crea o modifica una provincia
        :param id_pais:
        :param nombre:
        :return:
        '''

        # Validaciones

        # Declaro un dic para guardar los datos.
        datos = {}

        # Valido el id_provincia si no es None.
        if id_provincia is not None:
            # Valido el id_provincia.
            v_id_provincia = numero_entero_positivo(id_provincia)
            if v_id_provincia['Estado'] is not True:
                return v_id_provincia

        # Valido que exista el pais.
        v_id_pais = self.pais_postgres.obtener_pais(id_pais)
        if v_id_pais['Estado'] is not True:
            return v_id_pais

        # Valido el id_pais.
        v_nombre = string(nombre)
        if v_nombre['Estado'] is not True:
            return v_nombre

        # Guardo los datos.
        datos['id_pais'] = id_pais
        datos['nombre'] = nombre

        # Si no tengo id_provincia, trato de agregar.
        if id_provincia is None:

            ### VERIFICO SI YA EXISTE UNA PROVINCIA CON ESE NOMBRE.

            # Abro la sesin.
            session = self.gis2.binded_session()

            # Obtengo la tabla provincias.
            tabla_provincias = self.gis2.tabla('provincias')

            q_obj = session.query(tabla_provincias)
            q_obj = q_obj.filter(tabla_provincias.nombre == nombre)
            q_obj = q_obj.filter(tabla_provincias.id_pais == id_pais)

            # Obtengo el registro.
            registro = q_obj.first()

            # Cierro sesin.
            session.close()

            if registro is not None:
                return respuesta(
                    False, _(u'Ya existe una provincia con ese nombre')
                )

            ###

            return self.__agregar(datos)

        # Si recibo el id, modifico la provincia.

        # Abro la sesin.
        session = self.gis2.binded_session()

        # Obtengo la tabla provincias.
        tabla_provincias = self.gis2.tabla('provincias')

        q_obj = session.query(tabla_provincias)
        q_obj = q_obj.filter(tabla_provincias.id == id_provincia)

        # Obtengo el registro.
        registro = q_obj.first()

        # Si no existe, lo agrego.
        if registro is None:

            # Cierro sesin.
            session.close()

            datos['id'] = id_provincia

            #Agrego y retorno.
            return self.__agregar(datos)

        # Actualizo.
        update = q_obj.update(datos)

        # Aplico los cambios
        session.commit()

        # Cierro sesin.
        session.close()

        datos['id'] = id_provincia

        # Sino se pudo modificar, retorno la respuesta.
        if update == 0:
            return respuesta(False, _(u'No se puedo modificar la provincia.'))

        # Retorno.
        return respuesta(
            True, _(u'Se actualiz correctamente la provincia.'), datos
        )

    def eliminar(self, id_provincia):
        '''
        Esta funcin elimina una provincia.
        :param id_provincia:
        :return:
        '''
        # Valido el id_provincia.
        v_id_provincia = numero_entero_positivo(id_provincia)
        if v_id_provincia['Estado'] is not True:
            return v_id_provincia

        # Abro la sesin.
        session = self.gis2.binded_session()

        # Obtengo la tabla provincias.
        tabla_provincias = self.gis2.tabla('provincias')

        # Determino los campos que quiero obtener
        q_obj = session.query(
            tabla_provincias.id
        )

        # Filtro por id.
        q_obj = q_obj.filter(tabla_provincias.id == id_provincia)

        # Verifico si ya existe un usuario.
        if int(q_obj.count()) == 0:
            # Cierro sesin.
            session.close()
            return respuesta(False, _(u'No existe la provincia.'))

        # Cierro sesin.
        session.close()

        # Si supera todos los filtros borro la ciudad.
        datos = {}
        datos['id'] = ('==', id_provincia)
        borrar = self.gis2.borrar('provincias', **datos)
        # En el caso que falle la funcin retorno el error.
        if borrar is not True:
            return respuesta(False, _(u'Error al eliminar la provincia.'))

        # Retorno.
        return respuesta(True, _(u'Se elimin correctamente la provincia.'))

