# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import or_
from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import func
from sqlalchemy import text

from at.map import _
from at.map.utiles.validaciones_especificas import ValidacionesEspecificas
from at.map.utiles.topologia import Topologia
from at.orm.decorators import DBSwitch
from at.utiles.conversiones import respuesta
from at.utiles.validar import numero_entero_positivo
from at.utiles.validar import es_lista


@DBSwitch('gis2')
class Routing(ValidacionesEspecificas):
    '''
       Esta clase contiene las funciones para el ruteo.
    '''

    def __init__(self):
        self.proyeccion = 4326
        self.topologia = Topologia()

    def dijkstra(self, source, target, directed='false', has_rcost='false'):

        # Valido que sea un número entero positivo.
        v_source = numero_entero_positivo(source)
        if v_source['Estado'] is not True:
            return v_source

        # Valido que sea un número entero positivo.
        v_target = numero_entero_positivo(target)
        if v_target['Estado'] is not True:
            return v_target

        # Defino sentencias SQL. Agrego las claves primarias.
        sql = "SELECT seq, id1 AS node, id2 AS edge, cost "
        sql += "FROM pgr_dijkstra("
        sql += "'SELECT gid::integer AS id, source::integer AS source, target::integer AS target, cost_len:: double precision AS cost FROM calles'"
        sql += ", " + str(source) + ", " + str(target) + ", " + directed + ", " + has_rcost + ");"
        sql = text(sql)

        # Abro sesión.
        session = self.gis2.binded_session()

        # Trato de ejecutar la query.
        try:
            # Ejecuto las sentencias.
            resultado = self.gis2.engine.execute(sql)

        # Sinó retorno error.
        except Exception, error:

            # Cierro la sesión.
            session.close()

            return respuesta(False, _(u'Error al obtener la ruta.'), error)

        # Cierro la sesión.
        session.close()

        # Defino ruta.
        ruta = []

        # Recorro los datos.
        for registro in resultado:

            # Acomodo los datos.
            reg = {
                'secuencia': registro[0],
                'id_nodo': registro[1],
                'gid': registro[2],
                'metros': registro[3]
            }

            # Agrego el registro a la lista,
            ruta.append(reg)

        # Verifico si pude obtener la ruta.
        if len(ruta) <= 0:
            return respuesta(False, _(u'No se pudo obtener la ruta.'))

        return respuesta(True, _(u'Se encontró la ruta más corta.'), ruta)

    def obtener_nodo(self, id_nodo):

        # Valido el id_nodo.
        v_id_nodo = numero_entero_positivo(id_nodo)
        if v_id_nodo['Estado'] is not True:
            return v_id_nodo

        # Abro la sesión con la BD con gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles_vertices_pgr
        calles_vertices_pgr = self.gis2.tabla('calles_vertices_pgr')

        # Busco el punto inicial de cada segmento.
        q_obj = session.query(
            calles_vertices_pgr.id,
            func.st_x(calles_vertices_pgr.the_geom).label('x'),
            func.st_y(calles_vertices_pgr.the_geom).label('y')
        )

        # Filtro por el id.
        q_obj = q_obj.filter(calles_vertices_pgr.id == id_nodo)

        # Trato de obtener el registro.
        registro = q_obj.first()

        # Cierro sesión.
        session.close()

        # Verifico si pude encontrar el nodo.
        if registro is None:
            return respuesta(False, _(u'No se pudo encontrar el nodo.'))

        # Acomodo los datos.
        nodo = {
            'id': registro.id,
            'x': registro.x,
            'y': registro.y
        }

        return respuesta(True, _(u'Se obtuvo correctamente el nodo.'), nodo)

    def obtener_nodo_cercano(self, latitud, longitud):

        str_punto = 'POINT(%s %s)' % (str(longitud), str(latitud))

        # Abro la sesión con la BD con gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles_vertices_pgr
        calles_vertices_pgr = self.gis2.tabla('calles_vertices_pgr')

        # # Busco el id del nodo.
        # q_obj_1 = session.query(
        #     calles_vertices_pgr.id,
        #     func.st_distance(
        #         calles_vertices_pgr.the_geom,
        #         func.st_geomfromtext(
        #             str_punto, 4326
        #         )
        #     ).label('distancia')
        # )
        #
        # # Filtro.
        # q_obj_1 = q_obj_1.filter(
        #     and_(
        #         func.st_dwithin(
        #             calles_vertices_pgr.the_geom,
        #             func.st_geomfromtext(
        #                 str_punto,
        #                 4326
        #             ),
        #             0.001,
        #             use_spheroid=True
        #         )
        #     )
        # )
        #
        # # Ordenamos por distancia
        # q_obj_1 = q_obj_1.order_by('distancia')
        #
        # # Obtengo el nodo más cercano.
        # registro = q_obj_1.first()
        #
        #
        # #
        # #
        #





        # Busco el id del nodo.
        q_obj_1 = session.query(calles_vertices_pgr.id)

        # Filtro.
        q_obj_1 = q_obj_1.filter(
            and_(
                func.st_dwithin(
                    calles_vertices_pgr.the_geom,
                    func.st_geomfromtext(
                        str_punto,
                        4326
                    ),
                    0.001,
                    use_spheroid=True
                )
            )
        )

        q_obj_1 = q_obj_1.limit(1)

        # Obtengo el registro.
        registro = q_obj_1.first()












        # Cierro sesión.
        session.close()

        if registro is None:
            return respuesta(False, _(u'No hay calles cercanas.'))

        # Ordeno los datos.
        nodo = {
            'id': int(registro.id)
        }

        # Retorno.
        return respuesta(True, _(u'Se obtuvo correctamente el nodo.'), nodo)

    def obtener_recorrido(self, origen, destino, direccional=True):
        '''

        :param origen:
        :param destino:
        :param direccional:
        :return:
        '''

        # Valido el origen.
        v_origen = es_lista(origen, min_elementos=2, max_elementos=2)

        # Verifico.
        if v_origen['Estado'] is not True:
            return v_origen

        # Valido el destino.
        v_destino = es_lista(destino, min_elementos=2, max_elementos=2)

        # Verifico.
        if v_destino['Estado'] is not True:
            return v_destino

        rta_obtener_nodo_origen = self.obtener_nodo_cercano(
            origen[0], origen[1]
        )

        if rta_obtener_nodo_origen['Estado'] is not True:
            return rta_obtener_nodo_origen

        # Obtengo el source.
        id_source = rta_obtener_nodo_origen['Dato']['id']

        rta_obtener_nodo_destino = self.obtener_nodo_cercano(
            destino[0], destino[1]
        )

        if rta_obtener_nodo_destino['Estado'] is not True:
            return rta_obtener_nodo_destino

        # Obtengo el source.
        id_target = rta_obtener_nodo_destino['Dato']['id']

        if direccional is True:
            direccional = 'true'
        else:
            direccional = 'false'

        rta_ruta = self.dijkstra(id_source, id_target, directed=direccional)

        if rta_ruta['Estado'] is not True:
            return rta_ruta

        # Obtengo la ruta mas corta.
        ruta = rta_ruta['Dato']

        # Inicializo la variable.
        metros_total = 0

        # Defino la lista donde guardaré el recorrido.
        lista_recorrido = []

        # Recorro todos los puntos de la ruta.
        for reg in ruta:

            # Obtengo el id del nodo.
            id_nodo = reg['id_nodo']

            # Obtengo los metros.
            metros = round(reg['metros'], 3)

            # Sumatoria.
            metros_total += metros

            # Trato de obtener el nodo.
            rta_obtener_nodo = self.obtener_nodo(id_nodo)

            # Verifico.
            if rta_obtener_nodo['Estado'] is not True:
                return rta_obtener_nodo

            # Obtengo el nodo.
            nodo = rta_obtener_nodo['Dato']

            # Obtengo latitud y longitud.
            reg['x'] = nodo['x']
            reg['y'] = nodo['y']

            lista_nodo = [reg['x'], reg['y']]

            lista_recorrido.append(lista_nodo)

        # Ordeno los datos.
        datos = {
            'ruta': lista_recorrido,
            'metros_totales': round(metros_total, 3)
        }

        return respuesta(
            True, _(u'Se obtuvo el recorrido correctamente.'), datos
        )

    def recorrido_varios_destinos(self, destinos, direccional=True):

        '''

        :param destinos:
        :param direccional:
        :return:
        '''

        # Valido los destinos.
        v_destinos = es_lista(destinos, min_elementos=2)

        # Verifico.
        if v_destinos['Estado'] is not True:
            return v_destinos

        index_ultimo = len(destinos) - 1

        lista_destino = []

        for index, item in enumerate(destinos):

            if index_ultimo == index:
                continue

            reg_dest = [item, destinos[index + 1]]

            lista_destino.append(reg_dest)

        recorrido_total = []
        metros_totales = 0

        index_ultimo_destino = len(lista_destino) - 1

        for index, destino in enumerate(lista_destino):

            source = destino[0]
            target = destino[1]

            rta = self.obtener_recorrido(source, target, direccional=direccional)

            if rta['Estado'] is not True:
                return rta

            metros = rta['Dato']['metros_totales']
            ruta = rta['Dato']['ruta']

            if index != index_ultimo_destino:
                del ruta[-1]

            recorrido_total = recorrido_total + ruta
            metros_totales = metros_totales + metros

        # Armo el dic.
        dic = {
            'ruta': recorrido_total,
            'metros_totales': metros_totales
        }

        return respuesta(True, _(u'Se obtuvo el recorrido correctamente.'), dic)

