# -*- coding: utf-8 -*-

import unittest

from at.map import _
from at.map.ciudad import Ciudad
from at.map.ciudad import CiudadPostgres


class TestCiudad(unittest.TestCase):

    def setUp(self):
        self.ciudad = Ciudad()
        self.ciudad_postgres = CiudadPostgres()

    def tearDown(self):
        pass

    def test_obtener_ciudad(self):
        # caso correcto, le paso un ID existente
        id_ciudad = 1
        r = self.ciudad.obtener_ciudad(id_ciudad)
        self.assertTrue(r)

        # intento pasarle un ID None
        id_ciudad = None
        self.assertRaises(Exception, self.ciudad.obtener_ciudad, id_ciudad)

        # intento pasarle un ID que no existe
        id_ciudad = 999
        self.assertRaises(Exception, self.ciudad.obtener_ciudad, id_ciudad)

    def test_listar_ciudades(self):
        # caso correcto
        r = self.ciudad.listar_ciudades()
        self.assertTrue(len(r) > 0)

    def test_sugerir_ciudad(self):
        # le paso parte del nombre de una ciudad existente
        parte_ciudad = 'ros'
        r = self.ciudad.sugerir_ciudad(parte_ciudad)
        self.assertTrue(r)

        # caso correcto 2, le paso una palabra en may�sculas
        parte_ciudad = 'ROS'
        r = self.ciudad.sugerir_ciudad(parte_ciudad)
        self.assertTrue(r)

        # esto no debe devolver ning�n registro, devulve None
        parte_ciudad = 'sadiwduwdkk'
        r = self.ciudad.sugerir_ciudad(parte_ciudad)
        self.assertTrue(None == r)

        # le paso un string vacio como parte_calle
        # devulve None, porque ser�a el mismo caso que un string
        # de menos de 3 caracteres
        parte_ciudad = ''
        r = self.ciudad.sugerir_ciudad(parte_ciudad)
        self.assertTrue(None == r)

    def test_agregar_ciudad_aplicacion_postgres(self):
        #Incorrecto
        r = self.ciudad_postgres.agregar_ciudad_aplicacion('asd', True)
        self.assertFalse(r['Estado'])
        # Correcto
        id_app = 134278957
        id_ciudad = 200
        # Intentamos borrarla, por si esta
        self.ciudad_postgres.eliminar_ciudad_aplicacion(id_app)
        # Agregamos
        r = self.ciudad_postgres.agregar_ciudad_aplicacion(id_app, id_ciudad)
        self.assertTrue(r['Estado'])
        self.assertTrue(r['Respuesta'] != '')

    def test_eliminar_ciudad_aplicacion_postgres(self):
        # Caso incorrecto
        id_app = '100'
        r = self.ciudad_postgres.eliminar_ciudad_aplicacion(id_app)
        self.assertFalse(r['Estado'])
        # Caso correcto
        id_app = 134278957
        r = self.ciudad_postgres.eliminar_ciudad_aplicacion(id_app)
        self.assertTrue(r['Estado'])
        self.assertTrue(r['Respuesta'] != '')

    def test_obtener_ciudad_aplicacion_postgres(self):
        # Incorrecto
        id_ciudad = '1'
        r = self.ciudad_postgres.obtener_ciudad_aplicacion(id_ciudad)
        self.assertFalse(r['Estado'])
        # caso correcto
        id_ciudad = 1
        r = self.ciudad_postgres.obtener_ciudad_aplicacion(id_ciudad)
        self.assertTrue(r['Estado'])
        self.assertTrue(r['Respuesta'] != '')

    def test_obtener_id_ciudad_real_postgres(self):
        # Incorrecto
        id_app = 345.345
        id_ciudad = '200'
        r = self.ciudad_postgres.agregar_ciudad_aplicacion(id_app, id_ciudad)
        self.assertFalse(r['Estado'])

        # Correcto
        id_app = 134278957
        id_ciudad = 200
        # Intentamos borrarla, por si esta
        self.ciudad_postgres.eliminar_ciudad_aplicacion(id_app)
        # La agregamos
        r = self.ciudad_postgres.agregar_ciudad_aplicacion(id_app, id_ciudad)
        self.assertTrue(r['Estado'])

        r = self.ciudad_postgres.obtener_id_ciudad_real(id_app)
        self.assertTrue(r['Estado'])
        self.assertTrue(r['Respuesta'] == id_ciudad)

        r = self.ciudad_postgres.eliminar_ciudad_aplicacion(id_app)
        self.assertTrue(r['Respuesta'] != '')

    def test_obtener_id_ciudad_aplicacion_postgres(self):
        # Incorrecto
        id_app = 234.23
        id_ciudad = '200asd'
        r = self.ciudad_postgres.agregar_ciudad_aplicacion(id_app, id_ciudad)
        self.assertFalse(r['Estado'])
        # Correcto
        id_app = 134278957
        id_ciudad = 200
        # Intentamos borrarla, por si esta
        self.ciudad_postgres.eliminar_ciudad_aplicacion(id_app)
        # Agregamos
        r = self.ciudad_postgres.agregar_ciudad_aplicacion(id_app, id_ciudad)
        self.assertTrue(r['Estado'])
        # Obtenemos
        r = self.ciudad_postgres.obtener_id_ciudad_aplicacion(id_ciudad)
        self.assertTrue(r['Estado'])
        self.assertTrue(r['Respuesta'] == id_app)
        # Borramos
        r = self.ciudad_postgres.eliminar_ciudad_aplicacion(id_ciudad)
        self.assertTrue(r['Estado'])
        self.assertTrue(r['Respuesta'] != 0)

    def test_obtener_ciudad_postgres(self):
        id_ciudad = '1asd'
        r = self.ciudad_postgres.obtener_ciudad(id_ciudad)
        self.assertFalse(r['Estado'])
        # caso correcto
        id_ciudad = 1
        r = self.ciudad_postgres.obtener_ciudad(id_ciudad)
        self.assertTrue(r['Estado'])

    def test_listar_ciudades_postgres(self):
        # caso correcto
        id_provincia = 'asd'
        r = self.ciudad_postgres.listar_ciudades(id_provincia)
        self.assertFalse(r['Estado'])
        id_provincia = 9
        r = self.ciudad_postgres.listar_ciudades(id_provincia)
        self.assertTrue(r['Estado'])
        self.assertTrue(len(r['Respuesta']) > 0)

    def test_listar_ciudades_aplicacion_postgres(self):
        # caso correcto
        ordenar_por_nombre = True
        r = self.ciudad_postgres.listar_ciudades_aplicacion(ordenar_por_nombre)
        self.assertTrue(r['Estado'])
        self.assertTrue(len(r['Respuesta']) > 0)

    def test_obtener_id_ciudad_por_latlon_postgres(self):
        r = self.ciudad_postgres.obtener_id_ciudad_por_latlon(
            -2.93356, -60.718326
        )
        self.assertFalse(r['Estado'])
        r = self.ciudad_postgres.obtener_id_ciudad_por_latlon(
            -32.93356, -60.718326
        )
        self.assertTrue(r['Estado'])
        self.assertTrue(r['Respuesta'] == 324)


    # COMENTO ESTE TEST PORQUE SE REPITE.......REVISAR!!!!!!!!!
    # def test_sugerir_ciudad(self):
    #     # Caso correcto
    #     parte_nombre = 'ros'
    #     rta = self.ciudad_postgres.sugerir_ciudad(parte_nombre)
    #     print rta
    #     self.assertTrue(rta['Estado'] is True)
    #     self.assertTrue(isinstance(rta['Respuesta'], list))
    #
    #     # Le paso una ciudad que no existe
    #     parte_nombre = 'zzzzzzzz'
    #     rta = self.ciudad_postgres.sugerir_ciudad(parte_nombre)
    #     self.assertTrue(rta['Estado'] is False)
    #     self.assertTrue(rta['Respuesta'] == _(u'No se encontró ninguna ciudad'))
    #
    #     # Le paso solo dos caracteres
    #     parte_nombre = 'ro'
    #     rta = self.ciudad_postgres.sugerir_ciudad(parte_nombre)
    #     self.assertTrue(rta['Estado'] is False)
    #     self.assertTrue(
    #         rta['Respuesta'] ==
    #         _(u'Para buscar la ciudad debe ingresar un mínimo de 3 letras')
    #     )

    def test_grabar_ciudad_postgres(self):

        id_provincia = None
        nombre = None
        id_ciudad = None

        rta = self.ciudad_postgres.grabar(
            id_provincia, nombre, id_ciudad=id_ciudad
        )

        self.assertTrue(rta['Estado'] is False)

        ###

        id_provincia = 20
        nombre = u'Rosario'
        id_ciudad = None

        rta = self.ciudad_postgres.grabar(
            id_provincia, nombre, id_ciudad=id_ciudad
        )

        self.assertTrue(rta['Estado'] is False)


        ##
        #
        # id_provincia = 20
        # nombre = u'García'
        # id_ciudad = None
        #
        # rta = self.ciudad_postgres.grabar(
        #     id_provincia, nombre, id_ciudad=id_ciudad
        # )
        #
        # print rta
        #
        # self.assertTrue(rta['Estado'] is True)

        # id_provincia = 20
        # nombre = u'García modificada'
        # id_ciudad = 5017
        #
        # rta = self.ciudad_postgres.grabar(
        #     id_provincia, nombre, id_ciudad=id_ciudad
        # )
        #
        # print rta

        # self.assertTrue(rta['Estado'] is True)

    def test_eliminar_ciudad_postgres(self):

        id_ciudad = None

        rta = self.ciudad_postgres.eliminar(id_ciudad)

        self.assertTrue(rta['Estado'] is False)

        ###

        id_ciudad = 999999999

        rta = self.ciudad_postgres.eliminar(id_ciudad)

        self.assertTrue(rta['Estado'] is False)

        ##

        id_provincia = 20
        nombre = u'García'
        id_ciudad = 999999999

        rta = self.ciudad_postgres.grabar(
            id_provincia, nombre, id_ciudad=id_ciudad
        )

        self.assertTrue(rta['Estado'] is True)

        id_ciudad = 999999999

        rta = self.ciudad_postgres.eliminar(id_ciudad)

        self.assertTrue(rta['Estado'] is True)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCiudad))
    return suite
