# -*- coding: iso-8859-1 -*-
import unittest
from at.map.mapas import Mapas


class TestMapas(unittest.TestCase):

    def setUp(self):
        self.m = Mapas()

    def tearDown(self):
        pass

    def test_obtener_xy_por_latlon(self):
        latitud = -32.2563
        longitud = -60.2575
        z = 14

        # Fallamos con datos incorrectos
        self.assertRaises(
            Exception, self.m.obtener_xy_por_latlon, None, longitud, z
        )
        self.assertRaises(
            Exception, self.m.obtener_xy_por_latlon, latitud, 'longitudasd', z
        )
        self.assertRaises(
            Exception, self.m.obtener_xy_por_latlon, latitud, longitud, True
        )
        # Caso correcto
        x, y = self.m.obtener_xy_por_latlon(latitud, longitud, z)
        self.assertTrue(x == 2790202)
        self.assertTrue(y == 1345525)

    def test_obtener_tile_por_latlon(self):
        latitud = -32.2563
        longitud = -60.2575
        z = 14

        # Fallamos con datos incorrectos
        self.assertRaises(
            Exception, self.m.obtener_tile_por_latlon, None, longitud, z
        )
        self.assertRaises(
            Exception, self.m.obtener_tile_por_latlon, latitud, 'longitudasd', z
        )
        self.assertRaises(
            Exception, self.m.obtener_tile_por_latlon, latitud, longitud, True
        )
        # Caso correcto
        x, y = self.m.obtener_tile_por_latlon(latitud, longitud, z)
        self.assertTrue(x == 10899)
        self.assertTrue(y == 5255)

    def test_obtener_imagen_tile(self):

        # Fallamos al momento de pasar parametros incorrectos
        self.assertRaises(
            Exception,
            self.m.obtener_imagen_tile,
            'a',
            None,
            False
        )

        # Obtenemos none con una imagen inexistente
        self.assertTrue(self.m.obtener_imagen_tile(99, 99, 99) == None)


    def test_obtener_tile_padre_xy(self):
        # Fallamos al momento de pasar parametros incorrectos
        self.assertRaises(Exception, self.m.obtener_tile_padre_xy, 'a', None)

        # Obtenemos x e y:
        x, y = self.m.obtener_tile_padre_xy(6, 7)
        self.assertTrue(x == 0 and y == 0)

        x, y = self.m.obtener_tile_padre_xy(20, 45)
        self.assertTrue(x == 16 and y == 40)


    def test_obtener_imagen_cuadrante(self):
        ancho = 320
        alto = 480
        lat1 = -32.934659
        lon1 = -60.716311
        lat2 = -32.931507
        lon2 = -60.709616

        # Fallamos con el alto y ancho erroneos:
        rta = self.m.obtener_imagen_cuadrante(
            'altoasd', ancho, lat1, lon1, lat2, lon2
        )
        self.assertFalse(rta['Estado'])
        rta = self.m.obtener_imagen_cuadrante(
            alto, True, lat1, lon1, lat2, lon2
        )
        self.assertFalse(rta['Estado'])

        # Fallamos con las posiciones
        rta = self.m.obtener_imagen_cuadrante(
            alto, ancho, None, lon1, lat2, lon2
        )
        self.assertFalse(rta['Estado'])
        rta = self.m.obtener_imagen_cuadrante(
            alto, ancho, None, 'lon1', lat2, lon2
        )
        self.assertFalse(rta['Estado'])
        rta = self.m.obtener_imagen_cuadrante(
            alto, ancho, lat1, lon1, True, lon2
        )
        self.assertFalse(rta['Estado'])
        rta = self.m.obtener_imagen_cuadrante(
            alto, ancho, lat1, lon1, lat2, None
        )
        self.assertFalse(rta['Estado'])

        # Ejecutamos correctamente
        rta = self.m.obtener_imagen_cuadrante(
            alto, ancho, lat1, lon1, lat2, lon2
        )

        self.assertTrue(rta['Estado'])


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMapas))
    return suite
