# -*- coding: utf-8 -*-

import unittest

from at.map import _
from at.map.pais import PaisPostgres


class TestPais(unittest.TestCase):

    def setUp(self):
        self.pais_postgres = PaisPostgres()

    def tearDown(self):
        pass

    def test_obtener_pais_postgres(self):
        r = self.pais_postgres.obtener_pais('asd')
        self.assertFalse(r['Estado'])
        # caso correcto
        id_pais = 1
        r = self.pais_postgres.obtener_pais(id_pais)
        self.assertTrue(r['Estado'])

    def test_listar_paises_postgres(self):
        # caso correcto
        r = self.pais_postgres.listar_paises()
        self.assertTrue(r['Estado'])
        self.assertTrue(len(r) > 0)

    def test_obtener_pais_de_la_ciudad(self):
        # Caso correcto, le paso la ciudad de Rosario
        id_ciudad = 324
        rta = self.pais_postgres.obtener_pais_de_la_ciudad(id_ciudad)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_pais'] == 12)
        self.assertTrue(rta['Respuesta']['nombre_pais'] == 'Argentina')
        self.assertTrue(rta['Respuesta']['capital_pais'] == 'Buenos Aires')
        self.assertTrue(rta['Respuesta']['geom_pais'] is not None)
        self.assertTrue(isinstance(rta['Respuesta']['zona_horaria_pais'], int))

        # Le paso una ciudad que no existe
        id_ciudad = 9999
        rta = self.pais_postgres.obtener_pais_de_la_ciudad(id_ciudad)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
                            _(u'No se pudo obtener el país de la ciudad dada'))

        # Caso correcto, le paso la ciudad Santo Domingo
        id_ciudad = 5008
        rta = self.pais_postgres.obtener_pais_de_la_ciudad(id_ciudad)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_pais'] == 62)
        self.assertTrue(rta['Respuesta']['nombre_pais'] == 'Dominican Republic')
        self.assertTrue(rta['Respuesta']['capital_pais'] == 'Santo Domingo')
        self.assertTrue(rta['Respuesta']['geom_pais'] is not None)
        self.assertTrue(rta['Respuesta']['zona_horaria_pais'] is not None)


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestPais))
    return suite
