# -*- coding: iso-8859-1 -*-

import unittest

from at.map.utiles.referencia import Referencia
from at.map.utiles.punto import Punto


class TestReferencia(unittest.TestCase):
    
    def setUp(self):
        self.referencia = Referencia()
        
    def tearDown(self):
        pass
    
    def test_obtener_referencias(self):
        # caso correcto
        id_mapa = 1
        r = self.referencia.obtener_referencias(id_mapa)
        self.assertTrue(r)
        
        # intento pasarle None
        id_mapa = None
        self.assertRaises(Exception, self.referencia.obtener_referencias, id_mapa)
        
        # intento pasarle un ID que no existe
        id_mapa = 123
        self.assertRaises(Exception, self.referencia.obtener_referencias, id_mapa)

    
    def test_obtener_referencias_cercanas_xy(self):
        # caso correcto
        punto = Punto(id_mapa = 5, x = 800, y = 800)
        r = self.referencia.obtener_referencias_cercanas_xy(punto)
        self.assertTrue(r)

        # intento pasarle None
        punto = Punto(id_mapa = None, x = None, y = None)
        self.assertRaises(Exception, self.referencia.obtener_referencias_cercanas_xy, punto)
        
        # intento pasarle un ID que no existe
        punto = Punto(id_mapa = 123, x = 800, y = 800)
        self.assertRaises(Exception, self.referencia.obtener_referencias_cercanas_xy, punto)
    
    def test_obtener_referencias_cercanas_latlon(self):
        # caso correcto
        punto = Punto(id_mapa = 5, latitud = 800, longitud = 800)
        r = self.referencia.obtener_referencias_cercanas_latlon(punto)
        self.assertTrue(r)
        
        # intento pasarle None
        punto = Punto(id_mapa = None, latitud = None, longitud = None)
        self.assertRaises(Exception, self.referencia.obtener_referencias_cercanas_latlon, punto)
        
        # intento pasarle un ID que no existe
        punto = Punto(id_mapa = 123, latitud = 800, longitud = 800)
        self.assertRaises(Exception, self.referencia.obtener_referencias_cercanas_latlon, punto)
 
    def test_obtener_referencias_cercanas(self):
       
        # caso correcto con X e Y
        punto = Punto(id_mapa = 5, x = 800, y = 800)
        r = self.referencia.obtener_referencias_cercanas(punto)
        self.assertTrue(r)
        
        # caso correcto con Latitud y Longitud
        punto = Punto(id_mapa = 5, latitud = 800, longitud = 800)
        r = self.referencia.obtener_referencias_cercanas(punto)
        self.assertTrue(r)
        
        # intento pasarle None
        punto = Punto(id_mapa = None, latitud = None, longitud = None, y = None)
        self.assertRaises(Exception, self.referencia.obtener_referencias_cercanas, punto)
        
    def test_de_xy_a_segundos(self):
        # caso correcto
        punto = Punto(id_mapa = 1, x = 800, y = 800)
        r = self.referencia.de_xy_a_segundos(punto)
        self.assertTrue(r)
         
        # intento pasarle None
        punto = Punto(id_mapa = None, x = None, y = None)
        self.assertRaises(Exception, self.referencia.de_xy_a_segundos, punto)
        
        # intento pasarle un ID que no existe
        punto = Punto(id_mapa = 123, x = 800, y = 800)
        self.assertRaises(Exception, self.referencia.de_xy_a_segundos, punto)
    
    def test_de_segundos_a_xy(self):
        # caso correcto
        punto = Punto(id_mapa = 1, latitud =  -118397.030172, longitud = -218754.672494)
        r = self.referencia.de_segundos_a_xy(punto)
        self.assertTrue(r)
  
        # intento pasarle None
        punto = Punto(id_mapa = None, latitud = None, longitud = None)
        self.assertRaises(Exception, self.referencia.de_segundos_a_xy, punto)
        
        # intento pasarle un ID que no existe
        punto = Punto(id_mapa = 123, latitud = 800, longitud = 800)
        self.assertRaises(Exception, self.referencia.de_segundos_a_xy, punto)
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestReferencia))
    return suite