# -*- coding: utf-8 -*-

import unittest

from at.map.sitio import Sitio
from at.map import _


class TestSitio(unittest.TestCase):

    def setUp(self):
        self.sitio = Sitio()

    def tearDown(self):
        pass

    def test_obtener_sitio(self):
        # Caso correcto
        id_ciudad = 324
        id_sitio = 10
        r = self.sitio.obtener_sitio(id_ciudad, id_sitio)
        self.assertTrue(r['id_sitio'] == 10)
        self.assertTrue(int(r['id_sitio_tipo']) == 1)
        self.assertTrue(r['nombre_sitio'] == 'Aeropuerto Rosario')
        self.assertTrue(r['longitud_sitio'] == -218464.30206288)
        self.assertTrue(r['latitud_sitio'] == -118626.38854992)
        self.assertTrue(r['id_ciudad_sitio'] == 324)
        self.assertTrue(r['mostrar_sitio_en_mapa'] == 1)

        # Caso correcto, ya no importa la ciudad ya que id_sitio es único
        id_ciudad = 1234
        id_sitio = 10
        r = self.sitio.obtener_sitio(id_ciudad, id_sitio)
        self.assertTrue(r['id_sitio'] == 10)
        self.assertTrue(r['id_sitio_tipo'] == 1)
        self.assertTrue(r['nombre_sitio'] == 'Aeropuerto Rosario')
        self.assertTrue(r['longitud_sitio'] == -218464.30206288)
        self.assertTrue(r['latitud_sitio'] == -118626.38854992)
        self.assertTrue(r['id_ciudad_sitio'] == 324)
        self.assertTrue(r['mostrar_sitio_en_mapa'] == 1)

        # Le paso un sitio que no existe
        id_ciudad = 324
        id_sitio = 1234
        self.assertRaises(
            Exception, self.sitio.obtener_sitio, id_ciudad, id_sitio
        )

    def test_sugerir_sitio(self):
        # Caso correcto, le paso 3 letras de la palabra
        id_ciudad = 324
        parte_sitio = 'Aer'
        r = self.sitio.sugerir_sitio(id_ciudad, parte_sitio)
        self.assertTrue(r[0]['id_sitio'] == 10)
        self.assertTrue(r[0]['nombre_sitio'] == 'Aeropuerto Rosario')

        # Caso correcto, le paso dos palabras
        id_ciudad = 324
        parte_sitio = 'Aeropuerto'
        r = self.sitio.sugerir_sitio(id_ciudad, parte_sitio)
        self.assertTrue(r[0]['id_sitio'] == 10)
        self.assertTrue(r[0]['nombre_sitio'] == 'Aeropuerto Rosario')

        # Caso correcto, parte del nombre con mayúscula
        id_ciudad = 324
        parte_sitio = 'AERO'
        r = self.sitio.sugerir_sitio(id_ciudad, parte_sitio)
        self.assertTrue(r[0]['id_sitio'] == 10)
        self.assertTrue(r[0]['nombre_sitio'] == 'Aeropuerto Rosario')

        # Le paso una parte del sitio que no existe
        id_ciudad = 324
        parte_sitio = 'asd5478'
        r = self.sitio.sugerir_sitio(id_ciudad, parte_sitio)
        self.assertTrue(len(r) == 0)

        # Le paso una ciudad que no existe
        id_ciudad = 1
        parte_sitio = 'Aer'
        r = self.sitio.sugerir_sitio(id_ciudad, parte_sitio)
        self.assertTrue(len(r) == 0)

        # Le paso un string vacio como parte_sitio
        id_ciudad = 324
        parte_sitio = ''
        r = self.sitio.sugerir_sitio(id_ciudad, parte_sitio)
        self.assertTrue(r is None)

        # Le paso solo dos letras para buscar el sitio
        id_ciudad = 324
        parte_sitio = 'Ae'
        r = self.sitio.sugerir_sitio(id_ciudad, parte_sitio)
        self.assertTrue(r is None)


    def test_listar_tipos_sitios(self):
        rta = self.sitio.listar_tipos_sitios()

    def test_listar_sitios(self):
        rta = self.sitio.listar_sitios()

    def test_obtener_ubicacion_sitio(self):
        id_sitio = 4
        rta = self.sitio.obtener(id_sitio)

    def test_obtener_tipo_sitio(self):
        # Fallamos cuando le pasamos un string
        rta = self.sitio.obtener_tipo('asd')
        self.assertFalse(rta['Estado'])

        # Fallamos al pasarle un id inexistente
        rta = self.sitio.obtener_tipo(568)
        self.assertFalse(rta['Estado'])

        # Obtenemos correctamente un tipo
        rta = self.sitio.obtener_tipo(2)
        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta'] == 'Hospital')

    def test_grabar(self):
        # Fallamos cuando le pasamos datos incorrectos
        rta = self.sitio.grabar('asd', 234, True, '234', 324)
        self.assertFalse(rta['Estado'])
        #print rta

        ## Agregamos correctamente una zona sin marcaodr
        #rta = self.sitio.grabar(1, 'Prueba', [[34.5, 60.3]])
        #self.assertTrue(rta['Estado'])
        ## Obtenemos el id agregado
        #rta = self.sitio.listar()
        #self.assertTrue(rta['Estado'])
        #self.assertTrue(len(rta['Respuesta']) == 1)
        #self.assertTrue(rta['Respuesta'][0]['Descripcion'] == 'Prueba')

        # Modificamoss
        #id_sitio = 1
        #puntos = [20.3, 30.5]
        #id_ciudad = 324
        #rta = self.sitio.grabar(2, 'Prueba2', puntos=puntos,
        #                                        id_ciudad=id_ciudad, id_sitio=id_sitio)

        #print rta

        #self.assertTrue(rta['Estado'])

        #rta = self.sitio.listar_sitios()
        #self.assertTrue(rta['Estado'])
        #self.assertTrue(len(rta['Respuesta']) == 3)
        #self.assertTrue(rta['Respuesta'][0]['IdSitio'] == id_sitio)
        #self.assertTrue(rta['Respuesta'][0]['Nombre'] == 'Prueba2')

    def test_borrar_sitio(self):
        # Le paso un sitio que no existe
        id_sitio = 1234
        rta = self.sitio.borrar(id_sitio)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No existe el sitio a borrar'))


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestSitio))
    return suite
