# -*- coding: iso-8859-1 -*-

import unittest

from at.map.utiles.validaciones_especificas import ValidacionesEspecificas


class TestValidacionesEspecificas(unittest.TestCase):

    def setUp(self):
        self.especificas = ValidacionesEspecificas()
        
    def tearDown(self):
        pass
    
    def test_validar_id_calle(self): 
        # caso normal satisfactorio ...
        id = 1
        r = self.especificas.validar_id_calle(id)
        self.assertTrue(r)
        
        # caso normal 2
        id = 0
        r = self.especificas.validar_id_calle(id)
        self.assertTrue(r)
        
        # si id es un String debe fallar
        id = '1'
        self.assertRaises(ValueError, self.especificas.validar_id_calle, id)
        
        # si id es un float
        id = 1.34
        self.assertRaises(ValueError, self.especificas.validar_id_calle, id) 

        # paso un objeto creado al vuelo como parametro ...
        id = type('nombre', (object,), {})
        self.assertRaises(ValueError, self.especificas.validar_id_calle, id)
        
        # le paso un id vacio
        id = None
        self.assertRaises(Exception, self.especificas.validar_id_calle, id)
        
        # si id es un booleano
        id = True
        self.assertRaises(ValueError, self.especificas.validar_id_calle, id)
        
        # el valor del id no puede ser mayor a 99999999999
        id = 100000000000
        self.assertRaises(Exception, self.especificas.validar_id_calle, id)
        
        # no se aceptan id menores que cero
        id = -1
        self.assertRaises(Exception, self.especificas.validar_id_calle, id)
        
    def test_validar_id_ciudad(self): 
        # caso normal satisfactorio ...
        id = 1
        r = self.especificas.validar_id_ciudad(id)
        self.assertTrue(r)
        
        # caso normal 2
        id = 0
        r = self.especificas.validar_id_ciudad(id)
        self.assertTrue(r)
        
        # si id es un String debe fallar
        id = '1'
        self.assertRaises(ValueError, self.especificas.validar_id_ciudad, id)
        
        # si id es un float
        id = 1.34
        self.assertRaises(ValueError, self.especificas.validar_id_ciudad, id) 

        # paso un objeto creado al vuelo como parametro ...
        id = type('nombre', (object,), {})
        self.assertRaises(ValueError, self.especificas.validar_id_ciudad, id)
        
        # le paso un id vacio
        id = None
        self.assertRaises(Exception, self.especificas.validar_id_ciudad, id)
        
        # si id es un booleano
        id = True
        self.assertRaises(ValueError, self.especificas.validar_id_ciudad, id)
        
        # el valor del id no puede ser mayor a 99999999999
        id = 100000000000
        self.assertRaises(Exception, self.especificas.validar_id_ciudad, id)
        
        # no se aceptan id menores que cero
        id = -1
        self.assertRaises(Exception, self.especificas.validar_id_ciudad, id)
        
def suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestValidacionesEspecificas))
    return suite