# -*- coding: utf-8 -*-

import sys
import os
from PIL import Image
from time import time
import mapscript
from cStringIO import StringIO


from at.orm.decorators import DBSwitch

import at.map
from at.map import RUTA_MAPAS
from at.map import TILES_POR_LADO
from at.map import PIXELES_TILE
from at.map import RUTA_LOGS
from at.map.utiles.tile import Tile
from at.map.utiles.layer import Layer

# Iniciamos el log
import at.map.log
at.map.log.LOG_PATH = RUTA_LOGS
at.map.log.start('%s.debug' % RUTA_LOGS)

from twisted.python import log


@DBSwitch('gis2')
class TileGenerator(object):

    def __init__(self):

        # Inicializamos el objeto
        self.time = time()
        self.layer = Layer()
        self.map_path = os.path.join(at.map.__path__[0], 'render/map.map')
        # Creamos el objeto de mapscript
        self.wms = mapscript.mapObj(self.map_path)

        # Obtenemos los datos de conexion de at.orm
        if self.gis2.engine is None:
            self.gis2.connect()

        # OBtenemos los puertos
        con_data = self.gis2.engine.url.translate_connect_args()
        if con_data.get('port') is None:
            con_data['port'] = 5432

        # generamos el string de conexion
        str_con_data = 'host=%s dbname=%s user=%s password=%s port=%s' % (
            str(con_data['host']),
            str(con_data['database']),
            str(con_data['username']),
            str(con_data['password']),
            str(con_data['port'])
        )

        # Recorremos los layers y modificamos los datos de conexion con los
        # que sacamos del at.orm
        for i in range(self.wms.numlayers):
            self.wms.getLayer(i).connection = str_con_data

        # Esto lo ponemos porque sino PIL no puede modificar la imagen
        self.wms.outputformat.setOption('FORMATOPTIONS', 'INTERLACE=OFF')
        self.format = 'image/png'
        self.extension = self.format[self.format.find('/') + 1:]
        self.pixeles_por_lado = PIXELES_TILE * TILES_POR_LADO

    def obtener_imagen_bbox(self, bbox):

        # Logueamos
        log.msg('Generamos bbox %s' % str(bbox), system='tile')

        req = mapscript.OWSRequest()
        bbox = map(float, bbox)

        req.setParameter('LAYERS', 'calles,rutas,ferrocarriles,ciudades')
        req.setParameter('FORMAT', self.format)
        req.setParameter('SRS', 'epsg:4326')
        req.setParameter('WIDTH', str(self.pixeles_por_lado))
        req.setParameter('HEIGHT', str(self.pixeles_por_lado))
        req.setParameter('BBOX', ",".join(map(str, bbox)))
        req.setParameter("STYLES", "")
        req.setParameter("REQUEST", "GetMap")

        # Cargamos los parametros
        self.wms.loadOWSParameters(req)
        try:
            string_bytes = self.wms.draw().getBytes()
        except Exception:
            log.err()
            log.err('Error al generar: %s' % str(bbox), system='tile')
            return None

        return string_bytes

    def generate(self, x, y, z, basico=False, sub_directorio=False):

        # Creamos el tile pedido
        tile = Tile(self.layer, x, y, z)

        # Logueamos
        log.msg('Nos pieden generar tile %s' % str(tile), system='tile')

        # Obtenemos su padre
        padre = tile.obtener_padre()

        # Logueamos
        log.msg('Vamos a generar tile %s' % str(padre), system='tile')

        # Obtenemos los limites del padre
        limites_menor = padre.obtener_limites()

        # Vamos a pedir todo un cuadrante asi que a calcular. Ya tenemos el
        # tile correspondiente a la esquina inferior izquierda. Ahora vamos
        # por la superior derecha

        tile_mayor = Tile(
            self.layer,
            padre.x + TILES_POR_LADO - 1,
            padre.y + TILES_POR_LADO - 1,
            z
        )
        # Obtenemos limites mayores
        limites_mayor = tile_mayor.obtener_limites()

        # Generamos un bbox mayor, uniendo los datos del mayor y el menor
        bbox_cuadrante = [
            limites_menor[0],
            limites_menor[1],
            limites_mayor[2],
            limites_mayor[3],
        ]

        # Intentamos obtener la imagen
        try:
            # Obtenemos la imagen segun el bbox
            data = self.obtener_imagen_bbox(bbox_cuadrante)
            # Generamos una imagen que vamos a dividir
            im = Image.open(StringIO(data))
        except Exception:
            log.err('ERROR: al generar tile %s' % padre, system='tile')
            log.err(system='tile')
            return

        # Obtengo la ruta de las imagenes del mapa.
        ruta_img = RUTA_MAPAS

        # Verifico si tiene un subdirectorio.
        if sub_directorio is not False:
            ruta_img = os.path.join(ruta_img, sub_directorio)

        # Dividimos la imagen el la cantidad de tiles por lado
        for j in range(TILES_POR_LADO):
            for i in range(TILES_POR_LADO):
                im_tmp = im.crop((
                    PIXELES_TILE * i,
                    PIXELES_TILE * j,
                    PIXELES_TILE * i + PIXELES_TILE,
                    PIXELES_TILE * j + PIXELES_TILE,
                ))

                string_buffer = StringIO()
                # Guardamos la imagen en formato jpge para que pese menos
                im_tmp.save(string_buffer, 'JPEG', quality=95)
                string_buffer.seek(0)
                data = string_buffer.read()

                if basico:
                    path = os.path.join(ruta_img, str(z))
                else:
                    path = os.path.join(ruta_img, 'z' + str(z))

                # Si no esta el directorio de z, lo creamos
                try:
                    if not os.path.isdir(path):
                        os.makedirs(path)
                except:
                    pass

                if basico:
                    path = os.path.join(path, str(padre.x + i))
                else:
                    path = os.path.join(path, 'x' + str(padre.x + i))

                try:
                    # Si no esta el directorio de x, lo creamos
                    if not os.path.isdir(path):
                        os.makedirs(path)
                except:
                    pass

                # Generamos el path del archivo de salida
                if basico:
                    arvhivo = os.path.join(
                        path,
                        str(padre.y + TILES_POR_LADO - j - 1) + '.jpeg',
                    )
                else:
                    arvhivo = os.path.join(
                        path,
                        'y' + str(padre.y + TILES_POR_LADO - j - 1) + '.jpeg',
                    )
                # Abrimos y escribimos
                a = open(arvhivo, 'w')
                a.write(data)
                a.close()

        # Logueamos
        log.msg('Finalizamos tile %s' % str(padre), system='tile')


def generate_tile(x=None, y=None, z=None, basico=False):

    # Intentamos sacar los datos desde linea de comandos si no nos
    # dan argumentos
    if x is None or y is None or z is None:
        if len(sys.argv) > 2:
            x = sys.argv[1]
            y = sys.argv[2]
            z = sys.argv[3]
            if len(sys.argv) == 5:
                basico = sys.argv[4]
        else:
            return False

    try:
        x = int(x)
    except:
        print 'Los argumentos X,Y,Z deben ser numeros enteros'
        return False
    try:
        y = int(y)
    except:
        print 'Los argumentos X,Y,Z deben ser numeros enteros'
        return False
    try:
        z = int(z)
    except:
        print 'Los argumentos X,Y,Z deben ser numeros enteros'
        return False

    try:
        basico = bool(int(basico))
    except:
        print 'Los argumentos X,Y,Z o Basico deben ser numeros enteros'
        return False

    tg = TileGenerator()
    try:
        tg.generate(x, y, z, basico)
    except Exception, e:
        print 'Error', e
        return False
    # Salimos bien
    return True
