# -*- coding: utf-8 -*-

from at.map import _
from at.utiles.conversiones import respuesta

from math import sqrt
from math import sin
from math import cos
from math import pi
import sys

MAXIMO = sys.float_info.max
MINIMO = sys.float_info.min


class FuncionesGeo(object):
    '''
        Esta clase contiene funciones relacionadas con c?lculos geogr?ficos.
    '''

    def generar_geometria_punto(self, punto):
        ''' Funcion que toma como argumento una matriz de puntos y devueve
        un string para guardar en postgis, con la estructura especificada. '''

        p = 'POINT('
        p += '%s %s,' % (str(punto[1]), str(punto[0]))
        p = p[:-1] + ')'
        return p

    def generar_geometria_una_linea(self, puntos):
        ''' Funcion que toma como argumento una matriz de puntos y devueve
        un string para guardar en postgis, con la estructura especificada. '''

        linea = 'LINESTRING('
        for punto in puntos:
            linea += '%s %s,' % (str(punto[1]), str(punto[0]))
        linea = linea[:-1] + ')'
        return linea

    def generar_geometria_linea(self, puntos):
        ''' Funcion que toma como argumento una matriz de puntos y devueve
        un string para guardar en postgis, con la estructura especificada. '''

        linea = 'MULTILINESTRING(('
        for punto in puntos:
            linea += '%s %s,' % (str(punto[1]), str(punto[0]))
        linea = linea[:-1] + '))'
        return linea

    def generar_geometria_poligono(self, puntos):
        ''' Funcion que toma como argumento una matriz de puntos y devueve
        un string para guardar en postgis, con la estructura especificada. '''

        poligono = 'MULTIPOLYGON((('
        for punto in puntos:
            poligono += '%s %s,' % (str(punto[1]), str(punto[0]))
        poligono = poligono[:-1] + ')))'
        return poligono

    def km_entre_2_puntos(self, Lat1EnSeg, Lon1EnSeg, Lat2EnSeg, Lon2EnSeg):
        '''
            Esta funci?n recibe como par?metros latitudes y longitudes
            de 2 puntos y devuelve la distancia entre ellos en kil?metros.

            Argumentos:
                latitud1 (float)
                longitud1 (float)
                latitud2 (float)
                longitud2 (float)

            Retorna:
                distancia (float)
        '''

        # Convierto de segundos a grados
        lat1 = Lat1EnSeg / 3600
        lon1 = Lon1EnSeg / 3600
        lat2 = Lat2EnSeg / 3600
        lon2 = Lon2EnSeg / 3600

        x = 110.56 * (lat2 - lat1)
        y = 84.8 * (lon2 - lon1)

        distancia = sqrt(x ** 2 + y ** 2)
        distancia = round(distancia, 4)
        return distancia

    def kms_entre_puntos(self, lat1, lon1, lat2, lon2):
        ''' Esta funcion toma como argumento latitudes y longitudes en grados
        y calcula la distancia entre ambos puntos'''
        # Pasamos las latitudes y longitudes a radianes
        lat1 = lat1 * 2 * pi / 360.0
        lon1 = lon1 * 2 * pi / 360.0
        lat2 = lat2 * 2 * pi / 360.0
        lon2 = lon2 * 2 * pi / 360.0

        # Radio de la tierra en km
        R = 6371
        x = (lon2 - lon1) * cos((lat1 + lat2) / 2)
        y = (lat2 - lat1)
        distancia = sqrt(x ** 2 + y ** 2) * R
        distancia = round(distancia, 4)
        return distancia

    def mts_entre_puntos(self, lat1, lon1, lat2, lon2):
        ''' Esta funcion toma como argumento latitudes y longitudes en grados
        y calcula la distancia entre ambos puntos'''
        distancia = self.kms_entre_puntos(lat1, lon1, lat2, lon2) * 1000
        distancia = round(distancia, 3)
        return distancia

    def obtener_orientacion(self, latitud_origen, longitud_origen,
                            latitud_destino, longitud_destino):
        '''
            Esta funci?n recibe la latitud y longitud origen y destino y
            devuelve el rumbo que se debe tomar para ir de un punto a otro.

            Argumentos:
                latitud_origen (float)
                longitud_origen (float)
                latitud_destino (float)
                longitud_destino (float)

            Retorna:
                rumbo (str) // N, NE, E, SE, S, SO, O, NO
        '''

        cateto_opuesto = self.kms_entre_puntos(float(latitud_origen),
                                                    float(longitud_origen),
                                                    float(latitud_destino),
                                                    float(longitud_origen))
        #cateto_opuesto = cateto_opuesto * 1000
        hipotenusa = self.kms_entre_puntos(float(latitud_origen),
                                                    float(longitud_origen),
                                                    float(latitud_destino),
                                                    float(longitud_destino))

        if latitud_destino < latitud_origen:
            cateto_opuesto = -cateto_opuesto

        if latitud_destino >= latitud_origen:
            if longitud_destino >= longitud_origen:
                cuadrante = 1
            else:
                cuadrante = 2
        else:
            if longitud_destino >= longitud_origen:
                cuadrante = 4
            else:
                cuadrante = 3

        seno = cateto_opuesto / hipotenusa

        if seno >= -sin(pi / 8) and seno <= sin(pi / 8):
            if cuadrante == 1 or cuadrante == 4:
                return 'E'
            else:
                return 'O'
        if seno >= sin(pi / 8) and seno <= sin(pi / 8 * 3):
            if cuadrante == 1:
                return 'NE'
            else:
                return 'NO'
        if seno >= sin(pi / 8 * 3) and seno <= 1:
            return 'N'
        if seno >= -sin(pi / 8 * 3) and seno <= -sin(pi / 8):
            if cuadrante == 4:
                return 'SE'
            else:
                return 'SO'
        else:
            return 'S'

    def obtener_rumbo(self, latitud_origen, longitud_origen,
                            latitud_destino, longitud_destino):
        '''
            Esta funci?n recibe la latitud y longitud origen y destino y
            devuelve el rumbo que se debe tomar para ir de un punto a otro.

            Argumentos:
                latitud_origen (float)
                longitud_origen (float)
                latitud_destino (float)
                longitud_destino (float)

            Retorna:
                rumbo (str) // N, NE, E, SE, S, SO, O, NO, YA LLEGO
        '''

        cateto_opuesto = self.km_entre_2_puntos(float(latitud_origen),
                                                    float(longitud_origen),
                                                    float(latitud_destino),
                                                    float(longitud_origen))
        #cateto_opuesto = cateto_opuesto * 1000
        hipotenusa = self.km_entre_2_puntos(float(latitud_origen),
                                                    float(longitud_origen),
                                                    float(latitud_destino),
                                                    float(longitud_destino))

        if hipotenusa < 0.05:
            return _('YA LLEGO')

        if latitud_destino < latitud_origen:
            cateto_opuesto = -cateto_opuesto

        if latitud_destino >= latitud_origen:
            if longitud_destino >= longitud_origen:
                cuadrante = 1
            else:
                cuadrante = 2
        else:
            if longitud_destino >= longitud_origen:
                cuadrante = 4
            else:
                cuadrante = 3

        seno = cateto_opuesto / hipotenusa

        if seno >= -sin(pi / 8) and seno <= sin(pi / 8):
            if cuadrante == 1 or cuadrante == 4:
                return 'E'
            else:
                return 'O'
        if seno >= sin(pi / 8) and seno <= sin(pi / 8 * 3):
            if cuadrante == 1:
                return 'NE'
            else:
                return 'NO'
        if seno >= sin(pi / 8 * 3) and seno <= 1:
            return 'N'
        if seno >= -sin(pi / 8 * 3) and seno <= -sin(pi / 8):
            if cuadrante == 4:
                return 'SE'
            else:
                return 'SO'
        else:
            return 'S'

    def procesar_geometria(self, geometria, repetidos=True):
        ''' Esta funcion procesa una geometria y devuelve todos los puntos de
        la misma.

        Argumentos:
            geometria: str
            repetidos: bool (permite o no permite puntos repetidos)

        Retorna:
            lista:  [[longitud, latitud]]'''

        # Declaramos lista de puntos
        puntos = []

        # Parseamos la geometria
        aux_puntos = geometria[geometria.find('('):].replace('((', '').replace(
            '(', '').replace('))', '').replace(')', '').split(',')
        # Recorremos los puntos
        for aux_punto in aux_puntos:
            punto = aux_punto.split(' ')
            punto[0] = float(punto[0])
            punto[1] = float(punto[1])
            if repetidos:
                puntos.append(punto)
            else:
                if punto not in puntos:
                    puntos.append(punto)

        # Devolvemos los puntos
        return puntos

    def punto_en_geometria(self, punto, geometria):
        ''' Funcion que nos dice si un punto esta dentro de una geometria '''

        # Validamos que sea como minimo un triangulo. Si no es asi, devolvemos
        # que esta adrentro, ya que no es una region todavia
        if len(geometria) < 3:
            return False
        # Si el punto forma parte del poligono, salimos :)
        if punto in geometria:
            return True
        # Sino, continuamos y ahi si calculamos donde corno esta :)
        cantidad = len(geometria)
        valido = False
        # Buscamos el primer punto
        p1_latitud, p1_longitud = geometria[0]
        # Recorremos toda la geometria
        for i in range(cantidad + 1):
            # Obtenemos el siguiente punto
            p2_latitud, p2_longitud = geometria[i % cantidad]
            # Si la longitud esta entre ambos puntos
            if punto[1] > min(p1_longitud, p2_longitud):
                if punto[1] <= max(p1_longitud, p2_longitud):
                    # Y la latitud tambien esta entre ambos puntos
                    if punto[0] <= max(p1_latitud, p2_latitud):
                        # Y no es el mismo punto, jeje
                        if p1_longitud != p2_longitud:
                            # Calculamos la sumatoria entre ambos puntos
                            xints = (
                                (punto[1] - p1_longitud) *
                                (p2_latitud - p1_latitud) /
                                (p2_longitud - p1_longitud)
                            ) + p1_latitud
                        # Validamos si coinciden las latitudes
                        if p1_latitud == p2_latitud or punto[0] <= xints:
                            valido = not valido
            # Actualizamos los puntos iniciales
            p1_latitud, p1_longitud = p2_latitud, p2_longitud
        # Devolvemos si esta o no dentro de la geometria
        return valido

    def punto_geometria(self, longitud, latitud):
        ''' Esta funcion toma como parametro una longitud y una latitud en
        grados y los transforma a strings, en el formato necesario para
        postgis '''
        return 'POINT(%s %s)' % (str(longitud), str(latitud))

    def validar_punto_geometria(self, punto, permitir_vacio=False):
        ''' Funcion para validar si una lista de puntos corresponde a una
        geometria '''

        if not isinstance(punto, list):
            raise (ValueError, 'El punto debe ser una lista')
        if len(punto) != 2:
            if not permitir_vacio:
                raise (ValueError, 'El punto debe tener una lat, lon')

        if isinstance(punto[0], int) or isinstance(punto[0], long):
            punto[0] = float(punto[0])
        if isinstance(punto[1], int) or isinstance(punto[1], long):
            punto[1] = float(punto[1])
        if (not isinstance(punto[0], float) or
            not isinstance(punto[1], float)):
            raise (ValueError, 'Las coordenadas deben ser numericas.')
        # Si esta en segundos, pasamos a grados
        if abs(punto[0]) > 90:
            punto[0] = punto[0] / 3600
        if abs(punto[1]) > 180:
            punto[1] = punto[1] / 3600

    def validar_puntos_geometria(self, puntos, permitir_vacio=False):
        ''' Funcion para validar si una lista de puntos corresponde a una
        geometria '''

        if not isinstance(puntos, list):
            # Si no es una lista de puntos, salimos
            raise (ValueError, 'Las geometrias deben ser listas de puntos.')

        # Validamos los puntos
        for punto in puntos:
            self.validar_punto_geometria(punto, permitir_vacio)

    def obtener_diferencia_entre_geometrias(self, geometria1, geometria2):

        # Vamos a validar puntos de la gemetria
        self.validar_puntos_geometria(geometria1)
        self.validar_puntos_geometria(geometria2)

        # Funcion interna
        def hausdorff(geom1, geom2):
            distancias_husdorff = []
            for punto1 in geom1:
                menor_distancia = 1000000000000
                for punto2 in geom2:
                    delta_distancia = sqrt(
                        (punto1[0] - punto2[0]) ** 2 +
                        (punto1[1] - punto2[1]) ** 2
                    )
                    if delta_distancia < menor_distancia:
                        menor_distancia = delta_distancia
                # Agregamos la menor distancia del punto
                distancias_husdorff.append(menor_distancia)
            return sum(distancias_husdorff) / len(distancias_husdorff)

        # Devolvemos el promedio distancia
        h1 = hausdorff(geometria1, geometria2)
        h2 = hausdorff(geometria2, geometria1)
        # Salimos
        return (h1 + h2) / 2

    def obtener_extremos_linea(self, puntos):

        if len(puntos) != 2:

            return respuesta(
                False, _(u'Los puntos deben estar dentro de una lista.')
            )

        # Obtengo el punto 1.
        punto_1 = puntos[0]

        # Obtengo el punto 2.
        punto_2 = puntos[1]

        inferior_izquierdo = [0, 0]

        superior_derecho = [0, 0]

        # Obtengo el punto inferior_izquierdo.
        if punto_1[0] < punto_2[0]:
            inferior_izquierdo[0] = punto_1[0]
        else:
            inferior_izquierdo[0] = punto_2[0]

        if punto_1[1] < punto_2[1]:
            inferior_izquierdo[1] = punto_1[1]
        else:
            inferior_izquierdo[1] = punto_2[1]

        # Obtengo el punto superior_derecho.
        if punto_1[0] > punto_2[0]:
            superior_derecho[0] = punto_1[0]
        else:
            superior_derecho[0] = punto_2[0]

        if punto_1[1] > punto_2[1]:
            superior_derecho[1] = punto_1[1]
        else:
            superior_derecho[1] = punto_2[1]

        puntos = {
            'inferior_izquierdo': inferior_izquierdo,
            'superior_derecho': superior_derecho
        }

        return respuesta(True, puntos)

