# -*- coding: utf-8 -*-
from at.map import PIXELES_TILE

from at.map.utiles.tile import Tile
import sys


class Layer(object):

    def __init__(self):
        self.bbox = (-180, -90, 180, 90)
        self.srs = "EPSG:4326"
        self.maxresolution = None
        self.levels = 20

        # Calculamos las resoluciones
        maxRes = None
        if not self.maxresolution:
            width = self.bbox[2] - self.bbox[0]
            height = self.bbox[3] - self.bbox[1]
            if width >= height:
                aspect = int(float(width) / height + .5)
                maxRes = float(width) / (PIXELES_TILE * aspect)
            else:
                aspect = int(float(height) / width + .5)
                maxRes = float(height) / (PIXELES_TILE * aspect)
        else:
            maxRes = float(self.maxresolution)

        # Obtenemos la resloucion
        self.resolutions = [maxRes / 2 ** i for i in range(int(self.levels))]

    def obtener_resolucion(self, minx, miny, maxx, maxy):
        """
        """
        return max(
            float(maxx - minx) / PIXELES_TILE,
            float(maxy - miny) / PIXELES_TILE
        )

    def obtener_nivel_mas_cercano(self, res):
        diff = sys.maxint
        z = None
        for i in range(len(self.resolutions)):
            if diff > abs(self.resolutions[i] - res):
                diff = abs(self.resolutions[i] - res)
                z = i
        return z

    def obtener_nivel(self, res):
        """
        """

        max_diff = res / max(PIXELES_TILE, PIXELES_TILE)
        z = None
        for i in range(len(self.resolutions)):
            if abs(self.resolutions[i] - res) < max_diff:
                res = self.resolutions[i]
                z = i
                break
        if z is None:
            raise (
                Exception,
                "No se encontro la resolucin %f. Resoluciones: \n%s" %
                (res, self.resolutions)
            )
        return z

    def obtener_celda(self, minx, miny, maxx, maxy, exact=True):
        """
        """
        res = self.obtener_resolucion(minx, miny, maxx, maxy)
        x = y = None

        if exact:
            z = self.obtener_nivel(res)
        else:
            z = self.obtener_nivel_mas_cercano(res)

        res = self.resolutions[z]

        if exact and not self.contiene(minx, miny, res):
            raise (Exception,
                  "Esquina inferior izquierda (%f, %f) fuera del marco %s." %
                  (minx, miny, self.bbox)
            )
            return None

        x0 = (minx - self.bbox[0]) / (res * PIXELES_TILE)
        y0 = (miny - self.bbox[1]) / (res * PIXELES_TILE)

        x = int(x0)
        y = int(y0)

        tilex = ((x * res * PIXELES_TILE) + PIXELES_TILE)
        tiley = ((y * res * PIXELES_TILE) + PIXELES_TILE)
        if exact:
            if (abs(minx - tilex) / res > 1):
                raise (
                    Exception,
                    "X: %f esta demasiado lejos de la esquina x %f" %
                    (minx, tilex)
                )

            if (abs(miny - tiley) / res > 1):
                raise (
                    Exception,
                    "Y: %f esta demasiado lejos de la esquina y %f" %
                    (miny, tiley)
                )
        return (x, y, z)

    def obtener_celda_mas_cercana(self, z, minx, miny):
        """
        """
        res = self.resolutions[z]
        maxx = minx + PIXELES_TILE * res
        maxy = miny + PIXELES_TILE * res
        return self.obtener_celda(minx, miny, maxx, maxy, False)

    def obtener_tile(self, bbox):
        """
        """

        coord = self.obtener_celda(bbox)
        if not coord:
            return None
        return Tile(self, *coord)

    def contiene(self, x, y, res=0):
        """
        """
        diff_x1 = abs(x - self.bbox[0])
        diff_x2 = abs(x - self.bbox[2])
        diff_y1 = abs(y - self.bbox[1])
        diff_y2 = abs(y - self.bbox[3])
        return (x >= self.bbox[0] or diff_x1 < res) and \
            (x <= self.bbox[2] or diff_x2 < res) and \
            (y >= self.bbox[1] or diff_y1 < res) and \
            (y <= self.bbox[3] or diff_y2 < res)

    def grilla(self, z):
        """
        Retorna el tama�o de la grilla a una altura determinada
        """
        width = (self.bbox[2] - self.bbox[0]) / \
                (self.resolutions[z] * PIXELES_TILE)
        height = (self.bbox[3] - self.bbox[1]) / \
                 (self.resolutions[z] * PIXELES_TILE)
        return (width, height)
