# -*- coding: utf-8 -*-

from at.orm.decorators import DBSwitch

from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import func
from sqlalchemy.sql.expression import asc
from sqlalchemy.orm import aliased
from math import sqrt

from at.map.utiles.referencia import Referencia
from at.map.utiles.punto import Punto


@DBSwitch('gis2')
class MapasPostgres(object):

    def __init__(self):
        pass

    def buscar(self, punto_desde, punto_hasta, altura_desde, altura_hasta):

        session = self.gis2.binded_session()
        tabla_calles = self.gis2.tabla('calles')
        tabla_calles_nombres = self.gis2.tabla('calles_nombres')
        q_obj = session.query(
            tabla_calles.gid, tabla_calles.id_calle_nombre,
            func.st_astext(tabla_calles.geom).label('geom'),
            func.ST_HausdorffDistance(
                tabla_calles.geom,
                func.st_geomfromtext(
                    'MULTILINESTRING((%s %s, %s %s))' % (
                        str(punto_desde.longitud),
                        str(punto_desde.latitud),
                        str(punto_hasta.longitud),
                        str(punto_hasta.latitud),
                    ),
                    4326
                )
            ).label('distancia')
        ).join((
            tabla_calles_nombres, and_(
                tabla_calles_nombres.id == tabla_calles.id_calle_nombre,
                tabla_calles_nombres.id_ciudad == 1381
            )
        )).filter(
            func.ST_HausdorffDistance(
                tabla_calles.geom,
                func.st_geomfromtext(
                    'MULTILINESTRING((%s %s, %s %s))' % (
                        str(punto_desde.longitud),
                        str(punto_desde.latitud),
                        str(punto_hasta.longitud),
                        str(punto_hasta.latitud),
                    ),
                    4326
                )
            ).label('distancia') < 0.003
        ).order_by(asc('distancia'))
        calle = q_obj.first()
        if calle is not None:
            aux = calle.geom.replace('MULTILINESTRING', '')
            aux = aux.replace('((', '').replace('))', '')
            aux = aux.split(',')
            punto_p_desde = aux[0].split(' ')
            punto_p_hasta = aux[1].split(' ')

            delta_lat_desde = abs(float(punto_p_desde[1]) - punto_desde.latitud)
            delta_lon_desde = abs(float(punto_p_desde[0]) - punto_desde.longitud)
            delta_desde = sqrt(delta_lat_desde ** 2 + delta_lon_desde ** 2)

            delta_lat_hasta = abs(float(punto_p_hasta[1]) - punto_desde.latitud)
            delta_lon_hasta = abs(float(punto_p_hasta[0]) - punto_desde.longitud)
            delta_hasta = sqrt(delta_lat_hasta ** 2 + delta_lon_hasta ** 2)
            c = {}
            if delta_desde < delta_hasta:
                c['desde_izq'] = altura_desde
                c['hasta_izq'] = altura_hasta - 2
                c['desde_der'] = altura_desde + 1
                c['hasta_der'] = altura_hasta - 1
                print 'CORRECTO'
            else:
                c['desde_izq'] = altura_hasta - 1
                c['hasta_izq'] = altura_desde + 1
                c['desde_der'] = altura_hasta - 2
                c['hasta_der'] = altura_desde
                print 'INVERTIDO'

            try:
                q_obj = session.query(tabla_calles).filter(tabla_calles.gid == calle.gid)
                q_obj.update(c, synchronize_session=False)
                session.commit()
                session.close()
            except Exception, e:
                print e
                session.rollback()
                session.close()
            #print punto_p_desde, punto_p_hasta
            print calle
        session.close()
        #for c in calles:
            #print c.distancia


@DBSwitch('gis')
class Mapas(object):
    def __init__(self, id_ciudad):
        self.id_ciudad = id_ciudad
        self.mp = MapasPostgres()
        self.ref = Referencia()
        self.referencias = self.ref.obtener_referencias(self.id_ciudad)

    def listar(self):

        session = self.gis.binded_session()

        tabla_lineas = self.gis.tabla('lineas')
        tabla_nodos = self.gis.tabla('nodos')

        nodos2 = aliased(tabla_nodos)

        q_obj = session.query(
            tabla_nodos.X.label('x_desde'),
            tabla_nodos.Y.label('y_desde'),
            nodos2.X.label('x_hasta'),
            nodos2.Y.label('y_hasta'),
            #tabla_lineas.AlturaDesde,
            #tabla_lineas.AlturaHasta,
        )

        q_obj = q_obj.outerjoin((tabla_lineas, and_(
            tabla_nodos.idMapa == tabla_lineas.idMapa,
            tabla_nodos.idNodo == tabla_lineas.idNodoDesde,
            )
        )).outerjoin((nodos2, and_(
            nodos2.idMapa == tabla_lineas.idMapa,
            nodos2.idNodo == tabla_lineas.idNodoHasta,
            )
        )).filter(and_(tabla_lineas.idMapa == self.id_ciudad,
                       tabla_lineas.idCiudad == self.id_ciudad,
                       tabla_lineas.idTipoLinea == 1,
                       tabla_lineas.idLayer == 1,
                       )
                   )
        print q_obj.count()
        return []
        lineas = q_obj.all()
        rta = []
        for linea in lineas:
            p = Punto(int(linea.x_desde), int(linea.y_desde), self.id_ciudad)
            self.ref.de_xy_a_segundos(p)
            p.latitud = p.latitud / 3600
            p.longitud = p.longitud / 3600

            p2 = Punto(int(linea.x_hasta), int(linea.y_hasta), self.id_ciudad)
            self.ref.de_xy_a_segundos(p2)
            p2.latitud = p2.latitud / 3600
            p2.longitud = p2.longitud / 3600
            rta.append([[p.latitud, p.longitud], [p2.latitud, p2.longitud]])
            #self.mp.buscar(p, p2, linea.AlturaDesde, linea.AlturaHasta)
            #print '-----------------'
        session.close()
        return rta

if __name__ == '__main__':
    m = Mapas(1)
    m.listar()
