# -*- coding: utf-8 -*-

from at.orm.decorators import DBSwitch
from sqlalchemy.sql.expression import and_, func


@DBSwitch('gis2')
class Mapas(object):
    def __init__(self):
        self.debug = False

    def trasladar_calles(self, lat1, lon1, lat2, lon2,
                                        id_ciudad_viejo, id_ciudad_nuevo):

        '''
        lat1, lon1: punto inferior izquierdo del bbox
        lat2, lon2: punto superior derecho del bbox
        id_ciudad_viejo:

        '''
        session = self.gis2.binded_session()
        tabla_calles = self.gis2.tabla('calles')
        tabla_calles_nombres = self.gis2.tabla('calles_nombres')

        # Obtenemos el ultimo id de calle nombre
        q_obj = session.query(tabla_calles_nombres)
        q_obj = q_obj.order_by(tabla_calles_nombres.id.desc())
        calle = q_obj.first()
        ultimo_id_calle_nombre = calle.id + 1

        q_obj = session.query(
            #tabla_calles.gid,
            tabla_calles_nombres.id.distinct().label('id'),
            tabla_calles_nombres.nombre
        ).join((
            tabla_calles,
            tabla_calles_nombres.id == tabla_calles.id_calle_nombre
        )).filter(
            and_(
                func.st_intersects(
                    tabla_calles.geom,
                    func.st_geomfromtext(
                        'POLYGON((%s %s, %s %s, %s %s, %s %s, %s %s))' % (
                            str(lon1), str(lat1),
                            str(lon2), str(lat1),
                            str(lon2), str(lat2),
                            str(lon1), str(lat2),
                            str(lon1), str(lat1),
                        ),
                        4326
                    )
                ),
                tabla_calles_nombres.id_ciudad == id_ciudad_viejo
            )
        )
        total = q_obj.count()
        calles = q_obj.all()
        for calle in calles:
            q_obj = session.query(
                tabla_calles.gid
            ).join((
                tabla_calles_nombres,
                tabla_calles_nombres.id == tabla_calles.id_calle_nombre
            )).filter(
                and_(
                    (func.st_intersects(
                        tabla_calles.geom,
                        func.st_geomfromtext(
                            'POLYGON((%s %s, %s %s, %s %s, %s %s, %s %s))' % (
                                str(lon1), str(lat1),
                                str(lon2), str(lat1),
                                str(lon2), str(lat2),
                                str(lon1), str(lat2),
                                str(lon1), str(lat1),
                            ),
                            4326
                        )
                    )) == False,
                    tabla_calles_nombres.id == calle.id
                )
            )
            total_afuera = q_obj.count()
            if total_afuera > 0:
                sql = "INSERT INTO calles_nombres VALUES (%s, %s, '%s', '')" % (
                    str(ultimo_id_calle_nombre), str(id_ciudad_nuevo),
                    calle.nombre
                )
                print sql
                conn = session.connection()
                r = conn.execute(sql)
                print 'Salida', r

                segmentos = q_obj.all()
                for segmento in segmentos:
                    sql = 'UPDATE calles SET id_calle_nombre = %s WHERE gid = %s' % (
                        str(ultimo_id_calle_nombre), str(segmento.gid)
                    )
                    # Actualizamos los segmentos
                    conn = session.connection()
                    r = conn.execute(sql)
                    print 'Salida', r

                # Aumentamos el id
                ultimo_id_calle_nombre += 1
            else:
                sql = "UPDATE calles_nombres SET id_ciudad = '" + str(
                    id_ciudad_nuevo
                ) + "'"
                sql += " WHERE id ='" + str(calle.id) + "'"
                print sql
                conn = session.connection()
                r = conn.execute(sql)
                print 'Salida', r
        session.commit()
        session.close()
        print total

    def actualizar_ciudad(self, id_ciudad, nombre, latitud, longitud):
        session = self.gis2.binded_session()
        tabla_ciudades = self.gis2.tabla('ciudades')
        q_obj = session.query(tabla_ciudades).filter(
            tabla_ciudades.id == id_ciudad
        )
        if q_obj.count() == 0:
            return

        ciudad = {
            'nombre': nombre,
            'geom': func.st_geomfromtext(
                'POINT(%s %s)' % (str(longitud), str(latitud)),
                4326
            )
        }
        try:
            q_obj.update(ciudad, synchronize_session=False)
            session.commit()
            session.close()
        except Exception, e:
            session.rollback()
            session.close()
            print e

if __name__ == '__main__':
    m = Mapas()

    ### SAN Francisco de cordoba
    m.trasladar_calles(
        -31.524703, -62.239723,
        -31.307302, -61.913567,
        934,
        86
    )
    m.trasladar_calles(
        -31.524703, -62.239723,
        -31.307302, -61.913567,
        2920,
        86
    )

    ### SANTA FE
    m.trasladar_calles(
        -31.712397, -60.809383,
        -31.520605, -60.576611,
        934,
        2920
    )
    m.trasladar_calles(
        -31.712397, -60.809383,
        -31.520605, -60.576611,
        86,
        2920
    )
    ### San Francisco de Santa Fe
    m.trasladar_calles(
        -33.617477, -62.157211,
        -33.546753, -62.085114,
        666,
        934
    )
    m.trasladar_calles(
        -33.617477, -62.157211,
        -33.546753, -62.085114,
        86,
        934
    )
    m.trasladar_calles(
        -33.617477, -62.157211,
        -33.546753, -62.085114,
        2920,
        934
    )
    m.actualizar_ciudad(
        934,
        u'San Francisco de Santa Fe',
        -33.588123,
        -62.123737
    )

    ### SAN JERONIMO
    m.actualizar_ciudad(
        3648,
        u'San Jerónimo Sud',
        -32.878904,
        -61.024418
    )

    m.trasladar_calles(
        -32.957977, -61.132107,
        -32.780925, -60.92268,
        3909,
        3648
    )
    m.trasladar_calles(
        -32.957977, -61.132107,
        -32.780925, -60.92268,
        3064,
        3648
    )

    # ARROYO SECO!!
    m.trasladar_calles(
        -33.225477, -60.595894,
        -33.083487, -60.407066,
        3803,
        1600
    )

    # VILLA CONSTITUCION
    m.trasladar_calles(
        -33.291664, -60.361805,  # Punto inferior izquierdo del bbox
        -33.183517, -60.284386,  # Punto superior derecho del bbox
        4872,  # Empalme villa constitucion
        2326   # Villa constitucion
    )

    # EMPALME VILLA CONSTITUCION
    m.trasladar_calles(
        -33.273211, -60.391245,
        -33.246174, -60.368156,
        2326,
        4872
    )

    # CARCARAÑA
    m.trasladar_calles(
        -32.900963, -61.191101,
        -32.832642, -61.111107,
        4602,
        3270
    )

    # VILLA RAMALLO
    m.trasladar_calles(
        -33.52478, -60.083199,
        -33.490105, -60.038223,
        1578,
        4269
    )

    # Córdoba
    m.trasladar_calles(
        -31.543431, -64.379311,
        -31.159934, -63.942604,
        1318,
        4077
    )

    # Pavón
    m.trasladar_calles(
        -33.248363, -60.415406,
        -33.23802, -60.394206,
        1671,
        1918
    )

    # La Carolina de Santa Fe
    m.trasladar_calles(
        -33.116484, -60.733538,  # Punto inferior izquierdo del bbox
        -33.051167, -60.63921,  # Punto superior derecho del bbox
        2457,  # La Carolina de San Luis
        5009   # La Carolina de Santa Fe
    )