# -*- coding: utf-8 -*-

import math
from operator import itemgetter
from subprocess import Popen
from subprocess import PIPE
from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import or_
from sqlalchemy.sql.expression import func
from sqlalchemy.sql.expression import not_
from sqlalchemy import text

from at.map import _
from at.map.utiles.funciones_geo import FuncionesGeo
from at.map.calle import CallePostgres
from at.map.ciudad import CiudadPostgres
from at.orm.decorators import DBSwitch
from at.orm.utiles import obtener_datos_conexion_bd
from at.utiles.conversiones import respuesta
from at.utiles.conversiones import quitar_acentos
from at.utiles.validar import no_vacio
from at.utiles.shell import ejecutar_comando

v_mapadmin = True

try:
    from at.mapadmin.operaciones import Operaciones
except:
    print '>> warn: No hay at.mapadmin'
    v_mapadmin = False


@DBSwitch('gis2', 'gis_osm')
class MigracionesOsm(object):
    def __init__(self):
        self.proyeccion = 4326
        self.funciones_geo = FuncionesGeo()
        self.calle_postgres = CallePostgres()
        self.ciudad_postgres = CiudadPostgres()
        self.operaciones = Operaciones()

    def agregar_segmento(self, segm, id_calle_desconocida, id_ciudad, sesion):

        # Proceso la geometría.
        geom = self.funciones_geo.procesar_geometria(segm['geom_text'])

        nombre_calle_osm = segm['name']

        id_usuario = 1

        if nombre_calle_osm is None or nombre_calle_osm == '':

            segmento = {
                'IdCalleNombre': id_calle_desconocida,
                'IdTipo': 2,
                'Puntos': [[geom[0][1], geom[0][0]], [geom[1][1], geom[1][0]]],
                'Sentido': segm['one_way']
            }

            # Trato de agregar
            rta_g_seg = self.operaciones.grabar_segmento(
                id_usuario, segmento, sesion=sesion
            )

            return rta_g_seg

        session = self.gis2.binded_session()

        calles_nombres = self.gis2.tabla('calles_nombres')
        q_obj = session.query(calles_nombres.id)
        q_obj = q_obj.filter(
            and_(
                calles_nombres.id_ciudad == id_ciudad,
                calles_nombres.nombre == nombre_calle_osm,
            )
        )
        registro = q_obj.first()

        session.close()

        # Defino id_nueva_calle
        id_nueva_calle = None

        # El usuario 1 es el admin.
        id_usuario = 1

        if registro is None:

            # Trato de agregar la calle.
            rta_grabar_calle = self.operaciones.grabar_calle(
                id_usuario, nombre_calle_osm, int(id_ciudad), id_nueva_calle,
                sesion=sesion
            )

            # Verifico el resultado.
            if rta_grabar_calle['Estado'] is not True:
                return rta_grabar_calle

            # Guardo el id_calle.
            id_nueva_calle = rta_grabar_calle['DatosExtra']['id']

        else:

            id_nueva_calle = registro.id

        # Defino el segmento.
        segmento = {
            'IdCalleNombre': int(id_nueva_calle),
            'IdTipo': 2,
            'Puntos': [[geom[0][1], geom[0][0]], [geom[1][1], geom[1][0]]]
        }

        # Trato de agregar.
        rta_grabar = self.operaciones.grabar_segmento(
            id_usuario, segmento,  sesion=sesion
        )

        return rta_grabar

    def NorthAzimuth(self, punto1, punto2):
        degBearing = math.degrees(
            math.atan2(
                (punto2[0] - punto1[0]),
                (punto2[1] - punto1[1])
            )
        )

        if (degBearing < 0):
            # degBearing += 360.0
            degBearing += 180.0

        # degBearing = abs(degBearing)

        return degBearing

    def comparar_segmentos(self, calle_osm, id_ciudad):

        # Abro la sesión con la BD con gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles
        calles = self.gis2.tabla('calles')
        calles_nombres = self.gis2.tabla('calles_nombres')

        # Este dato se utilizara si osm esta entre los puntos de seg. gis2
        osm_entre_puntos_gis2 = False

        # Obtenemos los puntos del osm para poder calcular las distancias
        conj_linea = self.funciones_geo.procesar_geometria(
            calle_osm['geom_text'])

        linea_osm = (conj_linea[0], conj_linea[len(conj_linea) - 1])

        long_punto_uno_osm = linea_osm[0][0]
        lat_punto_uno_osm = linea_osm[0][1]

        long_punto_dos_osm = linea_osm[1][0]
        lat_punto_dos_osm = linea_osm[1][1]

        # Invertir coordenadas ya que la función procesar geometria me las
        # devuelve invertida
        linea_osm_inv = (conj_linea[0], conj_linea[len(conj_linea) - 1])

        punto_aux_1 = linea_osm_inv[0][1]
        linea_osm_inv[0][1] = linea_osm_inv[0][0]
        linea_osm_inv[0][0] = punto_aux_1

        punto_aux_2 = linea_osm_inv[1][1]
        linea_osm_inv[1][1] = linea_osm_inv[1][0]
        linea_osm_inv[1][0] = punto_aux_2

        # Obtenemos el largo del segmento osm
        dist_puntos_uno_dos_osm = self.funciones_geo.mts_entre_puntos(
            lat_punto_uno_osm,
            long_punto_uno_osm,
            lat_punto_dos_osm,
            long_punto_dos_osm)

        # Este angulo es el del registro osm con respecto al ecuador.
        angulo_osm = self.NorthAzimuth(linea_osm[0], linea_osm[1])

        # print calle_osm['geom']
        # print func.st_geomfromtext(calle_osm['geom'], self.proyeccion)

        # Obtenemos los datos necesarios de los segmentos de gis2
        q_obj = session.query(
            calles.gid,
            func.st_astext(calles.geom).label('geom'),
            func.ST_HausdorffDistance(
                calles.geom,
                calle_osm['geom'],
                # func.st_geomfromtext(calle_osm['geom'], self.proyeccion),
                0.5,
            ).label('hausdorf'),
            func.st_distance(
                calles.geom,
                calle_osm['geom'],
                # func.st_geomfromtext(calle_osm['geom'], self.proyeccion),
            ).label('distancia'),
            calles.id_calle_nombre
        )
        # Ahora filtramos los segmentos gis2 dentro del area que tenemos
        # como parametro, luego los que tengan st_distance < 0.0009 con
        # respecto al segmento osm

        q_obj = q_obj.join((
            calles_nombres,
            and_(
                calles.id_calle_nombre == calles_nombres.id,
                calles_nombres.id_ciudad == id_ciudad
            )
        ))

        q_obj = q_obj.filter(
            and_(
                func.st_distance(
                    calles.geom,
                    calle_osm['geom'],
                    # func.st_geomfromtext(calle_osm['geom'],
                    #                      self.proyeccion),
                ) < 0.0009,
            )
        )

        # if int(q_obj.count()) == 0:
        #     session.close()
        #     return respuesta(False)

        registros = q_obj.all()

        session.close()

        no_agregar_osm = 0
        # Recorremos los registros gis2 para compararlos con los osm y poder
        # saber si debemos agregar el segmento o no
        for registro in registros:

            se_agrega_osm = 0
            # Transformamos el geom del segmento gis2 en puntos para poder
            # comparar con los de osm
            conj_linea_gis2 = self.funciones_geo.procesar_geometria(
                registro[1])
            linea_gis2 = (conj_linea_gis2[0], conj_linea_gis2[1])

            long_punto_uno_gis2 = linea_gis2[0][0]
            lat_punto_uno_gis2 = linea_gis2[0][1]

            long_punto_dos_gis2 = linea_gis2[1][0]
            lat_punto_dos_gis2 = linea_gis2[1][1]
            # Damos vuelta los puntos ya que procesar_geometria nos los trae
            # al reves
            linea_gis2_inv = (conj_linea_gis2[0], conj_linea_gis2[1])

            punto_aux_gis2_1 = linea_gis2_inv[0][1]
            linea_gis2_inv[0][1] = linea_gis2_inv[0][0]
            linea_gis2_inv[0][0] = punto_aux_gis2_1

            punto_aux_gis2_2 = linea_gis2_inv[1][1]
            linea_gis2_inv[1][1] = linea_gis2_inv[1][0]
            linea_gis2_inv[1][0] = punto_aux_gis2_2

            # Obtenemos las distancias necesarias para poder hacer las
            # comparaciones.
            dist_p1_osm_p1_gis2 = self.funciones_geo.mts_entre_puntos(
                lat_punto_uno_osm,
                long_punto_uno_osm,
                lat_punto_uno_gis2,
                long_punto_uno_gis2)

            dist_p2_osm_p2_gis2 = self.funciones_geo.mts_entre_puntos(
                lat_punto_dos_osm,
                long_punto_dos_osm,
                lat_punto_dos_gis2,
                long_punto_dos_gis2)

            dist_p1_p2_gis2 = self.funciones_geo.mts_entre_puntos(
                lat_punto_uno_gis2,
                long_punto_uno_gis2,
                lat_punto_dos_gis2,
                long_punto_dos_gis2)

            # Para ver si se debe invertir se utilizan las sig. distancias
            dist_p1_osm_p2_gis2 = self.funciones_geo.mts_entre_puntos(
                lat_punto_uno_osm,
                long_punto_uno_osm,
                lat_punto_dos_gis2,
                long_punto_dos_gis2)

            dist_p2_osm_p1_gis2 = self.funciones_geo.mts_entre_puntos(
                lat_punto_dos_osm,
                long_punto_dos_osm,
                lat_punto_uno_gis2,
                long_punto_uno_gis2)

            # angulo_aux es el angulo del segmento gis2 con respecto
            # al ecuador.
            angulo_aux = self.NorthAzimuth(conj_linea_gis2[0],
                                           conj_linea_gis2[1])
            # dif_angulo es el valor absoluto de la diferencia entre
            # angulo osm y gis2

            dif_angulo = abs(angulo_aux - angulo_osm)

            # si estan invertidos los puntos los damos vueltas
            if dist_p1_osm_p2_gis2 < dist_p1_osm_p1_gis2 and \
                            dist_p2_osm_p1_gis2 < dist_p2_osm_p2_gis2:
                # asignamos las variables para poder invertir
                aux_dist_p1_osm_p2_gis2 = dist_p1_osm_p2_gis2
                aux_dist_p2_osm_p1_gis2 = dist_p2_osm_p1_gis2
                aux_dist_p1_osm_p1_gis2 = dist_p1_osm_p1_gis2
                aux_dist_p2_osm_p2_gis2 = dist_p2_osm_p2_gis2
                # invertimos los puntos de gis2
                dist_p1_osm_p1_gis2 = aux_dist_p1_osm_p2_gis2
                dist_p2_osm_p2_gis2 = aux_dist_p2_osm_p1_gis2
                dist_p1_osm_p2_gis2 = aux_dist_p1_osm_p1_gis2
                dist_p2_osm_p1_gis2 = aux_dist_p2_osm_p2_gis2

            dist_max = 50
            # Filtramos los segmentos gis2 con angulo menor a 10 grados con
            # respecto al osm para ver si se debe agregar
            if dif_angulo < 10:
                # Filtramos para ver si agregamos el segmento osm comparando
                # las distancias

                # Vemos si el segmento osm esta entre los puntos
                # p1 y p2 del gis2
                if (dist_p1_osm_p2_gis2 < dist_p1_p2_gis2 and
                            dist_p2_osm_p1_gis2 < dist_p1_p2_gis2):
                    osm_entre_puntos_gis2 = True
                # Filtramos para ver si debemos agregar el osm
                if (dist_p1_osm_p1_gis2 > dist_puntos_uno_dos_osm) and \
                        (osm_entre_puntos_gis2 is False) and \
                        (dist_p2_osm_p2_gis2 > dist_max):
                    se_agrega_osm += 1

                if (dist_p2_osm_p2_gis2 > dist_puntos_uno_dos_osm) and \
                        (osm_entre_puntos_gis2 is False) and \
                        (dist_p1_osm_p1_gis2 > dist_max):
                    se_agrega_osm += 1

                if (dist_p1_osm_p1_gis2 > dist_max) and \
                        (osm_entre_puntos_gis2 is False):
                    se_agrega_osm += 1

                if (dist_p2_osm_p2_gis2 > dist_max) and \
                        (osm_entre_puntos_gis2 is False):
                    se_agrega_osm += 1
                # Si no entro a algun filtro no se dibuja
                if se_agrega_osm == 0:
                    no_agregar_osm += 1

        if no_agregar_osm == 0:
            return respuesta(True)
        else:
            return respuesta(False)

    def procesar_multilinea(self, calle_osm_multi, nombre_ciudad_gis,
                            id_calle_desconocida, id_ciudad, sesion):

        registros_agregados = 0

        # Separamos en puntos el segmento osm para poder formar
        # segmentos de dos puntos
        conj_linea = self.funciones_geo.procesar_geometria(
            calle_osm_multi['geom_text'])

        # Defino one_way por defecto en 0, es decir que es doble sentido.
        one_way = 0

        tmp_one_way = calle_osm_multi.get('one_way')

        if tmp_one_way == 'yes':
            one_way = 1

        # Recorremos los puntos para formar los segmentos de dos puntos
        for i in range(len(conj_linea) - 1):
            geom_text_dos_puntos = None
            geom_text_dos_puntos = (conj_linea[i], conj_linea[i + 1])

            lat_punto1 = geom_text_dos_puntos[0][1]
            long_punto1 = geom_text_dos_puntos[0][0]

            lat_punto2 = geom_text_dos_puntos[1][1]
            long_punto2 = geom_text_dos_puntos[1][0]
            # Damos vueltas los puntos ya que la función procesar geometria
            # las trae al reves
            geom_text_dos_puntos_inv = ([lat_punto1, long_punto1],
                                        [lat_punto2, long_punto2])

            # Recuperamos el nombre del segmento osm
            name_osm_dos_puntos = calle_osm_multi['name']

            # Recuperamos la ciudad del segmento osm.
            city_osm_dos_puntos = calle_osm_multi['city']

            # Verifico el nombre de las ciudades.
            rta_v_c = self.comparar_nombre_ciudad(
                nombre_ciudad_gis, city_osm_dos_puntos
            )

            # Verifico el resultado.
            if rta_v_c['Estado'] is not True:
                continue

            # Generamos el formato para poder llevarlo a geom
            multi_dos_puntos = self.funciones_geo.generar_geometria_linea(
                geom_text_dos_puntos_inv)
            # Pasamos al formato geom
            geom_dos_puntos = func.st_geomfromtext(multi_dos_puntos, 4326)

            datos_osm = {}
            datos_osm['name'] = name_osm_dos_puntos
            datos_osm['geom_text'] = multi_dos_puntos
            datos_osm['geom'] = geom_dos_puntos
            datos_osm['one_way'] = one_way

            rta_comparar = self.comparar_segmentos(datos_osm, id_ciudad)

            if rta_comparar['Estado'] is True:
                rta_agregar_segmento = self.agregar_segmento(
                    datos_osm, id_calle_desconocida, id_ciudad, sesion
                )

                if rta_agregar_segmento['Estado'] is True:
                    registros_agregados += 1

        if registros_agregados == 0:
            return respuesta(False, _(u'No se pudieron agregar segmentos.'))

        else:
            return respuesta(
                True,
                _(u'Se agregaron %s segmentos.' % registros_agregados),
                registros_agregados
            )

    def generar_datos_de_entrada(self, id_ciudad, sesion):
        '''
            Esta función se encarga de comparar los segmentos de una ciudad
            OSM con los existentes en gis2 e insertar los faltantes.

            Argumento:
                lista_calles (lista de tuple)
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)
                id_calle_nombre (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo los datos de la ciudad a migrar.
        rta_ciudad = self.ciudad_postgres.obtener_ciudad(id_ciudad)

        # Verifico el resultado.
        if rta_ciudad['Estado'] is not True:
            return rta_ciudad

        # Obtengo el nombre.
        nombre_ciudad_gis = rta_ciudad['Respuesta']['Nombre'].decode('latin1')

        nombre_calle_desconida = 'Desconocida ' + nombre_ciudad_gis

        id_usuario = 1

        # Trato de agregar la calle desconocida.
        rta_grabar_calle = self.operaciones.grabar_calle(
            id_usuario, nombre_calle_desconida, int(id_ciudad), None,
            sesion=sesion
        )

        # Verifico el resultado.
        if rta_grabar_calle['Estado'] is not True:
            return rta_grabar_calle

        # Guardo el id_calle.
        id_calle_desconocida = rta_grabar_calle['DatosExtra']['id']

        dic_datos = {
            'id_calle_desconocida': int(id_calle_desconocida),
            'nombre_ciudad_gis': nombre_ciudad_gis
        }

        return respuesta(True, dato=dic_datos)

    def agregar_nuevos_segmentos_a_ciudad(self, nombre_ciudad_gis,
                                    id_calle_desconocida, id_ciudad, sesion):
        '''
            Esta función se encarga de obtener todos los segmentos de una
            zona dada.

            Argumento:
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de tuple)
        '''

        rta = {}

        session = self.gis_osm.binded_session()

        lines = self.gis_osm.tabla('planet_osm_line')

        #polygon = self.gis_osm.tabla('planet_osm_polygon')

        # Obtengo todos los segmentos de la zona dada
        q_obj2 = session.query(
            lines.osm_id.label('gid'),
            lines.name,
            func.st_astext(lines.way).label('geom'),
            lines.way,
            #polygon.name.label('city'),
            lines.oneway
        )

        # ARREGLADO: Saco este join porque al migrar un sector puede que no
        # este incluido el poligono de la ciudad
        # hacer join.
        # q_obj2 = q_obj2.join((
        #     polygon,
        #     and_(
        #         polygon.admin_level == '8',
        #         func.st_contains(
        #             func.st_astext(polygon.way),
        #             func.st_astext(lines.way)
        #         )
        #     ),
        # ))
        ######################################################################

        q_obj2 = q_obj2.filter(
            and_(
                lines.highway != '',
                func.st_isvalid(lines.way),
            )
        )

        if int(q_obj2.count()) == 0:
            session.close()
            return respuesta(
                False, _(u'No se pudieron obtener los segmentos osm.')
            )

        # Guardamos todos los registros encontrados
        registros = q_obj2.all()

        # Cierro la conexión con la base de datos
        session.close()

        segmentos_agregados = 0

        # Recorremos los registros para extraer los datos necesarios
        for registro in registros:

            multilinea = {}
            # multilinea['gid'] = registro[0]
            # multilinea['name'] = registro[1]
            # multilinea['city'] = registro[4]
            # multilinea['geom'] = registro[3]
            # multilinea['geom_text'] = registro[2]
            # multilinea['osm_id'] = registro[0]
            # multilinea['one_way'] = registro[5]

            multilinea['gid'] = registro.gid
            multilinea['name'] = registro.name
            multilinea['geom'] = registro.way
            multilinea['geom_text'] = registro.geom
            multilinea['osm_id'] = registro.gid
            multilinea['one_way'] = registro.oneway
            multilinea['city'] = nombre_ciudad_gis

            rta_procesar = self.procesar_multilinea(
                multilinea, nombre_ciudad_gis, id_calle_desconocida, id_ciudad,
                sesion
            )

            if rta_procesar['Estado'] is True:
                segmentos_agregados += rta_procesar['Dato']

        # Si no encuentro registros retorno el error
        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontraron segmentos en la zona dada')
            return rta

        # Si encontré registros retorno una lista de tuplas con los datos
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se agregaron %s segmentos en la zona dada.' % segmentos_agregados
        )
        rta['Dato'] = segmentos_agregados

        return rta

    def migrar_osm_a_gis2(self, id_ciudad, segmentos=False, alturas=False,
                          referencia=None, pisar_alturas=False, sesion=None):
        '''
            Esta función se encarga de insertar los segmentos de una ciudad
            de OSM a gis2.

            Argumento:
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)
                id_calle_nombre (int)

            Funciones:
                Migraciones().calles_ciudad
                Migraciones().comparar_calles

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Defino una respueta por defecto.
        rta = respuesta(False, _(u'No se ejecutó ninguna migración.'), {})

        if segmentos is True:

            rta_genear = self.generar_datos_de_entrada(id_ciudad, sesion)

            if rta_genear['Estado'] is not True:
                return rta_genear

            if referencia is None:

                nombre_ciudad_gis = rta_genear['Dato']['nombre_ciudad_gis']

            else:

                nombre_ciudad_gis = referencia

            id_calle_desconocida = rta_genear['Dato']['id_calle_desconocida']

            # Llamo a la función agregar_nuevos_segmentos_a_ciudad
            #  para traer todos los segmentos de
            # la zona en la BD osm
            rta_segmentos = self.agregar_nuevos_segmentos_a_ciudad(
                nombre_ciudad_gis, id_calle_desconocida, id_ciudad, sesion
            )

            rta = respuesta(True, _(u'Se ejecutaron las migraciones.'), {
                'segmentos': rta_segmentos
            })

        if alturas is True:
            rta_alturas = self.migrar_alturas(id_ciudad, pisar_alturas, sesion)
            rta['Dato']['alturas'] = rta_alturas

        return rta

    def comparar_nombre_ciudad(self, ciudad_gis, ciudad_osm):

        # trato de sacar acentos y paso a minuscula.
        ciudad_gis = quitar_acentos(ciudad_gis)['Respuesta'].lower()
        ciudad_osm = quitar_acentos(ciudad_osm)['Respuesta'].lower()

        if ciudad_gis == ciudad_osm:
            return respuesta(True)
        else:
            return respuesta(False)

    def calles_ciudad(self, id_ciudad):

        # Abro la sesión con la BD con gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles
        calles = self.gis2.tabla('calles')
        calles_nombres = self.gis2.tabla('calles_nombres')

        # Obtenemos los datos necesarios de los segmentos de gis2.
        q_obj = session.query(
            calles.gid,
            calles.geom.label('geom'),
            calles.desde_izq
        )

        q_obj = q_obj.join((
            calles_nombres,
            and_(
                calles.id_calle_nombre == calles_nombres.id,
                calles_nombres.id_ciudad == id_ciudad
            )
        ))

        if int(q_obj.count()) == 0:
            session.close()
            return respuesta(False, _(u'No se pudieron obtener los segmentos.'))

        registros = q_obj.all()

        session.close()

        return respuesta(True, rta=registros)


    def migrar_alturas(self, id_ciudad, pisar_alturas, sesion):
        '''
            Esta función se encarga de migrar las alturas de OSM a nuestra
            BD gis2

            Argumento:
                id_ciudad (int)

            Funciones:
                Migraciones().calles_ciudad
                Migraciones().obtener_altura_segmento
                CallesPostgres().actualizar_segmento

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Llamo a la función calles_ciudad
        rta_calles_ciudad = self.calles_ciudad(id_ciudad)
        # Si no se encuentra ningún segmento retorno el mensaje
        if rta_calles_ciudad['Estado'] is False:

            return rta_calles_ciudad

        # Guardo los segmentos obtenidos de la ciudad
        lista_segmentos = rta_calles_ciudad['Respuesta']

        alturas_migradas = 0

        # Recorro los segmento obtenidos
        for segmento in lista_segmentos:
            # Guardo el gid del segmento
            gid = segmento[0]
            geom = segmento[1]

            # Llamo a la función obtener_altura_segmento
            rta_obtener_altura_segmento = self.obtener_altura_segmento(
                gid, geom, pisar_alturas
            )

            # Si no se obtivieron las alturas
            if rta_obtener_altura_segmento['Estado'] is False:
                # Paso al siguiente segmento
                continue

            # Guardo las alturas
            desde_izq = rta_obtener_altura_segmento['Dato'][0]
            desde_der = rta_obtener_altura_segmento['Dato'][1]
            hasta_izq = rta_obtener_altura_segmento['Dato'][2]
            hasta_der = rta_obtener_altura_segmento['Dato'][3]

            # El id uno es el del admin.
            id_usuario = 1

            # Defino el segmento.
            segmento = {
                'Id': int(gid),
                'DesdeDer': desde_der,
                'HastaDer': hasta_der,
                'DesdeIzq': desde_izq,
                'HastaIzq': hasta_izq
            }

            # Llamo a la función actualizar_segmento
            rta_actualizar_segmento = self.operaciones.grabar_segmento(
                id_usuario, segmento, sesion=sesion
            )

            # Si no se pudo actualizar un segmento
            if rta_actualizar_segmento['Estado'] is False:
                # Retorno el error
                continue
                # return rta_actualizar_segmento

            alturas_migradas += 1

        # Si todo salió bien, retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se migraron las alturas de las calles')
        rta['Dato'] = alturas_migradas
        return rta

    def cargar_datos_osm(self):

        rta_obtener = obtener_datos_conexion_bd('gis_osm')

        if rta_obtener['Estado'] is not True:
            return rta_obtener

        host = rta_obtener['Respuesta']['host']
        usuario = rta_obtener['Respuesta']['user']
        clave = rta_obtener['Respuesta']['passwd']
        nombre_bd = rta_obtener['Respuesta']['name']

        # Defino el comando.
        comando = 'osm2pgsql -s'
        comando += ' -U ' + usuario
        comando += ' -d ' + nombre_bd
        comando += ' /var/lib/at/migraciones/osm/map.osm '
        comando += '-H ' + host
        # comando += ' --password ' + clave
        comando += ' -l --utf8-sanitize -S /var/lib/at/migraciones/osm/at.style'
        comando += ' --hstore'

        # Ejecuto el comando.
        rta_ejecutar = ejecutar_comando(comando)

        # Verifico si se ejecutó.
        if rta_ejecutar['Estado'] is not True:
            return rta_ejecutar

        # Defino sentencias SQL. Agrego las claves primarias.
        sql = 'ALTER TABLE planet_osm_point ADD gid serial PRIMARY KEY;'
        sql += 'ALTER TABLE planet_osm_line ADD gid serial PRIMARY KEY;'
        sql += 'ALTER TABLE planet_osm_polygon ADD gid serial PRIMARY KEY;'
        sql += 'ALTER TABLE planet_osm_roads ADD gid serial PRIMARY KEY;'
        sql = text(sql)

        # Abro sesión.
        session = self.gis_osm.binded_session()

        # Ejecuto las sentencias.
        self.gis_osm.engine.execute(sql)

        # Cierro la sesión.
        session.close()

        # Retorno.
        return rta_ejecutar

    def procesar_altura(self, altura):

        # Verifico si viene con Bis.
        altura_split = altura.split(' ')

        # Defino por defecto.
        altura_tmp = 0

        # Si no tiene bis guardo el registro.
        if len(altura_split) == 1:

            # Trato de pasar a entero.
            try:
                altura_tmp = int(altura)

            # Sinó retorno error.
            except:
                return respuesta(False, _(u'No tiene altura.'))

        # Si tiene 2 elementos verifico.
        if len(altura_split) == 2:

            # Si tiene bis.
            if altura_split[1] == 'Bis':

                # Trato de pasar a entero.
                try:
                    altura_tmp = 0 - int(altura_split[0])

                except:

                    return respuesta(False, _(u'No tiene altura.'))

            # Sinó retorno.
            else:

                return respuesta(False, _(u'No tiene altura.'))

        # Retorno la altura.
        return respuesta(True, dato=altura_tmp)

    def obtener_altura_segmento(self, gid, geom, pisar_alturas):

        # Declaro el diccionario a retornar
        rta = {}

        # Llamo a la función obtener_segmento
        rta_obtener_segmento = self.calle_postgres.obtener_segmento(gid)
        # Si no puedo obtener el segmento
        if rta_obtener_segmento['Estado'] is False:
            # Retorno el error
            return rta_obtener_segmento

        # Guardo los datos del segmento dado
        segmento = rta_obtener_segmento['Respuesta']

        if pisar_alturas is False:

            # Defino una bandera.
            tiene_altura = False

            # Verifico si tiene altura.
            v_desde_izq = no_vacio(segmento['desde_izq'])
            if v_desde_izq['Estado'] is True:
                tiene_altura = True

            v_hasta_izq = no_vacio(segmento['hasta_izq'])
            if v_hasta_izq['Estado'] is True:
                tiene_altura = True

            v_desde_der = no_vacio(segmento['desde_der'])
            if v_desde_der['Estado'] is True:
                tiene_altura = True

            v_hasta_der = no_vacio(segmento['hasta_der'])
            if v_hasta_der['Estado'] is True:
                tiene_altura = True

            if tiene_altura is True:
                return respuesta(False, _(u'El segmento %s tiene altura.' % gid))

            # Si el segmento no tiene el dato puntos
            if segmento['puntos'] is None:
                # No puedo buscar su altura, retorno False y un mensaje de error
                rta['Estado'] = False
                rta['Respuesta'] = \
                    _(u'El segmento %s no tiene puntos cargados' % gid)
                return rta

        # Abro la sesión con la base de datos osm
        session = self.gis_osm.binded_session()

        # Obtengo la tabla points
        lines = self.gis_osm.tabla('planet_osm_line')

        # Obtengo la tabla points
        points = self.gis_osm.tabla('planet_osm_point')

        columna_altura = 'addr:housenumber'
        columna_interpolacion = 'addr:interpolation'

        vacio = ''
        none = None

        # Busco el punto dado
        q_obj = session.query(
            getattr(points, columna_altura).label('altura'),
            getattr(lines, columna_interpolacion).label('interpolacion')
        )

        # Cruzo las tablas donde se tocan las lineas y los puntos.
        q_obj = q_obj.join((
            lines,
            func.st_touches(
                lines.way,
                points.way
            )
        ))

        # Filtro por distnacia, y que no sea vacio.
        q_obj = q_obj.filter(
            and_(
                func.st_hausdorffdistance(
                    geom,
                    lines.way
                ) < 0.0005,
                getattr(lines, columna_interpolacion) != vacio,
                getattr(lines, columna_interpolacion) != none,
                getattr(points, columna_altura) != none,
                getattr(points, columna_altura) != vacio
            )
        )

        # Ordeno por cercanía.
        q_obj = q_obj.order_by(
            func.st_distance(
                geom,
                lines.way
            ).asc()
        )

        # Obtengo solo 4 puntos.
        q_obj = q_obj.limit(4)

        # Solo sigo se obtiene más de un registro.
        if int(q_obj.count()) <= 1:
            session.close()
            return respuesta(False, _(u'No se puedo obtener la altura.'))

        # Guardo los registros obtenidos
        registros = q_obj.all()

        # Cierro sesión.
        session.close()

        # Ordeno por altura.
        registros = sorted(registros, key=itemgetter(0))

        # Defino la lista.
        lista = []

        # Recorro los registros.
        for registro in registros:

            # Verifico si viene con Bis.
            altura = registro[0]

            # Proceso la altura.
            rta_altura = self.procesar_altura(altura)

            # Verifico.
            if rta_altura['Estado'] is not True:
                return rta_altura

            # Obtengo la altura.
            altura = rta_altura['Dato']

            # Obtengo la interlacion.
            interpolacion = registro[1]

            lista.append([altura, interpolacion])

        # Verifico la longitud de la lista.
        if len(lista) <= 3:

            # Ordeno
            dato = [
                lista[0][0], lista[0][0] + 1, lista[-1][0], lista[-1][0] + 1
            ]

        else:

            # Ordeno.
            dato = [lista[0][0], lista[1][0], lista[2][0], lista[3][0]]

        # Retorno.
        return respuesta(True, dato=dato)





