# -*- coding: iso-8859-1 -*-
from at.orm.decorators import DBSwitch

from at.map.utiles.validaciones_especificas import ValidacionesEspecificas
from at.map.utiles.punto import Punto
from at.map import _

from math import sqrt, cos, sin, atan


@DBSwitch('gis')
class Referencia(ValidacionesEspecificas):

    def obtener_referencias(self, id_mapa):
        '''
            Dado el ID de un mapa obtengo las referencias del mismo.

            Args:
                id_mapa (int)

            Returns:
                referencias (list[dict{}])

            Raises:
                'No se han encontrado referencias'

            Functions:
                validar_id_mapa(id_mapa)
                binded_session()
                tabla('referencias')
        '''

        if not self.validar_id_mapa(id_mapa):
            return self.validar_id_mapa(id_mapa)

        session = self.gis.binded_session()
        tabla_referencias = self.gis.tabla('referencias')

        q_obj = session.query(tabla_referencias)
        q_obj = q_obj.filter(tabla_referencias.idMapa == id_mapa)

        referencias = q_obj.all()

        if int(q_obj.count()) == 0:
            session.close()
            raise Exception, _('No se han encontrado referencias.')

        lista_referencias = []

        for referencia in referencias:
            r = Punto(  x = int(referencia.X),
                        y = int(referencia.Y),
                        latitud = float(referencia.Latitud),
                        longitud = float(referencia.Longitud))
            lista_referencias.append(r)

        session.close()
        return lista_referencias

    def obtener_referencias_cercanas_xy(self, punto, referencias = None):
        '''
            Dado el ID de un mapa, X e Y obtengo las referencias mas cercanas
            del mismo.

            Args:
                punto (Punto)
                referencias: Opcional (Lista de referencias)
            Returns:
                referencias (list[Punto])

            Raises:
                'No se han encontrado referencias'

            Functions:
                validar_id_mapa(id_mapa)
                validar_no_vacio(x)
                validar_entero(x)

                obtener_referencias(id_mapa)
        '''

        if not isinstance(punto, Punto):
            raise ValueError, _('Debe ingresar un Punto, obtuve %s.' % punto)

        x = punto.x
        y = punto.y

        if not self.validar_entero(x):
            return self.validar_entero(x)
        if not self.validar_no_vacio(x):
            return self.validar_entero(x)

        if not self.validar_entero(y):
            return self.validar_entero(y)
        if not self.validar_no_vacio(y):
            return self.validar_entero(y)

        if not self.validar_id_mapa(punto.id_mapa):
            return self.validar_id_mapa(punto.id_mapa)

        if referencias == None:
            referencias = self.obtener_referencias(punto.id_mapa)

        if len(referencias) < 2:
            return None

        distancia1 = sqrt((referencias[0].x - x) ** 2 + (referencias[0].y - y)
                                                                        ** 2)
        distancia2 = sqrt((referencias[1].x - x) ** 2 + (referencias[0].y - y)
                                                                        ** 2)


        if distancia1 <= distancia2:
            indice_referencia1 = 0
            indice_referencia2 = 1
        else:
            indice_referencia1 = 1
            indice_referencia2 = 0
            distancia_temp = distancia2
            distancia2 = distancia1
            distancia1 = distancia_temp

        for i in range(2,len(referencias)):
            distancia_temp = sqrt((referencias[i].x - x) ** 2 +
                                                    (referencias[i].y - y) ** 2)
            if distancia_temp < distancia1:
                indice_referencia2 = indice_referencia1
                distancia2 = distancia1
                indice_referencia1 = i
                distancia1 = distancia_temp
            elif distancia_temp < distancia2:
                indice_referencia2 = i
                distancia2 = distancia_temp

        return [referencias[indice_referencia1],referencias[indice_referencia2]]



    def obtener_referencias_cercanas_latlon(self, punto, referencias = None):
        '''
        Dado el ID de un mapa, latitud y longitud obtengo las referencias mas
        cercanas  del mismo.

        Args:
            punto (Punto)
            referencias: Opcional (Lista de referencias)
        Returns:
            referencias (list[Punto])

        Raises:
            'No se han encontrado referencias'

        Functions:
            validar_id_mapa(id_mapa)
            validar_no_vacio(latitud)
            validar_numero(latitud)
            obtener_referencias(id_mapa)
        '''

        if punto == None:
            raise Exception, _('Debe ingresar un Punto o X e Y.')
        else:
            if not isinstance(punto, Punto):
                raise ValueError, _('Debe ingresar un Punto, \
            obtuve %s.' % punto)

        latitud = punto.latitud
        longitud = punto.longitud

        if not self.validar_numero(latitud):
            return self.validar_entero(latitud)
        if not self.validar_no_vacio(latitud):
            return self.validar_entero(latitud)

        if not self.validar_numero(longitud):
            return self.validar_entero(longitud)
        if not self.validar_no_vacio(longitud):
            return self.validar_entero(longitud)

        if not self.validar_id_mapa(punto.id_mapa):
            return self.validar_id_mapa(punto.id_mapa)

        if referencias == None:
            referencias = self.obtener_referencias(punto.id_mapa)

        if len(referencias) < 2:
            return None

        latitud = round(((-latitud / 60 * 1000) - 1959925) * 1856 / 1000)
        longitud = round(((-longitud / 60 * 1000) - 3619640) * 1556 / 1000)


        distancia1 = sqrt((round(((-referencias[0].longitud / 60 * 1000)
                                    - 3619640) * 1556 / 1000) - longitud) ** 2 +
                (round(((-referencias[0].latitud / 60 * 1000) - 1959925)
                                    * 1856 / 1000) - latitud) ** 2)
        distancia2 = sqrt((round(((-referencias[1].longitud / 60 * 1000)
                                    - 3619640) * 1556 / 1000) - longitud) ** 2 +
                (round(((-referencias[1].latitud / 60 * 1000) - 1959925)
                                                * 1856 / 1000) - latitud) ** 2)
        distancia1 = sqrt((referencias[0].longitud - longitud) ** 2 +
                                        (referencias[0].latitud - latitud) ** 2)
        distancia2 = sqrt((referencias[1].longitud - longitud) ** 2 +
                                        (referencias[1].latitud - latitud) ** 2)

        if distancia1 <= distancia2 :
            indice_referencia1 = 0
            indice_referencia2 = 1
        else:
            indice_referencia1 = 1
            indice_referencia2 = 0
            distancia_temp = distancia2
            distancia2 = distancia1
            distancia1 = distancia_temp

        for i in range(2, len(referencias)):
            distancia_temp = sqrt((round(((-referencias[i].longitud / 60 * 1000)
                                    - 3619640) * 1556 / 1000) - longitud) ** 2 +
                                    (round(((-referencias[i].latitud / 60 * 1000)
                                    - 1959925) * 1856 / 1000) - latitud) ** 2)
            if distancia_temp < distancia1 :
                indice_referencia2 = indice_referencia1
                distancia2 = distancia1
                indice_referencia1 = i
                distancia1 = distancia_temp
            elif distancia_temp < distancia2 :
                indice_referencia2 = i
                distancia2 = distancia_temp

        return [referencias[indice_referencia1],referencias[indice_referencia2]]

    def obtener_referencias_cercanas(self, punto, referencias = None):
        '''
            Dado un punto la funcion decide si obtener la referencia mas cercana
            utilizando X e Y o Latitud y Longitud

            Args:
                punto (Punto)
                referencias: Opcional (Lista de referencias)
            Returns:
                 referencias (list[Punto])

            Raises:
                'Debe ingresar un Punto'

            Functions:
                obtener_referencias_cercanas_xy(punto)
                obtener_referencias_cercanas_latlon(punto)

        '''

        if punto.x != None and punto.y != None:
            return self.obtener_referencias_cercanas_xy(punto, referencias)
        elif punto.latitud != None and punto.longitud != None:
            return self.obtener_referencias_cercanas_latlon(punto, referencias)
        else:
            raise Exception, _('Debe ingresar un punto con X:Y o Latitud:Longitud.')


    def de_xy_a_segundos(self, punto, referencias = None):
        '''
            Dado el ID de un mapa, X e Y obtengo las coordenadas correctas
            que corresponden. Para esto utilizamos las referencias mas cercanas
            para corregir cualquier error.

            Args:
                punto (Punto)
                referencias: Opcional (Lista de referencias)
            Returns:
                punto (Punto)

            Raises:
                'Debe ingresar un Punto'

            Functions:
                validar_id_mapa(id_mapa)
                validar_no_vacio(x)
                validar_entero(x)
                obtener_referencias_cercanas(punto)
        '''
        if not isinstance(punto, Punto):
            raise ValueError, _('Debe ingresar un Punto, obtuve %s.' % punto)

        x = punto.x
        y = punto.y

        if not self.validar_entero(x):
            return self.validar_entero(x)
        if not self.validar_no_vacio(x):
            return self.validar_entero(x)

        if not self.validar_entero(y):
            return self.validar_entero(y)
        if not self.validar_no_vacio(y):
            return self.validar_entero(y)

        if not self.validar_id_mapa(punto.id_mapa):
            return self.validar_id_mapa(punto.id_mapa)

        PI = 3.14159265358979

        #print 'referencia en de_segundos_a_xy ', referencias, '\n'
        arrPtos = self.obtener_referencias_cercanas(punto, referencias)

        if arrPtos == None:
            raise Exception, _(u'No hay referencias cercanas suficientes para \
        realizar el c?lculo.')

        Base1x = arrPtos[0].x
        Base1y = arrPtos[0].y
        Base2x = arrPtos[1].x
        Base2y = arrPtos[1].y
        Base1Lati = arrPtos[0].latitud
        Base1Longi = arrPtos[0].longitud
        Base2Lati = arrPtos[1].latitud
        Base2Longi = arrPtos[1].longitud

        Base1Longi = round(((Base1Longi*-1 / 60 * 1000) - 3619640) * 1556 / 1000)
        Base1Lati = round(((Base1Lati*-1 / 60 * 1000) - 1959925) * 1856 / 1000)
        Base2Longi = round(((Base2Longi*-1 / 60 * 1000) - 3619640) * 1556 / 1000)
        Base2Lati = round(((Base2Lati*-1 / 60 * 1000) - 1959925) * 1856 / 1000)

        pdeltax = float(Base2x - Base1x)
        pdeltay = float((Base2y - Base1y)*-1)
        deltax = float(Base2Longi - Base1Longi)
        deltay = float(Base2Lati - Base1Lati)
        dx = float(x - Base1x)
        dy = float(y - Base1y)*-1


        if pdeltax == 0:
            if pdeltay < 0:
                alfa = PI * 1.5
            else:
                alfa = PI * 0.5
        else:
            if pdeltax < 0:
                alfa = PI + atan(pdeltay / pdeltax)
            else:
                if pdeltay < 0:
                    alfa = PI * 2 + atan(pdeltay / pdeltax)
                else:
                    alfa = atan(pdeltay / pdeltax)

        if dx == 0:
            if dy == 0:
                punto.latitud = Base1Lati
                punto.longitud = Base1Longi
                return punto

            if dy < 0:
                alfaprima = PI * 1.5
            else:
                alfaprima = PI * 0.5
        else:
            if dx < 0:
                alfaprima = PI + atan(dy / dx)
            else:
                if dy < 0:
                    alfaprima = PI * 2 + atan(dy / dx)
                else:
                    alfaprima = atan(dy / dx)

        if deltax == 0:
            if deltay < 0:
                beta = PI * 1.5
            else:
                beta = PI * 0.5

        else:
            if deltax < 0:
                beta = PI + atan(deltay / deltax)
            else:
                if deltay < 0:
                    beta = PI * 2 + atan(deltay / deltax)
                else:
                    beta = atan(deltay / deltax)

        tita = alfa - beta
        deltaxprima = float( sqrt((pdeltax * pdeltax) + (pdeltay * pdeltay))
                                                            * cos(alfa - tita))
        deltayprima = float(sqrt((pdeltax * pdeltax) + (pdeltay * pdeltay))
                                                            * sin(alfa - tita))
        factorx = deltax / deltaxprima
        factory = deltay / deltayprima
        dblLat = round(Base1Lati + sqrt(dx * dx + dy * dy)
                                            * sin(alfaprima - tita) * factorx)
        dblLong = round(Base1Longi + sqrt(dx * dx + dy * dy)
                                            * cos(alfaprima - tita) * factory)

        punto.latitud  = ((dblLat * 1000 / 1856) + 1959925) / 1000 * 60 * (-1)
        punto.longitud  = ((dblLong * 1000 / 1556) + 3619640) / 1000 * 60 * (-1)

        return  punto


    def de_segundos_a_xy(self, punto, referencias = None):
        '''
            Dado el ID de un mapa, Latitud y Longitud obtengo las coordenadas X
            e Y que corresponden. Para esto utilizamos las referencias mas
            cercanas para corregir cualquier error.

            Args:
                punto (Punto)
                referencias: Opcional (Lista de referencias)
            Returns:
                punto (Punto)

            Raises:
                'Debe ingresar un Punto'

            Functions:
                validar_id_mapa(id_mapa)
                validar_no_vacio(latitud)
                validar_numero(latitud)
                obtener_referencias_cercanas(punto)
        '''
        if not isinstance(punto, Punto):
            raise ValueError, _('Debe ingresar un Punto, obtuve %s.' % punto)

        latitud = punto.latitud
        longitud = punto.longitud

        if not self.validar_numero(latitud):
            return self.validar_numero(latitud)
        if not self.validar_no_vacio(latitud):
            return self.validar_entero(latitud)

        if not self.validar_numero(longitud):
            return self.validar_numero(longitud)
        if not self.validar_no_vacio(longitud):
            return self.validar_entero(longitud)

        if not self.validar_id_mapa(punto.id_mapa):
            return self.validar_id_mapa(punto.id_mapa)

        PI = 3.14159265358979

        arrPtos = self.obtener_referencias_cercanas(punto, referencias)

        if arrPtos == None:
            raise Exception, _(u'No hay referencias cercanas suficientes para \
        realizar el c?lculo.')

        Base1RefLongi = arrPtos[0].longitud
        Base1RefLati = arrPtos[0].latitud
        Base2RefLongi = arrPtos[1].longitud
        Base2RefLati = arrPtos[1].latitud
        Base1RefX = arrPtos[0].x
        Base1RefY = arrPtos[0].y
        Base2RefX = arrPtos[1].x
        Base2RefY = arrPtos[1].y

        Base1RefLongi = round(((-Base1RefLongi / 60 * 1000) - 3619640)
                                                                * 1556 / 1000)
        Base1RefLati = round(((-Base1RefLati / 60 * 1000) - 1959925)
                                                                * 1856 / 1000)
        Base2RefLongi = round(((-Base2RefLongi / 60 * 1000) - 3619640)
                                                                * 1556 / 1000)
        Base2RefLati = round(((-Base2RefLati / 60 * 1000) - 1959925)
                                                                * 1856 / 1000)

        longitud = round(((-longitud / 60 * 1000) - 3619640) * 1556 / 1000)
        latitud = round(((-latitud / 60 * 1000) - 1959925) * 1856 / 1000)

        pdeltax = float(Base2RefLongi - Base1RefLongi)
        pdeltay = float(-(Base2RefLati - Base1RefLati))
        deltax = float(Base2RefX - Base1RefX)
        deltay = float(Base2RefY - Base1RefY)
        dx = float((longitud - Base1RefLongi))
        dy = float(-(latitud - Base1RefLati))

        if pdeltax == 0 :
            if pdeltay < 0 :
                alfa = PI * 1.5
            else:
                alfa = PI * 0.5

        else:
            if pdeltax < 0 :
                alfa = PI + atan(pdeltay / pdeltax)
            else:
                if pdeltay < 0 :
                    alfa = PI * 2 + atan(pdeltay / pdeltax)
                else:
                    alfa = atan(pdeltay / pdeltax)

        if dx == 0 :
            if dy == 0 :
                punto.y = Base1RefY
                punto.x = Base1RefX
                return  punto
            else:
                if dy < 0 :
                    alfaprima = PI * 1.5
                else:
                    alfaprima = PI * 0.5

        else:
            if dx < 0 :
                alfaprima = PI + atan(dy / dx)
            else:
                if dy < 0 :
                    alfaprima = PI * 2 + atan(dy / dx)
                else:
                    alfaprima = atan(dy / dx)

        if deltax == 0 :
            if deltay < 0 :
                beta = PI * 1.5
            else:
                beta = PI * 0.5

        else:
            if deltax < 0 :
                beta = PI + atan(deltay / deltax)
            else:
                if deltay < 0 :
                    beta = PI * 2 + atan(deltay / deltax)
                else:
                    beta = atan(deltay / deltax)


        tita = alfa - beta
        deltaxprima = float(sqrt((pdeltax * pdeltax) + (pdeltay * pdeltay))
                                                            * cos(alfa - tita))
        deltayprima = float(sqrt((pdeltax * pdeltax) + (pdeltay * pdeltay))
                                                            * sin(alfa - tita))
        factorx = deltax / deltaxprima
        factory = deltay / deltayprima
        punto.y = int(round(Base1RefY + sqrt(dx * dx + dy * dy)
                                        * sin(alfaprima - tita) * factorx, 0))
        punto.x = int(round(Base1RefX + sqrt(dx * dx + dy * dy)
                                        * cos(alfaprima - tita) * factory, 0))

        return  punto
