# -*- coding: utf-8 -*-

import unittest
from at.map.utiles.funciones_geo import FuncionesGeo
from at.map import _


class TestFuncionesGeo(unittest.TestCase):

    def setUp(self):
        self.f_geo = FuncionesGeo()

    def tearDown(self):
        pass

    def test_km_entre_2_puntos(self):
        latitud1 = -118570.661637931
        longitud1 = -218354.801028278
        latitud2 = -118571.631465517
        longitud2 = -218349.86529563
        r = self.f_geo.km_entre_2_puntos(
            latitud1, longitud1, latitud2, longitud2
        )

        self.assertTrue(r == 0.12)

        r = self.f_geo.kms_entre_puntos(
            latitud1 / 3600, longitud1 / 3600,
            latitud2 / 3600, longitud2 / 3600
        )

    def test_obtener_rumbo(self):
        latitud_origen = 1231
        longitud_origen = 4561
        latitud_destino = 1232
        longitud_destino = 4562
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == _('YA LLEGO'))

        latitud_origen = 1
        longitud_origen = 4
        latitud_destino = 1
        longitud_destino = 7
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'E')

        latitud_origen = 5
        longitud_origen = 2
        latitud_destino = 9
        longitud_destino = 2
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'N')

        latitud_origen = 9
        longitud_origen = 2
        latitud_destino = 5
        longitud_destino = 2
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'S')

        latitud_origen = 6
        longitud_origen = 8
        latitud_destino = 6
        longitud_destino = 2
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'O')

        latitud_origen = 3
        longitud_origen = 4
        latitud_destino = 7
        longitud_destino = 8
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'NE')

        latitud_origen = 8
        longitud_origen = 7
        latitud_destino = 3
        longitud_destino = 4
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'SO')

        latitud_origen = 2
        longitud_origen = 3
        latitud_destino = 4
        longitud_destino = 1
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'NO')

        latitud_origen = 5
        longitud_origen = 2
        latitud_destino = 3
        longitud_destino = 8
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'SE')

        latitud_origen = -32.940851 * 60 * 60
        longitud_origen = -60.690646 * 60 * 60
        latitud_destino = -32.937985 * 60 * 60
        longitud_destino = -60.690304 * 60 * 60
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'N')

        latitud_origen = -32.940851 * 60 * 60
        longitud_origen = -60.690646 * 60 * 60
        latitud_destino = -32.944254 * 60 * 60
        longitud_destino = -60.686193 * 60 * 60
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'SE')

        latitud_origen = -32.940851 * 60 * 60
        longitud_origen = -60.690646 * 60 * 60
        latitud_destino = -32.941706 * 60 * 60
        longitud_destino = -60.683307 * 60 * 60
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'E')

        latitud_origen = -32.940851 * 60 * 60
        longitud_origen = -60.690646 * 60 * 60
        latitud_destino = -32.942165 * 60 * 60
        longitud_destino = -60.691246 * 60 * 60
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'S')

        latitud_origen = -32.940851 * 60 * 60
        longitud_origen = -60.690646 * 60 * 60
        latitud_destino = -32.940527 * 60 * 60
        longitud_destino = -60.692266 * 60 * 60
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'O')

        latitud_origen = -32.940851 * 60 * 60
        longitud_origen = -60.690646 * 60 * 60
        latitud_destino = -32.934107 * 60 * 60
        longitud_destino = -60.697362 * 60 * 60
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'NO')

        latitud_origen = -32.940851 * 60 * 60
        longitud_origen = -60.690646 * 60 * 60
        latitud_destino = -32.947586 * 60 * 60
        longitud_destino = -60.697362 * 60 * 60
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'SO')

        latitud_origen = -32.940851 * 60 * 60
        longitud_origen = -60.690646 * 60 * 60
        latitud_destino = -32.936547 * 60 * 60
        longitud_destino = -60.680249 * 60 * 60
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'NE')

        latitud_origen = -118649.772
        longitud_origen = -218337.229
        latitud_destino = -118684.713362
        longitud_destino = -218389.158355
        r = self.f_geo.obtener_rumbo(latitud_origen, longitud_origen,
                                latitud_destino, longitud_destino)
        self.assertTrue(r == 'SO')

    def test_punto_en_geometria(self):
        punto = [1, 1]
        geometria = [[-2, -2], [1, 1], [2, 0]]
        self.assertTrue(self.f_geo.punto_en_geometria(punto, geometria))
        geometria = [[-2, -2], [0.999, 1], [2, 0]]
        self.assertFalse(self.f_geo.punto_en_geometria(punto, geometria))
        geometria = [[-2, -2], [-2, 2], [2, 2], [-2, 0], [2, -2]]
        self.assertFalse(self.f_geo.punto_en_geometria(punto, geometria))
        geometria = [[-2, -2], [-2, 2], [2, 2], [0, 0], [2, -2]]
        self.assertTrue(self.f_geo.punto_en_geometria(punto, geometria))

    def test_obtener_diferencia_entre_geometrias(self):
        geometria1 = [[-2, -2], [-2, 2], [2, 2], [0, 0], [9, -2]]
        geometria2 = [[-2, -2], [-2, 3.525], [4, 2], [0, 0], [2, -2]]
        diferencia1 = self.f_geo.obtener_diferencia_entre_geometrias(
            geometria1, geometria2
        )
        # Agregamos un punto mas, como acercandonos
        geometria2.append([8.5, -1.5])
        diferencia2 = self.f_geo.obtener_diferencia_entre_geometrias(
            geometria1, geometria2
        )
        # Validamos que estamos mas cerca
        self.assertTrue(diferencia2 < diferencia1)
        # Nos acercamos mas todavia
        geometria1.append([8, -1])
        diferencia3 = self.f_geo.obtener_diferencia_entre_geometrias(
            geometria1, geometria2
        )
        # Validamos que estamos mas cerca todavia
        self.assertTrue(diferencia3 < diferencia2)

    def test_obtener_extremos_linea(self):
        puntos = [[-32.9484701157, -60.7122436166], [-32.94742, -60.71105]]
        rta = self.f_geo.obtener_extremos_linea(puntos)
        self.assertTrue(rta['Estado'])


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestFuncionesGeo))
    return suite