# -*- coding: utf-8 -*-
import os
from at.map import RUTA_MAPAS
from at.map import TILES_POR_LADO
from at.map import PIXELES_TILE

from at.utiles.validaciones_generales import ValidacionesGenerales


try:
    if not os.path.exists(RUTA_MAPAS):
        os.makedirs(RUTA_MAPAS)
except:
    print ':: error: No hay ruta de mapas: %s' % RUTA_MAPAS


class Tile (ValidacionesGenerales):
    """
    Clase para manipular los tiles, con funciones basicas de los mismos.
    """

    def __init__(self, layer, x, y, z):
        """ Inicializamos un tile de un layer con sus datos correspondientes
        """

        # Validamos
        self.validar_entero(x)
        self.validar_entero(y)
        self.validar_entero(z)

        # Si no fallamos, guardamos
        self.layer = layer
        self.x = x
        self.y = y
        self.z = z
        self.data = None


    def __str__(self):

        return "[%s-%s-%s]" % (self.x, self.y, self.z)


    def existe_imagen(self, basico=False):
        '''
        Esta funcion devuelve si existe o no una imagen

        Params:
            basico (bool)

        Return:
            bool
        '''

        if basico:
            ruta_imagen = os.path.join(
                RUTA_MAPAS,
                str(self.z), str(self.x), str(self.y) + '.jpeg',
            )

        else:
            ruta_imagen = os.path.join(
                RUTA_MAPAS,
                'z' + str(self.z),
                'x' + str(self.x),
                'y' + str(self.y) + '.jpeg',
            )

        # Devolvemos si existe o no la imagen
        return os.path.exists(ruta_imagen)


    def obtener_limites(self):
        """
        Obtenemos los limites en lat y lon del tile en el mapa
        """

        res = self.layer.resolutions[self.z]

        # Obtenemos el minX
        min_lon = self.layer.bbox[0] + (res * self.x * PIXELES_TILE)
        min_lat = self.layer.bbox[1] + (res * self.y * PIXELES_TILE)

        # Pareseamos el minimo
        max_lon = self.layer.bbox[0] + (res * (self.x + 1) * PIXELES_TILE)
        max_lat = self.layer.bbox[1] + (res * (self.y + 1) * PIXELES_TILE)

        # Devolvemos
        return (min_lon, min_lat, max_lon, max_lat)


    def obtener_bbox(self):
        """
        Obtenemos el bonundib box que recorre todo el tile

        """

        return ",".join(map(str, self.obtener_limites()))


    def obtener_imagen(self):
        '''
        Esta funcion devuelve, si es que existe, los datos de la imagen
        generada en el servidor.

        Params:
            x, y, z (int, int, int)

        Return:
            data (binary)
            None

        '''

        ruta_imagen = os.path.join(
            RUTA_MAPAS,
            'z' + str(self.z),
            'x' + str(self.x),
            'y' + str(self.y) + '.jpeg',
        )

        # Si no existe la imagen, devolvemos None
        if not os.path.exists(ruta_imagen):
            # Buscamos el formato basico
            ruta_imagen = os.path.join(
                RUTA_MAPAS,
                str(self.z), str(self.x), str(self.y) + '.jpeg',
            )

            # Si no existe la imagen, devolvemos None
            if not os.path.exists(ruta_imagen):
                return None

        # Sino abrimos la imagen y enviamos los datos
        a = open(ruta_imagen, 'r')
        data = a.read()
        a.close()
        return data


    def obtener_padre(self, tiles_por_lado=TILES_POR_LADO):
        '''
        Funcion que devuelve el x e y del tile padre de un sector de
        8 x 8 tiles es decir, de 64 tiles.

        Params:
            x, y (int, int)

        Returns:
            x, y (int, int)
        '''

        self.validar_entero(tiles_por_lado)

        # Creamos nuevos enteros
        x_tmp = int(self.x)
        y_tmp = int(self.y)

        # Los modificamos para poder obtener los datos del padre
        for i in range(tiles_por_lado * 2):
            # Cuando llegamos a un punto
            if x_tmp % tiles_por_lado == 0:
                break
            # Reducimos
            x_tmp -= 1

        # Los modificamos para poder obtener los datos del padre
        for i in range(tiles_por_lado * 2):
            # Cuando llegamos a un punto
            if y_tmp % tiles_por_lado == 0:
                break
            # Reducimos
            y_tmp -= 1

        # Devolvemos
        return Tile(self.layer, x_tmp, y_tmp, self.z)