# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import or_
from sqlalchemy.sql.expression import func
from sqlalchemy.sql.expression import not_
from sqlalchemy import text

from at.map import _
from at.map.utiles.funciones_geo import FuncionesGeo
from at.map.ciudad import CiudadPostgres
from at.map.provincia import ProvinciaPostgres
from at.orm.decorators import DBSwitch
from at.utiles.conversiones import respuesta


@DBSwitch('gis2')
class Topologia(object):
    def __init__(self):
        self.proyeccion = 4326
        self.funciones_geo = FuncionesGeo()
        self.ciudad_postgres = CiudadPostgres()
        self.provincia_postgres = ProvinciaPostgres()

    def multiple(self, valor, multiple):
        """
        Funcion para calcular si el numero es multiplo
        utilizando el modulo de la division
        """
        resto = valor % multiple
        if resto == 0:
            return True
        else:
            return False

    def encontrar_nodo(self, punto):

        if punto is None:
            return respuesta(False)

        # Abro la sesión con la BD con gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles_vertices_pgr
        calles_vertices_pgr = self.gis2.tabla('calles_vertices_pgr')

        # Busco el id del nodo.
        q_obj_1 = session.query(calles_vertices_pgr.id)

        # Filtro.
        q_obj_1 = q_obj_1.filter(
            and_(
                func.st_dwithin(
                    calles_vertices_pgr.the_geom,
                    punto,
                    0.000009,
                    use_spheroid=True
                )
            )
        )

        q_obj_1 = q_obj_1.limit(1)

        # Obtengo el registro.
        registro = q_obj_1.first()

        # Cierro sesión.
        session.close()

        # Verifico.
        if registro is None:
            return respuesta(False)

        # Obtengo el id del nodo.
        id_nodo = registro.id

        return respuesta(True, _(u'Se econtró el nodo.'), id_nodo)

    def obtener_velocidad_maxima(self, id_tipo_segmento):

        # Velocidad por defecto en KM.
        velocidad = 40

        # Boulevar.
        if id_tipo_segmento == 4:
            return 60

        # Avenida secundaria.
        if id_tipo_segmento == 5:
            return 60

        # Avenida RP.
        if id_tipo_segmento == 6:
            return 110

        # Autopista Secundaria RN.
        if id_tipo_segmento == 7:
            return 120

        # Autopista.
        if id_tipo_segmento == 8:
            return 130

        # Rampa alta velocidad.
        if id_tipo_segmento == 10:
            return 60

        # Peatonal.
        if id_tipo_segmento == 13:
            return 0

        # Escalera.
        if id_tipo_segmento == 14:
            return 0

        # Si no es ninguna de las opciones retorno la velocidad por defecto.
        return velocidad

    def generar_red(self, id_ciudad=None):

        ciudades = []

        if id_ciudad is None:

            id_pais = 12

            rta_provincias = self.provincia_postgres.listar_provincias(id_pais)
            if rta_provincias['Estado'] is not True:
                return rta_provincias

            # Obtengo las provincias.
            provincias = rta_provincias['Respuesta']

            for provincia in provincias:
                id_provincia = provincia[0]

                rta_ciudades = self.ciudad_postgres.listar_ciudades(id_provincia)
                if rta_ciudades['Estado'] is not True:
                    return rta_ciudades

                ciudades_tmp = rta_ciudades['Respuesta']

                ciudades += ciudades_tmp

        else:

            rta_ciudad = self.ciudad_postgres.obtener_ciudad(id_ciudad)
            if rta_ciudad['Estado'] is not True:
                return rta_ciudad

            ciudad_tmp = rta_ciudad['Respuesta']

            ciudades = [[ciudad_tmp['Id'], ciudad_tmp['Nombre']]]

        cantidad_de_ciudades = len(ciudades)
        contador_ciudad = 0

        try:

            f = open('ciudades_procesadas.txt', 'r')
            f.close()

        except Exception:

            f = open('ciudades_procesadas.txt', 'w')
            f.close()

        for ciudad in ciudades:

            contador_ciudad += 1

            id_ciudad = ciudad[0]

            f = open('ciudades_procesadas.txt', 'r')

            ciudades_procesadas = f.readlines()

            f.close()

            if str(id_ciudad) + '\n' in ciudades_procesadas:

                continue

            f = open('ciudades_procesadas.txt', 'a')
            f.write(str(id_ciudad) + '\n')
            f.close()

            nombre_ciudad = ciudad[1]

            # Abro la sesión con la BD con gis2
            session = self.gis2.binded_session()

            # Obtengo la tabla calles y calles_nombre
            calles = self.gis2.tabla('calles')
            calles_nombres = self.gis2.tabla('calles_nombres')

            campo_source = func.ST_StartPoint(
                func.ST_LineMerge(calles.geom)
            ).label('source_geom')
            campo_target = func.ST_EndPoint(
                func.ST_LineMerge(calles.geom)
            ).label('target_geom')

            # Obtengo los origenes y destinos.
            campo_length = func.st_length(calles.geom, True).label('length')

            # campo_cost_len = length.label('cost_len')
            #
            # campo_rcost_len = length.label('rcost_len')

            # Busco el punto inicial de cada segmento.
            q_obj_1 = session.query(
                calles.gid,
                calles.sentido,
                campo_source,
                campo_target,
                calles.geom,
                calles.id_tipo,
                campo_length
            )

            # Filtro por ciudad.
            q_obj_1 = q_obj_1.join((
                calles_nombres,
                and_(
                    calles.id_calle_nombre == calles_nombres.id,
                    calles_nombres.id_ciudad == id_ciudad
                )
            ))

            # Filtro.
            q_obj_1 = q_obj_1.filter(
                and_(
                    calles.source == 0,
                    calles.target == 0
                )
            )

            # Obtengo los registros.
            registros = q_obj_1.all()

            cantidad_de_registros = len(registros)

            inicial = 0

            # Recorro los registros.
            for registro in registros:

                # Obtengo los datos solicitados.
                source_geom = registro.source_geom
                target_geom = registro.target_geom
                sentido = registro.sentido
                length = round(registro.length, 6)
                id_tipo = registro.id_tipo

                # Trato de econtrar el nodo origen.
                rta_source = self.encontrar_nodo(source_geom)

                if rta_source['Estado'] is True:
                    id_source = rta_source['Dato']
                else:
                    # Declaro el dic con los datos.
                    datos_source = {
                        'the_geom': source_geom
                    }

                    # agrego los nodos
                    agregar_nodo = self.gis2.insertar(
                        'calles_vertices_pgr', 'id', **datos_source
                    )

                    if agregar_nodo['Estado'] is not True:
                        return agregar_nodo

                    # Obtengo el número de id.
                    id_source = agregar_nodo['ValorCampo']

                rta_target = self.encontrar_nodo(target_geom)

                if rta_target['Estado'] is True:
                    id_target = rta_target['Dato']
                else:
                    # Declaro el dic con los datos.
                    datos_target = {
                        'the_geom': target_geom
                    }

                    # agrego los nodos
                    agregar_nodo = self.gis2.insertar(
                        'calles_vertices_pgr', 'id', **datos_target
                    )

                    if agregar_nodo['Estado'] is not True:
                        return agregar_nodo

                    # Obtengo el número de id.
                    id_target = agregar_nodo['ValorCampo']

                # Obtengo l
                max_speed_km = self.obtener_velocidad_maxima(id_tipo)

                # Calculo la longitud en kilometros.
                len_km = length / 1000

                len_km = round(len_km, 6)

                # Armo.
                datos_calle = {
                    'source': id_source,
                    'target': id_target,
                    'cost_len': length,
                    'rcost_len': length,
                    'len_km': len_km,
                    'max_speed_km': max_speed_km
                }

                if max_speed_km != 0:
                    # Calculo el tiempo.
                    cost_time = (len_km / max_speed_km) * 3600

                    datos_calle['cost_time'] = cost_time
                    datos_calle['rcost_time'] = cost_time

                # Verifico el sentido de la calle.
                if sentido == 1:
                    datos_calle['one_way'] = 'FT'
                else:
                    datos_calle['one_way'] = 'B'

                q_obj = session.query(
                    calles.gid,
                )

                # Filtro por el id.
                q_obj = q_obj.filter(calles.gid == registro.gid)

                # modifico el prestamo.
                update = q_obj.update(datos_calle)

                # Verifico si se realizó la modificación.
                if update == 0:

                    # Cierro la sesión.
                    session.close()

                    # Retorno.
                    return self.rta(
                        False, _(u'No se pudo actualizar el prestamo.')
                    )

                # Ejecuto la modificación.
                session.commit()

                inicial += 1

            session.close()

        return respuesta(True, _(u'Se enlazaron correctamente los nodos.'))

