# -*- coding: utf-8 *-*

from at.orm.config import databases
from at.orm.abm import ABM
from at.orm.singleton import Singleton
import sys


class DBSwitch(object):

    __shared = {}
    sgt = Singleton()

    def __init__(self, *args, **kwargs):
        # Si no estamos en multiprocessing, vamos a
        # utilizar el shared para guardar referencia a las dbs
        if not self.sgt.multiprocessing:
            self.__dict__ = self.__shared

        # GUardamos las dbs
        self.dbs = []
        for arg in args:

            self.dbs.append(str(arg))
        #self.db = db

    def __call__(self, clase):

        if self.sgt.multiprocessing:
            # Si no se especifica, le agregamos todas las conexiones. es raro siempre le decimos que bd es
            if len(self.dbs) == 0:

                # Recorremos todas las bases de datos
                for database in databases:
                    # Seteamos el argumento en la clase
                    setattr(
                        clase, database['name'], ABM(database)
                    )

                return clase

            # Sino, buscamos la base de datos especificada
            for database in databases:
                # Si coincide, la agregamos
                for db in self.dbs:
                    if db == database['name']:
                       # Seteamos el argumento en la clase
                        setattr(
                            clase, database['name'], ABM(database)
                        )
            # Devolvemos la clase
            return clase

        else:
            # Si no se especifica, le agregamos todas las conexiones
            if len(self.dbs) == 0:
                # Recorremos todas las bases de datos
                for database in databases:
                    # Si no esta creado el abm en el singleton, lo creamos
                    if self.sgt.abms.get(database['name']) is None:
                        self.sgt.abms[database['name']] = ABM(database)

                    # Seteamos el argumento en la clase
                    setattr(
                        clase, database['name'], self.sgt.abms[database['name']]
                    )
                return clase

            # Sino, buscamos la base de datos especificada
            for database in databases:
                # Si conincide, la agregamos

                for db in self.dbs:
                    if db == database['name']:
                        # Si no esta creado el abm en el singleton, lo creamos
                        if self.sgt.abms.get(database['name']) is None:
                            self.sgt.abms[database['name']] = ABM(database)
                        # Seteamos el argumento en la clase
                        setattr(
                            clase, database['name'], self.sgt.abms[database['name']]
                        )
            # Devolvemos la clase
            return clase


def db_switch(objeto, db=None):

    # Validamos que no este vacio
    if objeto is None:
        print('ERROR: Debe especificar un objeto no vacio!!!')
        sys.exit(1)
        return

    # Si esta vacia, vamos a buscar la base de datos de airtrack
    if db is None:
        db = 'default'

    # Validamos que la db no este vacia
    db = str(db).strip()
    if db == '':
        print('ERROR: Debe especificar una bd no vacia!!!')
        sys.exit(1)
        return

    # Instanciamos el singleton
    sgt = Singleton()

    if db == 'default':
        # Creamos el default
        abm_default = ABM({})

        # Si no estamos en multiprocessing, vamos a
        # cargar el dato en el singleton
        if not sgt.multiprocessing:
            # Si no esta creado el abm en el singleton, lo creamos
            if sgt.abms.get('default') is None:
                # Pasamos al mapper un diccionario vacio. El se va a
                # conectar a la base por defecto
                sgt.abms['default'] = abm_default

        # Seteamos el argumento en la clase
        setattr(objeto, 'default', abm_default)
        return True

    # Sino, buscamos la base de datos especificada
    for database in databases:
        # Si coincide, la agregamos
        if db == database['name']:
            # Creamos el abm
            abm_database = ABM(database)

            # Si no estamos en multiprocessing, vamos a
            # cargar el dato en el singleton
            if not sgt.multiprocessing:
                # Si no esta creado el abm en el singleton, lo creamos
                if sgt.abms.get(database['name']) is None:
                    sgt.abms[database['name']] = abm_database

            # Seteamos el argumento en la clase
            setattr(
                objeto, database['name'], abm_database
            )

            return True

    print('ERROR: No se encontro el archivo de configuracion para "%s"' % db)
    sys.exit(1)
    return
