# -*- encoding: utf-8 -*-

import sys
#from twisted.python import log
#import time
#import logging
import sqlalchemy.exc
import sqlalchemy.interfaces
from sqlalchemy import orm
from sqlalchemy import MetaData
from sqlalchemy import create_engine
from sqlalchemy import pool
from sqlalchemy.orm import sessionmaker
from sqlalchemy.interfaces import ConnectionProxy
# NOTA: Esta excepcion ya no funciona en Python 3
#import _mysql_exceptions

import at.orm.logger
at.orm.logger.init()
from collections import namedtuple

from at.orm.singleton import Singleton
from at.utiles.archivos import leer_configobj
#from at.utiles.hilos import ejecutar_tarea
#from at.utiles.log import start_custom_log

# start_custom_log(
#     '/var/lib/at/logs/ORM/ORM', 'verificar_estado_bd'
# )

# Por defecto el valor de POOL_SIZE va a ser 10
valor_pool_size = 10
# Por defecto el valor de MAX_OVERFLOW va a ser 30
valor_max_overflow = 30
# Por defecto el valor de POOL_TIMEOUT va a ser 360
valor_pool_timeout = 360
# Por defecto el valor de POOL_RECYCLE va a ser 300
valor_pool_recycle = 300
# Por defecto el valor de CON_POOL va a ser True
valor_con_pool = True
# Llamo a la función leer_configobj para obtener los parámetros
rta_leer_configobj = leer_configobj('/var/lib/at/etc/parametros.txt')
# Si se pudieron obtener los parámetros
if rta_leer_configobj['Estado'] is True:
    # Me fijo el valor de pool_size
    if rta_leer_configobj['Respuesta'].get('pool_size') is not None:
        valor_pool_size = rta_leer_configobj['Respuesta']['pool_size']
    # Me fijo el valor de max_overflow
    if rta_leer_configobj['Respuesta'].get('max_overflow') is not None:
        valor_max_overflow = rta_leer_configobj['Respuesta']['max_overflow']
    # Me fijo el valor de pool_timeout
    if rta_leer_configobj['Respuesta'].get('pool_timeout') is not None:
        valor_pool_timeout = rta_leer_configobj['Respuesta']['pool_timeout']
    # Me fijo el valor de pool_recycle
    if rta_leer_configobj['Respuesta'].get('pool_recycle') is not None:
        valor_pool_recycle = rta_leer_configobj['Respuesta']['pool_recycle']
    # Me fijo el valor de con_pool
    if rta_leer_configobj['Respuesta'].get('con_pool') is not None:
        valor_con_pool = rta_leer_configobj['Respuesta']['con_pool']


# Seteamos los parámetros de SlqAlchemy #######################################
# POOL_SIZE: número maximo de conexiones PERMANENTES que mantiene abiertas.
POOL_SIZE = valor_pool_size
###############################################################################
# MAX_OVERFLOW: conexiones que TEMPORALMENTE excede el pool_size si no hay conexiones disponibles
MAX_OVERFLOW = valor_max_overflow

# el numero total de conexiones al mismo tiempo esta dado de la suma de
# pool_size+max_overflow
###############################################################################
# POOL_TIMEOUT: Segundos a esperar para intentar hacer la consulta.
POOL_TIMEOUT = valor_pool_timeout
###############################################################################
# POOL_RECYCLE: Segundos para reiniciar una conexión.
POOL_RECYCLE = valor_pool_recycle
###############################################################################
###############################################################################
# CON_POOL: Indico si voy a usar un pool de conexiones o no.
CON_POOL = valor_con_pool
###############################################################################
###############################################################################


class Mapped(object):
    __shared = {}

    def __init__(self):
        self.__dict__ = self.__shared


class PoolQueueListener(sqlalchemy.interfaces.PoolListener):
    """ PoolListener is deprecated, we should use PoolEvent but
    sqlalchemy.event isnt available in sqlalchemy 0.6.6.
    """

    def __init__(self):
        self.retried = False

    def first_connect(self, dbapi_con, con_record):
        ''' Funcion que se llama cuando se crea una nueva conexion para
        el pool de conexiones '''

        pass

    def checkin(self, dbapi_con, con_record):
        ''' Esta funciono se llama cuando se devuelve una conexion al
        pool de conexiones '''

        pass

    def checkout(self, dbapi_con, con_record, con_proxy):
        ''' Esta funcion se llama cuando se saca una conexion del pool
        de conexiones '''

        # Intentamos pedir la hora
        try:
            dbapi_con.cursor().execute('select now()')
        # Si fallamos, salimos con un error de desconexion
        # NOTA: Esta excepcion ya no funciona en Python 3
        #except _mysql_exceptions.OperationalError as e:
        except Exception as e:
            if self.retried:
                self.retried = False
                raise
            self.retried = True
            print('ORM ERROR: %s' % str(e))
            raise sqlalchemy.exc.DisconnectionError

    #def checkout(self, dbapi_con, con_record, con_proxy):
        #''' Esta funcion se llama cuando se saca una conexion del pool
        #de conexiones '''

        ## Vamos a ver si la conexion que nos devolvieron esta
        #max_times = 20  # how many times should i try
        #iter_n = 0
        #retry = True
        ##log = logging.getLogger('at.orm')

        #while retry:
            #try:
                #dbapi_con.info()
                #retry = False
            #except sqlalchemy.exc.OperationalError as e:
                #if iter_n == max_times:
                    #raise sqlalchemy.exc.DisconnectionError
                #print('ORM ERROR: error de operacion, contador: %s' % str(
                    #iter_n
                #))
                #print('ORM ERROR: %s' % str(e))
                #iter_n += 1
                #time.sleep(0.5)
                #return
            #except Exception as e:
                ## En caso de que el dbapi_con no tenga la funcion info()
                ## va a entrar aca. Asi que no logueamos por que sino se
                ## genera basura
                #retry = False


class TimeOutProxy(ConnectionProxy):
    ''' Proxy para bloquear las consultas a un maximo de 600 segundos/10 min'''

    def __init__(self, engine):
        # Guardamos el engine
        self.engine = str(engine)

    def cursor_execute(self, execute, cursor, statement,
                             parameters, context, executemany):

        #log = logging.getLogger('at.orm')
        # Si estamos en postgres, seteamos el timeout
        if self.engine.find('postgres') != -1:
            try:
                c = cursor._parent.cursor()
                c.execute('SET statement_timeout TO %d;' % int(POOL_TIMEOUT * 1000))
                c.close()
            except Exception as e:
                #print('ORM ERROR: Se llego al timeout. %s' % str(e))
                pass
        # Si estamos en mysql, seteamos el timeout
        elif self.engine.find('mysql') != -1:
            try:
                c = cursor._parent.cursor()
                c.execute('SET wait_timeout = %d;' % POOL_TIMEOUT)
                c.close()
            except Exception as e:
                #print('ORM ERROR: Se llego al timeout. %s' % str(e))
                pass
        return execute(cursor, statement, parameters, context)


class ATMapper(object):
    test = False

    def __init__(self, data=None, *args, **kwargs):
        # Instanciamos nuestro singleton
        self.sgt = Singleton()

        # Vemos si nos conectamos
        auth = namedtuple('Auth', 'engine, user, passwd, host, name, db')
        auth.engine = data.get('engine', 'postgres')
        auth.user = data.get('user', 'postgres')
        auth.passwd = data.get('passwd', 'TrackAir')
        auth.host = data.get('host', 'localhost')
        auth.name = data.get('name', 'default')
        auth.db = data.get('dbname', 'airtrack')
        #log = logging.getLogger('at.orm')
        # Guardamos el auth
        self.auth = auth

        # Inicializamos las variables del mapper
        self.engine = None
        self.db = None
        self.meta = None

        # FIX de Paco - 27/05/2019 ############################################
        # Si comenta esta validación, el comentario estaba confuso
        # Ahora si funciona para que con multiproceso
        # se pueda usar más de una BD por clase
        # Si estamos en
        #if not self.sgt.multiprocessing:
        # Mapeamos la db con un engine sin Pool
        engine = self.__create_engine(False)
        # Creamos el objeto
        meta = MetaData(bind=engine)
        # Vamos a mapear la db con esta
        self.db = self.__map_db(engine, meta)
        # Lo cerramos al enginge de mapeo
        engine.dispose()
        #######################################################################

    def __bind_session(self, engine):
        '''
        FUncion interna para crear un objeto del tipo SessionMaker a partir
        de un engine.
        Este es el que se va a conectar.
        '''

        return sessionmaker(bind=engine)


    def __create_engine(self, with_pool=True):
        '''
        Funcion interna para crear un engine en relacion a los
        datos de conexion entregados anteriormente
        '''

        # Si CON_POOL es False
        if CON_POOL is False:
            # Quiere decir que no voy a trabajar con un pool de conexiones
            with_pool = False

        # Si queremos un engine con pool, lo creamos con proxys
        # y demases.
        if with_pool:
            engine = create_engine(
                ('%s://%s:%s@%s/%s' % (self.auth.engine,
                    self.auth.user,
                    self.auth.passwd,
                    self.auth.host,
                    self.auth.db)
                ),
                pool_size=POOL_SIZE,
                pool_timeout=POOL_TIMEOUT,
                pool_recycle=POOL_RECYCLE,
                max_overflow=MAX_OVERFLOW,
                listeners=[PoolQueueListener()],
                proxy=TimeOutProxy(self.auth.engine)
            )
        else:
            engine = create_engine(
                '%s://%s:%s@%s/%s' % (
                    self.auth.engine,
                    self.auth.user,
                    self.auth.passwd,
                    self.auth.host,
                    self.auth.db
                ),
                poolclass=pool.NullPool
            )

        # Devolvemos
        return engine


    def __map_db(self, engine, meta):
        '''
        Funcion interna para mapear la db a partir de un engine
        '''

        # Otenemos la db
        database = engine.url.database
        mapped = Mapped()

        _db = getattr(mapped, database, None)
        if _db is not None:
            return _db

        # Seteamos los atributos
        setattr(mapped, database, {})

        # Reflejamos
        meta.reflect(bind=engine)

        for tablename in meta.tables.keys():
            _db = getattr(mapped, database)
            if tablename not in _db.keys():
                klass = type(str(tablename), (object,), {})
                _db[tablename] = klass
                orm.mapper(_db[tablename], meta.tables[tablename])

        # Guardamos la db
        return _db


    def binded_session(self):
        # Si todavia no tenemos engine
        if self.engine is None:
            # Nos conectamos, si no hay engine
            self.connect()

        # Creamos una nueva session
        return self.session()


    def connect(self):
        ''' Esta funcion publica es para crear el engine y demas
        elementos del mapper.
        Se llama explicitamente al momento de obtener la primera session,
        pero podemos llamarla explicitamente cuando queramos
        '''

        # Validamos que no tengamos el engine
        if self.engine is None:
            try:
                # Creamos un engine. Si esta en multiprocessing, lo creamos sin pool.
                self.engine = self.__create_engine(not self.sgt.multiprocessing)

                # Creamos el meta
                self.meta = MetaData(bind=self.engine)

                # Mapeamos la db si no esta mapeada
                self.db = self.__map_db(self.engine, self.meta)

                # Bindeamos la session
                self.session = self.__bind_session(self.engine)

                # Logeamos solo si no es la default, no queremos que nos hackeen :)
                if self.auth.name != 'default':
                    if self.auth.db not in self.sgt.dbs.keys():
                        # Guardamos
                        self.sgt.dbs[self.auth.db] = self.auth.host
                        print("[db] %s:%s : multiprocess [%s]" % (
                            self.auth.db, self.auth.host, str(self.sgt.multiprocessing)
                        ))

                        # FIX: Saco este log de tareas, ya fue solucionado el
                        # problema, se recibían muchas peticiones en el mismo
                        # segundo.
                        # # Ejecuto como una tarea cada 300 segundos
                        # # para verificar el estado de la BD
                        # ejecutar_tarea(self.verificar_estado_bd, (), 300)

            except Exception as e:
                if self.auth.name == 'default':
                    return
                print('ORM ERROR: Al iniciar mapper con "%s" en "%s"' % (
                    self.auth.db.upper(), self.auth.engine.upper()
                ))
                print(str(e))
                # Si estamos en un test, salimos con la excepcion
                if self.test:
                    raise e
                # Sino, cortamos todo
                sys.exit(-1)

    # def verificar_estado_bd(self):
    #     # Guardo en un LOG el estado actual de la BD
    #     log.msg(
    #         '- Estado BD: [%s] %s' % (self.auth.name, self.engine.pool.status()),
    #         mode='verificar_estado_bd'
    #     )


########### CODIGO QUE ESTABA EN at.db #####################################

class conf(object):

    __shared = {}

    def __init__(self):
        self.__dict__ = self.__shared


class Database(object):
    '''
        Clase de at.db
    '''

    def __init__(self, **kwargs):
        ''' Un objeto de tipo Database. El mismo brinda los datos necesarios
        para conectarse a una base de datos y es utilizado por la clase
        DatabaseManager. Los datos que son obligatorios y DatabaseManager
        nos va a pedir son:
            * name: nombre de la base de datos
            * host: host de la base de datos
            * user: usuario para conectarse a la base de datos
            * alias: un alias para el nombre de la base de datos
            * passwd: password para conectarse a la base de datos
            * engine: el tipo de motor de la base de datos (mysql, postgres)
        Ejemplo:
            >>> mydb = Database(user='john', passwd='xxx', host='localhost'
                                name='cp2000', alias='cp', engine='mysql')
            >>> mydb.engine
            ... mysql
        '''

        #for kw in kwargs.iterkeys():
        # NOTA: En Python 3 no se usa "iterkeys", se usa solo "keys"
        for kw in kwargs.keys():
            if kw == 'engine':
                setattr(self, 'ENGINE', kwargs[kw])
                continue
            setattr(self, kw, kwargs[kw])


class DatabaseManager(object):
    '''
        Clase de at.db
    '''

    __shared_state = {}
    __ALIASES__ = {}
    sgt = Singleton()

    def __init__(self):
        # Si no estamos en multiprocessing, creamos
        # el estado compartido
        if not self.sgt.multiprocessing:
            self.__dict__ = self.__shared_state

    def _add_db(self, db):
        # Si estamos en multiprocessing, solo creamos un engine
        if self.sgt.multiprocessing:
            engine = create_engine('%s://%s:%s@%s/%s' % (
                    db.ENGINE,
                    db.user,
                    db.passwd,
                    db.host,
                    db.name),
            )

        # Sino, creamos un pool
        else:
            engine = create_engine('%s://%s:%s@%s/%s' % (
                    db.ENGINE,
                    db.user,
                    db.passwd,
                    db.host,
                    db.name),
                pool_size=POOL_SIZE,
                pool_timeout=POOL_TIMEOUT,
                pool_recycle=POOL_RECYCLE,
                max_overflow=MAX_OVERFLOW,
                proxy=TimeOutProxy(db.ENGINE)
            )

        setattr(db, 'engine', engine)
        setattr(db, 'meta', MetaData(bind=engine))
        setattr(db, 'session', sessionmaker(bind=engine))

        db.meta.reflect(bind=db.engine)

        if not getattr(self, db.ENGINE, None):
            setattr(self, db.ENGINE, type(db.ENGINE, (object,), {}))

        enginedb = getattr(self, db.ENGINE, None)
        setattr(enginedb, db.name, db)

        for kw in db.meta.tables.keys():
            obj = type(str(kw), (object,), {})
            setattr(db, kw, obj)
            orm.mapper(obj, db.meta.tables[kw])

        alias = getattr(db, 'alias', None)
        if alias:
            self.__ALIASES__[alias] = db

    def addDB(self, database):
        try:
            self._add_db(database)
        except Exception as ex:
            print(ex)
            sys.exit(1)

    def getDB(self, alias):
        return self.__ALIASES__.get(alias)
