# -*- coding: utf-8 -*-

########### CODIGO QUE ESTABA EN at.db #####################################

import re
import logging
# NOTA: sqlalchemy.engine.ddl no funciona en Python 3
#import sqlalchemy.engine.ddl
from sqlalchemy import MetaData
from sqlalchemy import Table
#from sqlalchemy import exceptions
#from sqlalchemy.dialects.mysql.base import *


class BaseModel(object):
    ''' Modelo Base de la que heredan nuestros modelos de base de datos.
    '''

    def __init__(self, manager):
        self.bd = manager.engine.url.database
        self.meta = manager.meta
        self.engine = manager.engine
        self.manager = manager

    def __exc_sql(self, query):
        ''' Ejecutamos una query directamente con nuestro engine
        '''

        try:
            self.engine.execute(query)
            logging.debug('query ejecutada: %s' % query)
        except Exception as ex:
            print('ERROR EN __exc_sql: ' + str(ex))
            logging.error('error en query: %s' % query)
            logging.error(ex)

    def __column_names(self, table):
        try:
            return set((unicode(i.name) for i in table.c))
        except:
            # NOTA: En Python 3 no se debe castear a "unicode", ya es unicode
            return set((i.name for i in table.c))

    def _check_table(self, model_table, db_metadata):
        # Vamos a checkear si la tabla existe
        try:
            # Intentamos obtener la tabla desde la base de datos
            db_table = Table(model_table.name, db_metadata, autoload=True)

        # Si fallamos al obtener la tabla es por que NO EXISTE!!
        #except exceptions.NoSuchTableError:
        except:
            #### Vamos a crear la tabla, ya que no esta creado
            print('|| info: Creando tabla %s' % model_table.name)

            # Creamos la tabla
            model_table.create(bind=self.engine)
            # Ahora si la obtenemos
            db_table = Table(model_table.name, db_metadata, autoload=True)

        # # Validacion del compilador, al cual le pasamos la tabla segun la version se sqlalchemy
        # if sqlalchemy.__version__.startswith('0.7'):
        #     ddl_c = self.engine.dialect.ddl_compiler(
        #                                 self.engine.dialect, None)
        # else:
        #     # otras versiones, con la 0.6 anda, anteriores no se ...
        #     ddl_c = self.engine.dialect.ddl_compiler(
        #                                 self.engine.dialect, db_table)
        # NOTA: Para Python 3 lo dejo así porque no puedo importar: import sqlalchemy.engine.ddl

        try:
            ddl_c = self.engine.dialect.ddl_compiler(self.engine.dialect, db_table)
        except:
            # NOTA: Con Python 3 se debe ejecutar de la siguiente manera
            ddl_c = self.engine.dialect.ddl_compiler(self.engine.dialect, None)

        # Obtenemos las columnas del model
        model_columns = self.__column_names(model_table)

        # Obenemos las columnas de la tabla en la bd
        db_columns = self.__column_names(db_table)

        # Vemos que tenemos que crear, que borrar y que actualizar
        to_create = model_columns - db_columns
        to_remove = db_columns - model_columns
        to_check = db_columns.intersection(model_columns)

        for c in to_create:
            model_column = getattr(model_table.c, c)
            print('|| info: Agregando columna %s.%s' % (model_table.name,
                                                  model_column.name))

            assert not model_column.constraints, 'no soporto contraints'
            model_col_spec = ddl_c.get_column_specification(model_column)

            #sql = None
            #if (self.engine.dialect.name == 'mysql' or
               #self.engine.dialect.name == 'postgresql'):
            # Generamos el sql
            sql = 'ALTER TABLE %s ADD %s' % (model_table.name,
                                             model_col_spec)

            self.__exc_sql(sql)

        for c in to_check:
            print('|| info: COLUMNA: ' + str(model_table.c[c]))
            model_column = model_table.c[c]
            db_column = db_table.c[c]

            if model_column == db_column:
                continue

            model_col_spec = ddl_c.get_column_specification(model_column)
            db_col_spec = ddl_c.get_column_specification(db_column)

            # Si estamos en postgres vamos a cambiar algunas cosas, como
            # los booleanos y decimales
            if (self.engine.dialect.name == 'postgres' or
               self.engine.dialect.name == 'postgresql'):
                model_col_spec = re.sub('[(][\d ,]+[)]', '', model_col_spec)
                # Si es entero o booleano no tiene que tener comillas
                if (model_col_spec.find('INTEGER') != -1 or
                   model_col_spec.find('BOOLEAN') != -1):
                    model_col_spec = model_col_spec.replace('\'', '')
                db_col_spec = re.sub('[(][\d ,]+[)]', '', db_col_spec)
                db_col_spec = db_col_spec.replace('DECIMAL', 'NUMERIC')
                db_col_spec = db_col_spec.replace('TINYINT', 'BOOL')
                db_col_spec = db_col_spec.replace('(', '')
                db_col_spec = db_col_spec.replace(')', '')
                db_col_spec = db_col_spec.replace(
                    '::time without time zone', ''
                )

            if model_col_spec != db_col_spec:
                msg = '\n%s.%s\n' % (model_table.name, model_column.name)
                msg += 'definicion en modelo:\t%r\n' % model_col_spec
                msg += 'definicion en db:\t%r\n' % db_col_spec
                logging.debug(msg)

                sql = None
                default_sql = None
                nulling_sql = None
                if self.engine.dialect.name == 'mysql':
                    sql = 'ALTER TABLE %s MODIFY %s' % (model_table.name,
                                                            model_col_spec)
                elif (self.engine.dialect.name == 'postgres' or
                   self.engine.dialect.name == 'postgresql'):
                    # Obtenemos el nombre y lo quitamos
                    aux = model_col_spec.split(' ')
                    column_name = aux[0]
                    # Si es not Null o Null, armamos otra query
                    if model_col_spec.find('NOT NULL') != -1:
                        model_col_spec = model_col_spec.replace('NOT NULL', '')
                        nulling_sql = 'ALTER TABLE %s' % (
                            model_table.name
                        )
                        nulling_sql += ' ALTER COLUMN %s SET NOT NULL' % (
                            column_name
                        )
                    # Vemos si es null
                    elif model_col_spec.find('NULL') != -1:
                        model_col_spec = model_col_spec.replace('NULL', '')
                        nulling_sql = 'ALTER TABLE %s' % (
                            model_table.name
                        )
                        nulling_sql += ' ALTER COLUMN %s SET NULL' % (
                            column_name
                        )
                    # Volvemos a cargar el auxiliar
                    aux = model_col_spec.split(' ')
                    aux.pop(0)
                    space_string = " "
                    # Vemos si tenemos default
                    if 'DEFAULT' in aux:
                        end = aux.index('DEFAULT')
                        column_type = space_string.join(aux[:end])
                        default_type = space_string.join(aux[end:])
                        # Ahora armamos la query default
                        default_sql = 'ALTER TABLE %s ALTER COLUMN %s' % (
                            model_table.name,
                            column_name,
                        )
                        default_sql += ' SET %s' % default_type
                    else:
                        column_type = space_string.join(aux)

                    sql = 'ALTER TABLE %s ALTER COLUMN %s TYPE %s' % (
                        model_table.name,
                        column_name,
                        column_type
                    )

                # Si tenemos sql lo ejecutamos
                if sql:
                    self.__exc_sql(sql)

                if default_sql:
                    self.__exc_sql(default_sql)

                if nulling_sql:
                    self.__exc_sql(nulling_sql)

            #if model_column.constraints or db_column.constraints:
            #    # TODO, check constraints
            #    msg = 'no entiendo constraints'
            #    logging.debug(msg)

        # Vemos si tenemos que remover alguna bd.
        for c in to_remove:
            model_column = getattr(db_table.c, c)
            msg = '%s.%s existe en la base de datos pero no en nuestro modelo.'
            print('>> warn:', msg % (model_table.name, model_column.name))

        # Vemos si tenemos que crear algun indice, recorriendo los que tenemos en el modelo, contrastando
        # contra los que tenemos en la db actualmente
        for model_index in model_table.indexes:
            index_exist = False
            for db_index in db_table.indexes:
                if db_index.name == model_index.name:
                    index_exist = True

            # Si no existe en la db, lo creamos
            if not index_exist:
                print("|| info: INDEX: creamos el indice  %s" % model_index.name)
                model_index.create(self.engine)

        # Vemos si tenemos que borrar algun indice, recorriendo los que tenemos en la bd, contrastando
        # contra los que tenemos en el model
        for db_index in db_table.indexes:
            index_exist = False
            # No tomameos en cuenta los que terminan en key
            if db_index.name.endswith('_key'):
                continue
            for model_index in model_table.indexes:
                if db_index.name == model_index.name:
                    index_exist = True
            # Si no existe en el modelo, lo borramos
            if not index_exist:
                print(">> warn: INDEX: borramos el indice  %s" % db_index.name)
                db_index.drop(self.engine)

    def __rename_unused_tables(self, models_metadata,
                                                db_metadata, prefix='x_'):
        ''' Dados como parametros el metadata del modelo y el metadata de la
        base de datos, buscamos las tablas que sobran en la base de datos, osea
        aquellas tablas que no estan definidas en el modelo. A estas tablas les
        cambiamos el nombre agregandoles un prefijo para poder reconocerlas
        como obsoletas.

        Ejemplo:
            una tabla 'carteras' pasaria a llamarse 'x_carteras'
        '''
        db_metadata.reflect()
        db_table_names = [t.name for t in db_metadata.sorted_tables]
        model_table_names = [t.name for t in self.metadata.sorted_tables]
        unused_tables = []
        for table in db_table_names:
            if table not in model_table_names:
                unused_tables.append(table)

        for table_name in unused_tables:
            if table_name.startswith(prefix):
                continue
            new_name = prefix + table_name
            if self.engine.dialect.name == 'mysql':
                sql = 'RENAME TABLE %s to %s' % (table_name, new_name)
                self.__exc_sql(sql)
            elif (self.engine.dialect.name == 'postgres' or
                   self.engine.dialect.name == 'postgresql'):
                sql = 'ALTER TABLE %s RENAME TO %s' % (table_name, new_name)

    def _merge_metadata_with_db(self, metadata):
        db_metadata = MetaData()
        db_metadata.bind = self.engine

        for model_table in metadata.sorted_tables:
            print('')
            print('|| info: Procesamos TABLA: ' + str(model_table.name))
            #if str(model_table.name) == 'historico':
                #print('*** SALTEO LA TABLA HISTORICO ***')
                #continue

            try:
                self._check_table(model_table, db_metadata)
            except Exception as ex:
                #print(':: error: En funcion "_check_table" ' + str(ex))
                pass

        #self.__rename_unused_tables(self.metadata, db_metadata)

    def merge_with_db(self):

        modelmetadata = getattr(self, 'modelmetadata', None)
        if modelmetadata is None:
            msg = 'no local metadata in class %s' % self.__class__.__name__
            raise Exception(msg)
        self._merge_metadata_with_db(modelmetadata)
