# -*- coding: utf-8 -*-

import unittest
from sqlalchemy.orm.query import Query

from at.orm import _
from at.orm.tests.databases import cp2000
from at.orm.tests.databases import tests


class TestABM(unittest.TestCase):

    def setUp(self):
        tests.borrar('actual')

    def tearDown(self):
        pass
        
    def test_atributos_instancia_abm(self):
        abm = cp2000
                
        self.assertTrue(hasattr(abm, 'db'))
        self.assertTrue(hasattr(abm, 'meta'))
        self.assertTrue(hasattr(abm, 'engine'))
        self.assertTrue(hasattr(abm, 'binded_session'))        

    def test_tabla(self):
        ''' Un metodo que me devuelve un 'objeto Tabla' ... 
        '''

        abm = cp2000
        
        ''' puedo pedir el objeto tabla de esta forma '''
        tabla_clientes = abm.tabla('clientes')
        self.assertTrue(tabla_clientes in abm.db.values())
        self.assertTrue(hasattr(tabla_clientes, 'Telefono'))
        self.assertTrue(hasattr(tabla_clientes, 'Direccion'))

        ''' Puedo utilizar los campos de esta tabla para realizar busquedas ...
        '''
        session = abm.binded_session()
        q_object = session.query(tabla_clientes)
        self.assertTrue(isinstance(q_object, Query))
        session.close()

    def test_agregar(self):
        ''' Instanciando la clase ABM puedo agregar datos en una tabla, 
        especificando la tabla y pasando los valores en un diccionario ...
        '''
        
        abm = cp2000
        
        ''' agreguemos un nuevo cliente '''
        datos = {
               'Telefono': '487514654864',
               'Nombre': 'Juan Perez',
               'Direccion': 'Foo Bar 123',
               'IdZona': 1,
               }
        
        self.assertTrue(abm.agregar('clientes', **datos))
                
        ''' si la tabla no existe, obtenemos una excepcion ... '''
        self.assertRaises(Exception, abm.agregar, ('foobar', datos))
        
        ''' si la tabla existe pero especificamos un campo invalido, obtenemos
        una excepcion '''
        self.assertRaises(Exception, abm.agregar, ('clientes', {'foo':'bar'}))
        
    def test_borrar(self):
        ''' Puedo borrar ?! ...
        '''
        
        abm = cp2000

        ''' agreguemos un nuevo cliente '''
        datos = {
               'Telefono': '565656',
               'Nombre': 'Pedro',
               'Direccion': 'Foo Bar 123',
               'IdZona': 1,
               }

        self.assertTrue(abm.agregar('clientes', **datos))

        ''' En los datos especifico que quiero borrar los registros cuyos 
        campos Telefono sean igual a 4112233 y cuyo campo Nombre sea igual
        a Juan Perez''' 
        datos = {'Telefono': ('==', '565656'),
                 'Nombre': ('==', 'Pedro'),
                 }

        self.assertTrue(abm.borrar('clientes', **datos))
        
        ''' Si quiero borrar un registro que no existe, obtengo None como
        valor de retorno de la funcion ... 
        '''
        self.assertTrue(None == abm.borrar('clientes', **datos))

    def test_insertar(self):
        abm = tests

        dic = {}
        dic['Telefono'] = '4110549'
        dic['FechaServicio'] = '2017-01-05'
        dic['HoraServicio'] = '1900-01-01 14:00:00'
        dic['Direccion'] = 'Italia 1500'

        valor_campo = 'IdServicio'

        rta = abm.insertar('actual', valor_campo, **dic)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se insertó el registro en la tabla "actual" de forma correcta')
        )
        self.assertTrue(rta['ValorCampo'] is not None)

        dic = {}
        dic['Telefono'] = '153705966'
        dic['FechaServicio'] = '2017-01-05'
        dic['HoraServicio'] = '1900-01-01 14:30:00'
        dic['Direccion'] = 'Rioja 2100'

        valor_campo = 'Telefono'
        rta = abm.insertar('actual', valor_campo, **dic)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se insertó el registro en la tabla "actual" de forma correcta')
        )
        self.assertTrue(rta['ValorCampo'] == '153705966')

        dic = {}
        dic['Telefono'] = '4856321'
        dic['FechaServicio'] = '2017-01-05'
        dic['HoraServicio'] = '1900-01-01 14:45:00'
        dic['Direccion'] = 'San Luis 600'

        valor_campo = 'PrioridadOriginal'
        rta = abm.insertar('actual', valor_campo, **dic)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se insertó el registro en la tabla "actual" de forma correcta')
        )
        self.assertTrue(rta['ValorCampo'] is None)

        dic = {}
        dic['Telefono'] = '4125253'
        dic['FechaServicio'] = '2017-01-05'
        dic['HoraServicio'] = '1900-01-01 14:50:00'
        dic['Direccion'] = 'Mitre 800'

        rta = abm.insertar('actual', **dic)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se insertó el registro en la tabla "actual" de forma correcta')
        )
        self.assertTrue(rta['ValorCampo'] is None)

        dic = {}
        dic['Telefono'] = '4125253'
        dic['FechaServicio'] = '2017-01-05'
        dic['HoraServicio'] = '1900-01-01 14:50:00'
        dic['Direccion'] = 'Mitre 800'

        rta = abm.insertar('actual', **dic)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(isinstance(rta['Respuesta'], str))

        dic = {}
        dic['Telefono'] = '4785214'
        dic['FechaServicio'] = '2017-01-05'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Direccion'] = 'San Juan 2500'
        dic['asd'] = 'asd'

        rta = abm.insertar('actual', **dic)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El campo "asd" no pertenece a la tabla "actual"')
        )

        dic = {}
        dic['Telefono'] = '4785214'
        dic['FechaServicio'] = '2017-01-05'
        dic['HoraServicio'] = '1900-01-01 15:10:00'
        dic['Direccion'] = 'Dorrego 1900'

        valor_campo = 'qwe'

        rta = abm.insertar('actual', valor_campo, **dic)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == "'actual' object has no attribute 'qwe'"
        )

        dic = {}
        dic['Telefono'] = '4156325'
        dic['FechaServicio'] = '2017-01-05'
        dic['HoraServicio'] = '1900-01-01 15:20:00'
        dic['Direccion'] = 'Paraguay 2600'

        valor_campo = None

        rta = abm.insertar('actual', valor_campo, **dic)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se insertó el registro en la tabla "actual" de forma correcta')
        )
        self.assertTrue(rta['ValorCampo'] is None)

        dic = {}
        dic['Telefono'] = '4526352'
        dic['FechaServicio'] = '2017-01-05'
        dic['HoraServicio'] = '1900-01-01 15:30:00'
        dic['Direccion'] = 'Esquivel 1370'

        valor_campo = ''

        rta = abm.insertar('actual', valor_campo, **dic)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == "'actual' object has no attribute ''"
        )

                
class TestABMDecorated(unittest.TestCase):
    ''' HACER
    Preciso tests que usen el decorador DBSwitcher  
    '''
    pass


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestABM))
    suite.addTest(makeSuite(TestABMDecorated))
    return suite
