import sys
import os
from setuptools import find_packages
from distutils.core import setup
from distutils.extension import Extension
from distutils.core import Command
from distutils.command.build_ext import build_ext
import token
import tokenize
import shutil
import tarfile
import zipfile

version = '4.0.3'
name = 'at.orm'
description = "Modulo con conexion de base de datos"
long_description = "Proyecto que proveera de funcionalidad de base de datos"
author = 'airtrack development team'
author_email = 'developers@airtrack.com.ar'
install_requires = []
entry_points = {}

# Archivos a incluir
include_files_type = []
# Directorios que se distribuiran pero no compilados
no_compile_dirs = []
# Directorios a evitar en distribuciones compiladas
compile_exclude_dirs = [
    'tests', 'examples', 'ez_setup', 'migrations', 'installer',
]


def remove_comments(fname):
    source = open(fname)
    mod = open(fname[:-3] + ".strip.py", "w")
    mod.write('# -*- coding: iso-8859-1 -*-\n')
    prev_toktype = token.INDENT
    last_lineno = -1
    last_col = 0
    tokgen = tokenize.generate_tokens(source.readline)
    for toktype, ttext, (slineno, scol), (elineno, ecol), ltext in tokgen:
        if slineno > last_lineno:
            last_col = 0
        if scol > last_col:
            mod.write(" " * (scol - last_col))
        if toktype == token.STRING and prev_toktype == token.INDENT:
            # Docstring
            mod.write("")
        elif toktype == tokenize.COMMENT:
            # Comment
            mod.write("")
        else:
            mod.write(ttext)
        prev_toktype = toktype
        last_col = ecol
        last_lineno = elineno


def get_module_files(dir, files=[]):
    # Recorremos los archivos de la carpeta
    for file in os.listdir(dir):
        path = os.path.join(dir, file)
        # Es un archivo
        if os.path.isfile(path):
            # Borramos cualquier basura restante de algun setup fallido
            if path.endswith(".strip.py"):
                os.remove(path)
            else:
                flag_include = False
                dir_name = os.path.basename(os.path.dirname(path))
                if path.endswith(".py") and \
                   dir_name not in no_compile_dirs:
                    flag_include = True
                else:
                    for ext in include_files_type:
                        if path.endswith(ext):
                            flag_include = True
                            break
                if flag_include:
                    files.append(path)
       # Es un directorio
        elif os.path.isdir(path):
            # Evitamos las carpetas excluidas cuando no estamos compilando
            # Y no es un directorio oculto
            if (not file in compile_exclude_dirs or not flag_compiling) and \
               not file.startswith('.'):
                get_module_files(path, files)
    return files


def get_compiled_files(dir, dest_dir, files=[]):
    try:
        # Intentamos crear la carpeta en dist
        os.mkdir(dest_dir)
    except:
        pass
    # Creamos un __init__.py por que el __init__.so no funciona
    f = open(os.path.join(dest_dir, '__init__.py'), 'w')
    f.write("\n")
    f.close()
    # Recorremos los archivos de la carpeta
    for file in os.listdir(dir):
        path = os.path.join(dir, file)
        dest_path = os.path.join(dest_dir, file)

        if os.path.isfile(path):
            # Borramos cualquier basura restante,
            # como los .c y los py sin comentarios
            if path.endswith(".strip.py") or path.endswith(".strip.c"):
                os.remove(path)
            else:
                flag_include = False
                if path.endswith('.so'):
                    flag_include = True
                elif path.endswith('.py') and \
                     os.path.basename(os.path.dirname(path)) in no_compile_dirs:
                    flag_include = True
                else:
                    for ext in include_files_type:
                        if path.endswith(ext):
                            flag_include = True
                            break
                if flag_include:
                    # Movemos los archivos correspondientes a la carpeta 'dist'
                    shutil.copy(path, dest_path)
                    if path.endswith(".so"):
                        # Luego de haberlo movido, boramos el .so
                        os.remove(path)

        elif os.path.isdir(path):
            # Evitamos las carpetas indeseadas y ocultas
            if file not in compile_exclude_dirs and not file.startswith('.'):
                get_compiled_files(path, dest_path, files)
    return files


def zipdir(dirPath=None, zipFilePath=None, includeDirInZip=True):

    if not zipFilePath:
        zipFilePath = dirPath + ".zip"
    if not os.path.isdir(dirPath):
        raise OSError("dirPath argument must point to a directory. "
            "'%s' does not." % dirPath)
    parentDir, dirToZip = os.path.split(dirPath)
    #Little nested function to prepare the proper archive path

    def trimPath(path):
        archivePath = path.replace(parentDir, "", 1)
        if parentDir:
            archivePath = archivePath.replace(os.path.sep, "", 1)
        if not includeDirInZip:
            archivePath = archivePath.replace(dirToZip + os.path.sep, "", 1)
        return os.path.normcase(archivePath)

    outFile = zipfile.ZipFile(zipFilePath, "w",
            compression=zipfile.ZIP_DEFLATED)
    for (archiveDirPath, dirNames, fileNames) in os.walk(dirPath):
        for fileName in fileNames:
            filePath = os.path.join(archiveDirPath, fileName)
            outFile.write(filePath, trimPath(filePath))
        #Make sure we get empty directories as well
        if not fileNames and not dirNames:
            zipInfo = zipfile.ZipInfo(trimPath(archiveDirPath) + "/")
            #some web sites suggest doing
            #zipInfo.external_attr = 16
            #or
            #zipInfo.external_attr = 48
            #Here to allow for inserting an empty directory.  Still TBD/TODO.
            outFile.writestr(zipInfo, "")
    outFile.close()


def remove_source_files(dir, files=[]):
    # Recorremos los archivos de la carpeta
    for file in os.listdir(dir):
        path = os.path.join(dir, file)
        if os.path.isfile(path):
            # Borramos todo lo que no sea un .so
            if path.endswith(".strip.py") or path.endswith(".strip.c") or \
               (path.endswith(".py") and  not path.endswith("__init__.py")) or \
               (path.endswith(".pyc") and  not path.endswith("__init__.pyc")):
                os.remove(path)
        elif os.path.isdir(path):
            # Evitamos las carpetas indeseadas y ocultas
            if file not in compile_exclude_dirs and not file.startswith('.'):
                remove_source_files(path, files)
            # Lo volamos al directorio
            else:
                shutil.rmtree(path)

    return files


def make_extension(ext_name):
    # Genera el objeto de extension a compilar
    ext_path = ext_name

    # Esto es para cuando son archivos .py
    if ext_name.endswith(".py"):
        # Generamos el nombre del modulo
        ext_name = ext_name.replace(os.path.sep, ".")
        ext_name = ext_name[:-3]

        # Si estamos compilando vamos a limpiar los comentarios
        if flag_compiling:
            # Removemos los comentarios
            remove_comments(ext_path)
            # Vamos a compilar el archivo sin comentarios, no el .py fuente
            ext_path = ext_path[:-3] + ".strip.py"

        return Extension(
            ext_name,
            [ext_path],
            include_dirs=[root_dir, "."],
            extra_compile_args=["-O3", "-Wall"],
            extra_link_args=['-g'],
        )

    # Cuando son otros archivos, devolvemos solo si no estamos compilando
    elif not flag_compiling:
        return Extension(
            ext_name,
            [ext_path],
        )


# Esta funcion obtiene todos los packages y los devuelve en forma de lista
def packagelist(directory):
    return list(sorted(root.replace(os.sep, '.')
        for root, dirs, files in os.walk(directory)
        if '__init__.py' in files))

# Declaramos variables a usar
flag_compiling = False
flag_packaging = False
flag_installing = False
flag_building = False
flag_egg = False
tmp_dir = None
extensions = []
pkg = [name]
pkg_data = {}
args = sys.argv[1:]
# Directorio del paquete
root_dir = name.split('.')[0]

# Segun los argumentos, vamos a actuar de diferentes formas

# Caso de querer compilar, genera una carpeta aparte
if 'compile' in args:
    flag_compiling = True
    args.append('build_ext')
    args.append('--inplace')
    args.remove('compile')

# Generar un tar en la carpeta dist, con los compilados
elif 'compile_dist' in args:
    flag_compiling = True
    flag_packaging = True
    args.append('build_ext')
    args.append('--inplace')
    args.remove('compile_dist')

elif 'install' in args or 'sdist' in args or \
     'build' in args or 'develop' in args or 'bdist_egg':
    if 'install' in args or 'develop' in args:
        flag_installing = True
    elif 'build' in args:
        flag_building = True
    elif 'bdist_egg' in args:
        flag_compiling = True
        args.insert(0, '--inplace')
        args.insert(0, 'build_ext')

        flag_egg = True
        if '--dist-dir' in args:
            indice = args.index('--dist-dir')
            tmp_dir = args[indice + 1]

    # Actualizamos los packages para que instale todo bien
    pkg = packagelist(root_dir)
    pkg_data = {'': ['*.*']}


cmd_class = {}
if flag_compiling:
    try:
        from Cython.Distutils import build_ext as cy_build_ext
        cmd_class['build_ext'] = cy_build_ext

    except Exception as e:
        # Puede que el zc.recipe este borrando el sys.path, asi que tratamos
        # de arreglarlo
        for path in sys.path:
            if path.find('.egg') != -1:
                dir = os.path.dirname(path)
        for file in os.listdir(dir):
            path = os.path.join(dir, file)
            if os.path.isfile(path):
                if path.find('Cython') != -1:
                    sys.path.append(os.path.join(dir, file))
            elif os.path.isdir(path):
                if path.find('Cython') != -1:
                    sys.path.append(os.path.join(dir, file))

        # Intentamos nuevamente el importar Cython
        try:
            from Cython.Distutils import build_ext as cy_build_ext
            cmd_class['build_ext'] = cy_build_ext
        except:
            print("No tenes instalado Cython.")
            print("Bajalo desde www.cython.org")
            sys.exit(1)

if not flag_installing and not flag_building:
    # obtenemos la lista de extensiones
    extensions_names = get_module_files(root_dir)

    # Creamos las extensiones
    for ext_name in extensions_names:
        ext = make_extension(ext_name)
        if ext != None:
            extensions.append(ext)

# Pasamos todo a distutils
setup(
    name=name,
    version=version,
    description=description,
    long_description=long_description,
    author=author,
    author_email=author_email,
    packages=pkg,
    package_data=pkg_data,
    ext_modules=extensions,
    cmdclass=cmd_class,
    install_requires=install_requires,
    script_args=args,
    entry_points=entry_points,
)

# Si estabamos compilando, vamos a quitar toda la basura y la vamos a mover
# a otra carpeta
if flag_compiling:
    pkg_dir = name + '-' + version
    # Vamos a mover todo al directorio del paquete asi que intentamos crearlo
    try:
        os.mkdir(os.path.join(pkg_dir))
    except:
        pass
    try:
        os.mkdir(os.path.join(pkg_dir, root_dir))
    except:
        pass
    # Obtenemos los archivos compilados
    get_compiled_files(root_dir, os.path.join(pkg_dir, root_dir))

    # Si estamos haciendo un egg, vamos a borrar la carpeta con los
    # compilados, ya que ya fueron movidos a la carpeta build por el setup
    if flag_egg:
        shutil.rmtree(pkg_dir)

    # Sino, continuamos con la carpeta de compilados
    else:
        # Copiamos el setup.py en la carpeta a empaquetar
        shutil.copy('setup.py', os.path.join(pkg_dir, 'setup.py'))
        # Si estamos empaquetando
        if flag_packaging:
            # Vamos a meter el tar en el directorio dist, asi que
            # tratamos de crearlo
            try:
                os.mkdir('dist')
            except:
                pass
            # Creamos el tar
            tar_name = os.path.join('dist', pkg_dir) + '-compiled.tar.gz'
            tar = tarfile.open(name=tar_name, mode='w:gz')
            tar.add(os.path.join(pkg_dir))
            tar.close()
            # Borramos el direcorio temporal
            shutil.rmtree(pkg_dir)

# Si estamos armando un egg, vamos a buscar la carpeta donde se genero
# y vamos a limpiar el codigo fuente
if flag_egg:
    if tmp_dir is None:
        # Si no se paso como parametro, el directorio del eg es 'dist'
        tmp_dir = 'dist'
    for file in os.listdir(tmp_dir):
        path = os.path.join(tmp_dir, file)
        if os.path.isfile(path):
            if path.endswith(".egg"):
                extract_dir = os.path.join(os.path.dirname(tmp_dir), 'tmp')
                egg = zipfile.ZipFile(path, 'r')
                egg.extractall(extract_dir)
                open(os.path.join(extract_dir, 'EGG-INFO', 'SOURCES.txt'),
                     'w').close()
                remove_source_files(extract_dir)
                zipdir(extract_dir, path, False)
                # Borramos el direcorio temporal
                shutil.rmtree(extract_dir)

