# -*- coding: utf-8 -*-

from at.utiles import _
from at.utiles import validar
from at.orm.decorators import DBSwitch

from datetime import datetime
from sqlalchemy import and_, or_

import time
import hashlib

# Estos son los posibles estados de los tokens
_ESTADOS_TOKENS = {
    'pedido': 0,
    'generado': 1,
    'usado': 2,
    'error': 3
}

@DBSwitch('atmanager')
class AtPagos(object):

    def __parse(self, estado, respuesta, exception=None):
        '''
        Funcion para procesar la respuesta
        :param estado:
        :param respuesta:
        :return:
        '''

        d = {'Estado': estado, 'Respuesta': respuesta}
        # Vemos si tenemos exception
        if exception is not None:
            d['Exception'] = exception
        # Salimos
        return d

    def generar_token(self, api, nombre_aplicacion, usuario_aplicacion, id_empresa_manager, ip,
            email, nombre, apellido, documento_tipo, documento_numero, id_ciudad, direccion,
            telefono, prefijo, zona_horaria=-3, interno=False):
        '''
        Funcion interna para generar un token de atpagos. La funcion genera un registro
        que luego sera rellenado por el servidor de pagos. De esta forma, la app
        no se conecta a la db de pagos.
        :param api: La api con la que vamos a trabajar
        :param nombre_aplicacion: La app con la que vamos a trabajar.
        :param usuario_aplicacion: El usuario de la app con el que vamos a trabajar
        :param nombre: El nombre del cliente que va a pagar
        :param apellido:
        :param email:
        :return:
        '''

        # Validamos
        rta = validar.string(api)
        if not rta['Estado']:
            return rta

        rta = validar.string(nombre_aplicacion)
        if not rta['Estado']:
            return rta

        rta = validar.string(usuario_aplicacion)
        if not rta['Estado']:
            return rta

        rta = validar.string(id_empresa_manager)
        if not rta['Estado']:
            return rta

        rta = validar.string(ip)
        if not rta['Estado']:
            return rta

        rta = validar.string(email)
        if not rta['Estado']:
            return rta

        rta = validar.string(nombre)
        if not rta['Estado']:
            return rta

        rta = validar.string(apellido)
        if not rta['Estado']:
            return rta

        rta = validar.string(documento_tipo)
        if not rta['Estado']:
            return rta

        rta = validar.string(documento_numero)
        if not rta['Estado']:
            return rta

        # Validamos
        rta = validar.numero_entero(id_ciudad)
        if not rta['Estado']:
            return rta

        rta = validar.string(direccion)
        if not rta['Estado']:
            return rta

        rta = validar.string(telefono)
        if not rta['Estado']:
            return rta

        rta = validar.string(prefijo)
        if not rta['Estado']:
            return rta

        rta = validar.booleano(interno)
        if not rta['Estado']:
            return rta

        # Si ya hay un token generado, lo devolvemos
        rta = self.obtener_token(api, nombre_aplicacion, usuario_aplicacion, id_empresa_manager)
        if rta['Estado']:
            return self.__parse(False, _(u'Ya hay un token generado para esa aplicacion.'))

        # Vamos a ver si hay un pedido de token
        # Abro la sesión con la base de datos
        session = self.atmanager.binded_session()

        # Obtengo la tabel clientes
        atpagos_tokens = self.atmanager.tabla('atpagos_tokens')

        # Busco el cliente
        q_obj = session.query(atpagos_tokens.id)

        # Filtro por el id dado
        q_obj = q_obj.filter(and_(
            atpagos_tokens.api == api,
            atpagos_tokens.nombre_aplicacion == nombre_aplicacion,
            atpagos_tokens.usuario_aplicacion == usuario_aplicacion,
            atpagos_tokens.estado == _ESTADOS_TOKENS['pedido']
        ))

        # Guardo el registro encontrado
        registro = q_obj.first()

        # Cierro la conexión con la base de datos
        session.close()

        # Si no se encontró el registro
        if registro is not None:
            return self.__parse(False, _(u'Ya hay una peticion de token realizada.'))

        # Guardo los datos de el cliente
        dic = {}
        dic['api'] = api
        dic['nombre_aplicacion'] = nombre_aplicacion
        dic['usuario_aplicacion'] = usuario_aplicacion
        dic['id_empresa_manager'] = id_empresa_manager
        dic['ip'] = ip
        dic['email'] = email
        dic['apellido'] = apellido
        dic['nombre'] = nombre
        dic['documento_tipo'] = documento_tipo
        dic['documento_numero'] = documento_numero
        dic['id_ciudad'] = id_ciudad
        dic['direccion'] = direccion
        dic['telefono'] = telefono
        dic['prefijo'] = prefijo
        dic['zona_horaria'] = zona_horaria
        dic['estado'] = 0
        dic['fecha_hora_inicio'] = datetime.now()
        hash_generado = '%s%s%s%s' % \
                        (nombre_aplicacion, usuario_aplicacion, api, str(time.time()))
        hash_generado = hash_generado.encode('utf-8')
        dic['hash'] = hashlib.sha256(hash_generado).hexdigest()
        dic['interno'] = interno

        # Agrego el registro a la base de datos
        rta = self.atmanager.agregar('atpagos_tokens', **dic)

        # Si no se puede agregar retorno el error
        if rta is not True:
            return self.__parse(
                False, _(u'No se pudo generar el token.'), rta
            )

        # Si se pudo agregar retorno el mensaje
        return self.__parse(
            True, _(u'Se guardó correctamente el token.')
        )


    def obtener_token(self, api, nombre_aplicacion, usuario_aplicacion, id_empresa_manager, pedido=False):
        '''
        Funcion para obtener un token valido para utilizar con
        at pagos
        :param api:
        :param nombre_aplicacion:
        :param usuario_aplicacion:
        :return:
        '''

        rta = validar.string(nombre_aplicacion)
        if not rta['Estado']:
            return rta

        rta = validar.string(usuario_aplicacion)
        if not rta['Estado']:
            return rta

        rta = validar.string(id_empresa_manager)
        if not rta['Estado']:
            return rta

        rta = validar.booleano(pedido)
        if not rta['Estado']:
            return rta

        # Abro la sesión con la base de datos
        session = self.atmanager.binded_session()

        # Obtengo la tabel clientes
        atpagos_tokens = self.atmanager.tabla('atpagos_tokens')

        # Busco el cliente
        q_obj = session.query(
            atpagos_tokens.estado,
            atpagos_tokens.error,
            atpagos_tokens.clave_publica,
            atpagos_tokens.token_header,
            atpagos_tokens.token_payload,
            atpagos_tokens.token_signature
        )

        # Filtro por el id dado
        q_obj = q_obj.filter(and_(
            atpagos_tokens.api == api,
            atpagos_tokens.nombre_aplicacion == nombre_aplicacion,
            atpagos_tokens.usuario_aplicacion == usuario_aplicacion,
            atpagos_tokens.id_empresa_manager == id_empresa_manager,
        ))

        if not pedido:
            q_obj = q_obj.filter(or_(
                atpagos_tokens.estado == _ESTADOS_TOKENS['generado'],
                atpagos_tokens.estado == _ESTADOS_TOKENS['error']
            ))
        else:
            q_obj = q_obj.filter(atpagos_tokens.estado == _ESTADOS_TOKENS['pedido'])

        # Guardo el registro encontrado
        registro = q_obj.first()

        # Cierro la conexión con la base de datos
        session.close()

        # Si no se encontró el registro
        if registro is None:
            # Salimos mal
            return self.__parse(False, _(u'No se encontró el token con la combinacion %s-%s-%s-%s' % (
                api, nombre_aplicacion, usuario_aplicacion, id_empresa_manager
            )))

        # NOTA: No puedo hacer esto en Python 3 así que armo el diccionar
        # de forma manual
        # Guardo sus datos en un diccionario
        # token = registro.__dict__
        #
        # # Limpiamos
        # try:
        #     del token['_sa_instance_state']
        # except:
        #     pass
        #
        # try:
        #     del token['_labels']
        # except:
        #     pass

        token = {}
        token['error'] = registro.error
        token['clave_publica'] = registro.clave_publica

        try:
            token['estado'] = int(registro.estado)
        except:
            pass

        try:
            token['zona_horaria'] = int(registro.zona_horaria)
        except:
            pass

        # Si falló, salimos con error
        if token['estado'] == _ESTADOS_TOKENS['error']:
            return self.__parse(True, token)

        # Si vemos los tokens pedidos, salimos
        if token['estado'] == _ESTADOS_TOKENS['pedido']:
            # Borramos los datos de claves
            return self.__parse(True, {'estado': token['estado']})

        # Generamos el token final
        token['token'] = '%s.%s.%s' % \
                         (registro.token_header, registro.token_payload, registro.token_signature)

        # # Limpiamos los tokens
        # try:
        #     del token['token_header']
        # except:
        #     pass
        #
        # try:
        #     del token['token_payload']
        # except:
        #     pass
        #
        # try:
        #     del token['token_signature']
        # except:
        #     pass

        # Retorno True y los datos
        return self.__parse(True, token)
