# -*- coding: utf-8 -*-

from at.utiles import _
from at.utiles.conversiones import respuesta
from at.utiles.http import enviar_por_http
from at.orm.decorators import DBSwitch


def verificar_conexion_cp():
    '''
        Esta función verifica si hay conexión con la base de datos cp.

        Retorna:
            Caso correcto:
                True
            Caso incorrecto:
                False
    '''

    @DBSwitch('cp')
    class cp_orm(object):
        pass

    try:
        session = cp_orm.cp.binded_session()
        session.close()
        return True
    except Exception as e:
        print(':: error:', e)
        return False


def verificar_conexion_internet():
    '''
        Esta función se encarga de verificar si hay o no conexión con internet.

        Funciones:
            at.utiles -> conversiones -> respuesta
            at.utiles -> http -> enviar_por_http

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Hago una petición HTTP a Google para ver si tengo internet
    rta_enviar_por_http = enviar_por_http(
        'https://www.google.com.ar', {}, 'GET', 5
    )
    # Si no se pudo hacer la petición
    if rta_enviar_por_http['Estado'] is False:
        # Retorno el error
        return rta_enviar_por_http

    # Guardo el estado resultado de la petición
    estado = rta_enviar_por_http['Respuesta']['resp'].status

    # Si el estado no es 200
    if estado != 200:
        # Retorno False y un mensaje
        return respuesta(False, _(u'No hay internet'))

    # Si el estado es 200 retorno True y un mensaje
    return respuesta(True, _(u'Hay internet'))
