# -*- coding: utf-8 -*-

import unicodedata
import json
from at.utiles import _
from at.utiles import validar
from unidecode import unidecode


def sqla_to_dict(datos):
    salida = []
    for row in datos:
        dictret = row.__dict__
        dictret.pop('_sa_instance_state', None)
        salida.append(dictret)
    return salida


def remover_duplicados_lista(lista):
    return list(set(lista))  # elimina duplicados


def string_to_list(string, delimitador):
    try:
        return string.split(delimitador)
    except:
        return None


def lista_to_string(lista, caracter):
    return caracter.join(map(str, lista))


def str_to_json(string):
    return json.loads(string)


def unicode_to_str(elemento):
    """

    convierte de unicode a string

    :param elemento: unicode u'm\xc3\xb3vil'
    :return: str 'movil'
    """
    return unidecode(elemento)


def __parse(estado, respuesta):
    '''

    :param estado:
    :param respuesta:
    :return:
    '''

    return {
        'Estado': estado,
        'Respuesta': respuesta
    }


def siempre_dos_decimales(valor):
    '''
        Valido que el valor ingresado sea un número, en caso correcto retorno
        el valor con 2 decimales en formato str.
        Parametros:
        Retorno:
    '''

    # Valido que sea un número.
    v = validar.numero(valor)
    if v['Estado'] is not True:
        return v

    # Redondeo el valor a 2 decimales.
    valor = round(valor, 2)

    # Paso a string y separo la cadena por el punto.
    sep = str(valor).split('.')

    # Si la longitud es 1 signigica que era un valor entero sin decimales.
    if len(sep) == 1:
        return __parse(True, str(valor) + '.00')

    # Si la longitud de los decimales es 1, le agrego un 0 más.
    if len(sep[1]) == 1:
        return __parse(True, str(valor) + '0')

    # Sino retorno el registro sin modificar en string.
    return __parse(True, str(valor))


def str_to_array(str, separador):
    try:
        return str.split(separador)
    except:
        return None


def quitar_acentos(texto):
    '''
        Esta función se encarga de recibir un texto y quitarle los acentos y
        las eñes.
        Puede servir para concatenar str con problemas de caracteres.

        Argumento:
            texto (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str) // Retorno str, no unicode
    '''

    rta = {}

    try:
        texto = unicode(texto)
    except:
        try:
            texto = unicode(texto.decode('iso-8859-1'))
        except:
            # NOTA: Esto es porque Python 3 ya no tiene la función "unicode"
            pass

    texto = ''.join(
        (
            c for c in unicodedata.normalize('NFD', texto)
            if unicodedata.category(c) != 'Mn'
        )
    )

    # texto = str(texto.encode('iso-8859-1'))
    texto = texto.encode('iso-8859-1')

    texto = texto.decode('iso-8859-1')

    rta['Estado'] = True
    rta['Respuesta'] = texto
    return rta


def respuesta(estado, rta=None, dato=None):
    '''
        Esta función se encarga de recibir un estado y respuesta y devolverlo
        en forma de diccionario.

        Argumento:
            estado (bool)
            rta (obj)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (obj)
    '''

    # Declaro el dic con la llave estado.
    dic = {
        'Estado': estado
    }

    # Si recibo el campo rta, lo agrego al dic.
    if rta is not None:
        dic['Respuesta'] = rta

    # Lo mismo con dato.
    if dato is not None:
        dic['Dato'] = dato

    # Retorno.
    return dic


def decode_string(valor, charset='iso-8859-1'):
    '''
        Esta función se encarga de recibir un valor y trata de decodearlo a
        charset iso-8859-1.

        Argumento:
            valor (str)

        Retorna:
            valor (str)
    '''

    valor_string = ''

    if valor is not None:
        valor_string = valor.decode(charset)

    return valor_string


def cambiar_de_signo(valor):
    '''
    Esta función recibe un valor numerico y retorno el mismo valor pero con el
    signo cambiado.
    :param valor:
    :return: {Estado: bool, Respuesta: valor}
    '''
    # Valido que sea un número.
    v = validar.numero(valor)
    if v['Estado'] is not True:
        return v

    # Si es cero retorno el valor sin modificar.
    if valor == 0:
        return respuesta(True, valor)

    # Si es menor a 0 retorno el valor absoluto.
    if valor < 0:
        return respuesta(True, abs(valor))
    else:

        # Sino, retorno el valor en negativo.
        valor = valor - valor - valor

        # Retorno.
        return respuesta(True, valor)


def solo_numeros(valor):
    '''
        Esta función recibo un string con números y debe retornar solo los
        números concatenados y en formato int.

        Argumentos:
            valor (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str o int)
    '''

    # Por defecto creo "solo_num" en vacío para ir concatenando los números
    solo_num = ''

    # Recorro cada letra del valor recibido
    for numero in valor:
        # Si la letra no es dígito
        if not numero.isdigit():
            # Paso a la siguiente letra
            continue
        # Si la letra es un dígito concateno cada número
        solo_num += numero

    # Si el valor recibido no tenía ningún número
    if solo_num == '':
        # Retorno False y un mensaje
        return respuesta(
            False,
            _(u'No se pudo obtener el número de %s') % valor
        )

    # Si se pudieron obtener los números del valor recibido
    # retorno True y los números en formato int
    return respuesta(
        True,
        int(solo_num)
    )
