# -*- coding: utf-8 -*-

from subprocess import Popen
from subprocess import PIPE

from datetime import datetime


def obtener_memoria_consumida(pid):
    '''
        Esta función recibirá un PID de un proceso y devolverá la cantidad de
        memoria que está consumiendo en este momento.

        Argumentos:
            pid (int)

        Retorna:
            str
    '''

    cmd = 'pmap -d %s' % pid
    p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
    salida, error = p.communicate()

    salida = salida.decode('utf-8')

    if salida != '':
        ultima_linea = salida.splitlines()[-1:]
        ultima_linea = ultima_linea[0].split(' ')
        salida = ultima_linea[6]

    return salida


def log_memoria_consumida(pid, destino='/tmp/log_memoria.txt', etiqueta=''):
    '''
        Esta función se encarga de escribir en un archivo de texto
        la memoria consumida por un proceso dado, también guarda la
        fecha y hora en la que se ejecutó.

        Argumentos:
            pid (int)
            destino (str)
            etiqueta (str)

        Retorna:
            None
    '''

    memoria = obtener_memoria_consumida(pid)

    fecha_hora_actual = datetime.now()
    fecha_hora_actual = fecha_hora_actual.strftime("%d/%m/%Y %H:%M:%S")

    linea = '[%s] %s %s\n' % (fecha_hora_actual, memoria, etiqueta)

    linea = linea.encode('utf-8')

    archivo = open(destino, 'a+b')
    archivo.write(linea)
    archivo.close()
