# -*- coding: utf-8 -*-

import os
import time
from functools import wraps
from twisted.python import log

from at.utiles.log import start_custom_log
from at.utiles.errores import obtener_traceback
from at.utiles.archivos import leer_configobj

ruta_logs = '/var/lib/at/logs'
ruta_parametros = '/var/lib/at/etc/parametros.txt'


def debe_loguear():
    pass


def obtener_parametros():
    pass


def con_log(modulo='general', nombre='funciones', llave_subcarpeta=None, siempre=False):
    def real_decorator(mi_funcion):
        # @wraps permite que luego se pueda usar el nombre,parametros,docstring de la funcion,
        # porque sino la pisa con "con_log"
        # mas info en https://stackoverflow.com/questions/308999/what-does-functools-wraps-do/309000#309000
        @wraps(mi_funcion)
        def wrapper(*args, **kwargs):

            # Llama a la función leer_configobj para obtener los parámetros
            rta_leer_configobj = leer_configobj(ruta_parametros)

            # Si se pudieron obtener los parámetros
            if rta_leer_configobj['Estado'] is True and siempre is False:

                # verifica si se debe loguear las funciones
                if rta_leer_configobj['Respuesta'].get('con_log') is False:
                    # Si no se quieren loguear las funciones
                    
                    # ejecuta la función y sale
                    return mi_funcion(*args, **kwargs)

            # NOTA #############################################################
            # Por defecto voy a usar la ruta definida arriba
            nueva_ruta = ruta_logs
            # Si recibo una llave de una subcarpeta
            if llave_subcarpeta is not None:
                # Tengo que encontrar el valor de esa llave
                # para poder saber cual es la subcarpeta a agregar
                try:
                    # Voy a intentar obtener el nombre de la subcarpeta
                    nombre_subcarpeta = args[0].get(llave_subcarpeta)
                except:
                    # Si no se puede dejo el nombre de la subcarpeta en None
                    nombre_subcarpeta = None
                # Si tengo una subcarpeta para agregar
                if nombre_subcarpeta is not None:
                    # Voy a concatenar la ruta por defecto con la subcarpeta
                    nueva_ruta = ruta_logs + '/' + nombre_subcarpeta
                    # Si no existe esa ruta
                    if os.path.exists(nueva_ruta) is False:
                        # La creo por única vez
                        os.mkdir(nueva_ruta)
            ####################################################################

            # Inicio el log
            start_custom_log(nueva_ruta + '/' + modulo + '/' + modulo, nombre)

            # Obtengo el nombre de la función que se ejecutó
            try:
                nombre_funcion = mi_funcion.func_name
            except:
                # Pongo esto en un try porque cuando el código está compilado
                # no funciona :S
                nombre_funcion = ''

            # Obtengo la ruta del archivo de la función que se ejecutó
            try:
                ruta_funcion = mi_funcion.func_globals['__file__']
            except:
                # Pongo esto en un try porque cuando el código está compilado
                # no funciona :S
                ruta_funcion = ''

            # Guardo el tiempo antes de ejecutar la función
            ahora = time.time()

            # Ejecuto la función con sus parámetros
            rta = mi_funcion(*args, **kwargs)

            # Obtengo cuanto se tardó en ejecutar la función
            retardo = round(time.time() - ahora, 2)

            # Guardo los parámetros recibidos
            parametros = '%s - %s' % (args, kwargs)

            # Armo la información que quiero logear con todos los datos
            # importantes de la función ejecutada y su respuesta
            info = 'Funcion: %s | Archivo: %s | Parametros: %s | Respuesta: %s | Retardo: %s seg' \
                   % (nombre_funcion, ruta_funcion, parametros, rta, retardo)

            # Escribo en el log
            log.msg(info, mode=nombre)

            return rta

        return wrapper

    return real_decorator


def con_try(modulo='general', nombre='errores', llave_subcarpeta=None):
    def real_decorator(mi_funcion):
        @wraps(mi_funcion)
        def wrapper(*args, **kwargs):
            # Llamo a la función leer_configobj para obtener los parámetros
            rta_leer_configobj = leer_configobj(ruta_parametros)
            # Si se pudieron obtener los parámetros
            if rta_leer_configobj['Estado'] is True:
                # Me fijo si se deben ejecutar las funciones en un try
                if rta_leer_configobj['Respuesta'].get('con_try') is False:
                    # Si no se quieren ejecutar en un try
                    # solo ejecuto la función
                    return mi_funcion(*args, **kwargs)

            # NOTA #############################################################
            # Por defecto voy a usar la ruta definida arriba
            nueva_ruta = ruta_logs
            # Si recibo una llave de una subcarpeta
            if llave_subcarpeta is not None:
                # Tengo que encontrar el valor de esa llave
                # para poder saber cual es la subcarpeta a agregar
                try:
                    # Voy a intentar obtener el nombre de la subcarpeta
                    nombre_subcarpeta = args[0].get(llave_subcarpeta)
                except:
                    # Si no se puede dejo el nombre de la subcarpeta en None
                    nombre_subcarpeta = None
                # Si tengo una subcarpeta para agregar
                if nombre_subcarpeta is not None:
                    # Voy a concatenar la ruta por defecto con la subcarpeta
                    nueva_ruta = ruta_logs + '/' + nombre_subcarpeta
                    # Si no existe esa ruta
                    if os.path.exists(nueva_ruta) is False:
                        # La creo por única vez
                        os.mkdir(nueva_ruta)
            ####################################################################

            # Inicio el log
            start_custom_log(nueva_ruta + '/' + modulo + '/' + modulo, nombre)

            # Obtengo el nombre de la función que se ejecutó
            try:
                nombre_funcion = mi_funcion.func_name
            except:
                # Pongo esto en un try porque cuando el código está compilado
                # no funciona :S
                nombre_funcion = ''

            # Obtengo la ruta del archivo de la función que se ejecutó
            try:
                ruta_funcion = mi_funcion.func_globals['__file__']
            except:
                # Pongo esto en un try porque cuando el código está compilado
                # no funciona :S
                ruta_funcion = ''

            try:
                # Ejecuto la función con sus parámetros dentro de un try
                rta = mi_funcion(*args, **kwargs)

                # Retorno la respuesta de la función que se ejecutó
                return rta
            except Exception as ex:
                # Guardo los parámetros recibidos
                parametros = '%s - %s' % (args, kwargs)

                # Obtengo el traceback del error
                error = obtener_traceback() + ' ' + str(ex)

                # Armo la información que quiero logear con todos los datos
                # importantes de la función ejecutada y su respuesta
                info = 'Funcion: %s | Archivo: %s | Parametros: %s | Error: %s' \
                       % (nombre_funcion, ruta_funcion, parametros, error)

                # Escribo en el log
                log.msg(info, mode=nombre)

                # Retorno el error
                rta = {}
                rta['Estado'] = False
                rta['Respuesta'] = 'ERROR EN EL TRY'
            return rta

        return wrapper

    return real_decorator
