# -*- coding: utf-8 -*-

from hashlib import md5
from hashlib import sha1
from hashlib import sha256

from at.utiles.validaciones_generales import ValidacionesGenerales


class Encriptaciones(ValidacionesGenerales):
    '''
        Esta clase contiene métedos para realizar
        distintos tipos de encriptaciones.
    '''

    def encriptar_md5(self, valor):
        '''
            Recibe un str o unicode como parámetro y lo encripta a md5.

            Argumento:
                valor (str)

            Retorna:
                valor (str)

            Funciones:
                Utiles().validar_no_vacio
                Utiles().validar_string
        '''

        self.validar_no_vacio(valor)

        self.validar_string(valor)

        m = md5()
        m.update(valor.encode('utf-8'))
        valor = m.hexdigest()

        return valor

    def encriptar_sha1(self, valor):
        '''
            Recibe un str o unicode como parámetro y lo encripta a sha1.

            Argumento:
                valor (str)

            Retorna:
                valor (str)

            Funciones:
                Utiles().validar_no_vacio
                Utiles().validar_string
        '''

        self.validar_no_vacio(valor)

        self.validar_string(valor)

        m = sha1()
        m.update(valor.encode('utf-8'))
        valor = m.hexdigest()

        return valor

    def encriptar_sha256(self, valor):
        '''
            Recibe un str o unicode como parámetro y lo encripta a sha2.

            Argumento:
                valor (str)

            Retorna:
                valor (str)
        '''

        m = sha256()
        m.update(valor.encode('utf-8'))
        valor = m.hexdigest()

        return valor

    def encriptar_cp(self, s, key=3091974, salt=False, bDo=True):
        '''
            Esta función se encarga de encriptar las claves con una formula
            propia para los usuarios de CallProcessor.

            MODO DE USO
                encriptar_cp('elpassword', 3091974, False, True)
        '''

        n = i = 0
        ss = None
        k1 = 0
        k2 = 0
        k3 = 0
        k4 = 0

        if bDo:
            if len(s) == 1:
                s = s + " "
            elif len(s) == 0:
                s = " "

            n = len(s)
            ss = ''

            sn = []
            sn.append(0)

            k1 = 11 + (key % 233)
            k2 = 7 + (key % 239)
            k3 = 5 + (key % 241)
            k4 = 3 + (key % 251)

            for i in range(n):
                sn.append(ord(s[i:i + 1]))

            for i in range(2, n + 1):
                sn[i] = sn[i] ^ sn[i - 1] ^ (k1 * sn[i - 1] % 256)
            for i in range(n - 1, 0, -1):
                sn[i] = sn[i] ^ sn[i + 1] ^ k2 * sn[i + 1] % 256
            for i in range(3, n + 1):
                sn[i] = sn[i] ^ sn[i - 2] ^ k3 * sn[i - 1] % 256
            for i in range(n - 2, 0, -1):
                sn[i] = sn[i] ^ sn[i + 2] ^ k4 * sn[i + 1] % 256

            for i in range(1, n + 1):
                ss += ''.join(chr(sn[i]))
            ss = ss.replace("|", "LAZ")
            return ss.replace("'", "LEO")
        else:
            '''Si NO Encripta'''
            return s

    def desencriptar_cp(self, s, key=3091974, salt=False, bDo=True):
        '''
            Esta función se encarga de encriptar las claves con una formula
            propia para los usuarios de CallProcessor.

            MODO DE USO
                encriptar_cp('elpassword', 3091974, False, True)
        '''

        n = i = 0
        ss = None
        k1 = 0
        k2 = 0
        k3 = 0
        k4 = 0

        if bDo:
            s = s.replace("LAZ", "|")
            s = s.replace("LEO", "'")
            n = len(s)
            ss = ''
            sn = []
            sn.append(0)

            k1 = 11 + (key % 233)
            k2 = 7 + (key % 239)
            k3 = 5 + (key % 241)
            k4 = 3 + (key % 251)

            for i in range(n):
                sn.append(ord(s[i]))

            for i in range(n - 1):
                sn[i] = sn[i] ^ sn[i + 2] ^ (k4 * sn[i + 1] % 256)
            for i in range(n, 2, -1):
                sn[i] = sn[i] ^ sn[i - 2] ^ (k3 * sn[i - 1] % 256)
            for i in range(n):
                sn[i] = sn[i] ^ sn[i + 1] ^ (k2 * sn[i + 1] % 256)
            for i in range(n, 1, -1):
                sn[i] = sn[i] ^ sn[i - 1] ^ (k1 * sn[i - 1] % 256)

            for i in range(1, n + 1):
                ss += ''.join(chr(sn[i]))
            return ss
        else:
            ##'Si NO está encriptado
            return s
