# -*- coding: utf-8 -*-

import os
import xlwt
import math
import zipfile
from shutil import rmtree


class Exportar(object):

    def toXLS(self, file, title, headers, data, columns, num=1):
        '''
            Esta función se encarga de generar un archivo .xls

            Argumentos:
                file (str) # ruta de donde queremos que se cree el archivo
                title (str) # título de la pestaña del excel
                headers (list de str) # cabecera de los datos
                data (list de list) # datos separados por filas
                columns (list de int) # determina el tamaño de cada columna

            Retorna:
                file (str)
        '''

        maximo_row = 65000

        total_archivos = float(len(data)) / maximo_row
        total_archivos = int(math.ceil(total_archivos))
        if total_archivos > 1 or num != 1:
            # La carpeta tiene el mismo nombre que el archivo, sin la extension
            carpeta = file.replace('.xls', '') + '_zip_compress'
            if num == 1:
                # Intentamos limpiar si existe alguna carpeta o archivo
                # del mismo nombre
                if os.path.exists(carpeta):
                    # Si es archivo, lo borramos
                    if os.path.isfile(carpeta):
                        os.remove(carpeta)
                    # Sino, es carpeta, la borramos
                    else:
                        rmtree(carpeta)
                # Creamos la carpeta
                os.makedirs(carpeta)

            nombre_archivo = os.path.basename(
                carpeta.replace('_zip_compress', '')
            )
            nombre_archivo_nuevo = nombre_archivo + '-' + str(num) + '_de_'
            nombre_archivo_nuevo += str(total_archivos + num - 1) + '.xls'

            file_nuevo = os.path.join(carpeta, nombre_archivo_nuevo)

            crear_zip = True
        else:
            crear_zip = False
            file_nuevo = file

        outbook = xlwt.Workbook(encoding='latin')
        outsheet = outbook.add_sheet(title)

        style = xlwt.XFStyle()
        font = xlwt.Font()
        font.bold = True
        style.font = font

        for h in range(len(headers)):
            outsheet.write(0, h, headers[h], style)

        for row in range(len(data)):
            if row > maximo_row:
                break

            for column in range(len(data[row])):
                outsheet.write(row + 2, column, data[row][column])

        for i in range(len(columns)):
            width = columns[i] * 50
            if width > 0:
                outsheet.col(i).width = width

        outbook.save(file_nuevo)

        if crear_zip is True:
            zipeado = carpeta.replace('_zip_compress', '') + '.zip'
            zip = zipfile.ZipFile(zipeado, "a")
            zip.write(file_nuevo, nombre_archivo_nuevo, zipfile.ZIP_DEFLATED)
            zip.close()
            zip_creado = True
        else:
            zip_creado = False

        if total_archivos > 1:
            # Llamamos de forma recursiva, indicando al siguiente numerador
            self.toXLS(file, title, headers, data[maximo_row + 1:], columns,
                                                                    num + 1)
            # Cuando termina el padre de todos, borramos la carpeta
            if num == 1:
                rmtree(carpeta)
        else:
            if zip_creado is True:
                file = zipeado
            return file
