# -*- coding: utf-8 -*-

import datetime
from datetime import timedelta

from at.utiles import _
from at.utiles.servidores import DateTimeHelper


class Fecha(object):
    '''
        Clase encargada de manipular los formatos de las fechas.
    '''

    def __init__(self):
        self.dth = DateTimeHelper()

    def separar_fecha(self, fecha):
        '''
            Dada una fecha en el formato yyyy-mm-dd hh:mm:ss
            devuelve un diccionario con los volares separados
            de la siguiente manera:

            AÑO = fecha_separada['anio']
            MES = fecha_separada['mes']
            DÍA = fecha_separada['dia']
            HORA = fecha_separada['hora']
            MINUTO = fecha_separada['minuto']
            SEGUNDO = fecha_separada['segundo']
        '''

        if not '-' in fecha:
            raise _(
    'Formato de fecha incorrecto, el formato debe ser: yyyy-mm-dd hh:mm:ss.'
            )

        fecha_separada = {}
        fecha = fecha.split("-")
        fecha_separada['anio'] = fecha[0]
        fecha_separada['mes'] = fecha[1]
        fecha = fecha[2].split(" ")
        fecha_separada['dia'] = fecha[0]
        try:
            fecha = fecha[1].split(":")
            fecha_separada['hora'] = fecha[0]
            fecha_separada['minuto'] = fecha[1]
            fecha_separada['segundo'] = fecha[2]
        except:
            pass

        if len(fecha_separada['anio']) != 4:
            raise _(
    'Formato de fecha incorrecto, el formato debe ser: yyyy-mm-dd hh:mm:ss.'
            )

        return fecha_separada

    def fecha_datetime(self, anio, mes, dia, hora=None, minuto=None,
                            segundo=None):
        '''
            Convierte la fecha en formato str a formato datetime
        '''

        try:
            if hora is None and minuto is None and segundo is None:
                fecha_datetime = datetime.datetime(
                    int(anio), int(mes), int(dia)
                )
            else:
                fecha_datetime = datetime.datetime(
                    int(anio), int(mes), int(dia),
                    int(hora), int(minuto), int(segundo)
                )
        except ValueError as e:
            raise e

        return fecha_datetime

    def tiempo_esp(self, tiempo):
        '''
            Recibe un tiempo en inglés y lo devuelve en español

            Argumento:
                tiempo (datetime.timedelta)

            Retorna:
                tiempo_esp (str)
        '''

        tiempo_esp = str(tiempo)

        if 'day' in tiempo_esp:
            tiempo_esp = tiempo_esp.replace("day", u"día")

        if 'days' in tiempo_esp:
            tiempo_esp = tiempo_esp.replace("day", u"días")

        return tiempo_esp

    def obtener_dia_semana(self, fecha):
        '''
            Esta función devuelve el número de día correspondiente a la fecha
            dada.

            Argumentos:
                fecha (datetime)

            Retorna:
                Domingo = 1
                Lunes = 2
                Martes = 3
                Miércoles = 4
                Jueves = 5
                Viernes = 6
                Sábado = 7
        '''

        num_dia = fecha.strftime("%w")
        num_dia = int(num_dia) + 1

        return num_dia

    def pasar_a_date(self, fecha_str):
        '''
            Esta función se encarga de pasar a date una fecha dada en str
            con formato dd/mm/yyyy.

            Argumento:
                fecha_str (str)

            Retorna:
                Caso correcto:
                    dic['Estado'] = True
                    dic['Mensaje'] = fecha_date (datetime.date)
                Caso incorrecto:
                    dic['Estado'] = False
                    dic['Mensaje'] = 'un mensaje de error'
        '''

        dic = {}

        try:
            fecha_str = fecha_str.split("/")

            dia = fecha_str[0]
            mes = fecha_str[1]
            anio = fecha_str[2]

            fecha_date = datetime.date(int(anio), int(mes), int(dia))

            dic['Estado'] = True
            dic['Mensaje'] = fecha_date
            return dic
        except:
            dic['Estado'] = False
            dic['Mensaje'] = _(
        u'Fecha recibida no válida. El formato de la fecha debe ser dd/mm/yyyy.'
            )
            return dic

    def obtener_fechas_pedidas(self, fecha_desde, fecha_hasta, orden='desc'):
        '''
            Esta función devuelve todas las fechas que están entre las dos
            fechas dadas con estas incluidas.

            Argumentos:
                fecha_desde (date)
                fecha_hasta (date)
                orden (str) // puede ser asc o desc

            Retorna:
                list
        '''

        fechas = []

        if orden == 'asc':
            while fecha_hasta >= fecha_desde:
                fechas.append(fecha_desde)
                fecha_desde = fecha_desde + timedelta(days=1)
        else:
            while fecha_desde <= fecha_hasta:
                fechas.append(fecha_hasta)
                fecha_hasta = fecha_hasta - timedelta(days=1)

        return fechas

    def anio_actual(self):
        '''
            Esta función se encarga de devolver el año actual.

            Retorna:
                anio (int)
        '''

        fecha = datetime.datetime.now()
        anio = fecha.year

        return anio

    def obtener_nombre_dia(self, fecha):
        '''
            Esta función se encarga de devolver el nombre del día
            de la fecha dada.

            Argumentos:
                fecha (date) o (datetime)

            Retorna:
                nombre_dia (str)

            Funciones:
                Fecha().obtener_dia_semana
        '''

        num_dia = self.obtener_dia_semana(fecha)

        nombre_dia = ''

        if num_dia == 1:
            nombre_dia = _(u'Domingo')

        if num_dia == 2:
            nombre_dia = _(u'Lunes')

        if num_dia == 3:
            nombre_dia = _(u'Martes')

        if num_dia == 4:
            nombre_dia = _(u'Miércoles')

        if num_dia == 5:
            nombre_dia = _(u'Jueves')

        if num_dia == 6:
            nombre_dia = _(u'Viernes')

        if num_dia == 7:
            nombre_dia = _(u'Sábado')

        return nombre_dia

    def cambiar_formato(self, fecha_hora):
        '''
            Esta función se encarga de cambiar el formato de una fecha dada,
            recibo una fecha como string con el formato dd/mm/yyyy hh:mm:ss
            y debo transformarla a datetime en formato yyyy-mm-dd hh:mm:ss

            Argumento;
                fecha_hora (str) dd/mm/yyyy hh:mm:ss

            Funciones:
                pasar_a_date

            Retorna:
                fecha_hora (str) yyyy-mm-dd hh:mm:ss
        '''

        fecha_hora_separada = fecha_hora.split(" ")
        fecha = fecha_hora_separada[0]
        fecha_separada = fecha.split("/")
        if len(fecha_separada[2]) == 2:
            fecha = '%s/%s/20%s' % (
                fecha_separada[0],
                fecha_separada[1],
                fecha_separada[2]
            )

        hora = fecha_hora_separada[1]
        fecha = self.pasar_a_date(fecha)

        fecha_hora = fecha['Mensaje'].strftime("%Y-%m-%d") + " " + hora

        return fecha_hora

    def obtener_nombre_mes(self, fecha):
        '''
            Esta función se encarga de devolver el nombre del mes
            de la fecha dada.

            Argumentos:
                fecha int

            Retorna:
                nombre_mes (str)

            Funciones:

        '''

        num_mes = fecha

        nombre_mes = ''

        if num_mes == 1:
            nombre_mes = _(u'Enero')

        if num_mes == 2:
            nombre_mes = _(u'Febrero')

        if num_mes == 3:
            nombre_mes = _(u'Marzo')

        if num_mes == 4:
            nombre_mes = _(u'Abril')

        if num_mes == 5:
            nombre_mes = _(u'Mayo')

        if num_mes == 6:
            nombre_mes = _(u'Junio')

        if num_mes == 7:
            nombre_mes = _(u'Julio')

        if num_mes == 8:
            nombre_mes = _(u'Agosto')

        if num_mes == 9:
            nombre_mes = _(u'Septiembre')

        if num_mes == 10:
            nombre_mes = _(u'Octubre')

        if num_mes == 11:
            nombre_mes = _(u'Noviembre')

        if num_mes == 12:
            nombre_mes = _(u'Diciembre')

        return nombre_mes

    def obtener_fechas_horas_pedidas(self, fecha_hora_desde, fecha_hora_hasta,
                                        orden='desc'):
        '''
            Esta función devuelve todas las fechas y horas que están entre dos
            fechas y horas dadas.
            Las ordena según como se le indique

            Argumentos:
                fecha_hora_desde (datetime)
                fecha_hora_hasta (datetime)
                orden (str) // puede ser asc o desc

            Retorna:
                list
        '''

        fechas = []

        if orden == 'asc':
            while fecha_hora_hasta >= fecha_hora_desde:
                fechas.append(fecha_hora_desde)
                fecha_hora_desde = fecha_hora_desde + timedelta(hours=1)
        else:
            while fecha_hora_desde <= fecha_hora_hasta:
                fechas.append(fecha_hora_hasta)
                fecha_hora_hasta = fecha_hora_hasta - timedelta(hours=1)

        return fechas

    def pasar_a_datetime(self, fecha_hora):
        '''
            Esta función se encarga de pasar un string con formato
            dd/mm/yyyy hh:mm:ss a un datetime

            Argumentos:
                fecha_hora (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (datetime o str)
        '''

        rta = {}

        try:
            fecha = fecha_hora.split(' ')[0]
            hora = fecha_hora.split(' ')[1]

            rta_pasar_a_date = self.pasar_a_date(fecha)
            if rta_pasar_a_date['Estado'] is False:
                rta['Estado'] = False
                rta['Respuesta'] = rta_pasar_a_date['Mensaje']
                return rta

            fecha = rta_pasar_a_date['Mensaje']

            fecha_hora = str(fecha) + ' ' + hora

            fecha_hora = datetime.datetime.strptime(
                fecha_hora, '%Y-%m-%d %H:%M:%S'
            )

            rta['Estado'] = True
            rta['Respuesta'] = fecha_hora
            return rta
        except:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'El formato de la fecha debe ser dd/mm/yyyy hh:mm:ss')
            return rta

    def timedelta_to_datetime(self, dif_fecha_hora):
        '''
            Esta función recibe un timedelta y debe ser pasado a un datetime
            con la fecha en 1900-01-01 y la hora debe ser la diferencia en
            tiempo.

            Argumentos:
                dif_fecha_hora (timedelta)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (datetime o str)
        '''

        # Creo el diccionario a retornar
        rta = {}

        # Valido que haya recibido un timedelta
        if not isinstance(dif_fecha_hora, timedelta):
            # Si no recibo un timedelta retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'La diferencia horaria no es válida')
            return rta

        # Obtengo la diferencia de días
        dias = dif_fecha_hora.days

        # Si días no es 0
        if dias != 0:
            # Retorno un mensaje de error porque la diferencia entre fechas
            # es muy alta, solo nos sirve la diferencia horaria
            rta['Estado'] = False
            rta['Respuesta'] = _(u'La diferencia entre fechas es muy alta')
            return rta

        # Obtengo las horas de diferencia
        horas = dif_fecha_hora.seconds // 3600

        # Obtengo los minutos de diferencia
        minutos = dif_fecha_hora.seconds // 60 % 60

        # Obtengo los segundos de diferencia
        segundos = dif_fecha_hora.seconds % 60

        # Guardo la diferencia horaria en formato datetime
        fecha_hora = datetime.datetime(1900, 1, 1, horas, minutos, segundos)

        # Retorno True y el datetime obtenido
        rta['Estado'] = True
        rta['Respuesta'] = fecha_hora
        return rta

    def validar_horario(self, hora_desde, hora_hasta, fecha_hora_test=None):
        '''
            Esta función se encarga de indicar si el horario está dentro del
            permitido teniendo en cuenta la fecha y hora actual.

            Argumentos:
                hora_desde (datetime)
                hora_hasta (datetime)
                fecha_hora_test (datetime)

            Funciones:
                DateTimeHelper().get_date
                DateTimeHelper().get_now

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Si tengo una fecha y hora para test
        if fecha_hora_test is not None:
            # Uso la fecha del test
            fecha_actual = fecha_hora_test.date()
        else:
            # Si no, obtengo la fecha actual
            fecha_actual = self.dth.get_date()

        # Si tengo una fecha y hora para test
        if fecha_hora_test is not None:
            # Uso la fecha y hora del test
            fecha_hora_actual = fecha_hora_test
        else:
            # Obtengo la fecha y hora actual
            fecha_hora_actual = self.dth.get_now()

        # Armo la hora desde con la fecha
        hora_fecha_desde = str(fecha_actual) + ' ' + str(hora_desde.time())

        # Armo la hora hasta con la fecha
        hora_fecha_hasta = str(fecha_actual) + ' ' + str(hora_hasta.time())

        # Si la hora desde es mayor a la hora hasta
        if hora_desde > hora_hasta:
            # Paso hora_fecha_desde a datetime
            hora_fecha_desde = datetime.datetime.strptime(
                hora_fecha_desde, '%Y-%m-%d %H:%M:%S'
            )

            # Si la fecha y hora actual es menor a la fecha y hora desde
            # Esto solo si la hora desde es mayor a la hora hasta
            if fecha_hora_actual < hora_fecha_desde:
                # La hora desde corresponde al día anterior
                hora_fecha_desde = str(fecha_actual - timedelta(days=1)) + \
                                    ' ' + str(hora_desde.time())

                # La hora fecha hasta queda con el día actual
                hora_fecha_hasta = str(fecha_actual) + ' ' + \
                                                        str(hora_hasta.time())
            else:
                # La hora fecha desde queda con el día actual
                hora_fecha_desde = str(fecha_actual) + ' ' + \
                                                        str(hora_desde.time())

                # La hora hasta corresponde al día siguiente
                hora_fecha_hasta = str(fecha_actual + timedelta(days=1)) + \
                                    ' ' + str(hora_hasta.time())

        # Paso hora_fecha_desde a datetime
        hora_fecha_desde = datetime.datetime.strptime(
            hora_fecha_desde, '%Y-%m-%d %H:%M:%S'
        )

        # Paso hora_fecha_hasta a datetime
        hora_fecha_hasta = datetime.datetime.strptime(
            hora_fecha_hasta, '%Y-%m-%d %H:%M:%S'
        )

        # Si estoy dentro del rango de horario
        if fecha_hora_actual >= hora_fecha_desde \
                                    and fecha_hora_actual <= hora_fecha_hasta:
            # Retorno True y un mensaje
            rta['Estado'] = True
            rta['Respuesta'] = _(u'Está dentro del horario permitido')
            return rta

        # Si el horario no está dentro del rango retorno False y un mensaje
        rta['Estado'] = False
        rta['Respuesta'] = _(u'No está dentro del horario permitido')
        return rta
