# -*- coding: utf-8 -*-

import json
from httplib2 import Http
try:
    from urllib import urlencode
except:
    from urllib.parse import urlencode

from at.utiles.conversiones import respuesta


def enviar_por_http(url, params, metodo,
                    timeout=10, headers=None, tipo_params='url', utf_8=True,
                    force_params_in_url=False):
    '''
        Esta función se encarga de enviar por GET o por POST una petición HTTP.

        Argumentos:
            url (str)
            params (dict)
            metodo (str) // pueder ser "GET" o "POST"
            timeout (int) // 10 segundos por defecto
            headers (dict)
            tipo_params (str)
            utf_8 (bool)
            force_params_in_url (bool)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (dict o str)
    '''

    # Si headers es None
    if headers is None:
        # Utilizo una cabecera genérica
        headers = {'Content-type': 'application/x-www-form-urlencoded'}

    # Si el tipo de parametro es 'json'
    if tipo_params == 'json':
        # Paso los parámetros a formato JSON
        parametros = json.dumps(params)
    else:
        # Si el tipo de parámetros es 'url' paso los parámetros a formato URL
        # Si utf_8 es True
        if utf_8 is True:
            # Le paso que debe ser utf-8
            parametros = urlencode(params, 'utf-8')
        else:
            # Si utf_8 es False no le paso nada
            parametros = urlencode(params)

        # Si se va a enviar por GET
        if (metodo == 'GET' and parametros != '') \
                or (force_params_in_url is True and parametros != ''):
            # Concateno la URL con los parámetros
            url += '?' + parametros
            # Seteo lo parámetros en None para evitar problemas con el "body"
            parametros = None

    # Creo el HTTP
    http = Http(disable_ssl_certificate_validation=True, timeout=timeout)

    try:
        # Hago el request
        resp, content = http.request(
            url, metodo, headers=headers, body=parametros
        )
    except Exception as error:
        return respuesta(False, str(error))

    # Retorno la respuesta obtenida
    dic = {}
    dic['resp'] = resp
    dic['content'] = content
    return respuesta(True, dic)
