# -*- coding: utf-8 -*-

import os
import math
from datetime import datetime
from datetime import timedelta
from glob import glob

from at.utiles import _
from at.utiles.parametros import Parametros
from at.utiles.servidores import DateTimeHelper
from at.utiles.montar_dir import montar_dir
from at.utiles.fecha import Fecha
from at.utiles.montar_dir import montar_logs_moviles

MLOGS = os.path.join('/tmp', 'mlogs-tmp')
RUTA_LOGS_LINUX = '/var/lib/at/logs_moviles'
RUTA_LOGS_LINUX_USUARIOS = '/var/lib/at/logs_usuarios'
TIEMPO_MAXIMO = timedelta(seconds=1000000000)


class LogParser(object):
    def __init__(self):
        self.parametros = Parametros()
        self.dth = DateTimeHelper()
        # Obtenemos la ip de los logs

        self.minima_diferencia = timedelta(seconds=10)
        self.maxima_diferencia = timedelta(seconds=30)
        self.ip_logs = self.parametros.obtener_parametrolocal('IpLogs')
        self.fecha = Fecha()

    def parse(self, linea):
        dato = linea.split('')
        return dato

    def fdate(self, arg):
        '''
            Esta función se encarga de armar bien el formato de la fecha,
            recibe el día o el mes y si es menor que 10 se le agrega un 0
            adelante, por ejemplo si recibe 5 devuelve '05'

            Argumento:
                arg (int)

            Retorna:
                arg (str)
        '''

        if arg < 10:
            arg = '0%s' % arg

        return str(arg)

    def logs_path(self, date):
        '''
            Esta función se encarga de armar la ruta donde se encuentran
            los logs según la fecha pasada como argumento.

            Argumento:
                date (date)

            Retorna:
                path (str) # 2011-05/27

            Funcion:
                LogParser().fdate
        '''

        year_month_path = '%s-%s' % (date.year, self.fdate(date.month))
        day_path = self.fdate(date.day)
        path = os.path.join(year_month_path, day_path)

        return path

    def grabar_log(self, id_movil, latitud='', longitud='', velocidad='',
                   estado1='', digital='', estado_gps='', edad_gps='',
                   ultima_posicion='', ip_movil='', num_mensaje='',
                   fecha_posicion='', rumbo='', evento='', precision='',
                   licencia_empresa='', id_flota='', carpeta_log='log2',
                   empresa='', estado_sesion=2, fecha_log=None,
                   estado_panico=0):
        '''
            Esta función se encarga de grabar un log tal como se generan los
            logs de los móviles que utilizan consola.

            Argumentos:
                id_movil (str)
                latitud (float)
                longitud (float)
                velocidad (float)
                estado1 (int)
                digital (str)
                estado_gps (int)
                edad_gps (str)
                ultima_posicion (datetime)
                ip_movil (str)
                num_mensaje (str)
                fecha_posicion (datetime)
                rumbo (int)
                evento (int)
                precision (float)
                licencia_empresa (int)
                id_flota (int)
                carpeta_log (str)
                empresa (str)
                estado_sesion (int)
                                1 = inicio
                                2 = conectado
                                3 = salir
                                4 = desconectado
                fecha_log (date)
                estado_panico (int)
                                0 = Nada
                                1 = Alerta
                                2 = Botón asiento
                                3 = Botón baúl

            Funciones:
                obtener_parametrolocal
                validar_mount
                montar_dir
                get_date
                logs_path

            Retorna:
                None
        '''

        if carpeta_log == 'usuarios':
            # Si carpeta_log es usuario la ruta debe ser la siguiente
            ruta_logs_moviles = RUTA_LOGS_LINUX_USUARIOS
        else:
            # Guardo en la ruta de linux
            ruta_logs_moviles = RUTA_LOGS_LINUX

        # RUTA DE LOGS PARA EMPRESAS WEB ######################################
        # Los logs se van a guardar en:
        # /var/lib/at/logs/empresas/1_66/logs_moviles
        # Este es un ejemplo
        # Si recibo una empresa
        if empresa != '':
            # Seteo la ruta de los logs nuevamente porque por defecto ya está
            # en "/var/lib/at/logs_moviles"
            ruta_logs_moviles = '/var/lib/at'
            # Tengo que guardar los logs en la carpeta "empresas"
            ruta_logs_moviles = os.path.join(ruta_logs_moviles, 'empresas')
            # Si la carpeta "/var/lib/at/empresas" no está creada
            if not os.path.isdir(ruta_logs_moviles):
                # La creamos
                os.makedirs(ruta_logs_moviles)
            # Además de usar la carpeta "empresas" se debe guarda la información
            # en la carpeta con el nombre de la empresa
            ruta_logs_moviles = os.path.join(ruta_logs_moviles, empresa)
            # Si la carpeta "/var/lib/at/empresas/nombre_empresa" no está creada
            if not os.path.isdir(ruta_logs_moviles):
                # La creamos
                os.makedirs(ruta_logs_moviles)
            # Y por último, ahora si, agregamos la carpeta "logs_moviles"
            ruta_logs_moviles = os.path.join(ruta_logs_moviles, 'logs_moviles')
            # Si la carpeta "/var/lib/at/empresas/nombre_empresa/logs_moviles"
            # no está creada
            if not os.path.isdir(ruta_logs_moviles):
                # La creamos
                os.makedirs(ruta_logs_moviles)
        #######################################################################

        # Obtenemos la ruta correspondiente a la fecha
        if fecha_log is None:
            fecha_actual = self.dth.get_date()
        else:
            fecha_actual = fecha_log
        ruta_fecha = self.logs_path(fecha_actual)
        ruta = os.path.join(ruta_logs_moviles, ruta_fecha)
        # Si no esta creada, la creamos
        if not os.path.isdir(ruta):
            os.makedirs(ruta)

        id_flota = str(id_flota)
        if len(id_flota) < 2:
            agregar_cero = 2 - len(id_flota)
            id_flota = '0' * agregar_cero + id_flota

        id_movil = str(id_movil)
        id_movil_para_el_nombre = id_movil
        if len(id_movil) < 4:
            agregar_cero = 4 - len(id_movil)
            id_movil_para_el_nombre = '0' * agregar_cero + id_movil

        nombre_log = id_flota + '-' + \
                    id_movil_para_el_nombre + '-' + \
                    str(fecha_actual) + '.log'
        ruta = os.path.join(ruta, nombre_log)

        if ultima_posicion != '':
            ultima_posicion = ultima_posicion.strftime("%d/%m/%Y %H:%M:%S")

        if fecha_posicion != '':
            fecha_posicion = fecha_posicion.strftime("%d/%m/%Y %H:%M:%S")

        sep = ''

        registro = str(id_movil) + sep + \
                    str(latitud) + sep + \
                    str(longitud) + sep + \
                    str(velocidad) + sep + \
                    str(estado1) + sep + \
                    str(digital) + sep + \
                    str(estado_gps) + sep + \
                    str(edad_gps) + sep + \
                    str(ultima_posicion) + sep + \
                    str(ip_movil) + sep + \
                    str(num_mensaje) + sep + \
                    str(fecha_posicion) + sep + \
                    str(rumbo) + sep + \
                    str(evento) + sep + \
                    str(precision) + sep + \
                    str(licencia_empresa) + sep + \
                    str(estado_sesion) + sep + \
                    str(estado_panico)

        archivo = open(ruta, 'a')
        archivo.write(registro + '\n')
        archivo.close()

    def obtener_archivos_log(self, directorio, fecha_hora, id_movil='',
                                                            flag_mount=True):
        ''' Esta función devuelve la ruta, si existe, del archivo
        correspondiente al log de moviles de una fecha determinada '''

        # Guardo la fecha y hora recibida
        fecha_hora_original = fecha_hora

        # Si el directorio trabaja en localhost vamos a usar
        # la carpeta interna
        if directorio.find('localhost') != -1 \
                or directorio.find('127.0.0.1') != -1 \
                or directorio.find('0.0.0.0') != -1:
            directorio = RUTA_LOGS_LINUX
        else:
            # Si hay que montar, montamos el directorio
            if flag_mount:
                # Chequeamos el directorio de logs
                if not os.path.isdir(MLOGS):
                    os.makedirs(MLOGS)

                # Intentamos montarlo, si no esta montado
                rta_mount = montar_dir(directorio, MLOGS)
                # Si fallamos, salimos
                if not rta_mount['Estado']:
                    return rta_mount
                # Modificamos el directorio
                directorio = MLOGS

        if isinstance(fecha_hora, str):
            # obtenemos la fecha y la hora
            fecha_hora = datetime.strptime(fecha_hora, '%d/%m/%Y %H:%M:%S')

        # Obtenemos los datos de cada dia
        anio_mes = fecha_hora.strftime('%Y-%m')
        dia = fecha_hora.strftime('%d')
        directorio = os.path.join(directorio, anio_mes, dia)

        # Declaramos nuestra respuesta
        rta = {}
        # Si el directorio no existe, salimos
        if not os.path.isdir(directorio):
            rta['Estado'] = False
            rta['Respuesta'] = _('No se encontraron datos para la fecha dada')
            return rta

        # Si el directorio existe
        files = os.listdir(directorio)
        archivos = []
        if id_movil == '' or id_movil is None:
            for file in [f for f in files]:
                archivos.append(os.path.join(directorio, file))
        else:
            # Si es mas corto el id_movil, lo acomodamos
            if len(id_movil) < 4:
                id_movil = ((4 - len(id_movil)) * '0') + id_movil
            # Generamos el nombre del log
            nombre_log = id_movil + '-' + anio_mes + '-' + dia
            for file in [f for f in files]:
                if str(file).find(nombre_log) != -1:
                    archivos.append(os.path.join(directorio, file))

        # Si se encontraron archivos
        if len(archivos) > 0:
            rta['Estado'] = True
            rta['Respuesta'] = archivos
        else:
            if directorio != RUTA_LOGS_LINUX:
                # Como no encontro el log vuelvo a ejecutar la funcion
                # indicandole que lo busque en la ruta local
                return self.obtener_archivos_log(
                    'localhost',
                    fecha_hora_original,
                    id_movil,
                    flag_mount
                )
            else:
                rta['Estado'] = False
                rta['Respuesta'] = _('No se encontro un archivo de ese movil')
        # Devolvemos
        return rta

    def obtener_datos_log_por_hora(self, archivo, fecha_hora,
                                fecha_hora_fin=None, aproximado=False,
                                alarma=False, validar_gps=True):
        '''
            Esta función recorre un archivo y tomando como inicio una fecha
            y hora, busca el dato mas próximo de un móvil en el log del mismo

            Argumentos:
                archivo: (str) # El path del archivo
                fecha_hora (datetime) # La fecha hora de inicio
                fecha_hora_fin: (datetime) # La fecha hora fin.

            Retorna:
                datos: lista(str)
        '''

        # Definimos la respuesta
        rta = {'Estado': False}
        # Abrimos el archivo
        try:
            f = open(archivo, 'r')
        except:
            rta['Respuesta'] = _(
                u'No se pudo abrir el archivo. Compruebe la ruta del mismo.'
            )
            return rta

        # Obtenemos y guardamos los punteros de cada linea del archivo
        punteros_a_lineas = []
        puntero = 0
        for line in f:
            punteros_a_lineas.append(puntero)
            puntero += len(line)
        cantidad_lineas = len(punteros_a_lineas)

        # Nos posicionamos al principio del archivo
        f.seek(0)
        linea_actual = 0
        i = 0

        # Seteamos las variables de referencia
        linea_mas_cercana = 0
        tiempo_mas_cercano = TIEMPO_MAXIMO
        # El desplazamiento es una variable que se va achicando a medida que
        # da vueltas el while. Esta es proporcional a la cantida de lineas del
        # archivo y es el que indica cuanto tenemos que mover para adelante
        # o hacia atras la posicion actual del puntero
        desplazamiento = int(math.floor(cantidad_lineas * (2 ** - (i + 1))))

        while desplazamiento > 0:
            # Obtenemos la fecha de la linea en la cual estamos parados
            fh_actual = self.obtener_fecha_linea_actual_log(f)

            # Obtenemos la diferencia entre la fecha buscada y la fecha
            # encontrada, teniendo cuidado de no obtener un delta negativo
            if fh_actual > fecha_hora:
                delta = fh_actual - fecha_hora
            else:
                delta = fecha_hora - fh_actual

            # Si es es delta mas chico encontrado, guardamos referencia a su
            # linea
            if delta < tiempo_mas_cercano:
                tiempo_mas_cercano = delta
                linea_mas_cercana = linea_actual

            # Si el delta es lo suficientemente chico, es por que la fecha
            # encontrada esta muy cercana a la buscada, por lo que salimos,
            # leyendo el resto de lineas que puedan estar dentro del rango
            if delta < self.minima_diferencia:
                rta['Estado'] = True
                rta['Respuesta'] = self.procesar_lineas_log(
                    f, fecha_hora, fecha_hora_fin, aproximado, alarma,
                    validar_gps=validar_gps
                )
                f.close()
                return rta

            # Si la fecha encontrada no esta lo suficientemente cerca de la
            # buscada nos movemos en el archivo. Si es menor, nos movemos hacia
            # adelante, sino, hacia atras
            if fh_actual > fecha_hora:
                linea_actual -= desplazamiento
                i += 1
            else:
                linea_actual += desplazamiento
                i += 1

            # Intentamos desplazar el puntero a la nueva linea actual
            try:
                f.seek(punteros_a_lineas[linea_actual])
            except:
                rta['Respuesta'] = _(u'Fallo al momento de mover puntero.')
                f.close()
                return rta

            # Recalculamos el desplazamiento, para la proxima vuelta del while.
            # Cada vuelta se va achicando la mitad.
            desplazamiento = int(
                                math.floor(cantidad_lineas * (2 ** - (i + 1))))

        # Si llegamos hasta aca es por que no encontramos nada lo suficiente -
        # mente cercano, pero vamos a ver si dentro del rango de la linea
        # mas cercana encontramos lo que buscamos.

        # Obtenemos la linea mas cercana
        try:
            f.seek(punteros_a_lineas[linea_mas_cercana])
            fh_actual = self.obtener_fecha_linea_actual_log(f)
        except:
            # Si no se encontro la fh_actual, salimos
            f.close()
            rta['Respuesta'] = _(u'Fallo al momento de mover puntero.')
            return rta

        # Obtenemos la linea anterior a la mas cercana
        try:
            # Esto lo ponemos para que no busque en la posicion -1 y de la
            # vuelta al final del archivo
            if linea_mas_cercana > 0:
                f.seek(punteros_a_lineas[linea_mas_cercana - 1])
                fh_anterior = self.obtener_fecha_linea_actual_log(f)
            else:
                fh_anterior = fh_actual
        except:
            fh_anterior = fh_actual

        # Obtenemos la linea posterior a la mas cercana
        try:
            f.seek(punteros_a_lineas[linea_mas_cercana + 1])
            fh_posterior = self.obtener_fecha_linea_actual_log(f)
        except:
            fh_posterior = fh_actual
        # Si la fecha de la mas cercana es mayor a la buscada, vamos a ver
        # si tenemos suerte viendo en la fecha anterior a la mas cercana
        if fh_actual > fecha_hora:
            if fh_anterior > fecha_hora:
                delta = fh_anterior - fecha_hora
            else:
                delta = fecha_hora - fh_anterior
            # Si la anterior a la mas cercana esta todavia mas cerca vamos
            # a devolver los datos de la misma
            if delta < self.minima_diferencia or delta < tiempo_mas_cercano:
                # Ubicamos el puntero
                try:
                    f.seek(punteros_a_lineas[linea_mas_cercana - 1])
                except:
                    # Si no se encontro la fh_actual, salimos
                    f.close()
                    rta['Respuesta'] = _(u'Fallo al momento de mover puntero.')
                    return rta

                # Procesamos las lineas
                rta['Estado'] = True
                rta['Respuesta'] = self.procesar_lineas_log(
                    f, fecha_hora, fecha_hora_fin, aproximado, alarma,
                    validar_gps=validar_gps
                )
                f.close()
                return rta

        # En cambio, si la fecha mas cercana es menor a la mas buscada, vamos
        # a ver si la fecha posterior a la mas cercana es la correcta
        else:
            if fh_posterior > fecha_hora:
                delta = fh_posterior - fecha_hora
            else:
                delta = fecha_hora - fh_posterior
            # Si la posterior a la mas cercana esta todavia mas cerca, vamos
            # a devolver los datos de la misma
            if delta < self.minima_diferencia or delta < tiempo_mas_cercano:
                # Ubicamos el puntero
                try:
                    f.seek(punteros_a_lineas[linea_mas_cercana + 1])
                except:
                    # Si no se encontro la fh_actual, salimos
                    f.close()
                    rta['Respuesta'] = _(u'Fallo al momento de mover puntero.')
                    return rta
                # Procesamos las lineas
                rta['Estado'] = True
                rta['Respuesta'] = self.procesar_lineas_log(
                    f, fecha_hora, fecha_hora_fin, aproximado, alarma,
                    validar_gps=validar_gps
                )
                f.close()
                return rta

        # Si ni la anterior ni la posterior eran mas cercanas a la buscada
        # devolvemos la mas cercana que habiamos encontrado con el metodo
        # de busqueda anterior
        try:
            f.seek(punteros_a_lineas[linea_mas_cercana])
        except:
            # Si no se encontro la fh_actual, salimos
            f.close()
            rta['Respuesta'] = _(u'Fallo al momento de mover puntero.')
            return rta

        # Armamos la respuesta
        rta['Estado'] = True
        rta['Respuesta'] = self.procesar_lineas_log(
            f, fecha_hora, fecha_hora_fin, aproximado, alarma,
            validar_gps=validar_gps
        )
        f.close()
        return rta

    def obtener_fecha_linea_actual_log(self, f):
        ''' Esta función recibe un objeto de archivo, lee la línea en la que
        se encuentra parada, mueve el puntero al punto inicial y retorna
        la fecha de la línea que leyo.

        Argumentos:
            f : (objeto Archivo)
        Retorna:
            fecha_hora: (datetime)
        '''

        # Leemos la linea
        linea = f.readline()
        # Obtenemos el dato y la fecha
        dato = self.parse(linea)
        try:
            fecha_hora = datetime.strptime(
                dato[8],
                "%d/%m/%Y %H:%M:%S"
            )
        except:
            fecha_hora = datetime.strptime(
                '01/01/1900 00:00:00',
                "%d/%m/%Y %H:%M:%S"
            )
        # Volvemos el puntero a la posicion original
        f.seek(f.tell() - len(linea))
        # Retornamos la fecha hora
        return fecha_hora

    def procesar_lineas_log(self, f, fecha_hora, fecha_hora_fin=None,
                            aproximado=False, alarma=False, validar_gps=True):

        ''' Esta función recibe un objeto de archivo, una fecha hora y una
        fecha hora fin.
        Lee una línea o las líneas hasta la fecha hora fin especificada y
        devuelve los datos en un array '''

        # Declaramos la lista de datos a devolver
        datos = []
        # Declaramos una lista que nos ayudara a evitar fechas repetidas
        fechas_horas_agregadas = []
        # Declaramos flag que nos permitira hacer una validacion con la
        # primera linea que encontremos
        flag_primera_linea = True
        # Seteamos el flag en caso de estar buscando alarmas
        flag_alarma = False
        # Recorremos las lineas desde la poscicion actual
        for linea in f:
            # Obtenemos los datos y la fecha
            dato = self.parse(linea)
            try:
                try:
                    # Intentamos obtener datetime desde fecha y hora
                    fecha_hora_tmp = datetime.strptime(
                        dato[8].strip(),
                        "%d/%m/%Y %H:%M:%S"
                    )
                except:
                    try:
                        #Si falla, intentamos tener datetime desde fecha
                        fecha_hora_tmp = datetime.strptime(
                            dato[8].strip(),
                            "%d/%m/%Y"
                        )
                    except:
                        # Sino, salimos y seguimos de largo ya que no tenemos
                        # fechas. Puede que nos estes llegando un \r\n
                        continue

                # Si la fecha hora que estamos leyendo no esta ni cerca
                # de la maxima diferencia, salimos
                if flag_primera_linea:

                    delta_inicio = fecha_hora - self.maxima_diferencia

                    # Si tenemos el flag de aproximado y la fecha de inicio
                    # es menor, entonces, seguios leyendo lineas hasta
                    # encontrar una fecha de inicio coherente
                    if fecha_hora_tmp < delta_inicio and aproximado:
                        continue

                    # Si hay una fecha fin, validamos que este en el rango
                    if fecha_hora_fin is not None:
                        # Validamos que la fecha de la primera linea este
                        # dentro del rango horario buscado
                        delta_fin = fecha_hora_fin + self.maxima_diferencia
                        if fecha_hora_tmp < delta_inicio or \
                           fecha_hora_tmp > delta_fin:
                            break
                    else:
                        # Validamos que la fecha de la primera linea este cerca
                        # de la fecha buscada
                        if fecha_hora_tmp < delta_inicio:
                            break
                    # Desactivamos el flag
                    flag_primera_linea = False

                # Si estamos buscando alarmas, validamos los datos
                if alarma:
                    if int(dato[4]) & 3 == 2:
                        if flag_alarma:
                            # Continuamos con el for
                            continue
                        else:
                            flag_alarma = True
                    else:
                        # Cambiamos el flag y continuamos
                        flag_alarma = False
                        continue
                # Si no hay fecha fin, devolvemos un solo datos
                if fecha_hora_fin is None:
                    # Si tiene señal y el gps es valido, continuamos
                    if validar_gps:
                        if int(dato[4]) & 4 == 0 and (
                           dato[6] == '1' or dato[6] == '2' or dato[6] == '3'):
                            # Solo si los datos son confiables, los agregamos
                            datos.append(dato)
                    else:
                        datos.append(dato)
                    break

                # Sino, devolvemos los datos que encontremos que sean menores
                # a la fecha fin
                if fecha_hora_tmp < fecha_hora_fin + self.minima_diferencia:
                    if validar_gps:
                        # Si tiene señal y el gps es valido, continuamos
                        if int(dato[4]) & 4 == 0 and (
                           dato[6] == '1' or dato[6] == '2' or dato[6] == '3'):
                            # Solo si los datos son confiables, los agregamos
                            # Validamos no repetir la hora
                            if not fecha_hora_tmp in fechas_horas_agregadas:
                                datos.append(dato)
                                fechas_horas_agregadas.append(fecha_hora_tmp)
                    else:
                        # Validamos que no este repetido
                        if not fecha_hora_tmp in fechas_horas_agregadas:
                            datos.append(dato)
                            fechas_horas_agregadas.append(fecha_hora_tmp)
                else:
                    break
            except Exception as e:
                print(str(e))
                pass
        # Devolvemos el array
        return datos

    def obtener_datos_historicos_en_directorio(self, directorio, fecha_hora,
                                     fecha_hora_anterior=None, alarma=False):
        '''
            Función que obtiene los datos historicos de los logs en un
            directorio remoto, montadolo en el sistema.

            Arumentos:
                directorio: (str) # Path del directorio donde estan los logs
                fecha_hora: (datetime) # Fecha y hora que sebusca
        '''
        # Declaramos respuesta
        rta = {'Estado': False}
        # Validamos la fecha hora
        if not isinstance(fecha_hora, datetime):
            try:
                fecha_hora = datetime.strptime(
                    fecha_hora, "%d/%m/%Y %H:%M:%S"
                )
            except:
                rta['Respuesta'] = _(
                    u'''La fecha hora debe ser un datetime o un string con el
 formato dd/mm/yyyy hh:mm:ss.'''
                )
                return rta

        # Si necesitamos la fecha anterior, la pareseamos
        if fecha_hora_anterior is not None:
            if not isinstance(fecha_hora_anterior, datetime):
                try:
                    fecha_hora_anterior = datetime.strptime(
                        fecha_hora_anterior, "%d/%m/%Y %H:%M:%S"
                    )
                except:
                    rta['Respuesta'] = _(
                    u'''La fecha hora debe ser un datetime o un string con el
 formato dd/mm/yyyy hh:mm:ss.'''
                    )
                    return rta

        # Buscamos los archivos de logs
        rta = self.obtener_archivos_log(directorio, fecha_hora)
        # Si falla, salimos
        if not rta['Estado']:
            return rta

        # Recorremos los archivos y obtenemos los datos de cada uno
        historicos = []
        for archivo in [a for a in rta['Respuesta']]:
            if fecha_hora_anterior is not None:
                datos = self.obtener_datos_log_por_hora(
                    archivo,
                    fecha_hora_anterior,
                    fecha_hora,
                    alarma=alarma
                )
            else:
                datos = self.obtener_datos_log_por_hora(archivo, fecha_hora)
            # Si salio todo bien, guardamos los datos
            if datos['Estado']:
                if len(datos['Respuesta']):
                    historicos.append(datos['Respuesta'][0])

        # Si fallo por algun motivo, salimos
        if not datos['Estado']:
            return datos

        # Devolvemos los datos
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = historicos

        return rta

    def log_reloj(self, ruta_archivo, dia_desde, dia_hasta, tipo_dato="Viaje",
                                                        rr2=False, rr1=False):
        '''
            Esta función se encarga de obtener todos los datos de un archivo
            de log dado desde un día a otro.

            Argumentos:
                ruta_archivo (str)
                dia_desde (int)
                dia_hasta (int)
                tipo_dato (str)
                    // "Viaje" es para tener en cuenta cada servicio realizado
                rr2 (bool)
                rr1 (bool)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (list de list)
                                            [0] tipo (str)
                                            [1] licencia (str)
                                            [2] equipo (str)
                                            [3] num_serie_reloj (str)
                                            [4] tipo_paquiete (str)
                                            [5] boleto (str)
                                            [6] indice (str)
                                            [7] turno (str)
                                            [8] vel_max_libre (str)
                                            [9] vel_max_ocupado (str)
                                            [10] km_libre (str)
                                            [11] km_ocupado (str)
                                            [12] km_libre_del_turno (str)
                                            [13] km_ocupado_del_turno (str)
                                            [14] importe (str)
                                            [15] acumulado (str)
                                            [16] fecha_hora_inicio (str)
                                            [17] fecha_hora_fin (str)
                                            [18] duraccion (str)
        '''

        rta = {}
        lista = []

        try:
            archivo = open(ruta_archivo, 'rb')
        except:
            rta['Estado'] = False
            rta['Respuesta'] = lista
            return rta

        lineas = archivo.xreadlines()

        sum_km_libres = 0

        for linea in lineas:
            if linea == '':
                continue

            # Limpiamos los saltos de linea
            linea = linea.replace('\r\n', '').replace('\n', '')
            linea = linea.split(chr(9))

            if linea[0] != tipo_dato:
                continue

            if rr2 is False and rr1 is False:
                # Si RR2 y RR1 es False solo me quedo con los RR0
                if linea[4] != 'RR0':
                    continue
            elif rr2 is True and rr1 is False:
                # Si RR2 es True y RR1 es False me quedo con los RR2 y los RR0
                if linea[4] != 'RR0' and linea[4] != 'RR2':
                    continue
            elif rr2 is False and rr1 is True:
                # Si RR2 es False y RR1 es True me quedo con los RR1 y los RR0
                if linea[4] != 'RR0' and linea[4] != 'RR1':
                    continue
            elif rr2 is True and rr1 is True:
                # Si RR2 es True y RR1 es True me quedo con
                # los RR2, los RR1 y los RR0
                if linea[4] != 'RR0' and linea[4] != 'RR2' \
                                                        and linea[4] != 'RR1':
                    continue

            if linea[6] == '492':
                continue

            if linea[6] == '493':
                sum_km_libres = float(linea[10])
                continue

            if linea[4] == 'RR2':
                lista.append(linea)
                continue

            fecha_hora_fin = linea[17]
            fecha_hora_fin_split = fecha_hora_fin.split(' ')
            fecha_fin_split = fecha_hora_fin_split[0].split('/')
            dia_log = fecha_fin_split[0]

            if int(dia_log) < dia_desde:
                continue

            if int(dia_log) > dia_hasta:
                break

            if sum_km_libres != 0:
                km_libres = float(linea[10]) + sum_km_libres
                linea[10] = str(km_libres)
                sum_km_libres = 0

            num_ultimo_dato = len(linea) - 1
            linea[num_ultimo_dato] = linea[num_ultimo_dato].replace('\r\n', '')

            fecha_hora_inicio_sep = linea[16].split(' ')
            fecha_inicio_sep = fecha_hora_inicio_sep[0].split('/')
            fecha_hora_inicio = (
                fecha_inicio_sep[2] + '-' +
                fecha_inicio_sep[1] + '-' +
                fecha_inicio_sep[0] + ' ' +
                fecha_hora_inicio_sep[1] + ':00'
            )

            fecha_hora_fin_sep = linea[17].split(' ')
            fecha_fin_sep = fecha_hora_fin_sep[0].split('/')
            fecha_hora_fin = (
                fecha_fin_sep[2] + '-' +
                fecha_fin_sep[1] + '-' +
                fecha_fin_sep[0] + ' ' +
                fecha_hora_fin_sep[1] + ':00'
            )

            linea[16] = fecha_hora_inicio
            linea[17] = fecha_hora_fin

            lista.append(linea)

        archivo.close()

        if len(lista) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = lista
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def obtener_log_reloj(self, fecha_desde, fecha_hasta, id_ciudad,
                        id_tipo_movil, licencia, tipo_dato='Viaje',
                        rr2=False, rr1=False):
        '''
            Esta función se encarga de llamar a la función obtener_log_reloj
            pasandole de a una fecha entre la fecha desde y la fecha hasta

            Argumentos:
                fecha_desde (str) 'dd/mm/yyyy'
                fecha_hasta (str) 'dd/mm/yyyy'
                id_ciudad (int)
                id_tipo_movil (int)
                licencia (str)
                ra (str)
                tipo_dato (str)
                    // "Viaje" es para tener en cuenta cada servicio realizado
                rr2 (bool)
                rr1 (bool)

            Funciones:
                Fecha().pasar_a_date
                LogParser().log_reloj
                LogParser().ruta_archivo_log_reloj

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de list)
                                            [0] tipo (str)
                                            [1] licencia (str)
                                            [2] equipo (str)
                                            [3] num_serie_reloj (str)
                                            [4] tipo_paquiete (str)
                                            [5] boleto (str)
                                            [6] indice (str)
                                            [7] turno (str)
                                            [8] vel_max_libre (str)
                                            [9] vel_max_ocupado (str)
                                            [10] km_libre (str)
                                            [11] km_ocupado (str)
                                            [12] km_libre_del_turno (str)
                                            [13] km_ocupado_del_turno (str)
                                            [14] importe (str)
                                            [15] acumulado (str)
                                            [16] fecha_hora_inicio (str)
                                            [17] duracion (str)
        '''

        rta = {}

        rta_pasar_a_date = self.fecha.pasar_a_date(fecha_desde)
        if rta_pasar_a_date['Estado'] is False:
            rta['Estado'] = False
            rta['Respuesta'] = rta_pasar_a_date['Mensaje']
            return rta
        fecha_desde_date = rta_pasar_a_date['Mensaje']

        rta_pasar_a_date = self.fecha.pasar_a_date(fecha_hasta)
        if rta_pasar_a_date['Estado'] is False:
            rta['Estado'] = False
            rta['Respuesta'] = rta_pasar_a_date['Mensaje']
            return rta
        fecha_hasta_date = rta_pasar_a_date['Mensaje']

        rta_fechas_pedidas = self.fecha.obtener_fechas_pedidas(
                                    fecha_desde_date, fecha_hasta_date, 'asc')

        lista = []
        anio = 0
        mes = 0
        dia_desde = 0
        dia_hasta = 0
        for fecha in rta_fechas_pedidas:
            if mes != fecha.month or anio != fecha.year:
                rta_ruta_archivo_log_reloj = self.ruta_archivo_log_reloj(
                    str(id_ciudad),
                    str(id_tipo_movil),
                    str(licencia),
                    str(anio),
                    str(mes),
                )
                if rta_ruta_archivo_log_reloj['Estado']:
                    ruta_archivo = rta_ruta_archivo_log_reloj['Respuesta']

                rta_log_parser = self.log_reloj(ruta_archivo,
                                        dia_desde, dia_hasta, tipo_dato, rr2)
                if rta_log_parser['Estado']:
                    lista.extend(rta_log_parser['Respuesta'])

                anio = fecha.year
                mes = fecha.month
                dia_desde = fecha.day
                dia_hasta = fecha.day
            else:
                anio = fecha.year
                mes = fecha.month
                dia_hasta = fecha.day

        rta_ruta_archivo_log_reloj = self.ruta_archivo_log_reloj(
            str(id_ciudad),
            str(id_tipo_movil),
            str(licencia),
            str(anio),
            str(mes),
        )
        if rta_ruta_archivo_log_reloj['Estado']:
            ruta_archivo = rta_ruta_archivo_log_reloj['Respuesta']
        rta_log_parser = self.log_reloj(ruta_archivo, dia_desde, dia_hasta,
                                                        tipo_dato, rr2, rr1)
        if rta_log_parser['Estado']:
            lista.extend(rta_log_parser['Respuesta'])

        if len(lista) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = \
                    _(u'No se encontraron datos del reloj para la fecha dada')
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def grabar_archivo_log_reloj(self, id_ciudad, id_tipo_movil, licencia,
            anio, mes, datos):
        '''
            Esta función se encarga de armar la ruta de un log de reloj según
            los datos recibidos.

            Argumentos:
                id_ciudad (str)
                id_tipo_movil (str)
                licencia (str)
                anio (str)
                mes (str)
                datos   [0] tipo (str)
                        [1] licencia (str)
                        [2] equipo (str)
                        [3] num_serie_reloj (str)
                        [4] tipo_paquiete (str)
                        [5] boleto (str)
                        [6] indice (str)
                        [7] turno (str)
                        [8] vel_max_libre (str)
                        [9] vel_max_ocupado (str)
                        [10] km_libre (str)
                        [11] km_ocupado (str)
                        [12] km_libre_del_turno (str)
                        [13] km_ocupado_del_turno (str)
                        [14] importe (str)
                        [15] acumulado (str)
                        [16] fecha_hora_inicio (str)
                        [17] duracion (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''
        rta = {'Estado': False}

        # Vemos si los datos están bien
        if len(datos) > 18:
            rta['Respuesta'] = _('Los datos a grabar no pueden ser tantos')
            return rta

        # Si nos pasan un RO, seguimos de largo pero indicamos que esta
        # todo bien
        if (datos[16][5:].startswith('00/00') or datos[16][8:].startswith('00') or
           datos[17][5:].startswith('00/00')) or datos[17][8:].startswith('00'):
            return {
                'Estado': True,
                'Respuesta': 'El dato no es valido, pero podemos saltearlo'
            }


        # Obtenemos la ruta del archivo
        rta_ruta = self.ruta_archivo_log_reloj(
            id_ciudad, id_tipo_movil, licencia, anio, mes
        )
        if not rta_ruta['Estado']:
            return rta_ruta
        # Guardamos la ruta
        ruta = rta_ruta['Respuesta']
        directorio = os.path.dirname(ruta)
        # Si no es directorio, lo creamos
        if not os.path.isdir(directorio):
            os.makedirs(directorio)
        # Si no existe el archivo, lo creamos
        if not os.path.isfile(ruta):
            f = open(ruta, 'wb')
            f.write('')
            f.close()

        # Abrimos el archivo y obtenemos los logs
        f = open(ruta, 'rb')
        logs_tmp = f.read()
        # Cerramos
        f.close()
        # Separamos
        logs_tmp = logs_tmp.split('\n')
        logs = []
        # Recorremos
        for log in logs_tmp:
            log = log.strip()
            if log == '':
                continue
            log = log.split(chr(9))
            # Si tenemos menos de 18 datos, pasamos
            if len(log) < 18:
                continue
            # Si es un RO cualquiera, seguimos de largo, por que no lo
            # podemos guardar
            if (log[16].startswith('1900/00/00') or
               log[17].startswith('1900/00/00')):
                continue

            try:
                log[17] = datetime.strptime(log[17], '%d/%m/%Y %H:%M')
            except Exception as e1:
                try:
                    log[17] = datetime.strptime(log[17], '%Y/%m/%d %H:%M')
                except Exception as e2:
                    print('ERROR al formatear fecha hora fin: %s' % str(log))
                    print(e1, e2)
                    continue

            try:
                log[16] = datetime.strptime(log[16], '%Y/%m/%d %H:%M')
            except Exception as e1:
                try:
                    log[16] = datetime.strptime(log[16], '%d/%m/%Y %H:%M')
                except Exception as e2:
                    print('ERROR al formatear fecha hora inicio: %s' % str(log))
                    print(e1, e2)
                    continue
            # Lo agregamos al log final, si no esta
            if not log in logs:
                logs.append(log)

        # Agregamos los datos nuevos a grabar
        try:
            datos[17] = datetime.strptime(datos[17], '%d/%m/%Y %H:%M')
            if not datos in logs:
                logs.append(datos)
        except:
            try:
                # Esta es la forma vieja
                datos[16] = datetime.strptime(datos[16], '%Y/%m/%d %H:%M')
            except Exception as e:
                try:
                    datos[16] = datetime.strptime(datos[16], '%d/%m/%Y %H:%M')
                except Exception as e:
                    print('ERROR - LogParser: %s' % str(e))
                    rta['Respuesta'] = _(u'Fallo al parsear la fecha')
                    return rta
            # Agregamos y ordenamos
            if not datos in logs:
                logs.append(datos)

        # Ordenamos los logs con la fecha acendente
        logs.sort(key=lambda log: log[17])

        # Grabamos los datos con la separacion de caracteres
        # correspondiente
        try:
            # Abrimos el archivo para escribir
            f = open(ruta, 'w')
            for log in logs:
                if isinstance(log[16], datetime):
                    log[16] = log[16].strftime('%d/%m/%Y %H:%M')
                if isinstance(log[17], datetime):
                    log[17] = log[17].strftime('%d/%m/%Y %H:%M')
                linea = chr(9).join(log)
                f.write(linea + '\n')
            # Cerramos el archivo
            f.close()
            rta['Estado'] = True
            rta['Respuesta'] = _(u'Se grabó correctamente el archivo')
            return rta
        except Exception as e:
            print('ERROR - LogParser: %s' % str(e))
            rta['Respuesta'] = _(u'Fallo al grabar los datos')
        return rta

    def ruta_archivo_log_reloj(self, id_ciudad, id_tipo_movil, licencia,
                                                                    anio, mes):
        '''
            Esta función se encarga de armar la ruta de un log de reloj según
            los datos recibidos.

            Argumentos:
                id_ciudad (str)
                id_tipo_movil (str)
                licencia (str)
                anio (str)
                mes (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        if len(licencia) < 4:
            agregar_cero = 4 - len(licencia)
            licencia = '0' * agregar_cero + licencia

        if len(mes) < 2:
            agregar_cero = 2 - len(mes)
            mes = '0' * agregar_cero + mes

        nombre_archivo = id_ciudad + '-' + id_tipo_movil + '-' + \
                        licencia + '-' + anio + '-' + mes + '.log'
        ruta = os.path.join(RUTA_LOGS_LINUX, anio + '-' + mes)
        ruta_archivo = os.path.join(ruta, nombre_archivo)

        rta['Estado'] = True
        rta['Respuesta'] = ruta_archivo
        return rta

    def obtener_rutas_logs_moviles(self, fecha_desde, fecha_hasta):
        '''
            Esta función se encarga de obtener todas las rutas de los logs
            de todos los móviles entre dos fechas dadas.
            Busca en los logs de Windows y en los de Linux.

            Argumento:
                fecha_desde (date)
                fecha_hasta (date)

            Funciones:
                montar_logs_moviles
                logs_path

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Obtengo todas las fechas que debe buscar
        fechas = self.fecha.obtener_fechas_pedidas(
            fecha_desde, fecha_hasta, 'asc'
        )

        # Llamo a la función montar_logs_moviles
        rta_montar_logs_moviles = montar_logs_moviles()
        # Si se pudieron montar los logs
        if rta_montar_logs_moviles['Estado'] is True:
            # Guardo la dir donde debe obtener los logs de Windows
            dir_logs_windows = rta_montar_logs_moviles['Respuesta']
        else:
            dir_logs_windows = ''

        # Me fijo si también tengo LOGS en LINUX
        ruta_linux = '/var/lib/at/logs_moviles'
        if os.path.isdir(ruta_linux) is True:
            dir_logs_linux = ruta_linux
        else:
            dir_logs_linux = ''

        # Creo una lista vacía para ir guardando los logs de los móviles
        logs = []
        # Recorro las fechas dadas
        for fecha in fechas:
            # Obtengo la ruta según la fecha
            path_fecha = self.logs_path(fecha)

            # Busco los LOGS de Windows (móviles con Consola)
            if dir_logs_windows != '':
                # Armo la ruta con la dir y la fecha
                ruta = os.path.join(dir_logs_windows, path_fecha)
                # Si la ruta existe
                if os.path.isdir(ruta) is True:
                    # Agrego los logs a la lista
                    logs += glob(ruta + '/*')

            # También busco los LOGS de LINUX (móviles con CpMobile)
            if dir_logs_linux != '':
                # Armo la ruta con la dir y la fecha
                ruta = os.path.join(dir_logs_linux, path_fecha)
                # Si la ruta existe
                if os.path.isdir(ruta) is True:
                    # Agrego los logs a la lista
                    logs += glob(ruta + '/*')

        # Si no tengo logs
        if len(logs) == 0:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontraron logs en la fecha dada')
            return rta

        # Si tengo LOGS retorno True y la lista de logs
        rta['Estado'] = True
        rta['Respuesta'] = logs
        return rta

    def obtener_datos_nombre_log(self, ruta_log):
        '''
            Está función se encarga de obtener los datos del nombre del log.
            La flota, el id del móvil y la fecha.

            Argumentos:
                ruta_log (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                            ['id_flota'] (str)
                                            ['id_movil'] (str) // tiene 00
                                            ['fecha'] (str) // yyyy-mm-dd
        '''

        # Declaro el diccionario a retornar
        rta = {}

        try:
            # Me quedo con el nombre del archivo
            nombre_archivo = os.path.basename(ruta_log)
            # Lo separo por el guión medio
            nombre_archivo_separado = nombre_archivo.split('-')

            # Guardo los datos del nombre del archivo
            dic = {}
            dic['id_flota'] = nombre_archivo_separado[0]
            dic['id_movil'] = nombre_archivo_separado[1]
            dic['fecha'] = nombre_archivo_separado[2] + '-' \
                           + nombre_archivo_separado[3] + '-' \
                           + nombre_archivo_separado[4][:-4]
        except:
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se pudieron obtener los datos del nombre del log %s'
            ) % ruta_log
            return rta

        # Retorno True y los datos del nombre del archivo
        rta['Estado'] = True
        rta['Respuesta'] = dic
        return rta

    def obtener_datos_log(self, ruta_log):
        '''
            Esta función se encarga de obtener todos los datos de un archivo
            de log.

            Argumentos:
                ruta_log (str)

            Retorno:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o lista de lista)
        '''

        # Creo el diccionario a retornar
        rta = {}

        # Intentamos abrimos el archivo
        try:
            # Guardo el archivo
            archivo = open(ruta_log, 'r')
        except:
            # Si no puedo abrir el archivo retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se pudo abrir el archivo con ruta %s'
            ) % ruta_log
            return rta

        # Creo una lista para ir guardando los datos
        datos = []

        # Recorro cada linea del archivo
        for linea in archivo:
            # Si la lista está vacía no la tengo en cuenta
            if linea == '':
                continue

            # Parseo cada línea
            dato = self.parse(linea)

            # Si la línea tiene menos de 16 datos no la tengo en cuenta
            if len(dato) < 16:
                continue

            # Guardo cada registro en la lista de datos
            datos.append(dato)

        # Si no tengo datos
        if len(datos) == 0:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se pudo obtener los datos del log %s'
            ) % ruta_log
            return rta

        # Retorno True y la lista de datos del log
        rta['Estado'] = True
        rta['Respuesta'] = datos
        return rta

    def obtener_archivos_log_empresa(self, empresa, id_movil, id_flota,
                                     fecha_hora_desde, fecha_hora_hasta):
        '''
            Esta función se encarga de obtener las rutas de los archivos de log
            según los datos recibidos, es importante la empresa porque
            cada empresa tiene su propia carpeta donde se guarda su información.

            Argumentos:
                empresa (str)
                id_movil (str)
                id_flota (int)
                fecha_hora_desde (str) dd/mm/yyyy hh:mm:ss
                fecha_hora_hasta (str) dd/mm/yyyy hh:mm:ss

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de str)
        '''

        # Creo el diccionario a retornar
        rta = {}

        # Defino la ruta base donde están todos los datos de las empresas
        ruta_base = '/var/lib/at/empresas/'

        # Le agrego la empresa a la ruta a la ruta
        ruta_base = os.path.join(ruta_base, empresa)

        # Le agrego la carpeta "logs_moviles" a la ruta
        ruta_base = os.path.join(ruta_base, 'logs_moviles')

        # Paso la fecha hora desde a datetime
        fecha_hora_desde = datetime.strptime(
            fecha_hora_desde, '%d/%m/%Y %H:%M:%S'
        )

        # Paso la fecha hora hasta a datetime
        fecha_hora_hasta = datetime.strptime(
            fecha_hora_hasta, '%d/%m/%Y %H:%M:%S'
        )

        # Obtengo todas las fechas pedidas
        rta_fechas_pedidas = self.fecha.obtener_fechas_pedidas(
            fecha_hora_desde, fecha_hora_hasta, 'asc'
        )

        # Creo una lista para ir guardando las rutas de logs
        lista = []

        # Recorro las fechas pedidas
        for fecha in rta_fechas_pedidas:
            # Obtengo el año y el mes de la fecha
            anio_mes = fecha.strftime('%Y-%m')

            # Agrego el año y el mes a la ruta
            ruta = os.path.join(ruta_base, anio_mes)

            # Obtengo el día de la fecha
            dia = fecha.strftime('%d')

            # Agrego el día a la ruta
            ruta = os.path.join(ruta, dia)

            # Agrego ceros al id del móvil para que complete
            # un número de 4 dígitos
            id_movil = str(id_movil)
            if len(id_movil) < 4:
                id_movil = ((4 - len(id_movil)) * '0') + id_movil

            # Agrego ceros al id de la flota
            # para que complete un número de 2 dígitos
            id_flota = str(id_flota)
            if len(id_flota) < 2:
                id_flota = ((2 - len(id_flota)) * '0') + id_flota

            # Genero el nombre del log
            nombre_log = id_flota + '-' + id_movil + '-' \
                         + anio_mes + '-' + dia + '.log'

            # Agrego el nombre del log a la ruta
            ruta = os.path.join(ruta, nombre_log)

            # Si no existe el archivo de log
            if not os.path.isfile(ruta):
                # Paso a la siguiente fecha
                continue

            # Agrego la ruta del log a la lista
            lista.append(ruta)

        # Si no se encontró ningún archivo de log
        if len(lista) == 0:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró ningún archivo de log')
            return rta

        # Retorno True y la lista de logs
        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta
