# -*- coding: utf-8 -*-

import os
import smtplib
from twisted.python import log
from email.mime.text import MIMEText
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart

from at.utiles import _
from at.orm.decorators import DBSwitch
from at.utiles.log import start_custom_log

# Iniciamos nuestro propio log
start_custom_log(
    '/var/lib/at/logs/at.utiles/at.utiles', 'mail'
)


@DBSwitch('cp')
class Mail(object):
    '''
        Clase encargada de enviar  mails.
    '''

    def enviar_mail(self, servidor, puerto, usuario, password,
                            de, para, asunto, mensaje, filename='',
                            responder_a='', test=False):
        '''
            Esta función se encarga de enviar un mail a uno o más correos
            mediante un servidor de gmail.

            Argumentos:
                servidor (str)
                puerto (int)
                usuario (str)
                password (str)
                de (str)
                para (list)
                asunto (str)
                mensaje (str)
                filename (str path)
                responder_a (str)
                test (bool)

            Retorna:
                Caso correcto:
                    dic['Estado'] = True
                    dic['Mensaje'] = None
                Caso incorrecto:
                    dic['Estado'] = False
                    dic['Mensaje'] = 'mensaje de error'
        '''

        dic = {}

        try:
            # Guardo un log para indicar que se ingresó a la función enviar_mail
            log.msg(
                'Se intenta enviar un mail a: %s' % para,
                mode='mail',
            )

            # Si test es True
            if test is True:
                # No envío ningún mail
                dic['Estado'] = True
                dic['Mensaje'] = _(u'No envío el mail porque estoy en un test')
                return dic

            msg = MIMEMultipart('alternative')

            if filename != '':
                f = open(filename, 'rb')
                archivo = f.read()
                f.close()
                nombre_archivo = os.path.basename(filename)
                attachment = MIMEApplication(archivo)
                attachment.add_header('Content-Disposition', 'attachment',
                                        filename=nombre_archivo)
                msg.attach(attachment)

            msg['Subject'] = asunto

            if responder_a != '':
                msg.add_header('reply-to', responder_a)

            mensaje = MIMEText(mensaje.encode("iso-8859-1"), 'html')
            msg.attach(mensaje)

            smtp = smtplib.SMTP(servidor, puerto)
            smtp.ehlo()
            smtp.starttls()
            smtp.login(usuario, password)

            dic['Estado'] = True
            dic['Mensaje'] = _(u'El mail se ha enviado correctamente.')

            for p in para:
                smtp.sendmail(de, p, msg.as_string())

            smtp.quit()

            # Guardo en un log la respuesta de la función enviar_mail
            log.msg(
                'Respuesta de enviar_mail: %s' % dic,
                mode='mail',
            )

            return dic
        except Exception as ex:
            # Guardo en un log la respuesta de la función enviar_mail
            log.msg(
                'No se pudo enviar el mail a: %s - Error: %s' % (para, ex),
                mode='mail',
            )

            dic['Estado'] = False
            dic['Mensaje'] = _(u'No se pudo enviar el mail')
            return dic

    def obtener_template(self):
        '''
            Esta función debe devolver un templeta para poder utilizar a la
            hora de enviar un mail.

            Retorna:
                unicode
        '''

        template = u'''
<div>
    <div style="background:#1f1f1f; padding:5px; width:560px; height:110px; color:#ffffff; border:1px solid #000000;">
        <a href="http://www.airtrack.com.ar/" target="_blank">
            <img src="http://www.airtrack.com.ar/imagenes/subidas/1098_airtrack-chico.png" alt="" title="" height="100" />
        </a>
    </div>

    <div style="background:#EEEEEE; padding:20px; width:530px; min-height:400px; border:1px solid #BDBDBD; font-family:'Arial'; font-size:13px; color:#404040; line-height: 16px;">
        %s
        <br /><br /><br /><br /><br /><br /><br />
        <label style="font-size:10px;">
            _____________________________________________________________________________________
            <br /><br />
            Este mensaje ha sido generado en forma autom&aacute;tica y la
            casilla es utilizada solamente para el env&iacute;o de &eacute;sta
            informaci&oacute;n. Por favor no respondas este email.
            <br /><br />
            El titular de los datos personales tiene la facultad de
            ejercer el derecho de acceso a los mismos en forma
            gratuita a intervalos no inferiores a seis meses,
            salvo que se acredite un inter&eacute;s leg&iacute;timo al efecto
            conforme lo establecido en el art&iacute;culo14, inciso 3 de la
            ley N&deg; 25.326. Ley 25326 - Art.27. - INC. 3. El titular
            podr&aacute; en cualquier momento solicitar el retiro o bloqueo
            de su nombre de los bancos de datos a los que se refiere
            el presente art&iacute;culo. Decreto 1558/01 - art. 27. - 3er. p&aacute;rrafo.
            En toda comunicaci&oacute;n con fines de publicidad que se
            realice por correo, tel&eacute;fono, correo electr&oacute;nico,
            Internet u otro medio a distancia a conocer, se deber&aacute;
            indicar, en forma expresa y destacada, la posibilidad del
            titular del dato de solicitar el retiro o bloqueo, total o
            parcial, de su nombre de la base de datos. A pedido del
            interesado, se deber&aacute; informar el nombre del responsable o
            usuario del banco de datos que provey&oacute; la informaci&oacute;n.
            <br /><br />
            Para asegurarte de que recibes nuestros mensajes de
            correo electr&oacute;nico en tu buz&oacute;n, a&ntilde;ade info@airtrack.com.ar
            a tu agenda de direcciones.
        </label>
    </div>

    <div style="background:#1f1f1f; padding:5px; width:560px; height:76px; border:1px solid #000000; font-family:'Arial'; font-size:11px; color:#BDBDBD; line-height: 16px;">
        Rosario | Santa Fe | Argentina
        <br />
        Urquiza 7164 &middot; Tel.: (+54) 341 4589364
        <br />
        <br />
        <br />
        <div style="text-align:right;">
            &copy; Copyright 2010 - 2012 | Air Track | Todos los derechos reservados
        </div>
    </div>
</div> '''

        return template

    def grabar_configuracion_mail(self, id_sucursal, servidor_smtp, puerto,
                                                            usuario, password):
        '''
            Esta función se encarga de agregar una configuración de mail
            o de modificar una existente.

            Argumentos:
                id_sucursal (int)
                servidor_smtp (str)
                puerto (int)
                usuario (str)
                password (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Creo el diccionario a retornar
        rta = {}

        # Guardo los datos del correo
        dic = {}
        dic['IdSucursal'] = id_sucursal
        dic['ServidorSmtp'] = servidor_smtp
        dic['Puerto'] = puerto
        dic['Usuario'] = usuario
        dic['Password'] = password

        # Primero me fijo si existe el correo, si existe lo modifico
        # y si no la agrego

        # Abro la sesión con la base de datos cp2000
        session = self.cp.binded_session()

        # Obtengo la tabla configuraciones_mails
        configuraciones_mails = self.cp.tabla('configuraciones_mails')

        # Busco si existe la configuración del mail de la sucursal dada
        q_obj = session.query(configuraciones_mails.IdSucursal)
        q_obj = q_obj.filter(configuraciones_mails.IdSucursal == id_sucursal)

        # Obtengo el registro si existe
        registro = q_obj.first()

        # Si el registro no existe
        if registro is None:
            # Cierro la sesión con la base de datos
            session.close()

            # Agrego el registro a la base de datos
            agregar = self.cp.agregar('configuraciones_mails', **dic)

            # Si no se puede agregar retorno el error
            if agregar is not True:
                rta['Estado'] = False
                rta['Respuesta'] = _(
                    u'No se pudo guardar la configuración del correo de la sucursal %s'
                    % id_sucursal
                )
                return rta

            # Si se pudo agregar retorno el mensaje
            rta['Estado'] = True
            rta['Respuesta'] = _(
                u'Se guardó correctamente la configuración del correo de la sucursal %s'
                % id_sucursal
            )
            return rta

        # Si el registro existe actualizo sus datos
        q_obj.update(dic, synchronize_session=False)

        # Realizo el commit con la base de datos
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Si se pudo actualizar el registro retorno el mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se modificó correctamente la configuración del correo de la sucursal %s'
            % id_sucursal
        )
        return rta

    def listar_configuraciones_mails(self):
        '''
            Esta función se encarga de listar todas las configuraciones
            de mails existentes.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de list)
        '''

        # Creo el diccionario a retornar
        rta = {}

        # Abro la sesión con la base de datos
        session = self.cp.binded_session()

        # Obtengo la tabla configuraciones_mails
        configuraciones_mails = self.cp.tabla('configuraciones_mails')

        # Busco todos los datos de la tabla
        q_obj = session.query(configuraciones_mails)

        # Ordeno los registros por el campo IdSucursal de forma ascendente
        q_obj = q_obj.order_by(configuraciones_mails.IdSucursal.asc())

        # Guardo los registros
        registros = q_obj.all()

        # Cierro la sesión con la base de datos
        session.close()

        # Si no tengo registros retorno un error
        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontraron configuraciones de mails cargadas'
            )
            return rta

        # Creo una lista para ir guardando los registros
        lista = []

        # Recorro los registros obtenidos
        for registro in registros:
            # El campo ServidorSmtp lo debo pasar al formato iso-8859-1
            if registro.ServidorSmtp is not None:
                try:
                    servidor_smtp = registro.ServidorSmtp.decode("iso-8859-1")
                except:
                    # NOTA: En Python 3 no se usa el decode en los "str"
                    servidor_smtp = registro.ServidorSmtp
            else:
                servidor_smtp = ''

            # El campo Usuario lo debo pasar al formato iso-8859-1
            if registro.Usuario is not None:
                try:
                    usuario = registro.Usuario.decode("iso-8859-1")
                except:
                    # NOTA: En Python 3 no se usa el decode en los "str"
                    usuario = registro.Usuario
            else:
                usuario = ''

            # El campo Password lo debo pasar al formato iso-8859-1
            if registro.Password is not None:
                try:
                    password = registro.Password.decode("iso-8859-1")
                except:
                    # NOTA: En Python 3 no se usa el decode en los "str"
                    password = registro.Password
            else:
                password = ''

            # Agrego a la lista cada registro
            lista.append(
                [
                    registro.IdSucursal,
                    usuario,
                    servidor_smtp,
                    registro.Puerto,
                    password,
                ]
            )

        # Retorno True y la lista obtenida
        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def borrar_configuracion_mail(self, id_sucursal):
        '''
            Esta función se encarga de borrar una configuración de mail.

            Argumentos:
                id_sucursal (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Creo el diccionario a retornar
        rta = {}

        # Abro la sesión con la base de datos
        session = self.cp.binded_session()

        # Obtengo la tabla configuraciones_mails
        configuraciones_mails = self.cp.tabla('configuraciones_mails')

        # Busco la configuración de mail a borrar
        q_obj = session.query(configuraciones_mails.IdSucursal)

        # Filtro por la sucursal dada de la configuración de mail
        q_obj = q_obj.filter(configuraciones_mails.IdSucursal == id_sucursal)

        # Si no se encontró el registro a borrar
        if q_obj.count() <= 0:
            # Cierro la conexión con la base de datos
            session.close()

            # Retorno False y el mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró la configuración de correo de la sucursal %s'
                % id_sucursal
            )
            return rta

        # Borro el registro de la tabla configuraciones_mails
        q_obj.delete(synchronize_session=False)

        # Realizo el commit con la base de datos
        session.commit()

        # Cierro la conexión con la base de datos
        session.close()

        # Si se pudo borrar el registro retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se eliminó correctamente la configuración de correo de la sucursal %s'
            % id_sucursal
        )
        return rta

    def obtener_configuracion_mail(self, id_sucursal):
        '''
            Esta función se encarga de obtener todos los datos de una
            configuración de mail según la sucursal dada.

            Argumentos:
                id_sucursal (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                        ['id_sucursal'] (int)
                                        ['servidor_smtp'] (str)
                                        ['puerto'] (int)
                                        ['usuario'] (str)
                                        ['password'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Abro la sesión con la base de datos
        session = self.cp.binded_session()

        # Obtengo la tabla configuraciones_mails
        configuraciones_mails = self.cp.tabla('configuraciones_mails')

        # Pido todos los datos de la configuración de mail
        q_obj = session.query(configuraciones_mails)

        # Filtro por la sucursal dada
        q_obj = q_obj.filter(configuraciones_mails.IdSucursal == id_sucursal)

        # Guardo el registro encontrado
        registro = q_obj.first()

        # Cierro la conexión con la base de datos
        session.close()

        # Si no se encontró el registro
        if registro is None:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró la configuración de correo de la sucursal %s'
                % id_sucursal
            )
            return rta

        # El campo ServidorSmtp lo debo pasar al formato iso-8859-1
        if registro.ServidorSmtp is not None:
            try:
                servidor_smtp = registro.ServidorSmtp.decode("iso-8859-1")
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                servidor_smtp = registro.ServidorSmtp
        else:
            servidor_smtp = ''

        # El campo Usuario lo debo pasar al formato iso-8859-1
        if registro.Usuario is not None:
            try:
                usuario = registro.Usuario.decode("iso-8859-1")
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                usuario = registro.Usuario
        else:
            usuario = ''

        # El campo Password lo debo pasar al formato iso-8859-1
        if registro.Password is not None:
            try:
                password = registro.Password.decode("iso-8859-1")
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                password = registro.Password
        else:
            password = ''

        # Guardo sus datos en un diccionario
        dic = {}
        dic['id_sucursal'] = registro.IdSucursal
        dic['servidor_smtp'] = servidor_smtp
        dic['puerto'] = registro.Puerto
        dic['usuario'] = usuario
        dic['password'] = password

        # Retorno True y los datos
        rta['Estado'] = True
        rta['Respuesta'] = dic
        return rta
