# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import and_

from at.utiles import _
from at.utiles.servidores import DateTimeHelper
from at.utiles.numerador import Numerador
from at.utiles.usuarios import Usuarios
from at.orm.decorators import DBSwitch


@DBSwitch('cp')
class Mensajes(object):
    '''
        Clase encargada de enviar mensajes con el formato adecuado.
    '''

    def __init__(self):
        self.dth = DateTimeHelper()
        self.numerador = Numerador()
        self.usuarios = Usuarios()

    def enviar_msg_a_consola(self, cmd, id_equipo, id_movil, user,
                                                            srv=None, msg=''):
        '''
            Esta función envía mensajes a moviles satelitales con distintos
            fines (mensajes informativos, viajes, cancelar servicios, etc)

            Argumentos:
                (str)    cmd - Comando
                (long)   id_equipo
                (str)    id_movil
                (str)    user
                (dict)   Servicio
                (str)    Mensaje

            Retorna:
                bool
        '''

        if not isinstance(cmd, str):
            raise(ValueError)
        allowed_cmds = ['210', '211', '204']
        if cmd not in allowed_cmds:
            raise(ValueError)

        if not isinstance(id_equipo, int):
            raise(ValueError)

        try:
            if not isinstance(id_equipo, long):
                raise(ValueError)
        except:
            raise(ValueError)

        if not isinstance(user, str):
            raise(ValueError)

        if not isinstance(id_movil, str):
            raise(ValueError)

        if srv is not None:
            if not isinstance(srv, dict):
                raise(ValueError)

        now = DateTimeHelper().get_now()

        if srv is not None:
            cuenta_corriente = srv.get('CuentaCorriente')
            espacio_en_baul = srv.get('EspacioEnBaul')
            observaciones = srv.get('Observaciones')
            fhs = srv.get('FechaHoraServicio')
            direccion = srv.get('Direccion')
            telefono = srv.get('Telefono')
            sin_gnc = srv.get('SinGNC')
            aire = srv.get('Aire')
            vip = srv.get('VIP')

        if cmd == '204':
            # Si el comando es 204, armar el siguiente msg ...
            x = chr(12)
            msg = msg.replace('<', x + '1')
            msg = msg.replace('>', x + '2')
            msg = msg.replace('[', x + '3')
            msg = msg.replace(']', x + '4')
            msg = msg.replace('*', x + '5')

            msg = x + "C" + x + "E" + "D" + now.strftime('%d') + " H" + \
                now.strftime('%H:%M') + x + "I" + msg

            # creo el insert ...
            new_controlmsg = self.cp.tabla('controlmsj')()
            new_controlmsg.FechaHora = now
            new_controlmsg.Id_Equipo = id_equipo
            new_controlmsg.IdMovil = id_movil
            new_controlmsg.PrioridadMsj = 0
            new_controlmsg.EstadoMsj = 0
            new_controlmsg.TipoMsj = 'TER'
            new_controlmsg.Comando = cmd
            new_controlmsg.Mensaje = msg
            new_controlmsg.EnviadoPor = user

            session = self.cp.binded_session()
            session.add(new_controlmsg)
            session.commit()
            session.close()

            return True

        if cmd == '210':
            x = chr(12)

            if observaciones is None:
                observaciones = ''

            if direccion is None:
                direccion = ''

            msg = direccion.strip() + ' ' + observaciones.strip() + msg
            msg = msg.replace('<', x + '1')
            msg = msg.replace('>', x + '2')
            msg = msg.replace('[', x + '3')
            msg = msg.replace(']', x + '4')
            msg = msg.replace('*', x + '5')

            msg2 = ''
            if aire == 1:
                msg2 = msg2 + x + 'M01'
            if cuenta_corriente == 1:
                msg2 = msg2 + x + 'M02'
            if espacio_en_baul == 1:
                msg2 = msg2 + x + 'M03'
            if sin_gnc == 1:
                msg2 = msg2 + x + 'M04'
            if vip == 1:
                msg2 = msg2 + x + 'M05'

            # formato super secreto para armar un mensaje ...
            msg = x + 'C' + x + 'E' + 'D' + fhs.strftime('%d') + ' H' + fhs.strftime('%H:%M') + x + 'I' + msg + msg2

            msg = msg[:120]

            # creo el insert ...
            new_controlmsg = self.cp.tabla('controlmsj')()

            new_controlmsg.FechaHora = now
            new_controlmsg.Id_Equipo = id_equipo
            new_controlmsg.IdMovil = id_movil
            new_controlmsg.PrioridadMsj = 0
            new_controlmsg.EstadoMsj = 0
            new_controlmsg.TipoMsj = 'TER'
            new_controlmsg.Comando = cmd
            new_controlmsg.Mensaje = msg
            new_controlmsg.Telefono = telefono
            new_controlmsg.FechaServicio = fhs.strftime('%Y-%m-%d 00:00:00')
            new_controlmsg.HoraServicio = '1900-01-01 ' + fhs.strftime(
                                                                    '%H:%M:%S')
            new_controlmsg.EnviadoPor = user

            session = self.cp.binded_session()
            session.add(new_controlmsg)
            session.commit()
            session.close()

            return True

        if cmd == '211':
            ''' cancelacion del servicio '''

            session = self.cp.binded_session()

            cmsg = self.cp.tabla('controlmsj')

            q_obj = session.query(cmsg.EstadoMsj, cmsg.MsjNum)

            q_obj = q_obj.filter(
                and_(
                    cmsg.Telefono == telefono,
                    cmsg.FechaServicio == fhs.strftime('%Y-%m-%d 00:00:00'),
                    cmsg.HoraServicio == '1900-01-01 ' + fhs.strftime(
                                                                    '%H:%M:%S'),
                    cmsg.Finalizado == 0,
                    cmsg.Comando == '210',
                )
            )

            q_obj = q_obj.order_by(cmsg.IdMsj.desc())

            q_obj = q_obj.limit(1)

            if int(q_obj.count()) != 0:
                registro = q_obj.first()

                q_obj = session.query(cmsg)

                q_obj = q_obj.filter(
                    and_(
                        cmsg.Telefono == telefono,
                        cmsg.FechaServicio == fhs.strftime('%Y-%m-%d 00:00:00'),
                        cmsg.HoraServicio == '1900-01-01 ' + fhs.strftime(
                                                                    '%H:%M:%S'),
                    )
                )

                u = {}
                u['EstadoMsj'] = 21
                u['Finalizado'] = 1

                q_obj.update(u)

                session.commit()

                if registro.EstadoMsj > 6 and registro.EstadoMsj != 12:
                    d = {}
                    d['Id_Equipo'] = id_equipo
                    d['IdMovil'] = id_movil
                    d['FechaHora'] = now
                    d['PrioridadMsj'] = 0
                    d['EstadoMsj'] = 0
                    d['TipoMsj'] = 'TER'
                    d['Comando'] = cmd
                    d['Mensaje'] = registro.MsjNum
                    d['Telefono'] = telefono
                    d['FechaServicio'] = fhs.strftime('%Y-%m-%d 00:00:00')
                    d['HoraServicio'] = '1900-01-01 ' + fhs.strftime(
                                                                    '%H:%M:%S')
                    d['EnviadoPor'] = user
                    self.cp.agregar('controlmsj', **d)

                return True

        return False

    def enviar_a_usuarios(self, remitente, mensaje,
                            destinatarios=None, permisos=None, tipo='interno'):
        '''
            Esta función se encarga de enviar un mensaje a un usuario o a un
            tipo de usuarios.

            Argumentos:
                remitente (str)
                mensaje (str)
                destinatarios (list de str)
                permisos (list de str)
                tipo (str)

            Funciones:
                DateTimeHelper().get_now
                Numerador().obtener_numerador
                Usuarios().listar

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Si no tengo ni destinatarios ni permisos retorno el error
        if destinatarios is None and permisos is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'El mensaje no se envió a ningún usuario')
            return rta

        # Obtengo la fecha y hora actual
        fecha_hora_actual = self.dth.get_now()

        # Tengo que buscar los usuarios que tengan los permisos dados
        if permisos is not None and len(permisos) != 0:
            destinatarios = []

            # Obtengo los usuarios del CP
            rta_listar_usuarios = self.usuarios.listar()
            if rta_listar_usuarios['Estado'] is False:
                return rta_listar_usuarios

            # Guardo la lista de usuarios
            usuarios = rta_listar_usuarios['Respuesta']

            # Recorro los usuarios
            for usuario in usuarios:
                if usuario['permisos'] in permisos:
                    destinatarios.append(usuario['usuario'])

            # Si no hay ningún usuario con los permisos dados retorno el error
            if len(destinatarios) == 0:
                rta['Estado'] = False
                rta['Respuesta'] = \
                            _(u'No existen usuarios con los permisos dados')
                return rta

        # Recorro los destinatarios y lo voy guardo el mensaje a cada uno
        for destinatario in destinatarios:
            # Obtengo el id mensaje siguiente
            id_mensaje = self.numerador.obtener_numerador('mensajes', remitente)

            # Guardo el mensaje en la base de datos
            dic = {}
            dic['IdMensaje'] = id_mensaje
            dic['Remitente'] = remitente
            dic['Destinatario'] = destinatario
            dic['Enviado'] = fecha_hora_actual
            dic['Mensaje'] = mensaje
            dic['Tipo'] = tipo
            agregar = self.cp.agregar('mensajes', **dic)
            if agregar is not True:
                rta['Estado'] = False
                rta['Respuesta'] = _(u'No se pudo grabar el mensaje')
                return rta

        # Si todo salió bien retorno un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se grabó correctamente el mensaje')
        return rta

    def guardar_sms(self, telefono, mensaje):
        '''
            Esta función se encarga de agregar un registro en la tabla "sms"
            de la base de datos "cp2000".

            Argumentos:
                telefono (str)
                mensaje (str)

            Funciones:
                DateTimeHelper -> get_now

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Obtengo la fecha y hora actual
        fecha_hora_actual = self.dth.get_now()

        # Guardo el SMS en la base de datos
        dic = {}
        dic['FechaHoraCreado'] = fecha_hora_actual
        dic['NumCelular'] = telefono
        dic['Mensaje'] = mensaje
        agregar = self.cp.agregar('sms', **dic)
        if agregar is not True:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo guardar el SMS')
            return rta

        # Si se pudo guardar el SMS retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se guardó el SMS')
        return rta
