# -*- coding: utf-8 -*-

import os
import shlex
import subprocess

from at.utiles import _
from at.utiles.parametros import Parametros as ParametrosLocales


def montar_dir(dir_remoto, dir_local):
    '''
        Funcion para montar un directorio remoto en un directorio local
        de la maquinad de linux

        Argumentos:
            dir_remoto (str)
            dir_local (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    rta = {"Estado": False}

    # Validamos si ya esta montado el directorio. Si es así, salimos
    if esta_montado(dir_local):
        rta['Estado'] = True
        rta['Respuesta'] = 'El directorio ya esta montado'
        return rta

    # Sino, intentamos montarlo
    try:
        # Validamos que no le pongamos las // dos veces, así que se las sacamos
        # si ya las tenia
        if dir_remoto.startswith('//'):
            dir_remoto = dir_remoto[2:]

        # Generamos la linea de comando
        if os.path.isfile('/root/.cifs') is True:
            cmd = 'mount -t cifs //' + dir_remoto + ' ' + dir_local + ' -o credentials=/root/.cifs'
        else:
            cmd = 'mount -t cifs //' + dir_remoto + ' ' + dir_local + ' -o rw,guest'

        pcmd = shlex.split(cmd)

        # Ejecutamos el subproceso
        p = subprocess.Popen(
            pcmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        # Obtenemos el resultado
        stdout, stderr = p.communicate()

        # Si fallamos en algo, salimos
        if stderr.strip() != '':
            rta['Respuesta'] = _(u'Fallo al momento de montar: %s' % stderr)
            return rta
    except:
        # Si fallamos, salimos
        rta['Respuesta'] = _(u'''Falló al momento de montar. Verifique el
 directorio local y los permisos del usuario''')
        return rta

    # Salimos correctamente
    rta['Estado'] = True
    rta['Respuesta'] = _(u'Se montó correctamente el directorio')
    return rta


def esta_montado(dir_local):
    '''
        Función para chequear si esta montado o no el directorio que se pasa
        como argumento.

        Argumento:
            dir_local (str)

        Retorna:
            bool
    '''

    # Buscamos en el archivo de montaje
    for line in open('/etc/mtab').readlines():
        # Si lo encontramos en el archivo es por que esta montado, así que
        # salimos con un true
        if dir_local in line:
            return True
    # Sino, con un false
    return False


def montar_logs_moviles():
    '''
        Esta función se encarga de obtener el IP de donde están los logs de
        los móviles para montarlos en un directorio en la máquina local.

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    iplogs = ParametrosLocales().obtener_parametrolocal('IpLogs')

    dir_remoto = os.path.join(iplogs, 'log')

    dir_local = os.path.join('/tmp', 'mlogs-tmp')
    if not os.path.isdir(dir_local):
        os.makedirs(dir_local)

    rta = montar_dir(dir_remoto, dir_local)

    if rta['Estado']:
        rta['Respuesta'] = dir_local

    return rta
