# -*- coding: utf-8 -*-

from datetime import timedelta
from datetime import datetime
from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import or_
from sqlalchemy.sql.expression import func

from at.utiles import _
from at.utiles.servidores import DateTimeHelper
from at.utiles.log_parser import LogParser
from at.utiles.flotas import Flotas
from at.utiles.fecha import Fecha
from at.utiles.parametros import Parametros
from at.utiles.conversiones import respuesta
from at.orm.decorators import DBSwitch
try:
    from at.map.utiles.funciones_geo import FuncionesGeo
    from at.map.zonas import Zonas as ZonasGeo
except:
    print('>> warn: No pudimos importar at.map')


@DBSwitch('cp')
class Moviles(object):
    '''
        Esta clase contiene funciones útiles de móviles.
    '''

    def __init__(self):
        self.dth = DateTimeHelper()
        self.log_parser = LogParser()
        self.flotas = Flotas()
        self.fecha = Fecha()
        self.parametros = Parametros()
        try:
            self.fg = FuncionesGeo()
            self.zonas_geo = ZonasGeo()
        except:
            self.fg = None
            self.zonas_geo = None

    def obtener_moviles_cercanos(self, id_sucursal=None, latitud=0, longitud=0,
                            cant_mov=None, en_servicio=None, con_cpmobile=None,
                            id_movil='', id_flota=None, dist_max=None,
                            id_ciudad=None, pos_en_grados=False):
        '''
            Esta función se encarga de obtener los móviles más cercanos a una
            latitud y una longitud dada.
            Puede retornar una cantidad de móviles más cercanos dada o una
            lista de móviles cercanos a un rango dado.

            Argumentos:
                id_sucursal (int)
                latitud (float)
                longitud (float)
                cant_mov (int)
                en_servicio (bool)
                con_cpmobile (bool)
                id_movil (str)
                dist_max (int)
                id_ciudad (int)
                pos_en_grados (bool)

            Funciones:
                DateTimeHelper().get_now
                DateTimeHelper().obtener_fecha_hora_ciudad
                Moviles().motor_detenido

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de srt)
        '''

        rta = {'Estado': False, 'Respuesta': 'No hay librerias para trabajar con mapas.'}

        # Si no hay fg, salimos
        if self.fg is None:
            return rta

        if cant_mov is None and dist_max is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'Debe ingresar una cantidad de móviles o una distancia máxima'
            )
            return rta

        if cant_mov is not None and cant_mov < 1:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'La cantidad de móviles cercanos a buscar debe ser 1 o más')
            return rta

        if id_ciudad is None:
            fecha_hora_actual = self.dth.get_now()
        else:
            # Obtengo la fecha y hora actual según la ciudad dada
            rta_obtener_fecha_hora_ciudad = self.dth.obtener_fecha_hora_ciudad(
                id_ciudad
            )
            if rta_obtener_fecha_hora_ciudad['Estado'] is False:
                # Si no puedo obtener la fecha y la hora
                # de la ciudad retorno el error
                return rta_obtener_fecha_hora_ciudad

            # Guardo la fecha y hora obtenida
            fecha_hora_actual = rta_obtener_fecha_hora_ciudad['Respuesta']

        none = None
        ip_null = '0.0.0.0'
        str_vacio = ''

        session = self.cp.binded_session()

        moviles = self.cp.tabla('moviles')

        q_obj = session.query(
            moviles.IdMovil, moviles.Latitud, moviles.Longitud,
            moviles.IdFlota, moviles.Estado1, moviles.Rumbo
        )

        q_obj = q_obj.filter(
            and_(
                moviles.Latitud != 0,
                moviles.Longitud != 0,
                moviles.UltPosicion >= fecha_hora_actual - timedelta(minutes=5),
                or_(
                    moviles.EstadoGPS == 2,
                    moviles.EstadoGPS == 3,
                ),
            )
        )

        if id_sucursal is not None:
            q_obj = q_obj.filter(moviles.Sucursal == id_sucursal)

        if en_servicio:
            q_obj = q_obj.filter(moviles.EstaEnServicio == 1)

        if en_servicio is False:
            q_obj = q_obj.filter(moviles.EstaEnServicio == 0)

        if con_cpmobile:
            q_obj = q_obj.filter(
                and_(
                    moviles.idSesion != str_vacio,
                    moviles.idSesion != none,
                    moviles.DNI != str_vacio,
                    moviles.DNI != none,
                )
            )

        if con_cpmobile is False:
            q_obj = q_obj.filter(
                and_(
                    moviles.Id_Equipo != 0,
                    moviles.IP != ip_null,
                )
            )

        if id_movil != '':
            q_obj = q_obj.filter(moviles.IdMovil != id_movil)

        if id_flota is not None:
            q_obj = q_obj.filter(moviles.IdFlota == id_flota)

        if q_obj.count() == 0:
            session.close()
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No hay móviles cercanos')
            return rta

        registros = q_obj.all()

        session.close()

        moviles_cercanos = []
        for registro in registros:
            distancia = self.fg.km_entre_2_puntos(
                float(registro.Latitud),
                float(registro.Longitud),
                float(latitud),
                float(longitud),
            )
            distancia = round(distancia * 1000, 2)

            motor_detenido = False
            rta_motor_detenido = self.motor_detenido(registro.Estado1)
            if rta_motor_detenido['Estado']:
                motor_detenido = rta_motor_detenido['Respuesta']

            dic = {}
            dic['id_movil'] = registro.IdMovil
            dic['id_flota'] = registro.IdFlota

            dic['latitud'] = float(registro.Latitud)
            dic['longitud'] = float(registro.Longitud)

            # Si "pos_en_grados" es True
            if pos_en_grados is True:
                # Paso la posición de segundos a grados
                dic['latitud'] /= 3600
                dic['longitud'] /= 3600

            dic['distancia'] = distancia
            dic['motor_detenido'] = motor_detenido
            dic['rumbo'] = registro.Rumbo

            if cant_mov is not None:
                if dist_max is not None:
                    if distancia > dist_max:
                        continue

                if len(moviles_cercanos) < cant_mov:
                    moviles_cercanos.append(dic)
                    moviles_cercanos.sort(key=lambda x: x['distancia'])
                    continue

                if moviles_cercanos[cant_mov-1] > dic['distancia']:
                    moviles_cercanos.pop(cant_mov-1)
                    moviles_cercanos.append(dic)
                    moviles_cercanos.sort(key=lambda x: x['distancia'])
                    continue
            else:
                if dist_max is not None:
                    if distancia <= dist_max:
                        moviles_cercanos.append(dic)

        if len(moviles_cercanos) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No hay móviles cercanos')
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = moviles_cercanos
        return rta

    def validar_movil(self, id_sucursal, id_movil):
        '''
            Dado un id sucursal y un id móvil valido si el móvil existe en la
            base de datos o no.

            Argumentos:
                id_sucursal (int)
                id_movil (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        session = self.cp.binded_session()

        moviles = self.cp.tabla('moviles')

        q_obj = session.query(moviles.IdMovil)

        q_obj = q_obj.filter(moviles.IdMovil == id_movil)

        if id_sucursal != 0:
            q_obj = q_obj.filter(moviles.Sucursal == id_sucursal)

        registro = q_obj.first()

        session.close()

        if registro is None:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No existe el móvil %s de la sucursal %s') \
                % (id_movil, id_sucursal)
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Existe el móvil %s de la sucursal %s') \
            % (id_movil, id_sucursal)
        return rta

    def validar_moviles(self, id_sucursal, moviles):
        '''
            Dado una lista de móviles esta función se encarga de devolver una
            lista de solo los móviles que existe en la base de datos.

            Argumentos:
                id_sucursal (int)
                moviles (list)

            Funciones:
                validar_movil

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (list de str o str)
        '''

        rta = {}

        moviles_existentes = []

        for movil in moviles:
            validar_movil = self.validar_movil(id_sucursal, movil)
            if validar_movil['Estado']:
                moviles_existentes.append(movil)

        if len(moviles_existentes) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No existe ningún móvil dado')
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = moviles_existentes
        return rta

    def validar_movil_celular(self, id_sucursal, id_movil):
        '''
            Dado un id sucursal y un id móvil valido si el móvil tiene celular.

            Argumentos:
                id_sucursal (int)
                id_movil (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        session = self.cp.binded_session()

        moviles = self.cp.tabla('moviles')

        q_obj = session.query(moviles.IdMovil)

        none = None
        str_vacio = ''

        q_obj = q_obj.filter(and_(
                                moviles.IdMovil == id_movil,
                                moviles.idSesion != str_vacio,
                                moviles.idSesion != none,
                                moviles.DNI != str_vacio,
                                moviles.DNI != none,
                                )
                            )

        if id_sucursal != 0:
            q_obj = q_obj.filter(moviles.Sucursal == id_sucursal)

        registro = q_obj.first()

        session.close()

        if registro is None:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'El móvil %s de la sucursal %s no tiene celular') \
                % (id_movil, id_sucursal)
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'El móvil %s de la sucursal %s tiene celular') \
            % (id_movil, id_sucursal)
        return rta

    def validar_moviles_celular(self, id_sucursal, moviles):
        '''
            Dado una lista de móviles esta función se encarga de validar si
            alguno tiene celular.

            Argumentos:
                id_sucursal (int)
                moviles (list)

            Funciones:
                validar_movil_celular

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (list de str o str)
        '''

        rta = {}

        moviles_con_celular = []

        for movil in moviles:
            validar_movil_celular = self.validar_movil_celular(id_sucursal,
                                                                movil)
            if validar_movil_celular['Estado']:
                moviles_con_celular.append(movil)

        if len(moviles_con_celular) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'Ningún móvil dado tiene celular')
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = moviles_con_celular
        return rta

    def obtener_empresa_del_movil(self, id_licencia):
        '''
            Esta función se encarga de devolver el id_empresa al que pertence
            un móvil dado.

            Argumento:
                id_licencia (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o int)
        '''

        rta = {}

        session = self.cp.binded_session()

        moviles = self.cp.tabla('moviles')
        sucursales = self.cp.tabla('sucursales')

        q_obj = session.query(moviles.IdMovil, moviles.Sucursal,
                                sucursales.LicenciaEmpresa)
        q_obj = q_obj.outerjoin((sucursales,
                                    moviles.Sucursal == sucursales.Sucursal))
        q_obj = q_obj.filter(and_(
                                moviles.LR == id_licencia,
                                moviles.Sucursal == sucursales.Sucursal,
                                )
                            )

        registro = q_obj.first()

        session.close()

        if registro is None:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo obtener la empresa a la que pertenece el móvil')
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = registro.LicenciaEmpresa
        return rta

    def obtener_movil_del_chofer(self, dni):
        '''
            Esta función se encarga de devolver el id_movil de un
            chofer dado.
            Si no encuentra el dato devuelve ''

            Argumento:
                dni (int)

            Retorna:
                (str)
        '''

        session = self.cp.binded_session()

        choferes = self.cp.tabla('choferes')

        none = None

        q_obj = session.query(choferes.idMovil)
        q_obj = q_obj.filter(and_(
                                choferes.DNI == dni,
                                choferes.idMovil != none,
                            )
                        )

        if q_obj.count() == 0:
            session.close()
            return ''

        registro = q_obj.first()

        id_movil = registro.idMovil

        session.close()

        return id_movil

    def obtener_licencia_del_movil(self, id_movil):
        '''
            Esta función se encarga de devolver el LR (licencia) de
            un móvil dado.

            Argumento:
                id_movil (str)

            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
        '''

        rta = {}

        session = self.cp.binded_session()

        moviles = self.cp.tabla('moviles')

        q_obj = session.query(moviles.LR)
        q_obj = q_obj.filter(moviles.IdMovil == id_movil)

        registro = q_obj.first()

        session.close()

        if registro is None:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo obtener la licencia del móvil dado')
            return rta

        id_licencia = registro.LR

        if id_licencia is None:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'El móvil dado no tiene una licencia cargada')
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = id_licencia
        return rta

    def listar(self, mostrar_por='id_movil', id_flota=None):
        '''
            Esta función se encarga de listar los móviles existentes,
            si se pasa un id_flota se debe filtrar por la flota dada.
            Devuelve el listado de móviles por su id, patente o licencia.

            Argumentos:
                mostrar_por (str)
                id_flota (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (list de dict)
                                            ['id_movil_unico'] (int)
                                            ['movil'] (str)
        '''

        rta = {}

        none = None

        session = self.cp.binded_session()
        moviles = self.cp.tabla('moviles')
        q_obj = session.query(
            moviles.IdMovil, moviles.Patente, moviles.LR, moviles.IdUnico
        )

        q_obj = q_obj.filter(moviles.IdUnico != none)
        if id_flota is not None:
            q_obj = q_obj.filter(moviles.IdFlota == id_flota)

        if mostrar_por == 'patente':
            q_obj = q_obj.order_by(moviles.Patente.asc())
        elif mostrar_por == 'licencia':
            q_obj = q_obj.order_by(moviles.LR.asc())
        else:
            q_obj = q_obj.order_by(moviles.IdMovil.asc())

        registros = q_obj.all()

        session.close()

        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontraron móviles')
            return rta

        lista = []
        for registro in registros:
            dic = {}
            dic['id_movil_unico'] = registro.IdUnico

            if mostrar_por == 'patente':
                movil = registro.Patente
            elif mostrar_por == 'licencia':
                movil = registro.LR
            else:
                movil = registro.IdMovil

            dic['movil'] = str(movil)
            lista.append(dic)

        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def motor_detenido(self, estado1):
        '''
            Esta función se encarga de determinar si el motor del móvil
            está detenido o no.

            Argumentos:
                estado1 (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (bool)
        '''

        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = False

        if estado1 == 8:
            rta['Respuesta'] = True
        elif estado1 == 9:
            rta['Respuesta'] = True
        elif estado1 == 10:
            rta['Respuesta'] = True
        elif estado1 == 12:
            rta['Respuesta'] = True
        elif estado1 == 13:
            rta['Respuesta'] = True
        elif estado1 == 14:
            rta['Respuesta'] = True

        return rta

    def obtener_moviles_activos(self, fecha_desde, fecha_hasta):
        '''
            Esta función se encarga de obtener todos los móviles que reportaron
            por lo menos una vez en el día.
            Dado una fecha desde y una fecha hasta retornar todos los móviles
            que trabajaron en esos días.

            Argumentos:
                fecha_desde (str) dd/mm/yyyy
                fecha_hasta (str) dd/mm/yyyy

            Funciones:
                LogParser().obtener_rutas_logs_moviles

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de list)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Paso la fecha desde a date
        fecha_desde_date = datetime.strptime(fecha_desde, '%d/%m/%Y')

        # Paso la fecha hasta a date
        fecha_hasta_date = datetime.strptime(fecha_hasta, '%d/%m/%Y')

        # Llamo a la función obtener_rutas_logs_moviles
        rta_obtener_rutas_logs_moviles = \
            self.log_parser.obtener_rutas_logs_moviles(
                fecha_desde_date, fecha_hasta_date
            )

        # Si no se pudieron encontrar las rutas de los logs
        if rta_obtener_rutas_logs_moviles['Estado'] is False:
            # Retorno el error
            return rta_obtener_rutas_logs_moviles

        # Creo una lista para ir guardando los móviles activos
        moviles_activos = []

        # Recorro cada ruta de logs de los móviles
        for ruta_log in rta_obtener_rutas_logs_moviles['Respuesta']:
            # Por cada ruta obtengo los datos del nombre del log
            rta_obtener_datos_nombre_log = \
                self.log_parser.obtener_datos_nombre_log(ruta_log)
            # Si no puedo obtener los nombres del log
            if rta_obtener_datos_nombre_log['Estado'] is False:
                # Paso a la siguiente ruta de log
                continue

            # Obtengo los datos de cada log
            rta_obtener_datos_log = self.log_parser.obtener_datos_log(ruta_log)
            # Si no se pudieron obtener los datos del log
            if rta_obtener_datos_log['Estado'] is False:
                # Paso a la siguiente ruta de log
                continue

            # Si tengo los datos del log me quedo con solo la primer línea
            primer_linea_log = rta_obtener_datos_log['Respuesta'][0]

            # Guardo los datos del móvil que necesito
            id_flota = rta_obtener_datos_nombre_log['Respuesta']['id_flota']
            patente = ''
            id_movil = primer_linea_log[0]
            licencia = primer_linea_log[15].replace('\n', '')
            fecha = rta_obtener_datos_nombre_log['Respuesta']['fecha']

            # Dejo los datos en una lista
            datos = [id_flota, patente, id_movil, licencia, fecha]

            # Agrego cada dato a la lista de moviles_activos
            moviles_activos.append(datos)

        # Si no tengo móviles activos
        if len(moviles_activos) == 0:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontraron móviles activos entre las fechas %s y %s'
            ) % (fecha_desde, fecha_hasta)
            return rta

        # Si tengo móviles activo retorno True y la lista
        rta['Estado'] = True
        rta['Respuesta'] = moviles_activos
        return rta

    def obtener(self, id_movil, id_flota=None):
        '''
            Esta función se encarga de obtener todos los datos de un móvil.

            Argumentos:
                id_movil (str)
                id_flota (int)

            Funciones:
                Flotas -> obtener_fecha_hora_flota
                Fecha -> timedelta_to_datetime

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                    ['id_movil'] (str)
                                    ['descripcion'] (str)
                                    ['id_flota'] (int)
                                    ['patente'] (str)
                                    ['licencia'] (str)
                                    ['velocidad'] (int)
                                    ['estado'] (int)
                                    ['latitud'] (float) // en grados
                                    ['longitud'] (float) // en grados
                                    ['ult_pos'] (str) // yyyy-mm-dd hh:mm:ss
                                    ['dif_ult_pos'] (str) // 1900-01-01 hh:mm:ss
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Si en id_movil recibo un string vacío
        if id_movil == '':
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'El id del móvil no debe ser vacío')
            return rta

        # Me conecto con la base de datos "cp2000"
        session = self.cp.binded_session()

        # Obtengo la tabla "moviles"
        moviles = self.cp.tabla('moviles')

        # Obtengo los datos del móvil
        q_obj = session.query(
            moviles.IdMovil,
            moviles.Movil,
            moviles.IdFlota,
            moviles.Patente,
            moviles.LR,
            moviles.Latitud,
            moviles.Longitud,
            moviles.UltPosicion,
            moviles.Velocidad,
            moviles.Estado1,
            moviles.Autorizado,
            moviles.HabilitadoHasta
        )

        # Filtro por el id del móvil.
        q_obj = q_obj.filter(
            moviles.IdMovil == id_movil
        )

        # Verifico si id_flota no sea None.
        if id_flota is not None:

            # Filtro por el id_flota del móvil
            q_obj = q_obj.filter(
                moviles.IdFlota == id_flota,
            )

        # Obtengo el registro
        registro = q_obj.first()

        # Cierro la sesión con la base de datos "cp2000"
        session.close()

        # Si no se encontró el registro
        if registro is None:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró el móvil con id %s de la flota %s'
            ) % (id_movil, id_flota)
            return rta

        # El campo IdMovil lo debo pasar al formato iso-8859-1
        if registro.IdMovil is not None:
            try:
                id_movil = registro.IdMovil.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                id_movil = registro.IdMovil
        else:
            id_movil = ''

        # El campo Patente lo debo pasar al formato iso-8859-1
        if registro.Patente is not None:
            try:
                patente = registro.Patente.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                patente = registro.Patente
        else:
            patente = ''

        # El campo LR lo debo pasar al formato iso-8859-1
        if registro.LR is not None:
            try:
                licencia = registro.LR.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                licencia = registro.LR
        else:
            licencia = ''

        # El campo Movil lo debo pasar al formato iso-8859-1
        if registro.Movil is not None:
            try:
                descripcion = registro.Movil.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                descripcion = registro.Movil
        else:
            descripcion = ''

        # Guardo los datos del móvil obtenido
        dic = {}
        dic['id_movil'] = id_movil
        dic['id_flota'] = registro.IdFlota
        dic['patente'] = patente
        dic['licencia'] = licencia
        dic['velocidad'] = registro.Velocidad
        dic['estado'] = registro.Estado1
        dic['latitud'] = None
        dic['longitud'] = None
        dic['ult_pos'] = None
        dic['dif_ult_pos'] = None
        dic['autorizado'] = registro.Autorizado
        dic['descripcion'] = descripcion

        # Obtengo la fecha en datetime.
        habilitado_hasta = registro.HabilitadoHasta

        # Verifico que no sea None.
        if habilitado_hasta is not None:

            # paso la fecha a string.
            habilitado_hasta = habilitado_hasta.strftime(
                '%d/%m/%Y %H:%M:%S'
            )

        # Agrego el dato.
        dic['habilitado_hasta'] = habilitado_hasta

        # Verifico si id_flota no sea None.
        if id_flota is not None:
            # Valido que el móvil tenga posición y este actualizada

            # Obtengo la fecha y hora actual de la flota
            rta_obtener_fecha_hora_flota = self.flotas.obtener_fecha_hora_flota(
                id_flota
            )
            # Si no se pudo obtener la fecha y hora actual de la flota
            if rta_obtener_fecha_hora_flota['Estado'] is False:
                # # Retorno False y un mensaje
                # return rta_obtener_fecha_hora_flota
                # NOTA: para no depender de at.map si o si
                # obtengo la fecha_hora actual en el caso de que no pueda
                # obtener la fecha según la flota/ciudad
                fecha_hora_actual = self.dth.get_now()
            else:
                # Si se pudo obtener la fecha y hora según la flota la guardo
                fecha_hora_actual = rta_obtener_fecha_hora_flota['Respuesta']
        else:
            # Obtengo la fecha_hora actual.
            fecha_hora_actual = self.dth.get_now()

        # Me fijo si la posición del móvil está actualizada
        if registro.Latitud != 0 \
                and registro.Longitud != 0 \
                and registro.Latitud is not None \
                and registro.Longitud is not None \
                and registro.UltPosicion is not None \
                and registro.UltPosicion >= fecha_hora_actual - timedelta(
                                                                seconds=120):
            # Paso la latitud del móvil a grados
            dic['latitud'] = float(registro.Latitud / 3600)
            # Paso la longitud del móvil a grados
            dic['longitud'] = float(registro.Longitud / 3600)

        # Si tengo la última posición del móvil
        if registro.UltPosicion is not None \
                and str(registro.UltPosicion) != '1900-01-01 00:00:00':
            # Guardo la última posición del móvil
            dic['ult_pos'] = str(registro.UltPosicion)
            # Obtengo la diferencia de tiempo que hay entre la última
            # posición del móvil y la fecha y hora de la flota (ciudad)
            dif_ult_pos = fecha_hora_actual - registro.UltPosicion
            # Paso la diferencia horaria a datetime
            rta_timedelta_to_datetime = self.fecha.timedelta_to_datetime(
                dif_ult_pos
            )
            # Si pude obtener la diferencia horaria en formato datetime
            if rta_timedelta_to_datetime['Estado'] is True:
                # La guardo en el dic junto con los demás datos del móvil
                dic['dif_ult_pos'] = str(rta_timedelta_to_datetime['Respuesta'])

        # Retorno True y los datos del móvil
        rta['Estado'] = True
        rta['Respuesta'] = dic
        return rta

    def anotar_en_zona_x_pos(self, id_movil, lat_movil, lon_movil, id_sucursal):
        '''
            Esta función se va a encargar de anotar a un móvil en una parada
            o en una zona según corresponda.

            Argumentos:
                id_movil (str)
                lat_movil (float) // en grados
                lon_movil (float) // en grados
                id_sucursal (int)

            Funciones:
                Parametros -> obtener_parametro
                FuncionesGeo -> mts_entre_puntos
                Moviles -> actualizar_zona_movil
                Zonas -> obtener_zonas_cp_por_punto
                Zonas -> obtener

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Me fijo si tengo at.map
        if self.zonas_geo is None or self.fg is None:
            # Si no tengo zonas_geo retorno False y un mensaje
            return respuesta(
                False,
                _(u'No tengo at.map para anotar en zona al móvil por su posición')
            )

        # Valido que la posición sea válida
        if lat_movil == 0 or lon_movil == 0 \
                or lat_movil is None or lon_movil is None:
            # Si la posición no es válida retorno False y un mensaje
            return respuesta(
                False,
                _(u'La posición del móvil %s no es válida') % id_movil
            )

        # Llamo a la función obtener_zonas_cp_por_punto
        punto = [lat_movil, lon_movil]
        rta_obtener_zona_cp = self.zonas_geo.obtener_zonas_cp_por_punto(
            punto, id_sucursal
        )
        # Si no se pudo obtener la zona donde está en móvil
        if rta_obtener_zona_cp['Estado'] is False:
            # Se va a setear la zona del móvil en None para sacarlo de la zona
            # Llamo a la función actualizar_zona_movil
            id_zona_cp = None
            rta_actualizar_zona_movil = self.actualizar_zona_movil(
                id_movil, id_zona_cp, 'zona'
            )
            # Retorno la respuesta de la función actualizar_zona_movil
            return rta_actualizar_zona_movil
        # Guardo el id de la zona cp obtenido
        id_zona_cp = rta_obtener_zona_cp['Respuesta'][0]['IdZonaCp']
        # Guardo el id de la zona de geodata obtenido
        id_zona_geo = rta_obtener_zona_cp['Respuesta'][0]['IdZona']

        # Obtengo los datos geométricos de la zona obtenida
        rta_obtener_zona_geo = self.zonas_geo.obtener(id_zona_geo)
        # Si no se pudieron obtener los datos geométricos de la zona
        if rta_obtener_zona_geo['Estado'] is False:
            # Retorno el error
            return rta_obtener_zona_geo
        # Guardo los datos de la zona geo obtenida
        zona_geo = rta_obtener_zona_geo['Respuesta']

        # Si la zona no tiene una parada
        if len(zona_geo['Marcador']) == 0:
            # Llamo a la función actualizar_zona_movil
            # con el id_zona_cp obtenido, se anota al móvil por zona
            rta_actualizar_zona_movil = self.actualizar_zona_movil(
                id_movil, id_zona_cp, 'zona'
            )
            # Retorno la respuesta de la función actualizar_zona_movil
            return rta_actualizar_zona_movil

        # Si la zona tiene una parada
        # Llamo a la función obtener_parametro
        # para obtener el valor de "DistParada"
        dist_parada = self.parametros.obtener_parametro(
            'DistParada', id_sucursal
        )
        # Si no se pudo obtener el parámetro "DistParada"
        if dist_parada is None:
            # Retorno False y un mensaje de error
            return respuesta(
                False,
                _(u'No se pudo obtener el valor del parámetro de la distancia de la parada')
            )

        # Guardo la latitud y la longitud de la parada
        lat_parada = zona_geo['Marcador'][0]
        lon_parada = zona_geo['Marcador'][1]

        # Obtengo la distancia en metros entre el móvil y la parada
        dist_parada_movil = self.fg.mts_entre_puntos(
            lat_movil,
            lon_movil,
            lat_parada,
            lon_parada,
        )
        # Si la distancia es menor o igual a la establecida en el parámetro
        if dist_parada_movil <= dist_parada:
            # Llamo a la función actualizar_zona_movil
            # con el id_zona_cp obtenido, se anota al móvil en la parada
            rta_actualizar_zona_movil = self.actualizar_zona_movil(
                id_movil, id_zona_cp, 'parada'
            )
            # Retorno la respuesta de la función actualizar_zona_movil
            return rta_actualizar_zona_movil

        # Si el móvil está en la zona pero no está cerca de la parada
        # Llamo a la función actualizar_zona_movil
        # para indicar que el móvil no está en la parada de la zona
        id_zona_cp = None
        rta_actualizar_zona_movil = self.actualizar_zona_movil(
            id_movil, id_zona_cp, 'parada'
        )
        # Retorno la respuesta de la función actualizar_zona_movil
        return rta_actualizar_zona_movil

    def actualizar_zona_movil(self, id_movil, id_zona, zona_o_parada='zona'):
        '''
            Esta función se encarga de actualizar la zona de un móvil dado.
            Si la zona recibida es la misma que la que ya tiene asignada
            el móvil no se debe actualizar ningún dato del móvil.

            Argumentos:
                id_movil (str)
                id_zona (int)
                zona_o_parada (str)

            Funciones:
                DateTimeHelper -> get_now

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Primero busco la zona actual del móvil en cuestión

        # Abro la sesión con la base de datos
        session = self.cp.binded_session()
        # Obtengo la tabla móviles
        moviles = self.cp.tabla('moviles')
        # Busco el id del móvil y su zona actual
        q_obj = session.query(moviles.IdMovil, moviles.IdZonaActual)
        # Filtro por el id del móvil dado
        q_obj = q_obj.filter(moviles.IdMovil == id_movil)
        # Guardo si se encontró algún registro
        registro = q_obj.first()
        # Si no se pudo obtener el móvil según su id
        if registro is None:
            # Cierro la sesión con la base de datos
            session.close()
            # Retorno False y un mensaje
            return respuesta(
                False,
                _(u'No se encontró el móvil %s') % id_movil
            )

        # Si el móvil ya está en la zona dada
        if id_zona == registro.IdZonaActual:
            # Cierro la sesión con la base de datos
            session.close()
            # Retorno False y un mensaje
            # Si el móvil sigue en la zona None (ninguna zona)
            if id_zona is None:
                # Muestro un mensaje que lo indica
                return respuesta(
                    False,
                    _(u'El móvil %s no está en ninguna %s')
                    % (id_movil, zona_o_parada)
                )
            # Si el móvil no cambió de zona muestro otro mensaje
            return respuesta(
                False,
                _(u'El móvil %s sigue ubicado en la %s %s') %
                (id_movil, zona_o_parada, id_zona)
            )

        # Obtengo la fecha y hora actual
        fecha_hora_actual = self.dth.get_now()

        # Creo un diccionario con los datos a actualizar
        dic = {}
        dic['IdZonaActual'] = id_zona
        dic['FechaHoraDeIngreso'] = fecha_hora_actual
        dic['FechaHoraUltActividad'] = fecha_hora_actual
        # Actualizo la zona del móvil y su hora de ingreso
        q_obj.update(dic)
        session.commit()
        # Cierro la sesión con la base de datos
        session.close()

        # Si el móvil estaba en una zona y ahora está en la zona None
        if id_zona is None:
            # Retorno False e indico que el móvil salió de la zona
            return respuesta(
                False,
                _(u'El móvil %s salió de la %s %s')
                % (id_movil, zona_o_parada, registro.IdZonaActual)
            )

        # Si el móvil cambió de una zona a la otra retorno True y un mensaje
        return respuesta(
            True,
            _(u'El móvil %s se anotó en la %s %s')
            % (id_movil, zona_o_parada, id_zona)
        )

    def tiene_consola(self, id_movil):
        '''
            Esta función se encarga de determinar si el móvil tiene consola.

            Argumento:
                id_movil (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Abro la sesión con la base de datos
        session = self.cp.binded_session()
        # Obtengo la tabla móviles
        moviles = self.cp.tabla('moviles')
        # Busco los datos del móvil
        q_obj = session.query(
            moviles.IdMovil, moviles.IP, moviles.Id_Equipo, moviles.TermEnLinea,
            moviles.EdadGps
        )
        # Filtro por el id del móvil dado
        q_obj = q_obj.filter(moviles.IdMovil == id_movil)
        # Guardo si se encontró algún registro
        registro = q_obj.first()
        # Cierro la sesión con la base de datos
        session.close()

        # Si no se pudo obtener el móvil según su id
        if registro is None:
            # Retorno False y un mensaje
            return respuesta(
                False,
                _(u'No se encontró el móvil %s') % id_movil
            )

        # Si el móvil no tiene una IP, no tiene un equipo o no está en línea
        # o la edad del GPS es mayor a 30 segundos
        if registro.IP == '0.0.0.0' \
                or registro.Id_Equipo == 0 \
                or registro.TermEnLinea == 0 \
                or registro.EdadGps > 30:
            # Considero que el móvil no tiene consola
            return respuesta(
                False,
                _(u'El móvil %s no tiene consola') % id_movil
            )

        # Si el móvil tiene una IP, un equipo y la terminal en línea
        # considero que el móvil tiene consola
        return respuesta(
            True,
            _(u'El móvil %s tiene consola') % id_movil
        )

    def actualizar_ping_cpmobile(self, id_movil):
        '''
            Esta función se encarga de actualizar el ultimo PING del CpMobile.
            Solo debe actualizar este dato, es para indicar que el móvil está
            utilizando el CpMobile pero envía su posición por la consola.

            Argumento:
                id_movil (str)

            Funciones:
                DateTimeHelper().get_now

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Abro la sesión con la base de datos
        session = self.cp.binded_session()
        # Obtengo la tabla móviles
        moviles = self.cp.tabla('moviles')
        # Busco los datos del móvil
        q_obj = session.query(
            moviles.IdMovil, moviles.UltPingCpMobile,
            moviles.IP, moviles.Id_Equipo, moviles.TermEnLinea,
            moviles.EdadGps
        )
        # Filtro por el id del móvil dado
        q_obj = q_obj.filter(moviles.IdMovil == id_movil)
        # Guardo si se encontró algún registro
        registro = q_obj.first()

        # Si no se pudo obtener el móvil según su id
        if registro is None:
            # Cierro la sesión con la base de datos
            session.close()
            # Retorno False y un mensaje
            return respuesta(
                False,
                _(u'No se encontró el móvil %s') % id_movil
            )

        # Obtengo la fecha y hora actual
        fecha_hora_actual = self.dth.get_now()

        # Actualizo el dato UltPingCpMobile con la fecha y hora actual
        dic = {}
        dic['UltPingCpMobile'] = fecha_hora_actual

        # Si el móvil no tiene una IP, no tiene un equipo o no está en línea
        # o la edad del GPS es mayor a 30 segundos
        if registro.IP == '0.0.0.0' \
                or registro.Id_Equipo == 0 \
                or registro.TermEnLinea == 0 \
                or registro.EdadGps > 30:
            # Considero que el móvil no tiene consola
            # y actualizo el campo "Time_UltimoComando"
            # con la fecha y hora actual
            dic['Time_UltimoComando'] = fecha_hora_actual

        q_obj.update(dic)
        session.commit()
        # Cierro la sesión con la base de datos
        session.close()

        # Si se pudo actualizare el PING del CpMobile retorno True y un mensaje
        return respuesta(
            True,
            _(u'El móvil %s actualizó su último PING del CpMobile') % id_movil
        )

    def __grabar_movil_activo(self, id_flota, id_movil, fecha,
                              patente, licencia):
        '''
            Esta función se encarga de agregar un registro en la tabla
            "moviles_activos".

            Argumentos:
                id_flota (int)
                id_movil (str)
                fecha (str) // yyyy-mm-dd
                patente (str)
                licencia (str)

            Funciones:
                at.utiles.conversiones -> respuesta

            Retorno:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Guardo los datos dato
        dic = {}
        dic['IdFlota'] = id_flota
        dic['IdMovil'] = id_movil
        dic['Fecha'] = fecha
        dic['Patente'] = patente
        dic['LR'] = licencia

        # Agrego el móvil en la tabla "moviles_activos"
        rta = self.cp.insertar('moviles_activos', **dic)
        # Si no se pudo grabar el registro
        if rta['Estado'] is False:
            # Retorno False y un mensaje
            return respuesta(
                False, _(u'No se pudo grabar el móvil como activo')
            )

        # Si se pudo grabar el móvil como activo retorno True y un mensaje
        return respuesta(True, _(u'Se pudo grabar el móvil como activo'))

    def grabar_moviles_activos(self):
        '''
            Esta función va a grabar todos los móviles que reportaron datos en
            la tabla "moviles_activos", si un móvil ya fue registrado en el día
            no lo va a volver a grabar.

            Funciones:
                at.utiles.conversiones -> respuesta
                at.utiles.servidores -> DateTimeHelper -> get_now
                at.utiles.moviles -> Moviles -> __grabar_movil_activo

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtenego la fecha actual
        fecha_actual = self.dth.get_date()

        # Abro la sesión con la base de datos "cp"
        session = self.cp.binded_session()

        # Obtengo la tabla "moviles"
        moviles = self.cp.tabla('moviles')

        # Armo la fecha hora desde y la fecha y hora hasta
        fecha_hora_desde = str(fecha_actual) + ' 00:00:00'
        fecha_hora_hasta = str(fecha_actual) + ' 23:59:59'

        # Busco los móviles que reportaron datos en el día actual
        q_obj = session.query(
            moviles.IdFlota,
            moviles.IdMovil,
            moviles.Patente,
            moviles.LR
        )

        # Filtro por la fecha y hora desde y hasta
        q_obj = q_obj.filter(
            and_(
                moviles.Time_UltimoComando >= fecha_hora_desde,
                moviles.Time_UltimoComando <= fecha_hora_hasta
            )
        )

        # Guardo los móviles obtenidos
        registros = q_obj.all()

        # Cierro la sesión
        session.close()

        # Si no se encontraron móviles activos
        if len(registros) == 0:
            # Retorno False y un mensaje
            return respuesta(False, _(u'No hay móviles activos'))

        # Recorro los móviles obtenidos
        for registro in registros:
            # Intento grabar el móvil como activo
            self.__grabar_movil_activo(
                registro.IdFlota, registro.IdMovil, fecha_actual,
                registro.Patente, registro.LR
            )

        # Retornamos True y un mensaje
        return respuesta(
            True, _(u'Se grabaron los móviles activos correctamente')
        )

    def limpiar_moviles_activos(self):
        '''
            Esta función se encarga de borrar los registros con más de 2 años
            en la tabla "moviles_activos".

            Funciones:
                at.utiles.conversiones -> respuesta
                at.utiles.servidores -> DateTimeHelper -> get_date

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la fecha actual
        fecha_actual = self.dth.get_date()

        # Borro todos los registros que tenga más de 2 años (730 días)
        # en la tabla "moviles_activos"
        session = self.cp.binded_session()
        moviles_activos = self.cp.tabla('moviles_activos')
        q_obj = session.query(
            moviles_activos.IdFlota,
            moviles_activos.IdMovil,
            moviles_activos.Fecha
        )
        q_obj = q_obj.filter(
            moviles_activos.Fecha < fecha_actual - timedelta(days=730)
        )
        rta_delete = q_obj.delete(synchronize_session=False)
        session.commit()
        session.close()

        # Si no se encontraron registros para limpiar
        if rta_delete == 0:
            # Retorno False y un mensaje
            return respuesta(
                False,
                _(u'No hay registros para limpiar de la tabla moviles_activos')
            )

        # Si se limpiaron registros retorno True y un mensaje
        return respuesta(
            True,
            _(u'Se limpiaron %s registros de la tabla moviles_activos')
            % rta_delete
        )

    def listar_moviles_activos(self, fecha_desde, fecha_hasta,
                               cant_dias_activos=0, detallado=False,
                               id_flota=None, pagina=None, cantidad=None,
                               agrupar_por_id_movil=False):
        '''
            Esta función se encarga de determinar cuantos móviles activos
            tiene una flota dada entre una fecha desde y otra hasta.

            Argumentos:
                fecha_desde (str) dd/mm/yyyy
                fecha_hasta (str) dd/mm/yyyy
                cant_dias_activos (int)
                detallado (bool)
                id_flota (int)
                pagina (int)
                cantidad (int)
                agrupar_por_id_movil (bool)

            Funciones:
                Fecha().pasar_a_date

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                        ['Datos'] (list)
                                        ['Total'] (list)
                                        ['Detallado'] (bool)
                                        ['CantRegistros'] (int)
        '''

        rta = {}

        none = None

        if detallado is True and id_flota is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'Para obtener el informe detallado debe seleccionar una flota'
            )
            return rta

        rta_pasar_a_date = self.fecha.pasar_a_date(fecha_desde)
        if rta_pasar_a_date['Estado'] is False:
            rta['Estado'] = False
            rta['Respuesta'] = rta_pasar_a_date['Mensaje']
            return rta
        fecha_desde_date = rta_pasar_a_date['Mensaje']

        rta_pasar_a_date = self.fecha.pasar_a_date(fecha_hasta)
        if rta_pasar_a_date['Estado'] is False:
            rta['Estado'] = False
            rta['Respuesta'] = rta_pasar_a_date['Mensaje']
            return rta
        fecha_hasta_date = rta_pasar_a_date['Mensaje']

        dif_fechas = fecha_hasta_date - fecha_desde_date
        if dif_fechas > timedelta(days=31):
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'Debe ingresar un rango de fechas menor o igual a 31 días')
            return rta

        session = self.cp.binded_session()

        moviles_activos = self.cp.tabla('moviles_activos')
        flotas = self.cp.tabla('flotas')

        q_obj = session.query(
            moviles_activos.IdMovil,
            moviles_activos.Patente,
            moviles_activos.LR,
            func.count(moviles_activos.IdMovil).label('cantidad'),
            flotas.Nombre.label('nombre_flota'),
        )

        q_obj = q_obj.outerjoin((
            flotas,
            moviles_activos.IdFlota == flotas.IdFlota
        ))

        q_obj = q_obj.filter(
            and_(
                moviles_activos.Fecha >= fecha_desde_date,
                moviles_activos.Fecha <= fecha_hasta_date,
                moviles_activos.IdMovil != none,
                moviles_activos.Patente != none,
                moviles_activos.LR != none,
            )
        )

        # NOTA ################################################################
        # Si id_flota es 0 quiere decir que no voy a filtrar por la flota
        # porque se trata de la base de datos del cliente actual y necesito
        # realizar el informe detallado.
        #######################################################################

        if id_flota is not None and id_flota != 0:
            q_obj = q_obj.filter(moviles_activos.IdFlota == id_flota)

        q_obj = q_obj.group_by(moviles_activos.IdMovil, moviles_activos.IdFlota)

        if detallado:
            q_obj = q_obj.group_by(moviles_activos.Patente, moviles_activos.LR)

        q_obj = q_obj.having(
            func.count(moviles_activos.IdMovil) >= cant_dias_activos
        )

        q_obj = q_obj.order_by(
            flotas.Nombre.asc(), func.count(moviles_activos.IdMovil).desc()
        )

        cant_registros = 0

        if detallado is False and agrupar_por_id_movil is False:
            q_obj = q_obj.subquery()
            q_obj2 = session.query(
                q_obj.c.nombre_flota,
                func.count(q_obj.c.nombre_flota).label('cantidad')
            )
            q_obj2 = q_obj2.group_by(q_obj.c.nombre_flota)
            q_obj = q_obj2

        if detallado is True and pagina is not None and cantidad is not None:
            cant_registros = q_obj.count()

            q_obj = q_obj.limit(cantidad)
            q_obj = q_obj.offset(pagina * cantidad)

        registros = q_obj.all()

        session.close()

        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se encontraron móviles activos según los datos dados')
            return rta

        datos = []
        total = 0
        for registro in registros:
            if id_flota != 0:
                if registro.nombre_flota is None:
                    continue

            if detallado:
                fila = [
                    registro.nombre_flota, registro.IdMovil, registro.Patente,
                    registro.LR, registro.cantidad
                ]
                if cant_registros == 0:
                    total += 1
                else:
                    total = cant_registros
            elif agrupar_por_id_movil:
                fila = [
                    registro.nombre_flota, registro.IdMovil, registro.cantidad
                ]
                if cant_registros == 0:
                    total += 1
                else:
                    total = cant_registros
            else:
                fila = [registro.nombre_flota, registro.cantidad]
                total += registro.cantidad

            datos.append(fila)

        rta['Estado'] = True
        rta['Respuesta'] = {}
        rta['Respuesta']['Datos'] = datos
        rta['Respuesta']['Total'] = [total]
        rta['Respuesta']['Detallado'] = detallado
        rta['Respuesta']['CantRegistros'] = cant_registros
        return rta

    def listar_moviles(self):
        '''
            Esta función se encarga de listar todos los móviles.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (list de dict)
        '''

        rta = {}

        session = self.cp.binded_session()

        moviles = self.cp.tabla('moviles')

        q_obj = session.query(
            moviles.IdMovil,
            moviles.Estado1,
            moviles.Latitud,
            moviles.Longitud,
            moviles.Sucursal
        )

        registros = q_obj.all()

        session.close()

        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontraron móviles')
            return rta

        lista = []
        for registro in registros:
            dic = {}
            dic['id_movil'] = registro.IdMovil
            dic['estado1'] = registro.Estado1
            dic['latitud'] = float(registro.Latitud)
            dic['longitud'] = float(registro.Longitud)
            dic['id_sucursal'] = registro.Sucursal
            lista.append(dic)

        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta
