# -*- coding: utf-8 -*-

from datetime import datetime
from datetime import timedelta

from at.utiles import _
from at.orm.decorators import DBSwitch
from at.utiles.servidores import DateTimeHelper


@DBSwitch('cp')
class Parametros(object):
    '''
        Esta clase contiene funciones para acceder o modificar los parametros.
    '''

    ########### tabla "parametros" ##############

    def obtener_parametro(self, nombre_parametro, id_sucursal=0):
        '''
            Esta función se encarga de devolver el valor de un parámetro dado.

            Argumentos:
                nombre_parametro (str)
                id_sucursal (int)

            Retorna:
                parametro
        '''

        parametro = None

        session = self.cp.binded_session()

        parametros = self.cp.tabla('parametros')

        q_obj = session.query(parametros)
        if id_sucursal != 0:
            q_obj = q_obj.filter(parametros.idSucursal == id_sucursal)
        registro = q_obj.first()

        if registro is not None:
            keys = registro.__dict__.keys()
            for key in keys:
                if key == nombre_parametro:
                    parametro = registro.__dict__[nombre_parametro]

        session.close()

        return parametro

    def actualizar_parametros(self, actualizar, id_sucursal=0):
        '''
            Esta función se encarga de actualizar datos de la tabla parametros.

            Argumento:
                actualizar (dict)
                id_sucursal (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        dic = {}

        session = self.cp.binded_session()

        parametros = self.cp.tabla('parametros')

        q_obj = session.query(parametros)
        if id_sucursal != 0:
            q_obj = q_obj.filter(parametros.idSucursal == id_sucursal)

        if q_obj.count() == 0:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No hay parámetros cargados para la sucursal %s') % id_sucursal
            session.close()
            return dic

        registro = q_obj.first()
        registro = registro.__dict__

        for key_actualizar in actualizar.keys():
            if not key_actualizar in registro.keys():
                dic['Estado'] = False
                dic['Respuesta'] = _(u'No existe el campo %s en la tabla parametros') % key_actualizar
                session.close()
                return dic

        try:
            q_obj.update(actualizar, synchronize_session=False)
            session.commit()

            dic['Estado'] = True
            dic['Respuesta'] = _(u'Se han actualizado los parámetros')
        except:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No se pudieron actualizar los parámetros')

        session.close()

        return dic

    def obtener_parametros(self, id_sucursal=0):
        '''
            Esta función se encarga de devolver todos los parámetros de la
            tabla parametros.

            Argumento:
                id_sucursal (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (dict)
        '''

        dic = {}

        session = self.cp.binded_session()

        parametros = self.cp.tabla('parametros')

        q_obj = session.query(parametros)

        if id_sucursal != 0:
            q_obj = q_obj.filter(parametros.idSucursal == id_sucursal)

        registro = q_obj.first()

        session.close()

        if registro is None:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No hay parámetros cargados para la sucursal %s') % id_sucursal
            return dic

        registro = registro.__dict__

        del registro['_sa_instance_state']

        for key in registro.keys():
            if isinstance(registro[key], datetime):
                registro[key] = str(registro[key])

        dic['Estado'] = True
        dic['Respuesta'] = registro
        return dic

    def despacho_automatico(self, id_sucursal):
        '''
            Esta función consulta el valor del campo
            UltimaVezDespachoAutomatico según la sucursal dada,
            y compara si ese tiempo es menor a 1 minuto.
            Si el tiempo es mayor quiere decir que no se está corriendo el
            despachador automático.

            Argumentos:
                id_sucursal (int)

            Funciones:
                obtener_parametros
                get_now

            Retorna:
                int
        '''

        if id_sucursal == 0:
            id_sucursal = 1

        # obtengo la fecha y hora actual del servidor
        fecha_hora_actual = DateTimeHelper().get_now()

        # obtengo UltimaVezDespachoAutomatico
        ultima_vez_despacho_automatico = self.obtener_parametro(
                                                'UltimaVezDespachoAutomatico',
                                                id_sucursal)

        # si no existe el valor de UltimaVezDespachoAutomatico pongo
        # el despachador en manual, devuelvo False
        if ultima_vez_despacho_automatico is None:
            return 0

        # Obtengo la diferencia entre la fecha y hora actual y la última vez
        # que se despacho automáticamente
        diferencia = fecha_hora_actual - ultima_vez_despacho_automatico

        # si la diferencia entre la fecha y hora actual y la ultima vez de
        # despacho automático es menor a un minuto considero que el
        # despachador está en automático
        if diferencia < timedelta(minutes=1):
            return 1
        else:
            return 0

    ########### tabla "parametroslocales" ##############

    def obtener_parametrolocal(self, nombre_parametro, ide=1):
        '''
            Esta función se encarga de devolver el valor de un parámetro de
            la tabla parametroslocales.

            Argumentos:
                nombre_parametro (str)
                id (int)

            Retorna:
                El valor del parametro o None si no existe
        '''

        parametrolocal = None

        session = self.cp.binded_session()

        parametroslocales = self.cp.tabla('parametroslocales')

        q_obj = session.query(parametroslocales)
        q_obj = q_obj.filter(parametroslocales.id == ide)
        registro = q_obj.first()

        if registro is not None:
            keys = registro.__dict__.keys()
            for key in keys:
                if key == nombre_parametro:
                    parametrolocal = registro.__dict__[nombre_parametro]

        session.close()

        return parametrolocal

    def actualizar_parametroslocales(self, actualizar, ide=1):
        '''
            Esta función se encarga de actualizar datos de la tabla
            parametroslocales.

            Argumento:
                actualizar (dict)
                id (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        dic = {}

        session = self.cp.binded_session()

        parametroslocales = self.cp.tabla('parametroslocales')

        q_obj = session.query(parametroslocales)
        q_obj = q_obj.filter(parametroslocales.id == ide)

        if q_obj.count() == 0:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No hay parámetros locales cargados')
            session.close()
            return dic

        registro = q_obj.first()
        registro = registro.__dict__

        for key_actualizar in actualizar.keys():
            if not key_actualizar in registro.keys():
                dic['Estado'] = False
                dic['Respuesta'] = _(u'No existe el campo %s en la tabla parametroslocales') % key_actualizar
                return dic

        try:
            q_obj.update(actualizar, synchronize_session=False)
            session.commit()

            dic['Estado'] = True
            dic['Respuesta'] = _(u'Se han actualizado los parámetros locales')
        except:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No se pudieron actualizar los parámetros locales')

        session.close()
        return dic

    def obtener_parametroslocales(self):
        '''
            Esta función se encarga de devolver todos los parámetros de la
            tabla parametroslocales.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (dict)
        '''

        dic = {}

        session = self.cp.binded_session()

        parametroslocales = self.cp.tabla('parametroslocales')

        q_obj = session.query(parametroslocales)

        registro = q_obj.first()

        session.close()

        if registro is None:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No hay parámetros locales cargados')
            return dic

        datos = registro.__dict__

        del datos['_sa_instance_state']

        for key in datos.keys():
            if isinstance(datos[key], datetime):
                datos[key] = str(datos[key])

        dic['Estado'] = True
        dic['Respuesta'] = datos
        return dic
