# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import or_

from at.utiles import _
from at.orm.decorators import DBSwitch


@DBSwitch('cp')
class AnticipadosNombres(object):
    '''
        Este script se encarga de grabar el nombre del cliente en su agenda
        de anticipados y en todos sus servicios anticipados.
    '''

    def __init__(self):
        pass

    def grabar_nombre_en_anticipados(self):
        '''
            Esta función se encarga de obtener el nombre de cada cliente con
            servicios anticipados para setearlo en su agenda y todos sus
            servicios anticipados.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Creo la respuesta a retornar
        rta = {}

        # None para SqlAlchemy
        none = None

        # Abro la sesión de la base de datos
        session = self.cp.binded_session()

        # Obtengo la tabla serviciosanticipados
        serviciosanticipados = self.cp.tabla('serviciosanticipados')

        # Obtengo la tabla clientes
        clientes = self.cp.tabla('clientes')

         # Obtengo la tabla serviciosanticipadosdetalle
        serviciosanticipadosdetalle = self.cp.tabla(
            'serviciosanticipadosdetalle'
        )

        # Busco el teléfono y el nombre de cada anticipado
        q_obj_serviciosanticipados = session.query(
            serviciosanticipados.Telefono, serviciosanticipados.Nombre,
        )

        # Guardo los registros obtenidos
        registros_serviciosanticipados = q_obj_serviciosanticipados.all()

        # Si no tengo ningún anticipado
        if len(registros_serviciosanticipados) == 0:
            # Cierro la sesión con la base de datos
            session.close()

            # Retorno False con un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No hay anticipados cargados')
            return rta

        # Recorro los registros obtenidos
        for registro_serviciosanticipados in registros_serviciosanticipados:
            # Si ya tengo guardado el nombre en la agenda
            if registro_serviciosanticipados.Nombre != '':
                # Paso a la siguiente agenda de anticipados
                continue

            # Busco el nombre del cliente que tiene el anticipado
            q_obj_clientes = session.query(clientes.Telefono, clientes.Nombre)

            # Filtro por el número de teléfono
            q_obj_clientes = q_obj_clientes.filter(
                clientes.Telefono == registro_serviciosanticipados.Telefono
            )

            # Obtengo el cliente que tiene el anticipado
            registro_cliente = q_obj_clientes.first()

            # Si no obtengo el cliente que tiene el anticipado
            if registro_cliente is None:
                # Paso a la siguiente agenda de anticipados
                continue

            # Si el cliente no tiene un nombre guardado
            if registro_cliente.Nombre is None or registro_cliente.Nombre == '':
                # Paso a la siguiente agenda de anticipados
                continue

            # Busco la agenda a actualizar
            q_obj_update_serviciosanticipados = session.query(
                serviciosanticipados.Telefono
            )

            # Filtro por el teléfono del cliente en cuestión
            # y que no tenga un nombre guardado
            q_obj_update_serviciosanticipados = \
                                    q_obj_update_serviciosanticipados.filter(
                and_(
                    or_(
                        serviciosanticipados.Nombre == none,
                        serviciosanticipados.Nombre == '',
                    ),
                    serviciosanticipados.Telefono == registro_cliente.Telefono,
                )
            )

            # Actualizo el campo Nombre en la tabla serviciosanticipados
            # con el nombre del cliente
            dic = {}
            dic['Nombre'] = registro_cliente.Nombre
            q_obj_update_serviciosanticipados.update(
                dic, synchronize_session=False
            )

            # Ahora actualizo los servicios anticipados de la agenda del cliente
            q_obj_update_serviciosanticipadosdetalle = session.query(
                serviciosanticipadosdetalle.Telefono
            )
            q_obj_update_serviciosanticipadosdetalle = \
                            q_obj_update_serviciosanticipadosdetalle.filter(
                and_(
                    or_(
                        serviciosanticipadosdetalle.SolicitadoPara == none,
                        serviciosanticipadosdetalle.SolicitadoPara == '',
                    ),
                    serviciosanticipadosdetalle.Telefono ==
                                                    registro_cliente.Telefono,
                )
            )
            dic = {}
            dic['SolicitadoPara'] = registro_cliente.Nombre
            q_obj_update_serviciosanticipadosdetalle.update(
                dic, synchronize_session=False
            )

        # Hago el commit con la base de datos
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Si todo salió bien retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Se grabó el nombre de los clientes en todos sus anticipados')
        return rta

    def grabar_direccion_en_anticipados(self):
        '''
            Esta función debe obtener la dirección de un detalle de
            un anticipado y se debe guardar en el campo nombre de la
            agenda de los anticipados.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Creo la respuesta a retornar
        rta = {}

        # Abro la sesión de la base de datos
        session = self.cp.binded_session()

        # Obtengo la tabla serviciosanticipados
        serviciosanticipados = self.cp.tabla('serviciosanticipados')

         # Obtengo la tabla serviciosanticipadosdetalle
        serviciosanticipadosdetalle = self.cp.tabla(
            'serviciosanticipadosdetalle'
        )

        # Busco el teléfono y el nombre de cada anticipado
        q_obj_serviciosanticipados = session.query(
            serviciosanticipados.Telefono, serviciosanticipados.Nombre,
        )

        # Guardo los registros obtenidos
        registros_serviciosanticipados = q_obj_serviciosanticipados.all()

        # Si no tengo ningún anticipado
        if len(registros_serviciosanticipados) == 0:
            # Cierro la sesión con la base de datos
            session.close()

            # Retorno False con un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No hay anticipados cargados')
            return rta

        # Recorro los registros obtenidos, son agendas de anticipados
        for registro_serviciosanticipados in registros_serviciosanticipados:
            # Si ya tengo guardado la dirección en la agenda
            if registro_serviciosanticipados.Nombre != '':
                # Paso a la siguiente agenda de anticipados
                continue

            # Guardo el teléfono de la agenda
            telefono_agenda = registro_serviciosanticipados.Telefono

            # Busco una dirección de un detalle del anticipado
            q_obj_serviciosanticipadosdetalle = session.query(
                serviciosanticipadosdetalle.Direccion
            )

            # Filtro por el teléfono de la agenda
            q_obj_serviciosanticipadosdetalle = \
                q_obj_serviciosanticipadosdetalle.filter(
                    serviciosanticipadosdetalle.Telefono == telefono_agenda,
            )

            # Guardo el primer registro de los servicios anticipados detalle
            registro_serviciosanticipadosdetalle = \
                                    q_obj_serviciosanticipadosdetalle.first()

            # Si no tengo ningún detalle
            if registro_serviciosanticipadosdetalle is None:
                # Paso a la siguiente agenda
                continue

            # Guardo la dirección del detalle
            direccion_detalle = registro_serviciosanticipadosdetalle.Direccion

            # Busco la agenda a actualizar
            q_obj_update_serviciosanticipados = session.query(
                serviciosanticipados.Telefono
            )

            # Filtro por el teléfono del cliente en cuestión
            # y que no tenga un nombre guardado
            q_obj_update_serviciosanticipados = \
                                    q_obj_update_serviciosanticipados.filter(
                serviciosanticipados.Telefono == telefono_agenda,
            )

            # Actualizo el nombre de la agenda con la dirección del detalle
            dic = {}
            dic['Nombre'] = direccion_detalle
            q_obj_update_serviciosanticipados.update(
                dic, synchronize_session=False
            )

        # Hago el commit con la base de datos
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Si todo salió bien retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se grabó la dirección en todos los anticipados')
        return rta
