# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import func

from at.utiles import _
from at.orm.decorators import DBSwitch


@DBSwitch('gis2')
class Ciudades(object):
    '''
        Este script se encarga de realizar tareas relacionadas con las ciudades
        que utilizan las aplicaciones.
    '''

    def __init__(self):
        pass

    def cargar_ciudades_aplicaciones(self, id_provincia):
        '''
            Esta función se encarga de cargar todas las ciudades
            de una provincia en la tabla "ciudades_aplicaciones".

            Argumentos:
                id_provincia (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Abre la sesión con la base de datos gis2
        session = self.gis2.binded_session()

        # Cargo la tabla "ciudades_aplicaciones"
        ciudades_aplicaciones = self.gis2.tabla('ciudades_aplicaciones')

        # Cargo la tabla "ciudades"
        ciudades = self.gis2.tabla('ciudades')

        # Obtengo el id de cada ciudad
        q_obj = session.query(ciudades.id)

        # Filtro por la provincia dada
        q_obj = q_obj.filter(ciudades.id_provincia == id_provincia)

        # Ordeno de forma ascendente por el nombre de la ciudad
        q_obj = q_obj.order_by(ciudades.nombre.asc())

        # Si no se encontró ningún registro
        if q_obj.count() == 0:
            # Cierro la conexión con la base de datos
            session.close()

            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontraron ciudades de la provincia %s'
            ) % id_provincia
            return rta

        # Guardo todos los registros obtenidos
        registros = q_obj.all()

        # Recorro los registros obtenidos
        for registro in registros:
            # Me fijo si ya existe la ciudad que se quiere agregar
            q_obj = session.query(ciudades_aplicaciones)
            q_obj = q_obj.filter(ciudades_aplicaciones.id_ciudad == registro.id)
            # Si la ciudad ya está agregada en "ciudades_aplicaciones"
            if q_obj.count() != 0:
                # Paso a la siguiente ciudad
                continue

            # Obtengo el último id cargado en la tabla "ciudades_aplicaciones"
            ciudad_aplicacion = session.query(
                func.MAX(ciudades_aplicaciones.id).label('ultimo_id')
            ).first()

            # Si no tengo ningún registro cargado
            if ciudad_aplicacion.ultimo_id is None:
                # La primer ciudad va a tener el id 2
                # Reservo el id 1 para la ciudad principal del cliente
                ciudad_aplicacion.ultimo_id = 2
            else:
                # Si tengo un último id agregado
                # le sumo 1 para el siguiente registro
                ciudad_aplicacion.ultimo_id += 1

            # Agrego cada ciudad de la provincia
            # a la tabla "ciudades_aplicaciones"
            dic = {}
            dic['id'] = ciudad_aplicacion.ultimo_id
            dic['id_ciudad'] = registro.id
            agregar = self.gis2.agregar('ciudades_aplicaciones', **dic)
            print(agregar)
            print('Se agrego la ciudad %s' % registro.id)

        # Cierro la conexión con la base de datos
        session.close()

        # Retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se agregaron todas las ciudades de la provincia %s'
        ) % id_provincia
        return rta
