# -*- coding: utf-8 -*-

import os
from twisted.python import log
from sqlalchemy.sql.expression import and_

from at.utiles import _
from at.orm.utiles import obtener_bd
from at.utiles.log import start_custom_log
from at.utiles.fecha import Fecha

# Iniciamos nuestro propio log
start_custom_log(
    '/var/lib/at/logs/at.utiles/at.utiles', 'migracion_gold'
)


class MigrarBD(object):
    '''
        Esta clase contiene funciones útiles para realizar una migración de
        base de datos.
    '''

    def procesar_archivo_csv(self, ruta_archivo):
        '''
            Dado un archivo de texto separado por comas esta función se encarga
            de recorrer todos sus datos y crear una lista de diccionarios.

            Argumentos:
                ruta_archivo (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de dict)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Me fijo si la ruta del archivo recibido es válida
        if os.path.isfile(ruta_archivo) is False:
            # Si no encuentro el archivo retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el archivo %s') % ruta_archivo
            return rta

        # Abro el archivo dado
        archivo = open(ruta_archivo, 'r')

        # Seteo la posición de las líneas en 1
        pos_linea = 1

        # Creo una lista para ir guardando todos los registros
        registros = []

        # Intancio una lista donde se va a guardar la cabecera
        cabecera = []

        # Recorro cada lína del archivo separada por enters
        for linea in archivo.xreadlines():
            # Separo la linea por comas
            datos = linea.split(';')

            # Si estoy en la primera línea del archivo
            if pos_linea == 1:
                # Recorro los datos de la cabecera
                for dato in datos:
                    # Limpio los datos
                    dato = dato.replace('\n', '')
                    # Creo la cabecera
                    cabecera.append(dato)

                # Guardo la cantidad de campos que tiene la cabecera
                cant_campos_cabecera = len(cabecera)

                # Seteo pos_linea en 2
                pos_linea = 2
                # Y paso a la segunda línea
                continue

            # Si tengo más o menos datos que la cabecera
            if cant_campos_cabecera != len(datos):
                print('DATOS INVALIDOS!')
                print(datos)
                # Guardo en un log el error
                log.msg(
                    'Funcion: procesar_archivo_csv | Archivo: %s | Datos invalidos: %s' % (ruta_archivo, datos),
                    mode='migracion_gold',
                )
                continue

            # Creo un diccionario para guardar el registro
            dic = {}

            # Seteo la posición del dato en 0
            pos_dato = 0

            # Recorro los datos del registro
            for dato in datos:
                # Limpio los datos
                dato = dato.replace('\n', '')

                # Cada dato de la cabecera va a ser una llave
                # de un diccionario y el dato su valor
                dic[cabecera[pos_dato]] = dato

                # Sumo 1 a la posición del dato para pasar al siguiente
                pos_dato += 1

            # Agrego el diccionario a la lista de registros
            registros.append(dic)

            # Paso a la siguiente línea
            pos_linea += 1
            continue

        # Cierro el archivo
        archivo.close()

        # Si no se encontraro registros
        if len(registros) == 0:
            # Retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontraron registros en el archivo %s'
            ) % ruta_archivo
            return rta

        # Si todo salió bien retorno True y la lista con los datos
        rta['Estado'] = True
        rta['Respuesta'] = registros
        return rta


class MigrarGold(object):
    '''
        Esta clase contiene funciones para realizar la migración de la base
        de datos del cliente GOLD.
    '''

    def __init__(self):
        self.migrar_bd = MigrarBD()
        self.fechas = Fecha()

    def agregar_filtros(self, nombre_bd):
        '''
            Esta función se encarga de agregar registros en la tabla filtros.

            Argumentos:
                nombre_bd (str)

            Funciones:
                obtener_bd

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Agrego el filtro CTA.CTE.
        dic = {}
        dic['IdFiltro'] = 1
        dic['Nombre'] = u'Cuenta corriente'
        dic['Abreviatura'] = 'CTA.CTE.'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro CTA.CTE.')
        else:
            print('El filtro CTA.CTE. ya estaba agregado')

        # Agrego el filtro BAUL
        dic = {}
        dic['IdFiltro'] = 2
        dic['Nombre'] = u'Espacio en baul'
        dic['Abreviatura'] = 'BAUL'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro BAUL')
        else:
            print('El filtro BAUL ya estaba agregado')

        # Agrego el filtro AIRE
        dic = {}
        dic['IdFiltro'] = 3
        dic['Nombre'] = u'Con aire'
        dic['Abreviatura'] = 'AIRE'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro AIRE')
        else:
            print('El filtro AIRE ya estaba agregado')

        # Agrego el filtro SIN GNC
        dic = {}
        dic['IdFiltro'] = 4
        dic['Nombre'] = u'Sin GNC'
        dic['Abreviatura'] = 'SIN GNC'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro SIN GNC')
        else:
            print('El filtro SIN GNC ya estaba agregado')

        # Agrego el filtro VIP
        dic = {}
        dic['IdFiltro'] = 5
        dic['Nombre'] = u'Es VIP'
        dic['Abreviatura'] = 'VIP'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro VIP')
        else:
            print('El filtro VIP ya estaba agregado')

        # Agrego el filtro NAFTA
        dic = {}
        dic['IdFiltro'] = 6
        dic['Nombre'] = u'Con nafta'
        dic['Abreviatura'] = 'NAFTA'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro NAFTA')
        else:
            print('El filtro NAFTA ya estaba agregado')

        # Agrego el filtro DIESEL
        dic = {}
        dic['IdFiltro'] = 7
        dic['Nombre'] = u'Con diesel'
        dic['Abreviatura'] = 'DIESEL'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro DIESEL')
        else:
            print('El filtro DIESEL ya estaba agregado')

        # Agrego el filtro GAS
        dic = {}
        dic['IdFiltro'] = 8
        dic['Nombre'] = u'Con gas'
        dic['Abreviatura'] = 'GAS'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro GAS')
        else:
            print('El filtro GAS ya estaba agregado')

        # Agrego el filtro TICKETERA
        dic = {}
        dic['IdFiltro'] = 9
        dic['Nombre'] = u'Con ticketera'
        dic['Abreviatura'] = 'TICKETERA'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro TICKETERA')
        else:
            print('El filtro TICKETERA ya estaba agregado')

        # Agrego el filtro BAJA
        dic = {}
        dic['IdFiltro'] = 10
        dic['Nombre'] = u'Categoría baja'
        dic['Abreviatura'] = 'BAJA'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro BAJA')
        else:
            print('El filtro BAJA ya estaba agregado')

        # Agrego el filtro BRONCE
        dic = {}
        dic['IdFiltro'] = 11
        dic['Nombre'] = u'Categoría bronce'
        dic['Abreviatura'] = 'BRONCE'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro BRONCE')
        else:
            print('El filtro BRONCE ya estaba agregado')

        # Agrego el filtro PLATA
        dic = {}
        dic['IdFiltro'] = 12
        dic['Nombre'] = u'Categoría plata'
        dic['Abreviatura'] = 'PLATA'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro PLATA')
        else:
            print('El filtro PLATA ya estaba agregado')

        # Agrego el filtro GAMA
        dic = {}
        dic['IdFiltro'] = 13
        dic['Nombre'] = u'Categoría gama'
        dic['Abreviatura'] = 'GAMA'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro GAMA')
        else:
            print('El filtro GAMA ya estaba agregado')

        # Agrego el filtro HIPODROMO
        dic = {}
        dic['IdFiltro'] = 14
        dic['Nombre'] = u'Toma viajes del hipódormo'
        dic['Abreviatura'] = 'HIPODROMO'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro HIPODROMO')
        else:
            print('El filtro HIPODROMO ya estaba agregado')

        # Agrego el filtro CASINO
        dic = {}
        dic['IdFiltro'] = 15
        dic['Nombre'] = u'Toma viajes del casino'
        dic['Abreviatura'] = 'CASINO'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro CASINO')
        else:
            print('El filtro CASINO ya estaba agregado')

        # Agrego el filtro MENSAJERIA
        dic = {}
        dic['IdFiltro'] = 16
        dic['Nombre'] = u'Mensajería'
        dic['Abreviatura'] = 'MENSJ'
        dic['IdTipo'] = 2
        agregar = bd.agregar('filtros', **dic)
        if agregar is True:
            print('Se agrega el filtro MENSAJERIA')
        else:
            print('El filtro MENSAJERIA ya estaba agregado')

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se agregaron los filtros de forma correcta')
        return rta

    def agregar_aseguradoras(self, nombre_bd, ruta_archivo):
        '''
            Está función se encarga de migrar los datos de la tabla aseguradoras
            del cliente GOLD.

            Argumentos:
                nombre_bd (str)
                ruta_archivo (str)

            Funciones:
                obtener_bd
                MigrarBd().procesar_archivo_csv

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Llamo a la función procesar_archivo_csv
        rta_procesar_archivo_csv = self.migrar_bd.procesar_archivo_csv(
            ruta_archivo
        )
        # Si no se pudo procesar el archivo
        if rta_procesar_archivo_csv['Estado'] is False:
            # Retorno el error
            return rta_procesar_archivo_csv

        # Guardo la lista de aseguradoras
        aseguradoras = rta_procesar_archivo_csv['Respuesta']

        # Recorro las aseguradoras
        for aseguradora in aseguradoras:
            # Guardo los datos de cada aseguradora
            dic = {}
            dic['Codigo'] = aseguradora['numero']
            dic['RazonSocial'] = aseguradora['razon social']
            dic['Contacto'] = ''
            dic['Direccion'] = aseguradora['Direccion']
            if aseguradora['Localidad'] != '':
                dic['Direccion'] += ' (' + aseguradora['Localidad'] + ')'
            dic['Telefono'] = aseguradora['telefono']
            dic['Cuit'] = ''
            dic['Observaciones'] = ''

            # Agrego la aseguradora
            agregar = bd.agregar('aseguradoras', **dic)
            if agregar is True:
                print('Se agrega la aseguradora %s' % aseguradora['numero'])
            else:
                print('No se pudo agregar la aseguradora %s' % aseguradora['numero'])
                # Si no se pudo agregar el registro guardo en un log el error
                log.msg(
                    'Funcion: agregar_aseguradoras | Registro: %s' % agregar,
                    mode='migracion_gold',
                )

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se agregaron las aseguradoras de forma correcta')
        return rta

    def agregar_administradoras(self, nombre_bd, ruta_archivo):
        '''
            Está función se encarga de migrar los datos de la tabla
            administradoras del cliente GOLD.

            Argumentos:
                nombre_bd (str)
                ruta_archivo (str)

            Funciones:
                obtener_bd
                MigrarBd().procesar_archivo_csv

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Llamo a la función procesar_archivo_csv
        rta_procesar_archivo_csv = self.migrar_bd.procesar_archivo_csv(
            ruta_archivo
        )
        # Si no se pudo procesar el archivo
        if rta_procesar_archivo_csv['Estado'] is False:
            # Retorno el error
            return rta_procesar_archivo_csv

        # Guardo la lista de administradoras
        administradoras = rta_procesar_archivo_csv['Respuesta']

        # Recorro las administradoras
        for administradora in administradoras:
            # Guardo los datos de cada administradora
            dic = {}
            dic['Codigo'] = administradora['Codigo']
            dic['RazonSocial'] = administradora['Razon Social']
            dic['Contacto'] = administradora['Contacto']
            dic['Direccion'] = administradora['Direccion']
            if administradora['Localidad'] != '':
                dic['Direccion'] += ' (' + administradora['Localidad'] + ')'
            dic['Telefono'] = administradora['Telefono']
            dic['Cuit'] = administradora['CUIT']
            dic['Observaciones'] = administradora['Observaciones']

            # Agrego la administradora
            agregar = bd.agregar('administradoras', **dic)
            if agregar is True:
                print('Se agrega la administradora %s' % administradora['Codigo'])
            else:
                print('No se pudo agregar la administradora %s' % administradora['Codigo'])
                # Si no se pudo agregar el registro guardo en un log el error
                log.msg(
                    'Funcion: agregar_administradoras | Registro: %s' % agregar,
                    mode='migracion_gold',
                )

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se agregaron las administradoras de forma correcta')
        return rta

    def agregar_titulares(self, nombre_bd, ruta_archivo):
        '''
            Está función se encarga de migrar los datos de la tabla
            titulares del cliente GOLD.

            Argumentos:
                nombre_bd (str)
                ruta_archivo (str)

            Funciones:
                obtener_bd
                MigrarBd().procesar_archivo_csv

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Llamo a la función procesar_archivo_csv
        rta_procesar_archivo_csv = self.migrar_bd.procesar_archivo_csv(
            ruta_archivo
        )
        # Si no se pudo procesar el archivo
        if rta_procesar_archivo_csv['Estado'] is False:
            # Retorno el error
            return rta_procesar_archivo_csv

        # Guardo la lista de titulares
        titulares = rta_procesar_archivo_csv['Respuesta']

        # Recorro los titulares
        for titular in titulares:
            # Guardo los datos de cada titular
            dic = {}
            dic['Codigo'] = titular['Codigo']
            dic['Nombre'] = titular['Razon Social']
            dic['Email'] = ''
            dic['Direccion'] = titular['Direccion']
            if titular['Localidad'] != '':
                dic['Direccion'] += ' (' + titular['Localidad'] + ')'
            dic['Telefono'] = titular['Telefono']
            dic['Cuit'] = titular['CUIT']
            dic['CodigoAdministradora'] = titular['Administradora']
            dic['Observaciones'] = titular['Observaciones']

            # Agrego el titular
            agregar = bd.agregar('titulares', **dic)
            if agregar is True:
                print('Se agrega el titular %s' % titular['Codigo'])
            else:
                print('No se pudo agregar el titular %s' % titular['Codigo'])
                # Si no se pudo agregar el registro guardo en un log el error
                log.msg(
                    'Funcion: agregar_titulares | Registro: %s' % agregar,
                    mode='migracion_gold',
                )

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se agregaron los titulares de forma correcta')
        return rta

    def agregar_choferes(self, nombre_bd, ruta_archivo):
        '''
            Está función se encarga de migrar los datos de la tabla
            choferes del cliente GOLD.

            Argumentos:
                nombre_bd (str)
                ruta_archivo (str)

            Funciones:
                obtener_bd
                MigrarBd().procesar_archivo_csv
                Fecha().pasar_a_date

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Llamo a la función procesar_archivo_csv
        rta_procesar_archivo_csv = self.migrar_bd.procesar_archivo_csv(
            ruta_archivo
        )
        # Si no se pudo procesar el archivo
        if rta_procesar_archivo_csv['Estado'] is False:
            # Retorno el error
            return rta_procesar_archivo_csv

        # Guardo la lista de choferes
        choferes = rta_procesar_archivo_csv['Respuesta']

        # Recorro los choferes
        for chofer in choferes:
            # Guardo los datos de cada chofer
            dic = {}
            dic['DNI'] = chofer['Movil']
            dic['Apellido'] = ''
            dic['Nombre'] = chofer['Nombre']
            # Solo guardo el celular si tengo el dato
            if chofer['Celular'] != '0':
                dic['Telefono'] = chofer['Celular']
            else:
                dic['Telefono'] = ''
            dic['Domicilio'] = chofer['Direccion']
            if chofer['Localidad'] != '':
                dic['Domicilio'] += ' (' + chofer['Localidad'] + ')'
            dic['idMovil'] = chofer['Movil']
            dic['Habilitado'] = 1
            dic['Password'] = chofer['Movil']
            dic['Observaciones'] = chofer['Obs']
            dic['idSesion'] = ''
            dic['Puntaje'] = None
            dic['VerMapaEnMovil'] = 1
            dic['HabilitarMensajeDeVoz'] = 1
            dic['EnviarSiempreZonasPedidas'] = 1
            dic['ZonasPedidasSoloEnLibre'] = 0
            dic['TurnoDesde'] = '1900-01-01 00:00:00'
            dic['TurnoHasta'] = '1900-01-01 23:59:59'
            dic['Foto'] = None
            dic['Cuit'] = chofer['Cuit']
            dic['RegistroConducir'] = chofer['Registro']

            # Cambio el formato de la fecha de vencimiento de registro
            rta_pasar_a_date = self.fechas.pasar_a_date(
                chofer['Vencimiesto Registro']
            )
            if rta_pasar_a_date['Estado'] is True:
                vencimiento_registro = rta_pasar_a_date['Mensaje']
            else:
                vencimiento_registro = None
            dic['VencimientoRegistro'] = vencimiento_registro

            # Cambio el formato de la fecha de vencimiento de magnetica
            rta_pasar_a_date = self.fechas.pasar_a_date(
                chofer['Vencimiento Magnetica']
            )
            if rta_pasar_a_date['Estado'] is True:
                vencimiento_licencia = rta_pasar_a_date['Mensaje']
            else:
                vencimiento_licencia = None
            dic['VencimientoLicencia'] = vencimiento_licencia

            # Agrego el chofer
            agregar = bd.agregar('choferes', **dic)
            if agregar is True:
                print('Se agrega el chofer del movil %s' % chofer['Movil'])
            else:
                print('No se pudo agregar el chofer del movil %s' % chofer['Movil'])
                # Si no se pudo agregar el registro guardo en un log el error
                log.msg(
                    'Funcion: agregar_choferes | Registro: %s' % agregar,
                    mode='migracion_gold',
                )

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se agregaron los choferes de forma correcta')
        return rta

    def agregar_moviles(self, nombre_bd, ruta_archivo):
        '''
            Está función se encarga de migrar los datos de la tabla
            moviles del cliente GOLD.

            Argumentos:
                nombre_bd (str)
                ruta_archivo (str)

            Funciones:
                obtener_bd
                MigrarBd().procesar_archivo_csv
                Fecha().pasar_a_date

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Llamo a la función procesar_archivo_csv
        rta_procesar_archivo_csv = self.migrar_bd.procesar_archivo_csv(
            ruta_archivo
        )
        # Si no se pudo procesar el archivo
        if rta_procesar_archivo_csv['Estado'] is False:
            # Retorno el error
            return rta_procesar_archivo_csv

        # Guardo la lista de moviles
        moviles = rta_procesar_archivo_csv['Respuesta']

        # Recorro los moviles
        for movil in moviles:
            # Guardo los datos de cada móvil
            dic = {}
            dic['IdMovil'] = movil['movil']

            # Si no tengo la patente del móvil
            if movil['Patente'] == '':
                # Guardo en el campo Patente el id del móvil
                # porque este dato es obligatorio
                dic['Patente'] = movil['movil']
            else:
                # Si tengo la Patente la guardo
                dic['Patente'] = movil['Patente']

            dic['CUIT'] = movil['cuit']

            # Me fijo que tipo de IVA tiene el móvil
            if movil['TipoIva'] == '3':
                dic['TipoIVA'] = 'M'
            elif movil['TipoIva'] == '5':
                dic['TipoIVA'] = 'E'
            else:
                dic['TipoIVA'] = ''

            dic['CodigoTitular'] = movil['Titular']
            dic['Sucursal'] = 1
            dic['Direccion'] = movil['Direccion']
            if movil['Localidad'] != '':
                dic['Direccion'] += ' (' + movil['Localidad'] + ')'
            dic['Telefono'] = movil['Telefono']

            # Si tengo el dato de la licencia del móvil
            if movil['Licencia'] != '0':
                # Lo guardo en el móvil
                dic['LR'] = movil['Licencia']

            dic['Movil'] = movil['Marca']
            if movil['Modelo'] != '':
                dic['Movil'] += ' (' + movil['Modelo'] + ')'
            dic['CodigoAseguradora'] = movil['CodCompSeguro']
            dic['ClaveEmpresaDeRastreo'] = movil['ClaveRastreo']

            # Cambio el formato de la fecha de vencimiento de seguro
            rta_pasar_a_date = self.fechas.pasar_a_date(
                movil['VenceSeguro']
            )
            if rta_pasar_a_date['Estado'] is True:
                vencimiento_seguro = rta_pasar_a_date['Mensaje']
            else:
                vencimiento_seguro = None
            dic['VencSeguro'] = vencimiento_seguro

            dic['NumeroDeCertificadoTecnico'] = movil['CertTecReloj']

            # Cambio el formato de la fecha de ingreso
            rta_pasar_a_date = self.fechas.pasar_a_date(
                movil['FechaIngreso']
            )
            if rta_pasar_a_date['Estado'] is True:
                fecha_ingreso = rta_pasar_a_date['Mensaje']
            else:
                fecha_ingreso = None
            dic['FechaDeIngreso'] = fecha_ingreso

            dic['NumeroDePrecinto'] = movil['PrecintoNro']

            dic['Observaciones'] = movil['Observaciones']

            dic['CodigoTipoAbono'] = movil['TipoAbono']

            # Cambio el formato de la fecha del abono desde
            rta_pasar_a_date = self.fechas.pasar_a_date(
                movil['AbonoDesde']
            )
            if rta_pasar_a_date['Estado'] is True:
                abono_desde = rta_pasar_a_date['Mensaje']
            else:
                abono_desde = None
            dic['FechaAbonoDesde'] = abono_desde

            # Cambio el formato de la fecha del vencimiento de la licencia
            rta_pasar_a_date = self.fechas.pasar_a_date(
                movil['VenceLicencia']
            )
            if rta_pasar_a_date['Estado'] is True:
                vencimiento_licencia = rta_pasar_a_date['Mensaje']
            else:
                vencimiento_licencia = None
            dic['VencimientoLicencia'] = vencimiento_licencia

            # El cliente GOLD es la flota 41
            dic['IdFlota'] = 41

            ### EQUIPOS #######################################################
            # Si el móvil tiene un equipo de radio
            if movil['EquipoRadio'] != '':
                # Agrego un registro en la tabla "equipo"
                equipo = {}
                equipo['IdEquipo'] = movil['movil']
                equipo['Descripcion'] = 'Equipo de radio'
                equipo['SerialNumber'] = movil['EquipoRadio']
                equipo['IdMovil'] = movil['movil']
                agregar_equipo = bd.agregar('equipos', **equipo)
                if agregar_equipo is True:
                    print('Se agrega el equipo de radio del movil %s' % movil['movil'])
                else:
                    print('No se pudo agregar el equipo de radio del movil %s' % movil['movil'])
                    # Si no se pudo agregar el registro guardo en un log el error
                    log.msg(
                        'Funcion: agregar_moviles (EQUIPOS) | Registro: %s' % agregar_equipo,
                        mode='migracion_gold',
                    )
            ###################################################################

            ### FILTROS #######################################################
            # Solo agrego los filtros si el móvil tiene una licencia
            if movil['Licencia'] != '0':
                # Agrego los filtros del móvil

                # Si el móvil tiene el baul vacío
                if movil['BaulVacio'] == '1':
                    # Le agrego el filtro 2 que es BAUL
                    filtro = {}
                    filtro['IdFiltro'] = 2
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil tiene aire
                if movil['Aire'] == '1':
                    # Le agrego el filtro 3 que es AIRE
                    filtro = {}
                    filtro['IdFiltro'] = 3
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil tiene nafta
                if movil['Nafta'] == '1':
                    # Le agrego el filtro 6 que es NAFTA
                    filtro = {}
                    filtro['IdFiltro'] = 6
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil tiene diesel
                if movil['Diesel'] == '1':
                    # Le agrego el filtro 7 que es DIESEL
                    filtro = {}
                    filtro['IdFiltro'] = 7
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil tiene gas
                if movil['Gas'] == '1':
                    # Le agrego el filtro 8 que es GAS
                    filtro = {}
                    filtro['IdFiltro'] = 8
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil tiene ticketera
                if movil['Ticketera'] == '1':
                    # Le agrego el filtro 9 que es TICKETERA
                    filtro = {}
                    filtro['IdFiltro'] = 9
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil tiene categoría baja
                if movil['Categoria'] == 'BAJA':
                    # Le agrego el filtro 10 que es BAJA
                    filtro = {}
                    filtro['IdFiltro'] = 10
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil tiene categoría bronce
                if movil['Categoria'] == 'BRONCE':
                    # Le agrego el filtro 11 que es BRONCE
                    filtro = {}
                    filtro['IdFiltro'] = 11
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil tiene categoría plata
                if movil['Categoria'] == 'PLATA':
                    # Le agrego el filtro 12 que es PLATA
                    filtro = {}
                    filtro['IdFiltro'] = 12
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil tiene categoría gama
                if movil['Categoria'] == 'GAMA':
                    # Le agrego el filtro 13 que es GAMA
                    filtro = {}
                    filtro['IdFiltro'] = 13
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil toma viajes con cta. cte.
                if movil['TomaViajeCtaCte'] == '1':
                    # Le agrego el filtro 1 que es CTA.CTE.
                    filtro = {}
                    filtro['IdFiltro'] = 1
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil toma viajes del hipódromo
                if movil['TomaViajeHipod'] == '1':
                    # Le agrego el filtro 14 que es HIPODROMO
                    filtro = {}
                    filtro['IdFiltro'] = 14
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)

                # Si el móvil toma viajes del casino
                if movil['TomaViajeCasino'] == '1':
                    # Le agrego el filtro 15 que es CASINO
                    filtro = {}
                    filtro['IdFiltro'] = 15
                    filtro['LicenciaMovil'] = movil['Licencia']
                    bd.agregar('filtros_moviles', **filtro)
            ###################################################################

            ### TURNOS DE CHOFERES ############################################
            # Si tengo el celular del chofer del primer turno
            # El celular debe tener 6 o más números
            if len(movil['Celular']) >= 6:
                # Abro la sesión de la base de datos
                session = bd.binded_session()

                # Obtengo la tabla choferes
                choferes = bd.tabla('choferes')

                # Busco el chofer del móvil
                q_obj_choferes = session.query(
                    choferes.DNI, choferes.idMovil, choferes.Telefono
                )

                # Filtro por el móvil y el primer celular
                q_obj_choferes = q_obj_choferes.filter(
                    and_(
                        choferes.idMovil == dic['IdMovil'],
                        #choferes.Telefono == movil['Celular'],
                        choferes.Telefono.like(
                            '%' + movil['Celular'][-8:] + '%'
                        ),
                    )
                )

                # Actualizo el primer turno del chofer del móvil
                actualizar_chofer = {}
                actualizar_chofer['TurnoDesde'] = '1900-01-01 ' \
                                                  + movil['CelularDesde']
                actualizar_chofer['TurnoHasta'] = '1900-01-01 ' \
                                                  + movil['CelularHasta']
                q_obj_choferes.update(
                    actualizar_chofer, synchronize_session=False
                )

                # Hago el commit con la base de datos
                session.commit()

                # Cierro la sesión con la base de datos
                session.close()

            # Si tengo el celular del chofer del segundo turno
            # El celular debe tener 6 o más números
            if len(movil['Celular2']) >= 6:
                # Abro la sesión de la base de datos
                session = bd.binded_session()

                # Obtengo la tabla choferes
                choferes = bd.tabla('choferes')

                # Busco el chofer del móvil
                q_obj_choferes = session.query(choferes.DNI)

                # Filtro por el móvil y el segundo celular
                q_obj_choferes = q_obj_choferes.filter(
                    and_(
                        choferes.idMovil == dic['IdMovil'],
                        #choferes.Telefono == movil['Celular2'],
                        choferes.Telefono.like(
                            '%' + movil['Celular2'][-8:] + '%'
                        ),
                    )
                )

                # Actualizo el segundo turno del chofer del móvil
                actualizar_chofer = {}
                actualizar_chofer['TurnoDesde'] = '1900-01-01 ' \
                                            + str(movil.get('Celular2Desde'))
                actualizar_chofer['TurnoHasta'] = '1900-01-01 ' \
                                            + str(movil.get('Celular2Hasta'))
                q_obj_choferes.update(
                    actualizar_chofer, synchronize_session=False
                )

                # Hago el commit con la base de datos
                session.commit()

                # Cierro la sesión con la base de datos
                session.close()
            ###################################################################

            # Agrego el móvil
            agregar = bd.agregar('moviles', **dic)
            if agregar is True:
                print('Se agrega el movil %s' % dic['IdMovil'])
            else:
                print('No se pudo agregar el movil %s' % dic['IdMovil'])
                # Si no se pudo agregar el registro guardo en un log el error
                log.msg(
                    'Funcion: agregar_moviles | Registro: %s' % agregar,
                    mode='migracion_gold',
                )

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se agregaron los móviles de forma correcta')
        return rta

    def agregar_tipos_abonos(self, nombre_bd, ruta_archivo):
        '''
            Está función se encarga de migrar los datos de la tabla
            tipos_abonos del cliente GOLD.

            Argumentos:
                nombre_bd (str)
                ruta_archivo (str)

            Funciones:
                obtener_bd
                MigrarBd().procesar_archivo_csv

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Llamo a la función procesar_archivo_csv
        rta_procesar_archivo_csv = self.migrar_bd.procesar_archivo_csv(
            ruta_archivo
        )
        # Si no se pudo procesar el archivo
        if rta_procesar_archivo_csv['Estado'] is False:
            # Retorno el error
            return rta_procesar_archivo_csv

        # Guardo la lista de tipos de abonos
        tipos_abonos = rta_procesar_archivo_csv['Respuesta']

        # Recorro los tipos de abonos
        for tipo_abono in tipos_abonos:
            # Guardo los datos de cada tipo de abono
            dic = {}
            dic['Codigo'] = tipo_abono['codigo']
            dic['Descripcion'] = tipo_abono['descripcion']
            dic['Importe1'] = float(tipo_abono['Importe1'].replace(',', '.'))
            dic['Importe2'] = float(tipo_abono['Importe2'].replace(',', '.'))
            dic['FondoProteccion'] = float(
                tipo_abono['ImporteDet1'].replace(',', '.')
            )
            dic['ViajesRealizados'] = float(
                tipo_abono['ImporteDet2'].replace(',', '.')
            )
            dic['AlquilerFrecuencia'] = float(
                tipo_abono['ImporteDet3'].replace(',', '.')
            )

            # Agrego el tipo de abono
            agregar = bd.agregar('tipos_abonos', **dic)
            if agregar is True:
                print('Se agrega el tipo de abono %s' % tipo_abono['codigo'])
            else:
                print('No se pudo agregar el tipo de abono %s' % tipo_abono['codigo'])
                # Si no se pudo agregar el registro guardo en un log el error
                log.msg(
                    'Funcion: agregar_tipos_abonos | Registro: %s' % agregar,
                    mode='migracion_gold',
                )

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se agregaron los tipos de abonos de forma correcta'
        )
        return rta

    def agregar_centros_de_costos_personas(self, nombre_bd, ruta_archivo):
        '''
            Está función se encarga de migrar los datos de la tabla
            centros_de_costos_personas del cliente GOLD.

            Argumentos:
                nombre_bd (str)
                ruta_archivo (str)

            Funciones:
                obtener_bd
                MigrarBd().procesar_archivo_csv

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Llamo a la función procesar_archivo_csv
        rta_procesar_archivo_csv = self.migrar_bd.procesar_archivo_csv(
            ruta_archivo
        )
        # Si no se pudo procesar el archivo
        if rta_procesar_archivo_csv['Estado'] is False:
            # Retorno el error
            return rta_procesar_archivo_csv

        # Guardo la lista de centros de costos personas
        centros_de_costos_personas = rta_procesar_archivo_csv['Respuesta']

        # Creo un id persona empezando del 1
        # esta dato no debe ser único, se tiene en cuenta el cliente y el
        # centro de costo, pero como no lo tengo para migrar lo hago único
        id_persona = 1

        # Recorro las personas de los centros de costos
        for persona in centros_de_costos_personas:
            # Guardo los datos de cada persona
            dic = {}
            dic['IdCliente'] = persona['CtaCte']
            dic['IdCentroDeCosto'] = persona['Centro de Costo']

            # Como no tengo el dato IdPersona le asigno un número único
            dic['IdPersona'] = id_persona
            # Le sumo 1 a id_persona
            id_persona += 1

            dic['Nombre'] = persona['Nombre autorizado']
            dic['DNI'] = persona['DNI']

            if persona['fecha de baja'] != '0':
                dic['Habilitado'] = 0
            else:
                dic['Habilitado'] = 1

            dic['PIN'] = persona['codigo/subcuenta']

            # Agrego la persona del centro de costo
            agregar = bd.agregar('centros_de_costos_personas', **dic)
            if agregar is True:
                print('Se agrega la persona %s' % persona['Nombre autorizado'])
            else:
                print('No se pudo agregar la persona %s' % persona['Nombre autorizado'])
                # Si no se pudo agregar el registro guardo en un log el error
                log.msg(
                    'Funcion: agregar_centros_de_costos_personas | Registro: %s' % agregar,
                    mode='migracion_gold',
                )

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se agregaron las personas de los centros de costos de forma correcta'
        )
        return rta

    def agregar_centros_de_costos(self, nombre_bd, ruta_archivo):
        '''
            Está función se encarga de migrar los datos de la tabla
            centros_de_costos del cliente GOLD.

            Argumentos:
                nombre_bd (str)
                ruta_archivo (str)

            Funciones:
                obtener_bd
                MigrarBd().procesar_archivo_csv

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Llamo a la función procesar_archivo_csv
        rta_procesar_archivo_csv = self.migrar_bd.procesar_archivo_csv(
            ruta_archivo
        )
        # Si no se pudo procesar el archivo
        if rta_procesar_archivo_csv['Estado'] is False:
            # Retorno el error
            return rta_procesar_archivo_csv

        # Guardo la lista de centros de costos
        centros_de_costos = rta_procesar_archivo_csv['Respuesta']

        # Recorro los centros de costos
        for centro_de_costo in centros_de_costos:
            # Guardo los datos de cada centro de costo
            dic = {}
            dic['IdCliente'] = centro_de_costo['CtaCte']
            dic['IdCentroDeCosto'] = centro_de_costo['Centro de Costo']
            dic['Nombre'] = centro_de_costo['Nombre del Ctro. Cto.']
            dic['Referencia'] = ''
            if centro_de_costo['Inhabilitar'] == '1':
                dic['Habilitado'] = 0
            else:
                dic['Habilitado'] = 1
            dic['Credito'] = centro_de_costo['Credito Limite']
            dic['CreditoAmpliado'] = centro_de_costo['Credito Limite Ampliado']
            dic['CreditoAcumulado'] = centro_de_costo['Credito Acumulado']
            dic['ResetCreditoCada'] = centro_de_costo['Reset Cada']

            # Cambio el formato de la fecha de reseteo del credito
            rta_pasar_a_date = self.fechas.pasar_a_date(
                centro_de_costo['Ultima Fecha Reset']
            )
            if rta_pasar_a_date['Estado'] is True:
                ult_fecha_reset_credito = rta_pasar_a_date['Mensaje']
            else:
                ult_fecha_reset_credito = None
            dic['UltFechaResetCredito'] = ult_fecha_reset_credito

            # Agrego el centro de costo
            agregar = bd.agregar('centros_de_costos', **dic)
            if agregar is True:
                print('Se agrega el centro de costo %s' % centro_de_costo['Nombre del Ctro. Cto.'])
            else:
                print('No se pudo agregar el centro de costo %s' % centro_de_costo['Nombre del Ctro. Cto.'])
                # Si no se pudo agregar el registro guardo en un log el error
                log.msg(
                    'Funcion: agregar_centros_de_costos | Registro: %s' % agregar,
                    mode='migracion_gold',
                )

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se agregaron los centros de costos de forma correcta'
        )
        return rta

    def agregar_clientes_ctacte(self, nombre_bd, ruta_archivo):
        '''
            Está función se encarga de migrar los datos de la tabla
            clientes_ctacte del cliente GOLD.

            Argumentos:
                nombre_bd (str)
                ruta_archivo (str)

            Funciones:
                obtener_bd
                MigrarBd().procesar_archivo_csv

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Llamo a la función procesar_archivo_csv
        rta_procesar_archivo_csv = self.migrar_bd.procesar_archivo_csv(
            ruta_archivo
        )
        # Si no se pudo procesar el archivo
        if rta_procesar_archivo_csv['Estado'] is False:
            # Retorno el error
            return rta_procesar_archivo_csv

        # Guardo la lista de clientes con cta. cte.
        clientes_cta_cte = rta_procesar_archivo_csv['Respuesta']

        # Recorro los clientes con cta. cte.
        for ctacte in clientes_cta_cte:
            # Guardo los datos de cada cliente con cta. cte.
            dic = {}
            dic['IdCliente'] = ctacte['numero']
            dic['RazonSocial'] = ctacte['Razon Social']
            dic['NombreContacto'] = ctacte['contacto']
            dic['Telefono'] = ctacte['Telefono']
            dic['Direccion'] = ctacte['direccion']
            dic['Localidad'] = ctacte['Localidad']
            dic['CUIT'] = ctacte['cuit']

            if ctacte['inhabilitar'] == '1':
                dic['Habilitado'] = 0
            else:
                dic['Habilitado'] = 1

            if ctacte['controla credito por'] == 'Cta. Cte.':
                dic['ControlarCredito'] = 1
            elif ctacte['controla credito por'] == 'Centro de Costo':
                dic['ControlarCredito'] = 2
            else:
                dic['ControlarCredito'] = 0

            dic['Credito'] = ctacte['credito limite']
            dic['CreditoAmpliado'] = ctacte['credito limite ampliado']
            dic['CreditoAcumulado'] = ctacte['credito acumulado']
            dic['ResetCreditoCada'] = ctacte['reset cada']

            # Cambio el formato de la fecha de reseteo del credito
            rta_pasar_a_date = self.fechas.pasar_a_date(
                ctacte['fecha ultimo reset']
            )
            if rta_pasar_a_date['Estado'] is True:
                ult_fecha_reset_credito = rta_pasar_a_date['Mensaje']
            else:
                ult_fecha_reset_credito = None
            dic['UltFechaResetCredito'] = ult_fecha_reset_credito

            # Agrego el cliente con cta. cte.
            agregar = bd.agregar('clientes_ctacte', **dic)
            if agregar is True:
                print('Se agrega el cliente con cta. cte. %s' % ctacte['numero'])
            else:
                print('No se pudo agregar el cliente con cta. cte. %s' % ctacte['numero'])
                # Si no se pudo agregar el registro guardo en un log el error
                log.msg(
                    'Funcion: agregar_clientes_ctacte | Registro: %s' % agregar,
                    mode='migracion_gold',
                )

            # Por cada cliente agrego su centro de costo por defecto
            # que es el centro de costo 0
            dic = {}
            dic['IdCliente'] = ctacte['numero']
            dic['IdCentroDeCosto'] = 0
            dic['Nombre'] = 'General'
            bd.agregar('centros_de_costos', **dic)

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se agregaron los clientes con cta. cte. de forma correcta'
        )
        return rta

    def migrar_gold(self, ruta_carpeta, nombre_bd):
        '''
            Esta función se encarga de migrar todas las bases de datos de Gold.
            Busca en la ruta recibida los archivos necesarios para migrar
            cada tabla.

            Argumentos:
                ruta_carpeta (str)
                nombre_bd (str)

            Funciones:
                MigrarBd().agregar_aseguradoras

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Creo el diccionario a retornar
        rta = {}

        # Reviso que este el archivo aseguradoras.csv
        ruta_archivo_aseguradoras = os.path.join(
            ruta_carpeta, 'aseguradoras.csv'
        )
        # Me fijo si la ruta del archivo es válida
        if os.path.isfile(ruta_archivo_aseguradoras) is False:
            # Si no encuentro el archivo retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró el archivo %s'
            ) % ruta_archivo_aseguradoras
            return rta

        # Reviso que este el archivo administradoras.csv
        ruta_archivo_administradoras = os.path.join(
            ruta_carpeta, 'administradoras.csv'
        )
        # Me fijo si la ruta del archivo es válida
        if os.path.isfile(ruta_archivo_administradoras) is False:
            # Si no encuentro el archivo retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró el archivo %s'
            ) % ruta_archivo_administradoras
            return rta

        # Reviso que este el archivo titulares.csv
        ruta_archivo_titulares = os.path.join(ruta_carpeta, 'titulares.csv')
        # Me fijo si la ruta del archivo es válida
        if os.path.isfile(ruta_archivo_titulares) is False:
            # Si no encuentro el archivo retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró el archivo %s'
            ) % ruta_archivo_titulares
            return rta

        # Reviso que este el archivo choferes.csv
        ruta_archivo_choferes = os.path.join(ruta_carpeta, 'choferes.csv')
        # Me fijo si la ruta del archivo es válida
        if os.path.isfile(ruta_archivo_choferes) is False:
            # Si no encuentro el archivo retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró el archivo %s'
            ) % ruta_archivo_choferes
            return rta

        # Reviso que este el archivo moviles.csv
        ruta_archivo_moviles = os.path.join(ruta_carpeta, 'moviles.csv')
        # Me fijo si la ruta del archivo es válida
        if os.path.isfile(ruta_archivo_moviles) is False:
            # Si no encuentro el archivo retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró el archivo %s'
            ) % ruta_archivo_moviles
            return rta

        # Reviso que este el archivo abonos.csv
        ruta_archivo_abonos = os.path.join(ruta_carpeta, 'abonos.csv')
        # Me fijo si la ruta del archivo es válida
        if os.path.isfile(ruta_archivo_abonos) is False:
            # Si no encuentro el archivo retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró el archivo %s'
            ) % ruta_archivo_abonos
            return rta

        # Reviso que este el archivo centros_de_costos_personas.csv
        ruta_archivo_centros_de_costos_personas = os.path.join(
            ruta_carpeta, 'centros_de_costos_personas.csv'
        )
        # Me fijo si la ruta del archivo es válida
        if os.path.isfile(ruta_archivo_centros_de_costos_personas) is False:
            # Si no encuentro el archivo retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró el archivo %s'
            ) % ruta_archivo_centros_de_costos_personas
            return rta

        # Reviso que este el archivo centros_de_costos.csv
        ruta_archivo_centros_de_costos = os.path.join(
            ruta_carpeta, 'centros_de_costos.csv'
        )
        # Me fijo si la ruta del archivo es válida
        if os.path.isfile(ruta_archivo_centros_de_costos) is False:
            # Si no encuentro el archivo retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró el archivo %s'
            ) % ruta_archivo_centros_de_costos
            return rta

        # Reviso que este el archivo clientes_ctacte.csv
        ruta_archivo_clientes_ctacte = os.path.join(
            ruta_carpeta, 'clientes_ctacte.csv'
        )
        # Me fijo si la ruta del archivo es válida
        if os.path.isfile(ruta_archivo_clientes_ctacte) is False:
            # Si no encuentro el archivo retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró el archivo %s'
            ) % ruta_archivo_clientes_ctacte
            return rta

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Antes de agregar los datos limpio todas las tablas que voy a migrar
        bd.borrar('filtros')
        bd.borrar('aseguradoras')
        bd.borrar('administradoras')
        bd.borrar('titulares')
        bd.borrar('choferes')
        bd.borrar('moviles')
        bd.borrar('equipos')
        bd.borrar('filtros_moviles')
        bd.borrar('tipos_abonos')
        bd.borrar('centros_de_costos_personas')
        bd.borrar('centros_de_costos')
        bd.borrar('clientes_ctacte')

        # Primero cargo los filtros
        self.agregar_filtros(nombre_bd)

        # Migro las aseguradoras
        self.agregar_aseguradoras(nombre_bd, ruta_archivo_aseguradoras)

        # Migro las administradoras
        self.agregar_administradoras(nombre_bd, ruta_archivo_administradoras)

        # Migro los titulares
        self.agregar_titulares(nombre_bd, ruta_archivo_titulares)

        # Migro los choferes
        self.agregar_choferes(nombre_bd, ruta_archivo_choferes)

        # Migro los móviles
        self.agregar_moviles(nombre_bd, ruta_archivo_moviles)

        # Migro los tipos de abonos
        self.agregar_tipos_abonos(nombre_bd, ruta_archivo_abonos)

        # Migro las personas de los centros de costos
        self.agregar_centros_de_costos_personas(
            nombre_bd, ruta_archivo_centros_de_costos_personas
        )

        # Migro los centros de costos
        self.agregar_centros_de_costos(
            nombre_bd, ruta_archivo_centros_de_costos
        )

        # Migro los clientes con cta. cte.
        self.agregar_clientes_ctacte(nombre_bd, ruta_archivo_clientes_ctacte)

        self.agregar_centros_inexistentes(nombre_bd)

        self.modificar_centros_de_costos_inexistentes(
            nombre_bd, ruta_archivo_centros_de_costos_personas
        )

        # Termino la migración de GOLD
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se terminó la migración de GOLD, revisar el LOG en /var/lib/at/logs/at.utiles/at.utiles.migracion_gold'
        )
        return rta

    def agregar_centros_inexistentes(self, nombre_bd):
        rta = {}

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        session = bd.binded_session()

        centros_de_costos_personas = bd.tabla(
            'centros_de_costos_personas'
        )

        q_obj = session.query(
            centros_de_costos_personas.IdCliente,
            centros_de_costos_personas.IdCentroDeCosto
        )

        registros = q_obj.all()

        for registro in registros:
            print(registro)

            dic = {}
            dic['IdCliente'] = registro.IdCliente
            dic['IdCentroDeCosto'] = registro.IdCentroDeCosto
            dic['Nombre'] = registro.IdCentroDeCosto
            agregar = bd.agregar('centros_de_costos', **dic)
            print('agregar')
            print(agregar)

        session.close()

        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se agregaron los centros de costos inexistentes')
        return rta

    def modificar_centros_de_costos_inexistentes(self, nombre_bd, ruta_archivo):
        '''
            Está función se encarga de modificar los nombres de los centros
            de costos inexistentes.

            Argumentos:
                nombre_bd (str)
                ruta_archivo (str)

            Funciones:
                obtener_bd
                MigrarBd().procesar_archivo_csv

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la base de datos dada
        rta_obtener_bd = obtener_bd(nombre_bd)

        # Si no se pudo obtener la base de datos dada
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Guardo la base de datos obtenida
        bd = rta_obtener_bd['Respuesta']

        # Llamo a la función procesar_archivo_csv
        rta_procesar_archivo_csv = self.migrar_bd.procesar_archivo_csv(
            ruta_archivo
        )
        # Si no se pudo procesar el archivo
        if rta_procesar_archivo_csv['Estado'] is False:
            # Retorno el error
            return rta_procesar_archivo_csv

        session = bd.binded_session()

        # Guardo la lista de centros de costos personas
        centros_de_costos_personas = rta_procesar_archivo_csv['Respuesta']

        # Recorro las personas de los centros de costos
        for persona in centros_de_costos_personas:
            if persona['Nombre ctro cto'] == '':
                continue

            # Modifico el nombre del centro de costo
            dic = {}
            dic['Nombre'] = persona['Nombre ctro cto']

            centros_de_costos = bd.tabla('centros_de_costos')

            q_obj = session.query(centros_de_costos)

            q_obj = q_obj.filter(
                and_(
                    centros_de_costos.IdCliente == persona['CtaCte'],
                    centros_de_costos.IdCentroDeCosto == persona['Centro de Costo'],
                )
            )

            q_obj.update(
                dic, synchronize_session=False
            )

        session.commit()

        session.close()

        # Si todo salió bien retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se modificaron los nombres de los centros de costos'
        )
        return rta
