# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import or_

from at.utiles import _
from at.orm.decorators import DBSwitch


@DBSwitch('gis2_sanfrancisco_09_03_2016', 'gis2_venadotuerto_09_03_2016',
          'gis2_airtrack_09_03_2016', 'cp', 'gis2')
class MigrarCalles(object):

    def select(self):
        '''
            SELECT *

            Retorna:
                str
        '''

        # Obtengo los nombres de las calles de San Francisco
        session = self.gis2.binded_session()

        calles_nombres = self.gis2.tabla('calles_nombres')

        q_obj = session.query(calles_nombres)

        q_obj = q_obj.filter(calles_nombres.id_ciudad == 86)

        registros_s = q_obj.all()

        session.close()
        lista_s = []
        for registro_s in registros_s:
            print(registro_s.id, registro_s.id_ciudad, registro_s.nombre)

    def prueba_base(self):
        for i in range(10000):
            print(i)
            dic = {}
            dic['id_ciudad'] = 1234
            dic['nombre'] = 'prueba ' + str(i)
            agregar = self.gis2Prueba.agregar(
                'calles_nombres', **dic
            )
            print(agregar)
            if agregar is not True:
                return

        return 'Termina prueba base'

    def actualizar_calles_modificadas(self, base_de_datos_vieja, base_de_datos_nueva):
        # Me conecto a la base de datos vieja.
        session = base_de_datos_vieja.binded_session()

        # Indico que tabla voy a usar.
        calles_nombres = base_de_datos_vieja.tabla('calles_nombres')

        # Hago una select de las calles filtrado por ciudades.
        q_obj = session.query(calles_nombres)
        q_obj = q_obj.filter(
            or_(
                # calles_nombres.id_ciudad == 86,
                # calles_nombres.id_ciudad == 1131,
                calles_nombres.id_ciudad == 4145,
                calles_nombres.id_ciudad == 4423,
                calles_nombres.id_ciudad == 1214,
            )
        )

        # Guardo los registros obtenidos.
        registros = q_obj.all()

        # Cierro la sesión
        session.close()

        # Me conecto a la base de datos nueva.
        session = base_de_datos_nueva.binded_session()
        # Indico que tabla voy a usar.
        calles_nombres = base_de_datos_nueva.tabla('calles_nombres')

        # Recorro los registros obtenidos en la select anterior.
        for registro in registros:

            # Solo recorro los registros mayores a 15000
            if registro.id < 15000:
                continue

            # Hago una select en la base de datos nueva filtrando por los
            # registros obtenidos anteriormente.
            q_obj = session.query(calles_nombres)
            q_obj = q_obj.filter(calles_nombres.id == registro.id)

            # Guardo la  consulta.
            calle = q_obj.first()

            # Si no encuentra la calle, fue porque la borraron.
            if calle is None:
                print('*** CALLE NO EXISTENTE')
                print(registro.id, registro.nombre, registro.id_ciudad)
                print('-----------------------------------------------------')
                continue

            # Si no tienen el mismo nombre o ciudad hay que actualizarlo.
            if registro.nombre != calle.nombre \
                    or registro.id_ciudad != calle.id_ciudad:
                print('datos de la base de datos vieja')
                print(registro.id, registro.nombre, registro.id_ciudad)
                print('datos de la base de datos nueva')
                print(calle.id, calle.nombre, calle.id_ciudad)
                print('--------------------------------------------------------')

                # Realizo el update.
                dic = {}
                dic['nombre'] = registro.nombre
                dic['id_ciudad'] = registro.id_ciudad

                q_obj.update(dic, synchronize_session=False)

        # Realizo los cambios en la base de datos nueva.
        session.commit()

        # Cierro la sesión.
        session.close()
        # Termina la función.
        return "Terminó la actualización"



    def borrar_calles_viejas(self, base_de_datos_vieja, base_de_datos_nueva):
        session = base_de_datos_nueva.binded_session()
        calles_nombres_nueva = base_de_datos_nueva.tabla('calles_nombres')

        q_obj = session.query(calles_nombres_nueva)
        q_obj = q_obj.filter(
            or_(
                # calles_nombres_nueva.id_ciudad == 86,
                # calles_nombres_nueva.id_ciudad == 1131,
                calles_nombres_nueva.id_ciudad == 4145,
                calles_nombres_nueva.id_ciudad == 4423,
                calles_nombres_nueva.id_ciudad == 1214,
            )
        )

        registros_nueva = q_obj.all()

        # session.close()

        session_2 = base_de_datos_vieja.binded_session()
        calles_nombres_vieja = base_de_datos_vieja.tabla('calles_nombres')

        for registro_nueva in registros_nueva:
            if registro_nueva.id < 15000:
                continue

            q_obj = session_2.query(calles_nombres_vieja)
            q_obj = q_obj.filter(calles_nombres_vieja.id == registro_nueva.id)

            registros_vieja = q_obj.first()

            if registros_vieja is None:
                q_obj_2 = session.query(calles_nombres_nueva)
                q_obj_2 = q_obj_2.filter(calles_nombres_nueva.id == registro_nueva.id)
                q_obj_2.delete(synchronize_session=False)

                print('*** CALLE NO EXISTENTE')
                print(registro_nueva.id, registro_nueva.nombre, registro_nueva.id_ciudad)
                print('-----------------------------------------------------')
                continue
        session.commit()
        session.close()
        session_2.close()
        return "Terminó la actualización"

    def insertar_calles_nuevas(self, base_de_datos_vieja, base_de_datos_nueva):
        session = base_de_datos_vieja.binded_session()
        calles_nombres = base_de_datos_vieja.tabla('calles_nombres')

        q_obj = session.query(calles_nombres)
        q_obj = q_obj.filter(
            or_(
                # calles_nombres.id_ciudad == 86,
                # calles_nombres.id_ciudad == 1131,
                calles_nombres.id_ciudad == 4145,
                calles_nombres.id_ciudad == 4423,
                calles_nombres.id_ciudad == 1214,
            )
        )

        registros = q_obj.all()

        session.close()

        session = base_de_datos_nueva.binded_session()


        for registro in registros:
            if registro.id > 15000 or registro.id <= 1778:
                continue
            dic = {}
            dic['id'] = registro.id + 5220
            dic['id_ciudad'] = registro.id_ciudad
            dic['nombre'] = registro.nombre
            agregar = base_de_datos_nueva.agregar(
                'calles_nombres', **dic
            )
            print(registro.id + 5220, registro.id_ciudad, registro.nombre, agregar)
            if agregar is not True:
                return "no termino la actuali"


        session.commit()
        session.close()
        return "Terminó la actualización"

    def actualizar_segmentos(self, base_de_datos_vieja, base_de_datos_nueva):

        session = base_de_datos_vieja.binded_session()
        calles_nombres = base_de_datos_vieja.tabla('calles_nombres')

        q_obj = session.query(calles_nombres)
        q_obj = q_obj.filter(
            or_(
                # calles_nombres.id_ciudad == 86,
                # calles_nombres.id_ciudad == 1131,
                calles_nombres.id_ciudad == 4145,
                calles_nombres.id_ciudad == 4423,
                calles_nombres.id_ciudad == 1214,
            )
        )

        registros = q_obj.all()

        session_2 = base_de_datos_nueva.binded_session()
        segmentos = base_de_datos_nueva.tabla('calles')

        for registro in registros:

            if registro.id <= 1778 or registro.id > 15000:
                continue
            calles = base_de_datos_vieja.tabla('calles')
            q_obj = session.query(calles)
            q_obj = q_obj.filter(
                or_(
                    calles.id_calle_nombre == registro.id
                )
            )
            segmentos_a_modificar = q_obj.all()

            if len(segmentos_a_modificar) == 0:
                print(registro.id, "no tiene segmento")
                continue
            for segmento_a_modificar in segmentos_a_modificar:



                # print(calle.gid, "======", registro.id)

                q_obj = session_2.query(segmentos)
                q_obj = q_obj.filter(segmentos.gid == segmento_a_modificar.gid)

                calle_2 = q_obj.first()

                if calle_2 is None:
                    print("No existe el segmento")
                    continue


                dic = {}
                dic['id_calle_nombre'] = registro.id + 5220

                q_obj.update(dic, synchronize_session=False)

                print("Registro", registro.id, "actualizado a:", registro.id + 5220)

        session.close()
        session_2.commit()

        session_2.close()

        return "Terminó la actualización"

    def borrar_segmentos_viejos(self, base_de_datos_vieja, base_de_datos_nueva):
        session = base_de_datos_nueva.binded_session()
        calles_nombres_nueva = base_de_datos_nueva.tabla('calles_nombres')

        q_obj = session.query(calles_nombres_nueva)
        q_obj = q_obj.filter(
            or_(
                # calles_nombres_nueva.id_ciudad == 86,
                # calles_nombres_nueva.id_ciudad == 1131,
                calles_nombres_nueva.id_ciudad == 4145,
                calles_nombres_nueva.id_ciudad == 4423,
                calles_nombres_nueva.id_ciudad == 1214,
            )
        )

        registros_nueva = q_obj.all()

        # session.close()


        segmentos_nueva = base_de_datos_nueva.tabla('calles')

        session_2 = base_de_datos_vieja.binded_session()

        for registro_nueva in registros_nueva:
            if registro_nueva.id < 15000:
                continue

            q_obj = session.query(segmentos_nueva)
            q_obj = q_obj.filter(segmentos_nueva.id_calle_nombre == registro_nueva.id)

            segmentos_a_chequear = q_obj.all()

            if len(segmentos_a_chequear) == 0:
                print(registro_nueva.id, registro_nueva.nombre, "no tiene segmentos")
                continue

            for segmento_a_chequear in segmentos_a_chequear:

                segmentos_vieja = base_de_datos_vieja.tabla('calles')
                q_obj = session_2.query(segmentos_vieja)
                q_obj = q_obj.filter(segmentos_vieja.gid == segmento_a_chequear.gid)

                segmentos_iguales = q_obj.first()


                if segmentos_iguales is None:
                    q_obj_2 = session.query(segmentos_nueva)
                    q_obj_2 = q_obj_2.filter(segmentos_nueva.gid == segmento_a_chequear.gid)
                    q_obj_2.delete(synchronize_session=False)

                    print('*** SEGMENTO NO EXISTENTE')
                    print(segmento_a_chequear.gid)
                    print('-----------------------------------------------------')
                    continue
        session.commit()
        session.close()
        session_2.close()
        return "Terminó la actualización"

    def actualizar_registros_clientes(self, cliente, tabla):

        if cliente == 'losmasrapidos':
            ciudades = [86, 1131]
            sumatoria = 3220
        elif cliente == 'remisesperanza':
            ciudades = [4145, 4423, 1214]
            sumatoria = 5220
        else:
            return "Debe ingresar una ciudad y una tabla a modificar valid"


        # Abrimos sesión con gis2 original
        session = self.gis2.binded_session()

        # Hago una select filtrando por ciudad
        calles_nombres = self.gis2.tabla('calles_nombres')

        q_obj = session.query(calles_nombres)
        q_obj = q_obj.filter(calles_nombres.id_ciudad.in_(ciudades))


        #     or_(
        #         calles_nombres.id_ciudad == 1131,
        #         calles_nombres.id_ciudad == 86,
        #         # calles_nombres.id_ciudad == 1131,
        #         # calles_nombres.id_ciudad == 86,
        #         # calles_nombres.id_ciudad == 1131,
        #     )
        # )

        calles = q_obj.all()

        # Cierro la sesión
        session.close()

        # Abro la sesión con cp2000
        session_2 = self.cp.binded_session()
        tabla_cp2000 = self.cp.tabla(tabla)

        # Recorro los registros obtenidos en la anterior select
        for calle in calles:

            # Trabajo solo con los registros nuevos en gis2
            if calle.id <= 1778 or calle.id > 15000:
                continue

            # Hago una select en cp2000 filtrando por los id que obtuve de gis2
            q_obj = session_2.query(tabla_cp2000)
            q_obj = q_obj.filter(
                tabla_cp2000.IdCalle1 == calle.id
            )
            registros_a_modificar = q_obj.all()

            # Si no obtengo datos sigo con el siguiente id
            if len(registros_a_modificar) == 0:
                print(" no se encontraron registros con IdCalle1:", calle.id)
            else:
                # Si encuentra un registro con el id determinado actualizo su valor
                dic = {}
                dic['IdCalle1'] = calle.id + sumatoria

                q_obj.update(dic, synchronize_session=False)

                print("Registro IdCalle1", calle.id, "actualizado a:", dic['IdCalle1'])

            # Hago una select en cp2000 filtrando por los id que obtuve de gis2
            q_obj = session_2.query(tabla_cp2000)
            q_obj = q_obj.filter(
                tabla_cp2000.IdCalle2 == calle.id
            )
            registros_a_modificar = q_obj.all()

            # Si no obtengo datos sigo con el siguiente id
            if len(registros_a_modificar) == 0:
                print(" no se encontraron registros con IdCalle2:", calle.id)
                continue

            # Si encuentra un registro con el id determinado actualizo su valor
            dic = {}
            dic['IdCalle2'] = calle.id + sumatoria

            q_obj.update(dic, synchronize_session=False)

            print("Registro IdCalle2", calle.id, "actualizado a:", calle.id + sumatoria)

        session_2.commit()
        session_2.close()

    def actualizar_alturas(self, base_de_datos_vieja, base_de_datos_nueva):

        # Abrimos sesión con gis2 original
        session = base_de_datos_vieja.binded_session()
        # Obtengo la tabla calles
        calles = base_de_datos_vieja.tabla('calles')
        # Obtengo la tabla calles_nombres
        calles_nombres = base_de_datos_vieja.tabla('calles_nombres')

        # Armo una subquery para obtener todas las calles nombres de la ciudad
        q_obj1 = session.query(calles_nombres.id)
        q_obj1 = q_obj1.filter(
            or_(
                # calles_nombres.id_ciudad == 86,
                # calles_nombres.id_ciudad == 1131,
                calles_nombres.id_ciudad == 4145,
                calles_nombres.id_ciudad == 4423,
                calles_nombres.id_ciudad == 1214,

            )
        )

        q_obj1 = q_obj1.subquery()

        # Armo una select para obtener todos los segmentos de la ciudad dada
        q_obj2 = session.query(calles)
        q_obj2 = q_obj2.filter(
            calles.id_calle_nombre.in_(q_obj1)
        )
        segmentos_viejos = q_obj2.all()

        session.close()

        session_2 = base_de_datos_nueva.binded_session()

        # Obtengo la tabla calles
        calles = base_de_datos_vieja.tabla('calles')

        for segmento_viejo in  segmentos_viejos:

            q_obj = session_2.query(calles)
            q_obj = q_obj.filter(
                calles.gid == segmento_viejo.gid
            )
            segmento_a_modificar = q_obj.first()

            if segmento_a_modificar is None:
                print("el segmento:", segmento_a_modificar, "no existe")
                continue
            if segmento_viejo.desde_izq is None:
                segmento_viejo.desde_izq = 0

            if segmento_viejo.hasta_izq is None:
                segmento_viejo.hasta_izq = 0

            if segmento_viejo.desde_der is None:
                segmento_viejo.desde_der = 0

            if segmento_viejo.hasta_der is None:
                segmento_viejo.hasta_der = 0

            dic = {}
            dic['desde_izq'] = segmento_viejo.desde_izq
            dic['hasta_izq'] = segmento_viejo.hasta_izq
            dic['desde_der'] = segmento_viejo.desde_der
            dic['hasta_der'] = segmento_viejo.hasta_der
            if segmento_viejo.id_calle_nombre > 15000:
                dic['id_calle_nombre'] = segmento_viejo.id_calle_nombre

            q_obj.update(dic, synchronize_session=False)

            print("se actualizo el segmento", segmento_a_modificar.gid, "######", segmento_viejo.desde_izq, segmento_viejo.hasta_izq, segmento_viejo.desde_der, segmento_viejo.hasta_der)

        session_2.commit()
        session_2.close()


    def agregar_nuevos_segmentos(self, base_de_datos_vieja, base_de_datos_nueva):

        # Abrimos sesión con gis2 original
        session = base_de_datos_vieja.binded_session()
        # Obtengo la tabla calles
        calles = base_de_datos_vieja.tabla('calles')
        # Obtengo la tabla calles_nombres
        calles_nombres = base_de_datos_vieja.tabla('calles_nombres')

        # Armo una subquery para obtener todas las calles nombres de la ciudad
        q_obj1 = session.query(calles_nombres.id)
        q_obj1 = q_obj1.filter(
            or_(
                # calles_nombres.id_ciudad == 86,
                # calles_nombres.id_ciudad == 1131,
                calles_nombres.id_ciudad == 4145,
                calles_nombres.id_ciudad == 4423,
                calles_nombres.id_ciudad == 1214,
            )
        )

        q_obj1 = q_obj1.subquery()

        # Armo una select para obtener todos los segmentos de la ciudad dada
        q_obj2 = session.query(calles)
        q_obj2 = q_obj2.filter(
            calles.id_calle_nombre.in_(q_obj1)
        )
        segmentos_viejos = q_obj2.all()

        session.close()

        session_2 = base_de_datos_nueva.binded_session()

        # Obtengo la tabla calles
        calles = base_de_datos_vieja.tabla('calles')

        for segmento_viejo in segmentos_viejos:

            q_obj = session_2.query(calles)
            q_obj = q_obj.filter(
                calles.geom == segmento_viejo.geom
            )
            segmento_a_modificar = q_obj.first()

            if segmento_a_modificar is None:
                if segmento_viejo.id_calle_nombre < 15000:
                    print(segmento_viejo.gid, "es un segmento de una calle nueva", segmento_viejo.id_calle_nombre)
                    dic = {}
                    dic['id_calle_nombre'] = segmento_viejo.id_calle_nombre + 5220
                    dic['id_tipo'] = segmento_viejo.id_tipo
                    dic['sentido'] = segmento_viejo.sentido
                    dic['geom'] = segmento_viejo.geom
                    dic['desde_izq'] = segmento_viejo.desde_izq
                    dic['hasta_izq'] = segmento_viejo.hasta_izq
                    dic['desde_der'] = segmento_viejo.desde_der
                    dic['hasta_der'] = segmento_viejo.hasta_der
                    dic['letra'] = segmento_viejo.letra
                    agregar = base_de_datos_nueva.agregar(
                        'calles', **dic
                    )
                    print(agregar)
                    if agregar is not True:
                        print(agregar)

                else:
                    print(segmento_viejo.gid, "es un segmento de una calle vieja", segmento_viejo.id_calle_nombre)
                    dic = {}
                    dic['id_calle_nombre'] = segmento_viejo.id_calle_nombre
                    dic['id_tipo'] = segmento_viejo.id_tipo
                    dic['sentido'] = segmento_viejo.sentido
                    dic['geom'] = segmento_viejo.geom
                    dic['desde_izq'] = segmento_viejo.desde_izq
                    dic['hasta_izq'] = segmento_viejo.hasta_izq
                    dic['desde_der'] = segmento_viejo.desde_der
                    dic['hasta_der'] = segmento_viejo.hasta_der
                    dic['letra'] = segmento_viejo.letra
                    agregar = base_de_datos_nueva.agregar(
                        'calles', **dic
                    )
                    print(agregar)
                    if agregar is not True:
                        print("no se pudo agregar")
                continue

        session_2.commit()
        session_2.close()
        return "Actualizacion completa"


