# -*- coding: utf-8 -*-

import time

from at.utiles import _
from at.orm.abm import ABM
from at.utiles.servidores import DateTimeHelper


datos_bd_cp = {
    'user': 'root',
    'passwd': 'changeme',
    'dbname': 'cp2000',
    'name': 'cp',
    'host': '192.168.0.51',
    'engine': 'mysql',
    'port': '3306',
}

bd_cp = ABM(datos_bd_cp)
dth = DateTimeHelper()


def simular_movil(id_movil, estado1, posiciones, segundos, infinito=False):
    '''
        Esta función se encarga de simular el recorrido de un móvil.

        Argumentos:
            id_movil (int)
            estado1 (int)
            posiciones (list de list de float)
            segundos (int)
            infinito (bool)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Declaro el diccionario a retornar
    rta = {}

    # Guardo las posiciones recibidas para que queden como las originales
    pos_originales = posiciones

    # Abro la sesión de la base de datos
    session = bd_cp.binded_session()

    # Obtengo la tabla moviles
    moviles = bd_cp.tabla('moviles')

    # Busco el móvil que quiero actualizar
    q_obj = session.query(moviles.Patente)

    # Filtro por el id del móvil dado
    q_obj = q_obj.filter(moviles.IdMovil == id_movil)

    # Si no se encontró el móvil dado
    if q_obj.count() == 0:
        # Cierro la sesión con la base de datos
        session.close()
        # Retorno False y un mensaje
        rta['Estado'] = False
        rta['Respuesta'] = _(u'No se encontró el móvil con id ' + id_movil)
        return rta

    # Actualizo los datos del móvil dado
    dic = {}
    dic['Estado1'] = estado1
    dic['EstadoGPS'] = 3
    dic['EstaEnServicio'] = 1

    # Recorro las posiciones recibidas
    for posicion in posiciones:
        # Paso la latitud y la longitud a segundos
        latitud = posicion[0] * 3600
        longitud = posicion[1] * 3600

        # Obtengo la fecha y hora actual
        fecha_hora_actual = dth.get_now()

        # Guardo los datos de la posición del móvil
        dic['Latitud'] = latitud
        dic['Longitud'] = longitud
        dic['UltPosicion'] = fecha_hora_actual
        dic['Time_UltimoComando'] = fecha_hora_actual

        # Actualizo el registro del móvil
        q_obj.update(dic, synchronize_session=False)
        session.commit()

        # Duermo los segundos dados
        time.sleep(segundos)

    # Cierro la sesión con la base de datos
    session.close()

    # Si la variable infinito es True
    if infinito is True:
        # Llamo nuevamente a la función para que el móvil se siga moviendo
        simular_movil(id_movil, estado1, pos_originales, segundos, infinito)

    # Retorno True y un mensaje indicando que se simuló un recorrido del móvil
    rta['Estado'] = True
    rta['Respuesta'] = _(u'Se simuló el recorrido del móvil %s' % id_movil)
    return rta
