# -*- coding: utf-8 -*-

import unittest

from at.utiles import _
from at.utiles.scripts.anticipados_nombres import AnticipadosNombres
from at.orm.abm import ABM


cp = {'user': 'frula',
      'passwd': 'changeme',
      'dbname': 'tests',
      'name': 'cp',
      'host': '192.168.0.51',
      'engine': 'mysql',
      'port': '3306',
      }


class AnticipadosNombresTesting(AnticipadosNombres):

    def __init__(self):
        AnticipadosNombres.__init__(self)
        self.cp = ABM(cp)


class TestAnticipadosNombres(unittest.TestCase):

    def setUp(self):
        self.ant = AnticipadosNombresTesting()

        self.ant.cp.borrar('clientes')
        self.ant.cp.borrar('serviciosanticipados')
        self.ant.cp.borrar('serviciosanticipadosdetalle')

    def tearDown(self):
        self.ant.cp.borrar('clientes')
        self.ant.cp.borrar('serviciosanticipados')
        self.ant.cp.borrar('serviciosanticipadosdetalle')

    def test_grabar_nombre_en_anticipados(self):
        # No hay ningún anticipado cargado
        rta = self.ant.grabar_nombre_en_anticipados()
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No hay anticipados cargados'))

        # Agrego un registro en la tabla serviciosanticipados
        dic = {}
        dic['Telefono'] = '153705967'
        dic['Nombre'] = 'Paco'
        agregar = self.ant.cp.agregar('serviciosanticipados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipados
        dic = {}
        dic['Telefono'] = '4636363'
        agregar = self.ant.cp.agregar('serviciosanticipados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipados
        dic = {}
        dic['Telefono'] = u'Canción'
        agregar = self.ant.cp.agregar('serviciosanticipados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipados
        dic = {}
        dic['Telefono'] = u'anticipado25'
        agregar = self.ant.cp.agregar('serviciosanticipados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla clientes
        dic = {}
        dic['Telefono'] = '4636363'
        dic['Nombre'] = 'Nombre del 4636363'
        dic['Direccion'] = u'la dirección'
        agregar = self.ant.cp.agregar('clientes', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla clientes
        dic = {}
        dic['Telefono'] = u'Canción'
        dic['Direccion'] = u'la dirección'
        agregar = self.ant.cp.agregar('clientes', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla clientes
        dic = {}
        dic['Telefono'] = 'anticipado25'
        dic['Nombre'] = 'Nombre de anticipado25'
        dic['Direccion'] = u'la dirección'
        agregar = self.ant.cp.agregar('clientes', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = '4636363'
        dic['Dia'] = 1
        dic['Hora'] = '09:00:00'
        dic['Direccion'] = u'la dirección'
        dic['IdZona'] = 0
        dic['SolicitadoPara'] = 'ya tiene nombre'
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = '4636363'
        dic['Dia'] = 2
        dic['Hora'] = '09:00:00'
        dic['Direccion'] = u'la dirección'
        dic['IdZona'] = 0
        dic['SolicitadoPara'] = ''
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = '4636363'
        dic['Dia'] = 3
        dic['Hora'] = '09:00:00'
        dic['Direccion'] = u'la dirección'
        dic['IdZona'] = 0
        dic['SolicitadoPara'] = None
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = '4636363'
        dic['Dia'] = 4
        dic['Hora'] = '09:00:00'
        dic['Direccion'] = u'la dirección'
        dic['IdZona'] = 0
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = u'Canción'
        dic['Dia'] = 1
        dic['Hora'] = '18:00:00'
        dic['Direccion'] = u'la dirección'
        dic['IdZona'] = 0
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = u'anticipado25'
        dic['Dia'] = 4
        dic['Hora'] = '22:00:00'
        dic['Direccion'] = u'la dirección'
        dic['IdZona'] = 0
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = u'anticipado25'
        dic['Dia'] = 5
        dic['Hora'] = '22:00:00'
        dic['Direccion'] = u'la dirección'
        dic['IdZona'] = 0
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = u'anticipado25'
        dic['Dia'] = 6
        dic['Hora'] = '22:00:00'
        dic['Direccion'] = u'la dirección'
        dic['IdZona'] = 0
        dic['SolicitadoPara'] = 'Oscar'
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        rta = self.ant.grabar_nombre_en_anticipados()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se grabó el nombre de los clientes en todos sus anticipados')
        )

    def test_grabar_direccion_en_anticipados(self):
        # No hay ningún anticiapdo
        rta = self.ant.grabar_direccion_en_anticipados()
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No hay anticipados cargados'))

        # Agrego un registro en la tabla serviciosanticipados
        dic = {}
        dic['Telefono'] = u'tel01'
        agregar = self.ant.cp.agregar('serviciosanticipados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipados
        dic = {}
        dic['Telefono'] = u'tel02'
        agregar = self.ant.cp.agregar('serviciosanticipados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipados
        dic = {}
        dic['Telefono'] = u'tel03'
        agregar = self.ant.cp.agregar('serviciosanticipados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = u'tel01'
        dic['Dia'] = 1
        dic['Hora'] = '22:00:00'
        dic['Direccion'] = u'la dirección del tel01'
        dic['IdZona'] = 0
        dic['SolicitadoPara'] = 'Oscar'
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = u'tel01'
        dic['Dia'] = 2
        dic['Hora'] = '22:00:00'
        dic['Direccion'] = u'la dirección del tel01'
        dic['IdZona'] = 0
        dic['SolicitadoPara'] = 'Oscar'
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = u'tel01'
        dic['Dia'] = 3
        dic['Hora'] = '22:00:00'
        dic['Direccion'] = u'la dirección del tel01 otra'
        dic['IdZona'] = 0
        dic['SolicitadoPara'] = 'Oscar'
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla serviciosanticipadosdetalle
        dic = {}
        dic['Telefono'] = u'tel02'
        dic['Dia'] = 5
        dic['Hora'] = '18:00:00'
        dic['Direccion'] = u'la dirección del tel02'
        dic['IdZona'] = 0
        dic['SolicitadoPara'] = 'Oscar'
        agregar = self.ant.cp.agregar('serviciosanticipadosdetalle', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        rta = self.ant.grabar_direccion_en_anticipados()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se grabó la dirección en todos los anticipados')
        )


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestAnticipadosNombres))
    return suite
