# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import func

from at.utiles import _
from at.utiles.servidores import DateTimeHelper
from at.orm.decorators import DBSwitch
from at.orm.utiles import obtener_bd


@DBSwitch('cp')
class Servicios(object):
    '''
        Esta clase va a contar con funciones útiles para tratar los servicios.
    '''

    def __init__(self):
        self.dth = DateTimeHelper()

    def id_servicio_autoincrement(self, ult_id_servicio=0, base_de_datos=''):
        '''
            Esta función se encarga de ingresar el valor en el campo IdServicio
            en todos los registros de la tabla actual que no lo tengan.

            Argumentos:
                ult_id_servicio (int)
                base_de_datos (str)

            Funciones:
                at.orm -> obtener_bd

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Si recibo una base de datos
        if base_de_datos != '':
            # FIX IMPORTANTE ##################################################
            #  si base_de_datos es "cp2000" se debe tomar como "cp"
            if base_de_datos == 'cp2000':
                base_de_datos = 'cp'
            ###################################################################
            # Llamo a obtener_bd
            rta_obtener_bd = obtener_bd(base_de_datos)
            # Si no se pudo obtener la conexión con la base de datos
            if rta_obtener_bd['Estado'] is False:
                # Retorno False y un mensaje
                return rta_obtener_bd
            # Piso self.cp con la conexión según el nombre la base de datos dada
            self.cp = rta_obtener_bd['Respuesta']

        # Creo la respuesta a retornar
        rta = {}

        # Abro la sesión de la base de datos
        session = self.cp.binded_session()

        # Obtengo la tabla actual
        actual = self.cp.tabla('actual')

        # Busco todos los registros de la tabla actual
        q_obj = session.query(
            actual.Telefono, actual.FechaServicio, actual.HoraServicio
        )

        # Filtro por los servicios que tiene 0 en IdServicio
        q_obj = q_obj.filter(actual.IdServicio == 0)

        # Ordeno los servicio por fecha
        q_obj = q_obj.order_by(
            actual.FechaServicio.asc(), actual.HoraServicio.asc()
        )

        # Guardo los registros obtenidos
        registros = q_obj.all()

        # SI NO TENGO REGISTROS PARA ACTUALIZAR
        # DE TODOS MODOS ACTUALIZO EL CAMPO IdServicio
        ## Si no hay ningún registro retorno el error
        #if len(registros) == 0:
            ## Cierro la sesión con la base de datos
            #session.close()

            #rta['Estado'] = False
            #rta['Respuesta'] = \
                #_(u'No se encontraron registros para guardar su id')
            #return rta

        # Seteo el id_servicio como el último id servicio que había
        id_servicio = ult_id_servicio

        # Recorro los registros obtenidos
        for registro in registros:
            # Le sumo 1 al id_servicio
            id_servicio += 1

            # Busco el servicio dado
            q_obj = session.query(
                actual.Telefono, actual.FechaServicio, actual.HoraServicio
            )
            q_obj = q_obj.filter(
                and_(
                    actual.Telefono == registro.Telefono,
                    actual.FechaServicio == registro.FechaServicio,
                    actual.HoraServicio == registro.HoraServicio,
                )
            )

            # Se actualiza el campo IdServicio
            dic = {}
            dic['IdServicio'] = id_servicio
            q_obj.update(dic, synchronize_session=False)

        # Saco IdServicio como UNICO para crearlo de nuevo
        try:
            session.execute('ALTER TABLE actual MODIFY IdServicio INT')
            session.execute('ALTER TABLE actual DROP INDEX IdServicio')
        except Exception as ex:
            print(ex)

        # Hago que IdServicio sea UNIQUE y AUTO INCREMENT
        session.execute(
            'ALTER TABLE actual MODIFY IdServicio INT AUTO_INCREMENT UNIQUE'
        )

        # Hago el commit con la base de datos
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Si todo salió bien se retorna un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Se actualizo correctamente el id de los servicios')
        return rta

    def obtener_servicios_por_telefono(self, telefono):
        '''
            Esta función se encarga de devolver una lista de servicios que
            contengan un número de teléfono similar.

            Argumentos:
                telefono (str o unicode)

            Funciones:
                DateTimeHelper().get_now

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de dict)
                                                ['id_servicio'] (int)
                                                ['fecha_hora_servicio'] (str)
                                                ['telefono'] (str)
                                                ['direccion'] (str)
                                                ['nombre'] (str)
                                                ['telefonista'] (str)
                                                ['latitud'] (float)
                                                ['longitud'] (float)
        '''

        # Creo la respuesta a retornar
        rta = {}

        # El teléfono debe ser un string o un unicode
        try:
            if not isinstance(telefono, str) and not isinstance(telefono, unicode):
                rta['Estado'] = False
                rta['Respuesta'] = \
                    _(u'El teléfono debe ser de tipo string o unicode')
                return rta
        except:
            # NOTA: En Python 3 no existe "unicode", "str" ya es unicode
            if not isinstance(telefono, str):
                rta['Estado'] = False
                rta['Respuesta'] = \
                    _(u'El teléfono debe ser de tipo string o unicode')
                return rta

        # Valido que el teléfono sea un string con un mínimo de 6 caracteres
        if len(telefono) < 6:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'El teléfono debe tener como mínimo 6 caracteres para buscarlo')
            return rta

        # Obtengo la fecha y hora actual
        fecha_hora_actual = self.dth.get_now()

        # Abro la sesión de la base de datos
        session = self.cp.binded_session()

        # Obtengo la tabla actual
        actual = self.cp.tabla('actual')

        # Busco todos los registros de la tabla actual
        q_obj = session.query(
            func.addtime(
                actual.FechaServicio, func.time(actual.HoraServicio)
            ).label('FechaHoraServicio'),
            actual.IdServicio,
            actual.Telefono,
            actual.Direccion,
            actual.Nombre,
            actual.Telefonista,
            actual.Latitud,
            actual.Longitud,
        )

        # Filtro por los servicios que tienen un teléfono similar
        q_obj = q_obj.filter(
            and_(
                actual.Telefono.like('%' + telefono + '%'),
                func.addtime(
                    actual.FechaServicio, func.time(actual.HoraServicio)
                ) < fecha_hora_actual,
            )
        )

        # Ordeno los servicio por fecha
        q_obj = q_obj.order_by(
            actual.FechaServicio.asc(), actual.HoraServicio.asc()
        )

        # Guardo los registros obtenidos
        registros = q_obj.all()

        # Cierro la sesión con la base de datos
        session.close()

        # Si no tengo registro retorno un mensaje
        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se encontraron servicios con el teléfono %s' % telefono)
            return rta

        # Creo una lista donde se van a guardar todos los servicios encontrados
        lista = []

        # Recorro los registros obtenidos
        for registro in registros:
            # El campo Telefono lo debo pasar al formato iso-8859-1
            telefono = ''
            if registro.Telefono is not None:
                try:
                    telefono = registro.Telefono.decode('iso-8859-1')
                except:
                    # NOTA: En Python 3 no se puede hacer el decode a un str
                    telefono = registro.Telefono

            # El campo Direccion lo debo pasar al formato iso-8859-1
            direccion = ''
            if registro.Direccion is not None:
                try:
                    direccion = registro.Direccion.decode('iso-8859-1')
                except:
                    # NOTA: En Python 3 no se puede hacer el decode a un str
                    direccion = registro.Direccion

            # El campo Nombre lo debo pasar al formato iso-8859-1
            nombre = ''
            if registro.Nombre is not None:
                try:
                    nombre = registro.Nombre.decode('iso-8859-1')
                except:
                    # NOTA: En Python 3 no se puede hacer el decode a un str
                    nombre = registro.Nombre

            # El campo Telefonista lo debo pasar al formato iso-8859-1
            telefonista = ''
            if registro.Telefonista is not None:
                try:
                    telefonista = registro.Telefonista.decode('iso-8859-1')
                except:
                    # NOTA: En Python 3 no se puede hacer el decode a un str
                    telefonista = registro.Telefonista

            # Paso el campo Latitud a float
            latitud = None
            if registro.Latitud is not None:
                latitud = float(registro.Latitud)

            # Paso el campo Longitud a float
            longitud = None
            if registro.Longitud is not None:
                longitud = float(registro.Longitud)

            # Creo un diccionario para guardar los datos de cada servicio
            dic = {}
            dic['id_servicio'] = registro.IdServicio
            dic['fecha_hora_servicio'] = str(registro.FechaHoraServicio)
            dic['telefono'] = telefono
            dic['direccion'] = direccion
            dic['nombre'] = nombre
            dic['telefonista'] = telefonista
            dic['latitud'] = latitud
            dic['longitud'] = longitud

            # Agrego cada servicio a la lista
            lista.append(dic)

        # Si todo salió bien retorno la lista de servicios
        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def grabar_importe(self, id_servicio, importe, usuario,
                                                        nombre_tabla='actual'):
        '''
            Esta función se encarga de agregar o modificar el importe de un
            servicio.

            Argumentos:
                id_servicio (int)
                importe (float)
                usuario (str)
                nombre_tabla (str)

            Funciones:
                DateTimeHelper().get_now

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Abro la sesión de la base de datos
        session = self.cp.binded_session()

        # Obtengo la tabla dada
        tabla_servicios = self.cp.tabla(nombre_tabla)

        # Busco el servicio según su id
        q_obj = session.query(tabla_servicios.IdServicio)

        # Filtro por el id_servicio recibido
        q_obj = q_obj.filter(tabla_servicios.IdServicio == id_servicio)

        # Actualizo el importe del servicio dado
        dic = {}
        dic['Importe'] = importe
        rta_update = q_obj.update(dic, synchronize_session=False)

        # Si no se pudo actualizar el servicio
        if rta_update != 1:
            # Cierro la sesión con la base de datos
            session.close()

            # Si estoy buscando en la tabla "actual" y no encuentro el servicio
            if nombre_tabla == 'actual':
                # Voy a buscar en la tabla "historico"
                return self.grabar_importe(
                    id_servicio, importe, usuario, nombre_tabla='historico'
                )

            # Si no se encontró en la tabla "actual" ni en la tabla "historico"
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo guardar el importe del servicio')
            return rta

        # Hago el commit con la base de datos
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Guardo en un log para indicar quién modificó el importe del servicio
        dic = {}
        dic['FechaHora'] = self.dth.get_now()
        dic['Descripcion'] = _(
            u'%s modificó el importe del servicio con id %s a $%s'
            % (usuario, id_servicio, importe)
        )
        dic['Tipo'] = 'IMP'
        dic['Usuario'] = usuario
        dic['Leido'] = 0
        self.cp.agregar('logbs', **dic)

        # Si se pudo actualizar el importe del servicio
        # retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se guardó el importe del servicio correctamente')
        return rta

    def obtener_por_id(self, id_servicio,
                       buscar_en_historico=False,
                       nombre_tabla='actual',
                       siniestro=None):
        '''
            Esta función se encarga de obtener los datos de un servicio según
            su id.
            Puede buscar en la tabla "actual" y en la tabla "historico".

            Argumentos:
                id_servicio (int)
                buscar_en_historico (bool)
                nombre_tabla (str)
                siniestro (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                            [id_servicio] (int)
                            ['telefono'] (str)
                            ['fecha_servicio'] (str) 'yyyy-mm-dd'
                            ['hora_servicio'] (str) '1900-01-01 hh:mm:ss'
                            ['prioridad'] (int)
                            ['id_movil'] (str)
                            ['direccion'] (str)
                            ['latitud'] (float)
                            ['longitud'] (float)
                            ['id_flota'] (int)
                            ['hora_salida'] (str o None) 'yyyy-mm-dd hh:mm:ss'
                            ['id_servicio_ptt'] (int)
                            ['id_zona'] (int)
                            ['nombre_zona'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Abro la sesión con la BD cp2000
        session = self.cp.binded_session()

        # Obtengo la tabla actual o historico
        tabla = self.cp.tabla(nombre_tabla)

        # Obtengo la tabla moviles
        moviles = self.cp.tabla('moviles')

        # Busco los datos que me interesan del servicio
        q_obj = session.query(
            tabla.IdServicio,
            tabla.Telefono,
            tabla.FechaServicio,
            tabla.HoraServicio,
            tabla.Prioridad,
            tabla.IdMovil,
            tabla.Direccion,
            tabla.Latitud,
            tabla.Longitud,
            tabla.HoraSalida,
            tabla.Siniestro,
            tabla.IdServicioPtt,
            tabla.IdZona,
            tabla.Zona,
            moviles.IdFlota
        )

        # Realizo el join con la tabla moviles por el id del móvil
        q_obj = q_obj.outerjoin((
            moviles,
            tabla.IdMovil == moviles.IdMovil
        ))

        # Si recibo el dato siniestro
        if siniestro is not None and siniestro != '':
            # Debo filtrar por el siniestro recibido
            q_obj = q_obj.filter(tabla.Siniestro == siniestro)
        else:
            # Si no tengo el dato siniestro debo filtrar por ID del servicio
            q_obj = q_obj.filter(tabla.IdServicio == id_servicio)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesión con la base de datos
        session.close()

        # Si no se pudo obtener el registro
        if registro is None:
            # Si buscar_en_historico es True
            # y no se encontró en la tabla "actual"
            if buscar_en_historico is True and nombre_tabla == 'actual':
                # Llamo nuevamente a la función obtener_por_id
                # pero en la tabla "historico"
                return self.obtener_por_id(
                    id_servicio, False, 'historico', siniestro
                )

            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo obtener el servicio')
            return rta

        # El campo Telefono lo debo pasar al formato iso-8859-1
        if registro.Telefono is not None:
            try:
                telefono = registro.Telefono.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                telefono = registro.Telefono
        else:
            telefono = ''

        # El campo IdMovil lo debo pasar al formato iso-8859-1
        if registro.IdMovil is not None:
            try:
                id_movil = registro.IdMovil.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                id_movil = registro.IdMovil
        else:
            id_movil = ''

        # El campo Direccion lo debo pasar al formato iso-8859-1
        if registro.Direccion is not None:
            try:
                direccion = registro.Direccion.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                direccion = registro.Direccion
        else:
            direccion = ''

        # Paso el campo Latitud a float
        if registro.Latitud is not None:
            latitud = float(registro.Latitud)
        else:
            latitud = 0.0

        # Paso el campo Longitud a float
        if registro.Longitud is not None:
            longitud = float(registro.Longitud)
        else:
            longitud = 0.0

        # Si tengo el dato HoraSalida
        if registro.HoraSalida is not None:
            # Lo paso a str
            hora_salida = '%s %s' \
                % (str(registro.FechaServicio), str(registro.HoraSalida)[11:])
        else:
            # Si no, lo dejo en None
            hora_salida = None

        # El campo Siniestro lo debo pasar al formato iso-8859-1
        if registro.Siniestro is not None:
            try:
                siniestro = registro.Siniestro.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                siniestro = registro.Siniestro
        else:
            siniestro = ''

        # El campo Zona lo debo pasar al formato iso-8859-1
        if registro.Zona is not None:
            try:
                nombre_zona = registro.Zona.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                nombre_zona = registro.Zona
        else:
            nombre_zona = ''

        # Si pude obtener el registro
        # Guardo el teléfono, la fecha y la hora en un diccionario
        dic = {}
        dic['id_servicio'] = registro.IdServicio
        dic['telefono'] = telefono
        dic['fecha_servicio'] = str(registro.FechaServicio)
        dic['hora_servicio'] = str(registro.HoraServicio)
        dic['prioridad'] = registro.Prioridad
        dic['id_movil'] = id_movil
        dic['direccion'] = direccion
        dic['latitud'] = latitud
        dic['longitud'] = longitud
        dic['id_flota'] = registro.IdFlota
        dic['hora_salida'] = hora_salida
        dic['siniestro'] = siniestro
        dic['id_servicio_ptt'] = registro.IdServicioPtt
        dic['id_zona'] = registro.IdZona
        dic['nombre_zona'] = nombre_zona

        # Retorno True y el teléfono, la fecha y la hora del servicio
        rta['Estado'] = True
        rta['Respuesta'] = dic
        return rta

    def listar(self, prioridad=None):
        '''
            Esta función se encarga de listar los servicios
            de la tabla "actual" según una prioridad.
            Si la prioridad es None se listan todos los servicios.

            Argumentos:
                prioridad (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de list)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Abro la sesión con la BD cp2000
        session = self.cp.binded_session()

        # Obtengo la tabla actual
        actual = self.cp.tabla('actual')

        # Busco los datos que me interesan del servicio
        q_obj = session.query(
            actual.IdServicio,
            actual.Telefono,
            actual.FechaServicio,
            actual.HoraServicio,
            actual.Prioridad,
            actual.Direccion,
            actual.Latitud,
            actual.Longitud,
            actual.Siniestro,
            actual.IdSucursal,
            actual.IdMovil
        )

        # Si tengo el dato prioridad
        if prioridad is not None:
            # Filtro los servicio por la prioridad
            q_obj = q_obj.filter(actual.Prioridad == prioridad)

        # Ordeno los servicio por fecha
        q_obj = q_obj.order_by(
            actual.FechaServicio.asc(), actual.HoraServicio.asc()
        )

        # Guardo los registros obtenidos
        registros = q_obj.all()

        # Cierro la sesión con la base de datos
        session.close()

        # Si no se encontró ningún registro
        if len(registros) == 0:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontraron servicios')
            return rta

        # Creo una lista para ir guardando los servicios
        lista = []

        # Recorro los registros obtenidos
        for registro in registros:
            # El campo Telefono lo debo pasar al formato iso-8859-1
            if registro.Telefono is not None:
                try:
                    telefono = registro.Telefono.decode('iso-8859-1')
                except:
                    # NOTA: En Python 3 no se usa decode en los "str"
                    telefono = registro.Telefono
            else:
                telefono = ''

            # El campo Direccion lo debo pasar al formato iso-8859-1
            if registro.Direccion is not None:
                try:
                    direccion = registro.Direccion.decode('iso-8859-1')
                except:
                    # NOTA: En Python 3 no se usa decode en los "str"
                    direccion = registro.Direccion
            else:
                direccion = ''

            # El campo Siniestro lo debo pasar al formato iso-8859-1
            if registro.Siniestro is not None:
                try:
                    siniestro = registro.Siniestro.decode('iso-8859-1')
                except:
                    # NOTA: En Python 3 no se usa decode en los "str"
                    siniestro = registro.Siniestro
            else:
                siniestro = ''

            # El campo IdMovil lo debo pasar al formato iso-8859-1
            if registro.IdMovil is not None:
                try:
                    id_movil = registro.IdMovil.decode('iso-8859-1')
                except:
                    # NOTA: En Python 3 no se usa decode en los "str"
                    id_movil = registro.IdMovil
            else:
                id_movil = ''

            # Si tengo el dato latitud
            if registro.Latitud is not None:
                # Lo paso a grados porque está en segundos
                latitud = float(registro.Latitud / 3600)
            else:
                # Si no tengo el dato latitud lo dejo en 0
                latitud = 0

            # Si tengo el dato longitud
            if registro.Longitud is not None:
                # Lo paso a grados porque está en segundos
                longitud = float(registro.Longitud / 3600)
            else:
                # Si no tengo el dato longitud lo dejo en 0
                longitud = 0

            # Agrego cada servicio a la lista
            lista.append(
                [
                    registro.IdServicio,
                    telefono,
                    str(registro.FechaServicio),
                    str(registro.HoraServicio),
                    registro.Prioridad,
                    direccion,
                    latitud,
                    longitud,
                    siniestro,
                    registro.IdSucursal,
                    id_movil
                ]
            )

        # Retorno True y el listado de servicios
        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def obtener_tipo_tarifa(self, id_servicio):
        '''
            Esta función se encarga de determinar de que tipo es el servicio
            para poder tarifarlo.

            Tipos de tarifas:
                1 = Predeterminada
                2 = Calle
                3 = Central
                4 = APP
                5 = Web
                6 = CtaCte
                7 = IVR
                8 = Whatsapp
                9 = SMS
                10 = Telegram

            Argumentos:
                id_servicio (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o int)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Abro la sesión con la BD cp2000
        session = self.cp.binded_session()

        # Obtengo la tabla actual
        actual = self.cp.tabla('actual')

        # Busco los datos que me interesan del servicio
        q_obj = session.query(
            actual.IdServicio,
            actual.Telefonista,
            actual.AceptaCtaCte,
            actual.IdServicioPtt
        )

        # Filtro por el id_servicio dado
        q_obj = q_obj.filter(actual.IdServicio == id_servicio)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesión con la base de datos
        session.close()

        # Si no se pudo obtener el registro
        if registro is None:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo obtener el servicio')
            return rta

        # Si el servicio es con cta. cte.
        if registro.AceptaCtaCte == 1:
            # Va a ser de tipo 6
            tipo = 6
        # Si el servicio es de una APP
        elif registro.IdServicioPtt is not None:
            # Va a ser de tipo 4
            tipo = 4
        # Si el servicio es del IVR
        elif registro.Telefonista == 'CpIvr':
            # Va a ser de tipo 7
            tipo = 7
        # Sino, se considera que es de tipo "Central"
        else:
            tipo = 3

        # Retorno True y el tipo del servicio
        rta['Estado'] = True
        rta['Respuesta'] = tipo
        return rta

    def cancelar(self, telefono):
        '''
            Esta función se encarga de cancelar todos los servicios pendientes
            de un teléfono dado.

            Parámetros:
                telefono (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Abro la sesión con la BD cp2000
        session = self.cp.binded_session()

        # Obtengo la tabla actual
        actual = self.cp.tabla('actual')

        # Busco los datos que me interesan del servicio
        q_obj = session.query(actual.Telefono, actual.Prioridad)

        # Filtro por el teléfono dado
        q_obj = q_obj.filter(
            and_(
                actual.Telefono == telefono,
                actual.Prioridad != 0,
                actual.Prioridad != 5
            )
        )

        # Actualizo la prioridad del servicio a cancelado
        dic = {}
        dic['Prioridad'] = 5
        rta_update = q_obj.update(dic, synchronize_session=False)

        # Si no se pudo actualizar el servicio
        if rta_update == 0:
            # Cierro la sesión con la base de datos
            session.close()

            # Si no se encontró el servicio retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró ningún servicio pendiente')
            return rta

        # Hago el commit con la base de datos
        session.commit()
        # Cierro la sesión con la base de datos
        session.close()

        # Si se pudo cancelar el servicio retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'El servicio se canceló de forma correcta')
        return rta
