# -*- coding: utf-8 -*-

import datetime
from datetime import timedelta

from at.utiles import _
from at.orm.decorators import DBSwitch
try:
    from at.map.pais import PaisPostgres
except:
    print('>> warn: No hay at.map')


@DBSwitch('cp')
class DateTimeHelper(object):

    def __init__(self):
        # Si podemos instanciamos
        try:
            self.pais = PaisPostgres()
        except:
            self.pais = None

    def get_now(self, as_str=False):
        '''
            Devuelve el día y hora del servidor de MySql.
            Se ajusta la hora según un parámetro de la tabla "ajuste_horario".
        '''

        if self.cp.engine is None:
            self.cp.connect()

        session = self.cp.binded_session()

        registro = session.execute(
            'SELECT ' +
            'now(), ajuste_horario.DifHora ' +
            'FROM ajuste_horario ' +
            'WHERE ajuste_horario.FechaVigencia <= now() ' +
            'ORDER BY ajuste_horario.FechaVigencia DESC ' +
            'LIMIT 1'
        )
        registro = registro.fetchone()

        hora = registro[0]
        dif_hora = registro[1]

        hora = hora + timedelta(hours=dif_hora)

        session.close()

        if as_str is True:
            return hora.strftime("%d/%m/%Y %H:%M:%S")

        return hora

    def get_date(self, as_str=False, date_str=False):
        '''
            Devuelve la fecha del servidor.
        '''

        now = self.get_now()
        today = now.date()

        if as_str is True:
            return today.strftime("%d/%m/%Y")

        if date_str is True:
            return today.strftime("%Y-%m-%d")

        return today

    def get_hora(self):
        '''
            Devuelve la hora del servidor.
        '''

        return self.get_now().strftime("%H:%M:%S")

    def get_fecha_null_con_hora(self):
        '''
            Devuelve un str de la siguiente manera
            '1900-01-01 H:M:S'

            Retorna:
                fecha_null_con_hora_actual (datetime)

            Funciones:
                get_hora()
        '''

        hora_actual = self.get_hora()

        fecha_null_con_hora_actual = '1900-01-01 ' + hora_actual
        fecha_null_con_hora_actual = datetime.datetime.strptime(
                            fecha_null_con_hora_actual, "%Y-%m-%d %H:%M:%S")

        return fecha_null_con_hora_actual

    def get_fecha_con_hora_null(self):
        '''
            Devulve la fecha actual con la hora en 00:00:00

            Retorna:
                fecha_con_hora_null (datetime)
        '''

        fecha_hora_actual = self.get_now()

        fecha_con_hora_null = datetime.datetime(fecha_hora_actual.year,
                                                fecha_hora_actual.month,
                                                fecha_hora_actual.day,
                                                00, 00, 00)

        return fecha_con_hora_null

    def obtener_fecha_hora_ciudad(self, id_ciudad):
        '''
            Esta función se encarga de obtener la fecha y hora según la zona
            horaria.

            Argumentos:
                id_ciudad (int)

            Funciones:
                PaisPostgres().obtener_pais_de_la_ciudad
                DateTimeHelper().get_now

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (datetime o str)
        '''

        # Declaramos la respuesta
        rta = {'Estado': False, 'Respuesta': 'No hay at.map disponible'}

        # Si no hay pais, salimos
        if self.pais is None:
            return rta

        # Obtengo los datos del país de la ciudad dada
        rta_obtener_pais_de_la_ciudad = self.pais.obtener_pais_de_la_ciudad(
                                                                    id_ciudad)
        if rta_obtener_pais_de_la_ciudad['Estado'] is False:
            return rta_obtener_pais_de_la_ciudad

        # Guardo la zona horaria del país al que pertence la flota
        zona_horaria = \
            rta_obtener_pais_de_la_ciudad['Respuesta']['zona_horaria_pais']

        # Si no tengo el dato de la zona horaria del país retorno el error
        if zona_horaria is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo obtener la zona horaria del país')
            return rta

        # Obtengo la fecha y hora actual
        fecha_hora_actual = self.get_now()

        # Hago la diferencia de la fecha según la zona horaria
        fecha_hora_ciudad = fecha_hora_actual + timedelta(hours=zona_horaria)

        # Si todo salió bien retorno la fecha y la hora de la flota
        rta['Estado'] = True
        rta['Respuesta'] = fecha_hora_ciudad
        return rta

    def get_year(self):
        '''
            Devuelvo el año actual.
        '''

        now = self.get_now()
        year = now.year

        return year
