# -*- coding: utf-8 -*-

from at.utiles import _


class Telefonos(object):
    '''
        Esta clase va a contar con funciones útiles para tratar los servicios.
    '''

    def __init__(self):
        pass

    def formatear(self, telefono, telefono_de_referencia):
        '''
            Esta función se encarga de formatear un teléfono al siguiente
            formato de ejemplo +5493413705967

            Argumentos:
                telefono (str)
                telefono_de_referencia (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Creo el diccionario a retornar
        rta = {}

        # El telefono debe ser un string
        if isinstance(telefono, str) is False:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'El teléfono %s no es un string'
                                                % str(telefono))
            return rta

        # El teléfono debe tener más de 5 caracteres
        if len(telefono) <= 5:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'El teléfono %s no cuanta con el mínimo de caracteres'
                                                            % str(telefono))
            return rta

        # Le saco los espacios en blanco al teléfono de atrás y de adelante
        telefono = telefono.strip()

        # Reemplazo si el teléfono tiene guión medio por un espacio vacío
        telefono = telefono.replace('-', '')

        # Reemplazo si el teléfono tiene algún espacio en blanco en el medio
        # por un espacio vacío
        telefono = telefono.replace(' ', '')

        # Si el teléfono recibido comienza con el signo + quiere decir que
        # ya cuenta con un formato correcto
        if telefono[0] == '+':
            rta['Estado'] = True
            rta['Respuesta'] = telefono
            return rta

        # Si el teléfono no tiene un + intento pasarlo a entero
        try:
            int(telefono)
        except:
            # Si no se puede es porque no es teléfono válido
            rta['Estado'] = False
            rta['Respuesta'] = _(u'El teléfono %s no es válido' % str(telefono))
            return rta

        # Si el teléfono comienza con dos ceros
        if telefono[:2] == '00':
            # Cambio los dos ceros por el signo +
            telefono = telefono.replace('00', '+', 1)

            # Retorno el teléfono formateado
            rta['Estado'] = True
            rta['Respuesta'] = telefono
            return rta

        # El teléfono de referencia debe ser un str
        if isinstance(telefono_de_referencia, str) is False:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'El teléfono de referencia %s no es un string'
                                % str(telefono_de_referencia))
            return rta

        # El teléfono de referencia debe tener más de 7 caracteres
        if len(telefono_de_referencia) <= 7:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'El teléfono de referencia %s no cuanta con el mínimo de caracteres'
                                            % str(telefono_de_referencia))
            return rta

        # El teléfono de referencia debe comenzar con el signo +
        if telefono_de_referencia[0] != '+':
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'El teléfono de referencia %s no comienza con el signo +'
                                % str(telefono_de_referencia))
            return rta

        # Guardo el prefijo del país del teléfono de referencia
        prefijo_pais = telefono_de_referencia[:4]

        # Guardo el prefijo de la ciudad del teléfono de referencia
        prefijo_ciudad = telefono_de_referencia[4:7]

        # Si el teléfono comienza con 15
        if telefono[:2] == '15':
            # Quiere decir que no tengo el prefijo del país ni de la ciudad
            # Primero le saco el 15
            telefono = telefono.replace('15', '', 1)

            # Y le agrego el prefijo de la ciudad
            telefono = prefijo_ciudad + telefono

        # Si el teléfono comienza con un solo cero
        # (Si comienza con dos ceros ya se trató arriba en la función)
        if telefono[0] == '0':
            # Quiere decir que tengo el prefijo de la ciudad pero no del país
            # Primero le saco el cero de adelante
            telefono = telefono.replace('0', '', 1)

        # Para el caso de que no comienza ni con 15 ni con 0
        # solo se le concatena el prefijo del país adelante
        # Le concateno al principio solo el prefijo del país
        telefono = prefijo_pais + telefono

        # Retorno el teléfono formateado
        rta['Estado'] = True
        rta['Respuesta'] = telefono
        return rta
