# -*- coding: utf-8 -*-

import os
import unittest

from at.utiles import _
from at.utiles.archivos import crear_archivo_zip
from at.utiles.archivos import leer_configobj
from at.utiles.archivos import escribir
from at.utiles.archivos import reemplazar
from at.utiles.archivos import eliminar


class TestArchivos(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_crear_archivo_zip(self):
        if os.getuid() != 0:
            print('-> Ejecutar test_crear_archivo_zip de test_archivos como root')
            return

        # le paso una ruta a comprimir que no existe
        ruta_archivo = '/noexiste/'
        ruta_destino = '/tmp/'
        nombre_zip = 'test.zip'
        carpeta_raiz = 'test'
        carpetas_excluidas = []
        archivos_excluidos = []
        nueva_extension = ''
        r = crear_archivo_zip(ruta_archivo, ruta_destino, nombre_zip,
                        carpeta_raiz, carpetas_excluidas, archivos_excluidos,
                        nueva_extension)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(r['Respuesta'] ==
                                        _(u'La ruta a comprimir no es válida'))

        # le paso una ruta destino que no existe
        ruta_archivo = '/tmp/'
        ruta_destino = '/noexiste/'
        nombre_zip = 'test.zip'
        carpeta_raiz = 'test'
        carpetas_excluidas = []
        archivos_excluidos = []
        nueva_extension = ''
        r = crear_archivo_zip(ruta_archivo, ruta_destino, nombre_zip,
                        carpeta_raiz, carpetas_excluidas, archivos_excluidos,
                        nueva_extension)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(r['Respuesta'] == _(u'La ruta destino no es válida'))

        # creo un archivo ZIP a partir de una carpeta
        ruta_archivo = '/tmp/'
        ruta_destino = '/tmp/'
        nombre_zip = 'test.zip'
        carpeta_raiz = 'test'
        carpetas_excluidas = []
        archivos_excluidos = []
        nueva_extension = ''
        r = crear_archivo_zip(ruta_archivo, ruta_destino, nombre_zip,
                        carpeta_raiz, carpetas_excluidas, archivos_excluidos,
                        nueva_extension)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(r['Respuesta'] ==
                            _(u'El archivo ZIP ha sido creado correctamente'))

        # creo un archivo ZIP a partir de una carpeta sin directorio raiz
        ruta_archivo = '/tmp/'
        ruta_destino = '/tmp/'
        nombre_zip = 'test.zip'
        carpeta_raiz = ''
        carpetas_excluidas = []
        archivos_excluidos = []
        nueva_extension = ''
        r = crear_archivo_zip(ruta_archivo, ruta_destino, nombre_zip,
                        carpeta_raiz, carpetas_excluidas, archivos_excluidos,
                        nueva_extension)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(r['Respuesta'] ==
                            _(u'El archivo ZIP ha sido creado correctamente'))

        # creo un archivo para hacer los tests
        ruta_archivo_creado = '/tmp/test.txt'
        archivo = open(ruta_archivo_creado, 'w')
        archivo.write('Este es un archivo para hacer tests')
        archivo.close()

        # creo un archivo ZIP a partir de un archivo
        ruta_archivo = ruta_archivo_creado
        ruta_destino = '/tmp/'
        nombre_zip = 'test.zip'
        carpeta_raiz = ''
        carpetas_excluidas = []
        archivos_excluidos = []
        nueva_extension = ''
        r = crear_archivo_zip(ruta_archivo, ruta_destino, nombre_zip,
                        carpeta_raiz, carpetas_excluidas, archivos_excluidos,
                        nueva_extension)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(r['Respuesta'] ==
                            _(u'El archivo ZIP ha sido creado correctamente'))

        # intento crear un archivo ZIP a partir de un archivo pero le paso
        # una ruta de archivo que no existe
        ruta_archivo = '/noexiste/tests.txt'
        ruta_destino = '/tmp/'
        nombre_zip = 'test.zip'
        carpeta_raiz = ''
        carpetas_excluidas = []
        archivos_excluidos = []
        nueva_extension = ''
        r = crear_archivo_zip(ruta_archivo, ruta_destino, nombre_zip,
                        carpeta_raiz, carpetas_excluidas, archivos_excluidos,
                        nueva_extension)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(r['Respuesta'] ==
                                        _(u'La ruta a comprimir no es válida'))

        # intento crear un archivo ZIP a partir de un archivo pero le paso
        # una ruta destino que no existe
        ruta_archivo = ruta_archivo_creado
        ruta_destino = '/noexiste/'
        nombre_zip = 'test.zip'
        carpeta_raiz = ''
        carpetas_excluidas = []
        archivos_excluidos = []
        nueva_extension = ''
        r = crear_archivo_zip(ruta_archivo, ruta_destino, nombre_zip,
                        carpeta_raiz, carpetas_excluidas, archivos_excluidos,
                        nueva_extension)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(r['Respuesta'] == _(u'La ruta destino no es válida'))

    def test_leer_configobj(self):
        # Caso correcto
        ruta_archivo = '/tmp/parametros.txt'
        f = open(ruta_archivo, 'w')
        f.write(
'''
con_log = True
con_try = True
''')
        f.close()

        rta = leer_configobj(ruta_archivo)

        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(isinstance(rta['Respuesta']['con_log'], bool))
        self.assertTrue(isinstance(rta['Respuesta']['con_try'], bool))

        # Le paso un archivo que no existe
        ruta_archivo = '/asd/no_existe'
        rta = leer_configobj(ruta_archivo)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No existe el archivo /asd/no_existe')
        )

    def test_escribir_archivo(self):
        # Caso correcto
        texto = 'agrego este texto al archivo\n'
        ruta_archivo = '/tmp/prueba_archivo.txt'
        rta = escribir(texto, ruta_archivo)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se escribió el archivo de forma correcta')
        )

        # Le paso mal la ruta
        texto = 'agrego este texto al archivo\n'
        ruta_archivo = '/asd/qwe.txt'
        rta = escribir(texto, ruta_archivo)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró la ruta /asd/qwe.txt')
        )

        # Le paso mal la ruta
        texto = 'agrego este texto al archivo\n'
        ruta_archivo = '/no_existe'
        rta = escribir(texto, ruta_archivo)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró la ruta /no_existe')
        )

        # Le paso mal la ruta
        texto = 'agrego este texto al archivo\n'
        ruta_archivo = 'no_existe/'
        rta = escribir(texto, ruta_archivo)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró la ruta no_existe/')
        )

        # Caso correcto
        texto = u'una canción\n'
        ruta_archivo = '/tmp/prueba_archivo_unicode.txt'
        rta = escribir(texto, ruta_archivo)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se escribió el archivo de forma correcta')
        )

    def test_reemplazar_texto(self):
        # Creo un archivo
        texto = 'hola'
        ruta_archivo = '/tmp/reemplazar_texto.txt'
        rta = escribir(texto, ruta_archivo, sobrescribir=True)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se escribió el archivo de forma correcta')
        )

        # Caso correcto
        texto_actual = 'hola'
        texto_nuevo = 'chau'
        ruta_archivo = '/tmp/reemplazar_texto.txt'
        rta = reemplazar(texto_actual, texto_nuevo, ruta_archivo)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se reemplazó el archivo de forma correcta')
        )

        # Creo un archivo
        texto = '\nprimer renglon\nsegundo renglon\ntercer renglon\n\n'
        ruta_archivo = '/tmp/reemplazar_texto_2.txt'
        rta = escribir(texto, ruta_archivo, sobrescribir=True)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se escribió el archivo de forma correcta')
        )

        # Caso correcto
        texto_actual = '\nprimer renglon\nsegundo renglon\ntercer renglon\n\n'
        texto_nuevo = 'este es el texto nuevo'
        ruta_archivo = '/tmp/reemplazar_texto_2.txt'
        rta = reemplazar(texto_actual, texto_nuevo, ruta_archivo)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se reemplazó el archivo de forma correcta')
        )

        # Creo un archivo
        texto = '\nprimer renglon\nsegundo renglon\ntercer renglon\n\n'
        ruta_archivo = '/tmp/reemplazar_texto_3.txt'
        rta = escribir(texto, ruta_archivo, sobrescribir=True)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se escribió el archivo de forma correcta')
        )

        # Caso correcto
        texto_actual = '\nsegundo renglon\n'
        texto_nuevo = '\nMODIFICADO EL SEGUNDO RENGLON\n'
        ruta_archivo = '/tmp/reemplazar_texto_3.txt'
        rta = reemplazar(texto_actual, texto_nuevo, ruta_archivo)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se reemplazó el archivo de forma correcta')
        )

        # Le paso un archivo que no existe
        texto_actual = 'hola'
        texto_nuevo = 'chau'
        ruta_archivo = '/tmp/no_existe.txt'
        rta = reemplazar(texto_actual, texto_nuevo, ruta_archivo)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró la ruta /tmp/no_existe.txt')
        )

    def test_eliminar_archivo(self):
        # Le paso un archivo que no existe
        ruta_archivo = '/tmp/no_existe.txt'
        rta = eliminar(ruta_archivo)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se pudo eliminar el archivo %s') % ruta_archivo
        )

        # Creo un archivo para luego eliminarlo
        texto = 'hola'
        ruta_archivo = '/tmp/eliminar.txt'
        rta = escribir(texto, ruta_archivo, sobrescribir=True)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se escribió el archivo de forma correcta')
        )

        # Caso correcto
        ruta_archivo = '/tmp/eliminar.txt'
        rta = eliminar(ruta_archivo)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se eliminó el archivo de forma correcta')
        )


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestArchivos))
    return suite
