# -*- coding: utf-8 -*-

import unittest

from at.utiles.atpagos import AtPagos
from at.orm.abm import ABM
from sqlalchemy import and_, func

from datetime import datetime


test_atmanager = {
    'user': 'root',
    'passwd': 'changeme',
    'dbname': 'atmanager_test',
    'name': 'atmanager',
    'host': '192.168.0.51',
    'engine': 'mysql',
    'port': '3306',
}


class AtPagosTesting(AtPagos):

    def __init__(self):
        AtPagos.__init__(self)
        self.atmanager = ABM(test_atmanager)


class TestAtPagos(unittest.TestCase):


    def setUp(self):
        self.apt = AtPagosTesting()
        self.apt.atmanager.borrar('atpagos_tokens')


    def tearDown(self):
        self.apt.atmanager.borrar('atpagos_tokens')

    def test_generar_token(self):
        # Fallamos por los datos incorrectos
        rta = self.apt.generar_token(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)
        self.assertFalse(rta['Estado'])

        api = 'mercado_pago'
        nombre_aplicacion = 'PideTuTaxi'
        usuario_aplicacion = '1'
        id_empresa_manager = '234234234asdasd34qwe9'
        email = 'asdas@asd.com'
        nombre = 'Juan'
        apellido = 'Paco'
        documento_tipo = 'DNI'
        documento_numero = '2234234'
        id_ciudad = 324
        direccion = 'Arguelles 234'
        telefono = '4342342'
        prefijo = '+540341'
        zona_horaria = -3
        ip = '1.1.1.1'
        # Vamos a generar con los datos correctos
        rta = self.apt.generar_token(api, nombre_aplicacion, usuario_aplicacion, id_empresa_manager, ip,
            email, nombre, apellido, documento_tipo, documento_numero, id_ciudad, direccion,
            telefono, prefijo, zona_horaria)
        self.assertTrue(rta['Estado'])

        # Queremos volver a generar un token y fallamos
        rta = self.apt.generar_token(api, nombre_aplicacion, usuario_aplicacion, id_empresa_manager, ip,
            email, nombre, apellido, documento_tipo, documento_numero, id_ciudad, direccion,
            telefono, prefijo, zona_horaria)
        self.assertFalse(rta['Estado'])

    def test_obtener_token(self):

        api = 'mercado_pago'
        nombre_aplicacion = 'PideTuTaxi'
        usuario_aplicacion = '1'
        id_empresa_manager = '234234234asdasd34qwe9'
        email = 'asdas@asd.com'
        nombre = 'Juan'
        apellido = 'Paco'
        documento_tipo = 'DNI'
        documento_numero = '2234234'
        id_ciudad = 324
        direccion = 'Arguelles 234'
        telefono = '4342342'
        prefijo = '+540341'
        zona_horaria = -3
        ip = '1.1.1.1'
        token_header = 'asdaasdas_:::::::____'
        token_payload = 'asd'
        token_signature = 'asda____'

        # Vamos a generar con los datos correctos
        rta = self.apt.generar_token(
            api, nombre_aplicacion, usuario_aplicacion, id_empresa_manager, ip,
            email, nombre, apellido, documento_tipo, documento_numero, id_ciudad, direccion,
            telefono, prefijo, zona_horaria)
        self.assertTrue(rta['Estado'])

        # Fallamos al obtener
        rta = self.apt.obtener_token(api, nombre_aplicacion, usuario_aplicacion, id_empresa_manager)
        self.assertFalse(rta['Estado'])

        # Vamos a simular que nos generaron el token valido
        session = self.apt.atmanager.binded_session()

        # Obtengo la tabel clientes
        atpagos_tokens = self.apt.atmanager.tabla('atpagos_tokens')

        # Busco el token
        q_obj = session.query(atpagos_tokens)

        # Especificamos
        estado = 0

        # Filtro por el id dado
        q_obj = q_obj.filter(and_(
            atpagos_tokens.api == api,
            atpagos_tokens.nombre_aplicacion == nombre_aplicacion,
            atpagos_tokens.usuario_aplicacion == usuario_aplicacion,
            atpagos_tokens.estado == estado
        ))

        dic = {}
        dic['estado'] = 1
        dic['token_header'] = token_header
        dic['token_payload'] = token_payload
        dic['token_signature'] = token_signature
        dic['fecha_hora_fin'] = datetime.now()

        # Guardo el registro encontrado

        try:
            # Borro el registro de la tabel aplicaciones
            q_obj.update(dic, synchronize_session=False)

            # Realizo el commit con la base de datos
            session.commit()

            # Cierro la conexión con la base de datos
            session.close()

        except Exception as e:
            # Cierro la conexión con la base de datos
            session.close()

        token = '%s.%s.%s' % (token_header, token_payload, token_signature)

        # Obtenemosr
        rta = self.apt.obtener_token(api, nombre_aplicacion, usuario_aplicacion, id_empresa_manager)
        self.assertTrue(rta['Estado'])
        self.assertEqual(rta['Respuesta']['token'], token)
        self.assertEqual(rta['Respuesta']['estado'], 1)


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestAtPagos))
    return suite
