# -*- coding: utf-8 -*-

import unittest
from datetime import datetime

from at.utiles import _
from at.utiles.comandos import Comandos
from at.orm.abm import ABM


test_pidetutaxi = {
    'user': 'postgres',
    'passwd': 'TrackAir',
    'dbname': 'test_pidetutaxi',
    'name': 'test_pidetutaxi',
    'host': '192.168.0.68',
    'engine': 'postgres',
    'port': '5432',
}


class ComandosTesting(Comandos):

    def __init__(self):
        Comandos.__init__(self, False)
        self.pidetutaxi = ABM(test_pidetutaxi)


class TestComandos(unittest.TestCase):

    def setUp(self):
        self.ct = ComandosTesting()

        self.ct.pidetutaxi.borrar('comandos')

    def tearDown(self):
        self.ct.pidetutaxi.borrar('comandos')

    def test_actualizar_estado_comando(self):
        # Le paso un comando que no existe
        hash_comando = 'noexiste'
        estado = 2
        rta = self.ct.actualizar_estado(hash_comando, estado)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
                                        _(u'No se encontró el comando dado'))

        # Agrego un comando
        dic = {}
        dic['id_comando'] = 3
        dic['fecha_hora'] = '2013-07-04 13:00:00'
        dic['mensaje'] = 'un mensaje desde el servidor de un radio taxi'
        dic['id_accion'] = 102
        dic['hash_comando'] = 'asdasd'
        dic['estado'] = 1
        dic['origen'] = 2
        dic['destino'] = 0
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        hash_comando = 'asdasd'
        estado = 2
        rta = self.ct.actualizar_estado(hash_comando, estado)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] ==
                                _(u'Se ha actualizado el estado del comando'))

        # Agrego un comando con estado 3
        dic = {}
        dic['id_comando'] = 4
        dic['fecha_hora'] = '2013-07-08 14:30:00'
        dic['mensaje'] = 'un mensaje cualquiera'
        dic['id_accion'] = 102
        dic['hash_comando'] = 'qwe'
        dic['estado'] = 3
        dic['origen'] = 2
        dic['destino'] = 0
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Le paso un comando que ya está finalizado
        hash_comando = 'qwe'
        estado = 1
        rta = self.ct.actualizar_estado(hash_comando, estado)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
                                        _(u'No se encontró el comando dado'))

        # Agrego un comando con estado 2
        dic = {}
        dic['id_comando'] = 5
        dic['fecha_hora'] = '2013-07-24 17:00:00'
        dic['mensaje'] = 'con estado 2'
        dic['id_accion'] = 102
        dic['hash_comando'] = 'edfv1254'
        dic['estado'] = 2
        dic['origen'] = 2
        dic['destino'] = 0
        dic['cant_envios'] = 3
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        hash_comando = 'edfv1254'
        estado = 2
        rta = self.ct.actualizar_estado(hash_comando, estado)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] ==
                                _(u'Se ha actualizado el estado del comando'))

        # Agrego un comando con estado 2
        dic = {}
        dic['id_comando'] = 6
        dic['fecha_hora'] = '2015-04-13 10:00:00'
        dic['mensaje'] = 'lo paso a estado 3'
        dic['id_accion'] = 120
        dic['hash_comando'] = 'hash del comando 6'
        dic['estado'] = 2
        dic['origen'] = 2
        dic['destino'] = 0
        dic['cant_envios'] = 1
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Actualizo el campo fecha_confirmacion
        hash_comando = 'hash del comando 6'
        estado = 3
        rta = self.ct.actualizar_estado(hash_comando, estado)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] ==
                                _(u'Se ha actualizado el estado del comando'))

    def test_grabar_comando(self):
        # Caso correcto, no le paso ningún parámetro
        rta = self.ct.grabar()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] ==
            _(u'El comando se pudo agregar correctamente')
        )

        # Caso correcto, le paso todos los parámetros
        id_usuario = 25
        telefono = u'Camión'
        nombre = u'Juañi'
        id_servicio = 186
        id_ciudad = 324
        id_calle1 = 112251
        id_calle2 = 235441
        altura = 901
        letra_altura = u'Ñ'
        depto = u'Depto. 2 piso D, Martín'
        aire = 1
        espacio_en_baul = 1
        observaciones = u'una observación'
        id_movil = u'36ñ'
        id_flota = 2
        mensaje = u'un mensaje, canción'
        id_accion = 852
        origen = 0
        destino = 2
        estado = 11
        empresa = u'Empresa Canción'
        patente = 'AAA 111'
        licencia = '111'
        latitud_movil = 123
        longitud_movil = 456
        fecha_hora_pos_movil = datetime(2015, 4, 24, 13, 30, 50)
        latitud_usuario = 789
        longitud_usuario = 963
        fecha_hora_pos_usuario = datetime(2015, 4, 24, 13, 30, 40)
        id_usuario_a_seguir = 15
        nombre_usuario_a_seguir = u'Martín'
        id_estado_usuario_a_seguir = 2
        id_subestado_usuario_a_seguir = 3
        moviles_cercanos = ['moviles cercanos']
        motivo_estado_usuario = 1
        dni_chofer = 12345678
        chofer = u'Chofer Agustín'
        puntaje_chofer = 8
        con_cta_cte = 0
        hash_comando = 'el hash del comando'
        solicitado_para_a_seguir = u'Seguir a Higuín'
        telefono_usuario_a_seguir = '4110566'
        telefono_empresa = '4555555'
        fecha_subio_al_movil = datetime(2015, 4, 24, 13, 30, 52)
        fecha_fin_servicio = datetime(2015, 4, 24, 13, 35, 52)
        tiempo_de_viaje = datetime(1900, 1, 1, 0, 5, 0)
        velocidad_movil = 52
        dif_ult_pos_movil = datetime(1900, 1, 1, 0, 0, 23)
        datos_ctacte = 'agrego datos de la cta. cte.'
        rta = self.ct.grabar(id_usuario, telefono, nombre, id_servicio,
            id_ciudad, id_calle1, id_calle2, altura, letra_altura, depto, aire,
            espacio_en_baul, observaciones, id_movil, id_flota, mensaje,
            id_accion, origen, destino, estado, empresa, patente, licencia,
            latitud_movil, longitud_movil, fecha_hora_pos_movil,
            latitud_usuario, longitud_usuario, fecha_hora_pos_usuario,
            id_usuario_a_seguir, nombre_usuario_a_seguir,
            id_estado_usuario_a_seguir, id_subestado_usuario_a_seguir,
            moviles_cercanos, motivo_estado_usuario, dni_chofer, chofer,
            puntaje_chofer, con_cta_cte, hash_comando, solicitado_para_a_seguir,
            telefono_usuario_a_seguir, telefono_empresa, fecha_subio_al_movil,
            fecha_fin_servicio, tiempo_de_viaje, velocidad_movil,
            dif_ult_pos_movil, datos_ctacte
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] ==
            _(u'El comando se pudo agregar correctamente')
        )

        # Caso correcto, solo le paso un mensaje
        id_usuario = 25
        mensaje = u'un mensaje'
        id_accion = 201
        origen = 0
        destino = 2
        rta = self.ct.grabar(id_usuario=id_usuario, mensaje=mensaje,
            id_accion=id_accion, origen=origen, destino=destino
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] ==
            _(u'El comando se pudo agregar correctamente')
        )

        # Caso correcto, le paso el hash_comando
        id_usuario = 26
        mensaje = u'un mensaje 2'
        id_accion = 201
        origen = 0
        destino = 2
        hash_comando = 'el hash del servicio'
        telefono_empresa = '4555555'
        rta = self.ct.grabar(id_usuario=id_usuario, mensaje=mensaje,
            id_accion=id_accion, origen=origen, destino=destino,
            hash_comando=hash_comando, telefono_empresa=telefono_empresa,
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] ==
            _(u'El comando se pudo agregar correctamente')
        )

        # Caso incorrecto, le paso el mismo hash_comando
        id_usuario = 26
        mensaje = u'un mensaje 2'
        id_accion = 201
        origen = 0
        destino = 2
        hash_comando = 'el hash del servicio'
        rta = self.ct.grabar(id_usuario=id_usuario, mensaje=mensaje,
            id_accion=id_accion, origen=origen, destino=destino,
            hash_comando=hash_comando
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
            _(u'No se pudo agregar el comando a la base de datos')
        )

    def test_existe_comando(self):
        # Le paso un comando nuevo
        hash_comando = 'asdasd'
        rta = self.ct.existe(hash_comando)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'Comando nuevo'))

        # Agrego un comando
        dic = {}
        dic['id_comando'] = 4
        dic['fecha_hora'] = '2013-07-10 16:30:00'
        dic['mensaje'] = 'un mensaje'
        dic['id_accion'] = 102
        dic['hash_comando'] = 'repetido'
        dic['estado'] = 1
        dic['origen'] = 2
        dic['destino'] = 0
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Le paso un comando que existe
        hash_comando = 'repetido'
        rta = self.ct.existe(hash_comando)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == _(u'Comando existente'))

    def test_finalizar_comandos(self):
        # Agrego un comando en estado 1
        dic = {}
        dic['fecha_hora'] = '2014-01-29 17:00:00'
        dic['hash_comando'] = 'asd1'
        dic['id_accion'] = 111
        dic['estado'] = 1
        dic['id_servicio'] = 20
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Agrego un comando en estado 2
        dic = {}
        dic['fecha_hora'] = '2014-01-29 17:00:00'
        dic['hash_comando'] = 'asd2'
        dic['id_accion'] = 111
        dic['estado'] = 2
        dic['id_servicio'] = 20
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Agrego un comando en estado 3
        dic = {}
        dic['fecha_hora'] = '2014-01-29 17:00:00'
        dic['hash_comando'] = 'asd3'
        dic['id_accion'] = 111
        dic['estado'] = 3
        dic['id_servicio'] = 20
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Agrego un comando en estado 4
        dic = {}
        dic['fecha_hora'] = '2014-01-29 17:00:00'
        dic['hash_comando'] = 'asd4'
        dic['id_accion'] = 111
        dic['estado'] = 4
        dic['id_servicio'] = 20
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Agrego un comando en estado 5
        dic = {}
        dic['fecha_hora'] = '2014-01-29 17:00:00'
        dic['hash_comando'] = 'asd5'
        dic['id_accion'] = 111
        dic['estado'] = 5
        dic['id_servicio'] = 20
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Agrego un comando en estado 2 pero de otro servicio
        dic = {}
        dic['fecha_hora'] = '2014-01-29 17:00:00'
        dic['hash_comando'] = 'asd6'
        dic['id_accion'] = 111
        dic['estado'] = 2
        dic['id_servicio'] = 30
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Agrego un comando en estado 2 pero de otro servicio
        dic = {}
        dic['fecha_hora'] = '2014-01-29 17:00:00'
        dic['hash_comando'] = 'asd7'
        dic['id_accion'] = 111
        dic['estado'] = 3
        dic['id_servicio'] = 40
        agregar = self.ct.pidetutaxi.agregar('comandos', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        id_servicio = 20
        rta = self.ct.finalizar(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] ==
            _(u'Todos los comandos del servicio fueron finalizados'))

        # Le paso un servicio que no tiene comandos por finalizar
        id_servicio = 40
        rta = self.ct.finalizar(id_servicio)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
            _(u'Los comandos ya estaban finalizados'))


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestComandos))
    return suite
