# -*- coding: utf-8 -*-

import sys
import unittest

from at.utiles.compresiones import ascci_to_binary
from at.utiles.compresiones import descomprimir_asciis
from at.utiles.compresiones import int_to_binary
from at.utiles.compresiones import comprimir_entero


class TestCompresiones(unittest.TestCase):

    def test_ascci_to_int(self):
        ascii = ''
        rta = ascci_to_binary(ascii)
        self.assertTrue(rta == "00010101")

        ascii = '@'
        rta = ascci_to_binary(ascii)
        self.assertTrue(rta == "01000000")

        ascii = 'hola mundo'
        rta = ascci_to_binary(ascii)
        self.assertTrue(rta == "1101000011011110110110001100001001000000110110101110101011011100110010001101111")

    def test_descomprimir_asciis(self):
        asciis = '@'
        rta = descomprimir_asciis(asciis)
        self.assertTrue(rta == 64)

        asciis = ''
        rta = descomprimir_asciis(asciis)
        self.assertTrue(rta == '')

    def test_int_to_binary(self):
        bianrio = 300
        rta = int_to_binary(bianrio)
        self.assertTrue(rta == "0000000100101100")

        bianrio = 1
        rta = int_to_binary(bianrio)
        self.assertTrue(rta == "00000001")

        bianrio = 999999
        rta = int_to_binary(bianrio)
        self.assertTrue(rta == "000011110100001000111111")

    def test_comprimir_entero(self):
        if sys.version_info.major >= 3:
            return

        entero = 64
        rta = comprimir_entero(entero)
        self.assertTrue(rta == "@")

        entero = 300
        rta = comprimir_entero(entero)
        rta2 = descomprimir_asciis(rta)
        self.assertTrue(rta2 == 300)

        entero = 123456789
        rta = comprimir_entero(entero)
        rta2 = descomprimir_asciis(rta)
        self.assertTrue(rta2 == 123456789)

        entero = 29777443
        rta = comprimir_entero(entero)
        rta2 = descomprimir_asciis(rta)
        self.assertTrue(rta2 == 29777443)


def suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCompresiones))
    return suite
