# -*- coding: utf-8 -*-

import unittest

from at.utiles import _
from at.utiles import conversiones


class TestConversiones(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_siempre_dos_decimales(self):
        # le paso un string.
        valor = 'asd'
        v = conversiones.siempre_dos_decimales(valor)
        self.assertTrue(v['Estado'] is False)
        un_decimal = 7.2
        v = conversiones.siempre_dos_decimales(un_decimal)
        self.assertTrue(v['Estado'] is True)
        self.assertTrue(v['Respuesta'] == '7.20')
        dos_decimales = 5.63
        v = conversiones.siempre_dos_decimales(dos_decimales)
        self.assertTrue(v['Estado'] is True)
        self.assertTrue(v['Respuesta'] == '5.63')
        solo_cero = 9.0
        v = conversiones.siempre_dos_decimales(solo_cero)
        self.assertTrue(v['Estado'] is True)
        self.assertTrue(v['Respuesta'] == '9.00')
        sin_decimales = 24
        v = conversiones.siempre_dos_decimales(sin_decimales)
        self.assertTrue(v['Estado'] is True)
        self.assertTrue(v['Respuesta'] == '24.00')
        muchos_decimales = 89.123456
        v = conversiones.siempre_dos_decimales(muchos_decimales)
        self.assertTrue(v['Estado'] is True)
        self.assertTrue(v['Respuesta'] == '89.12')

    def test_quitar_acentos(self):
        # Le paso un texto sin acentos en formato str
        texto = 'sin acentos'
        rta = conversiones.quitar_acentos(texto)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 'sin acentos')

        # Le paso un texto con acentos en formato unicode
        texto = u'una canción ñan'
        rta = conversiones.quitar_acentos(texto)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 'una cancion nan')

        # Le paso un texto con acentos en formato str
        texto = 'una canción ñan'
        rta = conversiones.quitar_acentos(texto)
        self.assertTrue(rta['Estado'] is True)

        # Le paso un texto con acentos en formato str y le hago un decode
        texto = 'una canción ñan'
        try:
            texto = texto.decode('iso-8859-1')
        except:
            # NOTA: Esto no se puede hacer en Python 3
            pass
        rta = conversiones.quitar_acentos(texto)
        self.assertTrue(rta['Estado'] is True)

        # Le paso un texto con acentos en formato unicode y le hago un encode
        texto = u'una canción ñan'
        #texto = texto.encode('iso-8859-1')
        rta = conversiones.quitar_acentos(texto)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 'una cancion nan')

    def test_cambiar_de_signo(self):

        valor = -5
        rta = conversiones.cambiar_de_signo(valor)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 5)

        valor = 5
        rta = conversiones.cambiar_de_signo(valor)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == -5)

        valor = -5.5
        rta = conversiones.cambiar_de_signo(valor)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 5.5)

        valor = 5.5
        rta = conversiones.cambiar_de_signo(valor)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == -5.5)

        valor = 0
        rta = conversiones.cambiar_de_signo(valor)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == -0)

        valor = 'asd'
        rta = conversiones.cambiar_de_signo(valor)
        self.assertTrue(rta['Estado'] is False)

        valor = True
        rta = conversiones.cambiar_de_signo(valor)
        self.assertTrue(rta['Estado'] is False)

    def test_solo_numeros(self):
        # Caso correcto
        valor = 'CC280041'
        rta = conversiones.solo_numeros(valor)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 280041)

        # Le paso un string sin números
        valor = 'CCasdqwe'
        rta = conversiones.solo_numeros(valor)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo obtener el número de CCasdqwe')
        )

        # Le paso un string con varias palabras y números
        valor = 'asd 1 qwe 2 a5f77g'
        rta = conversiones.solo_numeros(valor)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 12577)

        # Paso un unicode sin números
        valor = u'Canción'
        rta = conversiones.solo_numeros(valor)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo obtener el número de Canción')
        )

        # Paso un unicode con números
        valor = u'Canción1234 5'
        rta = conversiones.solo_numeros(valor)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 12345)


def suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestConversiones))
    return suite
