# -*- coding: utf-8 -*-

import unittest
from at.utiles.direcciones import *

lista = ['movil', 'mobil', 'móvil',
         'remis', 'remi', 'remiss',
         'remisse', 'vehiculo', 'vehículo',
         'servicio', 'tasi', 'taxi',
         'coche', 'auto', 'unidades',
         'unidad' ]


class TestAnalisis(unittest.TestCase):

    def test_analisis_falla(self):
        assert obtener_ultimo_numero("hola") == -2

    def test_analisis_falla0(self):
        assert obtener_ultimo_numero("hola 999") == '999'

    def test_analisis_ok(self):
        assert obtener_ultimo_numero("hola 9978") == '9978'

    def test_1(self):
        assert obtener_ultimo_numero("") == -1

    def test_2(self):
        assert obtener_ultimo_numero([]) == -1

    def test_3(self):
        assert obtener_ultimo_numero(None) == -1

    def test_4(self):
        assert obtener_ultimo_numero('9') == '9'

    def test_5(self):
        assert obtener_ultimo_numero('hola 8 9') == '9'

    def test_6(self):
        assert obtener_ultimo_numero('manzana 10 lote 4') == '4'

    def test_7(self):
        assert obtener_ultimo_numero('10 lote 4') == '4'

    def test_78(self):
        assert obtener_ultimo_numero('10 lote 400') == '400'

    def test_8(self):
        assert obtener_ultimo_numero('10 4') == '4'

    def test_9(self):
        assert obtener_ultimo_numero('4 900') == '900'

    def test_10(self):
        assert obtener_ultimo_numero('4 9 perrito') == '9'

    def test_11(self):
        assert obtener_ultimo_numero(['4', '9', 'perrito']) == '9'

    def test_obtener_indice_elemento(self):
        assert obtener_indice_elemento(None, None) == -1

    def test_obtener_indice_elemento1(self):
        assert obtener_indice_elemento(None, 'hola') == -1

    def test_obtener_indice_elemento2(self):
        assert obtener_indice_elemento('hooh', '') == -1

    def test_obtener_indice_elemento2a(self):
        assert obtener_indice_elemento('hooh', 'i') == -2

    def test_obtener_indice_elemento3(self):
        assert obtener_indice_elemento('hola', 'a') == 3

    def test_obtener_indice_elemento4(self):
        assert obtener_indice_elemento('holaa', 'a') == 3

    def test_obtener_indice_elemento5(self):
        assert obtener_indice_elemento('holaa 68', '68') == 6

    def test_obtener_indice_elemento6(self):
        assert obtener_indice_elemento(['holaa', '68'], '68') == 1

    def test_obtener_indice_elemento7(self):
        assert obtener_indice_elemento(['holaa', 'movil', '68'], 'movil') == 1

    def test_dividir(self):
        assert dividir_string(None) == -1

    def test_dividir1(self):
        assert dividir_string('hola') == ['hola']

    def test_dividir2(self):
        assert dividir_string('hola bebe') == ['hola', 'bebe']

    def test_substring(self):
        assert obtener_substring('', 'hola', 'hola') == -1

    def test_substring1(self):
        assert obtener_substring('hola', 'hola', None) == -1

    def test_substring2(self):
        assert obtener_substring('hola', None, 'hola') == -1

    def test_substring6(self):
        assert obtener_substring(['hola', 'caballo', 'perro'], '*inicio', 0) == ['hola']

    def test_substring4(self):
        assert obtener_substring(['hola', 'caballo', 'perro'], '*inicio', 1) == ['hola', 'caballo']

    def test_substring5(self):
        assert obtener_substring(['hola', 'caballo', 'perro'], '*inicio', 2) == ['hola', 'caballo', 'perro']

    def test_substring3(self):
        assert obtener_substring(['hola', 'caballo', 'perro'], '*inicio', '*fin') == ['hola', 'caballo', 'perro']

    def test_substring7(self):
        assert obtener_substring(['hola', 'caballo', 'perro'], 1, '*fin') == ['caballo', 'perro']

    def test_substring8(self):
        assert obtener_substring(['hola', 'caballo', 'perro'], 1, 1) == ['caballo']

    def test_limpiar_string(self):
        assert reemplazar_string("hola.", []) == -1

    def test_limpiar_string1(self):
        assert reemplazar_string("", []) == -1

    def test_limpiar_string2(self):
        assert reemplazar_string("hola.", '.') == 'hola '

    def test_limpiar_string3(self):
        assert reemplazar_string('Bv Segui 5302. Farmacia. Cuttiani', '-.,;') == 'Bv Segui 5302  Farmacia  Cuttiani'

    def test_reducir_espacios_string(self):
        assert reducir_espacios_string("") == -1

    def test_reducir_espacios_string1(self):
        assert reducir_espacios_string("        perro") == "perro"

    def test_reducir_espacios_string2(self):
        assert reducir_espacios_string("   hola       perro") == "hola perro"

    def test_superi1(self):
        assert caso_uso_desuso(None, 'movil') == -1

    def test_superi2(self):
        assert caso_uso_desuso('Bv Segui 5302 Farmacia. Cuttiani', '*inicio') == 'Bv Segui 5302'

    def test_superi3(self):
        assert caso_uso_desuso('Bv Segui. 5302. Farmacia. Cuttiani', '*inicio') == 'Bv Segui 5302'

    def test_superi4(self):
        assert caso_uso_desuso('laprida 6343 A, entre patricios y frias, Romina', '*inicio') == 'laprida 6343'

    def test_superi5(self):
        assert caso_uso_desuso('Olive 2274 Erika Cuestas', '*inicio') == 'Olive 2274'

    def test_superi6(self):
        assert caso_uso_desuso('Ovidio Lagos, 522 , dos Santos', '*inicio') == 'Ovidio Lagos 522'




    def test_reg(self): assert reg(['hola', 'perro'], "que tal quiero llevar a mi") == -2

    def test_reg1(self): assert reg(['hola', 'perro'], "que tal quiero llevar a mi perroA") == 27




    def test_busqueda(self): assert busqueda([], 'iouioi') == [-1, -1]

    def test_busqueda1(self): assert busqueda(lista, '') == [-1, -1]

    def test_busqueda2(self): assert busqueda(lista, 'movil') == [0, 'movil']

    def test_busqueda3(self): assert busqueda(lista, 'mi movil se llama jose') == [1, 'movil']

    def test_busqueda4(self): assert busqueda(lista, 'mi movil se llama taxi') == [1,
                                                                                   'movil']  # solo devuelve el primero

    def test_busqueda5(self): assert busqueda(lista, 'mi movil se llama movil') == [1,
                                                                                    'movil']  # solo devuelve el primero

    def test_busqueda_no_esta_en_el_String(self): assert busqueda(lista, 'mi perro lassi') == [-2, -2]







class TestObtenerDireccion(unittest.TestCase):
    def test_OD(self): assert obtener_direccion(lista, '') == -1

    def test_sarasa2(self): assert obtener_direccion('', 'jiji') == -1

    def test_sarasa3(self): assert obtener_direccion(lista, 'hola un movil para balcarce 123') == "balcarce 123"

    def test_sarasa4(self): assert obtener_direccion(lista, 'a que hora me mandas el movil') == -2

    def test_sarasa5(self): assert obtener_direccion(lista, 'balcarce 123') == "balcarce 123"

    def test_sarasa5a(self): assert obtener_direccion(lista, '9 de julio 123') == "9 de julio 123"

    def test_sarasa5b(self): assert obtener_direccion(lista, 'un movil a 9 de julio 123') == "9 de julio 123"

    def test_sarasa6(self): assert obtener_direccion(lista, 'mandame un auto a balcarce 123') == "balcarce 123"

    def test_sarasa7(self): assert obtener_direccion(lista, 'mandame un coche a balcarce 123') == "balcarce 123"

    def test_sarasa8(self): assert obtener_direccion(lista,
                                                     'mandame un coche para regimiento 11 123') == "regimiento 11 123"

    def test_sarasa9(self): assert obtener_direccion(lista, 'Almafuerte 1893 Herrera') == "Almafuerte 1893"

    def test_sarasa10(self): assert obtener_direccion(lista, '27 y contitucion mercado   de productores vanina') == -2

    def test_sarasa11(self): assert obtener_direccion(lista,
                                                      'Hola, queria un movil para ruben dario 1260 (ruben dario entre avenida alberdi y mar del plata)') == "ruben dario 1260"

    def test_sarasa12(self): assert obtener_direccion(lista, 'Hola me mandas un movil caferata 942') == "caferata 942"

    def test_sarasa13(self): assert obtener_direccion(lista,
                                                      'Necesito un coche a las 14.20 , en Colon 1444') == "Colon 1444"

    def test_sarasa70(self): assert obtener_direccion(lista,
                                                      'Hola me envias un movil magallanes 2376. Magallanes entre viamonte y la paz') == "magallanes 2376"

    def test_sarasa701(self): assert obtener_direccion(lista,
                                                       'Mandame rapido un taxi aca guaycuru 7727') == "guaycuru 7727"

    def test_sarasa702(self): assert obtener_direccion(lista,
                                                       'Hola me podes mandar un taxi a jujuy 2925 gracias') == "jujuy 2925"

    def test_sarasa703(self): assert obtener_direccion(lista,
                                                       'B tardes me mandas un movil a Jose ingenieros 8265') == "Jose ingenieros 8265"

    def test_sarasa704(self): assert obtener_direccion(lista,
                                                       'Hola, podrian enviarme un movil a Velez Sarsfield 178?') == "Velez Sarsfield 178"

    def test_sarasa705(self): assert obtener_direccion(lista,
                                                       'Hola buen dia me envias un movil a ayolas 851 ??') == "ayolas 851"

    def test_sarasa706(self): assert obtener_direccion(lista, 'Hola Galvez 2835') == "Galvez 2835"

    def test_sarasa707(self): assert obtener_direccion(lista,
                                                       'buenas tardes un movil para gaboto 1570') == "gaboto 1570"

    def test_sarasa708(self): assert obtener_direccion(lista, 'Un coche a pasaje cabrera 5091') == "pasaje cabrera 5091"

    def test_sarasa709(self): assert obtener_direccion(lista,
                                                       'Hola buenas tardes me podras mandar un movil a San Luis 2020 mi nombre es alicia') == "San Luis 2020"

    def test_sarasa7010(self): assert obtener_direccion(lista,
                                                        'Hola Hiba a necesitar 2 unidades en calle Batlle Ordonez 741, si podria ser') == "calle Batlle Ordonez 741"

    def test_sarasa7011(self): assert obtener_direccion(lista,
                                                        'Hola me mandarias un taxi a Vera Mujica 1921') == "Vera Mujica 1921"

    def test_sarasa7012(self): assert obtener_direccion(lista,
                                                        'Hola, buenas tardes. Me mandarias un auto para INDEAR? Ocampo 210 bis') == "INDEAR Ocampo 210 bis"

    def test_sarasa7012b(self): assert obtener_direccion(lista,
                                                         'Hola, buenas tardes. Me mandarias un auto Ocampo 210 bis') == "Ocampo 210 bis"

    def test_sarasa7013(self): assert obtener_direccion(lista, 'Hola me mandas un taxi liniers 1859') == "liniers 1859"

    def test_sarasa7014(self): assert obtener_direccion(lista,
                                                        'Hola me podes mandar un movil pj1849 N 6666') == "pj1849 N 6666"

    def test_sarasa7015(self): assert obtener_direccion(lista,
                                                        'Buen dia me mandaria un taxi calle labarden 2474 (ex olive) entre ottone y de angelis,  gracias') == "labarden 2474"

    def test_sarasa7016(self): assert obtener_direccion(lista,
                                                        'Hola, me mandas un movil Estanislao lopez 2671 piso 22A1? Gracias') == "Estanislao lopez 2671"

    def test_sarasa7017(self): assert obtener_direccion(lista,
                                                        'buenas tarde me podes mandar un coche a pj 1849 6666') == "pj 1849 6666"

    def test_sarasa7018(self): assert obtener_direccion(lista,
                                                        'Me mandas un movil a San Martin 525') == "San Martin 525"

    def test_sarasa7019(self): assert obtener_direccion(lista,
                                                        'Hola me podras mandar un movil a vila 1398 voy hasta la ex rural') == "vila 1398"

    def test_sarasa7020(self): assert obtener_direccion(lista,
                                                        'Buenas tardes le podria enviar un movil a french 748') == "french 748"

    def test_sarasa7021(self): assert obtener_direccion(lista,
                                                        'Hola! Buenas tardes Me mandarias un taxi a Riobamba 5863') == "Riobamba 5863"

    def test_sarasa7022(self): assert obtener_direccion(lista,
                                                        'Hola me mandas un coche a Garibaldi 5881') == "Garibaldi 5881"

    def test_sarasa7023(self): assert obtener_direccion(lista,
                                                        'un taxi para ovidio lagos, 522,por favor') == "ovidio lagos 522"

    def test_sarasa7024(self): assert obtener_direccion(lista, 'Buenas tardes. Bv. Segui 5203') == "Bv Segui 5203"

    def test_sarasa7025(self): assert obtener_direccion(lista, 'Ovidio Lagos, 522 , dos Santos') == "Ovidio Lagos 522"

    def test_sarasa7027(self): assert obtener_direccion(lista, 'Bv. Segui 5302. Farmacia. Cuttiani') == "Bv Segui 5302"

    def test_sarasa7028(self): assert obtener_direccion(lista,
                                                        'laprida 6343 A, entre patricios y frias, Romina') == "laprida 6343 A"

    def test_sarasa7029(self): assert obtener_direccion(lista,
                                                        'Hola un taxi en cambel 4129 puede ser?') == "cambel 4129"

    def test_sarasa7030(self): assert obtener_direccion(lista, 'Me mandas un taxi a Ceballos 2696') == "Ceballos 2696"

    def test_sarasa7031(self): assert obtener_direccion(lista,
                                                        'Hola me manda un taxi a Dorrego 1774 panaderia gracias') == "Dorrego 1774"

    def test_sarasa7032(self): assert obtener_direccion(lista,
                                                        'Me mandas un movil julio vanzo 7951 entre Hermana Paula y Manuel Gonzalez.') == "julio vanzo 7951"

    def test_sarasa7033(self): assert obtener_direccion(lista,
                                                        'Por favor necesito una unidad ahora en   av del Rosario 1144') == "av del Rosario 1144"

    def test_sarasa7034(self): assert obtener_direccion(lista,
                                                        'Hola buenas tardes me podrias mandar un coche a olive 2274') == "olive 2274"

    def test_sarasa7036(self): assert obtener_direccion(lista,
                                                        'Hola buenos tardes enviame un coche a magallanes 556,casi cba. Bien al lado de YPF') == "magallanes 556"

    def test_sarasa7037(self): assert obtener_direccion(lista,
                                                        'Magallanes 556 aqui mandame el coche') == "Magallanes 556"

    def test_sarasa7038(self): assert obtener_direccion(lista, 'Olive 2274 Erika Cuestas') == "Olive 2274"

    def test_sarasa7039(self): assert obtener_direccion(lista,
                                                        'Hola buenas tardes te pido un auto para rivera 1598') == "rivera 1598"

    def test_sarasa7041(self): assert obtener_direccion(lista, 'Pje Famailla 3424') == "Pje Famailla 3424"

    def test_sarasa7042(self): assert obtener_direccion(lista, 'Paraguay 3866 romano con debito') == "Paraguay 3866"

    def test_sarasa7043(self): assert obtener_direccion(lista,
                                                        'Hola, me mandas un taxi a tucuman 3273?') == "tucuman 3273"

    def test_sarasa7044(self): assert obtener_direccion(lista,
                                                        'Buenas tardes chicos me podrian mandar una unidad a Santa Fe 3746 para pagar con tarjeta de debito. Gracias') == "Santa Fe 3746"

    def test_sarasa7045(self): assert obtener_direccion(lista,
                                                        'Tucuman 3273 entre crespo y vera mujica') == "Tucuman 3273"

    def test_sarasa7046(self): assert obtener_direccion(lista, 'Paraguay 3866 romano DEBITO') == "Paraguay 3866"

    def test_sarasa7047(self): assert obtener_direccion(lista, 'Ed Juan Jose paso 5132') == "Ed Juan Jose paso 5132"

    def test_sarasa7048(self): assert obtener_direccion(lista, 'Av Santa Fe 3746 Cintia') == "Av Santa Fe 3746"

    def test_sarasa7049(self): assert obtener_direccion(lista,
                                                        'Hola, un taxi catamarca 2475 puede ser?') == "catamarca 2475"

    def test_sarasa7050(self): assert obtener_direccion(lista, 'Queria un movil aurora 3381') == "aurora 3381"

    def test_sarasa7051(self): assert obtener_direccion(lista,
                                                        'San Lorenzo 6277 entre barra y Campbell') == "San Lorenzo 6277"

    def test_sarasa7052(self): assert obtener_direccion(lista, 'Velez Sarsfield 1735') == "Velez Sarsfield 1735"

    def test_sarasa7053(self): assert obtener_direccion(lista, 'Buenas tardes. Patricios 886') == "Patricios 886"

    def test_sarasa7054(self): assert obtener_direccion(lista,
                                                        'San Lorenzo 6277 entre barra y Campbell') == "San Lorenzo 6277"

    def test_sarasa7055(self): assert obtener_direccion(lista, 'Hola me mandas un taxi ah 512  6373') == "512 6373"

    def test_sarasa7057(self): assert obtener_direccion(lista, 'Hola me mandarias un movil Jujuy 2316') == "Jujuy 2316"

    def test_sarasa7058(self): assert obtener_direccion(lista,
                                                        'Hola! Te pido un movil para 3 de febrero 196. (Esquina Colon). Es un edificio pero lo espero abajo. Gracias. Delfina.') == "3 de febrero 196"

    def test_sarasa7059(self): assert obtener_direccion(lista,
                                                        'Hola me envias un taxi a zeballos 3428') == "zeballos 3428"

    def test_sarasa7060(self): assert obtener_direccion(lista,
                                                        'Me podrias mandar un taxi a alippi 8834') == "alippi 8834"

    def test_sarasa7061(self): assert obtener_direccion(lista, '3 de febrero 196.') == "3 de febrero 196"

    def test_sarasa7062(self): assert obtener_direccion(lista, 'Jujuy 2316 2c') == "Jujuy 2316"

    def test_sarasa7063(self): assert obtener_direccion(lista,
                                                        'Hola me mandas un taxi a Rafaela 6082') == "Rafaela 6082"

    def test_sarasa7064(self): assert obtener_direccion(lista, 'Mandarian taxi a Balcarce 1444') == "Balcarce 1444"

    def test_sarasa7065(self): assert obtener_direccion(lista, 'Quiero un auto cerrito 6432') == "cerrito 6432"

    def test_sarasa7066(self): assert obtener_direccion(lista,
                                                        'Me mandas un taxi Cerrito 5543 torre 28') == "Cerrito 5543 torre 28"

    def test_sarasa7067(self): assert obtener_direccion(lista, 'Pellegrini 6974\nRocio\nDelgado') == "Pellegrini 6974"

    def test_sarasa7068(self): assert obtener_direccion(lista,
                                                        'Me mandarias un movil a Republica Dominicana 535 bis a nombre de Maca') == "Republica Dominicana 535 bis"

    def test_sarasa7069(self): assert obtener_direccion(lista,
                                                        'Republica Dominicana 535 bis entre rben Dario y pasaje Franco') == "Republica Dominicana 535 bis"

    def test_sarasa7071(self): assert obtener_direccion(lista,
                                                        'Hola me enviarias un movil Schweitzer 7780 tira 24') == "Schweitzer 7780 tira 24"

    def test_sarasa7072(self): assert obtener_direccion(lista, 'Montevideo 630 7 b') == "Montevideo 630 7 b"

    def test_sarasa7073(self): assert obtener_direccion(lista,
                                                        '24 de septiembre 3019,(forni )') == "24 de septiembre 3019"

    def test_sarasa7074(self): assert obtener_direccion(lista,
                                                        'Schweitzer 7780 entre donado y colores tira 24 Melani Renison') == "Schweitzer 7780 entre donado y colores tira 24"

class TestBis(unittest.TestCase):
    def test_bis01(self): assert ordenar_bis(None) == -1
    def test_bis02(self): assert ordenar_bis([]) == -1
    def test_bis03(self): assert ordenar_bis('') == -1
    def test_bis04(self): assert ordenar_bis('Uruguay 2020') == 'uruguay 2020'
    def test_bis05(self): assert ordenar_bis('uruguay 2020 bis') == 'uruguay bis 2020'
    def test_bis06(self): assert ordenar_bis('uruguay bis 2020') == 'uruguay bis 2020'
    def test_bis07(self): assert ordenar_bis('uruguay BiS 2020') == 'uruguay bis 2020'
    def test_bis08(self): assert ordenar_bis('uruguay 2020 Bis') == 'uruguay bis 2020'
    def test_bis09(self): assert ordenar_bis('bis uruguay 2020') == 'bis uruguay 2020'
    def test_bis10(self): assert ordenar_bis('para uruguay 2020 bis') == 'para uruguay bis 2020'
    def test_bis11(self): assert ordenar_bis('regimiento 11 123 bis') == 'regimiento 11 bis 123'

 # /home/matias/workspace/buildoutweb.cpbot/bin/mi_python /home/matias/workspace/buildoutweb.cpbot/src/at.utiles/at/utiles/tests/test_direcciones.py
if __name__ == '__main__':
    unittest.main()
