# -*- coding: utf-8 -*-

import unittest
import datetime

from at.utiles import _
from at.utiles.fecha import Fecha


class TestFecha(unittest.TestCase):
    '''
        Testea métodos del archivo fecha.py
    '''

    def setUp(self):
        self.fecha = Fecha()

    def tearDown(self):
        pass

    def test_fecha_datetime(self):
        # caso correcto
        r = self.fecha.fecha_datetime('2010', '08', '17', '14', '05', '54')
        self.assertTrue(r)

        # caso correcto 2, le paso una fecha sin hora
        r = self.fecha.fecha_datetime('2010', '08', '17')
        self.assertTrue(r)

        # caso correcto 3, paso los parámetros como enteros
        r = self.fecha.fecha_datetime(2010, 8, 17, 14, 5, 54)
        self.assertTrue(r)

        # caso correcto 4, paso algunos parámetros como enteros
        # y otros como string
        r = self.fecha.fecha_datetime('2010', 8, 17, 14, '5', 54)
        self.assertTrue(r)

        # paso mal los parametros, le paso 68 segundos
        self.assertRaises(ValueError, self.fecha.fecha_datetime, 2010, 8, 17,
                                                                 14, 5, 68)

        # le paso un string que no es un número
        self.assertRaises(ValueError, self.fecha.fecha_datetime, 2010, 'asd',
                                                                17, 14, 5, 68)

    def test_separar_fecha(self):
        # caso correcto
        fecha = '2010-08-17 14:36:55'
        r = self.fecha.separar_fecha(fecha)
        self.assertTrue(r)

        # caso correcto 2, le paso la fecha sin la hora
        fecha = '2010-08-17'
        r = self.fecha.separar_fecha(fecha)
        self.assertTrue(r)

        # le paso la fecha en otro formato
        fecha = '17-08-2010 14:46:41'
        self.assertRaises(Exception, self.fecha.separar_fecha, fecha)

        # le paso la fecha en otro formato
        fecha = '2010/08/17'
        self.assertRaises(Exception, self.fecha.separar_fecha, fecha)

        # le paso la fecha en otro formato,
        # en este caso guarda toda la hora key 'hora'
        fecha = '2010-08-17 14.46.41'
        r = self.fecha.separar_fecha(fecha)
        self.assertTrue(r)

    def test_tiempo_esp(self):
        f1 = self.fecha.fecha_datetime('2010', '08', '30', '13', '22', '27')
        f2 = self.fecha.fecha_datetime('2009', '07', '29', '12', '31', '43')

        diferencia = f1 - f2

        r = self.fecha.tiempo_esp(diferencia)
        self.assertTrue(r)

    def test_obtener_dia_semana(self):
        # Domingo
        fecha = datetime.datetime(2010, 11, 7, 17, 29, 5)
        r = self.fecha.obtener_dia_semana(fecha)
        self.assertTrue(r == 1)

        # Lunes
        fecha = datetime.datetime(2010, 11, 8, 17, 29, 5)
        r = self.fecha.obtener_dia_semana(fecha)
        self.assertTrue(r == 2)

        # Martes
        fecha = datetime.datetime(2010, 11, 9, 17, 29, 5)
        r = self.fecha.obtener_dia_semana(fecha)
        self.assertTrue(r == 3)

        # Mi�rcoles
        fecha = datetime.datetime(2010, 11, 10, 17, 29, 5)
        r = self.fecha.obtener_dia_semana(fecha)
        self.assertTrue(r == 4)

        # Jueves
        fecha = datetime.datetime(2010, 11, 11, 0, 0, 0)
        r = self.fecha.obtener_dia_semana(fecha)
        self.assertTrue(r == 5)

        # Viernes
        fecha = datetime.datetime(2010, 11, 12)
        r = self.fecha.obtener_dia_semana(fecha)
        self.assertTrue(r == 6)

        # S�bado
        fecha = datetime.datetime(2010, 11, 13)
        r = self.fecha.obtener_dia_semana(fecha)
        self.assertTrue(r == 7)

    def test_pasar_a_date(self):
        # caso correcto
        fecha_str = '27/01/2011'
        r = self.fecha.pasar_a_date(fecha_str)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(r['Mensaje'] == datetime.date(2011, 1, 27))

        # caso incorrecto
        fecha_str = '2011/08/25'
        r = self.fecha.pasar_a_date(fecha_str)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(r['Mensaje'] == _(u'Fecha recibida no válida. El formato de la fecha debe ser dd/mm/yyyy.'))

    def test_obtener_fechas_pedidas(self):
        fecha_inicio = datetime.date(2011, 1, 4)
        fecha_fin = datetime.date(2011, 1, 6)

        r = self.fecha.obtener_fechas_pedidas(fecha_inicio, fecha_fin)
        self.assertTrue(len(r) == 3)

    def test_obtener_nombre_dia(self):
        fecha = datetime.date(2011, 3, 13)
        r = self.fecha.obtener_nombre_dia(fecha)
        self.assertTrue(r == _(u'Domingo'))

        fecha = datetime.date(2011, 3, 14)
        r = self.fecha.obtener_nombre_dia(fecha)
        self.assertTrue(r == _(u'Lunes'))

        fecha = datetime.date(2011, 3, 15)
        r = self.fecha.obtener_nombre_dia(fecha)
        self.assertTrue(r == _(u'Martes'))

        fecha = datetime.date(2011, 3, 16)
        r = self.fecha.obtener_nombre_dia(fecha)
        self.assertTrue(r == _(u'Miércoles'))

        fecha = datetime.date(2011, 3, 17)
        r = self.fecha.obtener_nombre_dia(fecha)
        self.assertTrue(r == _(u'Jueves'))

        fecha = datetime.date(2011, 3, 18)
        r = self.fecha.obtener_nombre_dia(fecha)
        self.assertTrue(r == _(u'Viernes'))

        fecha = datetime.date(2011, 3, 19)
        r = self.fecha.obtener_nombre_dia(fecha)
        self.assertTrue(r == _(u'Sábado'))

        # le paso un datetime
        fecha = datetime.datetime(2010, 11, 9, 17, 29, 5)
        r = self.fecha.obtener_nombre_dia(fecha)
        self.assertTrue(r == _(u'Martes'))

    def test_cambiar_formato(self):
        # caso correcto
        fecha_hora = '16/02/2012 16:00:00'
        r = self.fecha.cambiar_formato(fecha_hora)
        self.assertTrue(r == '2012-02-16 16:00:00')

        # le paso una fecha con el año abreviado
        fecha_hora = '07/05/12 11:30:00'
        r = self.fecha.cambiar_formato(fecha_hora)
        self.assertTrue(r == '2012-05-07 11:30:00')

    def test_obtener_nombre_mes(self):

        # correcto
        mes = 1
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Enero'))
        mes = 2
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Febrero'))
        mes = 3
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Marzo'))
        mes = 4
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Abril'))
        mes = 5
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Mayo'))
        mes = 6
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Junio'))
        mes = 7
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Julio'))
        mes = 8
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Agosto'))
        mes = 9
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Septiembre'))
        mes = 10
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Octubre'))
        mes = 11
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Noviembre'))
        mes = 12
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertTrue(r == _(u'Diciembre'))

        # incorrecto
        mes = 1
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Junio'))
        mes = 2
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Falso'))
        mes = 3
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Otro'))
        mes = 4
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Diciembre'))
        mes = 5
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Marzo'))
        mes = 6
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Julio'))
        mes = 7
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Junio'))
        mes = 8
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Octubre'))
        mes = 9
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Noviembre'))
        mes = 10
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Dieicmebrt'))
        mes = 11
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Abril'))
        mes = 12
        r = self.fecha.obtener_nombre_mes(mes)
        self.assertFalse(r == _(u'Enero'))

    def test_obtener_fechas_horas_pedidas(self):
        fecha_hora_desde = datetime.datetime(2012, 12, 6, 12, 0, 0)
        fecha_hora_hasta = datetime.datetime(2012, 12, 7, 14, 0, 0)
        orden = 'asc'
        rta = self.fecha.obtener_fechas_horas_pedidas(fecha_hora_desde,
                                                    fecha_hora_hasta, orden)
        self.assertTrue(rta == [datetime.datetime(2012, 12, 6, 12, 0),
                                datetime.datetime(2012, 12, 6, 13, 0),
                                datetime.datetime(2012, 12, 6, 14, 0),
                                datetime.datetime(2012, 12, 6, 15, 0),
                                datetime.datetime(2012, 12, 6, 16, 0),
                                datetime.datetime(2012, 12, 6, 17, 0),
                                datetime.datetime(2012, 12, 6, 18, 0),
                                datetime.datetime(2012, 12, 6, 19, 0),
                                datetime.datetime(2012, 12, 6, 20, 0),
                                datetime.datetime(2012, 12, 6, 21, 0),
                                datetime.datetime(2012, 12, 6, 22, 0),
                                datetime.datetime(2012, 12, 6, 23, 0),
                                datetime.datetime(2012, 12, 7, 0, 0),
                                datetime.datetime(2012, 12, 7, 1, 0),
                                datetime.datetime(2012, 12, 7, 2, 0),
                                datetime.datetime(2012, 12, 7, 3, 0),
                                datetime.datetime(2012, 12, 7, 4, 0),
                                datetime.datetime(2012, 12, 7, 5, 0),
                                datetime.datetime(2012, 12, 7, 6, 0),
                                datetime.datetime(2012, 12, 7, 7, 0),
                                datetime.datetime(2012, 12, 7, 8, 0),
                                datetime.datetime(2012, 12, 7, 9, 0),
                                datetime.datetime(2012, 12, 7, 10, 0),
                                datetime.datetime(2012, 12, 7, 11, 0),
                                datetime.datetime(2012, 12, 7, 12, 0),
                                datetime.datetime(2012, 12, 7, 13, 0),
                                datetime.datetime(2012, 12, 7, 14, 0)]
                        )

    def test_pasar_a_datetime(self):
        # Caso correcto
        fecha_hora = '06/09/2013 16:00:00'
        rta = self.fecha.pasar_a_datetime(fecha_hora)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == datetime.datetime(2013, 9, 6, 16, 0)
        )

        # Le paso mal el formato
        fecha_hora = '2013-09-06 16:00:00'
        rta = self.fecha.pasar_a_datetime(fecha_hora)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            _(u'Fecha recibida no v\xe1lida. El formato de la fecha debe ser dd/mm/yyyy.')
        )

        # Le paso una fecha sin hora
        fecha_hora = '06/09/2013'
        rta = self.fecha.pasar_a_datetime(fecha_hora)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            _(u'El formato de la fecha debe ser dd/mm/yyyy hh:mm:ss')
        )

        # Le paso una fecha vacía
        fecha_hora = ''
        rta = self.fecha.pasar_a_datetime(fecha_hora)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            _(u'El formato de la fecha debe ser dd/mm/yyyy hh:mm:ss')
        )

        # Le paso una fecha None
        fecha_hora = None
        rta = self.fecha.pasar_a_datetime(fecha_hora)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            _(u'El formato de la fecha debe ser dd/mm/yyyy hh:mm:ss')
        )

        # Le paso un datetime
        fecha_hora = datetime.datetime(2013, 9, 6, 16, 0)
        rta = self.fecha.pasar_a_datetime(fecha_hora)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            _(u'El formato de la fecha debe ser dd/mm/yyyy hh:mm:ss')
        )

    def test_timedelta_to_datetime(self):
        # Caso correcto
        fecha1 = datetime.datetime(2015, 4, 24, 10, 0, 0)
        fecha2 = datetime.datetime(2015, 4, 24, 11, 30, 20)
        dif_fecha_hora = fecha2 - fecha1
        rta = self.fecha.timedelta_to_datetime(dif_fecha_hora)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == datetime.datetime(1900, 1, 1, 1, 30, 20)
        )

        # Le paso un dato que no es un timedelta
        dif_fecha_hora = '00:11:32'
        rta = self.fecha.timedelta_to_datetime(dif_fecha_hora)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'La diferencia horaria no es válida')
        )

        # Le paso una diferencia horaria muy grande
        fecha1 = datetime.datetime(2015, 4, 24, 10, 0, 0)
        fecha2 = datetime.datetime(2015, 4, 25, 12, 30, 20)
        dif_fecha_hora = fecha2 - fecha1
        rta = self.fecha.timedelta_to_datetime(dif_fecha_hora)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'La diferencia entre fechas es muy alta')
        )

        # Le paso una diferencia horaria muy grande, con día negativo
        fecha1 = datetime.datetime(2015, 4, 24, 10, 0, 0)
        fecha2 = datetime.datetime(2015, 4, 25, 12, 30, 20)
        dif_fecha_hora = fecha1 - fecha2
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'La diferencia entre fechas es muy alta')
        )

        # Caso correcto, dos hora exactas de diferencia
        fecha1 = datetime.datetime(2015, 4, 24, 10, 0, 0)
        fecha2 = datetime.datetime(2015, 4, 24, 12, 0, 0)
        dif_fecha_hora = fecha2 - fecha1
        rta = self.fecha.timedelta_to_datetime(dif_fecha_hora)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == datetime.datetime(1900, 1, 1, 2, 0, 0)
        )

        # Caso correcto, 15 minutos exactos de diferencia
        fecha1 = datetime.datetime(2015, 4, 24, 10, 0, 0)
        fecha2 = datetime.datetime(2015, 4, 24, 10, 15, 0)
        dif_fecha_hora = fecha2 - fecha1
        rta = self.fecha.timedelta_to_datetime(dif_fecha_hora)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == datetime.datetime(1900, 1, 1, 0, 15, 0)
        )

        # Caso correcto, 40 segundos exactos de diferencia
        fecha1 = datetime.datetime(2015, 4, 24, 10, 0, 0)
        fecha2 = datetime.datetime(2015, 4, 24, 10, 0, 40)
        dif_fecha_hora = fecha2 - fecha1
        rta = self.fecha.timedelta_to_datetime(dif_fecha_hora)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == datetime.datetime(1900, 1, 1, 0, 0, 40)
        )

        # Caso correcto
        fecha1 = datetime.datetime(2015, 4, 24, 10, 50, 10)
        fecha2 = datetime.datetime(2015, 4, 24, 11, 0, 0)
        dif_fecha_hora = fecha2 - fecha1
        rta = self.fecha.timedelta_to_datetime(dif_fecha_hora)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == datetime.datetime(1900, 1, 1, 0, 9, 50)
        )

        # Caso correcto, el máximo de diferencia posible
        fecha1 = datetime.datetime(2015, 4, 24, 0, 0, 0)
        fecha2 = datetime.datetime(2015, 4, 24, 23, 59, 59)
        dif_fecha_hora = fecha2 - fecha1
        rta = self.fecha.timedelta_to_datetime(dif_fecha_hora)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == datetime.datetime(1900, 1, 1, 23, 59, 59)
        )

    def test_validar_horario(self):
        # Caso válido, no le paso una fecha y hora de test
        hora_desde = datetime.datetime(1900, 1, 1, 9, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 18, 0, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Está dentro del horario permitido')
        )

        # Caso válido, con una fecha y hora de test
        hora_desde = datetime.datetime(1900, 1, 1, 9, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 18, 0, 0)
        fecha_hora_test = datetime.datetime(2015, 8, 24, 16, 0, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta, fecha_hora_test
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Está dentro del horario permitido')
        )

        # Caso inválido, no le paso una fecha y hora de test
        hora_desde = datetime.datetime(1900, 1, 1, 19, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 23, 0, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No está dentro del horario permitido')
        )

        # Caso válido, con una fecha y hora de test
        hora_desde = datetime.datetime(1900, 1, 1, 19, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 23, 0, 0)
        fecha_hora_test = datetime.datetime(2015, 8, 24, 16, 0, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta, fecha_hora_test
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No está dentro del horario permitido')
        )

        # Caso válido, con la hora hasta menor a la hora desde
        hora_desde = datetime.datetime(1900, 1, 1, 12, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 4, 0, 0)
        fecha_hora_test = datetime.datetime(2015, 8, 25, 1, 0, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta, fecha_hora_test
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Está dentro del horario permitido')
        )

        # Caso inválido, con la hora hasta menor a la hora desde
        hora_desde = datetime.datetime(1900, 1, 1, 12, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 4, 0, 0)
        fecha_hora_test = datetime.datetime(2015, 8, 24, 5, 0, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta, fecha_hora_test
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No está dentro del horario permitido')
        )

        # Caso válido, le paso todo el día
        hora_desde = datetime.datetime(1900, 1, 1, 0, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 23, 59, 59)
        fecha_hora_test = datetime.datetime(2015, 8, 24, 5, 0, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta, fecha_hora_test
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Está dentro del horario permitido')
        )

        # Caso válido, rango corto
        hora_desde = datetime.datetime(1900, 1, 1, 5, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 6, 0, 0)
        fecha_hora_test = datetime.datetime(2015, 8, 24, 5, 30, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta, fecha_hora_test
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Está dentro del horario permitido')
        )

        # Caso inválido, rango corto
        hora_desde = datetime.datetime(1900, 1, 1, 5, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 6, 0, 0)
        fecha_hora_test = datetime.datetime(2015, 8, 24, 4, 0, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta, fecha_hora_test
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No está dentro del horario permitido')
        )

        # Caso particular pasado por el cliente 24hs
        # Con un horario dentro del rango
        hora_desde = datetime.datetime(1900, 1, 1, 17, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 4, 0, 0)
        fecha_hora_test = datetime.datetime(2015, 8, 26, 20, 0, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta, fecha_hora_test
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Está dentro del horario permitido')
        )

        # Caso particular pasado por el cliente 24hs
        # Le paso un horario fuera del rango
        hora_desde = datetime.datetime(1900, 1, 1, 17, 0, 0)
        hora_hasta = datetime.datetime(1900, 1, 1, 4, 0, 0)
        fecha_hora_test = datetime.datetime(2015, 8, 26, 16, 0, 0)
        rta = self.fecha.validar_horario(
            hora_desde, hora_hasta, fecha_hora_test
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No está dentro del horario permitido')
        )


def suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestFecha))
    return suite
