# -*- coding: utf-8 -*-

import unittest
from datetime import datetime

from at.utiles import _
from at.utiles.feriados import Feriados
from at.utiles.servidores import DateTimeHelper
from at.orm.abm import ABM


class TestFeriados(unittest.TestCase):

    def setUp(self):
        test_cp = {
            'user': 'frula',
            'passwd': 'changeme',
            'dbname': 'tests',
            'name': 'cp',
            'host': '192.168.0.51',
            'engine': 'mysql',
            'port': '3306',
        }

        class DateTimeHelperTest(DateTimeHelper):

            def __init__(self):
                DateTimeHelper.__init__(self)

            def get_now(self):
                return datetime(2019, 3, 5, 17, 30, 20)

        self.feriados = Feriados()
        self.feriados.cp = ABM(test_cp)
        self.feriados.dth = DateTimeHelperTest()

        self.feriados.cp.borrar('feriados')

    def tearDown(self):
        self.feriados.cp.borrar('feriados')

    def test_obtener_feriados(self):
        # No hay feriados
        rta = self.feriados.obtener()
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudieron obtener los días feriados')
        )

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '1900-01-01'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '1900-12-25'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '2018-03-04'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '2018-03-05'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '2019-03-04'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '2019-03-05'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '2020-03-04'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '2020-03-05'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        rta = self.feriados.obtener()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Dato']) == 4)
        self.assertTrue(rta['Dato'][0] == '1900-01-01')
        self.assertTrue(rta['Dato'][1] == '1900-12-25')
        self.assertTrue(rta['Dato'][2] == '2019-03-04')
        self.assertTrue(rta['Dato'][3] == '2019-03-05')

    def test_es_feriado(self):
        # No hay feriados cargados
        fecha = '2019-03-07'
        rta = self.feriados._Feriados__es_feriado(fecha)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudieron obtener los días feriados')
        )

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '1900-01-01'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '1900-12-25'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '2019-03-04'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla feriados
        dic = {}
        dic['Fecha'] = '2019-03-05'
        agregar = self.feriados.cp.agregar('feriados', **dic)
        self.assertTrue(agregar is True)

        # Le paso un día que no es feriado
        fecha = '2019-03-07'
        rta = self.feriados._Feriados__es_feriado(fecha)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No es feriado'))

        # Le paso un día feriado
        fecha = '2019-03-04'
        rta = self.feriados._Feriados__es_feriado(fecha)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == _(u'Es feriado'))

        # Le paso un día feriado que se repite todos los años
        fecha = '2019-01-01'
        rta = self.feriados._Feriados__es_feriado(fecha)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == _(u'Es feriado'))

        # Me fijo si la fecha actual es feriado o no
        # Esta seteada la fecha fija en "2019-03-05", feriado por carnaval
        rta = self.feriados.es_feriado()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == _(u'Es feriado'))


def suite():
    from unittest import TestSuite
    from unittest import makeSuite
    test_suite = TestSuite()
    test_suite.addTest(makeSuite(TestFeriados))
    return test_suite
