# -*- coding: utf-8 -*-

import unittest

from at.utiles import _
from at.utiles.flotas import Flotas
from at.orm.abm import ABM


cp = {'user': 'frula',
      'passwd': 'changeme',
      'dbname': 'tests',
      'name': 'cp',
      'host': '192.168.0.51',
      'engine': 'mysql',
      'port': '3306',
      }


class FlotasTesting(Flotas):

    def __init__(self):
        Flotas.__init__(self)
        self.cp = ABM(cp)


class TestFlotas(unittest.TestCase):

    def setUp(self):
        self.ft = FlotasTesting()

        self.ft.cp.borrar('flotas')

    def tearDown(self):
        self.ft.cp.borrar('flotas')

    def test_flotas_conectadas(self):
        # Agrego una flota que no está conectada
        dic = {}
        dic['IdFlota'] = 80
        dic['Nombre'] = 'Flota 80'
        dic['Conectada'] = 0
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # No hay flotas conectadas
        rta = self.ft.conectadas()
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
            _(u'No se encontraron flotas conectadas')
        )

        # Agrego una flota conectada
        dic = {}
        dic['IdFlota'] = 60
        dic['Nombre'] = 'Flota 60'
        dic['Conectada'] = 1
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Agrego otra flota conectada
        dic = {}
        dic['IdFlota'] = 50
        dic['Nombre'] = 'Flota 50'
        dic['Conectada'] = 1
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Agrego una flota que no está conectada
        dic = {}
        dic['IdFlota'] = 70
        dic['Nombre'] = 'Flota 70'
        dic['Conectada'] = 0
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        rta = self.ft.conectadas()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'][0] == 50)
        self.assertTrue(rta['Respuesta'][1] == 60)

    def test_listar_flotas(self):
        # No existe flotas
        rta = self.ft.listar()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == [])

        # Agrego una flota
        dic = {}
        dic['IdFlota'] = 2
        dic['Nombre'] = 'Taxi del Sol'
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Agrego otra flota
        dic = {}
        dic['IdFlota'] = 1
        dic['Nombre'] = '380 Mil'
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Agrego otra flota
        dic = {}
        dic['IdFlota'] = 3
        dic['Nombre'] = u'Canción'
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Agrego otra flota
        dic = {}
        dic['IdFlota'] = 4
        dic['Nombre'] = ''
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        rta = self.ft.listar()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'][0]['id_flota'] == 1)
        self.assertTrue(rta['Respuesta'][0]['nombre'] == '380 Mil')
        self.assertTrue(rta['Respuesta'][1]['id_flota'] == 2)
        self.assertTrue(rta['Respuesta'][1]['nombre'] == 'Taxi del Sol')
        self.assertTrue(rta['Respuesta'][2]['id_flota'] == 3)
        self.assertTrue(rta['Respuesta'][2]['nombre'] == u'Canción')
        self.assertTrue(rta['Respuesta'][3]['id_flota'] == 4)
        self.assertTrue(rta['Respuesta'][3]['nombre'] == '')

    def test_obtener_flota(self):
        # Agrego una flota
        dic = {}
        dic['IdFlota'] = 96
        dic['Nombre'] = 'Flota 96'
        dic['Conectada'] = 1
        dic['Tipo'] = 1
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Agrego una flota
        dic = {}
        dic['IdFlota'] = 113
        dic['Nombre'] = 'Flota 113'
        dic['Conectada'] = 1
        dic['Tipo'] = 2
        dic['id_cliente'] = 128
        dic['id_sucursal'] = 1
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        id_flota = 96
        rta = self.ft.obtener(id_flota)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 13)
        self.assertTrue(rta['Respuesta']['id_flota'] == 96)
        self.assertTrue(rta['Respuesta']['nombre'] == 'Flota 96')
        self.assertTrue(rta['Respuesta']['ult_conexion'] is not None)
        self.assertTrue(rta['Respuesta']['orden'] == 0)
        self.assertTrue(rta['Respuesta']['latitud'] is None)
        self.assertTrue(rta['Respuesta']['longitud'] is None)
        self.assertTrue(rta['Respuesta']['id_cliente'] is None)
        self.assertTrue(rta['Respuesta']['id_sucursal'] is None)
        self.assertTrue(rta['Respuesta']['id_ciudad'] == 324)
        self.assertTrue(rta['Respuesta']['usuario_cliente'] == '')
        self.assertTrue(rta['Respuesta']['conectada'] == 1)
        self.assertTrue(rta['Respuesta']['tipo'] == 1)
        self.assertTrue(rta['Respuesta']['id_sucursal_empresa'] == 1)

        # Le paso una flota que no existe
        id_flota = 1234
        rta = self.ft.obtener(id_flota)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No se encontró la flota dada'))

        # Caso correcto filtrando por id_cliente e id_sucursal
        id_flota = None
        id_cliente = 128
        id_sucursal = 1
        rta = self.ft.obtener(id_flota, id_cliente, id_sucursal)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 13)
        self.assertTrue(rta['Respuesta']['id_flota'] == 113)
        self.assertTrue(rta['Respuesta']['nombre'] == 'Flota 113')
        self.assertTrue(rta['Respuesta']['ult_conexion'] is not None)
        self.assertTrue(rta['Respuesta']['orden'] == 0)
        self.assertTrue(rta['Respuesta']['latitud'] is None)
        self.assertTrue(rta['Respuesta']['longitud'] is None)
        self.assertTrue(rta['Respuesta']['id_cliente'] == 128)
        self.assertTrue(rta['Respuesta']['id_sucursal'] == 1)
        self.assertTrue(rta['Respuesta']['id_ciudad'] == 324)
        self.assertTrue(rta['Respuesta']['usuario_cliente'] == '')
        self.assertTrue(rta['Respuesta']['conectada'] == 1)
        self.assertTrue(rta['Respuesta']['tipo'] == 2)
        self.assertTrue(rta['Respuesta']['id_sucursal_empresa'] == 1)

        # Filtro por id_cliente e id_sucursal pero le paso datos que no existen
        id_flota = None
        id_cliente = 1234
        id_sucursal = 1
        rta = self.ft.obtener(id_flota, id_cliente, id_sucursal)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No se encontró la flota dada'))

    def test_obtener_fecha_hora_flota(self):
        try:
            import at.map
        except:
            print('>> warn: No hay at.map. Salteamos los tests.')
            return

        # Agrego una flota
        dic = {}
        dic['IdFlota'] = 97
        dic['Nombre'] = 'Flota 97'
        dic['IdCiudad'] = 324
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        id_flota = 97
        rta = self.ft.obtener_fecha_hora_flota(id_flota)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] is not None)

        # Le paso una flota que no existe
        id_flota = 1234
        rta = self.ft.obtener_fecha_hora_flota(id_flota)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No se encontró la flota dada'))

        # Agrego una flota con una ciudad que no existe
        dic = {}
        dic['IdFlota'] = 98
        dic['Nombre'] = 'Flota 98'
        dic['IdCiudad'] = 9999
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Le paso una flota con una ciudad que no existe
        id_flota = 98
        rta = self.ft.obtener_fecha_hora_flota(id_flota)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
                            _(u'No se pudo obtener el país de la ciudad dada'))

        # Agrego una flota de Santo Domingo
        dic = {}
        dic['IdFlota'] = 99
        dic['Nombre'] = 'Flota 99'
        dic['IdCiudad'] = 5008
        agregar = self.ft.cp.agregar('flotas', **dic)
        self.assertTrue(agregar is True)

        # Le paso una flota de Santo Domingo
        id_flota = 99
        rta = self.ft.obtener_fecha_hora_flota(id_flota)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] is not None)


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestFlotas))
    return suite
