# -*- coding: utf-8 -*-

import unittest
import datetime

from at.utiles.horalinux import ajustarsegundos
from at.orm.decorators import DBSwitch


# dame acceso a la DB
@DBSwitch('cp')
class db(object):
    pass

cp = db().cp


class TestAjustarSegundos(unittest.TestCase):

    @property
    def datenow(self):
        ''' Devuelvo mi datetime.datetime.now() '''
        date_now = datetime.datetime(2010, 11, 12, 15, 8, 37, 232907)
        return date_now

    @property
    def unixnow(self):
        ''' Reprecentacion en forma unixtime de mi datenow '''
        unix_now = '111215082010.37'
        return unix_now

    @property
    def zerodatenow(self):
        ''' Devuelvo mi datetime.datetime.now() '''
        date_now = datetime.datetime(2010, 11, 12, 15, 8, 00, 232907)
        return date_now

    @property
    def unixzeronow(self):
        ''' Reprecentacion en forma unixtime de mi zerodatenow '''
        unix_zero_now = '111215082010.00'
        return unix_zero_now

    @property
    def midnight(self):
        ''' devuelvo mi datetime de la madrugada '''
        date_now = datetime.datetime(2010, 11, 12, 00, 00, 00, 232907)
        return date_now

    @property
    def unixmidnight(self):
        ''' devuelvo unixtime de mi midnight '''
        unixmidnight = '111200002010.00'
        return unixmidnight

    @property
    def resetear(self):
        ''' resetea los campos AjustarHora a cero '''
        parametros = cp.tabla('parametros')
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': 0}
        q_obj.update(u)
        session.commit()
        session.close()

    def test_ajustar_segundos(self):
        ''' Voy y consulto la base de datos, si el campo AjustarHora
        de la tabla parametros es 0, obtengo None '''

        parametros = cp.tabla('parametros')
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()

        # aseguramos tener al menos un resultado ...
        self.assertTrue(len(q_obj) > 0)

        # tomo el primer registro, el resto no me importa y aseguro
        # que sea 0 :)
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == 0)

        # y bueno, llamemos a ajustarsegundos, devolveme None :)
        self.assertTrue(None == ajustarsegundos(debug=True))

        # y ahora te updeteo a 1 el campo AjustarHora, cosa de que luego
        # saltes :)
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': 1}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea 1 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == 1)

        # y paso mi q_obj por ajustarsegundos, deberia incrementar en
        # un segundo mi unixnow, debe retornarme esto en forma de string
        # en formato unix time
        # es mi unix time '111215082010.37'
        # retorname '111215082010.38'
        res = ajustarsegundos(debug=True, now=self.datenow)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111215082010.38')
        self.resetear

        # updeteo el campo AjustarHora a 2
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': 2}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea 2 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == 2)

        # y paso mi q_obj por ajustarsegundos, deberia incrementar en
        # dos segundos mi unixnow, debe retornarme esto en forma de
        # string en formato unix time
        # es mi unix time '111215082010.37'
        # retorname '111215082010.39'
        res = ajustarsegundos(debug=True, now=self.datenow)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111215082010.39')
        self.resetear

        # updeteo el campo AjustarHora a -2
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': -2}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea -2 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == -2)

        # y paso mi q_obj por ajustarsegundos, deberia decrementar en
        # dos segundos mi unixnow, debe retornarme esto en forma de
        # string en formato unix time
        # es mi unix time '111215082010.37'
        # retorname '111215082010.35'
        res = ajustarsegundos(debug=True, now=self.datenow)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111215082010.35')
        self.resetear

        # updeteo el campo AjustarHora a -4
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': -4}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea -4 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == -4)

        # y paso mi q_obj por ajustarsegundos, deberia decrementar en
        # cuatro segundos mi unixnow, debe retornarme esto en forma de
        # string en formato unix time
        # es mi unix time '111215082010.37'
        # retorname '111215082010.33'
        res = ajustarsegundos(debug=True, now=self.datenow)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111215082010.33')
        self.resetear

    def test_ajustar_segundos_hora_con_zero_segundos(self):
        ''' testeamos el ajuste de segundos cuando los mismos son cero
        en una hora determinada '''

        parametros = cp.tabla('parametros')

        # updeteo el campo AjustarHora a -2
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': -2}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea -2 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == -2)

        # y paso mi q_obj por ajustarsegundos, deberia decrementar en
        # dos segundos mi zerounixnow, debe retornarme esto en forma de
        # string en formato unix time
        # es mi unix time '111215082010.00'
        # retorname '111215072010.58'
        # esto demuestra el pasar al minuto anterior correctamente y que
        # los segundos sean tambien como corresponde
        res = ajustarsegundos(debug=True, now=self.zerodatenow)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111215072010.58')
        self.resetear

        # updeteo el campo AjustarHora a -1
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': -1}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea -1 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == -1)

        res = ajustarsegundos(debug=True, now=self.zerodatenow)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111215072010.59')
        self.resetear

        # updeteo el campo AjustarHora a 2
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': 2}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea 2 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == 2)

        res = ajustarsegundos(debug=True, now=self.zerodatenow)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111215082010.02')
        self.resetear

        # updeteo el campo AjustarHora a 1
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': 1}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea 1 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == 1)

        res = ajustarsegundos(debug=True, now=self.zerodatenow)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111215082010.01')
        self.resetear

    def test_ajustar_segundos_midnight(self):

        parametros = cp.tabla('parametros')

        # updeteo el campo AjustarHora a -2
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': -2}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea -2 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == -2)

        # y paso mi q_obj por ajustarsegundos, deberia decrementar en
        # dos segundos mi zeromidnight, debe retornarme esto en forma de
        # string en formato unix time
        # es mi unix time '111200002010.00'
        # retorname '111223592010.58'
        # esto demuestra el pasar a la hora anterior correctamente y que
        # los segundos sean tambien como corresponde y que cambia de dia
        # como corresponde, osea al dia anterior
        res = ajustarsegundos(debug=True, now=self.midnight)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111123592010.58')
        self.resetear

        # updeteo el campo AjustarHora a -1
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': -1}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea -1 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == -1)

        # y paso mi q_obj por ajustarsegundos, deberia decrementar en
        # un segundo mi zeromidnight, debe retornarme esto en forma de
        # string en formato unix time
        # es mi unix time '111200002010.00'
        # retorname '111223592010.59'
        # esto demuestra el pasar a la hora anterior correctamente y que
        # los segundos sean tambien como corresponde y que cambia de dia
        # como corresponde, osea al dia anterior
        res = ajustarsegundos(debug=True, now=self.midnight)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111123592010.59')
        self.resetear

        # updeteo el campo AjustarHora a 1
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': 1}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea 1 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == 1)

        # y paso mi q_obj por ajustarsegundos, deberia decrementar en
        # un segundo mi zeromidnight, debe retornarme esto en forma de
        # string en formato unix time
        # es mi unix time '111200002010.00'
        # retorname '111200002010.01'
        # esto demuestra el pasar a la hora anterior correctamente y que
        # los segundos sean tambien como corresponde y que cambia de dia
        # como corresponde, osea al dia anterior
        res = ajustarsegundos(debug=True, now=self.midnight)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111200002010.01')
        self.resetear

        # updeteo el campo AjustarHora a 2
        session = cp.binded_session()
        q_obj = session.query(parametros)
        u = {'AjustarHora': 2}
        q_obj.update(u)
        session.commit()
        session.close()

        # me aseguro que ahora sea 2 el campo AjustarHora
        session = cp.binded_session()
        q_obj = session.query(parametros).all()
        session.close()
        q_obj = q_obj[0].AjustarHora
        self.assertTrue(q_obj == 2)

        # y paso mi q_obj por ajustarsegundos, deberia decrementar en
        # dos segundos mi zeromidnight, debe retornarme esto en forma de
        # string en formato unix time
        # es mi unix time '111200002010.00'
        # retorname '111200002010.02'
        # esto demuestra el pasar a la hora anterior correctamente y que
        # los segundos sean tambien como corresponde y que cambia de dia
        # como corresponde, osea al dia anterior
        res = ajustarsegundos(debug=True, now=self.midnight)
        self.assertTrue(isinstance(res, str))
        self.assertTrue(res == '111200002010.02')
        self.resetear


def suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestAjustarSegundos))
    return suite
