# -*- coding: utf-8 -*-

import unittest

from at.utiles.http import enviar_por_http


class TestHttp(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_enviar_por_http(self):
        # Caso correcto
        url = 'https://www.google.com'
        params = {}
        rta = enviar_por_http(url, params, 'GET')
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(isinstance(rta['Respuesta'], dict))

        # Le paso una URL inválida
        url = 'asd'
        params = {}
        rta = enviar_por_http(url, params, 'GET')
        # El error se guarda en un LOG
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == 'Only absolute URIs are allowed. uri = asd'
        )

        # Caso con una URL local y con un puerto
        url = 'http://192.168.0.57:8001/at.utiles'
        params = {}
        rta = enviar_por_http(url, params, 'GET')
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(isinstance(rta['Respuesta'], dict))

        # Le paso mal el tipo, me retorna error 301
        url = 'http://192.168.0.57:8001/at.utiles'
        params = {}
        rta = enviar_por_http(url, params, 'qwe')
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(isinstance(rta['Respuesta'], dict))

        # Prueba de Whatsapp, dice que el browser no es válido
        url = 'https://web.whatsapp.com/'
        params = {}
        # NOTA: Le paso otros headers para poder decirle que vamos a usar
        # un Chrome nuevo para que sea válida la request
        headers = {}
        headers['Content-type'] = 'application/x-www-form-urlencoded'
        headers['User-Agent'] = 'Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36'
        rta = enviar_por_http(url, params, 'GET', headers=headers)
        #print(rta)


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestHttp))
    return suite
