# -*- coding: utf-8 -*-

import unittest

from at.utiles import _
from at.utiles.mail import Mail
from at.orm.abm import ABM


class MailTesting(Mail):

    def __init__(self):
        Mail.__init__(self)

        cp = {
            'user': 'frula',
            'passwd': 'changeme',
            'dbname': 'tests',
            'name': 'cp',
            'host': '192.168.0.51',
            'engine': 'mysql',
            'port': '3306',
        }
        self.cp = ABM(cp)


class TestMail(unittest.TestCase):

    def setUp(self):
        self.mt = MailTesting()

        self.mt.cp.borrar('configuraciones_mails')

    def tearDown(self):
        self.mt.cp.borrar('configuraciones_mails')

    def test_enviar_mail(self):
        # Caso correcto
        servidor = "smtp.gmail.com"
        puerto = 587
        usuario = "webs@airtrack.com.ar"
        password = "6gURFJj0]T"
        de = ['test']
        para = ['jfr@airtrack.com.ar']
        asunto = 'Mail de prueba'
        mensaje = 'El mensaje del mail de prueba'
        test = True
        rta = self.mt.enviar_mail(
            servidor, puerto, usuario, password, de, para, asunto, mensaje,
            test=test
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Mensaje'] == u'No envío el mail porque estoy en un test'
        )

        # Le paso un mail que no existe
        servidor = "smtp.gmail.com"
        puerto = 587
        usuario = "webs@airtrack.com.ar"
        password = "6gURFJj0]T"
        de = ['test']
        para = ['no_existe@airtrack.com.ar']
        asunto = 'Asunto no existe el mail'
        mensaje = 'Mensaje no existe el mail'
        test = True
        rta = self.mt.enviar_mail(
            servidor, puerto, usuario, password, de, para, asunto, mensaje,
            test=test
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Mensaje'] == u'No envío el mail porque estoy en un test'
        )

    def test_grabar_configuracion_mail(self):
        # Agrego una configuración de correo
        id_sucursal = 1
        servidor_smtp = 'servidor_smtp'
        puerto = 8563
        usuario = 'el_usuario'
        password = 'el_password'
        rta = self.mt.grabar_configuracion_mail(
            id_sucursal, servidor_smtp, puerto, usuario, password
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se guardó correctamente la configuración del correo de la sucursal 1')
        )

        # Modifico una configuración de correo
        id_sucursal = 1
        servidor_smtp = 'servidor_smtp_mod'
        puerto = 8569
        usuario = 'el_usuario_mod'
        password = 'el_password_mod'
        rta = self.mt.grabar_configuracion_mail(
            id_sucursal, servidor_smtp, puerto, usuario, password
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se modificó correctamente la configuración del correo de la sucursal 1')
        )

    def test_listar_configuraciones_mails(self):
        # No hay ninguna configuración de mail cargada
        rta = self.mt.listar_configuraciones_mails()
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se encontraron configuraciones de mails cargadas')
        )

        # Agrego una configuración de mail
        dic = {}
        dic['IdSucursal'] = 1
        dic['ServidorSmtp'] = 'servidor_smtp_1'
        dic['Puerto'] = 1111
        dic['Usuario'] = 'usuario_1'
        dic['Password'] = 'password_1'
        agregar = self.mt.cp.agregar('configuraciones_mails', **dic)
        self.assertTrue(agregar is True)

        # Agrego otra configuración de mail
        dic = {}
        dic['IdSucursal'] = 2
        dic['ServidorSmtp'] = 'servidor_smtp_2'
        dic['Puerto'] = 2222
        dic['Usuario'] = 'usuario_2'
        dic['Password'] = 'password_2'
        agregar = self.mt.cp.agregar('configuraciones_mails', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        rta = self.mt.listar_configuraciones_mails()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 2)

        self.assertTrue(len(rta['Respuesta'][0]) == 5)
        self.assertTrue(rta['Respuesta'][0][0] == 1)
        self.assertTrue(rta['Respuesta'][0][1] == 'usuario_1')
        self.assertTrue(rta['Respuesta'][0][2] == 'servidor_smtp_1')
        self.assertTrue(rta['Respuesta'][0][3] == 1111)
        self.assertTrue(rta['Respuesta'][0][4] == 'password_1')

        self.assertTrue(len(rta['Respuesta'][1]) == 5)
        self.assertTrue(rta['Respuesta'][1][0] == 2)
        self.assertTrue(rta['Respuesta'][1][1] == 'usuario_2')
        self.assertTrue(rta['Respuesta'][1][2] == 'servidor_smtp_2')
        self.assertTrue(rta['Respuesta'][1][3] == 2222)
        self.assertTrue(rta['Respuesta'][1][4] == 'password_2')

    def test_borrar_configuracion_mail(self):
        # Agrego una configuración de mail
        dic = {}
        dic['IdSucursal'] = 1
        dic['ServidorSmtp'] = 'servidor_smtp_1'
        dic['Puerto'] = 1111
        dic['Usuario'] = 'usuario_1'
        dic['Password'] = 'password_1'
        agregar = self.mt.cp.agregar('configuraciones_mails', **dic)
        self.assertTrue(agregar is True)

        # Agrego otra configuración de mail
        dic = {}
        dic['IdSucursal'] = 2
        dic['ServidorSmtp'] = 'servidor_smtp_2'
        dic['Puerto'] = 2222
        dic['Usuario'] = 'usuario_2'
        dic['Password'] = 'password_2'
        agregar = self.mt.cp.agregar('configuraciones_mails', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        id_sucursal = 1
        rta = self.mt.borrar_configuracion_mail(id_sucursal)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se eliminó correctamente la configuración de correo de la sucursal 1')
        )

        # Le paso una configuración que no existe
        id_sucursal = 1234
        rta = self.mt.borrar_configuracion_mail(id_sucursal)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se encontró la configuración de correo de la sucursal 1234')
        )

    def test_obtener_configuracion_mail(self):
        # Agrego una configuración de mail
        dic = {}
        dic['IdSucursal'] = 1
        dic['ServidorSmtp'] = 'servidor_smtp_1'
        dic['Puerto'] = 1111
        dic['Usuario'] = 'usuario_1'
        dic['Password'] = 'password_1'
        agregar = self.mt.cp.agregar('configuraciones_mails', **dic)
        self.assertTrue(agregar is True)

        # Agrego otra configuración de mail
        dic = {}
        dic['IdSucursal'] = 2
        dic['ServidorSmtp'] = 'servidor_smtp_2'
        dic['Puerto'] = 2222
        dic['Usuario'] = 'usuario_2'
        dic['Password'] = 'password_2'
        agregar = self.mt.cp.agregar('configuraciones_mails', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        id_sucursal = 1
        rta = self.mt.obtener_configuracion_mail(id_sucursal)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta'].keys()) == 5)
        self.assertTrue(rta['Respuesta']['id_sucursal'] == 1)
        self.assertTrue(rta['Respuesta']['servidor_smtp'] == 'servidor_smtp_1')
        self.assertTrue(rta['Respuesta']['puerto'] == 1111)
        self.assertTrue(rta['Respuesta']['usuario'] == 'usuario_1')
        self.assertTrue(rta['Respuesta']['password'] == 'password_1')

        # Le paso una configuración de mail que no existe
        id_sucursal = 1234
        rta = self.mt.obtener_configuracion_mail(id_sucursal)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se encontró la configuración de correo de la sucursal 1234')
        )


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMail))
    return suite
