# -*- coding: utf-8 -*-

import unittest

from at.utiles import _
from at.utiles.mensajes import Mensajes
from at.orm.abm import ABM


cp = {'user': 'frula',
      'passwd': 'changeme',
      'dbname': 'tests',
      'name': 'cp',
      'host': '192.168.0.51',
      'engine': 'mysql',
      'port': '3306',
      }


class MensajesTesting(Mensajes):

    def __init__(self):
        Mensajes.__init__(self)
        self.cp = ABM(cp)
        self.usuarios.cp = ABM(cp)
        self.numerador.cp = ABM(cp)


class TestMensajes(unittest.TestCase):

    def setUp(self):
        self.mt = MensajesTesting()

        self.mt.cp.borrar('mensajes')
        self.mt.cp.borrar('usuarios')
        self.mt.cp.borrar('sms')

    def tearDown(self):
        self.mt.cp.borrar('mensajes')
        self.mt.cp.borrar('usuarios')
        self.mt.cp.borrar('sms')

    def test_enviar_a_usuarios(self):
        # Le paso los permisos pero no hay usuarios cargados
        remitente = 'PTT'
        mensaje = u'mensaje de prueba con canción 3'
        destinatarios = []
        permisos = ['A', 'O']
        rta = self.mt.enviar_a_usuarios(
                                remitente, mensaje, destinatarios, permisos)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No hay usuarios cargados'))

        # Agrego un usuario
        dic = {}
        dic['Usuario'] = 'tele1'
        dic['Permisos'] = 'T'
        agregar = self.mt.cp.agregar('usuarios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un usuario
        dic = {}
        dic['Usuario'] = 'tele2'
        dic['Permisos'] = 'T'
        agregar = self.mt.cp.agregar('usuarios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un usuario
        dic = {}
        dic['Usuario'] = 'ope1'
        dic['Permisos'] = 'O'
        agregar = self.mt.cp.agregar('usuarios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un usuario
        dic = {}
        dic['Usuario'] = 'ope2'
        dic['Permisos'] = 'O'
        agregar = self.mt.cp.agregar('usuarios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un usuario
        dic = {}
        dic['Usuario'] = 'tele_ope1'
        dic['Permisos'] = 'TO'
        agregar = self.mt.cp.agregar('usuarios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un usuario
        dic = {}
        dic['Usuario'] = 'tele_ope2'
        dic['Permisos'] = 'TO'
        agregar = self.mt.cp.agregar('usuarios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un usuario
        dic = {}
        dic['Usuario'] = 'admin1'
        dic['Permisos'] = 'A'
        agregar = self.mt.cp.agregar('usuarios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un usuario
        dic = {}
        dic['Usuario'] = 'admin2'
        dic['Permisos'] = 'A'
        agregar = self.mt.cp.agregar('usuarios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un usuario
        dic = {}
        dic['Usuario'] = 'encargado1'
        dic['Permisos'] = 'E'
        agregar = self.mt.cp.agregar('usuarios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un usuario
        dic = {}
        dic['Usuario'] = 'encargado2'
        dic['Permisos'] = 'E'
        agregar = self.mt.cp.agregar('usuarios', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto, le paso los usuarios
        remitente = 'PTT'
        mensaje = u'mensaje de prueba con canción'
        destinatarios = ['leo', 'paco']
        permisos = None
        rta = self.mt.enviar_a_usuarios(
                                remitente, mensaje, destinatarios, permisos)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] ==
                                    _(u'Se grabó correctamente el mensaje'))

        # Caso correcto, le paso una lista de permisos
        remitente = 'PTT'
        mensaje = u'mensaje de prueba con canción 2'
        destinatarios = []
        permisos = ['A', 'O']
        tipo = 'ptt'
        rta = self.mt.enviar_a_usuarios(
                            remitente, mensaje, destinatarios, permisos, tipo)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] ==
                                    _(u'Se grabó correctamente el mensaje'))

        # Le paso un tipo de permiso que no existe
        remitente = 'PTT'
        mensaje = u'mensaje de prueba con canción 4'
        destinatarios = []
        permisos = ['X']
        rta = self.mt.enviar_a_usuarios(
                                remitente, mensaje, destinatarios, permisos)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
                            _(u'No existen usuarios con los permisos dados'))

        # Le paso en minuscula el permiso
        remitente = 'PTT'
        mensaje = u'mensaje de prueba con canción 5'
        destinatarios = None
        permisos = ['a']
        rta = self.mt.enviar_a_usuarios(
                                remitente, mensaje, destinatarios, permisos)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
                            _(u'No existen usuarios con los permisos dados'))

        # No le paso ni destinatarios ni permisos
        remitente = 'PTT'
        mensaje = u'mensaje de prueba con canción 6'
        destinatarios = None
        permisos = None
        rta = self.mt.enviar_a_usuarios(
                                remitente, mensaje, destinatarios, permisos)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
                                _(u'El mensaje no se envió a ningún usuario'))

    def test_guardar_sms(self):
        # Caso correcto
        telefono = '153705959'
        mensaje = 'el mensaje para el SMS'
        rta = self.mt.guardar_sms(telefono, mensaje)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == _(u'Se guardó el SMS'))


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMensajes))
    return suite
