# -*- coding: utf-8 -*-

import unittest
from at.utiles import _
from at.utiles.parametros import Parametros
from at.orm.abm import ABM


cp = {'user': 'frula',
      'passwd': 'changeme',
      'dbname': 'tests',
      'name': 'cp',
      'host': '192.168.0.51',
      'engine': 'mysql',
      'port': '3306',
      }


class ParametrosTesting(Parametros):

    def __init__(self):
        Parametros.__init__(self)
        self.cp = ABM(cp)


class TestParametros(unittest.TestCase):

    def setUp(self):
        self.parametros_t = ParametrosTesting()

    def tearDown(self):
        pass

    def test_obtener_parametro(self):
        # caso correcto
        parametro = 'ReproducirZonasPorVoz'
        id_sucursal = 1
        resultado = self.parametros_t.obtener_parametro(parametro, id_sucursal)
        self.assertTrue(resultado is not None)

        # caso correcto sin pasarle una sucursal
        parametro = 'ReproducirZonasPorVoz'
        resultado = self.parametros_t.obtener_parametro(parametro)
        self.assertTrue(resultado is not None)

        # le paso una sucursal que no existe
        id_sucursal = 1234
        nombre_parametro = 'ReproducirZonasPorVoz'
        resultado = self.parametros_t.obtener_parametro(nombre_parametro,
                                                        id_sucursal)
        self.assertTrue(resultado is None)

        # le paso un parametro que no existe
        id_sucursal = 1
        nombre_parametro = 'asd'
        resultado = self.parametros_t.obtener_parametro(nombre_parametro,
                                                        id_sucursal)
        self.assertTrue(resultado is None)

    def test_actualizar_parametros(self):
        # caso correcto
        id_sucursal = 1
        actualizar = {}
        actualizar['Firma1'] = 'prueba test_actualizar_parametros 1'
        actualizar['Firma2'] = 'prueba test_actualizar_parametros 2'
        resultado = self.parametros_t.actualizar_parametros(
                                                    actualizar,
                                                    id_sucursal)
        self.assertTrue(resultado['Estado'] is True)
        self.assertTrue(resultado['Respuesta'] ==
                                    _(u'Se han actualizado los parámetros'))

        # le paso una sucursal que no existe
        id_sucursal = 1234
        actualizar = {}
        actualizar['Firma1'] = 'le paso una sucursal que no existe 1'
        actualizar['Firma2'] = 'le paso una sucursal que no existe 2'
        resultado = self.parametros_t.actualizar_parametros(
                                                    actualizar,
                                                    id_sucursal)
        self.assertTrue(resultado['Estado'] is False)
        self.assertTrue(resultado['Respuesta'] ==
                        _(u'No hay parámetros cargados para la sucursal 1234'))

        # le paso un campo a actualizar que no existe
        id_sucursal = 1
        actualizar = {}
        actualizar['Firma2'] = 'no cambio la firma 2'
        actualizar['noexiste'] = 'asd'
        resultado = self.parametros_t.actualizar_parametros(
                                                    actualizar,
                                                    id_sucursal)
        self.assertTrue(resultado['Estado'] is False)
        self.assertTrue(resultado['Respuesta'] ==
                    _(u'No existe el campo noexiste en la tabla parametros'))

    def test_obtener_parametrolocal(self):
        # caso correcto
        nombre_parametro = 'IpLogs'
        resultado = self.parametros_t.obtener_parametrolocal(nombre_parametro)
        self.assertTrue(isinstance(resultado, str))

        # caso correcto
        nombre_parametro = 'CantDiasPrioridades'
        resultado = self.parametros_t.obtener_parametrolocal(nombre_parametro)
        self.assertTrue(resultado is not None)

        # le paso un parametro que no existe
        nombre_parametro = 'noexiste'
        resultado = self.parametros_t.obtener_parametrolocal(nombre_parametro)
        self.assertTrue(resultado is None)

    def test_obtener_parametros(self):
        # le paso una sucursal que no existe
        id_sucursal = 1234
        resultado = self.parametros_t.obtener_parametros(id_sucursal)
        self.assertTrue(resultado['Estado'] is False)
        self.assertTrue(resultado['Respuesta'] ==
                        _(u'No hay parámetros cargados para la sucursal 1234'))

        # caso correcto
        id_sucursal = 1
        resultado = self.parametros_t.obtener_parametros(id_sucursal)
        self.assertTrue(resultado['Estado'] is True)
        self.assertTrue(resultado['Respuesta']['idSucursal'] == 1)
        self.assertTrue(resultado['Respuesta']['HabilitarMapas'] == 1)
        self.assertTrue(resultado['Respuesta']['HabilitarMensajesDeVoz'] == 1)
        self.assertTrue(resultado['Respuesta']['CantMovMensajesDeVoz'] >= 0)

    def test_despacho_automatico(self):
        # le paso una sucursal que no existe
        id_sucursal = 1234
        resultado = self.parametros_t.despacho_automatico(id_sucursal)
        self.assertTrue(resultado == 0)

        # caso correcto
        id_sucursal = 1
        resultado = self.parametros_t.despacho_automatico(id_sucursal)
        self.assertTrue(resultado == 0)

    def test_actualizar_parametroslocales(self):
        # caso correcto
        actualizar = {}
        actualizar['Logo'] = 'el logo va aca'
        actualizar['NumeraSemanaXAnio'] = 1
        resultado = self.parametros_t.actualizar_parametroslocales(actualizar)
        self.assertTrue(resultado['Estado'] is True)
        self.assertTrue(resultado['Respuesta'] ==
                            _(u'Se han actualizado los parámetros locales'))

        ## le paso un id que no existe
        ide = 1234
        actualizar = {}
        actualizar['Logo'] = 'el logo va aca'
        actualizar['NumeraSemanaXAnio'] = 1
        resultado = self.parametros_t.actualizar_parametroslocales(
                                                            actualizar, ide)
        self.assertTrue(resultado['Estado'] is False)
        self.assertTrue(resultado['Respuesta'] ==
                                    _(u'No hay parámetros locales cargados'))

        # le paso un campo a actualizar que no existe
        ide = 1
        actualizar = {}
        actualizar['Logo'] = 'el logo va aca 2'
        actualizar['noexiste'] = 'asd'
        resultado = self.parametros_t.actualizar_parametroslocales(
                                                            actualizar, ide)
        self.assertTrue(resultado['Estado'] is False)
        self.assertTrue(resultado['Respuesta'] ==
            _(u'No existe el campo noexiste en la tabla parametroslocales'))

    def test_obtener_parametroslocales(self):
        rta = self.parametros_t.obtener_parametroslocales()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['IpLogs'] == '0.0.0.0')
        self.assertTrue(rta['Respuesta']['PermitirConfigurarCpIvr'] == 0)


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestParametros))
    return suite
