# -*- coding: utf-8 -*-

import unittest

from at.utiles import _
from at.utiles.servicios import Servicios
from at.orm.abm import ABM


cp = {
    'user': 'frula',
    'passwd': 'changeme',
    'dbname': 'tests',
    'name': 'cp',
    'host': '192.168.0.51',
    'engine': 'mysql',
    'port': '3306',
}


class TestServicios(unittest.TestCase):

    def setUp(self):
        self.st = Servicios()
        self.st.cp = ABM(cp)

        self.st.cp.borrar('actual')
        self.st.cp.borrar('historico')
        self.st.cp.borrar('logbs')
        self.st.cp.borrar('moviles')

    def tearDown(self):
        self.st.cp.borrar('actual')
        self.st.cp.borrar('historico')
        self.st.cp.borrar('logbs')
        self.st.cp.borrar('moviles')

    def test_id_servicio_autoincrement(self):
        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = 'tel1'
        dic['FechaServicio'] = '2014-10-03'
        dic['HoraServicio'] = '1900-01-01 14:10:00'
        dic['IdServicio'] = 0
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = 'tel2'
        dic['FechaServicio'] = '2014-10-03'
        dic['HoraServicio'] = '1900-01-01 14:20:00'
        dic['IdServicio'] = 0
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = 'tel3'
        dic['FechaServicio'] = '2014-10-03'
        dic['HoraServicio'] = '1900-01-01 14:30:00'
        dic['IdServicio'] = 0
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = 'tel4'
        dic['FechaServicio'] = '2014-10-02'
        dic['HoraServicio'] = '1900-01-01 19:00:00'
        dic['IdServicio'] = 0
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = 'tel1'
        dic['FechaServicio'] = '2014-10-02'
        dic['HoraServicio'] = '1900-01-01 14:10:00'
        dic['IdServicio'] = 0
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Le paso un último id como parámetros
        ult_id_servicio = 1026
        rta = self.st.id_servicio_autoincrement(ult_id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se actualizo correctamente el id de los servicios')
        )

    def test_obtener_servicios_por_telefono(self):
        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '153705967'
        dic['FechaServicio'] = '2015-05-26'
        dic['HoraServicio'] = '1900-01-01 17:00:00'
        dic['IdServicio'] = 60
        dic['Telefonista'] = 'PTT'
        dic['Direccion'] = u'España 1130'
        dic['Nombre'] = u'Martín'
        dic['Latitud'] = -118560.9456
        dic['Longitud'] = -218585.8188
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '0341155403020'
        dic['FechaServicio'] = '2015-05-29'
        dic['HoraServicio'] = '1900-01-01 12:00:00'
        dic['IdServicio'] = 61
        dic['Telefonista'] = 'CpIvr'
        dic['Direccion'] = u'Dorrego 1800'
        dic['Nombre'] = 'Juan'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '0341155403099'
        dic['FechaServicio'] = '2015-05-29'
        dic['HoraServicio'] = '1900-01-01 12:30:00'
        dic['IdServicio'] = 62
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un teléfono en unicode
        dic = {}
        dic['Telefono'] = u'canción'
        dic['FechaServicio'] = '2015-05-29'
        dic['HoraServicio'] = '1900-01-01 14:00:00'
        dic['IdServicio'] = 63
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Le paso un teléfono con 5 caracteres
        telefono = '12345'
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El teléfono debe tener como mínimo 6 caracteres para buscarlo')
        )

        # Pruebo con un unicode
        telefono = u'ñon'
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El teléfono debe tener como mínimo 6 caracteres para buscarlo')
        )

        # Le paso un teléfono que no es un string
        telefono = 1563
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El teléfono debe ser de tipo string o unicode')
        )

        # Le paso un teléfono que no es un string
        telefono = None
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El teléfono debe ser de tipo string o unicode')
        )

        # Le paso un string vacío
        telefono = ''
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El teléfono debe tener como mínimo 6 caracteres para buscarlo')
        )

        # Le paso un unicode vacío
        telefono = u''
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El teléfono debe tener como mínimo 6 caracteres para buscarlo')
        )

        # Le paso bien el teléfono, encuentra un solo registro
        telefono = '153705967'
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 1)
        self.assertTrue(len(rta['Respuesta'][0].keys()) == 8)
        self.assertTrue(rta['Respuesta'][0]['id_servicio'] == 60)
        self.assertTrue(
            rta['Respuesta'][0]['fecha_hora_servicio'] == '2015-05-26 17:00:00'
        )
        self.assertTrue(rta['Respuesta'][0]['telefono'] == '153705967')
        self.assertTrue(rta['Respuesta'][0]['direccion'] == u'España 1130')
        self.assertTrue(rta['Respuesta'][0]['nombre'] == u'Martín')
        self.assertTrue(rta['Respuesta'][0]['telefonista'] == 'PTT')
        self.assertTrue(rta['Respuesta'][0]['latitud'] == -118560.9456)
        self.assertTrue(rta['Respuesta'][0]['longitud'] == -218585.8188)

        # Le paso bien el teléfono, encuentra dos registros
        telefono = '1554030'
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 2)

        self.assertTrue(len(rta['Respuesta'][0].keys()) == 8)
        self.assertTrue(rta['Respuesta'][0]['id_servicio'] == 61)
        self.assertTrue(
            rta['Respuesta'][0]['fecha_hora_servicio'] == '2015-05-29 12:00:00'
        )
        self.assertTrue(rta['Respuesta'][0]['telefono'] == '0341155403020')
        self.assertTrue(rta['Respuesta'][0]['direccion'] == 'Dorrego 1800')
        self.assertTrue(rta['Respuesta'][0]['nombre'] == 'Juan')
        self.assertTrue(rta['Respuesta'][0]['telefonista'] == 'CpIvr')
        self.assertTrue(rta['Respuesta'][0]['latitud'] is None)
        self.assertTrue(rta['Respuesta'][0]['longitud'] is None)

        self.assertTrue(len(rta['Respuesta'][1].keys()) == 8)
        self.assertTrue(rta['Respuesta'][1]['id_servicio'] == 62)
        self.assertTrue(
            rta['Respuesta'][1]['fecha_hora_servicio'] == '2015-05-29 12:30:00'
        )
        self.assertTrue(rta['Respuesta'][1]['telefono'] == '0341155403099')
        self.assertTrue(rta['Respuesta'][1]['direccion'] == '')
        self.assertTrue(rta['Respuesta'][1]['nombre'] == '')
        self.assertTrue(rta['Respuesta'][1]['telefonista'] == '')
        self.assertTrue(rta['Respuesta'][1]['latitud'] is None)
        self.assertTrue(rta['Respuesta'][1]['longitud'] is None)

        # Le paso bien el teléfono en unicode
        telefono = u'1554030'
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 2)

        # Le paso un teléfono con acento
        telefono = u'canció'
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 1)

        self.assertTrue(len(rta['Respuesta'][0].keys()) == 8)
        self.assertTrue(rta['Respuesta'][0]['id_servicio'] == 63)
        self.assertTrue(
            rta['Respuesta'][0]['fecha_hora_servicio'] == '2015-05-29 14:00:00'
        )
        self.assertTrue(rta['Respuesta'][0]['telefono'] == u'canción')
        self.assertTrue(rta['Respuesta'][0]['direccion'] == '')
        self.assertTrue(rta['Respuesta'][0]['nombre'] == '')
        self.assertTrue(rta['Respuesta'][0]['telefonista'] == '')
        self.assertTrue(rta['Respuesta'][0]['latitud'] is None)
        self.assertTrue(rta['Respuesta'][0]['longitud'] is None)

        # Le paso un teléfono que no tienen ningún servicio
        telefono = '123456'
        rta = self.st.obtener_servicios_por_telefono(telefono)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se encontraron servicios con el teléfono 123456')
        )

    def test_grabar_importe_servicio(self):
        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4101010'
        dic['FechaServicio'] = '2016-05-27'
        dic['HoraServicio'] = '1900-01-01 12:00:00'
        dic['IdServicio'] = 10
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla historico
        dic = {}
        dic['Telefono'] = '4202020'
        dic['FechaServicio'] = '2016-06-02'
        dic['HoraServicio'] = '1900-01-01 16:00:00'
        dic['IdServicio'] = 5
        agregar = self.st.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto, el servicio está en la tabla "actual"
        id_servicio = 10
        importe = 120.5
        usuario = '32289977'
        rta = self.st.grabar_importe(id_servicio, importe, usuario)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se guardó el importe del servicio correctamente')
        )

        # Caso correcto, el servicio está en la tabla "historico"
        id_servicio = 5
        importe = 80.25
        usuario = 'juan'
        rta = self.st.grabar_importe(id_servicio, importe, usuario)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se guardó el importe del servicio correctamente')
        )

        # Le paso un servicio que no existe en ninguna tabla
        id_servicio = 1234
        importe = 80.75
        usuario = 'juan'
        rta = self.st.grabar_importe(id_servicio, importe, usuario)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo guardar el importe del servicio')
        )

    def test_obtener_servicio_por_id(self):
        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4110549'
        dic['FechaServicio'] = '2017-12-20'
        dic['HoraServicio'] = '1900-01-01 12:00:00'
        dic['IdServicio'] = 101
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4301818'
        dic['FechaServicio'] = '2017-12-20'
        dic['HoraServicio'] = '1900-01-01 12:15:00'
        dic['IdServicio'] = 102
        dic['Prioridad'] = 3
        dic['IdMovil'] = u'Ñai'
        dic['Direccion'] = u'España 1130'
        dic['Latitud'] = -123.456
        dic['Longitud'] = -987.654
        dic['IdServicioPtt'] = 52
        dic['IdZona'] = 13
        dic['Zona'] = u'Zona 13 canción'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4707070'
        dic['FechaServicio'] = '2018-04-20'
        dic['HoraServicio'] = '1900-01-01 12:30:00'
        dic['IdServicio'] = 201
        dic['Prioridad'] = 1
        dic['Siniestro'] = '44092'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla historico
        dic = {}
        dic['Telefono'] = '4258552'
        dic['FechaServicio'] = '2017-12-20'
        dic['HoraServicio'] = '1900-01-01 12:30:00'
        dic['IdServicio'] = 103
        dic['Prioridad'] = 0
        dic['IdMovil'] = '28'
        dic['Direccion'] = 'Italia 1200'
        dic['Latitud'] = -32.253632
        dic['Longitud'] = -60.635214
        dic['HoraSalida'] = '1900-01-01 10:55:00'
        agregar = self.st.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla historico
        dic = {}
        dic['Telefono'] = '4727272'
        dic['FechaServicio'] = '2018-04-19'
        dic['HoraServicio'] = '1900-01-01 13:00:00'
        dic['IdServicio'] = 202
        dic['Prioridad'] = 0
        dic['Siniestro'] = u'canción'
        agregar = self.st.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla moviles
        dic = {}
        dic['IdMovil'] = '28'
        dic['Patente'] = 'SFG 965'
        dic['Sucursal'] = 1
        dic['IdFlota'] = 4
        dic['IdEstacion'] = 15
        agregar = self.st.cp.agregar('moviles', **dic)
        self.assertTrue(agregar is True)

        # Le paso un servicio que está en la tabla actual con los mínimos datos
        id_servicio = 101
        rta = self.st.obtener_por_id(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 15)
        self.assertTrue(rta['Respuesta']['id_servicio'] == 101)
        self.assertTrue(rta['Respuesta']['telefono'] == u'4110549')
        self.assertTrue(rta['Respuesta']['fecha_servicio'] == '2017-12-20')
        self.assertTrue(
            rta['Respuesta']['hora_servicio'] == '1900-01-01 12:00:00'
        )
        self.assertTrue(rta['Respuesta']['id_movil'] == '')
        self.assertTrue(rta['Respuesta']['direccion'] == '')
        self.assertTrue(rta['Respuesta']['prioridad'] is None)
        self.assertTrue(rta['Respuesta']['latitud'] == 0.0)
        self.assertTrue(rta['Respuesta']['longitud'] == 0.0)
        self.assertTrue(rta['Respuesta']['id_flota'] is None)
        self.assertTrue(rta['Respuesta']['hora_salida'] is None)
        self.assertTrue(rta['Respuesta']['siniestro'] == '')
        self.assertTrue(rta['Respuesta']['id_servicio_ptt'] is None)
        self.assertTrue(rta['Respuesta']['id_zona'] is None)
        self.assertTrue(rta['Respuesta']['nombre_zona'] == '')

        # Le paso un servicio que está en la tabla actual con todos datos
        id_servicio = 102
        rta = self.st.obtener_por_id(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 15)
        self.assertTrue(rta['Respuesta']['id_servicio'] == 102)
        self.assertTrue(rta['Respuesta']['telefono'] == u'4301818')
        self.assertTrue(rta['Respuesta']['fecha_servicio'] == '2017-12-20')
        self.assertTrue(
            rta['Respuesta']['hora_servicio'] == '1900-01-01 12:15:00'
        )
        self.assertTrue(rta['Respuesta']['id_movil'] == u'Ñai')
        self.assertTrue(rta['Respuesta']['direccion'] == u'España 1130')
        self.assertTrue(rta['Respuesta']['prioridad'] == 3)
        self.assertTrue(rta['Respuesta']['latitud'] == -123.456)
        self.assertTrue(rta['Respuesta']['longitud'] == -987.654)
        self.assertTrue(rta['Respuesta']['id_flota'] is None)
        self.assertTrue(rta['Respuesta']['hora_salida'] is None)
        self.assertTrue(rta['Respuesta']['siniestro'] == '')
        self.assertTrue(rta['Respuesta']['id_servicio_ptt'] == 52)
        self.assertTrue(rta['Respuesta']['id_zona'] == 13)
        self.assertTrue(rta['Respuesta']['nombre_zona'] == u'Zona 13 canción')

        # Le paso un servicio que está en la tabla historico
        # pero solo lo busco en la tabla actual
        id_servicio = 103
        rta = self.st.obtener_por_id(id_servicio)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo obtener el servicio')
        )

        # Le paso un servicio que está en la tabla historico
        # y le indico que lo debo buscar en la tabla hisotrico
        id_servicio = 103
        buscar_en_historico = True
        rta = self.st.obtener_por_id(id_servicio, buscar_en_historico)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 15)
        self.assertTrue(rta['Respuesta']['id_servicio'] == 103)
        self.assertTrue(rta['Respuesta']['telefono'] == u'4258552')
        self.assertTrue(rta['Respuesta']['fecha_servicio'] == '2017-12-20')
        self.assertTrue(
            rta['Respuesta']['hora_servicio'] == '1900-01-01 12:30:00'
        )
        self.assertTrue(rta['Respuesta']['id_movil'] == '28')
        self.assertTrue(rta['Respuesta']['direccion'] == 'Italia 1200')
        self.assertTrue(rta['Respuesta']['prioridad'] == 0)
        self.assertTrue(rta['Respuesta']['latitud'] == -32.253632)
        self.assertTrue(rta['Respuesta']['longitud'] == -60.635214)
        self.assertTrue(rta['Respuesta']['id_flota'] == 4)
        self.assertTrue(
            rta['Respuesta']['hora_salida'] == '2017-12-20 10:55:00'
        )
        self.assertTrue(rta['Respuesta']['siniestro'] == '')
        self.assertTrue(rta['Respuesta']['id_servicio_ptt'] is None)
        self.assertTrue(rta['Respuesta']['id_zona'] is None)
        self.assertTrue(rta['Respuesta']['nombre_zona'] == '')

        # Le paso un servicio que no está en la tabla actual
        # ni en la tabla historico
        id_servicio = 1234
        buscar_en_historico = True
        rta = self.st.obtener_por_id(id_servicio, buscar_en_historico)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo obtener el servicio')
        )

        # Le paso el dato siniestro
        id_servicio = None
        siniestro = '44092'
        rta = self.st.obtener_por_id(id_servicio, siniestro=siniestro)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 15)
        self.assertTrue(rta['Respuesta']['id_servicio'] == 201)
        self.assertTrue(rta['Respuesta']['telefono'] == u'4707070')
        self.assertTrue(rta['Respuesta']['fecha_servicio'] == '2018-04-20')
        self.assertTrue(
            rta['Respuesta']['hora_servicio'] == '1900-01-01 12:30:00'
        )
        self.assertTrue(rta['Respuesta']['id_movil'] == '')
        self.assertTrue(rta['Respuesta']['direccion'] == '')
        self.assertTrue(rta['Respuesta']['prioridad'] == 1)
        self.assertTrue(rta['Respuesta']['latitud'] == 0.0)
        self.assertTrue(rta['Respuesta']['longitud'] == 0.0)
        self.assertTrue(rta['Respuesta']['id_flota'] is None)
        self.assertTrue(rta['Respuesta']['hora_salida'] is None)
        self.assertTrue(rta['Respuesta']['siniestro'] == '44092')
        self.assertTrue(rta['Respuesta']['id_servicio_ptt'] is None)
        self.assertTrue(rta['Respuesta']['id_zona'] is None)
        self.assertTrue(rta['Respuesta']['nombre_zona'] == '')

        # Le paso un siniestro que no existe
        id_servicio = None
        siniestro = 'asd'
        rta = self.st.obtener_por_id(id_servicio, siniestro=siniestro)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo obtener el servicio')
        )

        # Busco por el siniestro en la tabla historico
        id_servicio = None
        siniestro = u'canción'
        buscar_en_historico = True
        rta = self.st.obtener_por_id(
            id_servicio,
            buscar_en_historico=buscar_en_historico,
            siniestro=siniestro
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 15)
        self.assertTrue(rta['Respuesta']['id_servicio'] == 202)
        self.assertTrue(rta['Respuesta']['telefono'] == u'4727272')
        self.assertTrue(rta['Respuesta']['fecha_servicio'] == '2018-04-19')
        self.assertTrue(
            rta['Respuesta']['hora_servicio'] == '1900-01-01 13:00:00'
        )
        self.assertTrue(rta['Respuesta']['id_movil'] == '')
        self.assertTrue(rta['Respuesta']['direccion'] == '')
        self.assertTrue(rta['Respuesta']['prioridad'] == 0)
        self.assertTrue(rta['Respuesta']['latitud'] == 0.0)
        self.assertTrue(rta['Respuesta']['longitud'] == 0.0)
        self.assertTrue(rta['Respuesta']['id_flota'] is None)
        self.assertTrue(rta['Respuesta']['hora_salida'] is None)
        self.assertTrue(rta['Respuesta']['siniestro'] == u'canción')
        self.assertTrue(rta['Respuesta']['id_servicio_ptt'] is None)
        self.assertTrue(rta['Respuesta']['id_zona'] is None)
        self.assertTrue(rta['Respuesta']['nombre_zona'] == '')

        # Le paso un siniestro que ya está en la tabla historico
        # No busco en la tabla historico
        id_servicio = None
        siniestro = u'canción'
        buscar_en_historico = False
        rta = self.st.obtener_por_id(
            id_servicio,
            buscar_en_historico=buscar_en_historico,
            siniestro=siniestro
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo obtener el servicio')
        )

    def test_listar_servicios(self):
        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4111111'
        dic['FechaServicio'] = '2018-05-10'
        dic['HoraServicio'] = '1900-01-01 10:30:00'
        dic['IdServicio'] = 101
        dic['Prioridad'] = 4
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4222222'
        dic['FechaServicio'] = '2018-05-10'
        dic['HoraServicio'] = '1900-01-01 11:00:00'
        dic['IdServicio'] = 102
        dic['Prioridad'] = 4
        dic['Direccion'] = u'España 1130'
        dic['Latitud'] = -118560.42
        dic['Longitud'] = -218585.71
        dic['Siniestro'] = u'canción'
        dic['IdSucursal'] = 1
        dic['IdMovil'] = '21'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4333333'
        dic['FechaServicio'] = '2018-05-10'
        dic['HoraServicio'] = '1900-01-01 11:30:00'
        dic['IdServicio'] = 103
        dic['Prioridad'] = 1
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Listo los servicios anticipados
        prioridad = 4
        rta = self.st.listar(prioridad)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 2)

        self.assertTrue(len(rta['Respuesta'][0]) == 11)
        self.assertTrue(rta['Respuesta'][0][0] == 101)
        self.assertTrue(rta['Respuesta'][0][1] == '4111111')
        self.assertTrue(rta['Respuesta'][0][2] == '2018-05-10')
        self.assertTrue(rta['Respuesta'][0][3] == '1900-01-01 10:30:00')
        self.assertTrue(rta['Respuesta'][0][4] == 4)
        self.assertTrue(rta['Respuesta'][0][5] == '')
        self.assertTrue(rta['Respuesta'][0][6] == 0)
        self.assertTrue(rta['Respuesta'][0][7] == 0)
        self.assertTrue(rta['Respuesta'][0][8] == '')
        self.assertTrue(rta['Respuesta'][0][9] == 0)
        self.assertTrue(rta['Respuesta'][0][10] == '')

        self.assertTrue(len(rta['Respuesta'][1]) == 11)
        self.assertTrue(rta['Respuesta'][1][0] == 102)
        self.assertTrue(rta['Respuesta'][1][1] == '4222222')
        self.assertTrue(rta['Respuesta'][1][2] == '2018-05-10')
        self.assertTrue(rta['Respuesta'][1][3] == '1900-01-01 11:00:00')
        self.assertTrue(rta['Respuesta'][1][4] == 4)
        self.assertTrue(rta['Respuesta'][1][5] == u'España 1130')
        self.assertTrue(rta['Respuesta'][1][6] == -32.93345)
        self.assertTrue(rta['Respuesta'][1][7] == -60.71825277777778)
        self.assertTrue(rta['Respuesta'][1][8] == u'canción')
        self.assertTrue(rta['Respuesta'][1][9] == 1)
        self.assertTrue(rta['Respuesta'][1][10] == '21')

        # Listo todos los servicios
        prioridad = None
        rta = self.st.listar(prioridad)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 3)

        self.assertTrue(len(rta['Respuesta'][0]) == 11)
        self.assertTrue(rta['Respuesta'][0][0] == 101)
        self.assertTrue(rta['Respuesta'][0][1] == '4111111')
        self.assertTrue(rta['Respuesta'][0][2] == '2018-05-10')
        self.assertTrue(rta['Respuesta'][0][3] == '1900-01-01 10:30:00')
        self.assertTrue(rta['Respuesta'][0][4] == 4)
        self.assertTrue(rta['Respuesta'][0][5] == '')
        self.assertTrue(rta['Respuesta'][0][6] == 0)
        self.assertTrue(rta['Respuesta'][0][7] == 0)
        self.assertTrue(rta['Respuesta'][0][8] == '')
        self.assertTrue(rta['Respuesta'][0][9] == 0)
        self.assertTrue(rta['Respuesta'][0][10] == '')

        self.assertTrue(len(rta['Respuesta'][1]) == 11)
        self.assertTrue(rta['Respuesta'][1][0] == 102)
        self.assertTrue(rta['Respuesta'][1][1] == '4222222')
        self.assertTrue(rta['Respuesta'][1][2] == '2018-05-10')
        self.assertTrue(rta['Respuesta'][1][3] == '1900-01-01 11:00:00')
        self.assertTrue(rta['Respuesta'][1][4] == 4)
        self.assertTrue(rta['Respuesta'][1][5] == u'España 1130')
        self.assertTrue(rta['Respuesta'][1][6] == -32.93345)
        self.assertTrue(rta['Respuesta'][1][7] == -60.71825277777778)
        self.assertTrue(rta['Respuesta'][1][8] == u'canción')
        self.assertTrue(rta['Respuesta'][1][9] == 1)
        self.assertTrue(rta['Respuesta'][1][10] == '21')

        self.assertTrue(len(rta['Respuesta'][2]) == 11)
        self.assertTrue(rta['Respuesta'][2][0] == 103)
        self.assertTrue(rta['Respuesta'][2][1] == '4333333')
        self.assertTrue(rta['Respuesta'][2][2] == '2018-05-10')
        self.assertTrue(rta['Respuesta'][2][3] == '1900-01-01 11:30:00')
        self.assertTrue(rta['Respuesta'][2][4] == 1)
        self.assertTrue(rta['Respuesta'][2][5] == '')
        self.assertTrue(rta['Respuesta'][2][6] == 0)
        self.assertTrue(rta['Respuesta'][2][7] == 0)
        self.assertTrue(rta['Respuesta'][2][8] == '')
        self.assertTrue(rta['Respuesta'][2][9] == 0)
        self.assertTrue(rta['Respuesta'][2][10] == '')

        # Le paso una prioridad que no existe
        prioridad = 1234
        rta = self.st.listar(prioridad)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No se encontraron servicios'))

    def test_obtener_tipo_tarifa(self):
        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4333333'
        dic['FechaServicio'] = '2018-12-21'
        dic['HoraServicio'] = '1900-01-01 11:30:00'
        dic['IdServicio'] = 111
        dic['Prioridad'] = 1
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4333333'
        dic['FechaServicio'] = '2018-12-21'
        dic['HoraServicio'] = '1900-01-01 12:00:00'
        dic['IdServicio'] = 112
        dic['Prioridad'] = 3
        dic['AceptaCtaCte'] = 1
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4333333'
        dic['FechaServicio'] = '2018-12-21'
        dic['HoraServicio'] = '1900-01-01 12:15:00'
        dic['IdServicio'] = 113
        dic['Prioridad'] = 3
        dic['IdServicioPtt'] = 41285
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4333333'
        dic['FechaServicio'] = '2018-12-21'
        dic['HoraServicio'] = '1900-01-01 12:30:00'
        dic['IdServicio'] = 114
        dic['Prioridad'] = 3
        dic['Telefonista'] = 'CpIvr'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4333333'
        dic['FechaServicio'] = '2018-12-21'
        dic['HoraServicio'] = '1900-01-01 12:45:00'
        dic['IdServicio'] = 115
        dic['Prioridad'] = 3
        dic['AceptaCtaCte'] = 1
        dic['IdServicioPtt'] = 52142
        dic['Telefonista'] = 'CpIvr'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Le paso un servicio del tipo "Central"
        id_servicio = 111
        rta = self.st.obtener_tipo_tarifa(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 3)

        # Le paso un servicio del tipo "Cta. Cte."
        id_servicio = 112
        rta = self.st.obtener_tipo_tarifa(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 6)

        # Le paso un servicio del tipo "APP"
        id_servicio = 113
        rta = self.st.obtener_tipo_tarifa(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 4)

        # Le paso un servicio del tipo "IVR"
        id_servicio = 114
        rta = self.st.obtener_tipo_tarifa(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 7)

        # Le paso un servicio de todos los tipos, el más importante es Cta. Cte.
        id_servicio = 115
        rta = self.st.obtener_tipo_tarifa(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 6)

    def test_cancelar_servicio(self):
        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '3413705967'
        dic['FechaServicio'] = '2019-12-20'
        dic['HoraServicio'] = '1900-01-01 16:41:30'
        dic['Prioridad'] = 3
        dic['Telefonista'] = 'whatasapp'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4110549'
        dic['FechaServicio'] = '2019-12-23'
        dic['HoraServicio'] = '1900-01-01 17:00:28'
        dic['Prioridad'] = 0
        dic['Telefonista'] = 'whatasapp'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4505050'
        dic['FechaServicio'] = '2019-12-23'
        dic['HoraServicio'] = '1900-01-01 17:15:19'
        dic['Prioridad'] = 5
        dic['Telefonista'] = 'whatasapp'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4222222'
        dic['FechaServicio'] = '2019-12-23'
        dic['HoraServicio'] = '1900-01-01 17:25:13'
        dic['Prioridad'] = 1
        dic['Telefonista'] = 'whatasapp'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = '4222222'
        dic['FechaServicio'] = '2019-12-23'
        dic['HoraServicio'] = '1900-01-01 17:26:43'
        dic['Prioridad'] = 2
        dic['Telefonista'] = 'whatasapp'
        agregar = self.st.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        telefono = '3413705967'
        rta = self.st.cancelar(telefono)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'El servicio se canceló de forma correcta')
        )

        # Le paso un servicio que ya está despachado
        telefono = '4110549'
        rta = self.st.cancelar(telefono)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró ningún servicio pendiente')
        )

        # Le paso un servicio que ya está cancelado
        telefono = '4505050'
        rta = self.st.cancelar(telefono)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró ningún servicio pendiente')
        )

        # Le paso un servicio que no existe
        telefono = '1234'
        rta = self.st.cancelar(telefono)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró ningún servicio pendiente')
        )

        # Cancelo 2 servicios pendientes de un mismo teléfono
        telefono = '4222222'
        rta = self.st.cancelar(telefono)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'El servicio se canceló de forma correcta')
        )


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestServicios))
    return suite
