# -*- coding: utf-8 -*-

import unittest
import datetime
from datetime import date

from at.utiles import _
from at.utiles.servidores import DateTimeHelper


class TestDateTimeHelper(unittest.TestCase):

    def test_get_date(self):

        dth = DateTimeHelper()
        dtoday = date.today()
        today = dth.get_date()
        self.assertEqual(dtoday, today)
        today = dth.get_date(as_str=True)
        self.assertTrue(isinstance(today, str))
        self.assertTrue(dtoday.strftime("%d") in today)
        self.assertTrue(dtoday.strftime("%m") in today)
        self.assertTrue(dtoday.strftime("%Y") in today)

    def test_get_now(self):
        dth = DateTimeHelper()

        now = dth.get_now(as_str=True)
        self.assertTrue(isinstance(now, str))

        now = dth.get_now()
        self.assertTrue(isinstance(now, datetime.datetime))

    def test_get_fecha_null_con_hora(self):
        dth = DateTimeHelper()

        r = dth.get_fecha_null_con_hora()
        self.assertTrue(r)
        self.assertTrue(type(r) == datetime.datetime)

    def test_get_fecha_con_hora_null(self):
        dth = DateTimeHelper()

        r = dth.get_fecha_con_hora_null()
        self.assertTrue(type(r) == datetime.datetime)


    def test_obtener_fecha_hora_ciudad(self):
        dth = DateTimeHelper()

        try:
            import at.map
        except:
            print('>> warning: No hay at.map. Saltamos tests.')
            return

        # Caso correcto
        id_ciudad = 324
        rta = dth.obtener_fecha_hora_ciudad(id_ciudad)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] is not None)

        # Le paso una ciudad que no existe
        id_ciudad = 9999
        rta = dth.obtener_fecha_hora_ciudad(id_ciudad)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] ==
                            _(u'No se pudo obtener el país de la ciudad dada'))

        # Le paso la ciudad Santo Domingo
        id_ciudad = 5008
        rta = dth.obtener_fecha_hora_ciudad(id_ciudad)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] is not None)


def suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestDateTimeHelper))
    return suite