# -*- coding: utf-8 -*-

import unittest
from datetime import datetime

from at.utiles import _
from at.utiles.sucursales import Sucursales
from at.orm.abm import ABM


cp = {'user': 'frula',
      'passwd': 'changeme',
      'dbname': 'tests',
      'name': 'cp',
      'host': '192.168.0.51',
      'engine': 'mysql',
      'port': '3306',
      }


class SucursalesTesting(Sucursales):

    def __init__(self):
        Sucursales.__init__(self)
        self.cp = ABM(cp)


class TestSucursales(unittest.TestCase):

    def setUp(self):
        self.st = SucursalesTesting()

    def tearDown(self):
        pass

    def test_obtener_id_sucursales(self):
        resultado = self.st.obtener_id_sucursales()
        self.assertTrue(isinstance(resultado, list))

    def test_obtener_licencia_empresa(self):
        id_sucursal = 1
        resultado = self.st.obtener_licencia_empresa(id_sucursal)
        self.assertTrue(resultado['Estado'] is True)

    def test_obtener_datos(self):
        # Le paso una sucursal que no existe
        id_sucursal = 1234
        rta = self.st.obtener_datos(id_sucursal)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No se encontró la sucursal'))

        # Caso correcto
        id_sucursal = 1
        rta = self.st.obtener_datos(id_sucursal)
        self.assertTrue(rta['Estado'] is True)
        sucursal = rta['Respuesta']
        self.assertTrue(sucursal.Sucursal == 1)
        self.assertTrue(isinstance(sucursal.Nombre, str))
        self.assertTrue(isinstance(sucursal.CierreDeTurno, datetime))
        self.assertTrue(sucursal.idMapa is not None)
        self.assertTrue(isinstance(sucursal.Despachando, int))

    def test_activar_desactivar_despacho(self):
        # Activo el despacho automático
        id_sucursal = 1
        activar = 1
        rta = self.st.activar_desactivar_despacho(id_sucursal, activar)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se activa el despacho para la sucursal 1')
        )

        # Desactivo el despacho automático
        id_sucursal = 1
        activar = 0
        rta = self.st.activar_desactivar_despacho(id_sucursal, activar)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Se desactiva el despacho para la sucursal 1')
        )

        # Le paso una sucursal que no existe
        id_sucursal = 999
        activar = 1
        rta = self.st.activar_desactivar_despacho(id_sucursal, activar)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se pudo cambiar el estado del despacho de la sucursal 999')
        )

        # Le paso mal el dato "id_sucursal"
        id_sucursal = '1'
        activar = 1
        rta = self.st.activar_desactivar_despacho(id_sucursal, activar)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'El dato "id_sucursal" debe ser un número.')
        )

        # Le paso mal el dato "activar"
        id_sucursal = 1
        activar = 9
        rta = self.st.activar_desactivar_despacho(id_sucursal, activar)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El dato "activar" debe ser mayor o igual que 0 y menor o igual que 1')
        )


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestSucursales))
    return suite
