# -*- coding: utf-8 -*-

import unittest

from at.utiles import _
from at.utiles.telefonos import Telefonos


class TestTelefonos(unittest.TestCase):

    def setUp(self):
        self.tt = Telefonos()

    def tearDown(self):
        pass

    def test_formatear_telefono(self):
        # Le paso un teléfono que no es un string
        telefono = 3413403509
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'El teléfono 3413403509 no es un string')
        )

        # Le paso un teléfono como None
        telefono = None
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'El teléfono None no es un string')
        )

        # Le paso un teléfono de pocos carácteres
        telefono = '03509'
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El teléfono 03509 no cuanta con el mínimo de caracteres')
        )

        # Ya le paso bien el teléfono
        telefono = '+5493413403509'
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == '+5493413403509')

        # Le paso un teléfono de referencia que no es un string
        telefono = '341-340-3509'
        telefono_de_referencia = 5493413705967
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El teléfono de referencia 5493413705967 no es un string')
        )

        # Le paso un teléfono de referencia de pocos caracteres
        telefono = '341-340-3509'
        telefono_de_referencia = '+54934'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El teléfono de referencia +54934 no cuanta con el mínimo de caracteres')
        )

        # Le paso un teléfono de referencia sin el signo + al comienzo
        telefono = '341-340-3509'
        telefono_de_referencia = '5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El teléfono de referencia 5493413705967 no comienza con el signo +')
        )

        # Le paso un teléfono que comienza con dos ceros pero también tiene
        # dos ceros entre el número
        telefono = '005493413003509'
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == '+5493413003509')

        # Le paso un teléfono con guión medio
        telefono = '00549-341-3003509'
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == '+5493413003509')

        # Le paso un teléfono con espacios en blanco
        telefono = '00549 341 3003509'
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == '+5493413003509')

        # Le paso un teléfono con espacios en blanco atrás y adelante
        telefono = '     005493413003509    '
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == '+5493413003509')

        # Le paso un teléfono con espacios en blanco atrás, adelante y en el
        # medio, y con guiones
        telefono = '     00-549 341-3 003509    '
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == '+5493413003509')

        # Le paso un teléfono sin el código del país
        telefono = '3413003509'
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == '+5493413003509')

        # Le paso un teléfono sin el código del país y con el código de la
        # ciudad que comienza con 0
        telefono = '03413003509'
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == '+5493413003509')

        # Le paso un teléfono que comienza con 15, este no tiene código de país
        # ni de ciudad
        telefono = '153003509'
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == '+5493413003509')

        # Le paso un teléfono que no es valido
        telefono = '00asd qwe '
        telefono_de_referencia = '+5493413705967'
        rta = self.tt.formatear(telefono, telefono_de_referencia)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'El teléfono 00asdqwe no es válido')
        )


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestTelefonos))
    return suite