# -*- coding: utf-8 -*-

import unittest
from at.utiles import _
from at.utiles.trafico_de_datos import TraficoDeDatos
from at.orm.abm import ABM


cp = {'user': 'frula',
      'passwd': 'changeme',
      'dbname': 'tests',
      'name': 'cp',
      'host': '192.168.0.51',
      'engine': 'mysql',
      'port': '3306',
      }


class TraficoDeDatosTesting(TraficoDeDatos):

    def __init__(self):
        TraficoDeDatos.__init__(self)
        self.cp = ABM(cp)


class TestTraficoDeDatos(unittest.TestCase):

    def setUp(self):
        self.trafico_de_datos_t = TraficoDeDatosTesting()

        self.trafico_de_datos_t.cp.borrar('trafico_de_datos')

    def tearDown(self):
        self.trafico_de_datos_t.cp.borrar('trafico_de_datos')

    def test_guardar_trafico_por_dia(self):
        # caso de agregar un nuevo registro
        aplicacion = 'CpMobile'
        kb_recibidos = 0.16
        kb_enviados = 0.07
        resultado = self.trafico_de_datos_t.guardar_trafico_por_dia(
                                                                aplicacion,
                                                                kb_recibidos,
                                                                kb_enviados)
        self.assertTrue(resultado['Estado'] is True)
        self.assertTrue(resultado['Respuesta'] == _(
                u'Se agrego un nuevo registro en la tabla trafico_de_datos'))

        # caso de actualizar un registro existente
        aplicacion = 'CpMobile'
        kb_recibidos = 1.24
        kb_enviados = 1.03
        resultado = self.trafico_de_datos_t.guardar_trafico_por_dia(
                                                                aplicacion,
                                                                kb_recibidos,
                                                                kb_enviados)
        self.assertTrue(resultado['Estado'] is True)
        self.assertTrue(resultado['Respuesta'] == _(
                u'Se ha actualizado un registro en la tabla trafico_de_datos'))

        # le paso otra aplicación
        aplicacion = 'Asd'
        kb_recibidos = 2.52
        kb_enviados = 1.71
        resultado = self.trafico_de_datos_t.guardar_trafico_por_dia(
                                                                aplicacion,
                                                                kb_recibidos,
                                                                kb_enviados)
        self.assertTrue(resultado['Estado'] is True)
        self.assertTrue(resultado['Respuesta'] == _(
                u'Se agrego un nuevo registro en la tabla trafico_de_datos'))

    def test_obtener_trafico(self):
        # no hay ningún registro en la tabla trafico_de_datos
        aplicacion = 'Asd'
        resultado = self.trafico_de_datos_t.obtener_trafico(aplicacion)
        self.assertTrue(resultado['Estado'] is False)
        self.assertTrue(resultado['Respuesta'] == _(
                u'No se encontraron registros en la tabla trafico_de_datos'))

        # agrego un registro de la aplicación CpMobile
        d = {}
        d['fecha_hora_desde'] = '2012-03-08 00:00:00'
        d['fecha_hora_hasta'] = '2012-03-08 23:59:59'
        d['kb_recibidos'] = 5.6
        d['kb_enviados'] = 4.2
        d['aplicacion'] = 'CpMobile'
        agregar = self.trafico_de_datos_t.cp.agregar('trafico_de_datos', **d)
        self.assertTrue(agregar is True)

        # agrego otro registro de la aplicación CpMobile
        d = {}
        d['fecha_hora_desde'] = '2012-03-07 00:00:00'
        d['fecha_hora_hasta'] = '2012-03-07 23:59:59'
        d['kb_recibidos'] = 3.21
        d['kb_enviados'] = 8.43
        d['aplicacion'] = 'CpMobile'
        agregar = self.trafico_de_datos_t.cp.agregar('trafico_de_datos', **d)
        self.assertTrue(agregar is True)

        # agrego un registro de otra aplicación
        d = {}
        d['fecha_hora_desde'] = '2012-03-06 00:00:00'
        d['fecha_hora_hasta'] = '2012-03-06 23:59:59'
        d['kb_recibidos'] = 5.21
        d['kb_enviados'] = 4.43
        d['aplicacion'] = 'Asd'
        agregar = self.trafico_de_datos_t.cp.agregar('trafico_de_datos', **d)
        self.assertTrue(agregar is True)

        # listo todos los registros de la aplicación CpMobile
        aplicacion = 'CpMobile'
        resultado = self.trafico_de_datos_t.obtener_trafico(aplicacion)
        self.assertTrue(resultado['Estado'] is True)
        self.assertTrue(len(resultado['Respuesta']) == 2)

        self.assertTrue(resultado['Respuesta'][0]['aplicacion'] == 'CpMobile')
        self.assertTrue(resultado['Respuesta'][0]['fecha_hora_desde'] == '07/03/2012 00:00:00')
        self.assertTrue(resultado['Respuesta'][0]['fecha_hora_hasta'] == '07/03/2012 23:59:59')
        self.assertTrue(resultado['Respuesta'][0]['kb_recibidos'] == 3.21)
        self.assertTrue(resultado['Respuesta'][0]['kb_enviados'] == 8.43)
        self.assertTrue(resultado['Respuesta'][0]['dia'] == '07/03/2012')

        self.assertTrue(resultado['Respuesta'][1]['aplicacion'] == 'CpMobile')
        self.assertTrue(resultado['Respuesta'][1]['fecha_hora_desde'] == '08/03/2012 00:00:00')
        self.assertTrue(resultado['Respuesta'][1]['fecha_hora_hasta'] == '08/03/2012 23:59:59')
        self.assertTrue(resultado['Respuesta'][1]['kb_recibidos'] == 5.6)
        self.assertTrue(resultado['Respuesta'][1]['kb_enviados'] == 4.2)
        self.assertTrue(resultado['Respuesta'][1]['dia'] == '08/03/2012')

        # listo todos los registros de la aplicación CpMobile pero le pongo un
        # límete de 1 y orden descendente
        aplicacion = 'CpMobile'
        orden = 'desc'
        limite = 1
        resultado = self.trafico_de_datos_t.obtener_trafico(aplicacion,
                                                            orden, limite)
        self.assertTrue(resultado['Estado'] is True)
        self.assertTrue(len(resultado['Respuesta']) == 1)

        self.assertTrue(resultado['Respuesta'][0]['aplicacion'] == 'CpMobile')
        self.assertTrue(resultado['Respuesta'][0]['fecha_hora_desde'] == '08/03/2012 00:00:00')
        self.assertTrue(resultado['Respuesta'][0]['fecha_hora_hasta'] == '08/03/2012 23:59:59')
        self.assertTrue(resultado['Respuesta'][0]['kb_recibidos'] == 5.6)
        self.assertTrue(resultado['Respuesta'][0]['kb_enviados'] == 4.2)
        self.assertTrue(resultado['Respuesta'][0]['dia'] == '08/03/2012')


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestTraficoDeDatos))
    return suite
