# -*- coding: utf-8 -*-

import unittest
import datetime

from at.utiles.validaciones_generales import ValidacionesGenerales


class TestValidacionesGenerales(unittest.TestCase):
    def setUp(self):
        self.generales = ValidacionesGenerales()

    def tearDown(self):
        pass

    def test_es_posible_direccion(self):
        rta = self.generales.es_posible_direccion(None)
        print (rta)
        self.assertTrue(
            rta == {"Estado": False, "Respuesta": "no hay texto ni numero"})

        rta = self.generales.es_posible_direccion("15")
        self.assertTrue(rta == {"Estado": False,
                                "Respuesta": "Ingrese calle y numero porfavor"})

        rta = self.generales.es_posible_direccion("15 15")
        # print("es_posible_direccion rta bis", rta)
        self.assertTrue(rta == {"Estado": False,
                "Respuesta": "Ingrese primero la calle y luego la altura."})

        rta = self.generales.es_posible_direccion("15 hola perro")
        # print("es_posible_direccion rta bis", rta)
        self.assertTrue(rta == {"Estado": False,
                "Respuesta": "Ingrese primero la calle y luego la altura."})

        rta = self.generales.es_posible_direccion("15.13 15.2")
        self.assertTrue(rta == {"Estado": False,
                "Respuesta": "Ingrese primero la calle y luego la altura."})

        rta = self.generales.es_posible_direccion("hola hola")
        self.assertTrue(rta == {"Estado": False,
                "Respuesta": "Ingrese primero la calle y luego la altura."})

        rta = self.generales.es_posible_direccion("15 urquiza")

        self.assertTrue(
            rta == {"Estado": False, "Respuesta":
                "Ingrese primero la calle y luego la altura."})

        # rta = self.generales.es_posible_direccion("Urquiza 7164 4c")
        # self.assertTrue(
        #     rta == {"Estado": False, "Respuesta":
        #         "Ingrese primero la calle y luego la altura."})

        rta = self.generales.es_posible_direccion("Urquiza 7164")

        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "Urquiza",
                "numero": "7164",
                "extra": None,
                "bis": False
            }})


        rta = self.generales.es_posible_direccion("Urquiza 7164 bis")

        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "Urquiza",
                "numero": "7164",
                "extra": None,
                "bis": True
            }})


        rta = self.generales.es_posible_direccion("Urquiza 7164 7c")

        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "Urquiza",
                "numero": "7164",
                "extra": "7c",
                "bis": False
            }})

        rta = self.generales.es_posible_direccion("justo de Urquiza 7164 7c")

        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "justo de Urquiza",
                "numero": "7164",
                "extra": "7c",
                "bis": False
            }})

        rta = self.generales.es_posible_direccion(
            "justo de Urquiza 7164 7c bis")

        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "justo de Urquiza",
                "numero": "7164",
                "extra": "7c",
                "bis": True
            }})

        rta = self.generales.es_posible_direccion(
            "9 de julio 123")

        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "9 de julio",
                "numero": "123",
                "extra": None,
                "bis": False
            }})

        rta = self.generales.es_posible_direccion(
            "9 de julio 123 bis")
        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "9 de julio",
                "numero": "123",
                "extra": None,
                "bis": True
            }})

        rta = self.generales.es_posible_direccion(
            "9 de julio 123 7c bis")

        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "9 de julio",
                "numero": "123",
                "extra": "7c",
                "bis": True}})

        rta = self.generales.es_posible_direccion(
            "4 hermanos 123 7c bis")

        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "4 hermanos",
                "numero": "123",
                "extra": "7c",
                "bis": True}})

        rta = self.generales.es_posible_direccion(
            "regimiento 11 123 7c bis")
        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "regimiento 11",
                "numero": "123",
                "extra": "7c",
                "bis": True}})

        rta = self.generales.es_posible_direccion(
            "regimiento 11 123 7c")
        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "regimiento 11",
                "numero": "123",
                "extra": "7c",
                "bis": False}})

        rta = self.generales.es_posible_direccion(
            "regimiento 11 123")

        self.assertTrue(rta == {
            "Estado": True,
            "Respuesta": "ok",
            "Dato": {
                "calle": "regimiento 11",
                "numero": "123",
                "extra": None,
                "bis": False}})

    def test_validar_numero(self):
        # caso correcto, le paso un número entero
        valor = 5
        r = self.generales.validar_numero(valor)
        self.assertTrue(r)

        # caso correcto 2, le paso un número con coma
        valor = 5.3
        r = self.generales.validar_numero(valor)
        self.assertTrue(r)

        # caso correcto 3, le paso un número negativo
        valor = -5
        r = self.generales.validar_numero(valor)
        self.assertTrue(r)

        # caso correcto 4, le paso un número negativo con coma
        valor = -5.3
        r = self.generales.validar_numero(valor)
        self.assertTrue(r)

        # caso correcto 5, le paso None
        valor = None
        r = self.generales.validar_numero(valor)
        self.assertTrue(r)

        # le paso un string
        valor = '5.3'
        self.assertRaises(ValueError, self.generales.validar_numero, valor)

        # le paso un string vacío
        valor = ''
        self.assertRaises(ValueError, self.generales.validar_numero, valor)

        # le paso un booleano
        valor = False
        self.assertRaises(ValueError, self.generales.validar_numero, valor)

    def test_validar_entero(self):
        # caso correcto, le paso un número entero
        valor = 6
        r = self.generales.validar_entero(valor)
        self.assertTrue(r)

        # caso correcto 2, le paso un entero negativo
        valor = -6
        r = self.generales.validar_entero(valor)
        self.assertTrue(r)

        # caso correcto 3, le paso None
        valor = None
        r = self.generales.validar_entero(valor)
        self.assertTrue(r)

        # le paso un string
        valor = '6'
        self.assertRaises(ValueError, self.generales.validar_entero, valor)

        # le paso un string vacío
        valor = ''
        self.assertRaises(ValueError, self.generales.validar_entero, valor)

        # le paso un número con coma
        valor = 6.2
        self.assertRaises(ValueError, self.generales.validar_entero, valor)

        # le paso un número negativo con coma
        valor = 6.2
        self.assertRaises(ValueError, self.generales.validar_entero, valor)

        # le paso un booleano
        valor = True
        self.assertRaises(ValueError, self.generales.validar_entero, valor)

    def test_validar_positivo(self):
        # caso correcto, le paso un número positivo
        valor = 7
        r = self.generales.validar_positivo(valor)
        self.assertTrue(r)

        # caso correcto 2, le paso un número con coma positivo
        valor = 7.4
        r = self.generales.validar_positivo(valor)
        self.assertTrue(r)

        # caso correcto 3, le paso None
        valor = None
        r = self.generales.validar_positivo(valor)
        self.assertTrue(r)

        # le paso un número negativo
        valor = -7
        self.assertRaises(Exception, self.generales.validar_positivo, valor)

        # le paso un float negativo
        valor = -7.4
        self.assertRaises(Exception, self.generales.validar_positivo, valor)

        # le paso un string
        valor = '7'
        self.assertRaises(Exception, self.generales.validar_positivo, valor)

        # le paso un string vacío
        valor = ''
        self.assertRaises(Exception, self.generales.validar_positivo, valor)

        # le paso un booleano
        valor = True
        self.assertRaises(Exception, self.generales.validar_positivo, valor)

    def test_validar_numero_maximo(self):
        # caso correcto, le paso un número menor a su máximo
        maximo = 999999999
        valor = 21
        r = self.generales.validar_numero_maximo(maximo, valor)
        self.assertTrue(r)

        # caso correcto 2, le paso un número negativo menor a su máximo
        maximo = 999999999
        valor = -21
        r = self.generales.validar_numero_maximo(maximo, valor)
        self.assertTrue(r)

        # caso correcto 3, le paso un número negativo menor a su máximo
        maximo = 999999999
        valor = -1000000000
        r = self.generales.validar_numero_maximo(maximo, valor)
        self.assertTrue(r)

        # caso correcto 4, le paso None
        maximo = 999999999
        valor = None
        r = self.generales.validar_numero_maximo(maximo, valor)
        self.assertTrue(r)

        # caso correcto 5, le paso un número con coma
        maximo = 999999999
        valor = 21.58
        r = self.generales.validar_numero_maximo(maximo, valor)
        self.assertTrue(r)

        # caso correcto 6, le paso un número negativo con coma
        maximo = 999999999
        valor = -21.58
        r = self.generales.validar_numero_maximo(maximo, valor)
        self.assertTrue(r)

        # caso correcto 7, le paso un número igual a su máximo
        maximo = 999999999
        valor = 999999999
        r = self.generales.validar_numero_maximo(maximo, valor)
        self.assertTrue(r)

        # le paso un número mayor a su máximo
        maximo = 999999999
        valor = 1000000000
        self.assertRaises(Exception, self.generales.validar_numero_maximo,
                          maximo, valor)

        # le paso un número con coma mayor a su máximo
        maximo = 999999999
        valor = 999999999.1
        self.assertRaises(Exception, self.generales.validar_numero_maximo,
                          maximo, valor)

        # le paso un string
        maximo = 999999999
        valor = '21'
        self.assertRaises(Exception, self.generales.validar_numero_maximo,
                          maximo, valor)

        # le paso un string vacío
        maximo = 999999999
        valor = ''
        self.assertRaises(Exception, self.generales.validar_numero_maximo,
                          maximo, valor)

        # le paso un booleano
        maximo = 999999999
        valor = False
        self.assertRaises(Exception, self.generales.validar_numero_maximo,
                          maximo, valor)

    def test_validar_no_vacio(self):
        # caso correcto, le paso un valor
        valor = 'valor'
        r = self.generales.validar_no_vacio(valor)
        self.assertTrue(r)

        # le paso None
        valor = None
        self.assertRaises(Exception, self.generales.validar_no_vacio, valor)

        # le paso un string vacío
        valor = ''
        self.assertRaises(Exception, self.generales.validar_no_vacio, valor)

    def test_validar_string(self):
        # caso correcto, paso un string
        valor = 'un string'
        r = self.generales.validar_string(valor)
        self.assertTrue(r)

        # caso correcto 2, paso un string vacío
        valor = ''
        r = self.generales.validar_string(valor)
        self.assertTrue(r)

        # caso correcto 3, paso None
        valor = None
        r = self.generales.validar_string(valor)
        self.assertTrue(r)

        # le paso un int
        valor = 1
        self.assertRaises(ValueError, self.generales.validar_string, valor)

        # le paso un float
        valor = 1.8
        self.assertRaises(ValueError, self.generales.validar_string, valor)

        # le paso un booleano
        valor = True
        self.assertRaises(ValueError, self.generales.validar_string, valor)

    def test_validar_string_maximo(self):
        # caso correcto, paso un string menor a su máximo
        maximo = 50
        valor = 'un string'
        r = self.generales.validar_string_maximo(maximo, valor)
        self.assertTrue(r)

        # caso correcto 2, paso un string vacío
        maximo = 50
        valor = ''
        r = self.generales.validar_string_maximo(maximo, valor)
        self.assertTrue(r)

        # caso correcto 3, paso un string de tamaño igual a su máximo
        maximo = 50
        valor = 'a' * 50
        r = self.generales.validar_string_maximo(maximo, valor)
        self.assertTrue(r)

        # caso correcto 4, paso None como valor
        maximo = 50
        valor = None
        r = self.generales.validar_string_maximo(maximo, valor)
        self.assertTrue(r)

        # le paso un string de tamaño mayor a su máximo
        maximo = 50
        valor = 'a' * 51
        self.assertRaises(Exception, self.generales.validar_string_maximo,
                          maximo, valor)

        # le paso un int
        maximo = 50
        valor = 17
        self.assertRaises(ValueError, self.generales.validar_string_maximo,
                          maximo, valor)

        # le paso un float
        maximo = 50
        valor = 17.23
        self.assertRaises(ValueError, self.generales.validar_string_maximo,
                          maximo, valor)

        # le paso un booleano
        maximo = 50
        valor = False
        self.assertRaises(ValueError, self.generales.validar_string_maximo,
                          maximo, valor)

    def test_validar_sin_espacios(self):
        # caso correcto, string sin espacios
        valor = 'sinespacios'
        r = self.generales.validar_sin_espacios(valor)
        self.assertTrue(r)

        # caso correcto, string vacío
        valor = ''
        r = self.generales.validar_sin_espacios(valor)
        self.assertTrue(r)

        # le paso un string con espacios en blancos
        valor = 'con espacios'
        self.assertRaises(Exception, self.generales.validar_sin_espacios,
                          valor)

        # le paso un valor que no es un string
        valor = 8
        self.assertRaises(Exception, self.generales.validar_sin_espacios,
                          valor)

    def test_validar_sin_barras(self):
        # caso correcto, string sin barras
        valor = 'sinbarras'
        r = self.generales.validar_sin_barras(valor)
        self.assertTrue(r)

        # caso correcto, string vacío
        valor = ''
        r = self.generales.validar_sin_barras(valor)
        self.assertTrue(r)

        # le paso un string con espacios en barras
        valor = 'con/barras'
        self.assertRaises(Exception, self.generales.validar_sin_barras,
                          valor)

        # le paso un valor que no es un string
        valor = 9
        self.assertRaises(Exception, self.generales.validar_sin_barras,
                          valor)

    def test_validar_string_minimo(self):
        # caso correcto, le paso un string de más caracteres que su mínimo
        minimo = 4
        valor = 'palabra'
        r = self.generales.validar_string_minimo(minimo, valor)
        self.assertTrue(r)

        # caso correcto, le paso un string de 4 caracteres
        minimo = 4
        valor = 'pala'
        r = self.generales.validar_string_minimo(minimo, valor)
        self.assertTrue(r)

        # le paso un string vacío
        minimo = 4
        valor = ''
        self.assertRaises(Exception,
                          self.generales.validar_string_minimo,
                          minimo, valor)

        # le paso un string de menos de 4 caracteres
        minimo = 4
        valor = 'tre'
        self.assertRaises(Exception,
                          self.generales.validar_string_minimo,
                          minimo, valor)

        # le paso un valor que no es un string
        minimo = 4
        valor = 1986
        self.assertRaises(Exception,
                          self.generales.validar_string_minimo,
                          minimo, valor)

    def test_validar_0_o_1(self):
        # caso correcto, un 1
        valor = 1
        r = self.generales.validar_0_o_1(valor)
        self.assertTrue(r)

        # caso correcto 2, un 0
        valor = 0
        r = self.generales.validar_0_o_1(valor)
        self.assertTrue(r)

        # caso correcto 3, -0 no existe, lo toma como 0
        valor = -0
        r = self.generales.validar_0_o_1(valor)
        self.assertTrue(r)

        # no debe poder recibir string
        habilitado = 'palabra'
        self.assertRaises(ValueError,
                          self.generales.validar_0_o_1,
                          habilitado)

        # no debe poder recibir un float
        valor = 1.0
        self.assertRaises(ValueError,
                          self.generales.validar_0_o_1,
                          valor)

        # no debe poder recibir un valor vacío
        valor = None
        self.assertRaises(Exception,
                          self.generales.validar_0_o_1,
                          valor)

        # no debe poder recibir un string vacío
        valor = ''
        self.assertRaises(ValueError,
                          self.generales.validar_0_o_1,
                          valor)

        # no debe poder recibir un objeto
        valor = type('habilitado', (object,), {})
        self.assertRaises(ValueError,
                          self.generales.validar_0_o_1,
                          valor)

        # no debe poder recibir un booleano
        valor = True
        self.assertRaises(ValueError,
                          self.generales.validar_0_o_1,
                          valor)

        # solo puede recibir 1 o 0
        valor = 10
        self.assertRaises(Exception,
                          self.generales.validar_0_o_1,
                          valor)

        # no debe poder recibir un número negativo
        valor = -1
        self.assertRaises(Exception,
                          self.generales.validar_0_o_1,
                          valor)

    def test_validar_fecha_hora(self):
        # caso correcto, le paso una fecha con hora en formato correcto
        valor = '2010-08-12 10:53:22'
        r = self.generales.validar_fecha_hora(valor)
        self.assertTrue(r)

        # intento pasarle una fecha sin hora
        valor = '2010-08-12'
        self.assertRaises(Exception, self.generales.validar_fecha_hora, valor)

        # intento pasarle una fecha y hora que no existe
        valor = '2010-13-32 10:63:61'
        self.assertRaises(Exception, self.generales.validar_fecha_hora, valor)

        # intento pasarle una fecha con otro formato
        valor = '10/08/12 9:53:22'
        self.assertRaises(Exception, self.generales.validar_fecha_hora, valor)

        # intento pasarle un entero
        valor = 2010
        self.assertRaises(Exception, self.generales.validar_fecha_hora, valor)

        # intento pasarle un booleano
        valor = True
        self.assertRaises(Exception, self.generales.validar_fecha_hora, valor)

    def test_validar_fecha(self):
        # caso correcto, le paso una fecha en formato correcto
        valor = '2010-08-12'
        r = self.generales.validar_fecha(valor)
        self.assertTrue(r)

        # intento pasarle una fecha que no existe
        valor = '2010-13-32'
        self.assertRaises(Exception, self.generales.validar_fecha, valor)

        # intento pasarle una fecha con otro formato
        valor = '10-8-12'
        self.assertRaises(Exception, self.generales.validar_fecha, valor)

        # intento pasarle un entero
        valor = 2010
        self.assertRaises(Exception, self.generales.validar_fecha, valor)

        # intento pasarle un booleano
        valor = True
        self.assertRaises(Exception, self.generales.validar_fecha, valor)

    def test_validar_hora(self):
        # caso correcto, le paso una hora en formato correcto
        valor = '10:53:22'
        r = self.generales.validar_hora(valor)
        self.assertTrue(r)

        # intento pasarle una hora que no exite
        valor = '10:61:22'
        self.assertRaises(Exception, self.generales.validar_hora, valor)

        # intento pasarle una hora en otro formato
        valor = '10.05.22'
        self.assertRaises(Exception, self.generales.validar_hora, valor)

        # intento pasarle un entero
        valor = 2010
        self.assertRaises(Exception, self.generales.validar_hora, valor)

        # intento pasarle un booleano
        valor = True
        self.assertRaises(Exception, self.generales.validar_hora, valor)

    def test_validar_mail(self):
        # paso un mail válido
        valor = 'paco@hotmail.com'
        r = self.generales.validar_mail(valor)
        self.assertTrue(r is True)

        # paso un mail sin @
        valor = 'pacohotmail.com'
        self.assertRaises(Exception, self.generales.validar_mail, valor)

        # paso un mail sin .
        valor = 'paco@hotmailcom'
        self.assertRaises(Exception, self.generales.validar_mail, valor)

        # paso un mail con un . entre medio
        valor = 'paco.algo@hotmail.com'
        r = self.generales.validar_mail(valor)
        self.assertTrue(r is True)

        # paso un mail con un . entre medio despues del arroba
        valor = 'paco.algo@hot.mail.com'
        r = self.generales.validar_mail(valor)
        self.assertTrue(r is True)

        # paso un mail con una extensión desconocida
        valor = 'paco@hotmail.asdqwe'
        r = self.generales.validar_mail(valor)
        self.assertTrue(r is True)

        # paso un mail con número y algunos símbolos
        valor = 'pa-co_18@hot17mail.net'
        r = self.generales.validar_mail(valor)
        self.assertTrue(r is True)

        # paso un mail con /
        valor = 'paco/rivas@asdasd.org'
        self.assertRaises(Exception, self.generales.validar_mail, valor)

        # paso un mail con :
        valor = 'paco:rivas@asdasd.int'
        self.assertRaises(Exception, self.generales.validar_mail, valor)

        # no le paso un str
        valor = 1234
        self.assertRaises(Exception, self.generales.validar_mail, valor)

        # no le paso un str
        valor = True
        self.assertRaises(Exception, self.generales.validar_mail, valor)

        # le paso un mail con acentos
        valor = 'pakó@canción.com'
        self.assertRaises(Exception, self.generales.validar_mail, valor)

        # le paso un mail con ñ
        valor = 'eñe@asd.com'
        self.assertRaises(Exception, self.generales.validar_mail, valor)

        # le paso un mail con espacios en blanco
        valor = 'paco rivas@asd.com'
        self.assertRaises(Exception, self.generales.validar_mail, valor)

        # no le paso un str
        valor = None
        r = self.generales.validar_mail(valor)
        self.assertTrue(r is True)

        # le paso un str vacío
        valor = ''
        r = self.generales.validar_mail(valor)
        self.assertTrue(r is True)

    def test_validar_date(self):
        # caso correcto
        valor = datetime.date(2011, 1, 27)
        r = self.generales.validar_date(valor)
        self.assertTrue(r is True)

        # le paso un str
        valor = '2011-01-27'
        self.assertRaises(Exception, self.generales.validar_date, valor)

        # le paso None
        valor = None
        r = self.generales.validar_date(valor)
        self.assertTrue(r is True)

        # le paso ''
        valor = ''
        self.assertRaises(Exception, self.generales.validar_date, valor)


def suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestValidacionesGenerales))
    return suite
