# -*- coding: utf-8 -*-

import re
import time
import datetime

from at.utiles.validar import numero_entero_positivo
from at.utiles.validar import string
from at.utiles.conversiones import respuesta
from at.utiles import _


class ValidacionesGenerales(object):
    '''
        Esta clase contiene métodos encargados de validar
        distinto tipo de valores.
    '''

    def validar_numero(self, valor):
        '''
            Valido que el valor ingresado sea un número
        '''

        if valor is None:
            return True

        if isinstance(valor, float):
            return True

        if not isinstance(valor, int):
            raise(ValueError)

        if isinstance(valor, bool):
            raise(ValueError)

        return True

    def validar_entero(self, valor):
        '''
            Valido que el valor ingresado sea un número entero
        '''

        if valor is None:
            return True

        if not isinstance(valor, int):
            raise(ValueError)

        if isinstance(valor, bool):
            raise(ValueError)

        return True

    def validar_numero_maximo(self, maximo, valor):
        '''
            Valido que el numero ingresado sea menor que su máximo
        '''

        if valor is None:
            return True

        self.validar_numero(valor)

        if valor > maximo:
            raise(Exception)

        return True

    def validar_positivo(self, valor):
        '''
            Valido que el valor ingresado sea un número positivo
        '''

        if valor is None:
            return True

        self.validar_numero(valor)

        if not valor >= 0:
            raise(Exception)

        return True

    def validar_no_vacio(self, valor):
        '''
            Valido que el valor ingresado no sea ni None ni un string vacío
        '''

        if valor is None:
            raise(Exception)

        if valor == '':
            raise(Exception)

        return True

    def validar_string(self, valor):
        '''
            Valido que el valor sea un string
        '''

        if valor is None:
            return True

        try:
            if not isinstance(valor, str) and not isinstance(valor, unicode):
                raise(ValueError)
        except:
            # NOTA: En Python 3 solo se valida si es un str
            if not isinstance(valor, str):
                raise(ValueError)

        return True

    def validar_string_maximo(self, maximo, valor):
        '''
            Valido que el string sea menor a su máximo
        '''

        self.validar_string(valor)

        if valor is not None:
            if len(valor) > maximo:
                raise(Exception)

        return True

    def validar_string_minimo(self, minimo, valor):
        '''
            Valido que el string sea igual o mayor a su mínimo
        '''

        self.validar_string(valor)

        if len(valor) < minimo:
            raise(Exception)

        return True

    def validar_string_no_vacio(self, valor):

        self.validar_string_minimo(1, valor)

        return True

    def validar_sin_espacios(self, valor):
        '''
            Valido que el string no contenga espacios en blanco
        '''

        self.validar_string(valor)

        if ' ' in valor:
            raise(Exception)

        return True

    def validar_sin_barras(self, valor):
        '''
            Valido que el string no contenga barras "/"
        '''

        self.validar_string(valor)

        if '/' in valor:
            raise(Exception)

        return True

    def validar_0_o_1(self, valor):
        '''
            Valido que el valor sea un 0 o un 1
        '''

        self.validar_entero(valor)

        if valor != 0 and valor != 1:
            raise(Exception)

        return True

    def validar_fecha_hora(self, valor):
        '''
            Valido que el valor sea una fecha y una hora en formato correcto
            para la base de datos
        '''

        try:
            time.strptime(valor, "%Y-%m-%d %H:%M:%S")
            return True
        except:
            raise(Exception)

    def validar_fecha(self, valor):
        '''
            Valido que el valor sea una fecha en formato correcto para la
            base de datos
        '''

        try:
            time.strptime(valor, "%Y-%m-%d")
            return True
        except:
            raise(Exception)

    def validar_hora(self, valor):
        '''
            Valido que el valor sea una hora en formato correcto para la
            base de datos
        '''

        try:
            time.strptime(valor, "%H:%M:%S")
            return True
        except:
            raise(Exception)

    def validar_funcion(self, valor):
        '''
            Valido que el valor sea una funcion
        '''

        if not hasattr(valor, '__call__'):
            raise(Exception)

        return True

    def validar_mail(self, valor):
        '''
            Esta función debe validar que el mail ingresado tenga un
            formato válido.
            Si recibe None o '' no hacer nada.
        '''

        if valor is None or valor is '':
            return True

        p = re.compile(r"(?:^|\s)[-a-z0-9_.]+@(?:[-a-z0-9]+\.)+[a-z]{2,6}(?:\s|$)", re.IGNORECASE)
        m = p.match(valor)
        if m is None:
            raise(Exception)
        else:
            return True

    def validar_date(self, valor):
        '''
            Esta función solo debe aceptar un date.
            Si recibe None no hacer nada.
        '''

        if valor is None:
            return True

        if isinstance(valor, datetime.date):
            return True
        else:
            raise(Exception)

    def convertir_a_entero(self, valor):
        try:
            print(valor,":>:::")
            int(valor)
            dato = valor
            rta = {"Estado": True, "Respuesta": "hay numero", "Dato": dato}
            return rta
        except:
            rta = {"Estado": False, "Respuesta": "no hay numeros"}
            return rta
            # return False

    def es_posible_direccion(self, variable):

        # print ("mensaje_sin_espacios", variable)
        # verifico que el elemento no este vacio
        if variable is None:
            # si esta vacio, devuelvo la respuesta
            rta = {"Estado": False, "Respuesta": "no hay texto ni numero"}
            return rta

        # por defecto, no tiene bis
        tiene_bis = False
        # variable donde guardo el texto sin el "bis"
        palabra_sin_bis = ""
        # donde voy a guardar la calle
        calle = ""
        # donde voy a guardar el extra
        extra = ""
        # donde guardo el numero despues de estar verificado
        numero = ""

        # si detecto bis en el mensaje recibido
        if "bis" in variable:
            # extraigo la palabra bis, del texto recibido..
            palabra_sin_bis = variable.replace('bis', '')
            # convierto tiene bis en true
            tiene_bis = True

        # Si detecto que no tiene un bis
        if tiene_bis is not True:
            # uso validar_elementos_lista.
            # convierto el texto recibido en dic
            validar_elementos_lista = variable.split()
            # print("validar_elementos_lista", validar_elementos_lista, tiene_
            # bis)
        else:
            # si tiene bis
            # convierto el texto con el bis extraido en dic
            validar_elementos_lista = palabra_sin_bis.split()
        # print("palabra_sin_bis", validar_elementos_lista, tiene_bis)

        # Si los elementos de la lista, son menores a 2 
        # no tengo calle o numero
        if len(validar_elementos_lista) < 2:
            # retorno el error
            rta = {"Estado": False,
                   "Respuesta": "Ingrese calle y numero porfavor"}
            return rta

            ############
        if variable is not None:
            elementos = variable.split()
            if len(elementos) > 1 :
                # for i in elementos:
                    print(type(elementos),
                          elementos )
                    rta_convertir_uno = self.convertir_a_entero(
                        elementos[0])
                    rta_convertir_dos = self.convertir_a_entero(
                        elementos[1])
                    # si no lo puedo convertir
                    if rta_convertir_uno['Estado'] is True & rta_convertir_dos['Estado'] is True:
                        print("no es entero", rta_convertir_uno,
                              rta_convertir_dos)
                        rta = {"Estado": False,
                               "Respuesta":
                                   "Ingrese primero la calle y luego la altura."
                                   ""}
                        return rta
            ############

        # Si los elementos de la lista, son mayores a 3
        if len(validar_elementos_lista) >= 3:

            # Obtengo el primer elemento de la lista y veo si es un str
            if type(validar_elementos_lista[0]) is str:
                # intento convertirlo a entero
                rta_convertir_entero = self.convertir_a_entero(
                    validar_elementos_lista[0])

            # si no lo puedo convertir
            if rta_convertir_entero['Estado'] is not True:

                # verifico que el ultimo elemento no sea un numero
                rta_convertir_entero2 = self.convertir_a_entero(
                    validar_elementos_lista[-1])
                # si lo es, no lo quito
                if rta_convertir_entero2['Estado'] is True:
                    # no lo quito, ya que, lo tengo que usar luego.
                    print("No lo quito")
                else:
                    # pero si no lo es, lo quito. y lo guardo en extra
                    # ya que si el primer elemento es un numero y el ante-utimo
                    # tambien, entonces, una calle similiar a 9 de julio 765 7c)
                    # ver el elemento: print('+3', validar_elementos_lista[-1])
                    # quito el ultimo elemento de la lista, ya que debe ser 7c
                    extra = validar_elementos_lista[-1]
                    validar_elementos_lista.remove(extra)
                    print("\n el extra", extra)
            else:
                # si tengo 4 o mas elementos en la lista
                if len(validar_elementos_lista) >= 4:
                    # intento convertir el ultimo
                    rta_convertir_entero = self.convertir_a_entero(
                        validar_elementos_lista[-1])
                    # si no lo puedo convertir
                    if rta_convertir_entero['Estado'] is not True:
                        # es que es un string
                        # print('+5', validar_elementos_lista[-1])
                        # quito el ultimo elemento de la lista, ya que debe ser
                        # 7c y lo pongo como un extra
                        extra = validar_elementos_lista[-1]
                        validar_elementos_lista.remove(extra)
                    else:
                        # print('+4', validar_elementos_lista[-1])
                        # quito el ultimo elemento de la lista,
                        # ya que debe ser 7c
                        numero = validar_elementos_lista[-1]
                        # validar_elementos_lista.remove(extra)

        else:
            # sino, convierto extra en None
            extra = None

        # recorro la lista
        for txt in validar_elementos_lista:
            # obtengo el ultimo elemento de ella
            variable_primer_palabra_numero = txt.split()[-1]

        # print
        # ('variable_primer_palabra_numero', variable_primer_palabra_numero)

        # Si el ultimo elemento es un str
        if type(variable_primer_palabra_numero) is str:
            # intento convertirlo a entero
            rta_convertir_entero = self.convertir_a_entero(
                variable_primer_palabra_numero)

            # si no lo puedo convertir
            if rta_convertir_entero['Estado'] is not True:
                # quiere decir que el ultimo elemento no es un numero
                # por lo que devuelvo la respuesta
                rta = {"Estado": False, "Respuesta":
                    "Ingrese primero la calle y luego la altura."}
                # retorno la rta
                return rta
            else:
                # Si el ultimo elemento es un numero,
                # lo guardo en la variable para  enviarlo en los datos.
                numero = rta_convertir_entero['Dato']
                # y lo quito de la lista
                validar_elementos_lista.remove(numero)
                # calle = str(validar_elementos_lista).strip('[]').strip("''")
                # convierto a string
                calle = " ".join(map(str, validar_elementos_lista))

            # Ordeno los datos.
            if extra == '':
                extra = None

            # Mando esto si tiene bis
            if tiene_bis:

                datos = {
                        "calle": calle,
                        "numero": numero,
                        "extra": extra,
                        "bis": tiene_bis
                    }
            else:
                # Si el ultimo elemento en mi lista, es un numero, tomo que
                # el primero es la calle. se convierte la lista en str
                calle = " ".join(map(str, validar_elementos_lista))
                # calle = validar_elementos_lista[0]
                print('calle sin bis', calle)
                datos = {
                    "calle": calle,
                    "numero": numero,
                    "extra": extra,
                    "bis": tiene_bis
                }

            # Retorno True y los datos importantes
            return respuesta(True, _(u'ok'), datos)