# -*- coding: utf-8 -*-

import re
import datetime

from at.utiles import _


def __parse(estado, respuesta):
    return {
        'Estado': estado,
        'Respuesta': respuesta
    }


def no_vacio(valor, nombre_dato=''):
    '''
        Valido que el valor ingresado no sea vacio o None.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que no sea None ni un str vacío
    if valor is None or valor == '':
        return __parse(False, _(u'El dato%s no debe ser vacío.') % nombre_dato)

    return __parse(True, _(u'El dato%s no es vacío.') % nombre_dato)


def numero(valor, nombre_dato=''):
    '''
        Valido que el valor ingresado sea un número.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido  que no sea vacio.
    v_vacio = no_vacio(valor, nombre_dato)
    if v_vacio['Estado'] is not True:
        return v_vacio

    # Valido que sea un número.
    if isinstance(valor, bool) \
            or (not isinstance(valor, int) and not isinstance(valor, float)):
        return __parse(False, _(u'El dato%s debe ser un número.') % nombre_dato)

    return __parse(True, _(u'El dato%s es un número.') % nombre_dato)


def string(valor, nombre_dato=''):
    '''
        Valido que el valor ingresado sea un string.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido  que no sea vacio.
    v_vacio = no_vacio(valor, nombre_dato)
    if v_vacio['Estado'] is not True:
        return v_vacio

    try:
        # Valido que sea string
        if not isinstance(valor, str) and not isinstance(valor, unicode):
            return __parse(False, _(u'El dato%s debe ser un string.') % nombre_dato)
    except:
        # NOTA: En Python 3 solo se valida si es un str
        if not isinstance(valor, str):
            return __parse(False, _(u'El dato%s debe ser un string.') % nombre_dato)

    return __parse(True, _(u'El dato%s es un string.') % nombre_dato)


def lista_valores_no_vacio(lista, nombre_dato=''):
    '''
        Valido que los valores ingresados en la lista no sean vacío o None.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Si el dato no es una lista
    if not isinstance(lista, list):
        # Retorno False y un mensaje
        return __parse(False, _(u'El dato%s debe ser una lista') % nombre_dato)

    # Verifico que la lista tenga datos
    if len(lista) < 1:
        return __parse(False, _(u'El dato%s no tiene valores') % nombre_dato)

    for valor in lista:
        v = no_vacio(valor, nombre_dato)
        if v['Estado'] is not True:
            return __parse(
                False, _(u'El dato%s tiene un valor que es vacío') % nombre_dato
            )

    return __parse(True, _(u'El dato%s tiene valores') % nombre_dato)


def lista_valores_numero(lista, nombre_dato=''):
    '''
        Valido que los valores ingresados en la lista sean numeros.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Si el dato no es una lista
    if not isinstance(lista, list):
        # Retorno False y un mensaje
        return __parse(False, _(u'El dato%s debe ser una lista') % nombre_dato)

    # Verifico que la lista tenga datos
    if len(lista) < 1:
        return __parse(False, _(u'El dato%s no tiene valores') % nombre_dato)

    for valor in lista:
        v = numero(valor, nombre_dato)
        if v['Estado'] is not True:
            return __parse(
                False,
                _(u'El dato%s tiene un valor que no es un número') % nombre_dato
            )

    return __parse(
        True, _(u'El dato%s tiene todos valores que son números') % nombre_dato
    )


def lista_valores_string(lista, nombre_dato=''):
    '''
        Valido que los valores ingresados en la lista sean numeros.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Si el dato no es una lista
    if not isinstance(lista, list):
        # Retorno False y un mensaje
        return __parse(False, _(u'El dato%s debe ser una lista') % nombre_dato)

    # Verifico que la lista tenga datos
    if len(lista) < 1:
        return __parse(False, _(u'El dato%s no tiene valores') % nombre_dato)

    for valor in lista:
        v = string(valor, nombre_dato)
        if v['Estado'] is not True:
            return __parse(
                False,
                _(u'El dato%s tiene un valor que no es un string') % nombre_dato
            )

    return __parse(
        True, _(u'El dato%s tiene todos valores que son strings') % nombre_dato
    )


def numero_positivo(valor, incluye_cero=False, nombre_dato=''):
    '''
        Valido que el valor ingresado sea un número positivo.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que sea un número.
    v = numero(valor, nombre_dato)
    if v['Estado'] is not True:
        return v

    if incluye_cero is True:
        if valor < 0:
            return __parse(
                False,
                _(u'El dato%s debe ser un número positivo.') % nombre_dato
            )
    else:
        if valor <= 0:
            return __parse(
                False,
                _(u'El dato%s debe ser un número positivo.') % nombre_dato
            )

    return __parse(True, _(u'El dato%s es un número positivo.') % nombre_dato)


def lista_valores_numero_positivo(lista, nombre_dato=''):
    '''
        Valido que los valores ingresados en la lista sean numeros positivos.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Si el dato no es una lista
    if not isinstance(lista, list):
        # Retorno False y un mensaje
        return __parse(False, _(u'El dato%s debe ser una lista') % nombre_dato)

    # Verifico que la lista tenga datos
    if len(lista) < 1:
        return __parse(False, _(u'El dato%s no tiene valores') % nombre_dato)

    for valor in lista:
        v = numero_positivo(valor)
        if v['Estado'] is not True:
            return __parse(
                False,
                _(u'El dato%s tiene un valor que no es un número positivo')
                % nombre_dato
            )

    return __parse(
        True,
        _(u'El dato%s tiene todos valores que son números positivos')
        % nombre_dato
    )


def numero_entero(valor, nombre_dato=''):
    '''
        Valido que el valor ingresado sea un número entero.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que sea un número.
    v = numero(valor, nombre_dato)
    if v['Estado'] is not True:
        return v

    # Valido que sea entero. Incluye el 0.
    if type(valor) != int:
        return __parse(
            False, _(u'El dato%s debe ser un número entero.') % nombre_dato
        )

    return __parse(True, _(u'El dato%s es un número entero.') % nombre_dato)


def numero_entero_positivo(valor, incluye_cero=False, nombre_dato=''):
    '''
        Valido que el valor ingresado sea un número entero.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que sea un número entero.
    v = numero_entero(valor, nombre_dato)
    if v['Estado'] is not True:
        return v

    # Valido que sea positivo.
    v_p = numero_positivo(
        valor, incluye_cero=incluye_cero, nombre_dato=nombre_dato
    )
    if v_p['Estado'] is not True:
        return v_p

    # Si pasa las validaciones, retorno True.
    return __parse(
        True,
        _(u'El dato%s es un número entero positivo.') % nombre_dato
    )


def lista_numero_entero_positivo(lista, nombre_dato=''):
    '''
        Valido que los valores ingresados en la lista sean numeros enteros
        positivos.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Si el dato no es una lista
    if not isinstance(lista, list):
        # Retorno False y un mensaje
        return __parse(False, _(u'El dato%s debe ser una lista') % nombre_dato)

    # Verifico que la lista tenga datos
    if len(lista) < 1:
        return __parse(False, _(u'El dato%s no tiene valores') % nombre_dato)

    for valor in lista:
        v = numero_entero_positivo(valor)
        if v['Estado'] is not True:
            return __parse(
                False,
                _(u'El dato%s tiene un valor que no es un número entero positivo')
                % nombre_dato
            )

    return __parse(
        True,
        _(u'El dato%s tiene todos valores que son números enteros positivos')
        % nombre_dato
    )


def fecha_hora(fecha, nombre_dato=''):
    '''
        Funcion para validar un datetime.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    if not isinstance(fecha, datetime.datetime):
        return __parse(
            False,
            _(u'El dato%s debe ser un datetime.') % nombre_dato
        )

    return __parse(
        True,
        _(u'El dato%s es una fecha y hora válida.') % nombre_dato
    )


def email(direccion, nombre_dato=''):
    '''
        Funcion para validar un correo.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Expresión regular.
    match = re.match('^[\wA-Z.%+-]+@[\w.-]+\.[a-zA-Z]{2,6}$', direccion)
    # match = re.match('^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$', direccion)
    # Vamos si salio mal
    if match is None:
        return __parse(
            False,
            _(u'El dato%s no es un correo válido.') % nombre_dato
        )

    # Sino, salimos bien
    return __parse(True, _(u'El dato%s es un correo válido.') % nombre_dato)


def booleano(valor, nombre_dato=''):
    '''
        Funcion para validar un booleano.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    if isinstance(valor, bool):
        return __parse(True, _(u'El dato%s es booleano.') % nombre_dato)

    if isinstance(valor, int):
        if valor == 1 or valor == 0:
            return __parse(True, _(u'El dato%s es booleano.') % nombre_dato)

    return __parse(False, _(u'El dato%s no es booleano.') % nombre_dato)


def zona_horaria(valor, nombre_dato=''):
    '''
        Funcion para validar un numero de zona horaria.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    rta = numero_entero(valor, nombre_dato)
    if not rta['Estado']:
        return rta

    if valor > 12 or valor < -11:
        return __parse(
            False,
            _(u'El dato%s debe ser desde -11 a +12, incluidos.') % nombre_dato
        )

    return __parse(
        True,
        _(u'El dato%s es una zona horaria válida.') % nombre_dato
    )


def diccionario(valor, nombre_dato=''):
    '''
        Funcion para validar un diccionario no vacio.
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    if not isinstance(valor, dict):
        return __parse(
            False,
            _(u'El dato%s debe ser un diccionario.') % nombre_dato
        )

    if len(valor.keys()) == 0:
        return __parse(False, _(u'El dato%s debe tener llaves.') % nombre_dato)

    return __parse(
        True,
        _(u'El dato%s es un diccionario válido.') % nombre_dato
    )


def dic_con_datos(valor, nombre_dato=''):
    '''
        Esta función se encarga de validar que sea un diccionario con llaves
        con valores distintos a None.

        Argumento:
            valor (dict)
            nombre_dato (str)

        Funciones:
            validar -> diccionario

        Retorna
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Primero valido si es un diccionario
    rta_diccionario = diccionario(valor, nombre_dato)
    # Si no es un diccionario
    if rta_diccionario['Estado'] is False:
        # Retorno False y un mensaje
        return rta_diccionario

    # Por defecto indico que el diccionario no tiene datos
    dic_sin_datos = True
    # Recorro las llaves del diccionario
    for llave in valor:
        # Si tengo una llave con un valor
        if valor[llave] is not None:
            # Indico que el diccionario tiene datos
            dic_sin_datos = False
            # Termino el for
            break

    # Si es un diccionario sin datos
    if dic_sin_datos is True:
        # Retorno False y un mensaje
        return __parse(
            False,
            _(u'El dato%s tiene llaves sin datos.') % nombre_dato
        )

    # Si es un diccionario con llaves y datos retorno True y un mensaje
    return __parse(
        True,
        _(u'El dato%s es un diccionario con llaves y datos.') % nombre_dato
    )


def usuario(valor, nombre_dato=''):
    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que sea string.
    v_string = string(valor, nombre_dato)
    if v_string['Estado'] is not True:
        return v_string

    # Expresión regular.
    exp_reg = re.match('^[_a-zA-Z0-9-]{3,20}$', valor)
    # Verifico si cumple la expresión regular.
    if exp_reg is None:
        return __parse(
            False,
            _(u'El dato%s no es un usuario válido.') % nombre_dato
        )

    # Si sale todo bien retorno.
    return __parse(True, _(u'El dato%s es un usuario válido.') % nombre_dato)


def nombre(valor, nombre_dato=''):
    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que sea string.
    v_string = string(valor, nombre_dato)
    if v_string['Estado'] is not True:
        return v_string

    # Expresión regular.
    exp_reg = re.match("^\S[a-zA-Z\sáéíóúÁÉÍÓÚ']{3,50}$", valor)
    # Verifico si cumple la expresión regular.
    if exp_reg is None:
        return __parse(
            False,
            _(u'El dato%s no es un nombre válido.') % nombre_dato
        )

    # Si sale todo bien retorno.
    return __parse(True, _(u'El dato%s es un nombre válido.') % nombre_dato)


def clave(valor, nombre_dato=''):
    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que sea string.
    v_string = string(valor, nombre_dato)
    if v_string['Estado'] is not True:
        return v_string

    # Expresión regular.
    exp_reg = re.match("^[a-zA-Z0-9-_+*/!#$%&()=?¿¡!]{6,30}$", valor)
    # Verifico si cumple la expresión regular.
    if exp_reg is None:
        return __parse(False, _(u'El dato%s no es clave válida.') % nombre_dato)

    # Si sale todo bien retorno.
    return __parse(True, _(u'El dato%s es una clave válida.') % nombre_dato)


def fecha_date(fecha, nombre_dato=''):
    '''
        Funcion para validar un date

        Argumentos:
            fecha (date)
            nombre_dato (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Si la fecha no es del tipo date
    if not isinstance(fecha, datetime.date):
        # Retorno False y un mensaje
        return __parse(False, _(u'El dato%s debe ser un date') % nombre_dato)

    # Si la fecha es del tipo date retorno True y un mensaje
    return __parse(True, _(u'El dato%s es una fecha válida') % nombre_dato)


def fecha_min(fecha, nombre_dato=''):
    '''
        Esta función se encarga de validar que la fecha dada sea mayor a
        1900-01-01.
        Puede recibir una fecha como una fecha y hora.

        Argumentos:
            fecha (date, datetime o str // yyyy-mm-dd o yyy-mm-dd hh:mm:ss)
            nombre_dato (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    fecha_min_predeterminada = datetime.datetime(1900, 1, 1)
    try:
        fecha = datetime.datetime.strptime(str(fecha)[:10], '%Y-%m-%d')
    except:
        return __parse(
            False, _(u'El dato%s no es una fecha válida') % nombre_dato
        )

    if fecha < fecha_min_predeterminada:
        return __parse(
            False,
            _(u'El dato%s no puede ser menor a 1900-01-01') % nombre_dato
        )

    return __parse(True, _(u'El dato%s es una fecha válida') % nombre_dato)


def es_lista(dato, min_elementos=None, max_elementos=None, nombre_dato=''):
    '''
        Esta función se encarga de validar que el dato sea una lista y que tenga
        un mínimo de registros.

        Argumentos:
            dato (list)
            min_elementos (int)
            max_elementos (int)
            nombre_dato (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Si el dato no es una lista
    if not isinstance(dato, list):
        # Retorno False y un mensaje
        return __parse(False, _(u'El dato%s debe ser una lista') % nombre_dato)

    # Si la lista no contiene el mínimo de elementos indicado
    if min_elementos is not None and len(dato) < min_elementos:
        # Retorno False y un mensaje
        return __parse(
            False,
            _(u'El dato%s debe ser una lista con un mínimo de %s elementos')
            % (nombre_dato, min_elementos)
        )

    # Si la lista no contiene el máximo de elementos indicado
    if max_elementos is not None and len(dato) > max_elementos:
        # Retorno False y un mensaje
        return __parse(
            False,
            _(u'El dato%s debe ser una lista con un máximo de %s elementos')
            % (nombre_dato, max_elementos)
        )

    # Si el dato es una lista retorno True y un mensaje
    return __parse(True, _(u'El dato%s es una lista') % nombre_dato)


def numero_entre(valor, num_minimo, num_maximo, nombre_dato=''):
    '''
        Esta función se encarga de validar que un número
        este entre dos números dados.

        Argumentos:
            valor (int)
            num_minimo (int)
            num_maximo (int)
            nombre_dato (str)

        Funciones:
            at.utiles -> validar.py -> numero

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que el valor recibido sea un número
    rta = numero(valor, nombre_dato)
    if rta['Estado'] is False:
        return rta

    # Valido que num_minimo sea un número
    rta = numero(num_minimo, 'num_minimo')
    if rta['Estado'] is False:
        return rta

    # Valido que num_maximo sea un número
    rta = numero(num_minimo, 'num_maximo')
    if rta['Estado'] is False:
        return rta

    # Si el valor dado es menor al valor mínimo o es mayor al valor máximo
    if valor < num_minimo or valor > num_maximo:
        # Retorno False y un mensaje de error
        return __parse(
            False,
            _(u'El dato%s debe ser mayor o igual que %s y menor o igual que %s')
            % (nombre_dato, num_minimo, num_maximo)
        )

    # Si el valor está entre los dos número dados retorno True y un mensaje
    return __parse(
        True,
        _(u'El dato%s está entre %s y %s')
        % (nombre_dato, num_minimo, num_maximo)
    )


def horario(valor, nombre_dato=''):
    '''
        Esta función se encarga de validar que el dato recibido sea un horario.

        Argumentos:
            valor (str) // HH:MM
            nombre_dato (str)

        Funciones:
            at.utiles -> validar.py -> string

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que el valor recibido sea un str
    rta = string(valor, nombre_dato)
    if rta['Estado'] is False:
        return rta

    # Si el string tiene un tamaño de 5 caracteres
    # quiere decir que le faltan los segundos
    if len(valor) == 5:
        # Agrego los segundos al valor
        valor_tmp = '%s:00' % valor
    else:
        # Si no, dejo el valor como lo recibo
        valor_tmp = valor

    # Intento pasar el horario a datetime
    try:
        # Agrego una fecha al horario
        datetime.datetime.strptime(
            '1900-01-01 %s' % valor_tmp, '%Y-%m-%d %H:%M:%S'
        )
    except:
        # Si no se puede pasar el string a datetime retorno False y un mensaje
        return __parse(
            False, _(u'El dato%s debe ser un horario válido') % nombre_dato
        )

    # Si se pudo pasar el string a datetime retorno True y un mensaje
    return __parse(True, _(u'El dato%s es un horario válido') % nombre_dato)


def fecha_hora_str(valor, nombre_dato=''):
    '''
        Esta función se encarga de validar si una fecha y hora
        con formato str es válida.

        Argumentos:
            valor (str) // yyyy-mm-dd HH:MM:SS
            nombre_dato (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que el valor sea un str con formato datetime,
    # se debe poder pasar de string a datetime
    try:
        datetime.datetime.strptime(valor, '%Y-%m-%d %H:%M:%S')
    except:
        return __parse(
            False,
            _(u'El dato%s debe ser una fecha y hora válida') % nombre_dato
        )

    # Si se pudo pasar a datetime es porque el str tiene un formato válido
    return __parse(
        True,
        _(u'El dato%s es una fecha y hora válida') % nombre_dato
    )


def fecha_str(valor, nombre_dato=''):
    '''
        Esta función se encarga de validar si una fecha
        con formato str es válida.

        Argumentos:
            valor (str) // yyyy-mm-dd
            nombre_dato (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que el valor sea un str con formato date,
    # se debe poder pasar de string a datetime
    try:
        datetime.datetime.strptime(valor, '%Y-%m-%d')
    except:
        return __parse(
            False,
            _(u'El dato%s debe ser un string con formato de fecha')
            % nombre_dato
        )

    # Si se pudo pasar a date es porque el str tiene un formato válido
    return __parse(
        True,
        _(u'El dato%s es un string con formato de fecha') % nombre_dato
    )


def __llave(dic, nombre_llave, nombre_dato=''):
    '''
        Esta función se encarga de validar si el diccionario
        tiene una llave dada.

        Argumentos:
            dic (dict)
            nombre_llave (str)
            nombre_dato (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Si el diccionario no tiene la llave dada
    if dic.get(nombre_llave) is None:
        # Retorno False y un mensaje
        return __parse(
            False,
            _(u'El dato%s no tiene la llave "%s"') % (nombre_dato, nombre_llave)
        )

    # Si el diccionario tiene la llave dada retorno True y un mensaje
    return __parse(
        True,
        _(u'El dato%s tiene la llave "%s"') % (nombre_dato, nombre_llave)
    )


def llaves(dic, nombres_llaves, nombre_dato=''):
    '''
        Esta función se encarga de validar si el diccionario
        tiene todas las llaves dadas.

        Argumentos:
            dic (dict)
            nombres_llaves (list de str)
            nombre_dato (str)

        Funciones:
            at.utiles -> validar -> __llave

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si recibo el nombre del dato
    if nombre_dato != '':
        # Le agrego un espacio en blanco y lo pongo entre comillas
        # Por las dudas le saco las comillas y los espacios en blanco primero
        nombre_dato = ' "%s"' % nombre_dato.strip().replace('"', '')

    # Valido que el dato "dic" sea un diccionario
    rta = diccionario(dic, nombre_dato)
    # Si no es un diccionario
    if rta['Estado'] is False:
        # Retorno el error
        return rta

    # Valido que el dato "nombres_llaves" sea una lista
    # con un mínimo de 1 elemento
    rta = es_lista(nombres_llaves, min_elementos=1, nombre_dato=nombre_dato)
    # Si no es una lista
    if rta['Estado'] is False:
        # Retorno el error
        return rta

    # Recorro cada llave del diccionario
    for nombre_llave in nombres_llaves:
        # Valido que el diccionario tenga cada llave
        rta = __llave(dic, nombre_llave, nombre_dato)
        # Si el diccionario no tiene la llave dada
        if rta['Estado'] is False:
            # Retorno el error
            return rta

    # Si el diccionario tiene todas las llaves dadas retorno True y un mensaje
    return __parse(
        True,
        _(u'El dato%s tiene todas las llaves dadas') % nombre_dato
    )
