# -*- coding: utf-8 -*-

import json

from at.utiles import _


class VariablesLocales(object):
    '''
        Esta clase contiene funciones para poder guardar las variables locales
        del usuario.
    '''

    def grabar_variables_locales(self, variable, ruta):
        '''
            Esta función se encarga crear un archivo con variables
            que van a pertenecer a un solo puesto de trabajo.

            Argumentos:
                variable
                ruta (str) /ruta/archivo

            Retorna:
                Caso correcto
                    dic['Estado'] = True
                    dic['Mensaje'] = None
                Caso incorrecto
                    dic['Estado'] = False
                    dic['Mensaje'] = 'un mensaje'
        '''

        dic = {}

        try:
            # creamos un archivo .py que contendr� las variables
            archivo = open(ruta, 'w')
            # crear carpeta oculta en la ruta correspondiente

            # creo el contenido del archivo en formato json
            contenido = json.dumps(variable)

            # escribo en el archivo
            archivo.write(contenido)

            # cerramos el archivo
            archivo.close()

            dic['Estado'] = True
            dic['Mensaje'] = None
            return dic
        except:
            dic['Estado'] = False
            dic['Mensaje'] = _(u'No se pudo crear el archivo con variables locales.')
            return dic

    def cargar_variables_locales(self, ruta):
        '''
            Esta función se encarga de obtener las variables locales
            que están en un archivo que pertenece a un solo puesto de trabajo.

            Argumentos:
                ruta (str) /ruta/archivo

            Retorna:
                Caso correcto
                    dic['Estado'] = True
                    dic['Variable'] = // contenido del archivo
                Caso incorrecto
                    dic['Estado'] = False
                    dic['Mensaje'] = 'un mensaje'
        '''

        dic = {}

        try:
            archivo = open(ruta, 'r')

            variable = archivo.read()

            parametros_puesto = json.loads(str(variable))

            archivo.close()

            dic['Estado'] = True
            dic['Variable'] = parametros_puesto
            return dic
        except:
            dic['Estado'] = False
            dic['Mensaje'] = _(u'No se pudieron obtener las variables locales del puesto de trabajo.')
            return dic
