# -*- coding: utf-8 -*-

import sys
import logging
import argparse

from at.orm.mapper import Database
from at.orm.mapper import DatabaseManager
from model.cpbot.schema.schema import CpBotModel
from model.cpbot.rawsql import RAW_QUERIES


def merge_schema_with_db(user, passwd, db, host, engine, verbose=True):
    '''
        Comparamos la estructura de base de datos definida en nuestro
        schema.py con respecto a la existente en la base de datos destino
        y realizamos los cambios necesarios, como agregar/modificar campos.
        Params:
            user(str) = usuario de la db
            passwd(str) = password de la db
            db(str) = nombre de la base de datos
            host(str) = host de la base de datos
            engine(str) = motor de la db, por defecto es 'mysql'
            verbose=(bool) = loguea mucha info en el log
    '''

    mydb = Database(
        user=user, passwd=passwd, name=db, host=host, engine=engine
    )
    manager = DatabaseManager()
    manager.addDB(mydb)

    db = getattr(getattr(manager, engine), db)
    if verbose:
        logging.info('>> mergeando contra %s en host %s' % (db.name, db.host))

    model = CpBotModel(db)
    model.merge_with_db()
    if verbose:
        logging.info('>> merge completo!')

    for qw in RAW_QUERIES:
        try:
            db.engine.execute(qw)
        except Exception as ex:
            logging.critical('error: %s' % ex)


def ejecutar():
    # Creo la descripción del console-script
    parser = argparse.ArgumentParser(description='BD - Modelo de cpbot')

    # Indico los argumentos que necesito, son todos obligatorios
    parser.add_argument('-u', dest='usuario', required=True)
    parser.add_argument('-p', dest='password', required=True)
    parser.add_argument('-db', dest='db', required=True)
    parser.add_argument('-host', dest='host', required=True)

    # Guardo los parámetros
    args = []
    args.extend(sys.argv)
    args.pop(0)
    args = parser.parse_args(args)

    # Llamo a la función merge_schema_with_db
    merge_schema_with_db(
        args.usuario, args.password, args.db, args.host, 'postgres'
    )
