# -*- coding: utf-8 -*-

import os
import sys
import argparse
from configobj import ConfigObj

import model.cpbot
from at.orm.abm import ABM


def registros_por_defecto(usuario, password, db, host, engine, port):
    '''
        Esta función se encarga de agregar todos los registros por defecto
        que debe tener la base de datos.
    '''

    # Iniciamos los datos
    datos_bd = {
        'user': usuario,
        'passwd': password,
        'dbname': db,
        'name': db,
        'host': host,
        'engine': engine,
        'port': port
    }

    # Creamos el ABM de la base de datos
    clase_abm = ABM(datos_bd)
    # Buscamos los registros por defecto
    ruta_registros = os.path.join(
        model.cpbot.__path__[0],
        'schema',
        'registros.txt'
    )
    # Si no hay registros por defecto, salimos!!!
    if not os.path.isfile(ruta_registros):
        return 'No existe el archivo de registros por defecto en el directorio %s' % ruta_registros
    # Sino, cargamos los mismos
    datos = ConfigObj(ruta_registros, encoding='UTF8')
    # Recorremos los datos obtenidos
    for tabla in datos.keys():
        for num_registro in datos[tabla]:
            # Agregamos los datos
            dic = datos[tabla][num_registro]
            for key in dic:
                try:
                    dato = dic[key]
                    # Borramos la key en unicode
                    del dic[key]
                    # Guardamos en string
                    # NOTA: En Python 3 no hace falta hacer el encode
                    #dic[str(key)] = dato.encode('latin1')
                    dic[str(key)] = dato
                except Exception as e:
                    print('ERROR DE ENCODING:')
                    print(dic[key])
                    print(e)
            try:
                clase_abm.agregar(tabla, **dic)
            except Exception as e:
                print('ERROR: Agregar en tabla: %s' % str(tabla))
                print(dic)
                print(e)

    # Salimos
    return '* Todos los registros por defecto de la BD cpbot se han agregado correctamente *'


def ejecutar():
    # Creo la descripción del console-script
    parser = argparse.ArgumentParser(
        description='BD - Registros de cpbot'
    )

    # Indico los argumentos que necesito, son todos obligatorios
    parser.add_argument('-u', dest='usuario', required=True)
    parser.add_argument('-p', dest='password', required=True)
    parser.add_argument('-db', dest='db', required=True)
    parser.add_argument('-host', dest='host', required=True)

    # Guardo los parámetros
    args = []
    args.extend(sys.argv)
    args.pop(0)
    args = parser.parse_args(args)

    # Llamo a la función registros_por_defecto
    registros = registros_por_defecto(
        args.usuario, args.password, args.db, args.host, 'postgres', '5432'
    )
    print(registros)
