# -*- coding: utf-8 -*-
import datetime
import pprint
from decimal import Decimal

from srvweb.cpbot import nombre_producto
from at.utiles.decoradores import con_log
from at.utiles.decoradores import con_try
import at.cpbot.funciones as funciones
from at.cpbot.bdcpbot import CpBotBd
import at.cpbot.tagutiles as tagutiles
from at.cpbot.atmsjadapter import finalizar_charla
from at.utiles.http import enviar_por_http
from at.utiles.conversiones import respuesta, str_to_array, remover_duplicados_lista, lista_to_string, quitar_acentos, \
    unicode_to_str
import at.cpbot.modospermitidos as permitidos
import at.cpbot.atmsjadapter as atmsjadapter
from at.utiles.direcciones import reemplazar_string
from at.cpbot.domain.filtro import obtener_filtros
from at.cpbot.sqla_repository_charla import CharlasRepository

cpBotBd = CpBotBd()
sqla_repository_charla = CharlasRepository()


# @con_log(nombre_producto)
# @con_try(nombre_producto)
def nuevo_msj(datos_usuario):
    # raise Exception
    '''
    Esta funcion se encarga de verificar que el mensaje recibido es valido, tambien lo procesa para posicionar al usuario
    en el debido nodo y ejecutar la funcion necesaria si el mensaje coincide con el menu otorgado

    :param datos_usuario: Un diccionario con los datos:
                tel_origen: El telefono del usuario
                tel_destino: El telefono del radiotaxi
                msj: El cuerpo del mensaje
                enviado_por: El nombre de la persona que envio el mensaje ('')
                tipo: Si el mensaje es entrante o no
                fecha_hora: La fecha y hora en que se recibio el mensaje
                clase_msj: La clase del mensaje ('audio', 'mapa', 'txt')

    :return: -
    '''
    print("mensaje es:", datos_usuario['msj'], type(datos_usuario['msj']))

    rta = sqla_repository_charla.obtener_charla(datos_usuario['tel_origen'])  # trata de obtener la charla

    existe_charla = rta['Estado']

    datos_usuario['msj'] = reemplazar_caracteres(datos_usuario['msj'])  # si envia 1) le quita el parentesis

    # agrega los filtros encontrados en caso de haber
    filtros_encontrados = buscar_filtros_en_mensaje(datos_usuario['msj'])
    datos_usuario['filtros_palabras'] = filtros_encontrados.get('Dato')  # ojo que si no hay nada guarda NONE

    if existe_charla:
        return _charla_vieja(datos_usuario, rta)
    else:
        return _charla_nueva(datos_usuario)


def buscar_filtros_en_mensaje(mensaje):
    """
        busca si algun filtro de filtros_palabras esta en el mensaje recibido
        si es asi devuelve lista de ids
        entra: unicode
        sale: Estado,Respuesta,Dato con lista de ids
    """
    if not mensaje:
        return respuesta(False, u'no hay mensaje')

    filtros_palabras = obtener_filtros()  # agarra los filtros palabras guardados en memoria
    if not filtros_palabras:
        return respuesta(False, u'no hay filtros palabras guardados')

    mensaje = quitar_acentos(mensaje)['Respuesta']

    filtros_encontrados = []
    for filtro in filtros_palabras:

        # validacion porque si envia corrientes 123 y esta el filtro corriente lo toma
        if len(filtro['palabra'].split()) == 1:
            if filtro['palabra'] in mensaje.split():
                filtros_encontrados.append(filtro['IdFiltro'])

        # validacion para agarrar cta cte. que en la version 1.0.7 ya lo tomaba
        elif len(filtro['palabra'].split()) > 1:
            if filtro['palabra'] in mensaje:
                filtros_encontrados.append(filtro['IdFiltro'])

    if not filtros_encontrados:
        return respuesta(False, u"no hubo coincidencias")

    filtros_encontrados = remover_duplicados_lista(filtros_encontrados)  # porque a lo mejor encontro el mismo idfiltro

    filtros_encontrados = lista_to_string(filtros_encontrados, ',')  # pasa la lista a string.
    # lo separe de la otra funcion por el principio single responsability

    return respuesta(True, u"ok", filtros_encontrados)


def reemplazar_caracteres(mensaje):
    """
      si envia 1) le quita el parentesis y devuelve el 1 solo.
    """

    if len(mensaje) < 4:
        for i in ",.;()?":
            mensaje = mensaje.replace(i, "")
    return mensaje


def _charla_vieja(datos_usuario, rta):
    print("\n ojo que ya existe la charla#############################")

    # actualizo aca los filtros
    sqla_repository_charla.actualizar_filtros(rta['Dato']['id_charla'], datos_usuario['filtros_palabras'])

    parametros = obtener_parametros_cliente(datos_usuario)  # Guardamos los parametros del cliente

    charla = rta['Dato']  # Guarda los datos de la charla
    actualizar_fecha_charla(charla)  # Actualizamos la fecha y hora del ultimo mensaje de la charla

    seleccion_usuario, direccion = funciones.obtener_clase_msj(datos_usuario['clase'], datos_usuario['msj'])

    # Agrego a los datos_usuario las variables asignadas
    datos_usuario['cliente'] = parametros
    datos_usuario['charla'] = charla
    datos_usuario['direccion'] = direccion  # en caso que haya enviado una direccion en el mensaje
    datos_usuario['seleccion_usuario'] = seleccion_usuario
    id_charla = charla['id_charla']  # Guardo el id charla en una variable
    # breakpoint()
    # Intentamos obtener todas las opciones disponibles para lo que ingreso el usuario
    logs_coincidentes_con_tipo_mensaje_recibido = cpBotBd.obtener_logs_coincidentes(seleccion_usuario, id_charla)

    # Si no encontró algun log que coincida con lo que ingreso el usuario
    if logs_coincidentes_con_tipo_mensaje_recibido['Estado'] is False:
        return opcion_elegida_erronea(datos_usuario)  # Utilizo la funcion para ejecutar la logica
    else:  # Si coincide con alguna opcion de logs

        logs = logs_coincidentes_con_tipo_mensaje_recibido[
            'Dato']  # Guardo los logs obtenidos a partir de lo que ingreso el usuario

        # Valido si lo que ingreso el usuario es valido respecto a lo que se esperaba
        rta_validar_mensaje = funciones.validar_mensaje(logs, datos_usuario, seleccion_usuario)

        print('\n\nopcion :', rta_validar_mensaje)
        # Si lo que ingreso el usuario no es valido con lo disponible
        if rta_validar_mensaje['Estado'] is False:

            if 'Dato' in rta_validar_mensaje and rta_validar_mensaje['Dato'] is 'nocheck':
                datos_usuario['nocheck'] = 'nocheck'

            else:
                datos_usuario['nocheck'] = None

            rta_opcion_elegida = opcion_elegida_erronea(datos_usuario)  # Utilizo la funcion para ejecutar la logica

            return rta_opcion_elegida  # Devuelvo la respuesta

        datos_usuario['log'] = rta_validar_mensaje['Dato']['log']  # Guardo el log correspondiente en los datos

        if rta_validar_mensaje['Dato']['opcion_permitida']:  # Si la opcion seleccionada es una permitida

            datos = eval(datos_usuario['log']['datos'])

            # Si el modo de la charla no ha sido cambiado todavia
            if datos_usuario['charla']['id_modo'] != datos['modo']:
                # Guardamos el modo y ultimo nodo actuales
                datos_usuario['modo_previo'] = datos_usuario['charla']['id_modo']
                datos_usuario['ultimo_nodo_previo'] = datos_usuario['charla']['id_ultimo_nodo']

                # Cambiamos el modo en el diccionario de datos_usuario
                datos_usuario['charla']['id_modo'] = datos['modo']

                # Cambiamos el ultimo nodo en el diccionario de datos_usuario
                datos_usuario['charla']['id_ultimo_nodo'] = datos_usuario['log']['id_nodo']

                # Cambiamos el modo y la posicion del usuario en la bd
                cambiar_modo_y_posicion(datos_usuario['charla']['id_charla'],
                                        datos['modo'],
                                        datos_usuario['log']['id_nodo'])

            return modo_permitido(datos_usuario)

        return opcion_elegida_correcta(datos_usuario)  # Llamo a la funcion que ejecuta la logica


def _charla_nueva(datos_usuario):
    '''
    logica cuando una charla es nueva

    :param datos_usuario (dict): Un diccionario con los datos del usuario obtenidos al recibir el mensaje.
    :return: (dict) Un diccionario con Estado, Respuesta y de ser necesario un dato.
    '''

    # Obtenemos los parametros del cliente (hoy los trae desde la bd)
    parametros = obtener_parametros_cliente(datos_usuario)

    # inicializa una charla para que ande la funcion de obtener nodos hijos espera)
    datos_usuario = inicializar_datos_charla(datos_usuario)

    # todo Intenta obtener el nodo inicial del modo especificado en el parametro activo (creo que esta de mas, ya que lo
    # hace la funcion obtener_nodos)
    nodo_inicial = obtener_nodo_inicial(parametros)

    # obtiene los nodos
    nodos = obtener_nodos(parametros)

    arbol = funciones.armar_arbol(nodos)  # Arma el arbol con los nodos, podria ser un json, pero este está bueno

    nodo_at_inicial = arbol[nodo_inicial].data  # Obtengo el nodo inicial de todo el arbol

    parametros['charla'] = datos_usuario['charla']  # Guardo los datos de la charla en el diccionario

    # todo agregado por ivan ojo
    parametros['mensaje'] = datos_usuario['msj']  # Guardo los datos de la charla en el diccionario
    # hasta ahora solo guarda telefono origen y telefono destino

    nodo_at_inicial.parametros = parametros

    # hasta ahora
    # datos_usuario = {
    #     'tel_origen': telefono,
    #     'tel_destino': tel_destino,
    #     'msj': msj,
    #     'enviado_por': enviado_por,
    #     'tipo': 1,
    #     'fecha_hora': fecha_hora,
    #     'clase': texto,audio,etc,
    #     'charla':{
    #                'telefono_origen':telefono,
    #                'telefono_destino':tel_destino
    #              }
    # }

    # parametros={
    #     id_empresa:1_3,
    #     id_modo:1,
    #     limite_intentos:3,
    #     ...
    #     'charla':{
    #     #         'telefono_origen':telefono,
    #     #         'telefono_destino':tel_destino
    #     #        }
    # }

    # nodo_at_inicial={
    #     padre:0,
    #     hijos:{
    #         id_nodo:3
    #     },
    #     (de PARAMETROS)
    #     id_empresa:1_3,
    #     limite_intentos:3
    #     ...
    #     'charla':{
    #               'telefono_origen':telefono,
    #               'telefono_destino':tel_destino
    #               }
    #      }
    # }

    # breakpoint()
    filtros_encontrados = obtener_filtros_nodos_hijos(nodo_at_inicial)

    rta_dinamicos = nodo_at_inicial.obtener_dinamicos(None, filtros_encontrados)  # Intentamos obtener los dinamicos
    # print('rta_dinamicos:')
    # pprint.pprint(rta_dinamicos)
    if rta_dinamicos['Estado'] is not True:  # Si fallamos al obtener los dinamicos del primer nodo del modo
        print("ERROR AL OBTENER DINAMICOS EN CHARLA NUEVA.")

    # todo revisar esto porque si no tiene dinamicos rompe todo, ya que no devuelve nada cuando falla
    dinamicos = rta_dinamicos['Dato']  # Guardamos los dinamicos

    for dinamico in dinamicos:  # Convertimos la llave 'datos' en string
        dinamico['datos'] = str(dinamico['datos'])
        dinamico['mensaje_enviado'] = dinamico['texto']
        # al pedo setea en mensaje_enviado, si es lo mismoq ue devuelve texto
    # ("dinamico['mensaje_enviado']:", '1)Urquiza 7164')
    # ("dinamico['mensaje_enviado']:", '2)rioja 123')
    # ("dinamico['mensaje_enviado']:", u'Envia el *_MAPA_* con tu ubicacion actual.')
    # ("dinamico['mensaje_enviado']:", u'Escriba su direccion')

    # Si lo que ingreso el usuario coincide con una opcion disponible
    return verificar_coincidencia(arbol, datos_usuario, nodo_inicial, parametros, dinamicos)


def obtener_filtros_nodos_hijos(nodo_actual):
    filtros_encontrados = []
    for hijo in nodo_actual.hijos:
        filtros_encontrados.append({
            'id_nodo': hijo.data.id_nodo,
            'filtro': obtener_filtros_nodo(hijo.data.id_nodo)
        })
    # print("filtros en cada hijo encontrado:", filtros_encontrados)
    return filtros_encontrados


def obtener_filtros_nodo(id_nodo):
    # print("entro:", id_nodo)
    rta = cpBotBd.obtener_nodo(id_nodo)
    # print(rta)
    if rta['Estado'] is False: return None
    filtro = rta['Dato'].filtro

    return str_to_array(filtro, ',')  # ['una','varios']


def breakpoint():
    import web_pdb
    web_pdb.set_trace()


def verificar_coincidencia(arbol, datos_usuario, nodo_inicial, parametros, dinamicos):
    # Valida el mensaje que mando el usuario
    # todo esto lo podria validar el mismo funciones.validar_mensaje
    clase_mensaje, direccion = funciones.obtener_clase_msj(clase=datos_usuario['clase'], mensaje=datos_usuario['msj'])
    # si fuera una direccion seria <texto>, tambien puede ser imagen,mapa,audio

    # Valida si lo que ingreso el usuario coincide con alguna opcion disponible,1,2,urquiza 7164 o texto como por ejemplo Cancelar
    rta_validar_mensaje = funciones.validar_mensaje(dinamicos, datos_usuario, clase_mensaje)
    # print("rta validar mensajes es :", rta_validar_mensaje)

    # todo.puse que valide el numero aca, porque sino cuando ingresaba el numero 1 permitia cargar la direccion directamente!
    # tambien valida que si pone cancelar desde un inicio no se rompa. y le ofrezca el menu saludar
    if rta_validar_mensaje['Estado'] is False or clase_mensaje == '<numero>' or datos_usuario[
        'msj'].lower() == 'cancelar' or datos_usuario['msj'].lower() == 'telefonista':
        return no_coincide(datos_usuario, nodo_inicial, parametros)
    return coincide(arbol, datos_usuario, direccion, parametros, rta_validar_mensaje)


def no_coincide(datos_usuario, nodo_inicial, parametros):
    print("\n# si lo que ingreso no coinicide con una opcion ( o ingreso un numero)")
    # si lo que ingreso no coinicide con una opcion o es valido el mensaje pero espera una direccion
    # Graba la charla nueva en la base de datos
    charla = grabar_charla_nueva(datos_usuario, nodo_inicial, parametros)
    datos_usuario['charla'] = charla  # Agrego a los datos_usuario el diccionario de la charla
    datos_usuario['cliente'] = parametros  # Agrego los parametros a los datos_usuario

    nodos = obtener_nodos(parametros)
    arbol = funciones.armar_arbol(nodos)  # Armo el arbol correspondiente al modo
    nodo_at = arbol[nodo_inicial].data  # Obtengo el nodo donde esta posicionado el usuario
    parametros['charla'] = charla  # Guardo los datos de la charla en el diccionario
    nodo_at.parametros = parametros  # Guardo el diccionario como un atributo del nodo
    return ejecucion(datos_usuario, nodo_at)


def coincide(arbol, datos_usuario, direccion, parametros, rta_validar_mensaje):
    print("\nlo que ingreso coincide con una opcion")
    # generalmente cuando ingresa un mapa o una direccion desde el inicio

    log = rta_validar_mensaje['Dato']['log']  # Guardamos el log que coincidio con el mensaje
    try:
        nodo_at = arbol[log['id_nodo']].data  # Inicializamos el nodoat correspondiente al nodo
    except:
        raise Exception(
            u'Falla al iniciar el nodoat. posiblemente envio cancelar y ya estaba finalizada la charla. o mando cancelar como primer mensaje')

    charla = grabar_charla_nueva(datos_usuario, log['id_nodo'], parametros)

    datos_usuario['log'] = log  # Guardamos el log en los datos_usuario
    datos_usuario['charla'] = charla  # Seteamos la key 'charla' en los datos_usuario
    datos_usuario['cliente'] = parametros  # Seteamos la key 'cliente' en los datos_usuario
    datos_usuario['mensaje'] = datos_usuario['msj']
    datos_usuario['direccion'] = direccion
    datos_usuario['ciudad'] = parametros.get('ciudad')  # AGREGADO
    datos_usuario['provincia'] = parametros.get('provincia')  # AGREGADO
    datos_usuario['pais'] = parametros.get('pais')  # AGREGADO
    print("parametros a insertar coincide:")
    pprint.pprint(datos_usuario)
    nodo_at.parametros = datos_usuario  # Guardamos los parametros en el nodo
    return ejecucion(datos_usuario, nodo_at)


def ejecucion(datos_usuario, nodo_at):
    datos_ejecucion = ejecutar_nodoat(nodo_at)  # Ejecuto el nodo y guardo los datos de la ejecucion

    print("datos de ejecucion en funcion ejecucion")
    pprint.pprint(datos_ejecucion)

    # si tiene valores resultantes los agrega a los parametros del nodo para que lo puedan usar los hijos para
    # obtener sus dinamicos
    if datos_ejecucion.get('Valores_resultantes'):
        nodo_at.parametros['Valores_resultantes'] = datos_ejecucion.get('Valores_resultantes')

    filtros_encontrados = obtener_filtros_nodos_hijos(nodo_at)
    titulo, dinamico = obtener_titulo_y_dinamico(nodo_at, datos_ejecucion,
                                                 filtros_encontrados)  # Obtenemos el titulo y el dinamico

    # todo. resolver esto porque sino agrega la lista entera a cada campo datos de cada log
    if datos_ejecucion.get('Valores_resultantes'):
        valores = datos_ejecucion.get('Valores_resultantes')
        if valores == 'varias':
            datos_ejecucion['Dato'] = {}
    grabar_en_logs(datos_usuario, datos_ejecucion['Dato'], dinamico)  # Agregamos los logs correspondientes a la BD

    menu_enviado = enviar_menu(datos_usuario, titulo, dinamico, nodo_at)

    return respuesta(True, u"Se envió correctamente el menu:", menu_enviado)


# @con_log(nombre_producto)
# @con_try(nombre_producto)
def opcion_elegida_erronea(datos_usuario):
    '''
    Esta funcion se encarga de ejecutar la logica necesaria cuando el usuario elige una opcion que no existe en la tabla logs.

    :param datos_usuario (dict):  Un diccionario con distintos datos utiles.
                tel_origen: El telefono del usuario
                tel_destino: El telefono del radiotaxi
                msj: El cuerpo del mensaje
                enviado_por: El nombre de la persona que envio el mensaje ('')
                tipo: Si el mensaje es entrante o no
                fecha_hora: La fecha y hora en que se recibio el mensaje
                clase_msj: La clase del mensaje ('audio', 'map)
                charla: Un diccionario con datos de la charla
                cliente: Un diccionario con datos del cliente
                seleccion_usuario: La seleccion del usuario
    :return: (dict) Una respuesta con Estado, un mensaje y de ser necesario un Dato.
    '''

    arbol = obtener_arbol(datos_usuario)  # Obtengo el arbol

    # Posiciono al usuario donde corresponde en el arbol
    nodo_at = arbol[datos_usuario['charla']['id_ultimo_nodo']].data

    parametros = definir_parametros_opcion_erronea(datos_usuario)

    nodo_at.parametros = parametros

    # Incrementamos el numero de intentos fallidos y guardamos si supero o no el maximo de intentos
    supero_intentos_fallidos = incrementar_intentos_fallidos(datos_usuario)

    # Si el usuario excedio el numero de intentos fallidos dispuestos en la base de datos
    if supero_intentos_fallidos:

        pasar_a_manual(datos_usuario)  # Pasamos la charla a manual en cpbot y atmsj

        texto_derivar = ''

        if bool(datos_usuario['cliente']['ignorar_errores']) is False:
            texto_derivar = enviar_menu_opcion_incorrecta(datos_usuario, "falla_maximo")

        return respuesta(True, u'Se paso la charla a manual por exceso de intentos fallidos', dato=texto_derivar)

    else:  # si no supero el limite de intentos fallidos

        # todo arreglar try agregado porque se rompia cunado mandanba urquiza 7164, y despues perro
        # porque ya no tiene los dinamicos. ver como arreglarlo
        try:
            titulo, dinamico = obtener_titulo_y_dinamico(nodo_at)  # Obtenemos el titulo y el dinamico
        except:
            enviar_menu_opcion_incorrecta(datos_usuario, 'falla_nocheck')
            return respuesta(True, u'fallo al obtener dinamico', dato="")

        if datos_usuario.get('nocheck') is None:
            pass
        else:
            enviar_menu_opcion_incorrecta(datos_usuario, 'falla_nocheck')

        if bool(datos_usuario['cliente']['ignorar_errores']):  # Si se deben ignorar los errores
            # Enviamos la respuesta con true y un mensaje
            return respuesta(True, u'Se ignoro el error del usuario', dato="")

        enviar_menu_opcion_incorrecta(datos_usuario, 'mensaje_fallido')

        menu_enviado = enviar_menu(datos_usuario, titulo, dinamico, nodo_at)  # Enviamos el menu al usuario

        return respuesta(True, u"Se envió correctamente el menu", dato=menu_enviado)


# @con_log(nombre_producto)
# @con_try(nombre_producto)
def opcion_elegida_correcta(datos_usuario, ejecutar_nodo=True, con_titulo=True, titulo_alternativo=None):
    '''
    Esta funcion se encarga de ejecutar la logica cuando la opcion elegida es correcta

    :param datos_usuario (dict): Un diccionario con los datos de la charla, el usuario y el cliente.
    :param ejecutar_nodo (bool): Un booleano para determinar si se debe ejecutar el nodo o no

    :return: (dict) Una respuesta con Estado, un mensaje y de ser necesario un dato.
    '''

    print("ENTRA EN OPCION ELEGIDA CORRECTA")
    pprint.pprint(datos_usuario)

    arbol = obtener_arbol(datos_usuario)  # Obtengo el arbol
    pprint.pprint(arbol)
    id_nodo_hijos = obtener_nodos_hijo(datos_usuario, arbol)  # Obtengo el id_nodo de los hijos

    log = datos_usuario['log']  # Guardo el log en una variable

    es_nodo_sin_hijos = None  # Seteo la variable para verificar si el nodo es hoja (no tiene hijos)
    es_nodo_sin_hijos = _es_nodo_sin_hijos(datos_usuario, es_nodo_sin_hijos, id_nodo_hijos, log)

    # Posiciono al usuario donde corresponde en el arbol
    nodo_at = arbol[log['id_nodo']].data

    dev = definir_parametros_opcion_correcta(datos_usuario)
    # print("parametros a insertar correcta:")
    # pprint.pprint(dev)
    nodo_at.parametros = dev

    if ejecutar_nodo:  # Si se llamo a la funcion con la ejecucion del nodo

        # agrega los filtros palabras

        datos_ejecucion = ejecutar_nodoat(nodo_at)  # Ejecutamos el nodo y guardamos el dato obtenido del mismo
        # print("\ndatosejecu:")
        # pprint.pprint(datos_ejecucion)
        if datos_ejecucion.get('Valores_resultantes'):
            nodo_at.parametros['Valores_resultantes'] = datos_ejecucion.get('Valores_resultantes')

        # todo. negrada. si la ejecucion fue de un servicio, negrada porque esta todo armado como para que siempre el ejecutar
        # devuelva un servicio!!!!
        try:
            if datos_ejecucion['Dato'].get('id_servicio'):
                datos_log = agregar_ejecucion_al_log(log, datos_ejecucion['Dato'])  # NO A la base de datos!
                actualizar_datos_servicio(datos_usuario, datos_log)  # en la charla
        except:
            print("no ejecuto una carga d eun servicio")


    else:  # Si no debemos ejecutar el nodo porque es un permitido

        datos_log = log['datos']  # Guardamos los log sin la ejecucion del nodo
        datos_ejecucion = {}  # Seteo la variable datos ejecucion como un diccionario vacio
        actualizar_datos_servicio(datos_usuario, datos_log)

    # Actualizo la posicion del ultimo nodo del usuario en la base de datos
    actualizar_posicion_usuario(datos_usuario)

    filtros_encontrados = obtener_filtros_nodos_hijos(nodo_at)
    titulo, dinamico = obtener_titulo_y_dinamico(nodo_at, datos_ejecucion,
                                                 filtros_encontrados)  # Obtenemos el titulo y dinamico

    if con_titulo is False:  # Si no queremos pasar el titulo de la funcion

        titulo = ''  # Cambiamos el titulo por un string vacio

        if titulo_alternativo is not None:  # Si se paso un titulo alternativo como argumento
            titulo = titulo_alternativo  # Cambiamos el titulo por el alternativo obtenido

    if ejecutar_nodo:
        actualizar_estado_logs_y_seleccionado(datos_usuario)  # Actualizamos los logs en base al seleccionado

        print("\ndatos ejecucion antes de grabar logs:")
        pprint.pprint(datos_ejecucion)

        # todo bloque de negrada agregada porque sino queria guardar toda la devolucion de obtener direcciones en CADA LOG
        # y postgres me tiraba overflow
        if datos_ejecucion.get('Valores_resultantes'):
            valores = datos_ejecucion.get('Valores_resultantes')
            if valores == 'varias':
                datos_ejecucion['Dato'] = {}

        grabar_en_logs(datos_usuario, datos_ejecucion['Dato'], dinamico)  # Grabamos los logs disponibles

    menu_enviado = enviar_menu(datos_usuario, titulo, dinamico, nodo_at)  # Enviamos el menu al usuario

    # si el nodo es hoja o sea no tiene hijos, finaliza la charla
    if es_nodo_sin_hijos and ejecutar_nodo:
        finalizar_charla_(datos_usuario)  # Finalizamos la charla en atmsj y cpbot

    return respuesta(True, u"Se envió correctamente el menu:", menu_enviado)


def _es_nodo_sin_hijos(datos_usuario, es_nodo_sin_hijos, id_nodo_hijos, log):
    # Si el nodo al que debe ir el usuario no existe
    if int(log['id_nodo']) not in id_nodo_hijos:
        if len(id_nodo_hijos) == 0:  # Si el nodo no tiene hijos
            es_nodo_sin_hijos = True
        else:
            # Levantamos una excepcion para registro y que pase a manual la charla
            raise Exception(u'El nodo para la opcion elegida no existe',
                            u'Verificar el arbol del modo: %s' % datos_usuario['charla']['id_modo'])
    return es_nodo_sin_hijos


def modo_permitido(datos_usuario):
    '''
    Esta funcion se encarga de ejecutar la logica cuando la opcion elegida deriva a un modo secundario.

    :param datos_usuario (dict): Un diccionario con los datos de la charla, el usuario y el cliente.
    :return: (dict) Una respuesta con Estado, un mensaje y de ser necesario un dato.
    '''

    print("ENTRA EN OPCION MODO permitido")

    arbol = obtener_arbol(datos_usuario)  # Obtengo el arbol

    id_nodo_hijos = obtener_nodos_hijo(datos_usuario, arbol)  # Obtengo el id_nodo de los hijos

    log = datos_usuario['log']

    # Posiciono al usuario donde corresponde en el arbol
    nodo_at = arbol[log['id_nodo']].data

    nodo_at.parametros = definir_parametros_opcion_correcta(datos_usuario)

    datos_ejecucion = ejecutar_nodoat(nodo_at)  # Ejecutamos el nodoat y guardamos la respuesta
    datos_ejecucion = datos_ejecucion['Dato']  # ojo. lo pongo asi, porque un nodo al ejecutarse puede devolver valores
    # resultantes
    datos_ejecucion['modo_previo'] = datos_usuario['modo_previo']  # Agregamos el modo previo
    datos_ejecucion['ultimo_nodo_previo'] = datos_usuario['ultimo_nodo_previo']  # Agregamos el nodo previo

    # Determinamos si debemos eejcutar el nodo del modo anterior con o sin titulo
    con_titulo = datos_ejecucion['con_titulo']

    # Insertamos en los datos ejecucion el modo y nodo previos a ejecutar el modo permitido
    datos_log = agregar_ejecucion_al_log(log, datos_ejecucion)

    actualizar_datos_servicio(datos_usuario, datos_log)  # Actualizamos los datos del servicio en la BD

    # Actualizo la posicion del ultimo nodo del usuario en la base de datos
    actualizar_posicion_usuario(datos_usuario)

    titulo, dinamico = obtener_titulo_y_dinamico(nodo_at)

    # Intento grabar los logs nuevos en la base de datos
    grabar_en_logs(datos_usuario, datos_ejecucion, dinamico)

    # Enviamos el menu del nodo
    menu_enviado = enviar_menu(datos_usuario, titulo, dinamico, nodo_at)

    if id_nodo_hijos == []:  # Si el nodo en el que nos posicionamos es un nodo hoja

        # Verificamos si debemos volver al modo anterior
        debe_volver = debe_volver_a_modo_anterior(datos_usuario['charla']['id_modo'])

        if debe_volver:  # Si debemos volver al modo anterior

            cambiar_modo_y_posicion(datos_usuario['charla']['id_charla'],
                                    datos_usuario['modo_previo'],
                                    datos_usuario['ultimo_nodo_previo'])

            # Volvemos a setear el id modo previo como el actual en los datos_usuario
            datos_usuario['charla']['id_modo'] = datos_usuario['modo_previo']
            datos_usuario.pop('modo_previo')  # Borramos la entrada de modo_previo

            # Volvemos a setear el id nodo previo como el actual en los datos_usuario
            datos_usuario['charla']['id_ultimo_nodo'] = datos_usuario['ultimo_nodo_previo']
            datos_usuario['log']['id_nodo'] = datos_usuario['ultimo_nodo_previo']  # Modificamos el nodo del log
            datos_usuario['log']['_sa_instance_state'] = '-'  # Insertamos este dato para no romper en opcion_correcta
            datos_usuario.pop('ultimo_nodo_previo')  # Borramos la entrada de ultimo_nodo_previo

            #  Ejecutamos la logica detras del nodo previo en el modo anterior
            opcion_elegida_correcta(datos_usuario, ejecutar_nodo=False, con_titulo=con_titulo)

        else:
            # print("quiere finalizar la charla:")
            # print("datos_usuario:",pprint.pprint(datos_usuario))
            if datos_usuario['charla']['id_modo']=='cancelar':
                finalizar_charla_(datos_usuario)  # Finalizamos la charla en atmsj y cpbot

    # Devolvemos True y una respuesta
    return respuesta(True, u'Se ejecuto modo_permitido con exito', dato=menu_enviado)


######### Las funciones de abajo son para modularizar el codigo, envian mensajes, finalizan charlas, etc. ##############


def finalizar_charla_(datos_usuario):
    '''
    Esta funcion ejecuta la logica para finalizar una charla.
    Esto implica finalizar la charla tanto en cpbot como atmsj

    En caso de fallar levantara una excepcion

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    '''

    rta_obtener_texto_tag = cpBotBd.obtener_texto_tag("finalizar_charla")

    if rta_obtener_texto_tag['Estado'] is False:
        raise Exception(u'Falla al querer armar el mensaje de finalizar la charla:',
                        rta_obtener_texto_tag['Respuesta'])

    mensaje = rta_obtener_texto_tag['Dato']

    rta_formatear_mensaje = tagutiles.formatear_texto(mensaje, datos_usuario)

    if rta_formatear_mensaje['Estado'] is False:  # Si no pudimos formatear el texto
        # Levantamos una excepcion
        raise Exception(u'Falla al formatear finalizar_charla', rta_formatear_mensaje['Respuesta'])

    _mensaje = rta_formatear_mensaje['Dato']  # Guardamos el mensaje formateado

    rta_enviar_atmsj = funciones.enviar_atmsj(mensaje=_mensaje,
                                              tel_origen=datos_usuario['tel_destino'],
                                              tel_destino=datos_usuario['tel_origen'],
                                              ip_atmsj=datos_usuario['cliente']['ip_atmsj'])

    if rta_enviar_atmsj['Estado'] is False:
        raise Exception(u'Falla al enviar el mensaje a atmsj',
                        rta_enviar_atmsj['Respuesta'])

    # Finalizamos la charla en el cpbot
    rta_finalizar_en_cpbot = cpBotBd.cambiar_estado_charla(datos_usuario['charla']['id_charla'], "finalizada")

    # Si no pude finalizar la charla
    if rta_finalizar_en_cpbot['Estado'] is False:
        raise Exception(u'Error al pasar la charla a manual', rta_finalizar_en_cpbot['Respuesta'])

    # Finalizamos la charla en atmsj
    print("va a finaliza rla charla en atmsj todo mal")
    rta_finalizar_en_atmsj = atmsjadapter.finalizar_charla(ip_atmsj=datos_usuario['cliente']['ip_atmsj'],
                                                           telefono=datos_usuario['tel_origen'])

    # Si no pude finalizar la charla
    if rta_finalizar_en_atmsj['Estado'] is False:
        raise Exception(u'Error al finalizar la charla en atmsj', rta_finalizar_en_atmsj['Respuesta'])


def actualizar_estado_logs_y_seleccionado(datos_usuario):
    '''
    Esta funcion se encarga de actualizar el estado de los logs no seleccionados y del seleccionado.

    En caso de fallar levantara una excepcion

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    '''

    log = datos_usuario['log']  # Guardamos el log en una variable

    # Intento actualizar los logs anteriores
    rta_actualizar_logs = cpBotBd.actualizar_estado_logs(datos_usuario['charla']['id_charla'])

    # Si no pude actualizar los logs anteriores
    if rta_actualizar_logs['Estado'] is False:
        # Levanto una excepcion
        raise Exception(u'Falla al actualizar los logs que hay que desactivar:',
                        rta_actualizar_logs['Respuesta'])

    rta_actualizar_log_seleccionado = cpBotBd.actualizar_log_seleccionado(log['id_log'])
    # Si no pude actualizar el log seleccionado
    if rta_actualizar_log_seleccionado['Estado'] is False:
        # Levanto una excepcion
        raise Exception(u'Falla al actualizar el log seleccionado:',
                        rta_actualizar_log_seleccionado['Respuesta'])


def obtener_titulo_y_dinamico(nodo_at, datos_ejecucion=None, filtros=None):
    """
    Esta funcion se encarga de conseguir el titulo y menu dinamico de un nodo, para luego armar el menu.

    En caso de fallar levantara una excepcion

    """

    rta_obtener_menu = nodo_at.obtener_menu(datos_ejecucion, filtros)  # Intento obtener el menu del nodo

    # Si falla al obtener el menu
    if rta_obtener_menu['Estado'] is False:
        # Levanto una excepcion
        raise Exception(u'Fallo al obtener el menu', rta_obtener_menu['Respuesta'])

    titulo = rta_obtener_menu['Dato']['titulo']
    dinamico = rta_obtener_menu['Dato']['dinamico']

    return titulo, dinamico


def pasar_a_manual(datos_usuario):
    '''
    Esta funcion se encarga de pasar a manual una charla tanto en cpbot como en atmsj

    En caso de fallar levantara una excepcion.

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    '''

    rta_cambiar_charla_a_manual = cpBotBd.cambiar_automatico_charla(datos_usuario['charla']['id_charla'], 0)
    # Si no pude finalizar la charla
    if rta_cambiar_charla_a_manual['Estado'] is False:
        raise Exception(u'Error al pasar la charla a manual', rta_cambiar_charla_a_manual['Respuesta'])

    rta_cambiar_automatico_atmsj = funciones.cambiar_automatico(datos_usuario['tel_origen'], 0,
                                                                datos_usuario['cliente']['ip_atmsj'])

    # Si no pude finalizar la charla
    if rta_cambiar_automatico_atmsj['Estado'] is False:
        raise Exception(u'Error al pasar la charla a manual en atmsj',
                        rta_cambiar_automatico_atmsj['Respuesta'])


def definir_parametros_opcion_erronea(datos_usuario):
    '''
    Esta funcion se encarga de crear un diccionario 'parametros' que luego se pasar al nodo_at para la ejecucion de
    funciones.

    En caso de fallar levantara una excepcion

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    '''

    log = datos_usuario['charla']
    log['datos'] = unicode(datos_usuario['charla']['datos_servicio'])

    # Guardo los datos de la tabla parametros en un diccionario
    parametros = datos_usuario['cliente']
    parametros['charla'] = datos_usuario['charla']  # Guardo la charla dentro del diccionario
    parametros['log'] = log  # Insertamos en los parametros el log
    parametros['mensaje'] = datos_usuario['msj']

    return parametros


def definir_parametros_opcion_correcta(datos_usuario):
    '''
    Esta funcion se encarga de crear un diccionario 'parametros' que luego se pasar al nodo_at para la ejecucion de
    funciones.

    En caso de fallar levantara una excepcion

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    '''

    log = datos_usuario['log']

    # Guardo los datos de la tabla parametros en un diccionario

    parametros = datos_usuario['cliente']
    log.pop('_sa_instance_state')  # Eliminamos la entrada referente al objeto de SQLAlchemy del log
    parametros['charla'] = datos_usuario['charla']  # Guardo la charla dentro del diccionario
    parametros['log'] = log  # Insertamos en los parametros el log
    parametros['mensaje'] = datos_usuario['msj']

    if datos_usuario.get('direccion'):
        parametros['direccion'] = datos_usuario.get('direccion')  # AGREGADO
    if datos_usuario.get('ciudad'):
        parametros['ciudad'] = datos_usuario.get('ciudad')  # AGREGADO
    if datos_usuario.get('provincia'):
        parametros['provincia'] = datos_usuario.get('provincia')  # AGREGADO
    if datos_usuario.get('pais'):
        parametros['pais'] = datos_usuario.get('pais')  # AGREGADO

    return parametros


def obtener_nodos_hijo(datos_usuario, arbol):
    '''
    Esta funcion se encarga de obtener los nodos hijos del nodo en el que esta posicionado el usuario al momento
    de ingresar en la funcion opcion_elegida_correcta.

    En caso de fallar levantara una excepcion

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    '''

    nodo_at = arbol[datos_usuario['charla']['id_ultimo_nodo']].data  # Obtengo el nodo donde esta posicionado el usuario

    return nodo_at.obtener_id_nodo_hijos()


def obtener_arbol(datos_usuario):
    '''
    Esta funcion se encarga de obtener el arbol del modo en el que se encuentra el usuario.

    En caso de fallar levantara una excepcion

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    '''

    rta_nodos = cpBotBd.obtener_nodos(
        datos_usuario['charla']['id_modo'])  # Intento obtener los nodos correspondientes a la charla

    if rta_nodos['Estado'] is False:  # Si no pude obtener los nodos
        raise Exception(u'Falla al obtener nodos', rta_nodos['Respuesta'])  # Levanto una excepcion

    arbol = funciones.armar_arbol(rta_nodos['Dato'])  # Armo el arbol correspondiente al modo

    return arbol


def ejecutar_nodoat(nodo_at):
    '''
    Esta funcion se encarga de ejecutar el nodo en el que se ha posicionado al usuario.

    En caso de fallar levantara una excepcion

    :param nodo_at (NodoAt): Una clase creada especialmente para la ejecucion de nodos.
    '''

    rta_ejecutar_nodo = nodo_at.ejecutar_nodo()  # Ejecuto la funcion del nodo

    # Si falla al ejecutar el nodo
    if rta_ejecutar_nodo['Estado'] is False:
        # Levanto una excepcion
        raise Exception(u'Error al ejecutar el nodo', u'ID_NODO: %s' % nodo_at.id_nodo)

    return rta_ejecutar_nodo


def agregar_ejecucion_al_log(log, datos_ejecucion):
    '''
    Esta funcion se encarga de insertar en el log los datos de la ejecucion del nodo.



    :param log (dict): Un diccionario con informacion del log que selecciono el usuario en el menu anterior.
    :param datos_ejecucion (dict): Un diccionario con informacion de la ejecucion del nodo
    '''

    datos_log = eval(log['datos'])  # Convertimos los datos del log en diccionario
    datos_log['ejecucion'] = datos_ejecucion  # Insertamos los datos de ejecucion en el log
    datos_log = unicode(datos_log)  # Volvemos a convertir los datos del log en string

    return datos_log  # Devolvemos el log con la ejecucion agregada


def enviar_menu(datos_usuario, titulo, dinamico, nodo_at):
    '''
    Esta funcion se encarga de armar el menu basado en el titulo y el dinamico, y luego mandarselo al usuario

    En caso de fallar levantara una excepcion

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    :param titulo (str): El titulo del nodo en el que esta posicionado el usuario
    :param dinamico (str): El dinamico de los nodos hijos al que esta posicionado el usuario
    '''

    rta_armar_menu = nodo_at.armar_menu(titulo, dinamico)

    if rta_armar_menu['Estado'] is False:
        # Devuelvo la respuesta
        raise Exception(u'Falla al armar el menu a enviar:', rta_armar_menu['Respuesta'])

    menu = rta_armar_menu['Dato']

    # print("menu a enviar###########################################################################################:")
    # print("menu a enviar###########################################################################################:")
    pprint.pprint(menu)
    print(datos_usuario['tel_destino'])
    print(datos_usuario['tel_origen'])
    print(datos_usuario['cliente'])
    rta_enviar_atmsj = funciones.enviar_atmsj(mensaje=menu,
                                              tel_origen=datos_usuario['tel_destino'],
                                              tel_destino=datos_usuario['tel_origen'],
                                              ip_atmsj=datos_usuario['cliente']['ip_atmsj'])

    # Si la respuesta es False
    if rta_enviar_atmsj['Estado'] is False:
        # Levanto una excepcion
        raise Exception(u'Falla al armar enviar por http:', rta_enviar_atmsj['Respuesta'])

    return menu


def grabar_en_logs(datos_usuario, datos_ejecucion, dinamico):
    '''
    Esta funcion se encarga de grabar en la tabla 'logs' todas las opciones disponibles para el usuario, acorde al
    nodo en el que esta posicionado en el momento.

    :param datos_usuario: (dict) Un diccionario con distinta informacion del cliente, usuario, etc.
    :param datos_ejecucion: (dict) Un diccionario con informacion de la ejecucion del nodo
    :param dinamico: (str) El dinamico de los nodos hijos al que esta posicionado el usuario
    '''

    # Intento grabar los logs nuevos en la base de datos
    rta_grabar_logs = cpBotBd.grabar_logs(dinamico, datos_ejecucion, datos_usuario['charla']['id_charla'])

    # Si no se pudo guardar los logs
    if rta_grabar_logs['Estado'] is False:
        # Devuelvo la respuesta
        raise Exception(u'Falla al guardar los logs:', rta_grabar_logs['Respuesta'])


def actualizar_posicion_usuario(datos_usuario):
    '''
    Esta funcion se encarga de actualizar la posicion del usuario en la bd correspondiente al log que selecciono.

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    '''

    log = datos_usuario['log']

    # Actualizo la posicion del ultimo nodo del usuario en la base de datos
    rta_posicion = cpBotBd.actualizar_posicion_usuario(datos_usuario['charla']['id_charla'], log['id_nodo'])

    # Si la respuesta es False
    if rta_posicion['Estado'] is False:
        # Levanto una excepcion
        raise Exception(u'Fallo al actualizar la posicion del usuario', rta_posicion['Respuesta'])


def actualizar_datos_servicio(datos_usuario, datos_log):
    '''
    Esta funcion se encarga de actualizar los datos del servicio en la tabla 'charla' de la bd CpBot

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    :param datos_log (dict): Un diccionario que contiene la informacion del log seleccionado por el usuario.
    '''

    # Agrego los datos de la opcion seleccionada al campo 'datos_servicio'
    rta_datos_servicio = cpBotBd.actualizar_datos_servicio(datos_usuario['charla']['id_charla'], datos_log)

    # Si no pude actualizar los datos del servicio
    if rta_datos_servicio['Estado'] is False:
        # Levanto una excepcion
        raise Exception(u'No se pudieron actualizar los datos_servicio en la tabla charla',
                        rta_datos_servicio['Respuesta'])


def incrementar_intentos_fallidos(datos_usuario):
    '''
    Esta funcion se encarga de incrementar los intentos fallidos del usuario y devolver cuantos lleva.

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    '''

    rta = cpBotBd.incrementar_intentos_fallidos(datos_usuario['charla']['id_charla'])

    if rta['Estado'] is False:
        raise Exception(u'Falla al incrementar intentos fallidos:', rta['Respuesta'])

    # Devolvemos si excedio el numero maximo de intentos
    return rta['Dato'] >= datos_usuario['cliente']['limite_intentos']


def enviar_menu_opcion_incorrecta(datos_usuario, texto):
    '''
    Esta funcion se encarga de enviar el menu de la opcion elegida incorrecta.

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    :param texto (str): El id_tag segun corresponda ('opcion_incorrecta' o 'pasar_a_manual')
    '''

    # Obtengo el texto que le mandare al usuario, tanto si lo paso a manual como si le digo que se equivocó
    rta_texto_derivar = cpBotBd.obtener_texto_tag(texto)

    # Si fallo al obtener el texto
    if rta_texto_derivar['Estado'] is False:
        # Levanto la excepcion
        raise Exception(u'Error al obtener el texto para pasar a manual', rta_texto_derivar['Respuesta'])

    texto_derivar = rta_texto_derivar['Dato']  # Guardo el dato de la respuesta

    # Envio el mensaje a traves de atmsj
    rta_enviar_atmsj = funciones.enviar_atmsj(mensaje=texto_derivar,
                                              tel_origen=datos_usuario['tel_destino'],
                                              tel_destino=datos_usuario['tel_origen'],
                                              ip_atmsj=datos_usuario['cliente']['ip_atmsj'])

    # Si no pude enviar el mensaje por atmsj
    if rta_enviar_atmsj['Estado'] is False:
        raise Exception(u'Error al enviar el mensaje a atmsj', rta_enviar_atmsj['Respuesta'])

    return texto_derivar  # Devolvemos el mensaje enviado


def obtener_parametros_cliente(datos_usuario):
    '''
    Esta funcion se encarga de obtener los parametros del cliente.

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    '''

    rta_parametro = cpBotBd.obtener_parametros(
        datos_usuario['tel_destino'])  # Intento obtener el modo que utiliza el cliente

    # Si el estado de la respuesta es False
    if rta_parametro['Estado'] is False:
        # Devuelvo la respuesta
        raise Exception(u'Falla al obtener parametros:', rta_parametro['Respuesta'])

    return rta_parametro['Dato']


def obtener_nodo_inicial(parametros):
    '''
    obtener el nodo inicial del modo asignado al cliente en la BD

    :param parametros (dict): Los parametros del cliente (tabla 'parametros')
    '''

    # Intento obtener el nodo inicial del modo especificado en el parametro activo
    rta_nodo_inicial = cpBotBd.obtener_nodo_inicial(parametros['id_modo'])

    # Si no pude obtener el nodo inicial
    if rta_nodo_inicial['Estado'] is False:
        # Devuelvo la respuesta
        raise Exception(u'Falla al obtener nodo inicial:', rta_nodo_inicial['Respuesta'])

    return rta_nodo_inicial['Dato']


def grabar_charla_nueva(datos_usuario, nodo_inicial, parametros):
    '''
    Esta funcion se encarga de grabar en la base de datos la charla nueva.

    :param datos_usuario (dict): Un diccionario con distinta informacion del cliente, usuario, etc.
    :param nodo_inicial (int): El id_nodo inicial del modo correspondiente al cliente.
    :param parametros (dict): Un diccionario con los parametros del cliente.
    '''

    # Guardo los datos de la charla en un diccionario
    charla = {'telefono_origen': datos_usuario['tel_origen'],
              'automatico': 1,
              'estado': 'activa',
              'datos_servicio': "{}",
              'fecha_hora_ultimo_mensaje': datetime.datetime.now(),
              'telefono_destino': datos_usuario['tel_destino'],
              'id_ultimo_nodo': nodo_inicial,
              'usuario_responsable': 'Automatico',
              'intentos_fallidos': 0,
              'filtros_palabras': datos_usuario.get('filtros_palabras'),  # nuevo
              'id_modo': parametros['id_modo']}

    rta_charla = cpBotBd.grabar_charla(charla)  # Intento grabar la charla en la base de datos

    # Si no se pudo grabar la charla
    if rta_charla['Estado'] is False:
        # Devuelvo la respuesta
        raise Exception(u'Falla al crear la charla:', rta_charla['Respuesta'])

    charla['id_charla'] = rta_charla['Dato']

    return charla  # Devolvemos el diccionario de la charla


def obtener_nodos(parametros):
    '''
    Esta funcion se encarga de obtener los nodos del modo correspondiente al cliente.

    :param parametros (dict): Un diccionario con los parametros del cliente.
    '''

    # Si se creó la charla correctamente
    rta_nodos = cpBotBd.obtener_nodos(parametros['id_modo'])

    #  Si no pude obtener los nodos
    if rta_nodos['Estado'] is False:
        # Devuelvo la respuesta
        raise Exception(u'Falla al obtener_nodos:', rta_nodos['Respuesta'])

    return rta_nodos['Dato']  # Devolvemos el diccionario de nodos


def actualizar_fecha_charla(charla):
    '''
    Esta funcion se encarga de actualizar la fecha y hora del ultimo mensaje de la charla

    '''

    rta_actualizar_fecha_charla = cpBotBd.actualizar_fecha_charla(charla['id_charla'])

    if rta_actualizar_fecha_charla is False:  # Si fallo al actualizar la fecha de la charla
        return rta_actualizar_fecha_charla  # Devuelvo la respuesta


def debe_volver_a_modo_anterior(id_modo):
    '''
    Esta funcion se encarga de llamar a la funcion de la base de datos y verifica si el modo en el que esta el usuario
    debe regresar a uno anterior.
    '''

    rta = cpBotBd.verificar_modo_puede_volver(id_modo)  # Llamamos a la funcion de la base de datos

    if rta['Estado'] is False:  # Si no encontramos el modo
        raise Exception(u'Error al verificar si debe volver al modo anterior'.rta['Respuesta'])

    # Devolvemos si debe volver o no al modo anterior
    return bool(rta['Dato'])


def cambiar_modo_y_posicion(id_charla, id_modo, id_nodo):
    '''
    Esta funcion se encarga de cambiar el modo y nodo donde esta posicionado el usuario

    '''

    # Intentamos cambiar el modo de la charla
    rta_cambiar_modo = cpBotBd.cambiar_modo_charla(id_charla, id_modo)

    if rta_cambiar_modo['Estado'] is False:  # Si no pudimos cambiar el modo de la charla
        raise Exception(u'Error al cambiar el modo de la charla', rta_cambiar_modo['Respuesta'])

    # Intentamos cambiar el ultimo nodo de la charla por el primero del modo nuevo
    rta_cambiar_nodo = cpBotBd.actualizar_posicion_usuario(id_charla, id_nodo)

    if rta_cambiar_nodo['Estado'] is False:  # Si fallamos al actualizar la posicion
        raise Exception(u'Error en actualizar posicion usuario', rta_cambiar_nodo['Respuesta'])


def inicializar_datos_charla(datos_usuario):
    '''
    Esta funcion se encarga de simular una charla en el diccionario de datos_usuario.
    '''

    datos_usuario['charla'] = {}  # Creamos el diccionario charla dentro de datos_usuario

    # Seteamos el telefono origen en la charla
    datos_usuario['charla']['telefono_origen'] = datos_usuario['tel_origen']

    # Seteamos el telefono destino en la charla
    datos_usuario['charla']['telefono_destino'] = datos_usuario['tel_destino']

    return datos_usuario

# def grabar_log(dic):
