# -*- coding: utf-8 -*-

import cjson
import traceback
import sys
import argparse

from flask import Flask, request
from flask_socketio import SocketIO

from srvweb.cpbot.controlador_filtros import ControladorFiltros
from at.utiles.usuarios import Usuarios
from at.utiles.conversiones import respuesta
from at.orm.utiles import cambiar_bd, seleccionar_bd
from at.cpbot.bdcpbot import CpBotBd
from at.cpbot.cpadapter import CpBotAdapter
from at.cpbot.abmarbol import Arboles
from at.cpbot.repository_funciones import Funciones
from at.cpbot.sqla_repository_charla import CharlasRepository
from at.cpbot.sqla_repository_filtros_palabras import FiltrosRepository
from at.cpbot.tagsweb import Tags

import eventlet

eventlet.monkey_patch()

NOMBRE_PRODUCTO = "CpBotTags"
HOST = '0.0.0.0'
PUERTO = 9998

app = Flask(__name__)

app.config["DEBUG"] = False

socketio = SocketIO(app, async_mode='eventlet')
controlador_filtros = ControladorFiltros()

usuarios = Usuarios()


# @app.route('/filtros', methods=['POST'])
# @cross_origin()
# def filtros():
#     '''
#     Esta entrada al sistema permite que la web del CpAdmin modifique, borre, agrege y obtenga tags.
#     '''
#
#     datos = request.get_data()
#     datos = utiles.decode_json(datos)  # Obtenemos el json con todos los datos
#
#     datos_filtros = datos['dato']  # Guardamos los datos relevantes a tags
#
#     # Valida usuario
#     rta_login = usuarios.autenticar_usuario(usuario=datos['usuario'],
#                                             clave=datos['clave'])
#
#     if rta_login['Estado'] is False:  # Si el usuario y contrasenia no son validos
#         return utiles.responder(rta_login)  # Devolvemos la respuesta
#
#     rta = controlador_filtros.filtros(datos_filtros)
#     return utiles.responder(rta)

@socketio.on("filtros", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def filtros(contenido, protocolo):
    '''
    Esta entrada al sistema permite que la web del CpAdmin modifique, borre, agrege y obtenga filtros.
    '''
    print('\nentra en filtros')
    print(contenido)

    # Valida usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:  # Si el usuario y contrasenia no son validos
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    rta = controlador_filtros.filtros(datos=contenido)

    rta = cjson.encode(rta)

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    print("sale:", rta)
    return rta


def validar_contenido(contenido, *args):
    """
    Funcion util para validar el contenido de una request.
    """

    faltantes = [faltante for faltante in args if faltante not in contenido.keys()]

    if len(faltantes) == 0:
        return respuesta(estado=True,
                         rta="El contenido de la peticion es valido")

    else:
        return respuesta(estado=False,
                         rta="Faltan las llaves %s en la peticion %s" % (faltantes, contenido))


@app.route("/debug")
def debugear():
    dic = {
        'id_modo': 'prueba',
    }

    a = Arboles(dic)

    rta = a.obtener_nodos()

    return cjson.encode(rta)


@socketio.on_error(namespace="/")
def manejar_error(e):
    print("-" * 10)
    print("[[Se produjo una excepcion en una de las entradas]]")
    print("[[Excepcion]]", e)
    print(traceback.format_exc())
    print("-" * 10)


@socketio.on("pegar_rama", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def pegar_rama(contenido, protocolo):
    """Se encarga de volver a activar una rama dada en el arbol principal"""

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    print("lleg acontenido:", contenido)
    rta = cjson.encode(arboles.pegar_rama())

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on("cortar_rama", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def cortar_rama(contenido, protocolo):
    """Se encarga de cortar una rama del arbol principal, de esta manera queda desactivada"""

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    rta = cjson.encode(arboles.cortar_rama())
    print("salida de rocrtar rama:", rta)
    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on("editar_modo", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def editar_modo(contenido, protocolo):
    """Cambia el nombre de un modo"""

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    rta = cjson.encode(arboles.editar_modo())

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on("borrar_modo", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def borrar_modo(contenido, protocolo):
    """Borra un modo de la base de datos junto con sus nodos correspondientes"""

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    rta = cjson.encode(arboles.borrar_modo())

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on("agregar_modo", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def agregar_modo(contenido, protocolo):
    """
    Agrega un modo en la base de datos.
    """

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    rta = arboles.agregar_modo()

    if not rta['Estado']:
        socketio.emit(protocolo['retorno'], 'no se pudo agregar modo', room=request.sid)
        return rta

    print(rta['ValorCampo'])
    # Agregamos un nodo (funcion saludar) al principal por defecto
    dic = dict(funcion='saludar',
               id_modo=rta['ValorCampo'],
               padre=None,
               principal=1,
               permitido={'cancelar': True, 'pasar_manual_opcion': True},
               orden=1)

    arboles.datos = dic
    rta = cjson.encode(arboles.agregar_nodo())

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on('obtener_funciones', namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def obtener_funciones(contenido, protocolo):
    """
    Obtiene todas las funciones que se encuentren en la base de datos
    """

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    funcs = Funciones(datos=contenido)

    rta = cjson.encode(funcs.obtener_funciones())

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on('editar_nodo', namespace="/")
def editar_nodo(contenido, protocolo):
    """
    Edita un nodo con los datos correspondientes.
    """

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    rta = cjson.encode(arboles.editar_nodo())

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on("borrar_nodo", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def borrar_nodo(contenido, protocolo):
    """
    Borra un nodo de un modo dado.
    """

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    rta = cjson.encode(arboles.borrar_nodo())
    print(rta)
    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on("agregar_nodo", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def agregar_nodo(contenido, protocolo):
    """
    Agrega un nodo a un modo dado.
    """

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    rta = cjson.encode(arboles.agregar_nodo())

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on("borrar_nodo", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def borrar_nodo(contenido, protocolo):
    """
    Borra un nodo de un modo dado.
    """

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    rta = cjson.encode(arboles.borrar_nodo())
    print(rta)
    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on("obtener_modos", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def obtener_modos(contenido, protocolo):
    """
    Obtiene todos los modos habilitados de la base de datos.
    """
    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    rta = cjson.encode(arboles.obtener_modos())

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on("obtener_nodos", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def obtener_nodos(contenido, protocolo):
    """
    Obtiene todos los modos habilitados de la base de datos.
    """

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    arboles = Arboles(datos=contenido)

    rta = cjson.encode(arboles.obtener_nodos())

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on('login', namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def login(contenido, protocolo):
    print("entra en login:", contenido)
    print("protocoloin:", protocolo)
    """
    Entrada para el login de la seccion mensajes en el cpadmin.
    """

    rta = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                      clave=contenido.get('clave'))

    rta = cjson.encode(rta)

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on('obtener_tags', namespace='/')
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def obtener_tags(contenido, protocolo):
    """
    Obtiene todos los tags de la base de datos y los envia al cliente que peticiono.
    """

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    tags = Tags(datos=contenido)

    rta = cjson.encode(tags.obtener())

    socketio.emit(protocolo['retorno'], rta, room=request.sid)

    return rta


@socketio.on("borrar_tag", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def borrar_tag(contenido, protocolo):
    """
    Borra un tag dado de la base de datos.
    """

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    tags = Tags(datos=contenido)

    rta = cjson.encode(tags.borrar())

    socketio.emit(protocolo['retorno'],
                  rta,
                  room=request.sid)

    return rta


@socketio.on("agregar_tag", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def agregar_tag(contenido, protocolo):
    """
    Agrega un tag del usuario a la base de datos.
    """

    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    tags = Tags(datos=contenido)

    rta = cjson.encode(tags.agregar())

    socketio.emit(protocolo['retorno'],
                  rta,
                  room=request.sid)

    return rta


@socketio.on("actualizar_tag", namespace="/")
# @con_log(NOMBRE_PRODUCTO)
# @con_try(NOMBRE_PRODUCTO)
def actualizar_tag(contenido, protocolo):
    """
    Actualiza la informacion de un tag en la base de datos.
    """
    # Verificamos el login del usuario
    rta_login = usuarios.autenticar_usuario(usuario=contenido.get('usuario'),
                                            clave=contenido.get('clave'))

    if rta_login['Estado'] is False:
        socketio.emit(protocolo['retorno'], cjson.encode(rta_login), room=request.sid)

    tags = Tags(datos=contenido)

    rta = cjson.encode(tags.actualizar())

    socketio.emit(protocolo['retorno'],
                  rta,
                  room=request.sid)

    return rta


def run():
    print("levantando srvtags")

    # Al ejecutar el servicio se le deben pasar los siguientes parámetros
    parser = argparse.ArgumentParser(description='cpbot')

    # Se puede setear un puerto, en caso de ser local sera el puerto 10003
    parser.add_argument(
        '-puerto',
        help='puerto donde se va a conectar',
        required=True
    )

    # Se puede setear la empresa,
    # la empresa LOCAL o si se ejecuta en cliente clasico ES LA 1_3. es simplemente convencion
    parser.add_argument(
        '-empresa',
        help='codigo de la empresa que se va a conectar',
        required=True
    )

    # Guardo los parámetros recibidos
    args = []
    args.extend(sys.argv)
    args.pop(0)
    args = parser.parse_args(args)

    puerto = int(args.puerto)

    print(puerto)

    empresa = args.empresa

    # Si recibo una empresa carga la bd que corresponda
    pisar_bases_de_datos(empresa)

    socketio.run(
        app,
        host=HOST,
        port=puerto)


def pisar_bases_de_datos(empresa):
    # # esto funciona asi: se debe pasar la CLASE (NO UNA INSTANCIA Y NO EL NOMBRE) de cada clase que use el dbswitch
    # las clases no pueden tener el decorador singleton sino no funciona pòrque conecta a las bd locales
    # cp2000,atmsj,atapis

    seleccionar_bd(FiltrosRepository, 'cp', empresa, False)

    # # date time helper
    seleccionar_bd(Arboles, 'cpbot', empresa, False)
    seleccionar_bd(CpBotBd, 'cpbot', empresa, False)
    seleccionar_bd(Funciones, 'cpbot', empresa, False)
    seleccionar_bd(Tags, 'cpbot', empresa, False)
