# -*- coding: utf-8 -*-

import traceback
import sys
import argparse
from flask import Flask
from flask import request

from srvweb.cpbot import controlador_bot, nombre_producto
from srvweb.utiles import utiles
from at.orm.utiles import cambiar_bd, seleccionar_bd

from at.cpbot.bdcpbot import CpBotBd
from at.cpbot.atmsjadapter import anunciar_ip, cambiar_automatico
from at.cpbot.cpadapter import CpBotAdapter
import at.cpbot.funciones as funciones
import at.cpbot.service_layer.sesion as sesion
from at.cpbot.adapters.sqla_repository_parametros_globales import ParametrosGlobalesRepository
from at.cpbot.abmarbol import Arboles
from at.cpbot.repository_funciones import Funciones
from at.cpbot.sqla_repository_charla import CharlasRepository
from at.cpbot.sqla_repository_filtros_palabras import FiltrosRepository
from at.cpbot.tagsweb import Tags
from at.utiles.decoradores import con_try
from at.utiles.hilos import ejecutar_tarea
from at.utiles.singleton import Singleton

HOST = '0.0.0.0'

app = Flask(__name__)

utiles.errores(app)

singleton = Singleton()

cp_bot = CpBotBd()
cpadapter = CpBotAdapter()

datos_cliente = None


@app.route('/nuevo_msj', methods=['POST'])
@con_try(nombre_producto)
def nuevo_msj():
    print("\n" * 4)
    print("llega un mensajeeeeeeeeee##############################################################################")
    print("##############################################################################")
    print("##############################################################################")
    # Obtengo todos los parámetros recibidos
    datos_recibidos = request.get_data()

    # Los parámetros llegan en formato JSON
    datos_recibidos = utiles.decode_json(datos_recibidos)

    # Guardo cada parámetro
    tel_origen = str(datos_recibidos['tel_origen'])
    tel_destino = str(datos_recibidos['tel_destino'])
    msj = datos_recibidos['msj']

    # msj = msj.encode('utf-8')
    enviado_por = str(datos_recibidos['enviado_por'])

    # tipo 1 es menasje entrante
    tipo = 1
    fecha_hora = str(datos_recibidos['fecha_hora'])
    clase = str(datos_recibidos['clase'])

    try:  # Inicio un try para verificar si el telefono_origen es un numero(no esta agendado)

        telefono = tel_origen[1:]  # Guardo el telefono sin el '+'

        telefono = telefono.replace('-', '').replace(' ', '')  # Borro los guiones del telefono

        int(telefono)  # Intento convertirlo en un numero

        telefono = '+' + telefono  # Si llego hasta aca es porque es un numero de telefono

    except:

        telefono = tel_origen  # Si no es un telefono, lo guardo como lo recibi

    # Creo un diccionario para enviar a la funcion nuevo_msj
    dic = {
        'tel_origen': telefono,
        'tel_destino': tel_destino,
        'msj': msj,
        'enviado_por': enviado_por,
        'tipo': tipo,
        'fecha_hora': fecha_hora,
        'clase': clase
    }

    try:
        # Llamo a la función nuevo_msj
        rta = controlador_bot.nuevo_msj(dic)
        print("rta web###########################################################################################:")
        print("rta web###########################################################################################:")
        print("\n\nRTA_CONTROLADOR NUEVO MENSAJE ", rta)
        # Retorno la respuesta obtenida
        return utiles.responder(rta)

    except Exception as e:  # Si falla

        print('\n este es el trackeboack' + traceback.format_exc() + '\n')

        # cambia la charla a manual
        cambiar_automatico(telefono, 0, datos_cliente['ip_atmsj'])

        # Retorno la respuesta obtenida
        return utiles.responder({'Estado': True, 'Respuesta': u'Error en controlador bot, pasando a manual'})


def run():
    print("levantando cpbot")
    # para levantar local/servidor propio. usar este numero de empresa y ese puerto
    #     ./srvweb-cpbot -empresa 1_3 -puerto 9999
    # para levantar amazon
    #     ./srvweb-cpbot -empresa 1_66 -puerto 10066 -baselocal false

    IP_ATMSJ = None
    global datos_cliente

    # Al ejecutar el servicio se le deben pasar los siguientes parámetros
    parser = argparse.ArgumentParser(description='cpbot')

    # Se puede setear un puerto, en caso de ser local sera el puerto 10003
    parser.add_argument(
        '-puerto',
        help='puerto donde se va a conectar',
        required=True
    )

    # Se puede setear un puerto, en caso de ser local sera el puerto 10003
    parser.add_argument(
        '-baselocal',
        help='baselocal',
        required=False
    )

    # Se puede setear la empresa,
    # la empresa LOCAL o si se ejecuta en cliente clasico ES LA 1_3. es simplemente convencion
    parser.add_argument(
        '-empresa',
        help='codigo de la empresa que se va a conectar',
        required=True
    )

    # Guardo los parámetros recibidos
    args = []
    args.extend(sys.argv)
    args.pop(0)
    args = parser.parse_args(args)

    baselocal = args.baselocal
    puerto = int(args.puerto)
    empresa = args.empresa
    utiles.numero_empresa = empresa

    if not baselocal:
        # Si baselocal es false la pisa con el de la empresa porque el codigo estaria corriendo en amazono donde sea
        pisar_bases_de_datos(empresa)

    rta_datos = cp_bot.obtener_datos_cliente(empresa)


    try:
        datos_cliente = rta_datos['Dato']

        IP_ATMSJ = rta_datos['Dato']['ip_atmsj']

        IP_ATAPIS_MSJ = rta_datos['Dato']['ip_atapis']
    except Exception:
        raise (AttributeError, "no hay datos de empresa, revisar que este todo bien la bd del cpbot")

    print("ipatmsj:", IP_ATMSJ)
    print("ipatapis:", IP_ATAPIS_MSJ)

    rta = funciones.finalizar_charlas_viejas(IP_ATMSJ)

    print("fianlizado de charlas viejas:", rta)

    ejecutar_tarea(funciones.finalizar_charlas_viejas, (IP_ATMSJ,), 600)

    ejecutar_tarea(funciones.ping, (IP_ATAPIS_MSJ,), 20)

    # la llama aca asi se piden los filtros apenas se levanta el servidor.
    # sino tiene que esperar a q se ejecute la tarea
    funciones.actualizar_filtros_palabras()
    ejecutar_tarea(funciones.actualizar_filtros_palabras, (), 120)  # actualiza si hay nuevas palabras

    # inicializar parametros globales, inicializa parametros globales. obvio
    sesion.inicializar_parametros_globales()

    # Inicio el servidor web en el puerto dado
    utiles.iniciar_web(app, HOST, puerto)

    # pongo la ip hard-coded porque generalmente este servicio estará en el mismo server atmsj,
    # pero se podria modificar en un futuro
    mi_ip = "http://127.0.0.1:" + str(puerto)
    # anuncia el puerto a atmsj, asi despues atmsj puede responderle
    anunciar_ip(mi_ip, IP_ATMSJ)


def pisar_bases_de_datos(empresa):
    # # esto funciona asi: se debe pasar la CLASE (NO UNA INSTANCIA Y NO EL NOMBRE) de cada clase que use el dbswitch
    # las clases no pueden tener el decorador singleton sino no funciona pòrque conecta a las bd locales
    # cp2000,atmsj,atapis

    seleccionar_bd(CpBotAdapter, 'cp', empresa, False)
    seleccionar_bd(FiltrosRepository, 'cp', empresa, False)

    seleccionar_bd(Arboles, 'cpbot', empresa, False)
    seleccionar_bd(CharlasRepository, 'cpbot', empresa, False)
    seleccionar_bd(CpBotBd, 'cpbot', empresa, False)
    seleccionar_bd(Funciones, 'cpbot', empresa, False)
    seleccionar_bd(Tags, 'cpbot', empresa, False)

    seleccionar_bd(ParametrosGlobalesRepository, 'atapis', empresa, False)
