# -*- coding: utf-8 -*-

import json
from flask import make_response
from gevent.pywsgi import WSGIServer
from gevent import monkey
from geventwebsocket.handler import WebSocketHandler

from srvweb.utiles import _
from at.utiles.conversiones import respuesta


IP_VALIDAS = []
numero_empresa=None


# print('Return de la funcion')
# print(ipvalidas())
#
# print('IP_VALIDAS')
# print(IP_VALIDAS)

def iniciar_web(app, host, puerto, key=None, crt=None):
    # Uso monkey patch para Geven
    monkey.patch_all()

    # Si tengo una key y un certificado quiere decir que voy a levantar un
    # servidor web SSL
    if key is not None or crt is not None:
        # Inicio el servidor web en el puerto dado con los certificados
        http_server = WSGIServer(
            (host, puerto), app, handler_class=WebSocketHandler,
            keyfile=key, certfile=crt
        )
    else:
        # Si no tengo un certificado
        # solo inicio el servidor web en el puerto dado
        http_server = WSGIServer(
            (host, puerto), app, handler_class=WebSocketHandler
        )

    http_server.serve_forever()


def errores(app):
    @app.errorhandler(403)
    def error_403(error):
        rta = {}
        rta['Estado'] = False
        rta['Respuesta'] = 'ERROR 403'
        return responder(rta)

    @app.errorhandler(404)
    def error_404(error):
        rta = {}
        rta['Estado'] = False
        rta['Respuesta'] = 'ERROR 404'
        return responder(rta)

    @app.errorhandler(410)
    def error_410(error):
        rta = {}
        rta['Estado'] = False
        rta['Respuesta'] = 'ERROR 410'
        return responder(rta)

    @app.errorhandler(500)
    def error_500(error):
        rta = {}
        rta['Estado'] = False
        rta['Respuesta'] = 'ERROR 500'
        return responder(rta)


def encode_json(datos):
    # Paso a formato JSON
    return json.dumps(datos)


def decode_json(datos):
    # De formato JSON lo paso a DICT
    return json.loads(datos.decode('utf-8'))


def responder(rta):
    # Paso a JSON la respuesta recibida
    rta = encode_json(rta)

    # Armo la respuesta web
    rta = make_response(rta)

    # Saco el dato del servidor de la cabecera por seguridad
    rta.headers['server'] = 'server'

    # Retorno la respuesta web con la cabecera modificada
    return rta


def validar_ip(ip):
    # Si la IP recibida no es válida
    # if ip not in IP_VALIDAS:
    #     # Retorno False y un mensaje
    #     return respuesta(False, _(u'La IP no es válida:',ip))

    # Si la IP recibida es válida retorno True y un mensaje
    return respuesta(True, _(u'La IP es válida'))
