# -*- coding: utf-8 -*-

import unittest
import datetime
from datetime import timedelta

from at.cp import _
from at.cp.ope.servicio import Servicio
from at.cp.ope.params import ParametrosTool
from at.cp.ope.cliente import ClienteCP
from at.cp.utiles.estado_despacho import EstadoDespacho
from at.orm.abm import ABM
from at.utiles.servidores import DateTimeHelper


class ServicioTesting(Servicio):

    def __init__(self):
        cp = {'user': 'frula',
              'passwd': 'changeme',
              'dbname': 'tests',
              'name': 'cp',
              'host': '192.168.0.51',
              'engine': 'mysql',
              'port': '3306',
              }

        test_pidetutaxi = {
            'user': 'postgres',
            'passwd': 'TrackAir',
            'dbname': 'test_pidetutaxi',
            'name': 'test_pidetutaxi',
            'host': '192.168.0.68',
            'engine': 'postgres',
            'port': '5432',
        }

        Servicio.__init__(self)
        self.cp = ABM(cp)
        self.dth = DateTimeHelper()
        self.moviles.cp = ABM(cp)

        self.cliente = ClienteCP()
        self.cliente.cp = ABM(cp)
        self.cupones.cp = ABM(cp)
        self.cupones.clientes.cp = ABM(cp)
        self.cupones.usuario.cp = ABM(cp)
        self.clientes_ctacte.cp = ABM(cp)
        self.filtros.cp = ABM(cp)
        self.param_tool = ParametrosTool()
        self.ed = EstadoDespacho()
        self.importes_fijos.cp = ABM(cp)
        self.proveedores_web.cp = ABM(cp)
        self.proveedores_web_utiles.cp = ABM(cp)

        try:
            self.pidetutaxi = ABM(test_pidetutaxi)
            self.comandos.pidetutaxi = ABM(test_pidetutaxi)
        except:
            print 'No tengo la BD pidetutaxi'


class TestServicio(unittest.TestCase):

    def setUp(self):
        self.servicio = ServicioTesting()

        self.dth = DateTimeHelper()

        self.servicio.cp.borrar('actual')
        self.servicio.cp.borrar('moviles')
        self.servicio.cp.borrar('clientes_direcciones')
        self.servicio.cp.borrar('clientes')
        self.servicio.cp.borrar('logbs')
        self.servicio.cp.borrar('motivos_sanciones')
        self.servicio.cp.borrar('commands')
        self.servicio.cp.borrar('zonas')
        self.servicio.cp.borrar('cupones')
        self.servicio.cp.borrar('importes_fijos')
        self.servicio.cp.borrar('cupones_modificaciones')
        self.servicio.cp.borrar('clientes_ctacte')
        self.servicio.cp.borrar('centros_de_costos')
        self.servicio.cp.borrar('centros_de_costos_personas')
        self.servicio.cp.borrar('clientes_mails')
        self.servicio.cp.borrar('historico')
        self.servicio.cp.borrar('servicios_proveedores_web')
        self.servicio.cp.borrar('usuarios')
        self.servicio.cp.borrar('calles_zonas')
        self.servicio.cp.borrar('filtros')
        self.servicio.cp.borrar('filtros_servicios')
        try:
            self.servicio.pidetutaxi.borrar('comandos')
        except:
            print 'No se pudieron limpiar los comandos de pidetutaxi'

    def tearDown(self):
        self.servicio.cp.borrar('actual')
        self.servicio.cp.borrar('moviles')
        self.servicio.cp.borrar('clientes_direcciones')
        self.servicio.cp.borrar('clientes')
        self.servicio.cp.borrar('logbs')
        self.servicio.cp.borrar('motivos_sanciones')
        self.servicio.cp.borrar('commands')
        self.servicio.cp.borrar('zonas')
        self.servicio.cp.borrar('cupones')
        self.servicio.cp.borrar('importes_fijos')
        self.servicio.cp.borrar('cupones_modificaciones')
        self.servicio.cp.borrar('clientes_ctacte')
        self.servicio.cp.borrar('centros_de_costos')
        self.servicio.cp.borrar('centros_de_costos_personas')
        self.servicio.cp.borrar('clientes_mails')
        self.servicio.cp.borrar('historico')
        self.servicio.cp.borrar('servicios_proveedores_web')
        self.servicio.cp.borrar('usuarios')
        self.servicio.cp.borrar('calles_zonas')
        self.servicio.cp.borrar('filtros')
        self.servicio.cp.borrar('filtros_servicios')
        try:
            self.servicio.pidetutaxi.borrar('comandos')
        except:
            print 'No se pudieron limpiar los comandos de pidetutaxi'

    def test_pasar_servicio_manual(self):
        # el servicio no existe
        Telefono = '1234'
        FechaServicio = '2009-08-19'
        HoraServicio = '1900-01-01 16:00:00'
        IdUser = 'nombre_usuario'
        r = self.servicio.pasar_servicio_manual(Telefono, FechaServicio,
                                                HoraServicio, IdUser)
        self.assertTrue(r is False)

        # le paso mal los parametros
        Telefono = '1 asd'
        FechaServicio = None
        HoraServicio = None
        IdUser = 'zxc'
        r = self.servicio.pasar_servicio_manual(Telefono, FechaServicio,
                                                HoraServicio, IdUser)
        self.assertTrue(r is False)

        # Agrego un registro en la tabla "actual"
        dic = {}
        dic['Telefono'] = '4949494'
        dic['FechaServicio'] = '2018-05-18'
        dic['HoraServicio'] = '1900-01-01 17:00:00'
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        Telefono = '4949494'
        FechaServicio = '2018-05-18'
        HoraServicio = '1900-01-01 17:00:00'
        IdUser = 'paco'
        validar_si_esta_pausado = False
        rta = self.servicio.pasar_servicio_manual(
            Telefono, FechaServicio, HoraServicio, IdUser,
            validar_si_esta_pausado
        )
        self.assertTrue(rta is True)

    def test_grabar_servicio_actual(self):
        now = self.dth.get_now()
        fecha = datetime.date(now.year, now.month, now.day)
        hs = datetime.datetime(1900, 01, 01, now.hour, now.minute, now.second)

        # agrego un registro
        dic = {}
        dic['Telefono'] = '999666333'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'San Luis 931'
        dic['IdZona'] = 1
        #dic['Observaciones'] = ''
        dic['Observaciones'] = u'QUIEN VIAJA: ADRI\xc1N KITTER\nCUANDO: MA\xd1ANA 7:30 AM\nORIGEN: SAN LUIS 3365 3\xb0 A \u2013 ALMAGRO\nCELULAR: +54 9 11 3691-9055\nDESTINO: LIMA\n\nCon espera y regreso por favor\n\nGracias!\nKm estimados: 11.51\nMinutos de espera: 60\nDestinos:\nLima 1261, Constituci\xf3n, Comuna 1, Buenos Aires, Ciudad Aut\xf3noma de Buenos Aires, Argentina\nSan Luis 3365, Buenos Aires, Argentina\n'
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # un nuevo servicio del mismo cliente
        dic = {}
        dic['Telefono'] = '868686'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'San Luis 931'
        dic['IdZona'] = 1
        dic['Observaciones'] = 'Tiene rejas negras * Viaja con un gato'
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # modifico un servicio, le cambio la prioridad,
        # la prioridad original se debe mantener
        dic = {}
        dic['Telefono'] = '999666333'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Nombre modificado'
        dic['Direccion'] = 'Zeballos 102'
        dic['IdZona'] = 1
        dic['Observaciones'] = u'QUIEN VIAJA: ADRI\xc1N KITTER\nCUANDO: MA\xd1ANA 7:30 AM\nORIGEN: SAN LUIS 3365 3\xb0 A \u2013 ALMAGRO\nCELULAR: +54 9 11 3691-9055\nDESTINO: LIMA\n\nCon espera y regreso por favor\n\nGracias!\nKm estimados: 11.51\nMinutos de espera: 60\nDestinos:\nLima 1261, Constituci\xf3n, Comuna 1, Buenos Aires, Ciudad Aut\xf3noma de Buenos Aires, Argentina\nSan Luis 3365, Buenos Aires, Argentina\n MODIFICADO'
        dic['Prioridad'] = 2
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 102
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = dic['FechaServicio']
        dic['HoraServicioOriginal'] = dic['HoraServicio']

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # pruebas para pasar el servicio a automático
        dic = {}
        dic['Telefono'] = '885522'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Paco Rivas'
        dic['Direccion'] = 'España 1130'
        dic['IdZona'] = 1
        dic['Observaciones'] = None
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 1056
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1130
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # pruebas para pasar el servicio a automático
        # como la fecha es para más adelante no debería
        # pasarlo a manual
        # la prioridad de este servicio va a ser 4 () servicio anticipado)
        dic = {}
        dic['Telefono'] = '535353'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Del futuro'
        dic['Direccion'] = u'España 1130'
        dic['IdZona'] = 1
        dic['Observaciones'] = None
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 1056
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1130
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['Alias'] = 'casa'

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # grabo un servicio nuevo con un cliente nuevo
        # para ver las observaciones
        dic = {}

        dic['Telefono'] = '323232'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 25
        dic['Longitud'] = 45
        dic['TelefonoDeRef'] = '155252525'

        dic['Nombre'] = 'Jose'
        dic['Direccion'] = u'España 1130'
        dic['IdZona'] = 1
        dic['Observaciones'] = 'Casa de puerta de madera * Zona complicada'
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 1056
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1130
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # modifico este servicio
        dic['Telefono'] = '323232'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 60
        dic['Longitud'] = 75
        dic['TelefonoDeRef'] = '155606060'

        dic['Nombre'] = 'Jose Ramires'
        dic['Direccion'] = u'España 1130'
        dic['IdZona'] = 1
        dic['Observaciones'] = 'Casa con rejas'
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 1056
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1130
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = dic['FechaServicio']
        dic['HoraServicioOriginal'] = dic['HoraServicio']

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # prueba con todos los datos
        dic = {}

        dic['Telefono'] = '121212'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = '1'
        dic['Nivel'] = 1

        dic['Zona'] = '1'
        dic['ZonaHasta'] = '1'
        dic['Telefonista'] = 'Roberta'
        dic['EspacioEnBaul'] = 1
        dic['Aire'] = 1
        dic['SinGNC'] = 1
        dic['VIP'] = 1
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 50
        dic['Longitud'] = 75
        dic['TelefonoDeRef'] = '156626262'

        dic['Nombre'] = 'Teddy'
        dic['Direccion'] = u'España 1130'
        dic['IdZona'] = 1
        dic['Observaciones'] = 'No anda el timbre * Usa saco y corbata'
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 1056
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1130
        dic['LetraAltura'] = None
        dic['Depto'] = '3 A'
        dic['DireccionEnMapa'] = 0

        dic['Alias'] = 'casa'

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # prueba con dos calles
        dic = {}

        dic['Telefono'] = '747474'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = '1'
        dic['Nivel'] = 1

        dic['Zona'] = '1'
        dic['ZonaHasta'] = '1'
        dic['Telefonista'] = 'Roberta'
        dic['EspacioEnBaul'] = 1
        dic['Aire'] = 1
        dic['SinGNC'] = 1
        dic['VIP'] = 1
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 50
        dic['Longitud'] = 75
        dic['TelefonoDeRef'] = '155151515'

        dic['Nombre'] = 'Diego'
        dic['Direccion'] = u'España y San Juan'
        dic['IdZona'] = 1
        dic['Observaciones'] = 'No anda el timbre * Usa saco y corbata'
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 1056
        dic['IdCalle2'] = 1057
        dic['IdNodo'] = None
        dic['Altura'] = 1130
        dic['LetraAltura'] = 'a'
        dic['Depto'] = '3 a'
        dic['DireccionEnMapa'] = 0

        dic['Alias'] = 'casa'

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # agrego un servicio con fecha del pasado
        dic = {}

        dic['Telefono'] = '020202'

        dic['FechaServicio'] = '2010-09-01'
        dic['HoraServicio'] = '1900-01-01 14:00:00'
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = '1'
        dic['Nivel'] = 1

        dic['Zona'] = '1'
        dic['ZonaHasta'] = '1'
        dic['Telefonista'] = 'Roberta'
        dic['EspacioEnBaul'] = 1
        dic['Aire'] = 1
        dic['SinGNC'] = 1
        dic['VIP'] = 1
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 50
        dic['Longitud'] = 75
        dic['TelefonoDeRef'] = '155151515'

        dic['Nombre'] = 'Diego'
        dic['Direccion'] = u'España y San Juan'
        dic['IdZona'] = 1
        dic['Observaciones'] = 'No anda el timbre * Usa saco y corbata'
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 1056
        dic['IdCalle2'] = 1057
        dic['IdNodo'] = None
        dic['Altura'] = 1130
        dic['LetraAltura'] = 'a'
        dic['Depto'] = '3 a'
        dic['DireccionEnMapa'] = 0

        dic['Alias'] = 'casa'

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)

        # agrego un servicio a editar con fecha del pasado
        dic = {}

        dic['Telefono'] = '141414'

        dic['FechaServicio'] = '2010-09-21'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = '1'
        dic['Nivel'] = 1

        dic['Zona'] = '1'
        dic['ZonaHasta'] = '1'
        dic['Telefonista'] = 'Roberta'
        dic['EspacioEnBaul'] = 1
        dic['Aire'] = 1
        dic['SinGNC'] = 1
        dic['VIP'] = 1
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 50
        dic['Longitud'] = 75
        dic['TelefonoDeRef'] = '155151515'

        dic['Nombre'] = 'Diego'
        dic['Direccion'] = u'España y San Juan'
        dic['IdZona'] = 1
        dic['Observaciones'] = 'No anda el timbre * Usa saco y corbata'
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 1056
        dic['IdCalle2'] = 1057
        dic['IdNodo'] = None
        dic['Altura'] = 1130
        dic['LetraAltura'] = 'a'
        dic['Depto'] = '3 a'
        dic['DireccionEnMapa'] = 0

        dic['Alias'] = 'casa'

        dic['FechaServicioOriginal'] = '2010-09-21'
        dic['HoraServicioOriginal'] = '1900-01-01 15:00:00'

        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)

        # agrego un registro con prioridad 0, debe decir 1 en prioridad
        dic = {}
        dic['Telefono'] = '474747'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'San Luis 931'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 0
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # agrego un registro con un número de teléfono con espacios adelante
        dic = {}
        dic['Telefono'] = '   585858'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'San Luis 931'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 0
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # agrego un registro con un número de teléfono con espacios atras
        dic = {}
        dic['Telefono'] = '969696          '

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'San Luis 931'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 0
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # agrego un registro con un número con solo espacios en blanco
        dic = {}
        dic['Telefono'] = '      '

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'San Luis 931'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 0
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)

        # agrego un registro para luego intentar duplicado
        dic = {}
        dic['Telefono'] = '333333'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = u'España 1130'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 0
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # duplico un servicio, AHORA SE PUEDE, LE AGREGA UN SEGUNDO
        dic['Telefono'] = '333333'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = u'España 1130'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 0
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Agrego un servicio con barrio
        dic = {}
        dic['Telefono'] = 'telconbarrio'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'San Luis 931'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 0
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0
        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None
        dic['Barrio'] = u'Barrio Martín'
        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # agrego un registro
        dic = {}
        dic['Telefono'] = 'con posicion 1'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'Urquiza 7164'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333579
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 7164
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        # Agrego una zona para obtener su nombre
        dic_zona = {}
        dic_zona['IdSucursal'] = 1
        dic_zona['IdZona'] = 222
        dic_zona['Zona'] = 'Zona 222'
        agregar = self.servicio.cp.agregar('zonas', **dic_zona)
        self.assertTrue(agregar is True)

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # agrego un registro
        dic = {}
        dic['Telefono'] = 'con posicion 2'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'Calle con bis'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333411
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = -600
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        # Agrego una zona para obtener su nombre
        dic_zona = {}
        dic_zona['IdSucursal'] = 1
        dic_zona['IdZona'] = 4
        dic_zona['Zona'] = 'Zona 4'
        agregar = self.servicio.cp.agregar('zonas', **dic_zona)
        self.assertTrue(agregar is True)

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # agrego un registro
        dic = {}
        dic['Telefono'] = 'con posicion 3'

        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1

        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None

        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'Calle con letra'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = 'A'
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        #dic['Alias'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Agrego un servicio con cliente
        dic = {}
        dic['Telefono'] = 'con cliente'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        # Le paso el id del cliente con cta. cte.
        dic['IdClienteCtaCte'] = 25

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Agrego un servicio con el id cliente modificado
        dic = {}
        dic['Telefono'] = 'con cliente'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs + timedelta(minutes=1))
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        # Modifico el id del cliente
        dic['IdClienteCtaCte'] = 26

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Agrego un servicio con cta. cte.
        # No le paso datos del cliente
        dic = {}
        dic['Telefono'] = 'con cta cte 1'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = None
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        # Sin datos del cliente con cta. cte.
        dic['IdClienteCtaCte'] = None
        dic['IdCentroDeCosto'] = None
        dic['IdPersona'] = None
        dic['DNIPasajero'] = None

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Agrego un servicio con cta. cte.
        # Le paso todos los datos
        dic = {}
        dic['Telefono'] = 'con cta cte 2'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'Lorena'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Soy con cta cte 2'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        # Agrego los datos de la cta. cte.
        dic['IdClienteCtaCte'] = 32
        dic['IdCentroDeCosto'] = 4
        dic['IdPersona'] = 2
        dic['PIN'] = 'el_pin_de_la_persona_2'
        dic['DNIPasajero'] = 33228899

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        # Agrego una persona con cta. cte.
        a = {}
        a['IdCliente'] = 32
        a['IdCentroDeCosto'] = 4
        a['IdPersona'] = 2
        a['Nombre'] = 'Persona 2'
        a['Habilitado'] = 1
        a['PIN'] = 'el_pin_de_la_persona_2'
        agregar = self.servicio.cp.agregar('centros_de_costos_personas', **a)
        self.assertTrue(agregar is True)

        # Agrego el cliente 32
        a = {}
        a['IdCliente'] = 32
        agregar = self.servicio.cp.agregar('clientes_ctacte', **a)
        self.assertTrue(agregar is True)

        # Agrego el centro de costo 4 del cliente 32
        a = {}
        a['IdCliente'] = 32
        a['IdCentroDeCosto'] = 4
        a['Nombre'] = 'centro de costo 4 del cliente 32'
        agregar = self.servicio.cp.agregar('centros_de_costos', **a)
        self.assertTrue(agregar is True)

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Modifico un servicio con cta. cte que tiene un cupón guardado
        # Le paso mal el teléfono
        dic = {}
        dic['Telefono'] = 'con cta cte 2 asd'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'Lorena'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Soy con cta cte 2'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        # Agrego los datos de la cta. cte.
        dic['IdClienteCtaCte'] = 32
        dic['IdCentroDeCosto'] = 4
        dic['IdPersona'] = 2
        dic['PIN'] = 'el_pin_de_la_persona_2'
        dic['DNIPasajero'] = 33228899

        dic['FechaServicioOriginal'] = str(fecha)
        dic['HoraServicioOriginal'] = str(hs)

        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)

        # Modifico un servicio con cta. cte que tiene un cupón guardado
        # Le paso bien el teléfono
        dic = {}
        dic['Telefono'] = 'con cta cte 2'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'juan'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Soy con cta cte 2 mod'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = str(fecha)
        dic['HoraServicioOriginal'] = str(hs)

        # Agrego los datos de la cta. cte.
        dic['IdClienteCtaCte'] = 32
        dic['IdCentroDeCosto'] = 4
        dic['IdPersona'] = 2
        dic['PIN'] = 'el_pin_de_la_persona_2'
        dic['DNIPasajero'] = 33228899

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Agrego un servicio con cuenta corriente
        dic = {}
        dic['Telefono'] = 'con cta cte 3'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'Lorena'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Soy con cta cte 3'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        # Agrego los datos de la cta. cte.
        dic['IdClienteCtaCte'] = 32
        dic['IdCentroDeCosto'] = 4
        dic['IdPersona'] = 2
        dic['PIN'] = 'el_pin_de_la_persona_2'
        dic['DNIPasajero'] = 33228899

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Le saco la cta cte al servicio
        dic = {}
        dic['Telefono'] = 'con cta cte 3'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'Lorena'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Soy con cta cte 3'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = str(fecha)
        dic['HoraServicioOriginal'] = str(hs)

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        ### CASO PARA PROBAR EL PIN DE LA PERSONA ###

        # Agrego un servicio con cuenta corriente
        dic = {}
        dic['Telefono'] = 'con cta cte 4'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'Lorena'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Soy con cta cte 4'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        # Agrego los datos de la cta. cte.
        dic['IdClienteCtaCte'] = 40
        dic['IdCentroDeCosto'] = 6
        dic['IdPersona'] = 4
        dic['PIN'] = u'canción1234'
        dic['DNIPasajero'] = 33228899

        # Le paso una persona que no está cargada en el sistema
        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)

        # Agrego el cliente 40
        a = {}
        a['IdCliente'] = 40
        agregar = self.servicio.cp.agregar('clientes_ctacte', **a)
        self.assertTrue(agregar is True)

        # Agrego el centro de costo 6 del cliente 40
        a = {}
        a['IdCliente'] = 40
        a['IdCentroDeCosto'] = 6
        a['Nombre'] = 'centro de costo 6 del cliente 40'
        agregar = self.servicio.cp.agregar('centros_de_costos', **a)
        self.assertTrue(agregar is True)

        # Ahora si agrego la persona con cta. cte.
        a = {}
        a['IdCliente'] = 40
        a['IdCentroDeCosto'] = 6
        a['IdPersona'] = 4
        a['Nombre'] = 'Persona 4'
        a['Habilitado'] = 1
        a['PIN'] = u'canción1234'
        agregar = self.servicio.cp.agregar('centros_de_costos_personas', **a)
        self.assertTrue(agregar is True)

        # Le paso bien el PIN
        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Le paso mal el PIN
        dic['PIN'] = 'pin incorrecto'
        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)

        # Le paso el PIN vacío
        dic['PIN'] = ''
        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)

        # No le paso un PIN
        dic['PIN'] = None
        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)

        # Modifico un servicio con el dato PIN
        dic = {}
        dic['Telefono'] = 'con cta cte 4'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'juan'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Soy con cta cte 4'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = str(fecha)
        dic['HoraServicioOriginal'] = str(hs)

        # Agrego los datos de la cta. cte.
        dic['IdClienteCtaCte'] = 40
        dic['IdCentroDeCosto'] = 6
        dic['IdPersona'] = 5
        dic['PIN'] = u'otro_pin'
        dic['DNIPasajero'] = 33228899

        # El PIN ahora es incorrecto
        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)

        # Agrego una persona con cta. cte.
        a = {}
        a['IdCliente'] = 40
        a['IdCentroDeCosto'] = 6
        a['IdPersona'] = 5
        a['Nombre'] = 'Persona 5'
        a['Habilitado'] = 1
        a['PIN'] = 'otro_pin'
        agregar = self.servicio.cp.agregar('centros_de_costos_personas', **a)
        self.assertTrue(agregar is True)

        # Le cambio la persona y su PIN
        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        ### FIN CASO PARA PROBAR EL PIN DE LA PERSONA ###

        ### PROBANDO EL NUEVO CAMPO EntreCalles ###

        # Agrego un servicio con el dato "EntreCalles"
        dic = {}
        dic['Telefono'] = 'entre calles 1'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'juan'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Con entre calles'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        # Dato nuevo "EntreCalles"
        dic['EntreCalles'] = u'Entre asd y qwe'

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Modifico un servicio con el dato "EntreCalles"
        dic = {}
        dic['Telefono'] = 'entre calles 1'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'juan'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Con entre calles'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = str(fecha)
        dic['HoraServicioOriginal'] = str(hs)

        # Dato nuevo "EntreCalles"
        dic['EntreCalles'] = u'Entre asd y qwe MOD'

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Agrego un servicio con una observación interna
        dic = {}
        dic['Telefono'] = 'con obs int'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'juan'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Con observacion interna'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        dic['EntreCalles'] = u'Entre asd y qwe'
        dic['ObservacionInterna'] = u'con una observación interna * y una fija'

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Modifico un servicio con una observación interna
        dic = {}
        dic['Telefono'] = 'con obs int'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'juan'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Con observacion interna'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1333408
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 1250
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0

        dic['FechaServicioOriginal'] = str(fecha)
        dic['HoraServicioOriginal'] = str(hs)

        dic['EntreCalles'] = u'Entre asd y qwe'
        dic['ObservacionInterna'] = u'con una observación interna MOD * fija 2'

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Le paso un servicio con filtros
        dic = {}
        dic['Telefono'] = 'con_filtros'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 0
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'San Luis 931'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0
        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None
        dic['Filtros'] = [5, 8, 36]

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Modifico los filtros de un servicio
        dic = {}
        dic['Telefono'] = 'con_filtros'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 1
        dic['Nivel'] = 0
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = None
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 0
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Juan Francisco Riabiz'
        dic['Direccion'] = 'San Luis 931'
        dic['IdZona'] = 1
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '1'
        dic['IdCalle1'] = 775
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 931
        dic['LetraAltura'] = None
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0
        dic['FechaServicioOriginal'] = str(fecha)
        dic['HoraServicioOriginal'] = str(hs)
        dic['Filtros'] = [19, 36]

        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        ### PROBANDO LOS IMPORTES FIJOS #######################################
        # Agrego un servicio con cuenta corriente
        dic = {}
        dic['Telefono'] = 'con cta cte 10'
        dic['FechaServicio'] = str(fecha)
        dic['HoraServicio'] = str(hs)
        dic['IdSucursal'] = 1
        dic['IdZonaHasta'] = 15
        dic['Nivel'] = 1
        dic['Zona'] = None
        dic['ZonaHasta'] = None
        dic['Telefonista'] = 'juan'
        dic['EspacioEnBaul'] = 0
        dic['Aire'] = 0
        dic['SinGNC'] = 0
        dic['VIP'] = 0
        dic['AceptaCtaCte'] = 1
        dic['Latitud'] = 0
        dic['Longitud'] = 0
        dic['TelefonoDeRef'] = None
        dic['Nombre'] = 'Soy con cta cte 10'
        dic['Direccion'] = 'la direccion se arma sola'
        dic['IdZona'] = 26
        dic['Observaciones'] = ''
        dic['Prioridad'] = 1
        dic['IdCiudad'] = '324'
        dic['IdCalle1'] = 1234
        dic['IdCalle2'] = None
        dic['IdNodo'] = None
        dic['Altura'] = 789
        dic['LetraAltura'] = ''
        dic['Depto'] = None
        dic['DireccionEnMapa'] = 0
        dic['Siniestro'] = '12Ab'
        dic['EnviarSms'] = '153705966'

        dic['FechaServicioOriginal'] = None
        dic['HoraServicioOriginal'] = None

        # Agrego los datos de la cta. cte.
        dic['IdClienteCtaCte'] = 50

        # Agrego el cliente 50
        a = {}
        a['IdCliente'] = 50
        agregar = self.servicio.cp.agregar('clientes_ctacte', **a)
        self.assertTrue(agregar is True)

        # Agrego un importe fijo
        a = {}
        a['IdClienteCtaCte'] = 50
        a['IdSucursal'] = 1
        a['IdZonaDesde'] = 26
        a['IdZonaHasta'] = 15
        a['ImporteCliente'] = 59.5
        a['ImporteMovil'] = 50.5
        agregar = self.servicio.cp.agregar('importes_fijos', **a)
        self.assertTrue(agregar is True)

        # Le paso un cliente con un importe fijo entre la zona 26 y la 15
        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)

        # Modifico la zona hasta del cliente
        dic['IdZonaHasta'] = 99
        dic['FechaServicioOriginal'] = str(fecha)
        dic['HoraServicioOriginal'] = str(hs)
        r = self.servicio.grabar_servicio(dic)
        self.assertTrue(r is None)
        #######################################################################

        ### PROBANDO DE OBTENER LA CIUDAD CON LA LATITUD Y LA LONGITUD ########
        # Le paso buscar_ciudad_por_posicion en True
        # la latitud y la longitud no existen
        dic = {}
        dic['Telefono'] = 'obtener_ciudad'
        dic['FechaServicio'] = '2016-05-19'
        dic['HoraServicio'] = '1900-01-01 16:00:00'
        dic['Direccion'] = 'la direccion'
        dic['IdSucursal'] = 1
        dic['Prioridad'] = 1
        dic['AceptaCtaCte'] = 0
        dic['Telefonista'] = 'Tests'
        dic['Latitud'] = 123
        dic['Longitud'] = 456
        dic['IdCiudad'] = '0'
        dic['IdZona'] = 0
        dic['IdProveedorWeb'] = 35
        r = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(r is None)

        # Agrego con una latitud y una longitud valida para encontrar la ciudad
        dic = {}
        dic['Telefono'] = 'obtener_ciudad2'
        dic['FechaServicio'] = '2016-05-19'
        dic['HoraServicio'] = '1900-01-01 17:00:00'
        dic['Direccion'] = 'la direccion 2'
        dic['IdSucursal'] = 1
        dic['Prioridad'] = 1
        dic['AceptaCtaCte'] = 0
        dic['Telefonista'] = 'Tests'
        dic['Latitud'] = -118595.305803184
        dic['Longitud'] = -218582.171636761
        dic['IdCiudad'] = '0'
        dic['IdZona'] = 0
        dic['IdProveedorWeb'] = 36
        dic['IdClienteCtaCte'] = 80
        dic['Siniestro'] = '15879'
        r = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(r is None)

        # Le paso un servicio sin latitud ni longitud
        dic = {}
        dic['Telefono'] = 'obtener_ciudad3'
        dic['FechaServicio'] = '2016-05-19'
        dic['HoraServicio'] = '1900-01-01 18:00:00'
        dic['Direccion'] = 'la direccion 3'
        dic['IdSucursal'] = 1
        dic['Prioridad'] = 1
        dic['AceptaCtaCte'] = 0
        dic['Telefonista'] = 'Tests'
        dic['Latitud'] = None
        dic['Longitud'] = None
        dic['IdCiudad'] = '0'
        dic['IdZona'] = 0
        dic['IdProveedorWeb'] = 37
        r = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(r is None)

        # Agrego el cliente 0
        a = {}
        a['IdCliente'] = 0
        agregar = self.servicio.cp.agregar('clientes_ctacte', **a)
        self.assertTrue(agregar is True)

        # Agrego el centro de costo 0
        a = {}
        a['IdCliente'] = 0
        a['IdCentroDeCosto'] = 0
        a['Nombre'] = 'Centro de costo 0'
        agregar = self.servicio.cp.agregar('centros_de_costos', **a)
        self.assertTrue(agregar is True)

        # Agrego la persona 0
        a = {}
        a['IdCliente'] = 0
        a['IdCentroDeCosto'] = 0
        a['IdPersona'] = 0
        a['Nombre'] = 'Persona 0'
        agregar = self.servicio.cp.agregar('centros_de_costos_personas', **a)
        self.assertTrue(agregar is True)

        # Caso de Gestion Compartida de Gold
        dic = {'Latitud': -124666.45524,
        'IdPersona': 0,
        'IdCentroDeCosto': 0,
        'PIN': '',
        'IdSucursal': 1,
        'IdCiudad': '0',
        'Observaciones': u'Km estimados: 7.20\nDestinos:\nJos\xe9 Andr\xe9s Pacheco de Melo 2580, Buenos Aires, Ciudad Aut\xf3noma de Buenos Aires, Argentina\n',
        'IdZona': 0,
        'Direccion': u'Tacuar\xed 1842, Buenos Aires, Ciudad Aut\xf3noma de Bu',
        'IdClienteCtaCte': 0,
        'Siniestro': u'1381017',
        'AceptaCtaCte': 1,
        'IdProveedorWeb': 2,
        'HoraServicio': u'1900-01-01 16:10:00',
        'Nombre': u'ALBERTI, TEA',
        'FechaServicio': u'2016-05-23',
        'Telefonista': 'Gestion Compartida',
        'Telefono': u'~ 7488',
        'Prioridad': 1,
        'Longitud': -210150.38412}
        r = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(r is None)
        #######################################################################

        ### SERVICIO CON MISMO TELEFONO Y MISMA HORA ##########################
        # Agrego un servicio
        dic = {}
        dic['Telefono'] = 'tel30'
        dic['FechaServicio'] = '2016-05-30'
        dic['HoraServicio'] = '1900-01-01 10:05:11'
        dic['Direccion'] = 'Urquiza 7164'
        dic['IdSucursal'] = 1
        dic['Prioridad'] = 1
        dic['AceptaCtaCte'] = 1
        dic['Telefonista'] = 'Tests'
        dic['IdCiudad'] = '0'
        dic['IdZona'] = 0
        dic['IdProveedorWeb'] = 37
        dic['Latitud'] = -118561.5
        dic['Longitud'] = -218586.047636364
        dic['IdClienteCtaCte'] = 0
        r = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(r is None)

        # Intento agregar un servicio a la misma hora con el mismo teléfono
        # le suma un segundo a la hora
        dic = {}
        dic['Telefono'] = 'tel30'
        dic['FechaServicio'] = '2016-05-30'
        dic['HoraServicio'] = '1900-01-01 10:05:11'
        dic['Direccion'] = 'Zeballos 1870'
        dic['IdSucursal'] = 1
        dic['Prioridad'] = 1
        dic['AceptaCtaCte'] = 1
        dic['Telefonista'] = 'Tests'
        dic['IdCiudad'] = '0'
        dic['IdZona'] = 0
        dic['IdProveedorWeb'] = 37
        dic['Latitud'] = -118631.767898482
        dic['Longitud'] = -218343.310697658
        dic['IdClienteCtaCte'] = 0
        r = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(r is None)

        # Un tercero
        # Intento agregar un servicio a la misma hora con el mismo teléfono
        # le suma otro segundo a la hora
        dic = {}
        dic['Telefono'] = 'tel30'
        dic['FechaServicio'] = '2016-05-30'
        dic['HoraServicio'] = '1900-01-01 10:05:11'
        dic['Direccion'] = 'San Juan 1650'
        dic['IdSucursal'] = 1
        dic['Prioridad'] = 1
        dic['AceptaCtaCte'] = 1
        dic['Telefonista'] = 'Tests'
        dic['IdCiudad'] = '0'
        dic['IdZona'] = 0
        dic['IdProveedorWeb'] = 37
        dic['Latitud'] = -118616.265316756
        dic['Longitud'] = -218328.326710088
        dic['IdClienteCtaCte'] = 0
        r = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(r is None)

        # Agrego un servicio con un teléfono de más de 15 caracteres
        dic = {}
        dic['Telefono'] = u'telefono con más de quince caracteres'
        dic['FechaServicio'] = '2016-06-01'
        dic['HoraServicio'] = '1900-01-01 11:00:00'
        dic['Direccion'] = 'Mitre 1500'
        dic['IdSucursal'] = 1
        dic['Prioridad'] = 1
        dic['AceptaCtaCte'] = 1
        dic['Telefonista'] = 'Tests'
        dic['IdCiudad'] = '0'
        dic['IdZona'] = 0
        dic['IdProveedorWeb'] = 37
        dic['IdClienteCtaCte'] = 0
        r = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(r is None)
        #######################################################################

        # Caso de Gestión Compartida que tiró un error ########################
        # EL PROBLEMA SE DA CUANDO SE QUIERE PASAR UN ID CIUDAD COMO INT,
        # EL ID CIUDAD DEBE SER UN STR
        dic = {'IdCentroDeCosto': 1017L,
            'Aire': 0,
            'IdZona': 7,
            'Zona': 'DESCONOCIDA',
            'LetraAltura': '',
            'Direccion': u'Neuqu\xe9n 869, C1405CKG CABA, Argentina',
            'Siniestro': u'1441945',
            'EspacioEnBaul': 0,
            'Nombre': u'ANDRES TOMASONE',
            'FechaServicio': u'2016-10-11',
            'Telefono': u'1540854845',
            'Nivel': 0,
            'IdCalle1': None,
            'PIN': '1017',
            'Observaciones': u'05:40  ANDRES TOMASONE, NEUQUEN 869 3ro \u201cA\u201d, CEL: 15-4085-4845.\nKm estimados: 7.46\nDestinos:\nAv. San Juan 1132, C1147AAW CABA, Argentina\n',
            'EntreCalles': 'Entre Dr. Nicolas Repetto y Dr  Nicol\xe1s Repetto',
            'Prioridad': 3,
            'Longitud': -210400.3836,
            'ZonaHasta': None,
            'TelefonoDeRef': '',
            'IdSucursal': 1,
            'IdProveedorWeb': 2,
            'IdCiudad': 2238,
            'IdPersona': 1017L,
            'DireccionEnMapa': 1,
            'Depto': None,
            'HoraServicio': u'1900-01-01 05:40:00',
            'Telefonista': 'Gestion Compartida',
            'Altura': None,
            'Latitud': -124617.44664,
            'IdClienteCtaCte': 1017L,
            'IdCalle2': None,
            'SinGNC': 0,
            'IdZonaHasta': 0,
            'VIP': 0,
            'AceptaCtaCte': 1,
            'IdNodo': None}
        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)
        #######################################################################

        # Error de UNICODE con la dirección desde Sincropool
        dic = {
            'IdCentroDeCosto': 0,
            'Aire': 0,
            'IdZona': 0,
            'Zona': 'DESCONOCIDA',
            'LetraAltura': '',
            'Direccion': u'Humboldt 2094, C1414CTX CABA, Argentina (9 \u201cA',
            'Siniestro': '467385',
            'EspacioEnBaul': 0,
            'Nombre': 'Noriega, Francisco',
            'FechaServicio': '2018-04-05',
            'Telefono': '115 881 5839',
            'TelefonoDeRef': '',
            'Nivel': 0,
            'IdCalle1': None,
            'PIN': '',
            'Observaciones': 'AR 1432 a Mendoza\\nFrancisco Noriega de su domicilio a Aeroparque.\\nDestinos: \\nAeropuerto Aeroparque (Aeroparque Internacional Jorge Newbery - Avenida Costanera Rafael Obligado, CABA)',
            'Prioridad': 1,
            'Longitud': 0,
            'ZonaHasta': None,
            'SolicitadoPor': u'Forte , Bernab\\xe9 ',
            'IdSucursal': 1,
            'IdProveedorWeb': 6,
            'IdCiudad': '0',
            'IdPersona': 0,
            'DireccionEnMapa': 1,
            'Depto': None,
            'HoraServicio': '1900-01-01 22:00:00',
            'Telefonista': 'Sincropool',
            'Altura': None,
            'Latitud': 0,
            'IdClienteCtaCte': 0,
            'IdCalle2': None,
            'SinGNC': 0,
            'IdZonaHasta': 0,
            'VIP': 0,
            'AceptaCtaCte': 1,
            'IdNodo': None
        }
        rta = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=False
        )
        self.assertTrue(rta is None)

        dic = {
            'IdCentroDeCosto': 0,
            'Aire': 0,
            'IdZona': 0,
            'Zona': 'DESCONOCIDA',
            'LetraAltura': '',
            'Direccion': 'Cacique Coliqueo 1462, B1685GBV El Palomar, Bueno',
            'Siniestro': '467513',
            'EspacioEnBaul': 0,
            'Nombre': u'DALL \u2018AGNESE, Carlos Alberto',
            'FechaServicio': '2018-04-06',
            'Telefono': '1138849062',
            'TelefonoDeRef': '',
            'Nivel': 0,
            'IdCalle1': None,
            'PIN': '',
            'Observaciones': u'AR 2718\nAuto de mi domicilio a Aeroparque para viajar a Paran\xe1, partido  Colon Vs V\xe9lez\nDestinos: \nAeropuerto Aeroparque (Aeroparque Internacional Jorge Newbery - Avenida Costanera Rafael Obligado, CABA)',
            'Prioridad': 4,
            'Longitud': 0,
            'ZonaHasta': None,
            'SolicitadoPor': u'DALL \u2018AGNESE, Carlos Alberto',
            'IdSucursal': 1,
            'IdProveedorWeb': 6,
            'IdCiudad': '0',
            'IdPersona': 0,
            'DireccionEnMapa': 1,
            'Depto': None,
            'HoraServicio': '1900-01-01 18:15:00',
            'Telefonista': 'Sincropool',
            'Altura': None,
            'Latitud': 0,
            'IdClienteCtaCte': 0,
            'IdCalle2': None,
            'SinGNC': 0,
            'IdZonaHasta': 0,
            'VIP': 0,
            'AceptaCtaCte': 1,
            'IdNodo': None
        }
        rta = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=False
        )
        self.assertTrue(rta is None)

        # Agrego el cliente 60
        dic = {}
        dic['IdCliente'] = 60
        agregar = self.servicio.cp.agregar('clientes_ctacte', **dic)
        self.assertTrue(agregar is True)

        # Agrego el centro de costo 2 del cliente 60
        dic = {}
        dic['IdCliente'] = 60
        dic['IdCentroDeCosto'] = 2
        dic['Nombre'] = 'Centro 2 del cliente 60'
        agregar = self.servicio.cp.agregar('centros_de_costos', **dic)
        self.assertTrue(agregar is True)

        # Agrego la persona
        dic = {}
        dic['IdCliente'] = 60
        dic['IdCentroDeCosto'] = 2
        dic['IdPersona'] = 19
        dic['Nombre'] = 'Persona 19'
        dic['PIN'] = '1515'
        dic['Habilitado'] = 1
        agregar = self.servicio.cp.agregar('centros_de_costos_personas', **dic)
        self.assertTrue(agregar is True)

        # Le paso un PIN sin un ID de persona
        dic = {
            'IdCentroDeCosto': 2,
            'Aire': 0,
            'IdZona': 0,
            'Zona': 'DESCONOCIDA',
            'LetraAltura': '',
            'Direccion': 'San Juan 1200',
            'Siniestro': '',
            'EspacioEnBaul': 0,
            'Nombre': u'Juan',
            'FechaServicio': '2018-04-09',
            'Telefono': '4110050',
            'TelefonoDeRef': '',
            'Nivel': 0,
            'IdCalle1': None,
            'PIN': '1515',
            'Observaciones': u'',
            'Prioridad': 3,
            'Longitud': 0,
            'ZonaHasta': None,
            'SolicitadoPor': '',
            'IdSucursal': 1,
            'IdCiudad': '324',
            'IdPersona': None,
            'DireccionEnMapa': 1,
            'Depto': None,
            'HoraServicio': '1900-01-01 16:32:00',
            'Telefonista': 'Tests',
            'Altura': None,
            'Latitud': 0,
            'IdClienteCtaCte': 60,
            'IdCalle2': None,
            'SinGNC': 0,
            'IdZonaHasta': 0,
            'VIP': 0,
            'AceptaCtaCte': 1,
            'IdNodo': None
        }
        rta = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=False
        )
        self.assertTrue(rta is None)

        # Prueba para obtener la calle con la latitud y la longitud
        dic = {
            'IdCentroDeCosto': 2,
            'Aire': 0,
            'IdZona': 0,
            'Zona': 'DESCONOCIDA',
            'LetraAltura': '',
            'Direccion': 'Manuel Ugarte 1674',
            'Siniestro': '',
            'EspacioEnBaul': 0,
            'Nombre': u'Juan',
            'FechaServicio': '2018-04-13',
            'Telefono': 'latlng',
            'TelefonoDeRef': '',
            'Nivel': 0,
            'IdCalle1': None,
            'PIN': '1515',
            'Observaciones': u'',
            'Prioridad': 4,
            'Longitud': -210435.16788,
            'ZonaHasta': None,
            'SolicitadoPor': '',
            'IdSucursal': 1,
            'IdCiudad': '0',
            'IdPersona': 19,
            'DireccionEnMapa': 1,
            'Depto': None,
            'HoraServicio': '1900-01-01 11:00:00',
            'Telefonista': 'Sincropool',
            'Altura': None,
            'Latitud': -124389.44496,
            'IdClienteCtaCte': 60,
            'IdCalle2': None,
            'SinGNC': 0,
            'IdZonaHasta': 0,
            'VIP': 0,
            'AceptaCtaCte': 1,
            'IdNodo': None
        }
        rta = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(rta is None)

        # Le paso AceptaCtaCte en 2 para que me agregue el filtro "TARJETA"
        # pero todavía no agrego el filtro "TARJETA"
        dic = {
            'IdCentroDeCosto': None,
            'Aire': 0,
            'IdZona': 0,
            'Zona': 'DESCONOCIDA',
            'LetraAltura': '',
            'Direccion': 'San Luis 931',
            'Siniestro': '',
            'EspacioEnBaul': 0,
            'Nombre': u'Juan',
            'FechaServicio': '2018-05-09',
            'Telefono': '153705967',
            'TelefonoDeRef': '',
            'Nivel': 0,
            'IdCalle1': None,
            'PIN': None,
            'Observaciones': u'',
            'Prioridad': 3,
            'Longitud': -210435.16788,
            'ZonaHasta': None,
            'SolicitadoPor': '',
            'IdSucursal': 1,
            'IdCiudad': '0',
            'IdPersona': None,
            'DireccionEnMapa': 1,
            'Depto': None,
            'HoraServicio': '1900-01-01 12:00:00',
            'Telefonista': 'Paco',
            'Altura': None,
            'Latitud': -124389.44496,
            'IdClienteCtaCte': None,
            'IdCalle2': None,
            'SinGNC': 0,
            'IdZonaHasta': 0,
            'VIP': 0,
            'AceptaCtaCte': 2,
            'IdNodo': None
        }
        rta = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(rta is None)

        # Agrego el filtro "TARJETA"
        dic = {}
        dic['IdFiltro'] = 81
        dic['Nombre'] = 'Con tarjeta'
        dic['Abreviatura'] = 'TARJETA'
        dic['IdTipo'] = 2
        agregar = self.servicio.cp.agregar('filtros', **dic)
        self.assertTrue(agregar is True)

        # Le paso AceptaCtaCte en 2 para que me agregue el filtro "TARJETA"
        dic = {
            'IdCentroDeCosto': None,
            'Aire': 0,
            'IdZona': 0,
            'Zona': 'DESCONOCIDA',
            'LetraAltura': '',
            'Direccion': 'San Luis 931',
            'Siniestro': '',
            'EspacioEnBaul': 0,
            'Nombre': u'Juan',
            'FechaServicio': '2018-05-09',
            'Telefono': '4110549',
            'TelefonoDeRef': '',
            'Nivel': 0,
            'IdCalle1': None,
            'PIN': None,
            'Observaciones': u'',
            'Prioridad': 3,
            'Longitud': -210435.16788,
            'ZonaHasta': None,
            'SolicitadoPor': '',
            'IdSucursal': 1,
            'IdCiudad': '0',
            'IdPersona': None,
            'DireccionEnMapa': 1,
            'Depto': None,
            'HoraServicio': '1900-01-01 12:30:00',
            'Telefonista': 'Paco',
            'Altura': None,
            'Latitud': -124389.44496,
            'IdClienteCtaCte': None,
            'IdCalle2': None,
            'SinGNC': 0,
            'IdZonaHasta': 0,
            'VIP': 0,
            'AceptaCtaCte': 2,
            'IdNodo': None
        }
        rta = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(rta is None)

        # Le paso AceptaCtaCte en 2 para que me agregue el filtro "TARJETA"
        # y también le paso otros filtros
        dic = {
            'IdCentroDeCosto': None,
            'Aire': 0,
            'IdZona': 0,
            'Zona': 'DESCONOCIDA',
            'LetraAltura': '',
            'Direccion': 'San Luis 931',
            'Siniestro': '',
            'EspacioEnBaul': 0,
            'Nombre': u'Juan',
            'FechaServicio': '2018-05-09',
            'Telefono': '4110549',
            'TelefonoDeRef': '',
            'Nivel': 0,
            'IdCalle1': None,
            'PIN': None,
            'Observaciones': u'',
            'Prioridad': 3,
            'Longitud': -210435.16788,
            'ZonaHasta': None,
            'SolicitadoPor': '',
            'IdSucursal': 1,
            'IdCiudad': '0',
            'IdPersona': None,
            'DireccionEnMapa': 1,
            'Depto': None,
            'HoraServicio': '1900-01-01 13:00:00',
            'Telefonista': 'Paco',
            'Altura': None,
            'Latitud': -124389.44496,
            'IdClienteCtaCte': None,
            'IdCalle2': None,
            'SinGNC': 0,
            'IdZonaHasta': 0,
            'VIP': 0,
            'AceptaCtaCte': 2,
            'IdNodo': None,
            'Filtros': [75, 93]
        }
        rta = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(rta is None)

        # Agrego un servicio de IVR
        dic = {
            'IdCentroDeCosto': None,
            'Aire': 0,
            'IdZona': 0,
            'Zona': 'DESCONOCIDA',
            'LetraAltura': '',
            'Direccion': 'San Luis 931',
            'Siniestro': '',
            'EspacioEnBaul': 0,
            'Nombre': u'Juan',
            'FechaServicio': '2019-02-26',
            'Telefono': '4110549',
            'TelefonoDeRef': '',
            'Nivel': 0,
            'IdCalle1': None,
            'PIN': None,
            'Observaciones': u'',
            'Prioridad': 3,
            'Longitud': -210435.16788,
            'ZonaHasta': None,
            'SolicitadoPor': '',
            'IdSucursal': 1,
            'IdCiudad': '0',
            'IdPersona': None,
            'DireccionEnMapa': 1,
            'Depto': None,
            'HoraServicio': '1900-01-01 11:00:00',
            'Telefonista': 'CpIvr',
            'Altura': None,
            'Latitud': -124389.44496,
            'IdClienteCtaCte': None,
            'IdCalle2': None,
            'SinGNC': 0,
            'IdZonaHasta': 0,
            'VIP': 0,
            'AceptaCtaCte': 0,
            'IdNodo': None,
            'Filtros': []
        }
        rta = self.servicio.grabar_servicio(
            dic, validar_fecha_hora=False, buscar_ciudad_por_posicion=True
        )
        self.assertTrue(rta is None)

        # Intento duplicar un servicio de IVR
        dic = {
            'IdCentroDeCosto': None,
            'Aire': 0,
            'IdZona': 0,
            'Zona': 'DESCONOCIDA',
            'LetraAltura': '',
            'Direccion': 'San Luis 931',
            'Siniestro': '',
            'EspacioEnBaul': 0,
            'Nombre': u'Juan',
            'FechaServicio': '2019-02-26',
            'Telefono': '4110549',
            'TelefonoDeRef': '',
            'Nivel': 0,
            'IdCalle1': None,
            'PIN': None,
            'Observaciones': u'',
            'Prioridad': 3,
            'Longitud': -210435.16788,
            'ZonaHasta': None,
            'SolicitadoPor': '',
            'IdSucursal': 1,
            'IdCiudad': '0',
            'IdPersona': None,
            'DireccionEnMapa': 1,
            'Depto': None,
            'HoraServicio': '1900-01-01 11:00:00',
            'Telefonista': 'CpIvr',
            'Altura': None,
            'Latitud': -124389.44496,
            'IdClienteCtaCte': None,
            'IdCalle2': None,
            'SinGNC': 0,
            'IdZonaHasta': 0,
            'VIP': 0,
            'AceptaCtaCte': 0,
            'IdNodo': None,
            'Filtros': []
        }
        self.assertRaises(Exception, self.servicio.grabar_servicio, dic)

    def test_cambiar_suc_serv(self):
        # agrego un nuevo servicio
        dic = {}
        dic['Telefono'] = '4555555'
        dic['FechaServicio'] = '2010-09-16'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['IdSucursal'] = 1
        dic['Automatico'] = 1
        dic['Prioridad'] = 1
        self.servicio.cp.agregar('actual', **dic)

        # con los datos correctos debe devolver True
        telefono = '4555555'
        fecha = '2010-09-16'
        hora = '1900-01-01 15:00:00'
        id_sucursal = 2
        r = self.servicio.cambiar_suc_serv(telefono, fecha, hora, id_sucursal)
        self.assertTrue(r is True)

        # con un teléfono que no existe debe devolver False
        telefono = '4102145879624001'
        fecha = '2010-09-16'
        hora = '1900-01-01 15:00:00'
        id_sucursal = 2
        r = self.servicio.cambiar_suc_serv(telefono, fecha, hora, id_sucursal)
        self.assertTrue(r is False)

        # con otra fecha debe devolver False
        telefono = '4555555'
        fecha = '2010-08-16'
        hora = '1900-01-01 15:00:00'
        id_sucursal = 2
        r = self.servicio.cambiar_suc_serv(telefono, fecha, hora, id_sucursal)
        self.assertTrue(r is False)

        # con otro hora debe devolver False
        telefono = '4555555'
        fecha = '2010-09-16'
        hora = '1900-01-01 14:00:00'
        id_sucursal = 2
        r = self.servicio.cambiar_suc_serv(telefono, fecha, hora, id_sucursal)
        self.assertTrue(r is False)

        # agrego un nuevo servicio en automático con prioridad 6
        dic = {}
        dic['Telefono'] = '4666666'
        dic['FechaServicio'] = '2010-09-15'
        dic['HoraServicio'] = '1900-01-01 14:00:00'
        dic['IdSucursal'] = 1
        dic['Automatico'] = 1
        dic['Prioridad'] = 6
        self.servicio.cp.agregar('actual', **dic)

        # no puedo editar el comando porque el servicio está en automático
        # y tiene prioridad 6
        telefono = '4666666'
        fecha = '2010-09-15'
        hora = '1900-01-01 14:00:00'
        id_sucursal = 3
        r = self.servicio.cambiar_suc_serv(telefono, fecha, hora, id_sucursal)
        self.assertTrue(r is False)

        # agrego un nuevo servicio en automático con prioridad 8
        dic = {}
        dic['Telefono'] = '4888888'
        dic['FechaServicio'] = '2010-09-14'
        dic['HoraServicio'] = '1900-01-01 13:00:00'
        dic['IdSucursal'] = 2
        dic['Automatico'] = 1
        dic['Prioridad'] = 8
        self.servicio.cp.agregar('actual', **dic)

        # no puedo editar el comando porque el servicio tiene prioridad 8
        # y está en automático
        telefono = '4888888'
        fecha = '2010-09-14'
        hora = '1900-01-01 13:00:00'
        id_sucursal = 3
        r = self.servicio.cambiar_suc_serv(telefono, fecha, hora, id_sucursal)
        self.assertTrue(r is False)

        # agrego un nuevo servicio manual
        dic = {}
        dic['Telefono'] = '4999999'
        dic['FechaServicio'] = '2010-09-13'
        dic['HoraServicio'] = '1900-01-01 12:00:00'
        dic['IdSucursal'] = 1
        dic['Automatico'] = 0
        dic['Prioridad'] = 3
        self.servicio.cp.agregar('actual', **dic)

        # servicio manual con prioridad 3
        telefono = '4999999'
        fecha = '2010-09-13'
        hora = '1900-01-01 12:00:00'
        id_sucursal = 2
        r = self.servicio.cambiar_suc_serv(telefono, fecha, hora, id_sucursal)
        self.assertTrue(r is True)

        # agrego un nuevo servicio manual con prioridad 1
        dic = {}
        dic['Telefono'] = '4909090'
        dic['FechaServicio'] = '2010-09-23'
        dic['HoraServicio'] = '1900-01-01 10:00:00'
        dic['IdSucursal'] = 1
        dic['Automatico'] = 0
        dic['Prioridad'] = 1
        self.servicio.cp.agregar('actual', **dic)

        # servicio manual con prioridad 1
        telefono = '4909090'
        fecha = '2010-09-23'
        hora = '1900-01-01 10:00:00'
        id_sucursal = 2
        r = self.servicio.cambiar_suc_serv(telefono, fecha, hora, id_sucursal)
        self.assertTrue(r is True)

        # servicio manual con teléfono que no exista
        telefono = '47851215001200'
        fecha = '2010-09-23'
        hora = '1900-01-01 10:00:00'
        id_sucursal = 2
        r = self.servicio.cambiar_suc_serv(telefono, fecha, hora, id_sucursal)
        self.assertTrue(r is False)

        # agrego un nuevo servicio manual con prioridad 0
        dic = {}
        dic['Telefono'] = '4100100'
        dic['FechaServicio'] = '2010-09-23'
        dic['HoraServicio'] = '1900-01-01 09:00:00'
        dic['IdSucursal'] = 1
        dic['Automatico'] = 0
        dic['Prioridad'] = 0
        self.servicio.cp.agregar('actual', **dic)

        # servicio manual con prioridad 0
        telefono = '4100100'
        fecha = '2010-09-23'
        hora = '1900-01-01 09:00:00'
        id_sucursal = 2
        r = self.servicio.cambiar_suc_serv(telefono, fecha, hora, id_sucursal)
        self.assertTrue(r is False)

    def test_duplicar_servicio(self):
        # agrego un servicio y luego lo duplico
        dic = {}
        dic['Telefono'] = '4000000'
        dic['FechaServicio'] = '2010-09-11'
        dic['HoraServicio'] = '1900-01-01 10:05:45'
        dic['Prioridad'] = 3
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4000000'
        fecha = '2010-09-11'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        # agrego un nuevo servicio con fecha faltando un segundo para
        # cambiar de día
        dic = {}
        dic['Telefono'] = '4222222'
        dic['FechaServicio'] = '2010-09-10'
        dic['HoraServicio'] = '1900-01-01 23:59:59'
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['PrioridadOriginal'] = 2
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4222222'
        fecha = '2010-09-10'
        hora = '1900-01-01 23:59:59'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        # intento duplicar un servicio que no existe por el telefono
        telefono = '0214523650'
        fecha = '2010-09-12'
        hora = '1900-01-01 10:00:00'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(
            r['Respuesta'] ==
            _(u'No se encontraron los datos del servicio a duplicar')
        )

        # intento duplicar un servicio que no existe por la hora
        telefono = '4000000'
        fecha = '2010-09-11'
        hora = '1900-01-01 09:00:00'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(
            r['Respuesta'] ==
            _(u'No se encontraron los datos del servicio a duplicar')
        )

        # intento duplicar un servicio que no existe por la fecha
        telefono = '4000000'
        fecha = '2010-09-09'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(
            r['Respuesta'] ==
            _(u'No se encontraron los datos del servicio a duplicar')
        )

        # duplico un servicio ya duplicado
        telefono = '4000000'
        fecha = '2010-09-11'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        # duplico de nuevo el mismo servicio
        telefono = '4000000'
        fecha = '2010-09-11'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        # le paso mal el formato de la hora
        telefono = '4000000'
        fecha = '2010-09-11'
        hora = '2010-09-11 09:00:00'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(
            r['Respuesta'] ==
            _(u'No se encontraron los datos del servicio a duplicar')
        )

        # no le paso la fecha y la hora
        telefono = '4000000'
        fecha = None
        hora = None
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(
            r['Respuesta'] ==
            _(u'No se encontraron los datos del servicio a duplicar')
        )

        # no le paso el teléfono
        telefono = None
        fecha = '2010-09-11'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(
            r['Respuesta'] ==
            _(u'No se encontraron los datos del servicio a duplicar')
        )

        # agrego un servicio y luego lo duplico 6 veces, tiene un máximo de 5
        dic = {}
        dic['Telefono'] = '1'
        dic['FechaServicio'] = '2010-09-11'
        dic['HoraServicio'] = '1900-01-01 10:05:45'
        dic['Prioridad'] = 5
        dic['IdSucursal'] = 1
        dic['PrioridadOriginal'] = 3
        self.servicio.cp.agregar('actual', **dic)

        telefono = '1'
        fecha = '2010-09-11'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        telefono = '1'
        fecha = '2010-09-11'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        telefono = '1'
        fecha = '2010-09-11'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        telefono = '1'
        fecha = '2010-09-11'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        telefono = '1'
        fecha = '2010-09-11'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        # aca ya no duplica
        telefono = '1'
        fecha = '2010-09-11'
        hora = '1900-01-01 10:05:45'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        # agrego un servicio de PTT
        dic = {}
        dic['Telefono'] = 'de PTT'
        dic['FechaServicio'] = '2015-06-16'
        dic['HoraServicio'] = '1900-01-01 14:00:00'
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['IdServicioPtt'] = 80
        self.servicio.cp.agregar('actual', **dic)

        # No se puede duplicar un servicio cargado con PTT
        telefono = 'de PTT'
        fecha = '2015-06-16'
        hora = '1900-01-01 14:00:00'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(
            r['Respuesta'] ==
            _(u'No se puede duplicar un servicio pedido por la aplicación mobile')
        )

        # Agrego un servicio con cta. cte.
        dic = {}
        dic['Telefono'] = 'con cta cte'
        dic['FechaServicio'] = '2015-06-16'
        dic['HoraServicio'] = '1900-01-01 16:00:00'
        dic['Prioridad'] = 3
        dic['IdSucursal'] = 1
        dic['AceptaCtaCte'] = 1
        dic['Nombre'] = 'El nombre'
        dic['Direccion'] = u'La dirección'
        dic['PrioridadOriginal'] = 3
        dic['IdServicio'] = 105
        self.servicio.cp.agregar('actual', **dic)

        # Agrego el cupón del servicio con cta. cte.
        dic = {}
        dic['IdServicio'] = 105
        dic['IdCliente'] = 5
        dic['IdCentroDeCosto'] = 12
        dic['IdPersona'] = 28
        dic['Pasajero'] = 'El nombre del pasajero'
        dic['DNIPasajero'] = 33228899
        dic['SolicitadoPor'] = u'Martín'
        dic['PIN'] = u'el_pin_canción'
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego el cliente 5
        a = {}
        a['IdCliente'] = 5
        agregar = self.servicio.cp.agregar('clientes_ctacte', **a)
        self.assertTrue(agregar is True)

        # Agrego el centro de costo 12 del cliente 5
        a = {}
        a['IdCliente'] = 5
        a['IdCentroDeCosto'] = 12
        a['Nombre'] = 'centro de costo 12 del cliente 5'
        agregar = self.servicio.cp.agregar('centros_de_costos', **a)
        self.assertTrue(agregar is True)

        # Le paso un servicio con cta. cte. para duplicar
        telefono = 'con cta cte'
        fecha = '2015-06-16'
        hora = '1900-01-01 16:00:00'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(
            r['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

        # Agrego un servicio con cta. cte. pero sin cupón
        dic = {}
        dic['Telefono'] = 'con cta cte 2'
        dic['FechaServicio'] = '2015-07-24'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Prioridad'] = 3
        dic['IdSucursal'] = 1
        dic['AceptaCtaCte'] = 1
        dic['Nombre'] = 'El nombre'
        dic['Direccion'] = u'La dirección'
        dic['PrioridadOriginal'] = 3
        dic['IdServicio'] = 106
        self.servicio.cp.agregar('actual', **dic)

        # Le paso un servicio con cta. cte. para duplicar pero no tiene un
        # cupón vinculado
        telefono = 'con cta cte 2'
        fecha = '2015-07-24'
        hora = '1900-01-01 15:00:00'
        r = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(
            r['Respuesta'] ==
            _(u'No se encontró ningún cupón con los datos dados')
        )

        # Agrego un registro en la tabla filtros
        dic = {}
        dic['IdFiltro'] = 78
        dic['Nombre'] = 'filtro 78'
        dic['Abreviatura'] = 'F78'
        dic['IdTipo'] = 2
        agregar = self.servicio.cp.agregar('filtros', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla filtros
        dic = {}
        dic['IdFiltro'] = 79
        dic['Nombre'] = 'filtro 79'
        dic['Abreviatura'] = 'F79'
        dic['IdTipo'] = 2
        agregar = self.servicio.cp.agregar('filtros', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla filtros_servicios
        dic = {}
        dic['IdFiltro'] = 78
        dic['IdServicio'] = 201
        agregar = self.servicio.cp.agregar('filtros_servicios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla filtros_servicios
        dic = {}
        dic['IdFiltro'] = 79
        dic['IdServicio'] = 201
        agregar = self.servicio.cp.agregar('filtros_servicios', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['Telefono'] = 'con_filtros'
        dic['FechaServicio'] = '2016-12-02'
        dic['HoraServicio'] = '1900-01-01 10:00:00'
        dic['Prioridad'] = 3
        dic['IdSucursal'] = 1
        dic['IdServicio'] = 201
        self.servicio.cp.agregar('actual', **dic)

        # Caso para duplicar los filtros de un servicio
        telefono = 'con_filtros'
        fecha = '2016-12-02'
        hora = '1900-01-01 10:00:00'
        rta = self.servicio.duplicar_servicio(telefono, fecha, hora)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Se duplicó el servicio de forma correcta')
        )

    def test_cargar_servicio(self):
        # intento visualizar un servicio que no existe
        telefono = '5787502151001'
        fecha = '2010-09-20'
        hora = '1900-01-01 10:00:00'
        modificar_servicio = False
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)
        self.assertTrue(r['Modificar'] is False)
        self.assertTrue(r['ServicioAutomatico'] is False)

        # agrego un servicio para hacer las pruebas
        # este es solo para visualizar
        dic = {}
        dic['Telefono'] = '4010101'
        dic['FechaServicio'] = '2010-09-20'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['IdCiudad'] = 324
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4010101'
        fecha = '2010-09-20'
        hora = '1900-01-01 15:00:00'
        modificar_servicio = False
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)
        self.assertTrue(r['Modificar'] is False)

        # agrego un servicio para editarlo, que no este en automático
        dic = {}
        dic['Telefono'] = '4020202'
        dic['FechaServicio'] = '2010-09-20'
        dic['HoraServicio'] = '1900-01-01 16:00:00'
        dic['Automatico'] = 0
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['IdCiudad'] = 324
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4020202'
        fecha = '2010-09-20'
        hora = '1900-01-01 16:00:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)
        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['ServicioAutomatico'] is False)

        # agrego un servicio para editarlo, que este en automático pero pausado
        dic = {}
        dic['Telefono'] = '996644'
        dic['FechaServicio'] = '2010-09-20'
        dic['HoraServicio'] = '1900-01-01 13:51:00'
        dic['Automatico'] = 1
        dic['Pausado'] = 1
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['IdCiudad'] = 324
        self.servicio.cp.agregar('actual', **dic)

        telefono = '996644'
        fecha = '2010-09-20'
        hora = '1900-01-01 13:51:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)

        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['ServicioAutomatico'] is True)

        # borro el servicio
        b = {'Telefono': ('==', '996644')}
        self.servicio.cp.borrar('actual', **b)

        # agrego un servicio en automático
        dic = {}
        dic['Telefono'] = '997711'
        dic['FechaServicio'] = '2010-09-20'
        dic['HoraServicio'] = '1900-01-01 12:25:00'
        dic['Automatico'] = 1
        dic['Pausado'] = 0
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['IdCiudad'] = 324
        self.servicio.cp.agregar('actual', **dic)

        telefono = '997711'
        fecha = '2010-09-20'
        hora = '1900-01-01 12:25:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)
        self.assertTrue(r['Modificar'] is False)
        self.assertTrue(r['ServicioAutomatico'] is True)

        # borro el servicio
        b = {'Telefono': ('==', '997711')}
        self.servicio.cp.borrar('actual', **b)

        # agrego un servicio para editarlo que tenga IdCalle1,
        # IdCiudad e IdCalle2
        dic = {}
        dic['Telefono'] = '4030303'
        dic['FechaServicio'] = '2010-09-20'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Automatico'] = 0
        dic['Pausado'] = 0
        dic['IdCiudad'] = 324
        dic['IdCalle1'] = 1
        dic['IdCalle2'] = 2
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4030303'
        fecha = '2010-09-20'
        hora = '1900-01-01 15:00:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)

        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['ServicioAutomatico'] is False)

        # agrego un servicio para editar con varios datos
        dic = {}
        dic['Telefono'] = '4040404'
        dic['FechaServicio'] = '2010-09-20'
        dic['HoraServicio'] = '1900-01-01 10:00:00'
        dic['Automatico'] = 0
        dic['Pausado'] = 0
        dic['IdCiudad'] = 324
        dic['IdCalle1'] = 1
        dic['IdCalle2'] = 2
        dic['Aire'] = 1
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4040404'
        fecha = '2010-09-20'
        hora = '1900-01-01 10:00:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)

        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['ServicioAutomatico'] is False)

        # agrego un servicio para editar con id de calles que no existen
        dic = {}
        dic['Telefono'] = '4050505'
        dic['FechaServicio'] = '2010-09-20'
        dic['HoraServicio'] = '1900-01-01 09:00:00'
        dic['Automatico'] = 0
        dic['Pausado'] = 0
        dic['IdCiudad'] = 1234
        dic['IdCalle1'] = 1234
        dic['IdCalle2'] = 2234
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4050505'
        fecha = '2010-09-20'
        hora = '1900-01-01 09:00:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)

        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['ServicioAutomatico'] is False)

        # agrego un servicio pausado para editarlo
        dic = {}
        dic['Telefono'] = '4060606'
        dic['FechaServicio'] = '2010-09-20'
        dic['HoraServicio'] = '1900-01-01 08:00:00'
        dic['Pausado'] = 1
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['IdCiudad'] = 324
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4060606'
        fecha = '2010-09-20'
        hora = '1900-01-01 08:00:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)

        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['ServicioAutomatico'] is False)

        # obtengo la fecha y hora actual
        fecha_hora_actual = self.dth.get_now()

        # agrego un servicio automático pero anticipado para dentro de 1 hora
        futuro = fecha_hora_actual + timedelta(hours=1)
        fecha_futuro = futuro.strftime("%Y-%m-%d")
        hora_futuro = '1900-01-01 ' + futuro.strftime("%H:%M:%S")

        dic = {}
        dic['Telefono'] = '999999'
        dic['FechaServicio'] = fecha_futuro
        dic['HoraServicio'] = hora_futuro
        dic['Pausado'] = 1
        dic['Automatico'] = 1
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['IdCiudad'] = 324
        self.servicio.cp.agregar('actual', **dic)

        telefono = '999999'
        fecha = fecha_futuro
        hora = hora_futuro
        modificar_servicio = True
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)
        self.assertTrue(r['Telefono'] == '999999')
        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['ServicioAutomatico'] is True)

        # agrego un servicio automático pero anticipado para dentro
        # de 40 minutos
        futuro = fecha_hora_actual + timedelta(minutes=40)
        fecha_futuro = futuro.strftime("%Y-%m-%d")
        hora_futuro = '1900-01-01 ' + futuro.strftime("%H:%M:%S")

        dic = {}
        dic['Telefono'] = '888888'
        dic['FechaServicio'] = fecha_futuro
        dic['HoraServicio'] = hora_futuro
        dic['Pausado'] = 1
        dic['Automatico'] = 1
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['IdCiudad'] = 324
        self.servicio.cp.agregar('actual', **dic)

        telefono = '888888'
        fecha = fecha_futuro
        hora = hora_futuro
        modificar_servicio = True
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)
        self.assertTrue(r['Telefono'] == '888888')
        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['ServicioAutomatico'] is True)

        # agrego un servicio automático pero anticipado para dentro de 2 minutos
        futuro = fecha_hora_actual + timedelta(minutes=2)
        fecha_futuro = futuro.strftime("%Y-%m-%d")
        hora_futuro = '1900-01-01 ' + futuro.strftime("%H:%M:%S")

        dic = {}
        dic['Telefono'] = '777777'
        dic['FechaServicio'] = fecha_futuro
        dic['HoraServicio'] = hora_futuro
        dic['Pausado'] = 0
        dic['Automatico'] = 1
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['IdCiudad'] = 324
        self.servicio.cp.agregar('actual', **dic)

        telefono = '777777'
        fecha = fecha_futuro
        hora = hora_futuro
        modificar_servicio = True
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)
        self.assertTrue(r['Modificar'] is False)
        self.assertTrue(r['Mensaje'] is not None)

        dic = {}
        dic['Telefono'] = '2568497'
        dic['FechaServicio'] = fecha_futuro
        dic['HoraServicio'] = hora_futuro
        dic['Pausado'] = 0
        dic['Automatico'] = 0
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        dic['Latitud'] = 3455.3
        dic['Longitud'] = 34254.2
        dic['IdCiudad'] = 324
        self.servicio.cp.agregar('actual', **dic)

        telefono = '2568497'
        fecha = fecha_futuro
        hora = hora_futuro
        modificar_servicio = False
        r = self.servicio.cargar_servicio(telefono, fecha, hora,
                                            modificar_servicio)
        self.assertTrue(r['Latitud'] == dic['Latitud'])
        self.assertTrue(r['Longitud'] == dic['Longitud'])

        # Agrego un servicio con cta. cte.
        dic = {}
        dic['Telefono'] = 'con cta. cte.'
        dic['FechaServicio'] = '2015-03-05'
        dic['HoraServicio'] = '1900-01-01 14:00:00'
        dic['Nombre'] = 'este nombre no va'
        dic['AceptaCtaCte'] = 1
        dic['IdServicio'] = 10253
        dic['EntreCalles'] = 'Entre calles de con cta. cte.'
        dic['EnviarSms'] = '153707070'
        dic['Siniestro'] = '45jK'
        dic['TiempoEstimado'] = 25
        dic['Importe'] = 81.5
        dic['IdCiudad'] = 324
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego el cupón del servicio con todos los datos
        dic = {}
        dic['IdServicio'] = 10253
        dic['IdCliente'] = 5
        dic['IdCentroDeCosto'] = 12
        dic['IdPersona'] = 28
        dic['Pasajero'] = 'El nombre del pasajero'
        dic['DNIPasajero'] = 33228899
        dic['IdCupon'] = 'a720'
        dic['Importe'] = 48.5
        dic['Km'] = 20
        dic['GastosVarios'] = 1.5
        dic['Espera'] = 2.5
        dic['EsperaEnMinutos'] = 15
        dic['Peajes'] = 3.5
        dic['Estacionamiento'] = 4.5
        dic['ServicioRadioTaxi'] = 5.5
        dic['IdCuponAuto'] = 20
        dic['SolicitadoPor'] = u'Martín'
        dic['PIN'] = u'el_pin_canción'
        dic['ConImporteFijo'] = 1
        dic['FechaHoraInicioViaje'] = '2015-10-16 17:54:10'
        dic['FechaHoraFinViaje'] = '2015-10-16 17:59:50'
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un cliente con cta. cte
        dic = {}
        dic['IdCliente'] = 5
        dic['RazonSocial'] = 'el cliente 5'
        agregar = self.servicio.cp.agregar('clientes_ctacte', **dic)
        self.assertTrue(agregar is True)

        # Agrego un centro de costo
        dic = {}
        dic['IdCliente'] = 5
        dic['IdCentroDeCosto'] = 12
        dic['Nombre'] = 'Centro de costo 12'
        agregar = self.servicio.cp.agregar('centros_de_costos', **dic)
        self.assertTrue(agregar is True)

        # Agrego una persona de un centro de costo
        dic = {}
        dic['IdCliente'] = 5
        dic['IdCentroDeCosto'] = 12
        dic['IdPersona'] = 28
        dic['Nombre'] = u'Persona 28 canción'
        agregar = self.servicio.cp.agregar('centros_de_costos_personas', **dic)
        self.assertTrue(agregar is True)

        # Cargo un servicio con cta. cte.
        telefono = 'con cta. cte.'
        fecha = '2015-03-05'
        hora = '1900-01-01 14:00:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(
            telefono, fecha, hora, modificar_servicio
        )
        self.assertTrue(len(r.keys()) == 65)
        self.assertTrue(r['Zona'] is None)
        self.assertTrue(r['Aire'] == 0)
        self.assertTrue(r['IdZona'] is None)
        self.assertTrue(r['ServicioAutomatico'] is False)
        self.assertTrue(r['IdCentroDeCosto'] == 12)
        self.assertTrue(r['LetraAltura'] is None)
        self.assertTrue(r['Direccion'] is None)
        self.assertTrue(r['EspacioEnBaul'] == 0)
        self.assertTrue(r['Nombre'] == u'El nombre del pasajero')
        self.assertTrue(r['FechaServicio'] == '05/03/2015')
        self.assertTrue(r['Telefono'] == 'con cta. cte.')
        self.assertTrue(r['Nivel'] is None)
        self.assertTrue(r['IdCalle1'] is None)
        self.assertTrue(r['IdCalle2'] is None)
        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['Observaciones'] is None)
        self.assertTrue(r['NombreClienteCtaCte'] == u'el cliente 5')
        self.assertTrue(r['TieneCentrosDeCostos'] is True)
        self.assertTrue(r['Prioridad'] is None)
        self.assertTrue(r['ZonaHasta'] is None)
        self.assertTrue(r['TelefonoDeRef'] is None)
        self.assertTrue(r['IdSucursal'] == 0)
        self.assertTrue(r['SeAlejaMov'] == 0)
        self.assertTrue(r['IdCiudad'] == 324)
        self.assertTrue(r['IdClienteCtaCte'] == 5)
        self.assertTrue(r['Depto'] is None)
        self.assertTrue(r['HoraServicio'] == '14:00:00')
        self.assertTrue(r['Automatico'] == 0)
        self.assertTrue(r['Altura'] is None)
        self.assertTrue(r['IdPersona'] == 28)
        self.assertTrue(r['IdMovilTmp'] is None)
        self.assertTrue(r['SinGNC'] == 0)
        self.assertTrue(r['DNIPasajero'] == 33228899)
        self.assertTrue(r['IdServicio'] == 10253)
        self.assertTrue(r['IdZonaHasta'] is None)
        self.assertTrue(r['Barrio'] == '')
        self.assertTrue(r['VIP'] == 0)
        self.assertTrue(r['IdMovil'] is None)
        self.assertTrue(r['AceptaCtaCte'] == 1)
        self.assertTrue(r['IdNodo'] is None)
        self.assertTrue(r['NombreCentroDeCosto'] == u'Centro de costo 12')
        self.assertTrue(r['NumCupon'] == 'a720')
        self.assertTrue(r['Importe'] == 48.5)
        self.assertTrue(r['Km'] == 20)
        self.assertTrue(r['GastosVarios'] == 1.5)
        self.assertTrue(r['Espera'] == 15)
        self.assertTrue(r['Peajes'] == 3.5)
        self.assertTrue(r['Estacionamiento'] == 4.5)
        self.assertTrue(r['ServicioRadioTaxi'] == 5.5)
        self.assertTrue(r['Destinos'] == [])
        self.assertTrue(r['NombrePersona'] == u'Persona 28 canción')
        self.assertTrue(r['IdCuponAuto'] == 20)
        self.assertTrue(r['SolicitadoPor'] == u'Martín')
        self.assertTrue(r['PIN'] == u'el_pin_canción')
        self.assertTrue(r['EntreCalles'] == 'Entre calles de con cta. cte.')
        self.assertTrue(r['ObservacionInterna'] == '')
        self.assertTrue(r['Filtros'] == [])
        self.assertTrue(r['ConImporteFijo'] == 1)
        self.assertTrue(r['FechaHoraInicioViaje'] == '16/10/2015 17:54:10')
        self.assertTrue(r['FechaHoraFinViaje'] == '16/10/2015 17:59:50')
        self.assertTrue(r['EnviarSms'] == '153707070')
        self.assertTrue(r['Siniestro'] == '45jK')
        self.assertTrue(r['TiempoEstimado'] == 25)
        self.assertTrue(r['ImporteServicio'] == 81.5)
        self.assertTrue(r['NombreCiudad'] == 'Rosario')

        # Agrego un servicio sin cta. cte.
        dic = {}
        dic['Telefono'] = 'sin cta. cte.'
        dic['FechaServicio'] = '2015-03-05'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Nombre'] = 'este nombre si va'
        dic['AceptaCtaCte'] = 0
        dic['IdServicio'] = 10254
        dic['ObservacionInterna'] = 'con una observacion interna'
        dic['IdCiudad'] = 324
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Cargo un servicio sin cta. cte.
        telefono = 'sin cta. cte.'
        fecha = '2015-03-05'
        hora = '1900-01-01 15:00:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(
            telefono, fecha, hora, modificar_servicio
        )
        self.assertTrue(len(r.keys()) == 65)
        self.assertTrue(r['Zona'] is None)
        self.assertTrue(r['Aire'] == 0)
        self.assertTrue(r['IdZona'] is None)
        self.assertTrue(r['ServicioAutomatico'] is False)
        self.assertTrue(r['IdCentroDeCosto'] is None)
        self.assertTrue(r['LetraAltura'] is None)
        self.assertTrue(r['Direccion'] is None)
        self.assertTrue(r['EspacioEnBaul'] == 0)
        self.assertTrue(r['Nombre'] == u'este nombre si va')
        self.assertTrue(r['FechaServicio'] == '05/03/2015')
        self.assertTrue(r['Telefono'] == 'sin cta. cte.')
        self.assertTrue(r['Nivel'] is None)
        self.assertTrue(r['IdCalle1'] is None)
        self.assertTrue(r['IdCalle2'] is None)
        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['Observaciones'] is None)
        self.assertTrue(r['NombreClienteCtaCte'] == '')
        self.assertTrue(r['TieneCentrosDeCostos'] is None)
        self.assertTrue(r['Prioridad'] is None)
        self.assertTrue(r['ZonaHasta'] is None)
        self.assertTrue(r['TelefonoDeRef'] is None)
        self.assertTrue(r['IdSucursal'] == 0)
        self.assertTrue(r['SeAlejaMov'] == 0)
        self.assertTrue(r['IdCiudad'] == 324)
        self.assertTrue(r['IdClienteCtaCte'] is None)
        self.assertTrue(r['Depto'] is None)
        self.assertTrue(r['HoraServicio'] == '15:00:00')
        self.assertTrue(r['Automatico'] == 0)
        self.assertTrue(r['Altura'] is None)
        self.assertTrue(r['IdPersona'] is None)
        self.assertTrue(r['IdMovilTmp'] is None)
        self.assertTrue(r['SinGNC'] == 0)
        self.assertTrue(r['DNIPasajero'] is None)
        self.assertTrue(r['IdServicio'] == 10254)
        self.assertTrue(r['IdZonaHasta'] is None)
        self.assertTrue(r['Barrio'] == '')
        self.assertTrue(r['VIP'] == 0)
        self.assertTrue(r['IdMovil'] is None)
        self.assertTrue(r['AceptaCtaCte'] == 0)
        self.assertTrue(r['IdNodo'] is None)
        self.assertTrue(r['NombreCentroDeCosto'] == '')
        self.assertTrue(r['NumCupon'] is None)
        self.assertTrue(r['Importe'] is None)
        self.assertTrue(r['Km'] is None)
        self.assertTrue(r['GastosVarios'] is None)
        self.assertTrue(r['Espera'] is None)
        self.assertTrue(r['Peajes'] is None)
        self.assertTrue(r['Estacionamiento'] is None)
        self.assertTrue(r['ServicioRadioTaxi'] is None)
        self.assertTrue(r['Destinos'] is None)
        self.assertTrue(r['NombrePersona'] == '')
        self.assertTrue(r['IdCuponAuto'] is None)
        self.assertTrue(r['SolicitadoPor'] == '')
        self.assertTrue(r['PIN'] == '')
        self.assertTrue(r['EntreCalles'] == '')
        self.assertTrue(
            r['ObservacionInterna'] == 'con una observacion interna'
        )
        self.assertTrue(r['Filtros'] == [])
        self.assertTrue(r['ConImporteFijo'] == 0)
        self.assertTrue(r['FechaHoraInicioViaje'] is None)
        self.assertTrue(r['FechaHoraFinViaje'] is None)
        self.assertTrue(r['EnviarSms'] == '')
        self.assertTrue(r['Siniestro'] == '')
        self.assertTrue(r['TiempoEstimado'] is None)
        self.assertTrue(r['ImporteServicio'] == 0)
        self.assertTrue(r['NombreCiudad'] == 'Rosario')

        # Agrego un servicio con cta. cte.
        dic = {}
        dic['Telefono'] = 'con cta. cte. 2'
        dic['FechaServicio'] = '2015-03-06'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Nombre'] = 'este nombre no va'
        dic['AceptaCtaCte'] = 1
        dic['IdServicio'] = 10255
        dic['IdCiudad'] = 324
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego el cupón del servicio
        dic = {}
        dic['IdServicio'] = 10255
        dic['IdCliente'] = 6
        dic['IdCentroDeCosto'] = 13
        dic['IdPersona'] = 29
        dic['Pasajero'] = 'El nombre del pasajero 2'
        dic['DNIPasajero'] = 33228800
        dic['IdCuponAuto'] = 21
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Cargo un servicio con cta. cte. pero no está cargado su cliente y
        # su centro de costo
        telefono = 'con cta. cte. 2'
        fecha = '2015-03-06'
        hora = '1900-01-01 15:00:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(
            telefono, fecha, hora, modificar_servicio
        )
        self.assertTrue(len(r.keys()) == 65)
        self.assertTrue(r['Zona'] is None)
        self.assertTrue(r['Aire'] == 0)
        self.assertTrue(r['IdZona'] is None)
        self.assertTrue(r['ServicioAutomatico'] is False)
        self.assertTrue(r['IdCentroDeCosto'] == 13)
        self.assertTrue(r['LetraAltura'] is None)
        self.assertTrue(r['Direccion'] is None)
        self.assertTrue(r['EspacioEnBaul'] == 0)
        self.assertTrue(r['Nombre'] == u'El nombre del pasajero 2')
        self.assertTrue(r['FechaServicio'] == '06/03/2015')
        self.assertTrue(r['Telefono'] == 'con cta. cte. 2')
        self.assertTrue(r['Nivel'] is None)
        self.assertTrue(r['IdCalle1'] is None)
        self.assertTrue(r['IdCalle2'] is None)
        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['Observaciones'] is None)
        self.assertTrue(r['NombreClienteCtaCte'] == '')
        self.assertTrue(r['TieneCentrosDeCostos'] is None)
        self.assertTrue(r['Prioridad'] is None)
        self.assertTrue(r['ZonaHasta'] is None)
        self.assertTrue(r['TelefonoDeRef'] is None)
        self.assertTrue(r['IdSucursal'] == 0)
        self.assertTrue(r['SeAlejaMov'] == 0)
        self.assertTrue(r['IdCiudad'] == 324)
        self.assertTrue(r['IdClienteCtaCte'] == 6)
        self.assertTrue(r['Depto'] is None)
        self.assertTrue(r['HoraServicio'] == '15:00:00')
        self.assertTrue(r['Automatico'] == 0)
        self.assertTrue(r['Altura'] is None)
        self.assertTrue(r['IdPersona'] == 29)
        self.assertTrue(r['IdMovilTmp'] is None)
        self.assertTrue(r['SinGNC'] == 0)
        self.assertTrue(r['DNIPasajero'] == 33228800)
        self.assertTrue(r['IdServicio'] == 10255)
        self.assertTrue(r['IdZonaHasta'] is None)
        self.assertTrue(r['Barrio'] == '')
        self.assertTrue(r['VIP'] == 0)
        self.assertTrue(r['IdMovil'] is None)
        self.assertTrue(r['AceptaCtaCte'] == 1)
        self.assertTrue(r['IdNodo'] is None)
        self.assertTrue(r['NombreCentroDeCosto'] == '')
        self.assertTrue(r['NumCupon'] == '')
        self.assertTrue(r['Importe'] == 0.0)
        self.assertTrue(r['Km'] == 0.0)
        self.assertTrue(r['GastosVarios'] == 0.0)
        self.assertTrue(r['Espera'] == 0.0)
        self.assertTrue(r['Peajes'] == 0.0)
        self.assertTrue(r['Estacionamiento'] == 0.0)
        self.assertTrue(r['ServicioRadioTaxi'] == 0.0)
        self.assertTrue(r['Destinos'] == [])
        self.assertTrue(r['NombrePersona'] == '')
        self.assertTrue(r['IdCuponAuto'] == 21)
        self.assertTrue(r['SolicitadoPor'] == '')
        self.assertTrue(r['PIN'] == '')
        self.assertTrue(r['EntreCalles'] == '')
        self.assertTrue(r['ObservacionInterna'] == '')
        self.assertTrue(r['Filtros'] == [])
        self.assertTrue(r['ConImporteFijo'] == 0)
        self.assertTrue(r['FechaHoraInicioViaje'] is None)
        self.assertTrue(r['FechaHoraFinViaje'] is None)
        self.assertTrue(r['EnviarSms'] == '')
        self.assertTrue(r['Siniestro'] == '')
        self.assertTrue(r['TiempoEstimado'] is None)
        self.assertTrue(r['ImporteServicio'] == 0)
        self.assertTrue(r['NombreCiudad'] == 'Rosario')

        # Agrego un servicio en la tabla historico
        dic = {}
        dic['Telefono'] = 'es_historico'
        dic['FechaServicio'] = '2015-03-01'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Nombre'] = 'Juan'
        dic['IdServicio'] = 26302
        dic['EntreCalles'] = 'Entre calles de es_historico'
        agregar = self.servicio.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Pido un servicio que está en la tabla historico
        telefono = 'es_historico'
        fecha = '2015-03-01'
        hora = '1900-01-01 15:00:00'
        modificar_servicio = True
        buscar_tambien_en_historico = True
        r = self.servicio.cargar_servicio(
            telefono, fecha, hora,
            modificar_servicio, buscar_tambien_en_historico
        )
        self.assertTrue(len(r.keys()) == 65)
        self.assertTrue(r['Zona'] is None)
        self.assertTrue(r['Aire'] == 0)
        self.assertTrue(r['IdZona'] is None)
        self.assertTrue(r['ServicioAutomatico'] is False)
        self.assertTrue(r['IdCentroDeCosto'] is None)
        self.assertTrue(r['LetraAltura'] is None)
        self.assertTrue(r['Direccion'] is None)
        self.assertTrue(r['EspacioEnBaul'] == 0)
        self.assertTrue(r['Nombre'] == 'Juan')
        self.assertTrue(r['FechaServicio'] == '01/03/2015')
        self.assertTrue(r['Telefono'] == 'es_historico')
        self.assertTrue(r['Nivel'] is None)
        self.assertTrue(r['IdCalle1'] is None)
        self.assertTrue(r['IdCalle2'] is None)
        self.assertTrue(r['Modificar'] is True)
        self.assertTrue(r['Observaciones'] is None)
        self.assertTrue(r['NombreClienteCtaCte'] == '')
        self.assertTrue(r['TieneCentrosDeCostos'] is None)
        self.assertTrue(r['Prioridad'] is None)
        self.assertTrue(r['ZonaHasta'] is None)
        self.assertTrue(r['TelefonoDeRef'] is None)
        self.assertTrue(r['IdSucursal'] == 0)
        self.assertTrue(r['SeAlejaMov'] == 0)
        self.assertTrue(r['IdCiudad'] is None)
        self.assertTrue(r['IdClienteCtaCte'] is None)
        self.assertTrue(r['Depto'] is None)
        self.assertTrue(r['HoraServicio'] == '15:00:00')
        self.assertTrue(r['Automatico'] == 0)
        self.assertTrue(r['Altura'] is None)
        self.assertTrue(r['IdPersona'] is None)
        self.assertTrue(r['IdMovilTmp'] is None)
        self.assertTrue(r['SinGNC'] == 0)
        self.assertTrue(r['DNIPasajero'] is None)
        self.assertTrue(r['IdServicio'] == 26302)
        self.assertTrue(r['IdZonaHasta'] is None)
        self.assertTrue(r['Barrio'] == '')
        self.assertTrue(r['VIP'] == 0)
        self.assertTrue(r['IdMovil'] is None)
        self.assertTrue(r['AceptaCtaCte'] == 0)
        self.assertTrue(r['IdNodo'] is None)
        self.assertTrue(r['NombreCentroDeCosto'] == '')
        self.assertTrue(r['NumCupon'] is None)
        self.assertTrue(r['Importe'] is None)
        self.assertTrue(r['Km'] is None)
        self.assertTrue(r['GastosVarios'] is None)
        self.assertTrue(r['Espera'] is None)
        self.assertTrue(r['Peajes'] is None)
        self.assertTrue(r['Estacionamiento'] is None)
        self.assertTrue(r['ServicioRadioTaxi'] is None)
        self.assertTrue(r['Destinos'] is None)
        self.assertTrue(r['NombrePersona'] == '')
        self.assertTrue(r['IdCuponAuto'] is None)
        self.assertTrue(r['SolicitadoPor'] == '')
        self.assertTrue(r['PIN'] == '')
        self.assertTrue(r['EntreCalles'] == u'Entre calles de es_historico')
        self.assertTrue(r['ObservacionInterna'] == '')
        self.assertTrue(r['Filtros'] == [])
        self.assertTrue(r['ConImporteFijo'] == 0)
        self.assertTrue(r['FechaHoraInicioViaje'] is None)
        self.assertTrue(r['FechaHoraFinViaje'] is None)
        self.assertTrue(r['EnviarSms'] == '')
        self.assertTrue(r['Siniestro'] == '')
        self.assertTrue(r['TiempoEstimado'] is None)
        self.assertTrue(r['ImporteServicio'] == 0)

        # Pido un servicio que existe
        telefono = 'no_existe'
        fecha = '2015-07-06'
        hora = '1900-01-01 11:00:00'
        modificar_servicio = True
        buscar_tambien_en_historico = True
        r = self.servicio.cargar_servicio(
            telefono, fecha, hora,
            modificar_servicio, buscar_tambien_en_historico
        )
        self.assertTrue(r['Modificar'] is False)
        self.assertTrue(r['ServicioAutomatico'] is False)
        self.assertTrue(
            r['Mensaje'] == _(u'No existe dicho servicio ni en historico')
        )

        # Pido un servicio que existe pero solo lo busco en la tabla actual
        telefono = 'no_existe2'
        fecha = '2015-03-01'
        hora = '1900-01-01 15:00:00'
        modificar_servicio = True
        r = self.servicio.cargar_servicio(
            telefono, fecha, hora,
            modificar_servicio
        )
        self.assertTrue(r['Modificar'] is False)
        self.assertTrue(r['ServicioAutomatico'] is False)
        self.assertTrue(r['Mensaje'] == _(u'No existe dicho servicio'))

    def test_detener_despacho_servicio(self):
        # agrego un servicio para luego detenerlo
        dic = {}
        dic['Telefono'] = '4121212'
        dic['FechaServicio'] = '2010-09-21'
        dic['HoraServicio'] = '1900-01-01 09:00:00'
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4121212'
        fecha = '2010-09-21'
        hora = '1900-01-01 09:00:00'
        r = self.servicio.detener_despacho_servicio(telefono, fecha, hora)
        self.assertTrue(r is True)

        # intento detener un servicio que no existe
        telefono = '02140012360'
        fecha = '2010-09-18'
        hora = '1900-01-01 12:00:00'
        r = self.servicio.detener_despacho_servicio(telefono, fecha, hora)
        self.assertTrue(r is False)

        # intento detener un servicio que no existe por la hora
        telefono = '4121212'
        fecha = '2010-09-21'
        hora = '1900-01-01 10:00:00'
        r = self.servicio.detener_despacho_servicio(telefono, fecha, hora)
        self.assertTrue(r is False)

    def test_reanudar_despacho_servicio(self):
        # agrego un servicio detenido para luego reanudarlo
        dic = {}
        dic['Telefono'] = '4070707'
        dic['FechaServicio'] = '2010-09-21'
        dic['HoraServicio'] = '1900-01-01 13:00:00'
        dic['Automatico'] = 0
        dic['Comando'] = '01'
        dic['Pausado'] = 1
        dic['Prioridad'] = 1
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4070707'
        fecha = '2010-09-21'
        hora = '1900-01-01 13:00:00'
        r = self.servicio.reanudar_despacho_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)

        # intento reanudar un servicio que no existe
        telefono = '02140012360'
        fecha = '2010-09-18'
        hora = '1900-01-01 12:00:00'
        r = self.servicio.reanudar_despacho_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)

        # no le paso parámetros
        telefono = None
        fecha = None
        hora = None
        r = self.servicio.reanudar_despacho_servicio(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)

    def test_anular_o_reactivar_servicio(self):
        fecha_actual = self.dth.get_now().strftime("%Y-%m-%d")
        hora_actual = '1900-01-01 ' + self.dth.get_now().strftime("%H:%M:%S")
        fecha_hora_actual = self.dth.get_now().strftime("%Y-%m-%d %H:%M:%S")
        usuario = 'TestUser'
        # intento reanudar un servicio inexistente

        telefono = '407070707079684571354628'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        r = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(r['Estado'] is False)

        # agrego un servicio cancelado para luego reanudarlo

        dic = {}
        dic['Telefono'] = '40707070707'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 1
        dic['Pausado'] = 0
        dic['Prioridad'] = 5
        dic['PrioridadOriginal'] = 2
        dic['FechaHoraIniSat'] = '2010-09-28 12:58:51'
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        telefono = '40707070707'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        r = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(r['Estado'] is True)

        # agrego un servicio con prioridad baja y en automatico para luego
        # cancelarlo enviandole el comando 02

        dic = {}
        dic['Telefono'] = '40505050505'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 1
        dic['Pausado'] = 0
        dic['Prioridad'] = 2
        dic['PrioridadOriginal'] = 2
        dic['FechaHoraIniSat'] = fecha_hora_actual
        dic['IdSucursal'] = 1
        dic['Telefonista'] = 'PTT'
        self.servicio.cp.agregar('actual', **dic)

        telefono = '40505050505'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        r = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(r['Estado'] is True)

        # agrego un servicio con prioridad baja y en manual
        # para luego cancelarlo

        dic = {}
        dic['Telefono'] = '40606060606'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 0
        dic['Pausado'] = 0
        dic['Prioridad'] = 2
        dic['PrioridadOriginal'] = 2
        dic['FechaHoraIniSat'] = '2010-09-28 12:58:51'
        dic['IdSucursal'] = 1
        dic['Telefonista'] = 'PTT'
        self.servicio.cp.agregar('actual', **dic)

        telefono = '40606060606'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        r = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(r['Estado'] is True)

        # agrego un servicio con prioridad baja y en manual para luego
        # cancelarlo y le pasamos un IdMovil inexistente

        dic = {}
        dic['Telefono'] = '404040404040'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 0
        dic['Pausado'] = 0
        dic['Prioridad'] = 2
        dic['PrioridadOriginal'] = 2
        dic['IdMovil'] = '898'
        dic['FechaHoraIniSat'] = '2010-09-28 12:58:51'
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        telefono = '404040404040'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        r = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(r['Estado'] is False)

        #Ahora vamos a agregar un movil para poder trabajar con el
        # y luego agregamos un servicio para poder anularlo

        dic3 = {}
        dic3['Patente'] = 'ABC 123'
        dic3['IdMovil'] = '777'
        dic3['IdZonaActual'] = 1
        dic3['Sucursal'] = 1
        dic3['EstaEnServicio'] = 0
        dic3['FechaHoraDeIngreso'] = self.dth.get_now()
        dic3['HabilitadoDesde'] = None
        dic3['IdFlota'] = 16
        dic3['IdEstacion'] = 17
        self.servicio.cp.agregar('moviles', **dic3)

        dic = {}
        dic['Telefono'] = '4030303030'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 0
        dic['Pausado'] = 0
        dic['Prioridad'] = 2
        dic['PrioridadOriginal'] = 2
        dic['IdMovil'] = '777'
        dic['FechaHoraIniSat'] = '2010-09-28 12:58:51'
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        telefono = '4030303030'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        r = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(r['Estado'] is True)

        # Finalmente vamos a agregar un movil con DNI y session para poder
        # trabajar con el y asi agregar el comando de envio del GPRS. Luego
        # agregamos un servicio para poder anularlo

        dic3 = {}
        dic3['Patente'] = 'ABC 456'
        dic3['IdMovil'] = '778'
        dic3['IdZonaActual'] = 1
        dic3['Sucursal'] = 1
        dic3['DNI'] = '12345678'
        dic3['idSesion'] = '{session}'
        dic3['EstaEnServicio'] = 0
        dic3['FechaHoraDeIngreso'] = self.dth.get_now()
        dic3['HabilitadoDesde'] = None
        dic3['IdFlota'] = 16
        dic3['IdEstacion'] = 17
        self.servicio.cp.agregar('moviles', **dic3)

        dic = {}
        dic['Telefono'] = '402020202'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 0
        dic['Pausado'] = 0
        dic['Prioridad'] = 2
        dic['PrioridadOriginal'] = 2
        dic['IdMovil'] = '778'
        dic['FechaHoraIniSat'] = '2010-09-28 12:58:51'
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        # le paso la sucursal 1
        telefono = '402020202'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        r = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(r['Estado'] is True)

        # obtengo la fecha y hora actual
        fecha_hora_actual = self.dth.get_now()

        # agrego un servicio automático pero anticipado para dentro de 1 hora
        futuro = fecha_hora_actual + timedelta(hours=1)
        fecha_futuro = futuro.strftime("%Y-%m-%d")
        hora_futuro = '1900-01-01 ' + futuro.strftime("%H:%M:%S")

        dic = {}
        dic['Telefono'] = '999999'
        dic['FechaServicio'] = fecha_futuro
        dic['HoraServicio'] = hora_futuro
        dic['Automatico'] = 1
        dic['Pausado'] = 0
        dic['Prioridad'] = 1
        dic['FechaHoraIniSat'] = fecha_hora_actual
        #dic['FechaHoraIniSat'] = None
        dic['IdSucursal'] = 1
        a = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(a is True)

        # le paso el servicio agregado
        telefono = '999999'
        fecha = fecha_futuro
        hora = hora_futuro
        id_sucursal = 1
        r = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(r['Estado'] is True)

        ### CASO PARA UN SERVICIO CON CTA. CTE. ###

        # Agrego un servicio con cta. cte.
        dic = {}
        dic['Telefono'] = 'con cta. cte.'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Prioridad'] = 3
        dic['AceptaCtaCte'] = 1
        dic['IdSucursal'] = 1
        dic['IdServicio'] = 51
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un cupón
        dic = {}
        dic['IdCuponAuto'] = 86
        dic['IdServicio'] = 51
        dic['ServicioCancelado'] = 0
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un usuario
        a = {}
        a['Usuario'] = 'juan'
        a['Permisos'] = 'A'
        agregar = self.servicio.cp.agregar('usuarios', **a)
        self.assertTrue(agregar is True)

        # Le paso el servicio con cta. cte.
        telefono = 'con cta. cte.'
        fecha = fecha_actual
        hora = hora_actual
        usuario = 'juan'
        id_sucursal = 1
        rta = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(rta['Estado'] is True)

        # Agrego un servicio con cta. cte. cancelado para reactivarlo
        dic = {}
        dic['Telefono'] = 'con cta. cte. 2'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Prioridad'] = 5
        dic['AceptaCtaCte'] = 1
        dic['IdSucursal'] = 1
        dic['IdServicio'] = 52
        dic['PrioridadOriginal'] = 3
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un cupón
        dic = {}
        dic['IdCuponAuto'] = 87
        dic['IdServicio'] = 52
        dic['ServicioCancelado'] = 1
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Le paso el servicio con cta. cte. cancelado
        telefono = 'con cta. cte. 2'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        rta = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(rta['Estado'] is True)

        # Agrego un servicio con cta. cte.
        dic = {}
        dic['Telefono'] = 'con cta. cte. 3'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Prioridad'] = 3
        dic['AceptaCtaCte'] = 1
        dic['IdSucursal'] = 1
        dic['IdServicio'] = 53
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Le paso el servicio con cta. cte. que no tiene un cupón
        telefono = 'con cta. cte. 3'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        rta = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(rta['Estado'] is True)

        ########## CANCELACION CON MOTIVO E IMPORTE ##########################

        # Se pone la fecha y hora actual para no tener problemas con la demora
        # por si pasan muchos días, tira un WARNING mysql

        # Agrego un servicio con cuenta corriente, motivo e importe
        dic = {}
        dic['Telefono'] = 'con cta. cte. 4'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Prioridad'] = 3
        dic['AceptaCtaCte'] = 1
        dic['IdSucursal'] = 1
        dic['IdServicio'] = 54
        dic['PrioridadOriginal'] = 3
        dic['Siniestro'] = u'el siniestro de 88 canción'
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un cupón
        dic = {}
        dic['IdCuponAuto'] = 88
        dic['IdServicio'] = 54
        dic['IdCliente'] = 30
        dic['Siniestro'] = u'el siniestro de 88 canción'
        dic['PIN'] = u'El pin de Martín'
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego mails al cliente 30
        dic = {}
        dic['IdCliente'] = 30
        dic['Mail'] = 'jfr@airtrack.com.ar'
        dic['Nombre'] = u'Cliente 30'
        dic['Tipo'] = 'O'
        agregar = self.servicio.cp.agregar('clientes_mails', **dic)
        self.assertTrue(agregar is True)

        # Le paso el servicio con cuenta corriente, motivo e importe
        telefono = 'con cta. cte. 4'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        observacion = ''
        motivo = u'El motivo de la cancelación'
        importe = 50.5
        test = True
        rta = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal,
            observacion, motivo, importe, test
        )
        self.assertTrue(rta['Estado'] is True)

        # Agrego un servicio de PTT
        dic = {}
        dic['Telefono'] = 'de APP'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 1
        dic['Pausado'] = 0
        dic['Prioridad'] = 3
        dic['IdSucursal'] = 1
        dic['IdServicioPtt'] = 58
        self.servicio.cp.agregar('actual', **dic)

        # Le paso un servicio de PTT, se puede cancelar porque esta función
        # la utiliza at.tareas cuando llega un cancelado desde el celular
        # No se pueden cancelar los servicios de PTT desde el CallProcessor
        # los filtra la UI
        # Ahora se valida la cancelación desde la función pero si el usuario
        # es PTT se permite cancelar porque es una cancelación del usuario
        telefono = 'de APP'
        fecha = fecha_actual
        hora = hora_actual
        usuario = 'APP'
        id_sucursal = 1
        rta = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal,
        )
        self.assertTrue(rta['Estado'] is True)

        # Agrego un servicio cancelado
        dic = {}
        dic['Telefono'] = '4253030'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Prioridad'] = 5
        self.servicio.cp.agregar('actual', **dic)

        # Le paso un servicio cancelado pero no lo quiero reactivar
        telefono = '4253030'
        fecha = fecha_actual
        hora = hora_actual
        id_sucursal = 1
        solo_cancelar = True
        r = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal,
            solo_cancelar=solo_cancelar
        )
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(r['Mensaje'] == _('El servicio ya estaba cancelado.'))

        # Agrego un registro en la tabla "actual"
        dic = {}
        dic['Telefono'] = '4919191'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Prioridad'] = 3
        dic['IdSucursal'] = 1
        dic['IdServicioPtt'] = 91
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Intento cancelar un servicio generado por una APK
        telefono = '4919191'
        fecha = fecha_actual
        hora = hora_actual
        usuario = 'paco'
        id_sucursal = 1
        rta = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Mensaje'] ==
            _(u'No se puede cancelar un servicio pedido por la aplicación móvil.')
        )

        # Agrego un registro en la tabla "actual"
        dic = {}
        dic['Telefono'] = '4929292'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Prioridad'] = 3
        dic['IdSucursal'] = 1
        dic['IdServicioPtt'] = 92
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Intento cancelar un servicio generado por una APK
        telefono = '4929292'
        fecha = fecha_actual
        hora = hora_actual
        usuario = 'PTT'
        id_sucursal = 1
        rta = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(rta['Estado'] is True)

        # Agrego un registro en la tabla "actual"
        dic = {}
        dic['Telefono'] = '4939393'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Prioridad'] = 3
        dic['IdSucursal'] = 1
        dic['IdServicioPtt'] = 93
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Intento cancelar un servicio generado por una APK
        telefono = '4939393'
        fecha = fecha_actual
        hora = hora_actual
        usuario = 'PTT'
        id_sucursal = 1
        rta = self.servicio.anular_o_reactivar_servicio(
            telefono, fecha, hora, usuario, id_sucursal
        )
        self.assertTrue(rta['Estado'] is True)

    def test_pasar_servicio_a_automatico(self):
        dic = {}
        dic['Telefono'] = '616161'
        dic['FechaServicio'] = '2010-11-23'
        dic['HoraServicio'] = '1900-01-01 14:00:00'
        dic['Automatico'] = 0
        dic['Pausado'] = 1
        dic['Prioridad'] = 6
        dic['Distancia'] = 20
        dic['Comando'] = 'AC'
        dic['FechaHoraIniSat'] = None
        dic['IdSucursal'] = 1
        dic['PrioridadOriginal'] = 3
        a = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(a)

        # caso correcto
        telefono = '616161'
        fecha = '2010-11-23'
        hora = '1900-01-01 14:00:00'
        r = self.servicio.pasar_servicio_a_automatico(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(r['Mensaje'] is None)

        # le paso un servicio que no existe
        telefono = '1234'
        fecha = '2010-11-23'
        hora = '1900-01-01 14:00:00'
        r = self.servicio.pasar_servicio_a_automatico(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is False)
        self.assertTrue(r['Mensaje'] is not None)

        # le paso un servicio que no tiene PrioridadOriginal
        dic = {}
        dic['Telefono'] = '898989'
        dic['FechaServicio'] = '2010-11-24'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Automatico'] = 0
        dic['Pausado'] = 1
        dic['Prioridad'] = 7
        dic['Distancia'] = 20
        dic['Comando'] = 'AC'
        dic['FechaHoraIniSat'] = None
        dic['IdSucursal'] = 1
        dic['PrioridadOriginal'] = None
        a = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(a)

        # caso correcto
        telefono = '898989'
        fecha = '2010-11-24'
        hora = '1900-01-01 15:00:00'
        r = self.servicio.pasar_servicio_a_automatico(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(r['Mensaje'] is None)

        # le paso un servicio con prioridad 3 pero con prioridad original 1
        dic = {}
        dic['Telefono'] = '1122'
        dic['FechaServicio'] = '2010-11-24'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Automatico'] = 0
        dic['Pausado'] = 1
        dic['Prioridad'] = 3
        dic['Distancia'] = 20
        dic['Comando'] = 'AC'
        dic['FechaHoraIniSat'] = None
        dic['IdSucursal'] = 1
        dic['PrioridadOriginal'] = 1
        a = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(a)

        # caso correcto
        telefono = '1122'
        fecha = '2010-11-24'
        hora = '1900-01-01 15:00:00'
        r = self.servicio.pasar_servicio_a_automatico(telefono, fecha, hora)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(r['Mensaje'] is None)

    def test_listar_servicios_con_QTH(self):
        fecha_actual = self.dth.get_now().strftime("%Y-%m-%d")
        hora_actual = '1900-01-01 ' + self.dth.get_now().strftime("%H:%M:%S")
        fecha_hora_actual = self.dth.get_now().strftime("%Y-%m-%d %H:%M:%S")

        dic = {}
        dic['Telefono'] = '4050505'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 1
        dic['Direccion'] = '24 de septiembre 3797'
        dic['Pausado'] = 0
        dic['Prioridad'] = 2
        dic['PrioridadOriginal'] = 2
        dic['FechaHoraIniSat'] = fecha_hora_actual
        dic['IdSucursal'] = 1
        dic['Latitud'] = 1
        dic['Longitud'] = 1
        self.servicio.cp.agregar('actual', **dic)

        # Agregamos servicio sin posicion
        dic = {}
        dic['Telefono'] = '4052323'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 1
        dic['Direccion'] = '24 de septiembre 3797'
        dic['Pausado'] = 0
        dic['Prioridad'] = 2
        dic['PrioridadOriginal'] = 2
        dic['FechaHoraIniSat'] = fecha_hora_actual
        dic['IdSucursal'] = 1
        self.servicio.cp.agregar('actual', **dic)

        # Agregamos servicio despachado
        dic = {}
        dic['Telefono'] = '44444'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 1
        dic['Direccion'] = '24 de septiembre 3797'
        dic['Pausado'] = 0
        dic['Prioridad'] = 0
        dic['PrioridadOriginal'] = 0
        dic['FechaHoraIniSat'] = fecha_hora_actual
        dic['IdSucursal'] = 1
        dic['Latitud'] = 1
        dic['Longitud'] = 1
        self.servicio.cp.agregar('actual', **dic)

        # Agregamos servicio  cancelado
        dic = {}
        dic['Telefono'] = 'asdasd'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = hora_actual
        dic['Automatico'] = 1
        dic['Direccion'] = '24 de septiembre 3797'
        dic['Pausado'] = 0
        dic['Prioridad'] = 5
        dic['PrioridadOriginal'] = 5
        dic['FechaHoraIniSat'] = fecha_hora_actual
        dic['IdSucursal'] = 1
        dic['Latitud'] = 1
        dic['Longitud'] = 1
        self.servicio.cp.agregar('actual', **dic)

        # Agregamos un servicio en el futuro
        now = self.dth.get_now()
        fecha = datetime.date(now.year, now.month, now.day)
        hs = datetime.datetime(1900, 01, 01, now.hour, now.minute, now.second)
        hs = hs + timedelta(minutes=25)

        dic = {}
        dic['Telefono'] = '4056686785'
        dic['FechaServicio'] = fecha
        dic['HoraServicio'] = hs
        dic['Automatico'] = 1
        dic['Direccion'] = '24 de septiembre 3797'
        dic['Pausado'] = 0
        dic['Prioridad'] = 2
        dic['PrioridadOriginal'] = 2
        dic['FechaHoraIniSat'] = fecha_hora_actual
        dic['IdSucursal'] = 1
        dic['Latitud'] = 1
        dic['Longitud'] = 1
        self.servicio.cp.agregar('actual', **dic)

        r = self.servicio.listar_servicios_con_QTH()
        self.assertTrue(len(r) == 1)

    def test_sancionar_usuario_ptt(self):
        # Le paso un servicio que no existe
        telefono = '1234'
        fecha = '2014-10-17'
        hora = '1900-01-01 16:00:00'
        usuario_cp = 'nombre del usuario CP'
        motivo = u'motivo de la sancion'
        id_motivo = 1
        rta = self.servicio.sancionar_usuario_ptt(
            telefono, fecha, hora, usuario_cp, motivo, id_motivo
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró el servicio dado')
        )

        # Agrego un servicio de PTT
        dic = {}
        dic['Telefono'] = '4110549'
        dic['FechaServicio'] = '2014-10-17'
        dic['HoraServicio'] = '1900-01-01 16:10:00'
        dic['Nombre'] = 'Nombre del usuario PTT'
        dic['IdServicioPtt'] = 126
        dic['IdUsuarioPtt'] = 51
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        telefono = '4110549'
        fecha = '2014-10-17'
        hora = '1900-01-01 16:10:00'
        usuario_cp = 'nombre del usuario CP 2'
        motivo = u'motivo de la sancion 2'
        id_motivo = 2
        rta = self.servicio.sancionar_usuario_ptt(
            telefono, fecha, hora, usuario_cp, motivo, id_motivo
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'El usuario fue sancionado correctamente')
        )

        # Agrego un servicio de PTT
        dic = {}
        dic['Telefono'] = '4444444'
        dic['FechaServicio'] = '2014-10-17'
        dic['HoraServicio'] = '1900-01-01 16:40:00'
        dic['Nombre'] = u'Agustín'
        dic['IdServicioPtt'] = 127
        dic['IdUsuarioPtt'] = 52
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Le paso datos con acentos
        telefono = '4444444'
        fecha = '2014-10-17'
        hora = '1900-01-01 16:40:00'
        usuario_cp = u'Martín'
        motivo = u'No esperó el móvil'
        id_motivo = 1234
        rta = self.servicio.sancionar_usuario_ptt(
            telefono, fecha, hora, usuario_cp, motivo, id_motivo
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'El usuario fue sancionado correctamente')
        )

        # Agrego un servicio sin nombre de usuario
        dic = {}
        dic['Telefono'] = '4555555'
        dic['FechaServicio'] = '2014-10-17'
        dic['HoraServicio'] = '1900-01-01 16:50:00'
        dic['IdServicioPtt'] = 128
        dic['IdUsuarioPtt'] = 53
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla motivos_sanciones
        dic = {}
        dic['IdMotivoSancion'] = 1
        dic['MotivoSancion'] = u'No esperó el móvil'
        agregar = self.servicio.cp.agregar('motivos_sanciones', **dic)
        self.assertTrue(agregar is True)

        # Le paso un servicio sin nombre de usuario
        telefono = '4555555'
        fecha = '2014-10-17'
        hora = '1900-01-01 16:50:00'
        usuario_cp = u'Martín'
        motivo = u'No esperó el móvil'
        id_motivo = 1
        rta = self.servicio.sancionar_usuario_ptt(
            telefono, fecha, hora, usuario_cp, motivo, id_motivo
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'El usuario fue sancionado correctamente')
        )

        # Agrego un servicio que no es de PTT
        dic = {}
        dic['Telefono'] = '4666666'
        dic['FechaServicio'] = '2014-10-17'
        dic['HoraServicio'] = '1900-01-01 17:00:00'
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Le paso un servicio que no es de PTT
        telefono = '4666666'
        fecha = '2014-10-17'
        hora = '1900-01-01 17:00:00'
        usuario_cp = u'Martín'
        motivo = u'No esperó el móvil'
        id_motivo = 1
        rta = self.servicio.sancionar_usuario_ptt(
            telefono, fecha, hora, usuario_cp, motivo, id_motivo
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo sancionar al usuario dado')
        )

    def test_listar_motivos_sanciones(self):
        # No hay ningún reistro en la tabla motivos_sanciones
        rta = self.servicio.listar_motivos_sanciones()
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontraron motivos de sanciones')
        )

        # Agrego un registro en la tabla motivos_sanciones
        dic = {}
        dic['IdMotivoSancion'] = 1
        dic['MotivoSancion'] = u'No esperó el móvil'
        agregar = self.servicio.cp.agregar('motivos_sanciones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla motivos_sanciones
        dic = {}
        dic['IdMotivoSancion'] = 2
        dic['MotivoSancion'] = u'Otro motivo'
        agregar = self.servicio.cp.agregar('motivos_sanciones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla motivos_sanciones vacío
        dic = {}
        dic['IdMotivoSancion'] = 3
        dic['MotivoSancion'] = ''
        agregar = self.servicio.cp.agregar('motivos_sanciones', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        rta = self.servicio.listar_motivos_sanciones()
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'][0]['id_motivo_sancion'] == 1
        )
        self.assertTrue(
            rta['Respuesta'][0]['motivo_sancion'] == _(u'No esperó el móvil')
        )
        self.assertTrue(
            rta['Respuesta'][1]['id_motivo_sancion'] == 2
        )
        self.assertTrue(
            rta['Respuesta'][1]['motivo_sancion'] == _(u'Otro motivo')
        )
        self.assertTrue(
            rta['Respuesta'][2]['id_motivo_sancion'] == 3
        )
        self.assertTrue(
            rta['Respuesta'][2]['motivo_sancion'] == _(u'')
        )

    def test_obtener_motivo_sancion(self):
        # Agrego un registro en la tabla motivos_sanciones
        dic = {}
        dic['IdMotivoSancion'] = 1
        dic['MotivoSancion'] = u'No esperó el móvil'
        agregar = self.servicio.cp.agregar('motivos_sanciones', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        id_motivo_sancion = 1
        rta = self.servicio.obtener_motivo_sancion(id_motivo_sancion)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == _(u'No esperó el móvil'))

        # Le paso un motivo que no existe
        id_motivo_sancion = 1234
        rta = self.servicio.obtener_motivo_sancion(id_motivo_sancion)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró el motivo de la sanción')
        )

    def test_volver_a_despachar(self):
        # Obtengo la fecha y hora actual
        fecha_hora_actual = self.servicio.dth.get_now()

        # Agrego un servicio cancelado
        dic = {}
        dic['Telefono'] = 'servicio 1'
        dic['FechaServicio'] = '2014-12-03'
        dic['HoraServicio'] = '1900-01-01 14:00:00'
        dic['Prioridad'] = 5
        dic['PrioridadOriginal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio con un móvil que no existe con prioridad 6
        dic = {}
        dic['Telefono'] = 'servicio 2'
        dic['FechaServicio'] = '2014-12-03'
        dic['HoraServicio'] = '1900-01-01 17:00:00'
        dic['Prioridad'] = 6
        dic['PrioridadOriginal'] = 1
        dic['IdMovil'] = '1234'
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio de PTT con prioridad 8
        dic = {}
        dic['Telefono'] = 'servicio 3'
        dic['FechaServicio'] = '2014-12-03'
        dic['HoraServicio'] = '1900-01-01 17:20:00'
        dic['Prioridad'] = 8
        dic['PrioridadOriginal'] = 3
        dic['IdMovil'] = '50'
        dic['IdServicioPtt'] = 10
        dic['IdUsuarioPtt'] = 20
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio en automático
        dic = {}
        dic['Telefono'] = 'servicio 4'
        dic['FechaServicio'] = '2014-12-04'
        dic['HoraServicio'] = '1900-01-01 10:00:00'
        dic['Prioridad'] = 6
        dic['PrioridadOriginal'] = 1
        dic['IdMovil'] = '50'
        dic['Automatico'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio con un móvil con celular
        dic = {}
        dic['Telefono'] = 'servicio 5'
        dic['FechaServicio'] = '2014-12-04'
        dic['HoraServicio'] = '1900-01-01 10:20:00'
        dic['Prioridad'] = 8
        dic['PrioridadOriginal'] = 1
        dic['IdMovil'] = '51'
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio con un móvil con consola
        dic = {}
        dic['Telefono'] = 'servicio 6'
        dic['FechaServicio'] = '2014-12-04'
        dic['HoraServicio'] = '1900-01-01 10:30:00'
        dic['Prioridad'] = 8
        dic['PrioridadOriginal'] = 1
        dic['IdMovil'] = '52'
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio con un móvil con consola
        # pero hago que falle el envío
        dic = {}
        dic['Telefono'] = 'servicio 7'
        dic['FechaServicio'] = '2014-12-04'
        dic['HoraServicio'] = '1900-01-01 11:00:00'
        dic['Prioridad'] = 8
        dic['PrioridadOriginal'] = 1
        dic['IdMovil'] = '53'
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio cancelado de PTT
        dic = {}
        dic['Telefono'] = 'servicio 8'
        dic['FechaServicio'] = '2014-12-05'
        dic['HoraServicio'] = '1900-01-01 17:00:00'
        dic['Prioridad'] = 5
        dic['PrioridadOriginal'] = 3
        dic['IdServicioPtt'] = 80
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un móvil sin celular ni consola
        dic = {}
        dic['Patente'] = 'AAA 111'
        dic['IdMovil'] = '50'
        dic['IdFlota'] = 16
        dic['IdEstacion'] = 17
        agregar = self.servicio.cp.agregar('moviles', **dic)
        self.assertTrue(agregar is True)

        # Agrego un móvil con celular
        dic = {}
        dic['Patente'] = 'BBB 222'
        dic['IdMovil'] = '51'
        dic['IdFlota'] = 16
        dic['IdEstacion'] = 17
        dic['idSesion'] = 'la_sesion_del_51'
        dic['DNI'] = 51515151
        dic['UltPingCpMobile'] = fecha_hora_actual
        agregar = self.servicio.cp.agregar('moviles', **dic)
        self.assertTrue(agregar is True)

        # Agrego un móvil con consola
        dic = {}
        dic['Patente'] = 'CCC 333'
        dic['IdMovil'] = '52'
        dic['IdFlota'] = 16
        dic['IdEstacion'] = 17
        dic['Id_Equipo'] = 52
        dic['TermEnLinea'] = 1
        dic['Time_UltimoComando'] = fecha_hora_actual
        agregar = self.servicio.cp.agregar('moviles', **dic)
        self.assertTrue(agregar is True)

        # Agrego un móvil con consola
        dic = {}
        dic['Patente'] = 'DDD 444'
        dic['IdMovil'] = '53'
        dic['IdFlota'] = 16
        dic['IdEstacion'] = 17
        dic['Id_Equipo'] = 53
        dic['TermEnLinea'] = 1
        dic['Time_UltimoComando'] = fecha_hora_actual
        agregar = self.servicio.cp.agregar('moviles', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en controlmsj para que se pueda enviar el mensaje
        # al móvil con consola
        dic = {}
        dic['Telefono'] = 'servicio 6'
        dic['FechaServicio'] = '2014-12-04'
        dic['HoraServicio'] = '1900-01-01 10:30:00'
        dic['Comando'] = '210'
        dic['FechaHora'] = '2014-12-04 10:35:00'
        agregar = self.servicio.cp.agregar('controlmsj', **dic)
        self.assertTrue(agregar is True)

        # Le paso un servicio que no existe
        telefono = '1234'
        fecha = '2014-12-03'
        hora = '1900-01-01 13:00:00'
        usuario = u'Martín'
        rta = self.servicio.volver_a_despachar(telefono, fecha, hora, usuario)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró el servicio dado')
        )

        # Le paso un servicio que no tiene un móvil asignado
        telefono = 'servicio 1'
        fecha = '2014-12-03'
        hora = '1900-01-01 14:00:00'
        usuario = u'Martín'
        rta = self.servicio.volver_a_despachar(telefono, fecha, hora, usuario)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El servicio se vuelve a despachar normalmente')
        )

        # Le paso un servicio que tiene un móvil que no existe
        telefono = 'servicio 2'
        fecha = '2014-12-03'
        hora = '1900-01-01 17:00:00'
        usuario = u'Martín'
        rta = self.servicio.volver_a_despachar(telefono, fecha, hora, usuario)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró el móvil 1234')
        )

        # Le paso un servicio de PTT
        telefono = 'servicio 3'
        fecha = '2014-12-03'
        hora = '1900-01-01 17:20:00'
        usuario = u'Martín'
        rta = self.servicio.volver_a_despachar(telefono, fecha, hora, usuario)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El servicio se vuelve a despachar como rechazado')
        )

        # Le paso un servicio en automático
        telefono = 'servicio 4'
        fecha = '2014-12-04'
        hora = '1900-01-01 10:00:00'
        usuario = u'Martín'
        rta = self.servicio.volver_a_despachar(telefono, fecha, hora, usuario)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'Enviando la cancelación al móvil')
        )

        # Le paso un servicio con un móvil con celular
        telefono = 'servicio 5'
        fecha = '2014-12-04'
        hora = '1900-01-01 10:20:00'
        usuario = u'Martín'
        rta = self.servicio.volver_a_despachar(telefono, fecha, hora, usuario)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El servicio se vuelve a despachar como rechazado')
        )

        # Le paso un servicio con un móvil con consola
        telefono = 'servicio 6'
        fecha = '2014-12-04'
        hora = '1900-01-01 10:30:00'
        usuario = u'Martín'
        rta = self.servicio.volver_a_despachar(telefono, fecha, hora, usuario)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El servicio se vuelve a despachar como rechazado')
        )

        # Le paso un servicio con un móvil con consola pero falla el envío
        telefono = 'servicio 7'
        fecha = '2014-12-04'
        hora = '1900-01-01 11:00:00'
        usuario = u'Martín'
        rta = self.servicio.volver_a_despachar(telefono, fecha, hora, usuario)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se pudo cancelar el servicio al móvil con consola')
        )

        # Le paso un servicio cancelado de PTT
        telefono = 'servicio 8'
        fecha = '2014-12-05'
        hora = '1900-01-01 17:00:00'
        usuario = u'Martín'
        rta = self.servicio.volver_a_despachar(telefono, fecha, hora, usuario)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se puede volver a despachar este servicio')
        )

    def test_listar_despachados_con_ctacte(self):
        # Obtengo la fecha actual
        fecha_actual = self.servicio.dth.get_date()

        # Agrego un servicio con cta. cte. despachado por el móvil 58
        dic = {}
        dic['Telefono'] = 'despachado 1'
        dic['FechaServicio'] = '2015-03-16'
        dic['HoraServicio'] = '1900-01-01 14:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 91
        dic['IdMovil'] = '58'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'La dirección 1'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego el cupón del servicio 91
        dic = {}
        dic['IdServicio'] = 91
        dic['Importe'] = 50.5
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego otro servicio con cta. cte. despachado por el móvil 58
        # Es un servicio de la sucursal 2
        dic = {}
        dic['Telefono'] = u'último'
        dic['FechaServicio'] = '2015-03-16'
        dic['HoraServicio'] = '1900-01-01 14:30:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 92
        dic['IdMovil'] = '58'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'La dirección 2'
        dic['IdSucursal'] = 2
        dic['Importe'] = 130.85
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego otro servicio con cta. cte. despachado por otro móvil
        dic = {}
        dic['Telefono'] = 'despachado 3'
        dic['FechaServicio'] = '2015-03-16'
        dic['HoraServicio'] = '1900-01-01 14:40:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 93
        dic['IdMovil'] = '59'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'La dirección 3'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego otro servicio con cta. cte. para el móvil 58 pero todavía
        # no está despachado
        dic = {}
        dic['Telefono'] = 'despachado 4'
        dic['FechaServicio'] = '2015-03-16'
        dic['HoraServicio'] = '1900-01-01 14:50:00'
        dic['Prioridad'] = 8
        dic['IdServicio'] = 94
        dic['IdMovil'] = '58'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'La dirección 4'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego otro servicio para el móvil 58 pero sin cta. cte.
        dic = {}
        dic['Telefono'] = 'despachado 5'
        dic['FechaServicio'] = '2015-03-16'
        dic['HoraServicio'] = '1900-01-01 13:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 95
        dic['IdMovil'] = '58'
        dic['AceptaCtaCte'] = 0
        dic['Direccion'] = u'La dirección 5'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio con cta. cte. despachado por el móvil 60
        # Este servicio está despachado
        dic = {}
        dic['Telefono'] = '101'
        dic['FechaServicio'] = '2015-04-01'
        dic['HoraServicio'] = '1900-01-01 11:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 101
        dic['IdMovil'] = '60'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'Dir del 101'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio con cta. cte. despachado por el móvil 60
        # Este servicio está cancelado
        dic = {}
        dic['Telefono'] = '102'
        dic['FechaServicio'] = '2015-04-01'
        dic['HoraServicio'] = '1900-01-01 11:30:00'
        dic['Prioridad'] = 5
        dic['IdServicio'] = 102
        dic['IdMovil'] = '60'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'Dir del 102'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego el cupón del servicio 102
        dic = {}
        dic['IdServicio'] = 102
        dic['Importe'] = 72.25
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio con cta. cte. despachado por el móvil 60
        # Este servicio está despachado
        # Esta ya fue pasado a la tabla historico hace 2 días
        dic = {}
        dic['Telefono'] = '20'
        dic['FechaServicio'] = fecha_actual - timedelta(days=2)
        dic['HoraServicio'] = '1900-01-01 09:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 20
        dic['IdMovil'] = '60'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'Dir del 20'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio con cta. cte. despachado por el móvil 60
        # Este servicio está cancelado
        # Esta ya fue pasado a la tabla historico hace 1 día
        dic = {}
        dic['Telefono'] = '21'
        dic['FechaServicio'] = fecha_actual - timedelta(days=1)
        dic['HoraServicio'] = '1900-01-01 09:00:00'
        dic['Prioridad'] = 5
        dic['IdServicio'] = 21
        dic['IdMovil'] = '60'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'Dir del 21'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio SIN cta. cte. despachado por el móvil 60
        # Este servicio está despachado
        # Esta ya fue pasado a la tabla historico hace 2 días
        dic = {}
        dic['Telefono'] = '22'
        dic['FechaServicio'] = fecha_actual - timedelta(days=2)
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 22
        dic['IdMovil'] = '60'
        dic['AceptaCtaCte'] = 0
        dic['Direccion'] = u'Dir del 22'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio con cta. cte. despachado por otro móvil
        # Este servicio está despachado
        # Esta ya fue pasado a la tabla historico hace 20 días
        dic = {}
        dic['Telefono'] = '23'
        dic['FechaServicio'] = fecha_actual - timedelta(days=20)
        dic['HoraServicio'] = '1900-01-01 09:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 23
        dic['IdMovil'] = '71'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'Dir del 23'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio con cta. cte. despachado por el móvil 60
        # Este servicio está despachado
        # Esta ya fue pasado a la tabla historico hace 32 días
        dic = {}
        dic['Telefono'] = '24'
        dic['FechaServicio'] = fecha_actual - timedelta(days=32)
        dic['HoraServicio'] = '1900-01-01 09:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 21
        dic['IdMovil'] = '60'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'Dir del 24'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        id_movil = '58'
        rta = self.servicio.listar_despachados_con_ctacte(id_movil)
        self.assertTrue(rta['Estado'] is True)

        self.assertTrue(len(rta['Respuesta']) == 2)

        self.assertTrue(len(rta['Respuesta'][0]) == 9)
        self.assertTrue(rta['Respuesta'][0][0] == '2015-03-16')
        self.assertTrue(rta['Respuesta'][0][1] == '1900-01-01 14:30:00')
        self.assertTrue(rta['Respuesta'][0][2] == 2)
        self.assertTrue(rta['Respuesta'][0][3] == '16/03/2015 14:30:00')
        self.assertTrue(rta['Respuesta'][0][4] == u'La dirección 2')
        self.assertTrue(rta['Respuesta'][0][5] == u'último')
        self.assertTrue(rta['Respuesta'][0][6] == 92)
        self.assertTrue(rta['Respuesta'][0][7] == 130.85)
        self.assertTrue(rta['Respuesta'][0][8] == 16777215)

        self.assertTrue(len(rta['Respuesta'][1]) == 9)
        self.assertTrue(rta['Respuesta'][1][0] == '2015-03-16')
        self.assertTrue(rta['Respuesta'][1][1] == '1900-01-01 14:00:00')
        self.assertTrue(rta['Respuesta'][1][2] == 1)
        self.assertTrue(rta['Respuesta'][1][3] == '16/03/2015 14:00:00')
        self.assertTrue(rta['Respuesta'][1][4] == u'La dirección 1')
        self.assertTrue(rta['Respuesta'][1][5] == u'despachado 1')
        self.assertTrue(rta['Respuesta'][1][6] == 91)
        self.assertTrue(rta['Respuesta'][1][7] == 50.5)
        self.assertTrue(rta['Respuesta'][1][8] == 16777215)

        # Le paso un móvil que tiene un solo servicio despachado con cta. cte.
        id_movil = '59'
        rta = self.servicio.listar_despachados_con_ctacte(id_movil)
        self.assertTrue(rta['Estado'] is True)

        self.assertTrue(len(rta['Respuesta']) == 1)

        self.assertTrue(len(rta['Respuesta'][0]) == 9)
        self.assertTrue(rta['Respuesta'][0][0] == '2015-03-16')
        self.assertTrue(rta['Respuesta'][0][1] == '1900-01-01 14:40:00')
        self.assertTrue(rta['Respuesta'][0][2] == 1)
        self.assertTrue(rta['Respuesta'][0][3] == '16/03/2015 14:40:00')
        self.assertTrue(rta['Respuesta'][0][4] == u'La dirección 3')
        self.assertTrue(rta['Respuesta'][0][5] == u'despachado 3')
        self.assertTrue(rta['Respuesta'][0][6] == 93)
        self.assertTrue(rta['Respuesta'][0][7] == 0)
        self.assertTrue(rta['Respuesta'][0][8] == 16777215)

        # Le paso un móvil que no tiene ningún servicio con cta. cte. despachado
        id_movil = '1234'
        rta = self.servicio.listar_despachados_con_ctacte(id_movil)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se encontraron servicios con cta. cte. del móvil 1234')
        )

        # Caso correcto con servicio cancelados y de la tabla historico
        id_movil = '60'
        rta = self.servicio.listar_despachados_con_ctacte(id_movil)
        self.assertTrue(rta['Estado'] is True)

        self.assertTrue(len(rta['Respuesta']) == 4)

        self.assertTrue(len(rta['Respuesta'][0]) == 9)
        self.assertTrue(rta['Respuesta'][0][0] == '2015-04-01')
        self.assertTrue(rta['Respuesta'][0][1] == '1900-01-01 11:30:00')
        self.assertTrue(rta['Respuesta'][0][2] == 1)
        self.assertTrue(rta['Respuesta'][0][3] == '01/04/2015 11:30:00')
        self.assertTrue(rta['Respuesta'][0][4] == u'Dir del 102')
        self.assertTrue(rta['Respuesta'][0][5] == u'102')
        self.assertTrue(rta['Respuesta'][0][6] == 102)
        self.assertTrue(rta['Respuesta'][0][7] == 72.25)
        self.assertTrue(rta['Respuesta'][0][8] == 6607057)

        self.assertTrue(len(rta['Respuesta'][1]) == 9)
        self.assertTrue(rta['Respuesta'][1][0] == '2015-04-01')
        self.assertTrue(rta['Respuesta'][1][1] == '1900-01-01 11:00:00')
        self.assertTrue(rta['Respuesta'][1][2] == 1)
        self.assertTrue(rta['Respuesta'][1][3] == '01/04/2015 11:00:00')
        self.assertTrue(rta['Respuesta'][1][4] == u'Dir del 101')
        self.assertTrue(rta['Respuesta'][1][5] == u'101')
        self.assertTrue(rta['Respuesta'][1][6] == 101)
        self.assertTrue(rta['Respuesta'][1][7] == 0)
        self.assertTrue(rta['Respuesta'][1][8] == 16777215)

        self.assertTrue(len(rta['Respuesta'][2]) == 9)
        self.assertTrue(isinstance(rta['Respuesta'][2][0], str))
        self.assertTrue(isinstance(rta['Respuesta'][2][1], str))
        self.assertTrue(rta['Respuesta'][2][2] == 1)
        self.assertTrue(isinstance(rta['Respuesta'][2][3], str))
        self.assertTrue(rta['Respuesta'][2][4] == u'Dir del 21')
        self.assertTrue(rta['Respuesta'][2][5] == u'21')
        self.assertTrue(rta['Respuesta'][2][6] == 21)
        self.assertTrue(rta['Respuesta'][2][7] == 0)
        self.assertTrue(rta['Respuesta'][2][8] == 6607057)

        self.assertTrue(len(rta['Respuesta'][3]) == 9)
        self.assertTrue(isinstance(rta['Respuesta'][3][0], str))
        self.assertTrue(isinstance(rta['Respuesta'][3][1], str))
        self.assertTrue(rta['Respuesta'][3][2] == 1)
        self.assertTrue(isinstance(rta['Respuesta'][3][3], str))
        self.assertTrue(rta['Respuesta'][3][4] == u'Dir del 20')
        self.assertTrue(rta['Respuesta'][3][5] == u'20')
        self.assertTrue(rta['Respuesta'][3][6] == 20)
        self.assertTrue(rta['Respuesta'][3][7] == 0)
        self.assertTrue(rta['Respuesta'][3][8] == 16777215)

        # Agrego un servicio despachado sin cta. cte.
        dic = {}
        dic['Telefono'] = '770'
        dic['FechaServicio'] = '2015-08-03'
        dic['HoraServicio'] = '1900-01-01 16:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 170
        dic['IdMovil'] = '70'
        dic['AceptaCtaCte'] = 0
        dic['Direccion'] = u'Dir del 770'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio despachado con cta. cte.
        dic = {}
        dic['Telefono'] = '771'
        dic['FechaServicio'] = '2015-08-03'
        dic['HoraServicio'] = '1900-01-01 15:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 171
        dic['IdMovil'] = '70'
        dic['AceptaCtaCte'] = 1
        dic['Direccion'] = u'Dir del 771'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio despachado sin cta. cte. en la tabla historico
        dic = {}
        dic['Telefono'] = '72'
        dic['FechaServicio'] = '2015-08-03'
        dic['HoraServicio'] = '1900-01-01 09:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 72
        dic['IdMovil'] = '70'
        dic['AceptaCtaCte'] = 0
        dic['Direccion'] = u'Dir del 72'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Le paso una fecha para que me muestre los servicios que no se les
        # cargó la cta. cte.
        id_movil = '70'
        fecha = '2015-08-03'
        rta = self.servicio.listar_despachados_con_ctacte(id_movil, fecha)
        self.assertTrue(len(rta['Respuesta']) == 3)

        self.assertTrue(len(rta['Respuesta'][0]) == 9)
        self.assertTrue(rta['Respuesta'][0][0] == '2015-08-03')
        self.assertTrue(rta['Respuesta'][0][1] == '1900-01-01 16:00:00')
        self.assertTrue(rta['Respuesta'][0][2] == 1)
        self.assertTrue(rta['Respuesta'][0][3] == '03/08/2015 16:00:00')
        self.assertTrue(rta['Respuesta'][0][4] == u'Dir del 770')
        self.assertTrue(rta['Respuesta'][0][5] == u'770')
        self.assertTrue(rta['Respuesta'][0][6] == 170)
        self.assertTrue(rta['Respuesta'][0][7] == 0)
        self.assertTrue(rta['Respuesta'][0][8] == 16777215)

        self.assertTrue(len(rta['Respuesta'][1]) == 9)
        self.assertTrue(rta['Respuesta'][1][0] == '2015-08-03')
        self.assertTrue(rta['Respuesta'][1][1] == '1900-01-01 15:00:00')
        self.assertTrue(rta['Respuesta'][1][2] == 1)
        self.assertTrue(rta['Respuesta'][1][3] == '03/08/2015 15:00:00')
        self.assertTrue(rta['Respuesta'][1][4] == u'Dir del 771')
        self.assertTrue(rta['Respuesta'][1][5] == u'771')
        self.assertTrue(rta['Respuesta'][1][6] == 171)
        self.assertTrue(rta['Respuesta'][1][7] == 0)
        self.assertTrue(rta['Respuesta'][1][8] == 16777215)

        self.assertTrue(len(rta['Respuesta'][2]) == 9)
        self.assertTrue(rta['Respuesta'][2][0] == '2015-08-03')
        self.assertTrue(rta['Respuesta'][2][1] == '1900-01-01 09:00:00')
        self.assertTrue(rta['Respuesta'][2][2] == 1)
        self.assertTrue(rta['Respuesta'][2][3] == '03/08/2015 09:00:00')
        self.assertTrue(rta['Respuesta'][2][4] == u'Dir del 72')
        self.assertTrue(rta['Respuesta'][2][5] == u'72')
        self.assertTrue(rta['Respuesta'][2][6] == 72)
        self.assertTrue(rta['Respuesta'][2][7] == 0)
        self.assertTrue(rta['Respuesta'][2][8] == 16777215)

        # Le paso una fecha donde no hay servicio del móvil dado
        id_movil = '70'
        fecha = '2015-08-01'
        rta = self.servicio.listar_despachados_con_ctacte(id_movil, fecha)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se encontraron servicios del móvil 70 en la fecha dada')
        )

    def test_pasar_servicio_con_ctacte(self):
        # Agrego un servicio sin cta. cte. en la tabla actual
        dic = {}
        dic['Telefono'] = 'sin_ctacte'
        dic['FechaServicio'] = '2015-08-05'
        dic['HoraServicio'] = '1900-01-01 13:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 80
        dic['IdMovil'] = '25'
        dic['AceptaCtaCte'] = 0
        dic['Direccion'] = u'Dir del servicio sin_ctacte'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio sin cta. cte. en la tabla actual
        dic = {}
        dic['Telefono'] = 'sin_ctacte2'
        dic['FechaServicio'] = '2015-08-05'
        dic['HoraServicio'] = '1900-01-01 14:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 81
        dic['IdMovil'] = '26'
        dic['AceptaCtaCte'] = 0
        dic['Direccion'] = u'Dir del servicio sin_ctacte2'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio sin cta. cte. en la tabla historico
        dic = {}
        dic['Telefono'] = 'sin_ctacte3'
        dic['FechaServicio'] = '2015-08-01'
        dic['HoraServicio'] = '1900-01-01 12:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 72
        dic['IdMovil'] = '15'
        dic['AceptaCtaCte'] = 0
        dic['Direccion'] = u'Dir del servicio sin_ctacte3 de historico'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Agrego un servicio sin cta. cte. en la tabla historico
        dic = {}
        dic['Telefono'] = 'sin_ctacte4'
        dic['FechaServicio'] = '2015-08-01'
        dic['HoraServicio'] = '1900-01-01 12:30:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 73
        dic['IdMovil'] = '16'
        dic['AceptaCtaCte'] = 0
        dic['Direccion'] = u'Dir del servicio sin_ctacte4 de historico'
        dic['IdSucursal'] = 1
        agregar = self.servicio.cp.agregar('historico', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto para la tabla actual
        id_servicio = 80
        rta = self.servicio.pasar_servicio_con_ctacte(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'El servicio pasó a ser con cta. cte.')
        )

        # Le paso un servicio que no existe
        id_servicio = 1234
        rta = self.servicio.pasar_servicio_con_ctacte(id_servicio)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo pasar el servicio a cta. cte.')
        )

        # Caso correcto para la tabla historico
        id_servicio = 72
        rta = self.servicio.pasar_servicio_con_ctacte(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'El servicio pasó a ser con cta. cte.')
        )

    def test_setear_etiqueta(self):
        # Agrego un servicio
        dic = {}
        dic['Telefono'] = 'sin etiqueta'
        dic['FechaServicio'] = '2016-01-22'
        dic['HoraServicio'] = '1900-01-01 12:00:00'
        dic['Prioridad'] = 0
        dic['IdServicio'] = 91
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        id_servicio = 91
        id_etiqueta = 1
        rta = self.servicio.setear_etiqueta(id_servicio, id_etiqueta)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'El servicio 91 recibió la etiqueta 1')
        )

        # Caso correcto
        id_servicio = 91
        id_etiqueta = 2
        rta = self.servicio.setear_etiqueta(id_servicio, id_etiqueta)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] == _(u'El servicio 91 recibió la etiqueta 2')
        )

        # Le paso un servicio que no existe
        id_servicio = 1234
        id_etiqueta = 2
        rta = self.servicio.setear_etiqueta(id_servicio, id_etiqueta)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo etiquetar el servicio 1234')
        )

    def test_obtener_zona_por_calle(self):
        # Agrego un registro en la tabla "calles_zonas"
        dic = {}
        dic['idSucursal'] = 1
        dic['idCiudad'] = 324
        dic['idCalle'] = 1333475
        dic['idTipoNumeracion'] = 1
        dic['AlturaDesde'] = 1200
        dic['AlturaHasta'] = 1900
        dic['IdZona'] = 5
        agregar = self.servicio.cp.agregar('calles_zonas', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla "calles_zonas"
        dic = {}
        dic['idSucursal'] = 1
        dic['idCiudad'] = 324
        dic['idCalle'] = 1333475
        dic['idTipoNumeracion'] = 2
        dic['AlturaDesde'] = 200
        dic['AlturaHasta'] = 800
        dic['IdZona'] = 6
        agregar = self.servicio.cp.agregar('calles_zonas', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla "calles_zonas"
        dic = {}
        dic['idSucursal'] = 1
        dic['idCiudad'] = 324
        dic['idCalle'] = 1333579
        dic['idTipoNumeracion'] = 3
        dic['AlturaDesde'] = 2301
        dic['AlturaHasta'] = 2601
        dic['IdZona'] = 7
        agregar = self.servicio.cp.agregar('calles_zonas', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto (todos)
        id_sucursal = 1
        id_ciudad = 324
        id_calle = 1333475
        altura = 1264
        rta = self.servicio.obtener_zona_por_calle(
            id_sucursal, id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 5)

        # Caso correcto (par)
        id_sucursal = 1
        id_ciudad = 324
        id_calle = 1333475
        altura = 252
        rta = self.servicio.obtener_zona_por_calle(
            id_sucursal, id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 6)

        # Caso correcto (impar)
        id_sucursal = 1
        id_ciudad = 324
        id_calle = 1333579
        altura = 2441
        rta = self.servicio.obtener_zona_por_calle(
            id_sucursal, id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 7)

        # Le paso una altura impar y debe ser par
        id_sucursal = 1
        id_ciudad = 324
        id_calle = 1333475
        altura = 253
        rta = self.servicio.obtener_zona_por_calle(
            id_sucursal, id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró la zona de la calle dada')
        )

        # Le paso una altura par y debe ser impar
        id_sucursal = 1
        id_ciudad = 324
        id_calle = 1333579
        altura = 2442
        rta = self.servicio.obtener_zona_por_calle(
            id_sucursal, id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró la zona de la calle dada')
        )

        # Le paso una calle que no existe
        id_sucursal = 1
        id_ciudad = 324
        id_calle = 1234
        altura = 1264
        rta = self.servicio.obtener_zona_por_calle(
            id_sucursal, id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró la zona de la calle dada')
        )

        # Le paso otra sucursal
        id_sucursal = 2
        id_ciudad = 324
        id_calle = 1333475
        altura = 1264
        rta = self.servicio.obtener_zona_por_calle(
            id_sucursal, id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró la zona de la calle dada')
        )

        # Le paso otra ciudad
        id_sucursal = 1
        id_ciudad = 1234
        id_calle = 1333475
        altura = 1264
        rta = self.servicio.obtener_zona_por_calle(
            id_sucursal, id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró la zona de la calle dada')
        )

        # Le paso una altura que no abarca
        id_sucursal = 1
        id_ciudad = 324
        id_calle = 1333475
        altura = 1901
        rta = self.servicio.obtener_zona_por_calle(
            id_sucursal, id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró la zona de la calle dada')
        )

    def test_listar_por_ctacte(self):
        # Obtengo la fecha y hora actual
        fecha_hora_actual = self.servicio.dth.get_now()

        # Obtengo la fecha actual
        fecha_actual = fecha_hora_actual.date()

        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 41000
        dic['Telefono'] = 'tel_41000'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = '1900-01-01 11:00:00'
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 40000
        dic['Telefono'] = 'tel_40000'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = '1900-01-01 11:05:00'
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 41001
        dic['Telefono'] = 'tel_41001'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = '1900-01-01 11:30:00'
        dic['Direccion'] = 'Urquiza 7164'
        dic['Prioridad'] = 0
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 41002
        dic['Telefono'] = 'tel_41002'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = '1900-01-01 11:45:00'
        dic['Direccion'] = u'España 1130'
        dic['Prioridad'] = 3
        dic['Demora'] = '1900-01-01 00:03:42'
        dic['TiempoEstimado'] = 15
        dic['IdMovil'] = '68'
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 42003
        dic['Telefono'] = 'tel_42003'
        dic['FechaServicio'] = fecha_actual
        dic['HoraServicio'] = '1900-01-01 12:00:00'
        dic['Direccion'] = 'Urquiza 7164'
        dic['Prioridad'] = 5
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla cupones
        dic = {}
        dic['IdCuponAuto'] = 201
        dic['IdCliente'] = 41
        dic['FechaHoraServicio'] = fecha_hora_actual
        dic['IdServicio'] = 41000
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla cupones
        dic = {}
        dic['IdCuponAuto'] = 202
        dic['IdCliente'] = 41
        dic['FechaServicio'] = fecha_actual
        dic['IdServicio'] = 40000
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla cupones
        dic = {}
        dic['IdCuponAuto'] = 203
        dic['IdCliente'] = 41
        dic['FechaServicio'] = fecha_actual
        dic['FechaHoraServicio'] = fecha_hora_actual
        dic['IdCentroDeCosto'] = 141
        dic['IdPersona'] = 241
        dic['IdServicio'] = 41001
        dic['Direccion'] = 'Urquiza 7164'
        dic['Siniestro'] = '145282'
        dic['Pasajero'] = 'Juan'
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla cupones
        dic = {}
        dic['IdCuponAuto'] = 204
        dic['IdCliente'] = 41
        dic['FechaServicio'] = fecha_actual
        dic['FechaHoraServicio'] = fecha_hora_actual
        dic['IdCentroDeCosto'] = 341
        dic['IdPersona'] = 441
        dic['IdServicio'] = 41002
        dic['Direccion'] = u'España 1130'
        dic['Siniestro'] = u'canción'
        dic['Pasajero'] = u'Martín'
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla cupones
        dic = {}
        dic['IdCuponAuto'] = 205
        dic['IdCliente'] = 42
        dic['FechaServicio'] = fecha_actual
        dic['FechaHoraServicio'] = fecha_hora_actual
        dic['IdCentroDeCosto'] = 142
        dic['IdPersona'] = 242
        dic['IdServicio'] = 42003
        dic['Direccion'] = 'Urquiza 7164'
        dic['Siniestro'] = '145282'
        dic['Pasajero'] = 'Juan'
        dic['ServicioCancelado'] = 1
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla cupones
        dic = {}
        dic['IdCuponAuto'] = 106
        dic['IdCliente'] = 41
        dic['FechaServicio'] = fecha_actual - timedelta(hours=25)
        dic['FechaHoraServicio'] = fecha_hora_actual - timedelta(hours=25)
        dic['IdCentroDeCosto'] = 141
        dic['IdPersona'] = 241
        dic['IdServicio'] = 40904
        dic['Direccion'] = 'Urquiza 7164'
        dic['Siniestro'] = '145282'
        dic['Pasajero'] = 'Juan'
        agregar = self.servicio.cp.agregar('cupones', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla moviles
        dic = {}
        dic['Patente'] = 'ABC 123'
        dic['IdMovil'] = '68'
        dic['IdFlota'] = 16
        dic['IdEstacion'] = 17
        agregar = self.servicio.cp.agregar('moviles', **dic)
        self.assertTrue(agregar is True)

        # Filtro solo por el cliente
        id_cliente = 41
        rta = self.servicio.listar_por_ctacte(id_cliente)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 4)

        self.assertTrue(len(rta['Respuesta'][0]) == 17)
        self.assertTrue(rta['Respuesta'][0][0] == 204)
        self.assertTrue(rta['Respuesta'][0][1] == 41002)
        self.assertTrue(rta['Respuesta'][0][2] == 41)
        self.assertTrue(rta['Respuesta'][0][3] == 341)
        self.assertTrue(rta['Respuesta'][0][4] == 441)
        self.assertTrue(isinstance(rta['Respuesta'][0][5], str))
        self.assertTrue(isinstance(rta['Respuesta'][0][6], str))
        self.assertTrue(rta['Respuesta'][0][7] == u'España 1130')
        self.assertTrue(rta['Respuesta'][0][8] == u'canción')
        self.assertTrue(rta['Respuesta'][0][9] == u'Martín')
        self.assertTrue(rta['Respuesta'][0][10] == u'Pendiente')
        self.assertTrue(rta['Respuesta'][0][11] == 3)
        self.assertTrue(rta['Respuesta'][0][12] == 'tel_41002')
        self.assertTrue(rta['Respuesta'][0][13] == '00:03:42')
        self.assertTrue(rta['Respuesta'][0][14] == '15 min.')
        self.assertTrue(rta['Respuesta'][0][15] == '68')
        self.assertTrue(rta['Respuesta'][0][16] == 'ABC 123')

        self.assertTrue(len(rta['Respuesta'][1]) == 17)
        self.assertTrue(rta['Respuesta'][1][0] == 203)
        self.assertTrue(rta['Respuesta'][1][1] == 41001)
        self.assertTrue(rta['Respuesta'][1][2] == 41)
        self.assertTrue(rta['Respuesta'][1][3] == 141)
        self.assertTrue(rta['Respuesta'][1][4] == 241)
        self.assertTrue(isinstance(rta['Respuesta'][1][5], str))
        self.assertTrue(isinstance(rta['Respuesta'][1][6], str))
        self.assertTrue(rta['Respuesta'][1][7] == 'Urquiza 7164')
        self.assertTrue(rta['Respuesta'][1][8] == '145282')
        self.assertTrue(rta['Respuesta'][1][9] == 'Juan')
        self.assertTrue(rta['Respuesta'][1][10] == u'Despachado')
        self.assertTrue(rta['Respuesta'][1][11] == 0)
        self.assertTrue(rta['Respuesta'][1][12] == u'tel_41001')
        self.assertTrue(rta['Respuesta'][1][13] == '')
        self.assertTrue(rta['Respuesta'][1][14] == '')
        self.assertTrue(rta['Respuesta'][1][15] == '')
        self.assertTrue(rta['Respuesta'][1][16] == '')

        self.assertTrue(len(rta['Respuesta'][2]) == 17)
        self.assertTrue(rta['Respuesta'][2][0] == 202)
        self.assertTrue(rta['Respuesta'][2][1] == 40000)
        self.assertTrue(rta['Respuesta'][2][2] == 41)
        self.assertTrue(rta['Respuesta'][2][3] is None)
        self.assertTrue(rta['Respuesta'][2][4] is None)
        self.assertTrue(isinstance(rta['Respuesta'][2][5], str))
        self.assertTrue(isinstance(rta['Respuesta'][2][6], str))
        self.assertTrue(rta['Respuesta'][2][7] == '')
        self.assertTrue(rta['Respuesta'][2][8] == '')
        self.assertTrue(rta['Respuesta'][2][9] == '')
        self.assertTrue(rta['Respuesta'][2][10] == u'Pendiente')
        self.assertTrue(rta['Respuesta'][2][11] is None)
        self.assertTrue(rta['Respuesta'][2][12] == u'tel_40000')
        self.assertTrue(rta['Respuesta'][2][13] == '')
        self.assertTrue(rta['Respuesta'][2][14] == '')
        self.assertTrue(rta['Respuesta'][2][15] == '')
        self.assertTrue(rta['Respuesta'][2][16] == '')

        self.assertTrue(len(rta['Respuesta'][3]) == 17)
        self.assertTrue(rta['Respuesta'][3][0] == 201)
        self.assertTrue(rta['Respuesta'][3][1] == 41000)
        self.assertTrue(rta['Respuesta'][3][2] == 41)
        self.assertTrue(rta['Respuesta'][3][3] is None)
        self.assertTrue(rta['Respuesta'][3][4] is None)
        self.assertTrue(isinstance(rta['Respuesta'][1][5], str))
        self.assertTrue(isinstance(rta['Respuesta'][1][6], str))
        self.assertTrue(rta['Respuesta'][3][7] == '')
        self.assertTrue(rta['Respuesta'][3][8] == '')
        self.assertTrue(rta['Respuesta'][3][9] == '')
        self.assertTrue(rta['Respuesta'][3][10] == u'Pendiente')
        self.assertTrue(rta['Respuesta'][3][11] is None)
        self.assertTrue(rta['Respuesta'][3][12] == u'tel_41000')
        self.assertTrue(rta['Respuesta'][3][13] == '')
        self.assertTrue(rta['Respuesta'][3][14] == '')
        self.assertTrue(rta['Respuesta'][3][15] == '')
        self.assertTrue(rta['Respuesta'][3][16] == '')

        # Le paso un cliente que no existe
        id_cliente = 1234
        rta = self.servicio.listar_por_ctacte(id_cliente)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No se encontraron servicios'))

        # Le paso otro cliente
        id_cliente = 42
        rta = self.servicio.listar_por_ctacte(id_cliente)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 1)

        self.assertTrue(len(rta['Respuesta'][0]) == 17)
        self.assertTrue(rta['Respuesta'][0][0] == 205)
        self.assertTrue(rta['Respuesta'][0][1] == 42003)
        self.assertTrue(rta['Respuesta'][0][2] == 42)
        self.assertTrue(rta['Respuesta'][0][3] == 142)
        self.assertTrue(rta['Respuesta'][0][4] == 242)
        self.assertTrue(isinstance(rta['Respuesta'][0][5], str))
        self.assertTrue(isinstance(rta['Respuesta'][0][6], str))
        self.assertTrue(rta['Respuesta'][0][7] == 'Urquiza 7164')
        self.assertTrue(rta['Respuesta'][0][8] == '145282')
        self.assertTrue(rta['Respuesta'][0][9] == 'Juan')
        self.assertTrue(rta['Respuesta'][0][10] == u'Cancelado')
        self.assertTrue(rta['Respuesta'][0][11] == 5)
        self.assertTrue(rta['Respuesta'][0][12] == u'tel_42003')
        self.assertTrue(rta['Respuesta'][0][13] == '')
        self.assertTrue(rta['Respuesta'][0][14] == '')
        self.assertTrue(rta['Respuesta'][0][15] == '')
        self.assertTrue(rta['Respuesta'][0][16] == '')

        # Filtro por cliente y centro de costo
        id_cliente = 41
        id_centro_de_costo = 141
        rta = self.servicio.listar_por_ctacte(id_cliente, id_centro_de_costo)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 1)

        self.assertTrue(len(rta['Respuesta'][0]) == 17)
        self.assertTrue(rta['Respuesta'][0][0] == 203)
        self.assertTrue(rta['Respuesta'][0][1] == 41001)
        self.assertTrue(rta['Respuesta'][0][2] == 41)
        self.assertTrue(rta['Respuesta'][0][3] == 141)
        self.assertTrue(rta['Respuesta'][0][4] == 241)
        self.assertTrue(isinstance(rta['Respuesta'][0][5], str))
        self.assertTrue(isinstance(rta['Respuesta'][0][6], str))
        self.assertTrue(rta['Respuesta'][0][7] == 'Urquiza 7164')
        self.assertTrue(rta['Respuesta'][0][8] == '145282')
        self.assertTrue(rta['Respuesta'][0][9] == 'Juan')
        self.assertTrue(rta['Respuesta'][0][10] == u'Despachado')
        self.assertTrue(rta['Respuesta'][0][11] == 0)
        self.assertTrue(rta['Respuesta'][0][12] == u'tel_41001')
        self.assertTrue(rta['Respuesta'][0][13] == '')
        self.assertTrue(rta['Respuesta'][0][14] == '')
        self.assertTrue(rta['Respuesta'][0][15] == '')
        self.assertTrue(rta['Respuesta'][0][16] == '')

        # Le paso un centro de costo que no existe
        id_cliente = 41
        id_centro_de_costo = 1234
        rta = self.servicio.listar_por_ctacte(id_cliente, id_centro_de_costo)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No se encontraron servicios'))

        # Filtro por cliente, por centro de costo y por persona
        id_cliente = 41
        id_centro_de_costo = 141
        id_persona = 241
        rta = self.servicio.listar_por_ctacte(
            id_cliente, id_centro_de_costo, id_persona
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 1)

        self.assertTrue(len(rta['Respuesta'][0]) == 17)
        self.assertTrue(rta['Respuesta'][0][0] == 203)
        self.assertTrue(rta['Respuesta'][0][1] == 41001)
        self.assertTrue(rta['Respuesta'][0][2] == 41)
        self.assertTrue(rta['Respuesta'][0][3] == 141)
        self.assertTrue(rta['Respuesta'][0][4] == 241)
        self.assertTrue(isinstance(rta['Respuesta'][0][5], str))
        self.assertTrue(isinstance(rta['Respuesta'][0][6], str))
        self.assertTrue(rta['Respuesta'][0][7] == 'Urquiza 7164')
        self.assertTrue(rta['Respuesta'][0][8] == '145282')
        self.assertTrue(rta['Respuesta'][0][9] == 'Juan')
        self.assertTrue(rta['Respuesta'][0][10] == u'Despachado')
        self.assertTrue(rta['Respuesta'][0][11] == 0)
        self.assertTrue(rta['Respuesta'][0][12] == u'tel_41001')
        self.assertTrue(rta['Respuesta'][0][13] == '')
        self.assertTrue(rta['Respuesta'][0][14] == '')
        self.assertTrue(rta['Respuesta'][0][15] == '')
        self.assertTrue(rta['Respuesta'][0][16] == '')

        # Le paso una persona que no existe
        id_cliente = 41
        id_centro_de_costo = 141
        id_persona = 1234
        rta = self.servicio.listar_por_ctacte(
            id_cliente, id_centro_de_costo, id_persona
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No se encontraron servicios'))

    def test_datos_movil(self):
        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 51
        dic['Telefono'] = 'tel_51'
        dic['FechaServicio'] = '2016-11-17'
        dic['HoraServicio'] = '1900-01-01 16:00:00'
        dic['Direccion'] = u'España 1130'
        dic['Prioridad'] = 8
        dic['Demora'] = '1900-01-01 00:05:42'
        dic['TiempoEstimado'] = 15
        dic['IdMovil'] = '71'
        dic['Distancia'] = 362
        dic['Latitud'] = -118560.9456
        dic['Longitud'] = -218585.8188
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla moviles
        dic = {}
        dic['Patente'] = u'ñan 123'
        dic['IdMovil'] = '71'
        dic['IdFlota'] = 16
        dic['IdEstacion'] = 17
        dic['Latitud'] = -118560.9411
        dic['Longitud'] = -218585.8199
        dic['Estado1'] = 0
        dic['Movil'] = u'Corsa Ñan 2006'
        agregar = self.servicio.cp.agregar('moviles', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 52
        dic['Telefono'] = 'tel_52'
        dic['FechaServicio'] = '2016-11-18'
        dic['HoraServicio'] = '1900-01-01 10:00:00'
        dic['Direccion'] = u'Urquiza 7164'
        dic['Prioridad'] = 6
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 53
        dic['Telefono'] = 'tel_53'
        dic['FechaServicio'] = '2016-11-18'
        dic['HoraServicio'] = '1900-01-01 11:00:00'
        dic['Direccion'] = u'Rioja 1500'
        dic['Prioridad'] = 0
        dic['Demora'] = '1900-01-01 00:10:31'
        dic['TiempoEstimado'] = 20
        dic['IdMovil'] = '73'
        dic['Distancia'] = 42
        dic['Latitud'] = -118560.9456
        dic['Longitud'] = -218585.8188
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla moviles
        dic = {}
        dic['Patente'] = u'ASD 452'
        dic['IdMovil'] = '73'
        dic['IdFlota'] = 16
        dic['IdEstacion'] = 17
        dic['Latitud'] = -118560.9411
        dic['Longitud'] = -218585.8199
        dic['Estado1'] = 1
        dic['Movil'] = u'Palio 2008'
        agregar = self.servicio.cp.agregar('moviles', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 54
        dic['Telefono'] = 'tel_54'
        dic['FechaServicio'] = '2016-11-23'
        dic['HoraServicio'] = '1900-01-01 10:00:00'
        dic['Direccion'] = u'Paraguay 600'
        dic['Prioridad'] = 8
        dic['Demora'] = '1900-01-01 00:20:10'
        dic['TiempoEstimado'] = 15
        dic['IdMovil'] = '74'
        dic['Distancia'] = 260
        dic['Latitud'] = -118560.9456
        dic['Longitud'] = -218585.8188
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Agrego un registro en la tabla moviles
        dic = {}
        dic['Patente'] = 'WEV 478'
        dic['IdMovil'] = '74'
        dic['IdFlota'] = 16
        dic['IdEstacion'] = 17
        dic['Latitud'] = -118560.9411
        dic['Longitud'] = -218585.8199
        dic['Estado1'] = 0
        dic['Movil'] = 'Fiat 1'
        dic['Estado1'] = 4
        agregar = self.servicio.cp.agregar('moviles', **dic)
        self.assertTrue(agregar is True)

        # Caso con todos los datos
        id_servicio = 51
        rta = self.servicio.datos_movil(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 14)
        self.assertTrue(rta['Respuesta']['id_servicio'] == 51)
        self.assertTrue(rta['Respuesta']['lat_servicio'] == -32.933596)
        self.assertTrue(rta['Respuesta']['lon_servicio'] == -60.718283)
        self.assertTrue(rta['Respuesta']['lat_movil'] == -32.93359475)
        self.assertTrue(rta['Respuesta']['lon_movil'] == -60.71828330555556)
        self.assertTrue(rta['Respuesta']['estado_movil'] == 0)
        self.assertTrue(rta['Respuesta']['tiempo_estimado'] == 15)
        self.assertTrue(rta['Respuesta']['distancia'] == 362)
        self.assertTrue(rta['Respuesta']['patente'] == u'ñan 123')
        self.assertTrue(rta['Respuesta']['id_movil'] == u'71')
        self.assertTrue(
            rta['Respuesta']['nombre_prioridad'] == u'Aceptado por el móvil'
        )
        self.assertTrue(rta['Respuesta']['demora'] == '00:05:42')
        self.assertTrue(rta['Respuesta']['prioridad'] == 8)
        self.assertTrue(rta['Respuesta']['desc_movil'] == u'Corsa Ñan 2006')

        # Le paso un móvil que no existe
        id_servicio = 1234
        rta = self.servicio.datos_movil(id_servicio)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró el servicio con ID 1234')
        )

        # Le paso un servicio con los mínimos datos
        id_servicio = 52
        rta = self.servicio.datos_movil(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 14)
        self.assertTrue(rta['Respuesta']['id_servicio'] == 52)
        self.assertTrue(rta['Respuesta']['lat_servicio'] == 0)
        self.assertTrue(rta['Respuesta']['lon_servicio'] == 0)
        self.assertTrue(rta['Respuesta']['lat_movil'] == 0)
        self.assertTrue(rta['Respuesta']['lon_movil'] == 0)
        self.assertTrue(rta['Respuesta']['estado_movil'] is None)
        self.assertTrue(rta['Respuesta']['tiempo_estimado'] == '')
        self.assertTrue(rta['Respuesta']['distancia'] == '')
        self.assertTrue(rta['Respuesta']['patente'] == '')
        self.assertTrue(rta['Respuesta']['id_movil'] == '')
        self.assertTrue(
            rta['Respuesta']['nombre_prioridad'] == u'Esperando respuesta'
        )
        self.assertTrue(rta['Respuesta']['demora'] == '')
        self.assertTrue(rta['Respuesta']['prioridad'] == 6)
        self.assertTrue(rta['Respuesta']['desc_movil'] == '')

        # Le paso un servicio ya despachado
        id_servicio = 53
        rta = self.servicio.datos_movil(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 14)
        self.assertTrue(rta['Respuesta']['id_servicio'] == 53)
        self.assertTrue(rta['Respuesta']['lat_servicio'] == -32.933596)
        self.assertTrue(rta['Respuesta']['lon_servicio'] == -60.718283)
        self.assertTrue(rta['Respuesta']['lat_movil'] == 0)
        self.assertTrue(rta['Respuesta']['lon_movil'] == 0)
        self.assertTrue(rta['Respuesta']['estado_movil'] == 0)
        self.assertTrue(rta['Respuesta']['tiempo_estimado'] == 20)
        self.assertTrue(rta['Respuesta']['distancia'] == '')
        self.assertTrue(rta['Respuesta']['patente'] == 'ASD 452')
        self.assertTrue(rta['Respuesta']['id_movil'] == '73')
        self.assertTrue(
            rta['Respuesta']['nombre_prioridad'] == u'Despachado'
        )
        self.assertTrue(rta['Respuesta']['demora'] == '00:10:31')
        self.assertTrue(rta['Respuesta']['prioridad'] == 0)
        self.assertTrue(rta['Respuesta']['desc_movil'] == u'Palio 2008')

        # Le paso un servicio con un móvil sin la posición actualizada
        id_servicio = 54
        rta = self.servicio.datos_movil(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 14)
        self.assertTrue(rta['Respuesta']['id_servicio'] == 54)
        self.assertTrue(rta['Respuesta']['lat_servicio'] == -32.933596)
        self.assertTrue(rta['Respuesta']['lon_servicio'] == -60.718283)
        self.assertTrue(rta['Respuesta']['lat_movil'] == -32.93359475)
        self.assertTrue(rta['Respuesta']['lon_movil'] == -60.71828330555556)
        self.assertTrue(rta['Respuesta']['estado_movil'] == 4)
        self.assertTrue(rta['Respuesta']['tiempo_estimado'] == 15)
        self.assertTrue(rta['Respuesta']['distancia'] == 260)
        self.assertTrue(rta['Respuesta']['patente'] == 'WEV 478')
        self.assertTrue(rta['Respuesta']['id_movil'] == '74')
        self.assertTrue(
            rta['Respuesta']['nombre_prioridad'] == u'Aceptado por el móvil'
        )
        self.assertTrue(rta['Respuesta']['demora'] == '00:20:10')
        self.assertTrue(rta['Respuesta']['prioridad'] == 8)
        self.assertTrue(rta['Respuesta']['desc_movil'] == u'Fiat 1')

    def test_obtener_id(self):
        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 61
        dic['Telefono'] = '4110549'
        dic['FechaServicio'] = '2017-01-26'
        dic['HoraServicio'] = '1900-01-01 14:15:36'
        dic['Direccion'] = u'Paraguay 600'
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        telefono = '4110549'
        fecha_servicio = '2017-01-26'
        hora_servicio = '1900-01-01 14:15:36'
        rta = self.servicio.obtener_id(telefono, fecha_servicio, hora_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == 61)

        # Le paso un servicio que no existe
        telefono = 'no_existe'
        fecha_servicio = '2017-01-26'
        hora_servicio = '1900-01-01 14:30:39'
        rta = self.servicio.obtener_id(telefono, fecha_servicio, hora_servicio)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudo obtener el número del servicio')
        )

    def test_obtener_telefono_fecha_hora(self):
        # Agrego un registro en la tabla actual
        dic = {}
        dic['IdServicio'] = 71
        dic['Telefono'] = '153705967'
        dic['FechaServicio'] = '2017-01-26'
        dic['HoraServicio'] = '1900-01-01 15:58:32'
        dic['Direccion'] = u'Rioja 1800'
        dic['Prioridad'] = 3
        agregar = self.servicio.cp.agregar('actual', **dic)
        self.assertTrue(agregar is True)

        # Caso correcto
        id_servicio = 71
        rta = self.servicio.obtener_telefono_fecha_hora(id_servicio)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 4)
        self.assertTrue(rta['Respuesta']['telefono'] == '153705967')
        self.assertTrue(rta['Respuesta']['fecha_servicio'] == '2017-01-26')
        self.assertTrue(
            rta['Respuesta']['hora_servicio'] == '1900-01-01 15:58:32'
        )
        self.assertTrue(rta['Respuesta']['prioridad'] == 3)

        # Le paso un servicio que no existe
        id_servicio = 1234
        rta = self.servicio.obtener_telefono_fecha_hora(id_servicio)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se pudo obtener el servicio con número 1234')
        )


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestServicio))
    return suite
